package servicefabric

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

import (
	"encoding/json"
	"github.com/Azure/go-autorest/autorest"
	"github.com/Azure/go-autorest/autorest/date"
	"github.com/gofrs/uuid"
)

// The package's fully qualified name.
const fqdn = "github.com/Azure/azure-sdk-for-go/services/servicefabric/6.5/servicefabric"

// AadMetadata azure Active Directory metadata used for secured connection to cluster.
type AadMetadata struct {
	// Authority - The AAD authority url.
	Authority *string `json:"authority,omitempty"`
	// Client - The AAD client application Id.
	Client *string `json:"client,omitempty"`
	// Cluster - The AAD cluster application Id.
	Cluster *string `json:"cluster,omitempty"`
	// Login - The AAD login url.
	Login *string `json:"login,omitempty"`
	// Redirect - The client application redirect address.
	Redirect *string `json:"redirect,omitempty"`
	// Tenant - The AAD tenant Id.
	Tenant *string `json:"tenant,omitempty"`
}

// AadMetadataObject azure Active Directory metadata object used for secured connection to cluster.
type AadMetadataObject struct {
	autorest.Response `json:"-"`
	// Type - The client authentication method.
	Type *string `json:"type,omitempty"`
	// Metadata - Azure Active Directory metadata used for secured connection to cluster.
	Metadata *AadMetadata `json:"metadata,omitempty"`
}

// AddRemoveIncrementalNamedPartitionScalingMechanism represents a scaling mechanism for adding or removing
// named partitions of a stateless service. Partition names are in the format '0','1''N-1'
type AddRemoveIncrementalNamedPartitionScalingMechanism struct {
	// MinPartitionCount - Minimum number of named partitions of the service.
	MinPartitionCount *int32 `json:"MinPartitionCount,omitempty"`
	// MaxPartitionCount - Maximum number of named partitions of the service.
	MaxPartitionCount *int32 `json:"MaxPartitionCount,omitempty"`
	// ScaleIncrement - The number of instances to add or remove during a scaling operation.
	ScaleIncrement *int32 `json:"ScaleIncrement,omitempty"`
	// Kind - Possible values include: 'KindScalingMechanismDescription', 'KindPartitionInstanceCount', 'KindAddRemoveIncrementalNamedPartition'
	Kind KindBasicScalingMechanismDescription `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for AddRemoveIncrementalNamedPartitionScalingMechanism.
func (arinpsm AddRemoveIncrementalNamedPartitionScalingMechanism) MarshalJSON() ([]byte, error) {
	arinpsm.Kind = KindAddRemoveIncrementalNamedPartition
	objectMap := make(map[string]interface{})
	if arinpsm.MinPartitionCount != nil {
		objectMap["MinPartitionCount"] = arinpsm.MinPartitionCount
	}
	if arinpsm.MaxPartitionCount != nil {
		objectMap["MaxPartitionCount"] = arinpsm.MaxPartitionCount
	}
	if arinpsm.ScaleIncrement != nil {
		objectMap["ScaleIncrement"] = arinpsm.ScaleIncrement
	}
	if arinpsm.Kind != "" {
		objectMap["Kind"] = arinpsm.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionInstanceCountScaleMechanism is the BasicScalingMechanismDescription implementation for AddRemoveIncrementalNamedPartitionScalingMechanism.
func (arinpsm AddRemoveIncrementalNamedPartitionScalingMechanism) AsPartitionInstanceCountScaleMechanism() (*PartitionInstanceCountScaleMechanism, bool) {
	return nil, false
}

// AsAddRemoveIncrementalNamedPartitionScalingMechanism is the BasicScalingMechanismDescription implementation for AddRemoveIncrementalNamedPartitionScalingMechanism.
func (arinpsm AddRemoveIncrementalNamedPartitionScalingMechanism) AsAddRemoveIncrementalNamedPartitionScalingMechanism() (*AddRemoveIncrementalNamedPartitionScalingMechanism, bool) {
	return &arinpsm, true
}

// AsScalingMechanismDescription is the BasicScalingMechanismDescription implementation for AddRemoveIncrementalNamedPartitionScalingMechanism.
func (arinpsm AddRemoveIncrementalNamedPartitionScalingMechanism) AsScalingMechanismDescription() (*ScalingMechanismDescription, bool) {
	return nil, false
}

// AsBasicScalingMechanismDescription is the BasicScalingMechanismDescription implementation for AddRemoveIncrementalNamedPartitionScalingMechanism.
func (arinpsm AddRemoveIncrementalNamedPartitionScalingMechanism) AsBasicScalingMechanismDescription() (BasicScalingMechanismDescription, bool) {
	return &arinpsm, true
}

// AddRemoveReplicaScalingMechanism describes the horizontal auto scaling mechanism that adds or removes
// replicas (containers or container groups).
type AddRemoveReplicaScalingMechanism struct {
	// MinCount - Minimum number of containers (scale down won't be performed below this number).
	MinCount *int32 `json:"minCount,omitempty"`
	// MaxCount - Maximum number of containers (scale up won't be performed above this number).
	MaxCount *int32 `json:"maxCount,omitempty"`
	// ScaleIncrement - Each time auto scaling is performed, this number of containers will be added or removed.
	ScaleIncrement *int32 `json:"scaleIncrement,omitempty"`
	// Kind - Possible values include: 'KindAutoScalingMechanism', 'KindAddRemoveReplica'
	Kind KindBasicAutoScalingMechanism `json:"kind,omitempty"`
}

// MarshalJSON is the custom marshaler for AddRemoveReplicaScalingMechanism.
func (arrsm AddRemoveReplicaScalingMechanism) MarshalJSON() ([]byte, error) {
	arrsm.Kind = KindAddRemoveReplica
	objectMap := make(map[string]interface{})
	if arrsm.MinCount != nil {
		objectMap["minCount"] = arrsm.MinCount
	}
	if arrsm.MaxCount != nil {
		objectMap["maxCount"] = arrsm.MaxCount
	}
	if arrsm.ScaleIncrement != nil {
		objectMap["scaleIncrement"] = arrsm.ScaleIncrement
	}
	if arrsm.Kind != "" {
		objectMap["kind"] = arrsm.Kind
	}
	return json.Marshal(objectMap)
}

// AsAddRemoveReplicaScalingMechanism is the BasicAutoScalingMechanism implementation for AddRemoveReplicaScalingMechanism.
func (arrsm AddRemoveReplicaScalingMechanism) AsAddRemoveReplicaScalingMechanism() (*AddRemoveReplicaScalingMechanism, bool) {
	return &arrsm, true
}

// AsAutoScalingMechanism is the BasicAutoScalingMechanism implementation for AddRemoveReplicaScalingMechanism.
func (arrsm AddRemoveReplicaScalingMechanism) AsAutoScalingMechanism() (*AutoScalingMechanism, bool) {
	return nil, false
}

// AsBasicAutoScalingMechanism is the BasicAutoScalingMechanism implementation for AddRemoveReplicaScalingMechanism.
func (arrsm AddRemoveReplicaScalingMechanism) AsBasicAutoScalingMechanism() (BasicAutoScalingMechanism, bool) {
	return &arrsm, true
}

// AnalysisEventMetadata metadata about an Analysis Event.
type AnalysisEventMetadata struct {
	// Delay - The analysis delay.
	Delay *string `json:"Delay,omitempty"`
	// Duration - The duration of analysis.
	Duration *string `json:"Duration,omitempty"`
}

// ApplicationBackupConfigurationInfo backup configuration information for a specific Service Fabric
// application specifying what backup policy is being applied and suspend description, if any.
type ApplicationBackupConfigurationInfo struct {
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// PolicyName - The name of the backup policy which is applicable to this Service Fabric application or service or partition.
	PolicyName *string `json:"PolicyName,omitempty"`
	// PolicyInheritedFrom - Specifies the scope at which the backup policy is applied. Possible values include: 'BackupPolicyScopeInvalid', 'BackupPolicyScopePartition', 'BackupPolicyScopeService', 'BackupPolicyScopeApplication'
	PolicyInheritedFrom BackupPolicyScope `json:"PolicyInheritedFrom,omitempty"`
	// SuspensionInfo - Describes the backup suspension details.
	SuspensionInfo *BackupSuspensionInfo `json:"SuspensionInfo,omitempty"`
	// Kind - Possible values include: 'KindBasicBackupConfigurationInfoKindBackupConfigurationInfo', 'KindBasicBackupConfigurationInfoKindApplication', 'KindBasicBackupConfigurationInfoKindService', 'KindBasicBackupConfigurationInfoKindPartition'
	Kind KindBasicBackupConfigurationInfo `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationBackupConfigurationInfo.
func (abci ApplicationBackupConfigurationInfo) MarshalJSON() ([]byte, error) {
	abci.Kind = KindBasicBackupConfigurationInfoKindApplication
	objectMap := make(map[string]interface{})
	if abci.ApplicationName != nil {
		objectMap["ApplicationName"] = abci.ApplicationName
	}
	if abci.PolicyName != nil {
		objectMap["PolicyName"] = abci.PolicyName
	}
	if abci.PolicyInheritedFrom != "" {
		objectMap["PolicyInheritedFrom"] = abci.PolicyInheritedFrom
	}
	if abci.SuspensionInfo != nil {
		objectMap["SuspensionInfo"] = abci.SuspensionInfo
	}
	if abci.Kind != "" {
		objectMap["Kind"] = abci.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for ApplicationBackupConfigurationInfo.
func (abci ApplicationBackupConfigurationInfo) AsApplicationBackupConfigurationInfo() (*ApplicationBackupConfigurationInfo, bool) {
	return &abci, true
}

// AsServiceBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for ApplicationBackupConfigurationInfo.
func (abci ApplicationBackupConfigurationInfo) AsServiceBackupConfigurationInfo() (*ServiceBackupConfigurationInfo, bool) {
	return nil, false
}

// AsPartitionBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for ApplicationBackupConfigurationInfo.
func (abci ApplicationBackupConfigurationInfo) AsPartitionBackupConfigurationInfo() (*PartitionBackupConfigurationInfo, bool) {
	return nil, false
}

// AsBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for ApplicationBackupConfigurationInfo.
func (abci ApplicationBackupConfigurationInfo) AsBackupConfigurationInfo() (*BackupConfigurationInfo, bool) {
	return nil, false
}

// AsBasicBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for ApplicationBackupConfigurationInfo.
func (abci ApplicationBackupConfigurationInfo) AsBasicBackupConfigurationInfo() (BasicBackupConfigurationInfo, bool) {
	return &abci, true
}

// ApplicationBackupEntity identifies the Service Fabric application which is being backed up.
type ApplicationBackupEntity struct {
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// EntityKind - Possible values include: 'EntityKindBackupEntity', 'EntityKindApplication1', 'EntityKindService1', 'EntityKindPartition1'
	EntityKind EntityKindBasicBackupEntity `json:"EntityKind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationBackupEntity.
func (abe ApplicationBackupEntity) MarshalJSON() ([]byte, error) {
	abe.EntityKind = EntityKindApplication1
	objectMap := make(map[string]interface{})
	if abe.ApplicationName != nil {
		objectMap["ApplicationName"] = abe.ApplicationName
	}
	if abe.EntityKind != "" {
		objectMap["EntityKind"] = abe.EntityKind
	}
	return json.Marshal(objectMap)
}

// AsApplicationBackupEntity is the BasicBackupEntity implementation for ApplicationBackupEntity.
func (abe ApplicationBackupEntity) AsApplicationBackupEntity() (*ApplicationBackupEntity, bool) {
	return &abe, true
}

// AsServiceBackupEntity is the BasicBackupEntity implementation for ApplicationBackupEntity.
func (abe ApplicationBackupEntity) AsServiceBackupEntity() (*ServiceBackupEntity, bool) {
	return nil, false
}

// AsPartitionBackupEntity is the BasicBackupEntity implementation for ApplicationBackupEntity.
func (abe ApplicationBackupEntity) AsPartitionBackupEntity() (*PartitionBackupEntity, bool) {
	return nil, false
}

// AsBackupEntity is the BasicBackupEntity implementation for ApplicationBackupEntity.
func (abe ApplicationBackupEntity) AsBackupEntity() (*BackupEntity, bool) {
	return nil, false
}

// AsBasicBackupEntity is the BasicBackupEntity implementation for ApplicationBackupEntity.
func (abe ApplicationBackupEntity) AsBasicBackupEntity() (BasicBackupEntity, bool) {
	return &abe, true
}

// ApplicationCapacityDescription describes capacity information for services of this application. This
// description can be used for describing the following.
// - Reserving the capacity for the services on the nodes
// - Limiting the total number of nodes that services of this application can run on
// - Limiting the custom capacity metrics to limit the total consumption of this metric by the services of
// this application
type ApplicationCapacityDescription struct {
	// MinimumNodes - The minimum number of nodes where Service Fabric will reserve capacity for this application. Note that this does not mean that the services of this application will be placed on all of those nodes. If this property is set to zero, no capacity will be reserved. The value of this property cannot be more than the value of the MaximumNodes property.
	MinimumNodes *int64 `json:"MinimumNodes,omitempty"`
	// MaximumNodes - The maximum number of nodes where Service Fabric will reserve capacity for this application. Note that this does not mean that the services of this application will be placed on all of those nodes. By default, the value of this property is zero and it means that the services can be placed on any node.
	MaximumNodes *int64 `json:"MaximumNodes,omitempty"`
	// ApplicationMetrics - List of application capacity metric description.
	ApplicationMetrics *[]ApplicationMetricDescription `json:"ApplicationMetrics,omitempty"`
}

// ApplicationContainerInstanceExitedEvent container Exited event.
type ApplicationContainerInstanceExitedEvent struct {
	// ServiceName - Name of Service.
	ServiceName *string `json:"ServiceName,omitempty"`
	// ServicePackageName - Name of Service package.
	ServicePackageName *string `json:"ServicePackageName,omitempty"`
	// ServicePackageActivationID - Activation Id of Service package.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// IsExclusive - Indicates IsExclusive flag.
	IsExclusive *bool `json:"IsExclusive,omitempty"`
	// CodePackageName - Name of Code package.
	CodePackageName *string `json:"CodePackageName,omitempty"`
	// EntryPointType - Type of EntryPoint.
	EntryPointType *string `json:"EntryPointType,omitempty"`
	// ImageName - Name of Container image.
	ImageName *string `json:"ImageName,omitempty"`
	// ContainerName - Name of Container.
	ContainerName *string `json:"ContainerName,omitempty"`
	// HostID - Host Id.
	HostID *string `json:"HostId,omitempty"`
	// ExitCode - Exit code of process.
	ExitCode *int64 `json:"ExitCode,omitempty"`
	// UnexpectedTermination - Indicates if termination is unexpected.
	UnexpectedTermination *bool `json:"UnexpectedTermination,omitempty"`
	// StartTime - Start time of process.
	StartTime *date.Time `json:"StartTime,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) MarshalJSON() ([]byte, error) {
	aciee.Kind = KindApplicationContainerInstanceExited
	objectMap := make(map[string]interface{})
	if aciee.ServiceName != nil {
		objectMap["ServiceName"] = aciee.ServiceName
	}
	if aciee.ServicePackageName != nil {
		objectMap["ServicePackageName"] = aciee.ServicePackageName
	}
	if aciee.ServicePackageActivationID != nil {
		objectMap["ServicePackageActivationId"] = aciee.ServicePackageActivationID
	}
	if aciee.IsExclusive != nil {
		objectMap["IsExclusive"] = aciee.IsExclusive
	}
	if aciee.CodePackageName != nil {
		objectMap["CodePackageName"] = aciee.CodePackageName
	}
	if aciee.EntryPointType != nil {
		objectMap["EntryPointType"] = aciee.EntryPointType
	}
	if aciee.ImageName != nil {
		objectMap["ImageName"] = aciee.ImageName
	}
	if aciee.ContainerName != nil {
		objectMap["ContainerName"] = aciee.ContainerName
	}
	if aciee.HostID != nil {
		objectMap["HostId"] = aciee.HostID
	}
	if aciee.ExitCode != nil {
		objectMap["ExitCode"] = aciee.ExitCode
	}
	if aciee.UnexpectedTermination != nil {
		objectMap["UnexpectedTermination"] = aciee.UnexpectedTermination
	}
	if aciee.StartTime != nil {
		objectMap["StartTime"] = aciee.StartTime
	}
	if aciee.ApplicationID != nil {
		objectMap["ApplicationId"] = aciee.ApplicationID
	}
	if aciee.EventInstanceID != nil {
		objectMap["EventInstanceId"] = aciee.EventInstanceID
	}
	if aciee.Category != nil {
		objectMap["Category"] = aciee.Category
	}
	if aciee.TimeStamp != nil {
		objectMap["TimeStamp"] = aciee.TimeStamp
	}
	if aciee.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = aciee.HasCorrelatedEvents
	}
	if aciee.Kind != "" {
		objectMap["Kind"] = aciee.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &aciee, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return &aciee, true
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationContainerInstanceExitedEvent.
func (aciee ApplicationContainerInstanceExitedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &aciee, true
}

// ApplicationCreatedEvent application Created event.
type ApplicationCreatedEvent struct {
	// ApplicationTypeName - Application type name.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// ApplicationTypeVersion - Application type version.
	ApplicationTypeVersion *string `json:"ApplicationTypeVersion,omitempty"`
	// ApplicationDefinitionKind - Application definition kind.
	ApplicationDefinitionKind *string `json:"ApplicationDefinitionKind,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) MarshalJSON() ([]byte, error) {
	ace.Kind = KindApplicationCreated
	objectMap := make(map[string]interface{})
	if ace.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = ace.ApplicationTypeName
	}
	if ace.ApplicationTypeVersion != nil {
		objectMap["ApplicationTypeVersion"] = ace.ApplicationTypeVersion
	}
	if ace.ApplicationDefinitionKind != nil {
		objectMap["ApplicationDefinitionKind"] = ace.ApplicationDefinitionKind
	}
	if ace.ApplicationID != nil {
		objectMap["ApplicationId"] = ace.ApplicationID
	}
	if ace.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ace.EventInstanceID
	}
	if ace.Category != nil {
		objectMap["Category"] = ace.Category
	}
	if ace.TimeStamp != nil {
		objectMap["TimeStamp"] = ace.TimeStamp
	}
	if ace.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ace.HasCorrelatedEvents
	}
	if ace.Kind != "" {
		objectMap["Kind"] = ace.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &ace, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return &ace, true
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationCreatedEvent.
func (ace ApplicationCreatedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ace, true
}

// ApplicationDeletedEvent application Deleted event.
type ApplicationDeletedEvent struct {
	// ApplicationTypeName - Application type name.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// ApplicationTypeVersion - Application type version.
	ApplicationTypeVersion *string `json:"ApplicationTypeVersion,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) MarshalJSON() ([]byte, error) {
	ade.Kind = KindApplicationDeleted
	objectMap := make(map[string]interface{})
	if ade.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = ade.ApplicationTypeName
	}
	if ade.ApplicationTypeVersion != nil {
		objectMap["ApplicationTypeVersion"] = ade.ApplicationTypeVersion
	}
	if ade.ApplicationID != nil {
		objectMap["ApplicationId"] = ade.ApplicationID
	}
	if ade.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ade.EventInstanceID
	}
	if ade.Category != nil {
		objectMap["Category"] = ade.Category
	}
	if ade.TimeStamp != nil {
		objectMap["TimeStamp"] = ade.TimeStamp
	}
	if ade.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ade.HasCorrelatedEvents
	}
	if ade.Kind != "" {
		objectMap["Kind"] = ade.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &ade, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return &ade, true
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationDeletedEvent.
func (ade ApplicationDeletedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ade, true
}

// ApplicationDescription describes a Service Fabric application.
type ApplicationDescription struct {
	// Name - The name of the application, including the 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// TypeName - The application type name as defined in the application manifest.
	TypeName *string `json:"TypeName,omitempty"`
	// TypeVersion - The version of the application type as defined in the application manifest.
	TypeVersion *string `json:"TypeVersion,omitempty"`
	// ParameterList - List of application parameters with overridden values from their default values specified in the application manifest.
	ParameterList *[]ApplicationParameter `json:"ParameterList,omitempty"`
	// ApplicationCapacity - Describes capacity information for services of this application. This description can be used for describing the following.
	// - Reserving the capacity for the services on the nodes
	// - Limiting the total number of nodes that services of this application can run on
	// - Limiting the custom capacity metrics to limit the total consumption of this metric by the services of this application
	ApplicationCapacity *ApplicationCapacityDescription `json:"ApplicationCapacity,omitempty"`
	// ManagedApplicationIdentity - Managed application identity description.
	ManagedApplicationIdentity *ManagedApplicationIdentityDescription `json:"ManagedApplicationIdentity,omitempty"`
}

// BasicApplicationEvent represents the base for all Application Events.
type BasicApplicationEvent interface {
	AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool)
	AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool)
	AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool)
	AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool)
	AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool)
	AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool)
	AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool)
	AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool)
	AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool)
	AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool)
	AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool)
	AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool)
	AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool)
	AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool)
	AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool)
	AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool)
	AsApplicationEvent() (*ApplicationEvent, bool)
}

// ApplicationEvent represents the base for all Application Events.
type ApplicationEvent struct {
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

func unmarshalBasicApplicationEvent(body []byte) (BasicApplicationEvent, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindApplicationCreated):
		var ace ApplicationCreatedEvent
		err := json.Unmarshal(body, &ace)
		return ace, err
	case string(KindApplicationDeleted):
		var ade ApplicationDeletedEvent
		err := json.Unmarshal(body, &ade)
		return ade, err
	case string(KindApplicationNewHealthReport):
		var anhre ApplicationNewHealthReportEvent
		err := json.Unmarshal(body, &anhre)
		return anhre, err
	case string(KindApplicationHealthReportExpired):
		var ahree ApplicationHealthReportExpiredEvent
		err := json.Unmarshal(body, &ahree)
		return ahree, err
	case string(KindApplicationUpgradeCompleted):
		var auce ApplicationUpgradeCompletedEvent
		err := json.Unmarshal(body, &auce)
		return auce, err
	case string(KindApplicationUpgradeDomainCompleted):
		var audce ApplicationUpgradeDomainCompletedEvent
		err := json.Unmarshal(body, &audce)
		return audce, err
	case string(KindApplicationUpgradeRollbackCompleted):
		var aurce ApplicationUpgradeRollbackCompletedEvent
		err := json.Unmarshal(body, &aurce)
		return aurce, err
	case string(KindApplicationUpgradeRollbackStarted):
		var aurse ApplicationUpgradeRollbackStartedEvent
		err := json.Unmarshal(body, &aurse)
		return aurse, err
	case string(KindApplicationUpgradeStarted):
		var ause ApplicationUpgradeStartedEvent
		err := json.Unmarshal(body, &ause)
		return ause, err
	case string(KindDeployedApplicationNewHealthReport):
		var danhre DeployedApplicationNewHealthReportEvent
		err := json.Unmarshal(body, &danhre)
		return danhre, err
	case string(KindDeployedApplicationHealthReportExpired):
		var dahree DeployedApplicationHealthReportExpiredEvent
		err := json.Unmarshal(body, &dahree)
		return dahree, err
	case string(KindApplicationProcessExited):
		var apee ApplicationProcessExitedEvent
		err := json.Unmarshal(body, &apee)
		return apee, err
	case string(KindApplicationContainerInstanceExited):
		var aciee ApplicationContainerInstanceExitedEvent
		err := json.Unmarshal(body, &aciee)
		return aciee, err
	case string(KindDeployedServicePackageNewHealthReport):
		var dspnhre DeployedServicePackageNewHealthReportEvent
		err := json.Unmarshal(body, &dspnhre)
		return dspnhre, err
	case string(KindDeployedServicePackageHealthReportExpired):
		var dsphree DeployedServicePackageHealthReportExpiredEvent
		err := json.Unmarshal(body, &dsphree)
		return dsphree, err
	case string(KindChaosCodePackageRestartScheduled):
		var ccprse ChaosCodePackageRestartScheduledEvent
		err := json.Unmarshal(body, &ccprse)
		return ccprse, err
	default:
		var ae ApplicationEvent
		err := json.Unmarshal(body, &ae)
		return ae, err
	}
}
func unmarshalBasicApplicationEventArray(body []byte) ([]BasicApplicationEvent, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	aeArray := make([]BasicApplicationEvent, len(rawMessages))

	for index, rawMessage := range rawMessages {
		ae, err := unmarshalBasicApplicationEvent(*rawMessage)
		if err != nil {
			return nil, err
		}
		aeArray[index] = ae
	}
	return aeArray, nil
}

// MarshalJSON is the custom marshaler for ApplicationEvent.
func (ae ApplicationEvent) MarshalJSON() ([]byte, error) {
	ae.Kind = KindApplicationEvent
	objectMap := make(map[string]interface{})
	if ae.ApplicationID != nil {
		objectMap["ApplicationId"] = ae.ApplicationID
	}
	if ae.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ae.EventInstanceID
	}
	if ae.Category != nil {
		objectMap["Category"] = ae.Category
	}
	if ae.TimeStamp != nil {
		objectMap["TimeStamp"] = ae.TimeStamp
	}
	if ae.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ae.HasCorrelatedEvents
	}
	if ae.Kind != "" {
		objectMap["Kind"] = ae.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return &ae, true
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &ae, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationEvent.
func (ae ApplicationEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ae, true
}

// ApplicationHealth represents the health of the application. Contains the application aggregated health
// state and the service and deployed application health states.
type ApplicationHealth struct {
	autorest.Response `json:"-"`
	// Name - The name of the application, including the 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// ServiceHealthStates - Service health states as found in the health store.
	ServiceHealthStates *[]ServiceHealthState `json:"ServiceHealthStates,omitempty"`
	// DeployedApplicationHealthStates - Deployed application health states as found in the health store.
	DeployedApplicationHealthStates *[]DeployedApplicationHealthState `json:"DeployedApplicationHealthStates,omitempty"`
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

// ApplicationHealthEvaluation represents health evaluation for an application, containing information
// about the data and the algorithm used by the health store to evaluate health.
type ApplicationHealthEvaluation struct {
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the current aggregated health state of the application. The types of the unhealthy evaluations can be DeployedApplicationsHealthEvaluation, ServicesHealthEvaluation or EventHealthEvaluation.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) MarshalJSON() ([]byte, error) {
	ahe.Kind = KindApplication
	objectMap := make(map[string]interface{})
	if ahe.ApplicationName != nil {
		objectMap["ApplicationName"] = ahe.ApplicationName
	}
	if ahe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = ahe.UnhealthyEvaluations
	}
	if ahe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = ahe.AggregatedHealthState
	}
	if ahe.Description != nil {
		objectMap["Description"] = ahe.Description
	}
	if ahe.Kind != "" {
		objectMap["Kind"] = ahe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return &ahe, true
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationHealthEvaluation.
func (ahe ApplicationHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &ahe, true
}

// ApplicationHealthPolicies defines the application health policy map used to evaluate the health of an
// application or one of its children entities.
type ApplicationHealthPolicies struct {
	// ApplicationHealthPolicyMap - The wrapper that contains the map with application health policies used to evaluate specific applications in the cluster.
	ApplicationHealthPolicyMap *[]ApplicationHealthPolicyMapItem `json:"ApplicationHealthPolicyMap,omitempty"`
}

// ApplicationHealthPolicy defines a health policy used to evaluate the health of an application or one of
// its children entities.
type ApplicationHealthPolicy struct {
	// ConsiderWarningAsError - Indicates whether warnings are treated with the same severity as errors.
	ConsiderWarningAsError *bool `json:"ConsiderWarningAsError,omitempty"`
	// MaxPercentUnhealthyDeployedApplications - The maximum allowed percentage of unhealthy deployed applications. Allowed values are Byte values from zero to 100.
	// The percentage represents the maximum tolerated percentage of deployed applications that can be unhealthy before the application is considered in error.
	// This is calculated by dividing the number of unhealthy deployed applications over the number of nodes where the application is currently deployed on in the cluster.
	// The computation rounds up to tolerate one failure on small numbers of nodes. Default percentage is zero.
	MaxPercentUnhealthyDeployedApplications *int32 `json:"MaxPercentUnhealthyDeployedApplications,omitempty"`
	// DefaultServiceTypeHealthPolicy - The health policy used by default to evaluate the health of a service type.
	DefaultServiceTypeHealthPolicy *ServiceTypeHealthPolicy `json:"DefaultServiceTypeHealthPolicy,omitempty"`
	// ServiceTypeHealthPolicyMap - The map with service type health policy per service type name. The map is empty by default.
	ServiceTypeHealthPolicyMap *[]ServiceTypeHealthPolicyMapItem `json:"ServiceTypeHealthPolicyMap,omitempty"`
}

// ApplicationHealthPolicyMapItem defines an item in ApplicationHealthPolicyMap.
type ApplicationHealthPolicyMapItem struct {
	// Key - The key of the application health policy map item. This is the name of the application.
	Key *string `json:"Key,omitempty"`
	// Value - The value of the application health policy map item. This is the ApplicationHealthPolicy for this application.
	Value *ApplicationHealthPolicy `json:"Value,omitempty"`
}

// ApplicationHealthReportExpiredEvent application Health Report Expired event.
type ApplicationHealthReportExpiredEvent struct {
	// ApplicationInstanceID - Id of Application instance.
	ApplicationInstanceID *int64 `json:"ApplicationInstanceId,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) MarshalJSON() ([]byte, error) {
	ahree.Kind = KindApplicationHealthReportExpired
	objectMap := make(map[string]interface{})
	if ahree.ApplicationInstanceID != nil {
		objectMap["ApplicationInstanceId"] = ahree.ApplicationInstanceID
	}
	if ahree.SourceID != nil {
		objectMap["SourceId"] = ahree.SourceID
	}
	if ahree.Property != nil {
		objectMap["Property"] = ahree.Property
	}
	if ahree.HealthState != nil {
		objectMap["HealthState"] = ahree.HealthState
	}
	if ahree.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = ahree.TimeToLiveMs
	}
	if ahree.SequenceNumber != nil {
		objectMap["SequenceNumber"] = ahree.SequenceNumber
	}
	if ahree.Description != nil {
		objectMap["Description"] = ahree.Description
	}
	if ahree.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = ahree.RemoveWhenExpired
	}
	if ahree.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = ahree.SourceUtcTimestamp
	}
	if ahree.ApplicationID != nil {
		objectMap["ApplicationId"] = ahree.ApplicationID
	}
	if ahree.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ahree.EventInstanceID
	}
	if ahree.Category != nil {
		objectMap["Category"] = ahree.Category
	}
	if ahree.TimeStamp != nil {
		objectMap["TimeStamp"] = ahree.TimeStamp
	}
	if ahree.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ahree.HasCorrelatedEvents
	}
	if ahree.Kind != "" {
		objectMap["Kind"] = ahree.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &ahree, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return &ahree, true
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationHealthReportExpiredEvent.
func (ahree ApplicationHealthReportExpiredEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ahree, true
}

// ApplicationHealthState represents the health state of an application, which contains the application
// identifier and the aggregated health state.
type ApplicationHealthState struct {
	// Name - The name of the application, including the 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
}

// ApplicationHealthStateChunk represents the health state chunk of a application.
// The application health state chunk contains the application name, its aggregated health state and any
// children services and deployed applications that respect the filters in cluster health chunk query
// description.
type ApplicationHealthStateChunk struct {
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ApplicationTypeName - The application type name as defined in the application manifest.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// ServiceHealthStateChunks - The list of service health state chunks in the cluster that respect the filters in the cluster health chunk query description.
	ServiceHealthStateChunks *ServiceHealthStateChunkList `json:"ServiceHealthStateChunks,omitempty"`
	// DeployedApplicationHealthStateChunks - The list of deployed application health state chunks in the cluster that respect the filters in the cluster health chunk query description.
	DeployedApplicationHealthStateChunks *DeployedApplicationHealthStateChunkList `json:"DeployedApplicationHealthStateChunks,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
}

// ApplicationHealthStateChunkList the list of application health state chunks in the cluster that respect
// the input filters in the chunk query. Returned by get cluster health state chunks query.
type ApplicationHealthStateChunkList struct {
	// Items - The list of application health state chunks that respect the input filters in the chunk query.
	Items *[]ApplicationHealthStateChunk `json:"Items,omitempty"`
	// TotalCount - Total number of entity health state objects that match the specified filters from the cluster health chunk query description.
	TotalCount *int64 `json:"TotalCount,omitempty"`
}

// ApplicationHealthStateFilter defines matching criteria to determine whether a application should be
// included in the cluster health chunk.
// One filter can match zero, one or multiple applications, depending on its properties.
type ApplicationHealthStateFilter struct {
	// ApplicationNameFilter - The name of the application that matches the filter, as a fabric uri. The filter is applied only to the specified application, if it exists.
	// If the application doesn't exist, no application is returned in the cluster health chunk based on this filter.
	// If the application exists, it is included in the cluster health chunk if it respects the other filter properties.
	// If not specified, all applications are matched against the other filter members, like health state filter.
	ApplicationNameFilter *string `json:"ApplicationNameFilter,omitempty"`
	// ApplicationTypeNameFilter - The name of the application type that matches the filter.
	// If specified, the filter is applied only to applications of the selected application type, if any exists.
	// If no applications of the specified application type exists, no application is returned in the cluster health chunk based on this filter.
	// Each application of the specified application type is included in the cluster health chunk if it respects the other filter properties.
	// If not specified, all applications are matched against the other filter members, like health state filter.
	ApplicationTypeNameFilter *string `json:"ApplicationTypeNameFilter,omitempty"`
	// HealthStateFilter - The filter for the health state of the applications. It allows selecting applications if they match the desired health states.
	// The possible values are integer value of one of the following health states. Only applications that match the filter are returned. All applications are used to evaluate the cluster aggregated health state.
	// If not specified, default value is None, unless the application name or the application type name are specified. If the filter has default value and application name is specified, the matching application is returned.
	// The state values are flag-based enumeration, so the value could be a combination of these values obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6, it matches applications with HealthState value of OK (2) and Warning (4).
	// - Default - Default value. Matches any HealthState. The value is zero.
	// - None - Filter that doesn't match any HealthState value. Used in order to return no results on a given collection of states. The value is 1.
	// - Ok - Filter that matches input with HealthState value Ok. The value is 2.
	// - Warning - Filter that matches input with HealthState value Warning. The value is 4.
	// - Error - Filter that matches input with HealthState value Error. The value is 8.
	// - All - Filter that matches input with any HealthState value. The value is 65535.
	HealthStateFilter *int32 `json:"HealthStateFilter,omitempty"`
	// ServiceFilters - Defines a list of filters that specify which services to be included in the returned cluster health chunk as children of the application. The services are returned only if the parent application matches a filter.
	// If the list is empty, no services are returned. All the services are used to evaluate the parent application aggregated health state, regardless of the input filters.
	// The application filter may specify multiple service filters.
	// For example, it can specify a filter to return all services with health state Error and another filter to always include a service identified by its service name.
	ServiceFilters *[]ServiceHealthStateFilter `json:"ServiceFilters,omitempty"`
	// DeployedApplicationFilters - Defines a list of filters that specify which deployed applications to be included in the returned cluster health chunk as children of the application. The deployed applications are returned only if the parent application matches a filter.
	// If the list is empty, no deployed applications are returned. All the deployed applications are used to evaluate the parent application aggregated health state, regardless of the input filters.
	// The application filter may specify multiple deployed application filters.
	// For example, it can specify a filter to return all deployed applications with health state Error and another filter to always include a deployed application on a specified node.
	DeployedApplicationFilters *[]DeployedApplicationHealthStateFilter `json:"DeployedApplicationFilters,omitempty"`
}

// ApplicationInfo information about a Service Fabric application.
type ApplicationInfo struct {
	autorest.Response `json:"-"`
	// ID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ID *string `json:"Id,omitempty"`
	// Name - The name of the application, including the 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// TypeName - The application type name as defined in the application manifest.
	TypeName *string `json:"TypeName,omitempty"`
	// TypeVersion - The version of the application type as defined in the application manifest.
	TypeVersion *string `json:"TypeVersion,omitempty"`
	// Status - The status of the application. Possible values include: 'ApplicationStatusInvalid', 'ApplicationStatusReady', 'ApplicationStatusUpgrading', 'ApplicationStatusCreating', 'ApplicationStatusDeleting', 'ApplicationStatusFailed'
	Status ApplicationStatus `json:"Status,omitempty"`
	// Parameters - List of application parameters with overridden values from their default values specified in the application manifest.
	Parameters *[]ApplicationParameter `json:"Parameters,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// ApplicationDefinitionKind - The mechanism used to define a Service Fabric application. Possible values include: 'Invalid', 'ServiceFabricApplicationDescription', 'Compose'
	ApplicationDefinitionKind ApplicationDefinitionKind `json:"ApplicationDefinitionKind,omitempty"`
}

// ApplicationLoadInfo load Information about a Service Fabric application.
type ApplicationLoadInfo struct {
	autorest.Response `json:"-"`
	// ID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ID *string `json:"Id,omitempty"`
	// MinimumNodes - The minimum number of nodes for this application.
	// It is the number of nodes where Service Fabric will reserve Capacity in the cluster which equals to ReservedLoad * MinimumNodes for this Application instance.
	// For applications that do not have application capacity defined this value will be zero.
	MinimumNodes *int64 `json:"MinimumNodes,omitempty"`
	// MaximumNodes - The maximum number of nodes where this application can be instantiated.
	// It is the number of nodes this application is allowed to span.
	// For applications that do not have application capacity defined this value will be zero.
	MaximumNodes *int64 `json:"MaximumNodes,omitempty"`
	// NodeCount - The number of nodes on which this application is instantiated.
	// For applications that do not have application capacity defined this value will be zero.
	NodeCount *int64 `json:"NodeCount,omitempty"`
	// ApplicationLoadMetricInformation - List of application capacity metric description.
	ApplicationLoadMetricInformation *[]ApplicationMetricDescription `json:"ApplicationLoadMetricInformation,omitempty"`
}

// ApplicationMetricDescription describes capacity information for a custom resource balancing metric. This
// can be used to limit the total consumption of this metric by the services of this application.
type ApplicationMetricDescription struct {
	// Name - The name of the metric.
	Name *string `json:"Name,omitempty"`
	// MaximumCapacity - The maximum node capacity for Service Fabric application.
	// This is the maximum Load for an instance of this application on a single node. Even if the capacity of node is greater than this value, Service Fabric will limit the total load of services within the application on each node to this value.
	// If set to zero, capacity for this metric is unlimited on each node.
	// When creating a new application with application capacity defined, the product of MaximumNodes and this value must always be smaller than or equal to TotalApplicationCapacity.
	// When updating existing application with application capacity, the product of MaximumNodes and this value must always be smaller than or equal to TotalApplicationCapacity.
	MaximumCapacity *int64 `json:"MaximumCapacity,omitempty"`
	// ReservationCapacity - The node reservation capacity for Service Fabric application.
	// This is the amount of load which is reserved on nodes which have instances of this application.
	// If MinimumNodes is specified, then the product of these values will be the capacity reserved in the cluster for the application.
	// If set to zero, no capacity is reserved for this metric.
	// When setting application capacity or when updating application capacity; this value must be smaller than or equal to MaximumCapacity for each metric.
	ReservationCapacity *int64 `json:"ReservationCapacity,omitempty"`
	// TotalApplicationCapacity - The total metric capacity for Service Fabric application.
	// This is the total metric capacity for this application in the cluster. Service Fabric will try to limit the sum of loads of services within the application to this value.
	// When creating a new application with application capacity defined, the product of MaximumNodes and MaximumCapacity must always be smaller than or equal to this value.
	TotalApplicationCapacity *int64 `json:"TotalApplicationCapacity,omitempty"`
}

// ApplicationNameInfo information about the application name.
type ApplicationNameInfo struct {
	autorest.Response `json:"-"`
	// ID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ID *string `json:"Id,omitempty"`
	// Name - The name of the application, including the 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
}

// ApplicationNewHealthReportEvent application Health Report Created event.
type ApplicationNewHealthReportEvent struct {
	// ApplicationInstanceID - Id of Application instance.
	ApplicationInstanceID *int64 `json:"ApplicationInstanceId,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) MarshalJSON() ([]byte, error) {
	anhre.Kind = KindApplicationNewHealthReport
	objectMap := make(map[string]interface{})
	if anhre.ApplicationInstanceID != nil {
		objectMap["ApplicationInstanceId"] = anhre.ApplicationInstanceID
	}
	if anhre.SourceID != nil {
		objectMap["SourceId"] = anhre.SourceID
	}
	if anhre.Property != nil {
		objectMap["Property"] = anhre.Property
	}
	if anhre.HealthState != nil {
		objectMap["HealthState"] = anhre.HealthState
	}
	if anhre.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = anhre.TimeToLiveMs
	}
	if anhre.SequenceNumber != nil {
		objectMap["SequenceNumber"] = anhre.SequenceNumber
	}
	if anhre.Description != nil {
		objectMap["Description"] = anhre.Description
	}
	if anhre.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = anhre.RemoveWhenExpired
	}
	if anhre.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = anhre.SourceUtcTimestamp
	}
	if anhre.ApplicationID != nil {
		objectMap["ApplicationId"] = anhre.ApplicationID
	}
	if anhre.EventInstanceID != nil {
		objectMap["EventInstanceId"] = anhre.EventInstanceID
	}
	if anhre.Category != nil {
		objectMap["Category"] = anhre.Category
	}
	if anhre.TimeStamp != nil {
		objectMap["TimeStamp"] = anhre.TimeStamp
	}
	if anhre.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = anhre.HasCorrelatedEvents
	}
	if anhre.Kind != "" {
		objectMap["Kind"] = anhre.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &anhre, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return &anhre, true
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationNewHealthReportEvent.
func (anhre ApplicationNewHealthReportEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &anhre, true
}

// ApplicationParameter describes an application parameter override to be applied when creating or
// upgrading an application.
type ApplicationParameter struct {
	// Key - The name of the parameter.
	Key *string `json:"Key,omitempty"`
	// Value - The value of the parameter.
	Value *string `json:"Value,omitempty"`
}

// ApplicationProcessExitedEvent process Exited event.
type ApplicationProcessExitedEvent struct {
	// ServiceName - Name of Service.
	ServiceName *string `json:"ServiceName,omitempty"`
	// ServicePackageName - Name of Service package.
	ServicePackageName *string `json:"ServicePackageName,omitempty"`
	// ServicePackageActivationID - Activation Id of Service package.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// IsExclusive - Indicates IsExclusive flag.
	IsExclusive *bool `json:"IsExclusive,omitempty"`
	// CodePackageName - Name of Code package.
	CodePackageName *string `json:"CodePackageName,omitempty"`
	// EntryPointType - Type of EntryPoint.
	EntryPointType *string `json:"EntryPointType,omitempty"`
	// ExeName - Name of executable.
	ExeName *string `json:"ExeName,omitempty"`
	// ProcessID - Process Id.
	ProcessID *int64 `json:"ProcessId,omitempty"`
	// HostID - Host Id.
	HostID *string `json:"HostId,omitempty"`
	// ExitCode - Exit code of process.
	ExitCode *int64 `json:"ExitCode,omitempty"`
	// UnexpectedTermination - Indicates if termination is unexpected.
	UnexpectedTermination *bool `json:"UnexpectedTermination,omitempty"`
	// StartTime - Start time of process.
	StartTime *date.Time `json:"StartTime,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) MarshalJSON() ([]byte, error) {
	apee.Kind = KindApplicationProcessExited
	objectMap := make(map[string]interface{})
	if apee.ServiceName != nil {
		objectMap["ServiceName"] = apee.ServiceName
	}
	if apee.ServicePackageName != nil {
		objectMap["ServicePackageName"] = apee.ServicePackageName
	}
	if apee.ServicePackageActivationID != nil {
		objectMap["ServicePackageActivationId"] = apee.ServicePackageActivationID
	}
	if apee.IsExclusive != nil {
		objectMap["IsExclusive"] = apee.IsExclusive
	}
	if apee.CodePackageName != nil {
		objectMap["CodePackageName"] = apee.CodePackageName
	}
	if apee.EntryPointType != nil {
		objectMap["EntryPointType"] = apee.EntryPointType
	}
	if apee.ExeName != nil {
		objectMap["ExeName"] = apee.ExeName
	}
	if apee.ProcessID != nil {
		objectMap["ProcessId"] = apee.ProcessID
	}
	if apee.HostID != nil {
		objectMap["HostId"] = apee.HostID
	}
	if apee.ExitCode != nil {
		objectMap["ExitCode"] = apee.ExitCode
	}
	if apee.UnexpectedTermination != nil {
		objectMap["UnexpectedTermination"] = apee.UnexpectedTermination
	}
	if apee.StartTime != nil {
		objectMap["StartTime"] = apee.StartTime
	}
	if apee.ApplicationID != nil {
		objectMap["ApplicationId"] = apee.ApplicationID
	}
	if apee.EventInstanceID != nil {
		objectMap["EventInstanceId"] = apee.EventInstanceID
	}
	if apee.Category != nil {
		objectMap["Category"] = apee.Category
	}
	if apee.TimeStamp != nil {
		objectMap["TimeStamp"] = apee.TimeStamp
	}
	if apee.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = apee.HasCorrelatedEvents
	}
	if apee.Kind != "" {
		objectMap["Kind"] = apee.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &apee, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return &apee, true
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationProcessExitedEvent.
func (apee ApplicationProcessExitedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &apee, true
}

// ApplicationProperties describes properties of a application resource.
type ApplicationProperties struct {
	// Description - User readable description of the application.
	Description *string `json:"description,omitempty"`
	// Services - Describes the services in the application. This property is used to create or modify services of the application. On get only the name of the service is returned. The service description can be obtained by querying for the service resource.
	Services *[]ServiceResourceDescription `json:"services,omitempty"`
	// Diagnostics - Describes the diagnostics definition and usage for an application resource.
	Diagnostics *DiagnosticsDescription `json:"diagnostics,omitempty"`
	// DebugParams - Internal - used by Visual Studio to setup the debugging session on the local development environment.
	DebugParams *string `json:"debugParams,omitempty"`
	// ServiceNames - READ-ONLY; Names of the services in the application.
	ServiceNames *[]string `json:"serviceNames,omitempty"`
	// Status - READ-ONLY; Status of the application. Possible values include: 'ResourceStatusUnknown', 'ResourceStatusReady', 'ResourceStatusUpgrading', 'ResourceStatusCreating', 'ResourceStatusDeleting', 'ResourceStatusFailed'
	Status ResourceStatus `json:"status,omitempty"`
	// StatusDetails - READ-ONLY; Gives additional information about the current status of the application.
	StatusDetails *string `json:"statusDetails,omitempty"`
	// HealthState - READ-ONLY; Describes the health state of an application resource. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"healthState,omitempty"`
	// UnhealthyEvaluation - READ-ONLY; When the application's health state is not 'Ok', this additional details from service fabric Health Manager for the user to know why the application is marked unhealthy.
	UnhealthyEvaluation *string `json:"unhealthyEvaluation,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationProperties.
func (ap ApplicationProperties) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if ap.Description != nil {
		objectMap["description"] = ap.Description
	}
	if ap.Services != nil {
		objectMap["services"] = ap.Services
	}
	if ap.Diagnostics != nil {
		objectMap["diagnostics"] = ap.Diagnostics
	}
	if ap.DebugParams != nil {
		objectMap["debugParams"] = ap.DebugParams
	}
	return json.Marshal(objectMap)
}

// ApplicationResourceDescription this type describes a application resource.
type ApplicationResourceDescription struct {
	autorest.Response `json:"-"`
	// Name - Name of the Application resource.
	Name *string `json:"name,omitempty"`
	// ApplicationProperties - Describes properties of a application resource.
	*ApplicationProperties `json:"properties,omitempty"`
	// Identity - Describes the identity of the application.
	Identity *IdentityDescription `json:"identity,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationResourceDescription.
func (ard ApplicationResourceDescription) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if ard.Name != nil {
		objectMap["name"] = ard.Name
	}
	if ard.ApplicationProperties != nil {
		objectMap["properties"] = ard.ApplicationProperties
	}
	if ard.Identity != nil {
		objectMap["identity"] = ard.Identity
	}
	return json.Marshal(objectMap)
}

// UnmarshalJSON is the custom unmarshaler for ApplicationResourceDescription struct.
func (ard *ApplicationResourceDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				ard.Name = &name
			}
		case "properties":
			if v != nil {
				var applicationProperties ApplicationProperties
				err = json.Unmarshal(*v, &applicationProperties)
				if err != nil {
					return err
				}
				ard.ApplicationProperties = &applicationProperties
			}
		case "identity":
			if v != nil {
				var identity IdentityDescription
				err = json.Unmarshal(*v, &identity)
				if err != nil {
					return err
				}
				ard.Identity = &identity
			}
		}
	}

	return nil
}

// ApplicationScopedVolume describes a volume whose lifetime is scoped to the application's lifetime.
type ApplicationScopedVolume struct {
	// CreationParameters - Describes parameters for creating application-scoped volumes.
	CreationParameters BasicApplicationScopedVolumeCreationParameters `json:"creationParameters,omitempty"`
	// Name - Name of the volume being referenced.
	Name *string `json:"name,omitempty"`
	// ReadOnly - The flag indicating whether the volume is read only. Default is 'false'.
	ReadOnly *bool `json:"readOnly,omitempty"`
	// DestinationPath - The path within the container at which the volume should be mounted. Only valid path characters are allowed.
	DestinationPath *string `json:"destinationPath,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ApplicationScopedVolume struct.
func (asv *ApplicationScopedVolume) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "creationParameters":
			if v != nil {
				creationParameters, err := unmarshalBasicApplicationScopedVolumeCreationParameters(*v)
				if err != nil {
					return err
				}
				asv.CreationParameters = creationParameters
			}
		case "name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				asv.Name = &name
			}
		case "readOnly":
			if v != nil {
				var readOnly bool
				err = json.Unmarshal(*v, &readOnly)
				if err != nil {
					return err
				}
				asv.ReadOnly = &readOnly
			}
		case "destinationPath":
			if v != nil {
				var destinationPath string
				err = json.Unmarshal(*v, &destinationPath)
				if err != nil {
					return err
				}
				asv.DestinationPath = &destinationPath
			}
		}
	}

	return nil
}

// BasicApplicationScopedVolumeCreationParameters describes parameters for creating application-scoped volumes.
type BasicApplicationScopedVolumeCreationParameters interface {
	AsApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk() (*ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk, bool)
	AsApplicationScopedVolumeCreationParameters() (*ApplicationScopedVolumeCreationParameters, bool)
}

// ApplicationScopedVolumeCreationParameters describes parameters for creating application-scoped volumes.
type ApplicationScopedVolumeCreationParameters struct {
	// Description - User readable description of the volume.
	Description *string `json:"description,omitempty"`
	// Kind - Possible values include: 'KindApplicationScopedVolumeCreationParameters', 'KindServiceFabricVolumeDisk'
	Kind KindBasicApplicationScopedVolumeCreationParameters `json:"kind,omitempty"`
}

func unmarshalBasicApplicationScopedVolumeCreationParameters(body []byte) (BasicApplicationScopedVolumeCreationParameters, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["kind"] {
	case string(KindServiceFabricVolumeDisk):
		var asvcpsfvd ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk
		err := json.Unmarshal(body, &asvcpsfvd)
		return asvcpsfvd, err
	default:
		var asvcp ApplicationScopedVolumeCreationParameters
		err := json.Unmarshal(body, &asvcp)
		return asvcp, err
	}
}
func unmarshalBasicApplicationScopedVolumeCreationParametersArray(body []byte) ([]BasicApplicationScopedVolumeCreationParameters, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	asvcpArray := make([]BasicApplicationScopedVolumeCreationParameters, len(rawMessages))

	for index, rawMessage := range rawMessages {
		asvcp, err := unmarshalBasicApplicationScopedVolumeCreationParameters(*rawMessage)
		if err != nil {
			return nil, err
		}
		asvcpArray[index] = asvcp
	}
	return asvcpArray, nil
}

// MarshalJSON is the custom marshaler for ApplicationScopedVolumeCreationParameters.
func (asvcp ApplicationScopedVolumeCreationParameters) MarshalJSON() ([]byte, error) {
	asvcp.Kind = KindApplicationScopedVolumeCreationParameters
	objectMap := make(map[string]interface{})
	if asvcp.Description != nil {
		objectMap["description"] = asvcp.Description
	}
	if asvcp.Kind != "" {
		objectMap["kind"] = asvcp.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk is the BasicApplicationScopedVolumeCreationParameters implementation for ApplicationScopedVolumeCreationParameters.
func (asvcp ApplicationScopedVolumeCreationParameters) AsApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk() (*ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk, bool) {
	return nil, false
}

// AsApplicationScopedVolumeCreationParameters is the BasicApplicationScopedVolumeCreationParameters implementation for ApplicationScopedVolumeCreationParameters.
func (asvcp ApplicationScopedVolumeCreationParameters) AsApplicationScopedVolumeCreationParameters() (*ApplicationScopedVolumeCreationParameters, bool) {
	return &asvcp, true
}

// AsBasicApplicationScopedVolumeCreationParameters is the BasicApplicationScopedVolumeCreationParameters implementation for ApplicationScopedVolumeCreationParameters.
func (asvcp ApplicationScopedVolumeCreationParameters) AsBasicApplicationScopedVolumeCreationParameters() (BasicApplicationScopedVolumeCreationParameters, bool) {
	return &asvcp, true
}

// ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk describes parameters for creating
// application-scoped volumes provided by Service Fabric Volume Disks
type ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk struct {
	// SizeDisk - Volume size. Possible values include: 'SizeTypesSmall', 'SizeTypesMedium', 'SizeTypesLarge'
	SizeDisk SizeTypes `json:"sizeDisk,omitempty"`
	// Description - User readable description of the volume.
	Description *string `json:"description,omitempty"`
	// Kind - Possible values include: 'KindApplicationScopedVolumeCreationParameters', 'KindServiceFabricVolumeDisk'
	Kind KindBasicApplicationScopedVolumeCreationParameters `json:"kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk.
func (asvcpsfvd ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk) MarshalJSON() ([]byte, error) {
	asvcpsfvd.Kind = KindServiceFabricVolumeDisk
	objectMap := make(map[string]interface{})
	if asvcpsfvd.SizeDisk != "" {
		objectMap["sizeDisk"] = asvcpsfvd.SizeDisk
	}
	if asvcpsfvd.Description != nil {
		objectMap["description"] = asvcpsfvd.Description
	}
	if asvcpsfvd.Kind != "" {
		objectMap["kind"] = asvcpsfvd.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk is the BasicApplicationScopedVolumeCreationParameters implementation for ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk.
func (asvcpsfvd ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk) AsApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk() (*ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk, bool) {
	return &asvcpsfvd, true
}

// AsApplicationScopedVolumeCreationParameters is the BasicApplicationScopedVolumeCreationParameters implementation for ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk.
func (asvcpsfvd ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk) AsApplicationScopedVolumeCreationParameters() (*ApplicationScopedVolumeCreationParameters, bool) {
	return nil, false
}

// AsBasicApplicationScopedVolumeCreationParameters is the BasicApplicationScopedVolumeCreationParameters implementation for ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk.
func (asvcpsfvd ApplicationScopedVolumeCreationParametersServiceFabricVolumeDisk) AsBasicApplicationScopedVolumeCreationParameters() (BasicApplicationScopedVolumeCreationParameters, bool) {
	return &asvcpsfvd, true
}

// ApplicationsHealthEvaluation represents health evaluation for applications, containing health
// evaluations for each unhealthy application that impacted current aggregated health state.
type ApplicationsHealthEvaluation struct {
	// MaxPercentUnhealthyApplications - Maximum allowed percentage of unhealthy applications from the ClusterHealthPolicy.
	MaxPercentUnhealthyApplications *int32 `json:"MaxPercentUnhealthyApplications,omitempty"`
	// TotalCount - Total number of applications from the health store.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state. Includes all the unhealthy ApplicationHealthEvaluation that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) MarshalJSON() ([]byte, error) {
	ahe.Kind = KindApplications
	objectMap := make(map[string]interface{})
	if ahe.MaxPercentUnhealthyApplications != nil {
		objectMap["MaxPercentUnhealthyApplications"] = ahe.MaxPercentUnhealthyApplications
	}
	if ahe.TotalCount != nil {
		objectMap["TotalCount"] = ahe.TotalCount
	}
	if ahe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = ahe.UnhealthyEvaluations
	}
	if ahe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = ahe.AggregatedHealthState
	}
	if ahe.Description != nil {
		objectMap["Description"] = ahe.Description
	}
	if ahe.Kind != "" {
		objectMap["Kind"] = ahe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return &ahe, true
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationsHealthEvaluation.
func (ahe ApplicationsHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &ahe, true
}

// ApplicationTypeApplicationsHealthEvaluation represents health evaluation for applications of a
// particular application type. The application type applications evaluation can be returned when cluster
// health evaluation returns unhealthy aggregated health state, either Error or Warning. It contains health
// evaluations for each unhealthy application of the included application type that impacted current
// aggregated health state.
type ApplicationTypeApplicationsHealthEvaluation struct {
	// ApplicationTypeName - The application type name as defined in the application manifest.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// MaxPercentUnhealthyApplications - Maximum allowed percentage of unhealthy applications for the application type, specified as an entry in ApplicationTypeHealthPolicyMap.
	MaxPercentUnhealthyApplications *int32 `json:"MaxPercentUnhealthyApplications,omitempty"`
	// TotalCount - Total number of applications of the application type found in the health store.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state. Includes all the unhealthy ApplicationHealthEvaluation of this application type that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) MarshalJSON() ([]byte, error) {
	atahe.Kind = KindApplicationTypeApplications
	objectMap := make(map[string]interface{})
	if atahe.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = atahe.ApplicationTypeName
	}
	if atahe.MaxPercentUnhealthyApplications != nil {
		objectMap["MaxPercentUnhealthyApplications"] = atahe.MaxPercentUnhealthyApplications
	}
	if atahe.TotalCount != nil {
		objectMap["TotalCount"] = atahe.TotalCount
	}
	if atahe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = atahe.UnhealthyEvaluations
	}
	if atahe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = atahe.AggregatedHealthState
	}
	if atahe.Description != nil {
		objectMap["Description"] = atahe.Description
	}
	if atahe.Kind != "" {
		objectMap["Kind"] = atahe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return &atahe, true
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for ApplicationTypeApplicationsHealthEvaluation.
func (atahe ApplicationTypeApplicationsHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &atahe, true
}

// ApplicationTypeHealthPolicyMapItem defines an item in ApplicationTypeHealthPolicyMap.
type ApplicationTypeHealthPolicyMapItem struct {
	// Key - The key of the application type health policy map item. This is the name of the application type.
	Key *string `json:"Key,omitempty"`
	// Value - The value of the application type health policy map item.
	// The max percent unhealthy applications allowed for the application type. Must be between zero and 100.
	Value *int32 `json:"Value,omitempty"`
}

// ApplicationTypeImageStorePath path description for the application package in the image store specified
// during the prior copy operation.
type ApplicationTypeImageStorePath struct {
	// ApplicationTypeBuildPath - The relative image store path to the application package.
	ApplicationTypeBuildPath *string `json:"ApplicationTypeBuildPath,omitempty"`
}

// ApplicationTypeInfo information about an application type.
type ApplicationTypeInfo struct {
	// Name - The application type name as defined in the application manifest.
	Name *string `json:"Name,omitempty"`
	// Version - The version of the application type as defined in the application manifest.
	Version *string `json:"Version,omitempty"`
	// DefaultParameterList - List of application type parameters that can be overridden when creating or updating the application.
	DefaultParameterList *[]ApplicationParameter `json:"DefaultParameterList,omitempty"`
	// Status - The status of the application type. Possible values include: 'ApplicationTypeStatusInvalid', 'ApplicationTypeStatusProvisioning', 'ApplicationTypeStatusAvailable', 'ApplicationTypeStatusUnprovisioning', 'ApplicationTypeStatusFailed'
	Status ApplicationTypeStatus `json:"Status,omitempty"`
	// StatusDetails - Additional detailed information about the status of the application type.
	StatusDetails *string `json:"StatusDetails,omitempty"`
	// ApplicationTypeDefinitionKind - The mechanism used to define a Service Fabric application type. Possible values include: 'ApplicationTypeDefinitionKindInvalid', 'ApplicationTypeDefinitionKindServiceFabricApplicationPackage', 'ApplicationTypeDefinitionKindCompose'
	ApplicationTypeDefinitionKind ApplicationTypeDefinitionKind `json:"ApplicationTypeDefinitionKind,omitempty"`
}

// ApplicationTypeManifest contains the manifest describing an application type registered in a Service
// Fabric cluster.
type ApplicationTypeManifest struct {
	autorest.Response `json:"-"`
	// Manifest - The XML manifest as a string.
	Manifest *string `json:"Manifest,omitempty"`
}

// ApplicationUpgradeCompletedEvent application Upgrade Completed event.
type ApplicationUpgradeCompletedEvent struct {
	// ApplicationTypeName - Application type name.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// ApplicationTypeVersion - Application type version.
	ApplicationTypeVersion *string `json:"ApplicationTypeVersion,omitempty"`
	// OverallUpgradeElapsedTimeInMs - Overall upgrade time in milli-seconds.
	OverallUpgradeElapsedTimeInMs *float64 `json:"OverallUpgradeElapsedTimeInMs,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) MarshalJSON() ([]byte, error) {
	auce.Kind = KindApplicationUpgradeCompleted
	objectMap := make(map[string]interface{})
	if auce.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = auce.ApplicationTypeName
	}
	if auce.ApplicationTypeVersion != nil {
		objectMap["ApplicationTypeVersion"] = auce.ApplicationTypeVersion
	}
	if auce.OverallUpgradeElapsedTimeInMs != nil {
		objectMap["OverallUpgradeElapsedTimeInMs"] = auce.OverallUpgradeElapsedTimeInMs
	}
	if auce.ApplicationID != nil {
		objectMap["ApplicationId"] = auce.ApplicationID
	}
	if auce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = auce.EventInstanceID
	}
	if auce.Category != nil {
		objectMap["Category"] = auce.Category
	}
	if auce.TimeStamp != nil {
		objectMap["TimeStamp"] = auce.TimeStamp
	}
	if auce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = auce.HasCorrelatedEvents
	}
	if auce.Kind != "" {
		objectMap["Kind"] = auce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &auce, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return &auce, true
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationUpgradeCompletedEvent.
func (auce ApplicationUpgradeCompletedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &auce, true
}

// ApplicationUpgradeDescription describes the parameters for an application upgrade. Note that upgrade
// description replaces the existing application description. This means that if the parameters are not
// specified, the existing parameters on the applications will be overwritten with the empty parameters
// list. This would result in the application using the default value of the parameters from the
// application manifest. If you do not want to change any existing parameter values, please get the
// application parameters first using the GetApplicationInfo query and then supply those values as
// Parameters in this ApplicationUpgradeDescription.
type ApplicationUpgradeDescription struct {
	// Name - The name of the target application, including the 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// TargetApplicationTypeVersion - The target application type version (found in the application manifest) for the application upgrade.
	TargetApplicationTypeVersion *string `json:"TargetApplicationTypeVersion,omitempty"`
	// Parameters - List of application parameters with overridden values from their default values specified in the application manifest.
	Parameters *[]ApplicationParameter `json:"Parameters,omitempty"`
	// UpgradeKind - The kind of upgrade out of the following possible values. Possible values include: 'UpgradeKindInvalid', 'UpgradeKindRolling'
	UpgradeKind UpgradeKind `json:"UpgradeKind,omitempty"`
	// RollingUpgradeMode - The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto, UnmonitoredManual, and Monitored. Possible values include: 'UpgradeModeInvalid', 'UpgradeModeUnmonitoredAuto', 'UpgradeModeUnmonitoredManual', 'UpgradeModeMonitored'
	RollingUpgradeMode UpgradeMode `json:"RollingUpgradeMode,omitempty"`
	// UpgradeReplicaSetCheckTimeoutInSeconds - The maximum amount of time to block processing of an upgrade domain and prevent loss of availability when there are unexpected issues. When this timeout expires, processing of the upgrade domain will proceed regardless of availability loss issues. The timeout is reset at the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive. (unsigned 32-bit integer).
	UpgradeReplicaSetCheckTimeoutInSeconds *int64 `json:"UpgradeReplicaSetCheckTimeoutInSeconds,omitempty"`
	// ForceRestart - If true, then processes are forcefully restarted during upgrade even when the code version has not changed (the upgrade only changes configuration or data).
	ForceRestart *bool `json:"ForceRestart,omitempty"`
	// SortOrder - Defines the order in which an upgrade proceeds through the cluster. Possible values include: 'UpgradeSortOrderInvalid', 'UpgradeSortOrderDefault', 'UpgradeSortOrderNumeric', 'UpgradeSortOrderLexicographical', 'UpgradeSortOrderReverseNumeric', 'UpgradeSortOrderReverseLexicographical'
	SortOrder UpgradeSortOrder `json:"SortOrder,omitempty"`
	// MonitoringPolicy - Describes the parameters for monitoring an upgrade in Monitored mode.
	MonitoringPolicy *MonitoringPolicyDescription `json:"MonitoringPolicy,omitempty"`
	// ApplicationHealthPolicy - Defines a health policy used to evaluate the health of an application or one of its children entities.
	ApplicationHealthPolicy *ApplicationHealthPolicy `json:"ApplicationHealthPolicy,omitempty"`
}

// ApplicationUpgradeDomainCompletedEvent application Upgrade Domain Completed event.
type ApplicationUpgradeDomainCompletedEvent struct {
	// ApplicationTypeName - Application type name.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// CurrentApplicationTypeVersion - Current Application type version.
	CurrentApplicationTypeVersion *string `json:"CurrentApplicationTypeVersion,omitempty"`
	// ApplicationTypeVersion - Target Application type version.
	ApplicationTypeVersion *string `json:"ApplicationTypeVersion,omitempty"`
	// UpgradeState - State of upgrade.
	UpgradeState *string `json:"UpgradeState,omitempty"`
	// UpgradeDomains - Upgrade domains.
	UpgradeDomains *string `json:"UpgradeDomains,omitempty"`
	// UpgradeDomainElapsedTimeInMs - Upgrade time of domain in milli-seconds.
	UpgradeDomainElapsedTimeInMs *float64 `json:"UpgradeDomainElapsedTimeInMs,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) MarshalJSON() ([]byte, error) {
	audce.Kind = KindApplicationUpgradeDomainCompleted
	objectMap := make(map[string]interface{})
	if audce.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = audce.ApplicationTypeName
	}
	if audce.CurrentApplicationTypeVersion != nil {
		objectMap["CurrentApplicationTypeVersion"] = audce.CurrentApplicationTypeVersion
	}
	if audce.ApplicationTypeVersion != nil {
		objectMap["ApplicationTypeVersion"] = audce.ApplicationTypeVersion
	}
	if audce.UpgradeState != nil {
		objectMap["UpgradeState"] = audce.UpgradeState
	}
	if audce.UpgradeDomains != nil {
		objectMap["UpgradeDomains"] = audce.UpgradeDomains
	}
	if audce.UpgradeDomainElapsedTimeInMs != nil {
		objectMap["UpgradeDomainElapsedTimeInMs"] = audce.UpgradeDomainElapsedTimeInMs
	}
	if audce.ApplicationID != nil {
		objectMap["ApplicationId"] = audce.ApplicationID
	}
	if audce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = audce.EventInstanceID
	}
	if audce.Category != nil {
		objectMap["Category"] = audce.Category
	}
	if audce.TimeStamp != nil {
		objectMap["TimeStamp"] = audce.TimeStamp
	}
	if audce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = audce.HasCorrelatedEvents
	}
	if audce.Kind != "" {
		objectMap["Kind"] = audce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &audce, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return &audce, true
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationUpgradeDomainCompletedEvent.
func (audce ApplicationUpgradeDomainCompletedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &audce, true
}

// ApplicationUpgradeProgressInfo describes the parameters for an application upgrade.
type ApplicationUpgradeProgressInfo struct {
	autorest.Response `json:"-"`
	// Name - The name of the target application, including the 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// TypeName - The application type name as defined in the application manifest.
	TypeName *string `json:"TypeName,omitempty"`
	// TargetApplicationTypeVersion - The target application type version (found in the application manifest) for the application upgrade.
	TargetApplicationTypeVersion *string `json:"TargetApplicationTypeVersion,omitempty"`
	// UpgradeDomains - List of upgrade domains and their statuses.
	UpgradeDomains *[]UpgradeDomainInfo `json:"UpgradeDomains,omitempty"`
	// UpgradeState - The state of the upgrade domain. Possible values include: 'UpgradeStateInvalid', 'UpgradeStateRollingBackInProgress', 'UpgradeStateRollingBackCompleted', 'UpgradeStateRollingForwardPending', 'UpgradeStateRollingForwardInProgress', 'UpgradeStateRollingForwardCompleted', 'UpgradeStateFailed'
	UpgradeState UpgradeState `json:"UpgradeState,omitempty"`
	// NextUpgradeDomain - The name of the next upgrade domain to be processed.
	NextUpgradeDomain *string `json:"NextUpgradeDomain,omitempty"`
	// RollingUpgradeMode - The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto, UnmonitoredManual, and Monitored. Possible values include: 'UpgradeModeInvalid', 'UpgradeModeUnmonitoredAuto', 'UpgradeModeUnmonitoredManual', 'UpgradeModeMonitored'
	RollingUpgradeMode UpgradeMode `json:"RollingUpgradeMode,omitempty"`
	// UpgradeDescription - Describes the parameters for an application upgrade. Note that upgrade description replaces the existing application description. This means that if the parameters are not specified, the existing parameters on the applications will be overwritten with the empty parameters list. This would result in the application using the default value of the parameters from the application manifest. If you do not want to change any existing parameter values, please get the application parameters first using the GetApplicationInfo query and then supply those values as Parameters in this ApplicationUpgradeDescription.
	UpgradeDescription *ApplicationUpgradeDescription `json:"UpgradeDescription,omitempty"`
	// UpgradeDurationInMilliseconds - The estimated total amount of time spent processing the overall upgrade.
	UpgradeDurationInMilliseconds *string `json:"UpgradeDurationInMilliseconds,omitempty"`
	// UpgradeDomainDurationInMilliseconds - The estimated total amount of time spent processing the current upgrade domain.
	UpgradeDomainDurationInMilliseconds *string `json:"UpgradeDomainDurationInMilliseconds,omitempty"`
	// UnhealthyEvaluations - List of health evaluations that resulted in the current aggregated health state.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// CurrentUpgradeDomainProgress - Information about the current in-progress upgrade domain.
	CurrentUpgradeDomainProgress *CurrentUpgradeDomainProgressInfo `json:"CurrentUpgradeDomainProgress,omitempty"`
	// StartTimestampUtc - The estimated UTC datetime when the upgrade started.
	StartTimestampUtc *string `json:"StartTimestampUtc,omitempty"`
	// FailureTimestampUtc - The estimated UTC datetime when the upgrade failed and FailureAction was executed.
	FailureTimestampUtc *string `json:"FailureTimestampUtc,omitempty"`
	// FailureReason - The cause of an upgrade failure that resulted in FailureAction being executed. Possible values include: 'None', 'Interrupted', 'HealthCheck', 'UpgradeDomainTimeout', 'OverallUpgradeTimeout'
	FailureReason FailureReason `json:"FailureReason,omitempty"`
	// UpgradeDomainProgressAtFailure - Information about the upgrade domain progress at the time of upgrade failure.
	UpgradeDomainProgressAtFailure *FailureUpgradeDomainProgressInfo `json:"UpgradeDomainProgressAtFailure,omitempty"`
	// UpgradeStatusDetails - Additional detailed information about the status of the pending upgrade.
	UpgradeStatusDetails *string `json:"UpgradeStatusDetails,omitempty"`
}

// ApplicationUpgradeRollbackCompletedEvent application Upgrade Rollback Completed event.
type ApplicationUpgradeRollbackCompletedEvent struct {
	// ApplicationTypeName - Application type name.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// ApplicationTypeVersion - Application type version.
	ApplicationTypeVersion *string `json:"ApplicationTypeVersion,omitempty"`
	// FailureReason - Describes reason of failure.
	FailureReason *string `json:"FailureReason,omitempty"`
	// OverallUpgradeElapsedTimeInMs - Overall upgrade time in milli-seconds.
	OverallUpgradeElapsedTimeInMs *float64 `json:"OverallUpgradeElapsedTimeInMs,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) MarshalJSON() ([]byte, error) {
	aurce.Kind = KindApplicationUpgradeRollbackCompleted
	objectMap := make(map[string]interface{})
	if aurce.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = aurce.ApplicationTypeName
	}
	if aurce.ApplicationTypeVersion != nil {
		objectMap["ApplicationTypeVersion"] = aurce.ApplicationTypeVersion
	}
	if aurce.FailureReason != nil {
		objectMap["FailureReason"] = aurce.FailureReason
	}
	if aurce.OverallUpgradeElapsedTimeInMs != nil {
		objectMap["OverallUpgradeElapsedTimeInMs"] = aurce.OverallUpgradeElapsedTimeInMs
	}
	if aurce.ApplicationID != nil {
		objectMap["ApplicationId"] = aurce.ApplicationID
	}
	if aurce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = aurce.EventInstanceID
	}
	if aurce.Category != nil {
		objectMap["Category"] = aurce.Category
	}
	if aurce.TimeStamp != nil {
		objectMap["TimeStamp"] = aurce.TimeStamp
	}
	if aurce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = aurce.HasCorrelatedEvents
	}
	if aurce.Kind != "" {
		objectMap["Kind"] = aurce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &aurce, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return &aurce, true
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackCompletedEvent.
func (aurce ApplicationUpgradeRollbackCompletedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &aurce, true
}

// ApplicationUpgradeRollbackStartedEvent application Upgrade Rollback Started event.
type ApplicationUpgradeRollbackStartedEvent struct {
	// ApplicationTypeName - Application type name.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// CurrentApplicationTypeVersion - Current Application type version.
	CurrentApplicationTypeVersion *string `json:"CurrentApplicationTypeVersion,omitempty"`
	// ApplicationTypeVersion - Target Application type version.
	ApplicationTypeVersion *string `json:"ApplicationTypeVersion,omitempty"`
	// FailureReason - Describes reason of failure.
	FailureReason *string `json:"FailureReason,omitempty"`
	// OverallUpgradeElapsedTimeInMs - Overall upgrade time in milli-seconds.
	OverallUpgradeElapsedTimeInMs *float64 `json:"OverallUpgradeElapsedTimeInMs,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) MarshalJSON() ([]byte, error) {
	aurse.Kind = KindApplicationUpgradeRollbackStarted
	objectMap := make(map[string]interface{})
	if aurse.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = aurse.ApplicationTypeName
	}
	if aurse.CurrentApplicationTypeVersion != nil {
		objectMap["CurrentApplicationTypeVersion"] = aurse.CurrentApplicationTypeVersion
	}
	if aurse.ApplicationTypeVersion != nil {
		objectMap["ApplicationTypeVersion"] = aurse.ApplicationTypeVersion
	}
	if aurse.FailureReason != nil {
		objectMap["FailureReason"] = aurse.FailureReason
	}
	if aurse.OverallUpgradeElapsedTimeInMs != nil {
		objectMap["OverallUpgradeElapsedTimeInMs"] = aurse.OverallUpgradeElapsedTimeInMs
	}
	if aurse.ApplicationID != nil {
		objectMap["ApplicationId"] = aurse.ApplicationID
	}
	if aurse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = aurse.EventInstanceID
	}
	if aurse.Category != nil {
		objectMap["Category"] = aurse.Category
	}
	if aurse.TimeStamp != nil {
		objectMap["TimeStamp"] = aurse.TimeStamp
	}
	if aurse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = aurse.HasCorrelatedEvents
	}
	if aurse.Kind != "" {
		objectMap["Kind"] = aurse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &aurse, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return &aurse, true
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationUpgradeRollbackStartedEvent.
func (aurse ApplicationUpgradeRollbackStartedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &aurse, true
}

// ApplicationUpgradeStartedEvent application Upgrade Started event.
type ApplicationUpgradeStartedEvent struct {
	// ApplicationTypeName - Application type name.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// CurrentApplicationTypeVersion - Current Application type version.
	CurrentApplicationTypeVersion *string `json:"CurrentApplicationTypeVersion,omitempty"`
	// ApplicationTypeVersion - Target Application type version.
	ApplicationTypeVersion *string `json:"ApplicationTypeVersion,omitempty"`
	// UpgradeType - Type of upgrade.
	UpgradeType *string `json:"UpgradeType,omitempty"`
	// RollingUpgradeMode - Mode of upgrade.
	RollingUpgradeMode *string `json:"RollingUpgradeMode,omitempty"`
	// FailureAction - Action if failed.
	FailureAction *string `json:"FailureAction,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) MarshalJSON() ([]byte, error) {
	ause.Kind = KindApplicationUpgradeStarted
	objectMap := make(map[string]interface{})
	if ause.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = ause.ApplicationTypeName
	}
	if ause.CurrentApplicationTypeVersion != nil {
		objectMap["CurrentApplicationTypeVersion"] = ause.CurrentApplicationTypeVersion
	}
	if ause.ApplicationTypeVersion != nil {
		objectMap["ApplicationTypeVersion"] = ause.ApplicationTypeVersion
	}
	if ause.UpgradeType != nil {
		objectMap["UpgradeType"] = ause.UpgradeType
	}
	if ause.RollingUpgradeMode != nil {
		objectMap["RollingUpgradeMode"] = ause.RollingUpgradeMode
	}
	if ause.FailureAction != nil {
		objectMap["FailureAction"] = ause.FailureAction
	}
	if ause.ApplicationID != nil {
		objectMap["ApplicationId"] = ause.ApplicationID
	}
	if ause.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ause.EventInstanceID
	}
	if ause.Category != nil {
		objectMap["Category"] = ause.Category
	}
	if ause.TimeStamp != nil {
		objectMap["TimeStamp"] = ause.TimeStamp
	}
	if ause.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ause.HasCorrelatedEvents
	}
	if ause.Kind != "" {
		objectMap["Kind"] = ause.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &ause, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return &ause, true
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ApplicationUpgradeStartedEvent.
func (ause ApplicationUpgradeStartedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ause, true
}

// ApplicationUpgradeUpdateDescription describes the parameters for updating an ongoing application
// upgrade.
type ApplicationUpgradeUpdateDescription struct {
	// Name - The name of the application, including the 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// UpgradeKind - The kind of upgrade out of the following possible values. Possible values include: 'UpgradeKindInvalid', 'UpgradeKindRolling'
	UpgradeKind UpgradeKind `json:"UpgradeKind,omitempty"`
	// ApplicationHealthPolicy - Defines a health policy used to evaluate the health of an application or one of its children entities.
	ApplicationHealthPolicy *ApplicationHealthPolicy `json:"ApplicationHealthPolicy,omitempty"`
	// UpdateDescription - Describes the parameters for updating a rolling upgrade of application or cluster.
	UpdateDescription *RollingUpgradeUpdateDescription `json:"UpdateDescription,omitempty"`
}

// BasicAutoScalingMechanism describes the mechanism for performing auto scaling operation. Derived classes will
// describe the actual mechanism.
type BasicAutoScalingMechanism interface {
	AsAddRemoveReplicaScalingMechanism() (*AddRemoveReplicaScalingMechanism, bool)
	AsAutoScalingMechanism() (*AutoScalingMechanism, bool)
}

// AutoScalingMechanism describes the mechanism for performing auto scaling operation. Derived classes will
// describe the actual mechanism.
type AutoScalingMechanism struct {
	// Kind - Possible values include: 'KindAutoScalingMechanism', 'KindAddRemoveReplica'
	Kind KindBasicAutoScalingMechanism `json:"kind,omitempty"`
}

func unmarshalBasicAutoScalingMechanism(body []byte) (BasicAutoScalingMechanism, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["kind"] {
	case string(KindAddRemoveReplica):
		var arrsm AddRemoveReplicaScalingMechanism
		err := json.Unmarshal(body, &arrsm)
		return arrsm, err
	default:
		var asm AutoScalingMechanism
		err := json.Unmarshal(body, &asm)
		return asm, err
	}
}
func unmarshalBasicAutoScalingMechanismArray(body []byte) ([]BasicAutoScalingMechanism, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	asmArray := make([]BasicAutoScalingMechanism, len(rawMessages))

	for index, rawMessage := range rawMessages {
		asm, err := unmarshalBasicAutoScalingMechanism(*rawMessage)
		if err != nil {
			return nil, err
		}
		asmArray[index] = asm
	}
	return asmArray, nil
}

// MarshalJSON is the custom marshaler for AutoScalingMechanism.
func (asm AutoScalingMechanism) MarshalJSON() ([]byte, error) {
	asm.Kind = KindAutoScalingMechanism
	objectMap := make(map[string]interface{})
	if asm.Kind != "" {
		objectMap["kind"] = asm.Kind
	}
	return json.Marshal(objectMap)
}

// AsAddRemoveReplicaScalingMechanism is the BasicAutoScalingMechanism implementation for AutoScalingMechanism.
func (asm AutoScalingMechanism) AsAddRemoveReplicaScalingMechanism() (*AddRemoveReplicaScalingMechanism, bool) {
	return nil, false
}

// AsAutoScalingMechanism is the BasicAutoScalingMechanism implementation for AutoScalingMechanism.
func (asm AutoScalingMechanism) AsAutoScalingMechanism() (*AutoScalingMechanism, bool) {
	return &asm, true
}

// AsBasicAutoScalingMechanism is the BasicAutoScalingMechanism implementation for AutoScalingMechanism.
func (asm AutoScalingMechanism) AsBasicAutoScalingMechanism() (BasicAutoScalingMechanism, bool) {
	return &asm, true
}

// BasicAutoScalingMetric describes the metric that is used for triggering auto scaling operation. Derived classes will
// describe resources or metrics.
type BasicAutoScalingMetric interface {
	AsAutoScalingResourceMetric() (*AutoScalingResourceMetric, bool)
	AsAutoScalingMetric() (*AutoScalingMetric, bool)
}

// AutoScalingMetric describes the metric that is used for triggering auto scaling operation. Derived classes
// will describe resources or metrics.
type AutoScalingMetric struct {
	// Kind - Possible values include: 'KindAutoScalingMetric', 'KindResource'
	Kind KindBasicAutoScalingMetric `json:"kind,omitempty"`
}

func unmarshalBasicAutoScalingMetric(body []byte) (BasicAutoScalingMetric, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["kind"] {
	case string(KindResource):
		var asrm AutoScalingResourceMetric
		err := json.Unmarshal(body, &asrm)
		return asrm, err
	default:
		var asm AutoScalingMetric
		err := json.Unmarshal(body, &asm)
		return asm, err
	}
}
func unmarshalBasicAutoScalingMetricArray(body []byte) ([]BasicAutoScalingMetric, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	asmArray := make([]BasicAutoScalingMetric, len(rawMessages))

	for index, rawMessage := range rawMessages {
		asm, err := unmarshalBasicAutoScalingMetric(*rawMessage)
		if err != nil {
			return nil, err
		}
		asmArray[index] = asm
	}
	return asmArray, nil
}

// MarshalJSON is the custom marshaler for AutoScalingMetric.
func (asm AutoScalingMetric) MarshalJSON() ([]byte, error) {
	asm.Kind = KindAutoScalingMetric
	objectMap := make(map[string]interface{})
	if asm.Kind != "" {
		objectMap["kind"] = asm.Kind
	}
	return json.Marshal(objectMap)
}

// AsAutoScalingResourceMetric is the BasicAutoScalingMetric implementation for AutoScalingMetric.
func (asm AutoScalingMetric) AsAutoScalingResourceMetric() (*AutoScalingResourceMetric, bool) {
	return nil, false
}

// AsAutoScalingMetric is the BasicAutoScalingMetric implementation for AutoScalingMetric.
func (asm AutoScalingMetric) AsAutoScalingMetric() (*AutoScalingMetric, bool) {
	return &asm, true
}

// AsBasicAutoScalingMetric is the BasicAutoScalingMetric implementation for AutoScalingMetric.
func (asm AutoScalingMetric) AsBasicAutoScalingMetric() (BasicAutoScalingMetric, bool) {
	return &asm, true
}

// AutoScalingPolicy describes the auto scaling policy
type AutoScalingPolicy struct {
	// Name - The name of the auto scaling policy.
	Name *string `json:"name,omitempty"`
	// Trigger - Determines when auto scaling operation will be invoked.
	Trigger BasicAutoScalingTrigger `json:"trigger,omitempty"`
	// Mechanism - The mechanism that is used to scale when auto scaling operation is invoked.
	Mechanism BasicAutoScalingMechanism `json:"mechanism,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for AutoScalingPolicy struct.
func (asp *AutoScalingPolicy) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				asp.Name = &name
			}
		case "trigger":
			if v != nil {
				trigger, err := unmarshalBasicAutoScalingTrigger(*v)
				if err != nil {
					return err
				}
				asp.Trigger = trigger
			}
		case "mechanism":
			if v != nil {
				mechanism, err := unmarshalBasicAutoScalingMechanism(*v)
				if err != nil {
					return err
				}
				asp.Mechanism = mechanism
			}
		}
	}

	return nil
}

// AutoScalingResourceMetric describes the resource that is used for triggering auto scaling.
type AutoScalingResourceMetric struct {
	// Name - Name of the resource. Possible values include: 'CPU', 'MemoryInGB'
	Name AutoScalingResourceMetricName `json:"name,omitempty"`
	// Kind - Possible values include: 'KindAutoScalingMetric', 'KindResource'
	Kind KindBasicAutoScalingMetric `json:"kind,omitempty"`
}

// MarshalJSON is the custom marshaler for AutoScalingResourceMetric.
func (asrm AutoScalingResourceMetric) MarshalJSON() ([]byte, error) {
	asrm.Kind = KindResource
	objectMap := make(map[string]interface{})
	if asrm.Name != "" {
		objectMap["name"] = asrm.Name
	}
	if asrm.Kind != "" {
		objectMap["kind"] = asrm.Kind
	}
	return json.Marshal(objectMap)
}

// AsAutoScalingResourceMetric is the BasicAutoScalingMetric implementation for AutoScalingResourceMetric.
func (asrm AutoScalingResourceMetric) AsAutoScalingResourceMetric() (*AutoScalingResourceMetric, bool) {
	return &asrm, true
}

// AsAutoScalingMetric is the BasicAutoScalingMetric implementation for AutoScalingResourceMetric.
func (asrm AutoScalingResourceMetric) AsAutoScalingMetric() (*AutoScalingMetric, bool) {
	return nil, false
}

// AsBasicAutoScalingMetric is the BasicAutoScalingMetric implementation for AutoScalingResourceMetric.
func (asrm AutoScalingResourceMetric) AsBasicAutoScalingMetric() (BasicAutoScalingMetric, bool) {
	return &asrm, true
}

// BasicAutoScalingTrigger describes the trigger for performing auto scaling operation.
type BasicAutoScalingTrigger interface {
	AsAverageLoadScalingTrigger() (*AverageLoadScalingTrigger, bool)
	AsAutoScalingTrigger() (*AutoScalingTrigger, bool)
}

// AutoScalingTrigger describes the trigger for performing auto scaling operation.
type AutoScalingTrigger struct {
	// Kind - Possible values include: 'KindAutoScalingTrigger', 'KindAverageLoad'
	Kind KindBasicAutoScalingTrigger `json:"kind,omitempty"`
}

func unmarshalBasicAutoScalingTrigger(body []byte) (BasicAutoScalingTrigger, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["kind"] {
	case string(KindAverageLoad):
		var alst AverageLoadScalingTrigger
		err := json.Unmarshal(body, &alst)
		return alst, err
	default:
		var astVar AutoScalingTrigger
		err := json.Unmarshal(body, &astVar)
		return astVar, err
	}
}
func unmarshalBasicAutoScalingTriggerArray(body []byte) ([]BasicAutoScalingTrigger, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	astVarArray := make([]BasicAutoScalingTrigger, len(rawMessages))

	for index, rawMessage := range rawMessages {
		astVar, err := unmarshalBasicAutoScalingTrigger(*rawMessage)
		if err != nil {
			return nil, err
		}
		astVarArray[index] = astVar
	}
	return astVarArray, nil
}

// MarshalJSON is the custom marshaler for AutoScalingTrigger.
func (astVar AutoScalingTrigger) MarshalJSON() ([]byte, error) {
	astVar.Kind = KindAutoScalingTrigger
	objectMap := make(map[string]interface{})
	if astVar.Kind != "" {
		objectMap["kind"] = astVar.Kind
	}
	return json.Marshal(objectMap)
}

// AsAverageLoadScalingTrigger is the BasicAutoScalingTrigger implementation for AutoScalingTrigger.
func (astVar AutoScalingTrigger) AsAverageLoadScalingTrigger() (*AverageLoadScalingTrigger, bool) {
	return nil, false
}

// AsAutoScalingTrigger is the BasicAutoScalingTrigger implementation for AutoScalingTrigger.
func (astVar AutoScalingTrigger) AsAutoScalingTrigger() (*AutoScalingTrigger, bool) {
	return &astVar, true
}

// AsBasicAutoScalingTrigger is the BasicAutoScalingTrigger implementation for AutoScalingTrigger.
func (astVar AutoScalingTrigger) AsBasicAutoScalingTrigger() (BasicAutoScalingTrigger, bool) {
	return &astVar, true
}

// AverageLoadScalingTrigger describes the average load trigger used for auto scaling.
type AverageLoadScalingTrigger struct {
	// Metric - Description of the metric that is used for scaling.
	Metric BasicAutoScalingMetric `json:"metric,omitempty"`
	// LowerLoadThreshold - Lower load threshold (if average load is below this threshold, service will scale down).
	LowerLoadThreshold *float64 `json:"lowerLoadThreshold,omitempty"`
	// UpperLoadThreshold - Upper load threshold (if average load is above this threshold, service will scale up).
	UpperLoadThreshold *float64 `json:"upperLoadThreshold,omitempty"`
	// ScaleIntervalInSeconds - Scale interval that indicates how often will this trigger be checked.
	ScaleIntervalInSeconds *int32 `json:"scaleIntervalInSeconds,omitempty"`
	// Kind - Possible values include: 'KindAutoScalingTrigger', 'KindAverageLoad'
	Kind KindBasicAutoScalingTrigger `json:"kind,omitempty"`
}

// MarshalJSON is the custom marshaler for AverageLoadScalingTrigger.
func (alst AverageLoadScalingTrigger) MarshalJSON() ([]byte, error) {
	alst.Kind = KindAverageLoad
	objectMap := make(map[string]interface{})
	objectMap["metric"] = alst.Metric
	if alst.LowerLoadThreshold != nil {
		objectMap["lowerLoadThreshold"] = alst.LowerLoadThreshold
	}
	if alst.UpperLoadThreshold != nil {
		objectMap["upperLoadThreshold"] = alst.UpperLoadThreshold
	}
	if alst.ScaleIntervalInSeconds != nil {
		objectMap["scaleIntervalInSeconds"] = alst.ScaleIntervalInSeconds
	}
	if alst.Kind != "" {
		objectMap["kind"] = alst.Kind
	}
	return json.Marshal(objectMap)
}

// AsAverageLoadScalingTrigger is the BasicAutoScalingTrigger implementation for AverageLoadScalingTrigger.
func (alst AverageLoadScalingTrigger) AsAverageLoadScalingTrigger() (*AverageLoadScalingTrigger, bool) {
	return &alst, true
}

// AsAutoScalingTrigger is the BasicAutoScalingTrigger implementation for AverageLoadScalingTrigger.
func (alst AverageLoadScalingTrigger) AsAutoScalingTrigger() (*AutoScalingTrigger, bool) {
	return nil, false
}

// AsBasicAutoScalingTrigger is the BasicAutoScalingTrigger implementation for AverageLoadScalingTrigger.
func (alst AverageLoadScalingTrigger) AsBasicAutoScalingTrigger() (BasicAutoScalingTrigger, bool) {
	return &alst, true
}

// UnmarshalJSON is the custom unmarshaler for AverageLoadScalingTrigger struct.
func (alst *AverageLoadScalingTrigger) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "metric":
			if v != nil {
				metric, err := unmarshalBasicAutoScalingMetric(*v)
				if err != nil {
					return err
				}
				alst.Metric = metric
			}
		case "lowerLoadThreshold":
			if v != nil {
				var lowerLoadThreshold float64
				err = json.Unmarshal(*v, &lowerLoadThreshold)
				if err != nil {
					return err
				}
				alst.LowerLoadThreshold = &lowerLoadThreshold
			}
		case "upperLoadThreshold":
			if v != nil {
				var upperLoadThreshold float64
				err = json.Unmarshal(*v, &upperLoadThreshold)
				if err != nil {
					return err
				}
				alst.UpperLoadThreshold = &upperLoadThreshold
			}
		case "scaleIntervalInSeconds":
			if v != nil {
				var scaleIntervalInSeconds int32
				err = json.Unmarshal(*v, &scaleIntervalInSeconds)
				if err != nil {
					return err
				}
				alst.ScaleIntervalInSeconds = &scaleIntervalInSeconds
			}
		case "kind":
			if v != nil {
				var kind KindBasicAutoScalingTrigger
				err = json.Unmarshal(*v, &kind)
				if err != nil {
					return err
				}
				alst.Kind = kind
			}
		}
	}

	return nil
}

// AveragePartitionLoadScalingTrigger represents a scaling trigger related to an average load of a
// metric/resource of a partition.
type AveragePartitionLoadScalingTrigger struct {
	// MetricName - The name of the metric for which usage should be tracked.
	MetricName *string `json:"MetricName,omitempty"`
	// LowerLoadThreshold - The lower limit of the load below which a scale in operation should be performed.
	LowerLoadThreshold *string `json:"LowerLoadThreshold,omitempty"`
	// UpperLoadThreshold - The upper limit of the load beyond which a scale out operation should be performed.
	UpperLoadThreshold *string `json:"UpperLoadThreshold,omitempty"`
	// ScaleIntervalInSeconds - The period in seconds on which a decision is made whether to scale or not.
	ScaleIntervalInSeconds *int64 `json:"ScaleIntervalInSeconds,omitempty"`
	// Kind - Possible values include: 'KindScalingTriggerDescription', 'KindAveragePartitionLoad', 'KindAverageServiceLoad'
	Kind KindBasicScalingTriggerDescription `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for AveragePartitionLoadScalingTrigger.
func (aplst AveragePartitionLoadScalingTrigger) MarshalJSON() ([]byte, error) {
	aplst.Kind = KindAveragePartitionLoad
	objectMap := make(map[string]interface{})
	if aplst.MetricName != nil {
		objectMap["MetricName"] = aplst.MetricName
	}
	if aplst.LowerLoadThreshold != nil {
		objectMap["LowerLoadThreshold"] = aplst.LowerLoadThreshold
	}
	if aplst.UpperLoadThreshold != nil {
		objectMap["UpperLoadThreshold"] = aplst.UpperLoadThreshold
	}
	if aplst.ScaleIntervalInSeconds != nil {
		objectMap["ScaleIntervalInSeconds"] = aplst.ScaleIntervalInSeconds
	}
	if aplst.Kind != "" {
		objectMap["Kind"] = aplst.Kind
	}
	return json.Marshal(objectMap)
}

// AsAveragePartitionLoadScalingTrigger is the BasicScalingTriggerDescription implementation for AveragePartitionLoadScalingTrigger.
func (aplst AveragePartitionLoadScalingTrigger) AsAveragePartitionLoadScalingTrigger() (*AveragePartitionLoadScalingTrigger, bool) {
	return &aplst, true
}

// AsAverageServiceLoadScalingTrigger is the BasicScalingTriggerDescription implementation for AveragePartitionLoadScalingTrigger.
func (aplst AveragePartitionLoadScalingTrigger) AsAverageServiceLoadScalingTrigger() (*AverageServiceLoadScalingTrigger, bool) {
	return nil, false
}

// AsScalingTriggerDescription is the BasicScalingTriggerDescription implementation for AveragePartitionLoadScalingTrigger.
func (aplst AveragePartitionLoadScalingTrigger) AsScalingTriggerDescription() (*ScalingTriggerDescription, bool) {
	return nil, false
}

// AsBasicScalingTriggerDescription is the BasicScalingTriggerDescription implementation for AveragePartitionLoadScalingTrigger.
func (aplst AveragePartitionLoadScalingTrigger) AsBasicScalingTriggerDescription() (BasicScalingTriggerDescription, bool) {
	return &aplst, true
}

// AverageServiceLoadScalingTrigger represents a scaling policy related to an average load of a
// metric/resource of a service.
type AverageServiceLoadScalingTrigger struct {
	// MetricName - The name of the metric for which usage should be tracked.
	MetricName *string `json:"MetricName,omitempty"`
	// LowerLoadThreshold - The lower limit of the load below which a scale in operation should be performed.
	LowerLoadThreshold *string `json:"LowerLoadThreshold,omitempty"`
	// UpperLoadThreshold - The upper limit of the load beyond which a scale out operation should be performed.
	UpperLoadThreshold *string `json:"UpperLoadThreshold,omitempty"`
	// ScaleIntervalInSeconds - The period in seconds on which a decision is made whether to scale or not.
	ScaleIntervalInSeconds *int64 `json:"ScaleIntervalInSeconds,omitempty"`
	// Kind - Possible values include: 'KindScalingTriggerDescription', 'KindAveragePartitionLoad', 'KindAverageServiceLoad'
	Kind KindBasicScalingTriggerDescription `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for AverageServiceLoadScalingTrigger.
func (aslst AverageServiceLoadScalingTrigger) MarshalJSON() ([]byte, error) {
	aslst.Kind = KindAverageServiceLoad
	objectMap := make(map[string]interface{})
	if aslst.MetricName != nil {
		objectMap["MetricName"] = aslst.MetricName
	}
	if aslst.LowerLoadThreshold != nil {
		objectMap["LowerLoadThreshold"] = aslst.LowerLoadThreshold
	}
	if aslst.UpperLoadThreshold != nil {
		objectMap["UpperLoadThreshold"] = aslst.UpperLoadThreshold
	}
	if aslst.ScaleIntervalInSeconds != nil {
		objectMap["ScaleIntervalInSeconds"] = aslst.ScaleIntervalInSeconds
	}
	if aslst.Kind != "" {
		objectMap["Kind"] = aslst.Kind
	}
	return json.Marshal(objectMap)
}

// AsAveragePartitionLoadScalingTrigger is the BasicScalingTriggerDescription implementation for AverageServiceLoadScalingTrigger.
func (aslst AverageServiceLoadScalingTrigger) AsAveragePartitionLoadScalingTrigger() (*AveragePartitionLoadScalingTrigger, bool) {
	return nil, false
}

// AsAverageServiceLoadScalingTrigger is the BasicScalingTriggerDescription implementation for AverageServiceLoadScalingTrigger.
func (aslst AverageServiceLoadScalingTrigger) AsAverageServiceLoadScalingTrigger() (*AverageServiceLoadScalingTrigger, bool) {
	return &aslst, true
}

// AsScalingTriggerDescription is the BasicScalingTriggerDescription implementation for AverageServiceLoadScalingTrigger.
func (aslst AverageServiceLoadScalingTrigger) AsScalingTriggerDescription() (*ScalingTriggerDescription, bool) {
	return nil, false
}

// AsBasicScalingTriggerDescription is the BasicScalingTriggerDescription implementation for AverageServiceLoadScalingTrigger.
func (aslst AverageServiceLoadScalingTrigger) AsBasicScalingTriggerDescription() (BasicScalingTriggerDescription, bool) {
	return &aslst, true
}

// AzureBlobBackupStorageDescription describes the parameters for Azure blob store used for storing and
// enumerating backups.
type AzureBlobBackupStorageDescription struct {
	// ConnectionString - The connection string to connect to the Azure blob store.
	ConnectionString *string `json:"ConnectionString,omitempty"`
	// ContainerName - The name of the container in the blob store to store and enumerate backups from.
	ContainerName *string `json:"ContainerName,omitempty"`
	// FriendlyName - Friendly name for this backup storage.
	FriendlyName *string `json:"FriendlyName,omitempty"`
	// StorageKind - Possible values include: 'StorageKindBackupStorageDescription', 'StorageKindAzureBlobStore', 'StorageKindFileShare'
	StorageKind StorageKind `json:"StorageKind,omitempty"`
}

// MarshalJSON is the custom marshaler for AzureBlobBackupStorageDescription.
func (abbsd AzureBlobBackupStorageDescription) MarshalJSON() ([]byte, error) {
	abbsd.StorageKind = StorageKindAzureBlobStore
	objectMap := make(map[string]interface{})
	if abbsd.ConnectionString != nil {
		objectMap["ConnectionString"] = abbsd.ConnectionString
	}
	if abbsd.ContainerName != nil {
		objectMap["ContainerName"] = abbsd.ContainerName
	}
	if abbsd.FriendlyName != nil {
		objectMap["FriendlyName"] = abbsd.FriendlyName
	}
	if abbsd.StorageKind != "" {
		objectMap["StorageKind"] = abbsd.StorageKind
	}
	return json.Marshal(objectMap)
}

// AsAzureBlobBackupStorageDescription is the BasicBackupStorageDescription implementation for AzureBlobBackupStorageDescription.
func (abbsd AzureBlobBackupStorageDescription) AsAzureBlobBackupStorageDescription() (*AzureBlobBackupStorageDescription, bool) {
	return &abbsd, true
}

// AsFileShareBackupStorageDescription is the BasicBackupStorageDescription implementation for AzureBlobBackupStorageDescription.
func (abbsd AzureBlobBackupStorageDescription) AsFileShareBackupStorageDescription() (*FileShareBackupStorageDescription, bool) {
	return nil, false
}

// AsBackupStorageDescription is the BasicBackupStorageDescription implementation for AzureBlobBackupStorageDescription.
func (abbsd AzureBlobBackupStorageDescription) AsBackupStorageDescription() (*BackupStorageDescription, bool) {
	return nil, false
}

// AsBasicBackupStorageDescription is the BasicBackupStorageDescription implementation for AzureBlobBackupStorageDescription.
func (abbsd AzureBlobBackupStorageDescription) AsBasicBackupStorageDescription() (BasicBackupStorageDescription, bool) {
	return &abbsd, true
}

// AzureInternalMonitoringPipelineSinkDescription diagnostics settings for Geneva.
type AzureInternalMonitoringPipelineSinkDescription struct {
	// AccountName - Azure Internal monitoring pipeline account.
	AccountName *string `json:"accountName,omitempty"`
	// Namespace - Azure Internal monitoring pipeline account namespace.
	Namespace *string `json:"namespace,omitempty"`
	// MaConfigURL - Azure Internal monitoring agent configuration.
	MaConfigURL *string `json:"maConfigUrl,omitempty"`
	// FluentdConfigURL - Azure Internal monitoring agent fluentd configuration.
	FluentdConfigURL *string `json:"fluentdConfigUrl,omitempty"`
	// AutoKeyConfigURL - Azure Internal monitoring pipeline autokey associated with the certificate.
	AutoKeyConfigURL *string `json:"autoKeyConfigUrl,omitempty"`
	// Name - Name of the sink. This value is referenced by DiagnosticsReferenceDescription
	Name *string `json:"name,omitempty"`
	// Description - A description of the sink.
	Description *string `json:"description,omitempty"`
	// Kind - Possible values include: 'KindDiagnosticsSinkProperties', 'KindAzureInternalMonitoringPipeline'
	Kind KindBasicDiagnosticsSinkProperties `json:"kind,omitempty"`
}

// MarshalJSON is the custom marshaler for AzureInternalMonitoringPipelineSinkDescription.
func (aimpsd AzureInternalMonitoringPipelineSinkDescription) MarshalJSON() ([]byte, error) {
	aimpsd.Kind = KindAzureInternalMonitoringPipeline
	objectMap := make(map[string]interface{})
	if aimpsd.AccountName != nil {
		objectMap["accountName"] = aimpsd.AccountName
	}
	if aimpsd.Namespace != nil {
		objectMap["namespace"] = aimpsd.Namespace
	}
	if aimpsd.MaConfigURL != nil {
		objectMap["maConfigUrl"] = aimpsd.MaConfigURL
	}
	if aimpsd.FluentdConfigURL != nil {
		objectMap["fluentdConfigUrl"] = aimpsd.FluentdConfigURL
	}
	if aimpsd.AutoKeyConfigURL != nil {
		objectMap["autoKeyConfigUrl"] = aimpsd.AutoKeyConfigURL
	}
	if aimpsd.Name != nil {
		objectMap["name"] = aimpsd.Name
	}
	if aimpsd.Description != nil {
		objectMap["description"] = aimpsd.Description
	}
	if aimpsd.Kind != "" {
		objectMap["kind"] = aimpsd.Kind
	}
	return json.Marshal(objectMap)
}

// AsAzureInternalMonitoringPipelineSinkDescription is the BasicDiagnosticsSinkProperties implementation for AzureInternalMonitoringPipelineSinkDescription.
func (aimpsd AzureInternalMonitoringPipelineSinkDescription) AsAzureInternalMonitoringPipelineSinkDescription() (*AzureInternalMonitoringPipelineSinkDescription, bool) {
	return &aimpsd, true
}

// AsDiagnosticsSinkProperties is the BasicDiagnosticsSinkProperties implementation for AzureInternalMonitoringPipelineSinkDescription.
func (aimpsd AzureInternalMonitoringPipelineSinkDescription) AsDiagnosticsSinkProperties() (*DiagnosticsSinkProperties, bool) {
	return nil, false
}

// AsBasicDiagnosticsSinkProperties is the BasicDiagnosticsSinkProperties implementation for AzureInternalMonitoringPipelineSinkDescription.
func (aimpsd AzureInternalMonitoringPipelineSinkDescription) AsBasicDiagnosticsSinkProperties() (BasicDiagnosticsSinkProperties, bool) {
	return &aimpsd, true
}

// BasicBackupConfigurationInfo describes the backup configuration information.
type BasicBackupConfigurationInfo interface {
	AsApplicationBackupConfigurationInfo() (*ApplicationBackupConfigurationInfo, bool)
	AsServiceBackupConfigurationInfo() (*ServiceBackupConfigurationInfo, bool)
	AsPartitionBackupConfigurationInfo() (*PartitionBackupConfigurationInfo, bool)
	AsBackupConfigurationInfo() (*BackupConfigurationInfo, bool)
}

// BackupConfigurationInfo describes the backup configuration information.
type BackupConfigurationInfo struct {
	// PolicyName - The name of the backup policy which is applicable to this Service Fabric application or service or partition.
	PolicyName *string `json:"PolicyName,omitempty"`
	// PolicyInheritedFrom - Specifies the scope at which the backup policy is applied. Possible values include: 'BackupPolicyScopeInvalid', 'BackupPolicyScopePartition', 'BackupPolicyScopeService', 'BackupPolicyScopeApplication'
	PolicyInheritedFrom BackupPolicyScope `json:"PolicyInheritedFrom,omitempty"`
	// SuspensionInfo - Describes the backup suspension details.
	SuspensionInfo *BackupSuspensionInfo `json:"SuspensionInfo,omitempty"`
	// Kind - Possible values include: 'KindBasicBackupConfigurationInfoKindBackupConfigurationInfo', 'KindBasicBackupConfigurationInfoKindApplication', 'KindBasicBackupConfigurationInfoKindService', 'KindBasicBackupConfigurationInfoKindPartition'
	Kind KindBasicBackupConfigurationInfo `json:"Kind,omitempty"`
}

func unmarshalBasicBackupConfigurationInfo(body []byte) (BasicBackupConfigurationInfo, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindBasicBackupConfigurationInfoKindApplication):
		var abci ApplicationBackupConfigurationInfo
		err := json.Unmarshal(body, &abci)
		return abci, err
	case string(KindBasicBackupConfigurationInfoKindService):
		var sbci ServiceBackupConfigurationInfo
		err := json.Unmarshal(body, &sbci)
		return sbci, err
	case string(KindBasicBackupConfigurationInfoKindPartition):
		var pbci PartitionBackupConfigurationInfo
		err := json.Unmarshal(body, &pbci)
		return pbci, err
	default:
		var bci BackupConfigurationInfo
		err := json.Unmarshal(body, &bci)
		return bci, err
	}
}
func unmarshalBasicBackupConfigurationInfoArray(body []byte) ([]BasicBackupConfigurationInfo, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	bciArray := make([]BasicBackupConfigurationInfo, len(rawMessages))

	for index, rawMessage := range rawMessages {
		bci, err := unmarshalBasicBackupConfigurationInfo(*rawMessage)
		if err != nil {
			return nil, err
		}
		bciArray[index] = bci
	}
	return bciArray, nil
}

// MarshalJSON is the custom marshaler for BackupConfigurationInfo.
func (bci BackupConfigurationInfo) MarshalJSON() ([]byte, error) {
	bci.Kind = KindBasicBackupConfigurationInfoKindBackupConfigurationInfo
	objectMap := make(map[string]interface{})
	if bci.PolicyName != nil {
		objectMap["PolicyName"] = bci.PolicyName
	}
	if bci.PolicyInheritedFrom != "" {
		objectMap["PolicyInheritedFrom"] = bci.PolicyInheritedFrom
	}
	if bci.SuspensionInfo != nil {
		objectMap["SuspensionInfo"] = bci.SuspensionInfo
	}
	if bci.Kind != "" {
		objectMap["Kind"] = bci.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for BackupConfigurationInfo.
func (bci BackupConfigurationInfo) AsApplicationBackupConfigurationInfo() (*ApplicationBackupConfigurationInfo, bool) {
	return nil, false
}

// AsServiceBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for BackupConfigurationInfo.
func (bci BackupConfigurationInfo) AsServiceBackupConfigurationInfo() (*ServiceBackupConfigurationInfo, bool) {
	return nil, false
}

// AsPartitionBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for BackupConfigurationInfo.
func (bci BackupConfigurationInfo) AsPartitionBackupConfigurationInfo() (*PartitionBackupConfigurationInfo, bool) {
	return nil, false
}

// AsBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for BackupConfigurationInfo.
func (bci BackupConfigurationInfo) AsBackupConfigurationInfo() (*BackupConfigurationInfo, bool) {
	return &bci, true
}

// AsBasicBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for BackupConfigurationInfo.
func (bci BackupConfigurationInfo) AsBasicBackupConfigurationInfo() (BasicBackupConfigurationInfo, bool) {
	return &bci, true
}

// BasicBackupEntity describes the Service Fabric entity that is configured for backup.
type BasicBackupEntity interface {
	AsApplicationBackupEntity() (*ApplicationBackupEntity, bool)
	AsServiceBackupEntity() (*ServiceBackupEntity, bool)
	AsPartitionBackupEntity() (*PartitionBackupEntity, bool)
	AsBackupEntity() (*BackupEntity, bool)
}

// BackupEntity describes the Service Fabric entity that is configured for backup.
type BackupEntity struct {
	// EntityKind - Possible values include: 'EntityKindBackupEntity', 'EntityKindApplication1', 'EntityKindService1', 'EntityKindPartition1'
	EntityKind EntityKindBasicBackupEntity `json:"EntityKind,omitempty"`
}

func unmarshalBasicBackupEntity(body []byte) (BasicBackupEntity, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["EntityKind"] {
	case string(EntityKindApplication1):
		var abe ApplicationBackupEntity
		err := json.Unmarshal(body, &abe)
		return abe, err
	case string(EntityKindService1):
		var sbe ServiceBackupEntity
		err := json.Unmarshal(body, &sbe)
		return sbe, err
	case string(EntityKindPartition1):
		var pbe PartitionBackupEntity
		err := json.Unmarshal(body, &pbe)
		return pbe, err
	default:
		var be BackupEntity
		err := json.Unmarshal(body, &be)
		return be, err
	}
}
func unmarshalBasicBackupEntityArray(body []byte) ([]BasicBackupEntity, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	beArray := make([]BasicBackupEntity, len(rawMessages))

	for index, rawMessage := range rawMessages {
		be, err := unmarshalBasicBackupEntity(*rawMessage)
		if err != nil {
			return nil, err
		}
		beArray[index] = be
	}
	return beArray, nil
}

// MarshalJSON is the custom marshaler for BackupEntity.
func (be BackupEntity) MarshalJSON() ([]byte, error) {
	be.EntityKind = EntityKindBackupEntity
	objectMap := make(map[string]interface{})
	if be.EntityKind != "" {
		objectMap["EntityKind"] = be.EntityKind
	}
	return json.Marshal(objectMap)
}

// AsApplicationBackupEntity is the BasicBackupEntity implementation for BackupEntity.
func (be BackupEntity) AsApplicationBackupEntity() (*ApplicationBackupEntity, bool) {
	return nil, false
}

// AsServiceBackupEntity is the BasicBackupEntity implementation for BackupEntity.
func (be BackupEntity) AsServiceBackupEntity() (*ServiceBackupEntity, bool) {
	return nil, false
}

// AsPartitionBackupEntity is the BasicBackupEntity implementation for BackupEntity.
func (be BackupEntity) AsPartitionBackupEntity() (*PartitionBackupEntity, bool) {
	return nil, false
}

// AsBackupEntity is the BasicBackupEntity implementation for BackupEntity.
func (be BackupEntity) AsBackupEntity() (*BackupEntity, bool) {
	return &be, true
}

// AsBasicBackupEntity is the BasicBackupEntity implementation for BackupEntity.
func (be BackupEntity) AsBasicBackupEntity() (BasicBackupEntity, bool) {
	return &be, true
}

// BackupInfo represents a backup point which can be used to trigger a restore.
type BackupInfo struct {
	// BackupID - Unique backup ID .
	BackupID *uuid.UUID `json:"BackupId,omitempty"`
	// BackupChainID - Unique backup chain ID. All backups part of the same chain has the same backup chain id. A backup chain is comprised of 1 full backup and multiple incremental backups.
	BackupChainID *uuid.UUID `json:"BackupChainId,omitempty"`
	// ApplicationName - Name of the Service Fabric application this partition backup belongs to.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ServiceName - Name of the Service Fabric service this partition backup belongs to.
	ServiceName *string `json:"ServiceName,omitempty"`
	// PartitionInformation - Information about the partition to which this backup belongs to
	PartitionInformation BasicPartitionInformation `json:"PartitionInformation,omitempty"`
	// BackupLocation - Location of the backup, relative to the backup store.
	BackupLocation *string `json:"BackupLocation,omitempty"`
	// BackupType - Describes the type of backup, whether its full or incremental. Possible values include: 'BackupTypeInvalid', 'BackupTypeFull', 'BackupTypeIncremental'
	BackupType BackupType `json:"BackupType,omitempty"`
	// EpochOfLastBackupRecord - Epoch of the last record in this backup.
	EpochOfLastBackupRecord *Epoch `json:"EpochOfLastBackupRecord,omitempty"`
	// LsnOfLastBackupRecord - LSN of the last record in this backup.
	LsnOfLastBackupRecord *string `json:"LsnOfLastBackupRecord,omitempty"`
	// CreationTimeUtc - The date time when this backup was taken.
	CreationTimeUtc *date.Time `json:"CreationTimeUtc,omitempty"`
	// ServiceManifestVersion - Manifest Version of the service this partition backup belongs to.
	ServiceManifestVersion *string `json:"ServiceManifestVersion,omitempty"`
	// FailureError - Denotes the failure encountered in getting backup point information.
	FailureError *FabricErrorError `json:"FailureError,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for BackupInfo struct.
func (bi *BackupInfo) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "BackupId":
			if v != nil {
				var backupID uuid.UUID
				err = json.Unmarshal(*v, &backupID)
				if err != nil {
					return err
				}
				bi.BackupID = &backupID
			}
		case "BackupChainId":
			if v != nil {
				var backupChainID uuid.UUID
				err = json.Unmarshal(*v, &backupChainID)
				if err != nil {
					return err
				}
				bi.BackupChainID = &backupChainID
			}
		case "ApplicationName":
			if v != nil {
				var applicationName string
				err = json.Unmarshal(*v, &applicationName)
				if err != nil {
					return err
				}
				bi.ApplicationName = &applicationName
			}
		case "ServiceName":
			if v != nil {
				var serviceName string
				err = json.Unmarshal(*v, &serviceName)
				if err != nil {
					return err
				}
				bi.ServiceName = &serviceName
			}
		case "PartitionInformation":
			if v != nil {
				partitionInformation, err := unmarshalBasicPartitionInformation(*v)
				if err != nil {
					return err
				}
				bi.PartitionInformation = partitionInformation
			}
		case "BackupLocation":
			if v != nil {
				var backupLocation string
				err = json.Unmarshal(*v, &backupLocation)
				if err != nil {
					return err
				}
				bi.BackupLocation = &backupLocation
			}
		case "BackupType":
			if v != nil {
				var backupType BackupType
				err = json.Unmarshal(*v, &backupType)
				if err != nil {
					return err
				}
				bi.BackupType = backupType
			}
		case "EpochOfLastBackupRecord":
			if v != nil {
				var epochOfLastBackupRecord Epoch
				err = json.Unmarshal(*v, &epochOfLastBackupRecord)
				if err != nil {
					return err
				}
				bi.EpochOfLastBackupRecord = &epochOfLastBackupRecord
			}
		case "LsnOfLastBackupRecord":
			if v != nil {
				var lsnOfLastBackupRecord string
				err = json.Unmarshal(*v, &lsnOfLastBackupRecord)
				if err != nil {
					return err
				}
				bi.LsnOfLastBackupRecord = &lsnOfLastBackupRecord
			}
		case "CreationTimeUtc":
			if v != nil {
				var creationTimeUtc date.Time
				err = json.Unmarshal(*v, &creationTimeUtc)
				if err != nil {
					return err
				}
				bi.CreationTimeUtc = &creationTimeUtc
			}
		case "ServiceManifestVersion":
			if v != nil {
				var serviceManifestVersion string
				err = json.Unmarshal(*v, &serviceManifestVersion)
				if err != nil {
					return err
				}
				bi.ServiceManifestVersion = &serviceManifestVersion
			}
		case "FailureError":
			if v != nil {
				var failureError FabricErrorError
				err = json.Unmarshal(*v, &failureError)
				if err != nil {
					return err
				}
				bi.FailureError = &failureError
			}
		}
	}

	return nil
}

// BackupPartitionDescription describes the parameters for triggering partition's backup.
type BackupPartitionDescription struct {
	// BackupStorage - Specifies the details of the backup storage where to save the backup.
	BackupStorage BasicBackupStorageDescription `json:"BackupStorage,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for BackupPartitionDescription struct.
func (bpd *BackupPartitionDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "BackupStorage":
			if v != nil {
				backupStorage, err := unmarshalBasicBackupStorageDescription(*v)
				if err != nil {
					return err
				}
				bpd.BackupStorage = backupStorage
			}
		}
	}

	return nil
}

// BackupPolicyDescription describes a backup policy for configuring periodic backup.
type BackupPolicyDescription struct {
	autorest.Response `json:"-"`
	// Name - The unique name identifying this backup policy.
	Name *string `json:"Name,omitempty"`
	// AutoRestoreOnDataLoss - Specifies whether to trigger restore automatically using the latest available backup in case the partition experiences a data loss event.
	AutoRestoreOnDataLoss *bool `json:"AutoRestoreOnDataLoss,omitempty"`
	// MaxIncrementalBackups - Defines the maximum number of incremental backups to be taken between two full backups. This is just the upper limit. A full backup may be taken before specified number of incremental backups are completed in one of the following conditions
	// - The replica has never taken a full backup since it has become primary,
	// - Some of the log records since the last backup has been truncated, or
	// - Replica passed the MaxAccumulatedBackupLogSizeInMB limit.
	MaxIncrementalBackups *int32 `json:"MaxIncrementalBackups,omitempty"`
	// Schedule - Describes the backup schedule parameters.
	Schedule BasicBackupScheduleDescription `json:"Schedule,omitempty"`
	// Storage - Describes the details of backup storage where to store the periodic backups.
	Storage BasicBackupStorageDescription `json:"Storage,omitempty"`
	// RetentionPolicy - Describes the policy to retain backups in storage.
	RetentionPolicy BasicBasicRetentionPolicyDescription `json:"RetentionPolicy,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for BackupPolicyDescription struct.
func (bpd *BackupPolicyDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "Name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				bpd.Name = &name
			}
		case "AutoRestoreOnDataLoss":
			if v != nil {
				var autoRestoreOnDataLoss bool
				err = json.Unmarshal(*v, &autoRestoreOnDataLoss)
				if err != nil {
					return err
				}
				bpd.AutoRestoreOnDataLoss = &autoRestoreOnDataLoss
			}
		case "MaxIncrementalBackups":
			if v != nil {
				var maxIncrementalBackups int32
				err = json.Unmarshal(*v, &maxIncrementalBackups)
				if err != nil {
					return err
				}
				bpd.MaxIncrementalBackups = &maxIncrementalBackups
			}
		case "Schedule":
			if v != nil {
				schedule, err := unmarshalBasicBackupScheduleDescription(*v)
				if err != nil {
					return err
				}
				bpd.Schedule = schedule
			}
		case "Storage":
			if v != nil {
				storage, err := unmarshalBasicBackupStorageDescription(*v)
				if err != nil {
					return err
				}
				bpd.Storage = storage
			}
		case "RetentionPolicy":
			if v != nil {
				retentionPolicy, err := unmarshalBasicBasicRetentionPolicyDescription(*v)
				if err != nil {
					return err
				}
				bpd.RetentionPolicy = retentionPolicy
			}
		}
	}

	return nil
}

// BackupProgressInfo describes the progress of a partition's backup.
type BackupProgressInfo struct {
	autorest.Response `json:"-"`
	// BackupState - Represents the current state of the partition backup operation. Possible values include: 'BackupStateInvalid', 'BackupStateAccepted', 'BackupStateBackupInProgress', 'BackupStateSuccess', 'BackupStateFailure', 'BackupStateTimeout'
	BackupState BackupState `json:"BackupState,omitempty"`
	// TimeStampUtc - TimeStamp in UTC when operation succeeded or failed.
	TimeStampUtc *date.Time `json:"TimeStampUtc,omitempty"`
	// BackupID - Unique ID of the newly created backup.
	BackupID *uuid.UUID `json:"BackupId,omitempty"`
	// BackupLocation - Location, relative to the backup store, of the newly created backup.
	BackupLocation *string `json:"BackupLocation,omitempty"`
	// EpochOfLastBackupRecord - Specifies the epoch of the last record included in backup.
	EpochOfLastBackupRecord *Epoch `json:"EpochOfLastBackupRecord,omitempty"`
	// LsnOfLastBackupRecord - The LSN of last record included in backup.
	LsnOfLastBackupRecord *string `json:"LsnOfLastBackupRecord,omitempty"`
	// FailureError - Denotes the failure encountered in performing backup operation.
	FailureError *FabricErrorError `json:"FailureError,omitempty"`
}

// BasicBackupScheduleDescription describes the backup schedule parameters.
type BasicBackupScheduleDescription interface {
	AsFrequencyBasedBackupScheduleDescription() (*FrequencyBasedBackupScheduleDescription, bool)
	AsTimeBasedBackupScheduleDescription() (*TimeBasedBackupScheduleDescription, bool)
	AsBackupScheduleDescription() (*BackupScheduleDescription, bool)
}

// BackupScheduleDescription describes the backup schedule parameters.
type BackupScheduleDescription struct {
	// ScheduleKind - Possible values include: 'ScheduleKindBackupScheduleDescription', 'ScheduleKindFrequencyBased', 'ScheduleKindTimeBased'
	ScheduleKind ScheduleKind `json:"ScheduleKind,omitempty"`
}

func unmarshalBasicBackupScheduleDescription(body []byte) (BasicBackupScheduleDescription, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ScheduleKind"] {
	case string(ScheduleKindFrequencyBased):
		var fbbsd FrequencyBasedBackupScheduleDescription
		err := json.Unmarshal(body, &fbbsd)
		return fbbsd, err
	case string(ScheduleKindTimeBased):
		var tbbsd TimeBasedBackupScheduleDescription
		err := json.Unmarshal(body, &tbbsd)
		return tbbsd, err
	default:
		var bsd BackupScheduleDescription
		err := json.Unmarshal(body, &bsd)
		return bsd, err
	}
}
func unmarshalBasicBackupScheduleDescriptionArray(body []byte) ([]BasicBackupScheduleDescription, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	bsdArray := make([]BasicBackupScheduleDescription, len(rawMessages))

	for index, rawMessage := range rawMessages {
		bsd, err := unmarshalBasicBackupScheduleDescription(*rawMessage)
		if err != nil {
			return nil, err
		}
		bsdArray[index] = bsd
	}
	return bsdArray, nil
}

// MarshalJSON is the custom marshaler for BackupScheduleDescription.
func (bsd BackupScheduleDescription) MarshalJSON() ([]byte, error) {
	bsd.ScheduleKind = ScheduleKindBackupScheduleDescription
	objectMap := make(map[string]interface{})
	if bsd.ScheduleKind != "" {
		objectMap["ScheduleKind"] = bsd.ScheduleKind
	}
	return json.Marshal(objectMap)
}

// AsFrequencyBasedBackupScheduleDescription is the BasicBackupScheduleDescription implementation for BackupScheduleDescription.
func (bsd BackupScheduleDescription) AsFrequencyBasedBackupScheduleDescription() (*FrequencyBasedBackupScheduleDescription, bool) {
	return nil, false
}

// AsTimeBasedBackupScheduleDescription is the BasicBackupScheduleDescription implementation for BackupScheduleDescription.
func (bsd BackupScheduleDescription) AsTimeBasedBackupScheduleDescription() (*TimeBasedBackupScheduleDescription, bool) {
	return nil, false
}

// AsBackupScheduleDescription is the BasicBackupScheduleDescription implementation for BackupScheduleDescription.
func (bsd BackupScheduleDescription) AsBackupScheduleDescription() (*BackupScheduleDescription, bool) {
	return &bsd, true
}

// AsBasicBackupScheduleDescription is the BasicBackupScheduleDescription implementation for BackupScheduleDescription.
func (bsd BackupScheduleDescription) AsBasicBackupScheduleDescription() (BasicBackupScheduleDescription, bool) {
	return &bsd, true
}

// BasicBackupStorageDescription describes the parameters for the backup storage.
type BasicBackupStorageDescription interface {
	AsAzureBlobBackupStorageDescription() (*AzureBlobBackupStorageDescription, bool)
	AsFileShareBackupStorageDescription() (*FileShareBackupStorageDescription, bool)
	AsBackupStorageDescription() (*BackupStorageDescription, bool)
}

// BackupStorageDescription describes the parameters for the backup storage.
type BackupStorageDescription struct {
	// FriendlyName - Friendly name for this backup storage.
	FriendlyName *string `json:"FriendlyName,omitempty"`
	// StorageKind - Possible values include: 'StorageKindBackupStorageDescription', 'StorageKindAzureBlobStore', 'StorageKindFileShare'
	StorageKind StorageKind `json:"StorageKind,omitempty"`
}

func unmarshalBasicBackupStorageDescription(body []byte) (BasicBackupStorageDescription, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["StorageKind"] {
	case string(StorageKindAzureBlobStore):
		var abbsd AzureBlobBackupStorageDescription
		err := json.Unmarshal(body, &abbsd)
		return abbsd, err
	case string(StorageKindFileShare):
		var fsbsd FileShareBackupStorageDescription
		err := json.Unmarshal(body, &fsbsd)
		return fsbsd, err
	default:
		var bsd BackupStorageDescription
		err := json.Unmarshal(body, &bsd)
		return bsd, err
	}
}
func unmarshalBasicBackupStorageDescriptionArray(body []byte) ([]BasicBackupStorageDescription, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	bsdArray := make([]BasicBackupStorageDescription, len(rawMessages))

	for index, rawMessage := range rawMessages {
		bsd, err := unmarshalBasicBackupStorageDescription(*rawMessage)
		if err != nil {
			return nil, err
		}
		bsdArray[index] = bsd
	}
	return bsdArray, nil
}

// MarshalJSON is the custom marshaler for BackupStorageDescription.
func (bsd BackupStorageDescription) MarshalJSON() ([]byte, error) {
	bsd.StorageKind = StorageKindBackupStorageDescription
	objectMap := make(map[string]interface{})
	if bsd.FriendlyName != nil {
		objectMap["FriendlyName"] = bsd.FriendlyName
	}
	if bsd.StorageKind != "" {
		objectMap["StorageKind"] = bsd.StorageKind
	}
	return json.Marshal(objectMap)
}

// AsAzureBlobBackupStorageDescription is the BasicBackupStorageDescription implementation for BackupStorageDescription.
func (bsd BackupStorageDescription) AsAzureBlobBackupStorageDescription() (*AzureBlobBackupStorageDescription, bool) {
	return nil, false
}

// AsFileShareBackupStorageDescription is the BasicBackupStorageDescription implementation for BackupStorageDescription.
func (bsd BackupStorageDescription) AsFileShareBackupStorageDescription() (*FileShareBackupStorageDescription, bool) {
	return nil, false
}

// AsBackupStorageDescription is the BasicBackupStorageDescription implementation for BackupStorageDescription.
func (bsd BackupStorageDescription) AsBackupStorageDescription() (*BackupStorageDescription, bool) {
	return &bsd, true
}

// AsBasicBackupStorageDescription is the BasicBackupStorageDescription implementation for BackupStorageDescription.
func (bsd BackupStorageDescription) AsBasicBackupStorageDescription() (BasicBackupStorageDescription, bool) {
	return &bsd, true
}

// BackupSuspensionInfo describes the backup suspension details.
type BackupSuspensionInfo struct {
	// IsSuspended - Indicates whether periodic backup is suspended at this level or not.
	IsSuspended *bool `json:"IsSuspended,omitempty"`
	// SuspensionInheritedFrom - Specifies the scope at which the backup suspension was applied. Possible values include: 'BackupSuspensionScopeInvalid', 'BackupSuspensionScopePartition', 'BackupSuspensionScopeService', 'BackupSuspensionScopeApplication'
	SuspensionInheritedFrom BackupSuspensionScope `json:"SuspensionInheritedFrom,omitempty"`
}

// BasicRetentionPolicyDescription describes basic retention policy.
type BasicRetentionPolicyDescription struct {
	// RetentionDuration - It is the minimum duration for which a backup created, will remain stored in the storage and might get deleted after that span of time. It should be specified in ISO8601 format.
	RetentionDuration *string `json:"RetentionDuration,omitempty"`
	// MinimumNumberOfBackups - It is the minimum number of backups to be retained at any point of time. If specified with a non zero value, backups will not be deleted even if the backups have gone past retention duration and have number of backups less than or equal to it.
	MinimumNumberOfBackups *int32 `json:"MinimumNumberOfBackups,omitempty"`
	// RetentionPolicyType - Possible values include: 'RetentionPolicyTypeRetentionPolicyDescription', 'RetentionPolicyTypeBasic1'
	RetentionPolicyType RetentionPolicyTypeBasicBasicRetentionPolicyDescription `json:"RetentionPolicyType,omitempty"`
}

// MarshalJSON is the custom marshaler for BasicRetentionPolicyDescription.
func (brpd BasicRetentionPolicyDescription) MarshalJSON() ([]byte, error) {
	brpd.RetentionPolicyType = RetentionPolicyTypeBasic1
	objectMap := make(map[string]interface{})
	if brpd.RetentionDuration != nil {
		objectMap["RetentionDuration"] = brpd.RetentionDuration
	}
	if brpd.MinimumNumberOfBackups != nil {
		objectMap["MinimumNumberOfBackups"] = brpd.MinimumNumberOfBackups
	}
	if brpd.RetentionPolicyType != "" {
		objectMap["RetentionPolicyType"] = brpd.RetentionPolicyType
	}
	return json.Marshal(objectMap)
}

// AsBasicRetentionPolicyDescription is the BasicBasicRetentionPolicyDescription implementation for BasicRetentionPolicyDescription.
func (brpd BasicRetentionPolicyDescription) AsBasicRetentionPolicyDescription() (*BasicRetentionPolicyDescription, bool) {
	return &brpd, true
}

// AsRetentionPolicyDescription is the BasicBasicRetentionPolicyDescription implementation for BasicRetentionPolicyDescription.
func (brpd BasicRetentionPolicyDescription) AsRetentionPolicyDescription() (*RetentionPolicyDescription, bool) {
	return nil, false
}

// AsBasicBasicRetentionPolicyDescription is the BasicBasicRetentionPolicyDescription implementation for BasicRetentionPolicyDescription.
func (brpd BasicRetentionPolicyDescription) AsBasicBasicRetentionPolicyDescription() (BasicBasicRetentionPolicyDescription, bool) {
	return &brpd, true
}

// BinaryPropertyValue describes a Service Fabric property value of type Binary.
type BinaryPropertyValue struct {
	// Data - Array of bytes to be sent as an integer array. Each element of array is a number between 0 and 255.
	Data *[]int32 `json:"Data,omitempty"`
	// Kind - Possible values include: 'KindPropertyValue', 'KindBinary', 'KindInt64', 'KindDouble', 'KindString', 'KindGUID'
	Kind KindBasicPropertyValue `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for BinaryPropertyValue.
func (bpv BinaryPropertyValue) MarshalJSON() ([]byte, error) {
	bpv.Kind = KindBinary
	objectMap := make(map[string]interface{})
	if bpv.Data != nil {
		objectMap["Data"] = bpv.Data
	}
	if bpv.Kind != "" {
		objectMap["Kind"] = bpv.Kind
	}
	return json.Marshal(objectMap)
}

// AsBinaryPropertyValue is the BasicPropertyValue implementation for BinaryPropertyValue.
func (bpv BinaryPropertyValue) AsBinaryPropertyValue() (*BinaryPropertyValue, bool) {
	return &bpv, true
}

// AsInt64PropertyValue is the BasicPropertyValue implementation for BinaryPropertyValue.
func (bpv BinaryPropertyValue) AsInt64PropertyValue() (*Int64PropertyValue, bool) {
	return nil, false
}

// AsDoublePropertyValue is the BasicPropertyValue implementation for BinaryPropertyValue.
func (bpv BinaryPropertyValue) AsDoublePropertyValue() (*DoublePropertyValue, bool) {
	return nil, false
}

// AsStringPropertyValue is the BasicPropertyValue implementation for BinaryPropertyValue.
func (bpv BinaryPropertyValue) AsStringPropertyValue() (*StringPropertyValue, bool) {
	return nil, false
}

// AsGUIDPropertyValue is the BasicPropertyValue implementation for BinaryPropertyValue.
func (bpv BinaryPropertyValue) AsGUIDPropertyValue() (*GUIDPropertyValue, bool) {
	return nil, false
}

// AsPropertyValue is the BasicPropertyValue implementation for BinaryPropertyValue.
func (bpv BinaryPropertyValue) AsPropertyValue() (*PropertyValue, bool) {
	return nil, false
}

// AsBasicPropertyValue is the BasicPropertyValue implementation for BinaryPropertyValue.
func (bpv BinaryPropertyValue) AsBasicPropertyValue() (BasicPropertyValue, bool) {
	return &bpv, true
}

// Chaos contains a description of Chaos.
type Chaos struct {
	autorest.Response `json:"-"`
	// ChaosParameters - If Chaos is running, these are the parameters Chaos is running with.
	ChaosParameters *ChaosParameters `json:"ChaosParameters,omitempty"`
	// Status - Current status of the Chaos run. Possible values include: 'ChaosStatusInvalid', 'ChaosStatusRunning', 'ChaosStatusStopped'
	Status ChaosStatus `json:"Status,omitempty"`
	// ScheduleStatus - Current status of the schedule. Possible values include: 'ChaosScheduleStatusInvalid', 'ChaosScheduleStatusStopped', 'ChaosScheduleStatusActive', 'ChaosScheduleStatusExpired', 'ChaosScheduleStatusPending'
	ScheduleStatus ChaosScheduleStatus `json:"ScheduleStatus,omitempty"`
}

// ChaosCodePackageRestartScheduledEvent chaos Restart Code Package Fault Scheduled event.
type ChaosCodePackageRestartScheduledEvent struct {
	// FaultGroupID - Id of fault group.
	FaultGroupID *uuid.UUID `json:"FaultGroupId,omitempty"`
	// FaultID - Id of fault.
	FaultID *uuid.UUID `json:"FaultId,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// ServiceManifestName - Service manifest name.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// CodePackageName - Code package name.
	CodePackageName *string `json:"CodePackageName,omitempty"`
	// ServicePackageActivationID - Id of Service package activation.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) MarshalJSON() ([]byte, error) {
	ccprse.Kind = KindChaosCodePackageRestartScheduled
	objectMap := make(map[string]interface{})
	if ccprse.FaultGroupID != nil {
		objectMap["FaultGroupId"] = ccprse.FaultGroupID
	}
	if ccprse.FaultID != nil {
		objectMap["FaultId"] = ccprse.FaultID
	}
	if ccprse.NodeName != nil {
		objectMap["NodeName"] = ccprse.NodeName
	}
	if ccprse.ServiceManifestName != nil {
		objectMap["ServiceManifestName"] = ccprse.ServiceManifestName
	}
	if ccprse.CodePackageName != nil {
		objectMap["CodePackageName"] = ccprse.CodePackageName
	}
	if ccprse.ServicePackageActivationID != nil {
		objectMap["ServicePackageActivationId"] = ccprse.ServicePackageActivationID
	}
	if ccprse.ApplicationID != nil {
		objectMap["ApplicationId"] = ccprse.ApplicationID
	}
	if ccprse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ccprse.EventInstanceID
	}
	if ccprse.Category != nil {
		objectMap["Category"] = ccprse.Category
	}
	if ccprse.TimeStamp != nil {
		objectMap["TimeStamp"] = ccprse.TimeStamp
	}
	if ccprse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ccprse.HasCorrelatedEvents
	}
	if ccprse.Kind != "" {
		objectMap["Kind"] = ccprse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &ccprse, true
}

// AsClusterEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return &ccprse, true
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ChaosCodePackageRestartScheduledEvent.
func (ccprse ChaosCodePackageRestartScheduledEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ccprse, true
}

// ChaosContext describes a map, which is a collection of (string, string) type key-value pairs. The map
// can be used to record information about
// the Chaos run. There cannot be more than 100 such pairs and each string (key or value) can be at most
// 4095 characters long.
// This map is set by the starter of the Chaos run to optionally store the context about the specific run.
type ChaosContext struct {
	// Map - Describes a map that contains a collection of ChaosContextMapItem's.
	Map map[string]*string `json:"Map"`
}

// MarshalJSON is the custom marshaler for ChaosContext.
func (cc ChaosContext) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if cc.Map != nil {
		objectMap["Map"] = cc.Map
	}
	return json.Marshal(objectMap)
}

// BasicChaosEvent represents an event generated during a Chaos run.
type BasicChaosEvent interface {
	AsExecutingFaultsChaosEvent() (*ExecutingFaultsChaosEvent, bool)
	AsStartedChaosEvent() (*StartedChaosEvent, bool)
	AsStoppedChaosEvent() (*StoppedChaosEvent, bool)
	AsTestErrorChaosEvent() (*TestErrorChaosEvent, bool)
	AsValidationFailedChaosEvent() (*ValidationFailedChaosEvent, bool)
	AsWaitingChaosEvent() (*WaitingChaosEvent, bool)
	AsChaosEvent() (*ChaosEvent, bool)
}

// ChaosEvent represents an event generated during a Chaos run.
type ChaosEvent struct {
	// TimeStampUtc - The UTC timestamp when this Chaos event was generated.
	TimeStampUtc *date.Time `json:"TimeStampUtc,omitempty"`
	// Kind - Possible values include: 'KindChaosEvent', 'KindExecutingFaults', 'KindStarted', 'KindStopped', 'KindTestError', 'KindValidationFailed', 'KindWaiting'
	Kind KindBasicChaosEvent `json:"Kind,omitempty"`
}

func unmarshalBasicChaosEvent(body []byte) (BasicChaosEvent, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindExecutingFaults):
		var efce ExecutingFaultsChaosEvent
		err := json.Unmarshal(body, &efce)
		return efce, err
	case string(KindStarted):
		var sce StartedChaosEvent
		err := json.Unmarshal(body, &sce)
		return sce, err
	case string(KindStopped):
		var sce StoppedChaosEvent
		err := json.Unmarshal(body, &sce)
		return sce, err
	case string(KindTestError):
		var tece TestErrorChaosEvent
		err := json.Unmarshal(body, &tece)
		return tece, err
	case string(KindValidationFailed):
		var vfce ValidationFailedChaosEvent
		err := json.Unmarshal(body, &vfce)
		return vfce, err
	case string(KindWaiting):
		var wce WaitingChaosEvent
		err := json.Unmarshal(body, &wce)
		return wce, err
	default:
		var ce ChaosEvent
		err := json.Unmarshal(body, &ce)
		return ce, err
	}
}
func unmarshalBasicChaosEventArray(body []byte) ([]BasicChaosEvent, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	ceArray := make([]BasicChaosEvent, len(rawMessages))

	for index, rawMessage := range rawMessages {
		ce, err := unmarshalBasicChaosEvent(*rawMessage)
		if err != nil {
			return nil, err
		}
		ceArray[index] = ce
	}
	return ceArray, nil
}

// MarshalJSON is the custom marshaler for ChaosEvent.
func (ce ChaosEvent) MarshalJSON() ([]byte, error) {
	ce.Kind = KindChaosEvent
	objectMap := make(map[string]interface{})
	if ce.TimeStampUtc != nil {
		objectMap["TimeStampUtc"] = ce.TimeStampUtc
	}
	if ce.Kind != "" {
		objectMap["Kind"] = ce.Kind
	}
	return json.Marshal(objectMap)
}

// AsExecutingFaultsChaosEvent is the BasicChaosEvent implementation for ChaosEvent.
func (ce ChaosEvent) AsExecutingFaultsChaosEvent() (*ExecutingFaultsChaosEvent, bool) {
	return nil, false
}

// AsStartedChaosEvent is the BasicChaosEvent implementation for ChaosEvent.
func (ce ChaosEvent) AsStartedChaosEvent() (*StartedChaosEvent, bool) {
	return nil, false
}

// AsStoppedChaosEvent is the BasicChaosEvent implementation for ChaosEvent.
func (ce ChaosEvent) AsStoppedChaosEvent() (*StoppedChaosEvent, bool) {
	return nil, false
}

// AsTestErrorChaosEvent is the BasicChaosEvent implementation for ChaosEvent.
func (ce ChaosEvent) AsTestErrorChaosEvent() (*TestErrorChaosEvent, bool) {
	return nil, false
}

// AsValidationFailedChaosEvent is the BasicChaosEvent implementation for ChaosEvent.
func (ce ChaosEvent) AsValidationFailedChaosEvent() (*ValidationFailedChaosEvent, bool) {
	return nil, false
}

// AsWaitingChaosEvent is the BasicChaosEvent implementation for ChaosEvent.
func (ce ChaosEvent) AsWaitingChaosEvent() (*WaitingChaosEvent, bool) {
	return nil, false
}

// AsChaosEvent is the BasicChaosEvent implementation for ChaosEvent.
func (ce ChaosEvent) AsChaosEvent() (*ChaosEvent, bool) {
	return &ce, true
}

// AsBasicChaosEvent is the BasicChaosEvent implementation for ChaosEvent.
func (ce ChaosEvent) AsBasicChaosEvent() (BasicChaosEvent, bool) {
	return &ce, true
}

// ChaosEventsSegment contains the list of Chaos events and the continuation token to get the next segment.
type ChaosEventsSegment struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// History - List of Chaos events that meet the user-supplied criteria.
	History *[]ChaosEventWrapper `json:"History,omitempty"`
}

// ChaosEventWrapper wrapper object for Chaos event.
type ChaosEventWrapper struct {
	// ChaosEvent - Represents an event generated during a Chaos run.
	ChaosEvent BasicChaosEvent `json:"ChaosEvent,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ChaosEventWrapper struct.
func (cew *ChaosEventWrapper) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "ChaosEvent":
			if v != nil {
				chaosEvent, err := unmarshalBasicChaosEvent(*v)
				if err != nil {
					return err
				}
				cew.ChaosEvent = chaosEvent
			}
		}
	}

	return nil
}

// ChaosNodeRestartScheduledEvent chaos Restart Node Fault Scheduled event.
type ChaosNodeRestartScheduledEvent struct {
	// NodeInstanceID - Id of Node instance.
	NodeInstanceID *int64 `json:"NodeInstanceId,omitempty"`
	// FaultGroupID - Id of fault group.
	FaultGroupID *uuid.UUID `json:"FaultGroupId,omitempty"`
	// FaultID - Id of fault.
	FaultID *uuid.UUID `json:"FaultId,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) MarshalJSON() ([]byte, error) {
	cnrse.Kind = KindChaosNodeRestartScheduled
	objectMap := make(map[string]interface{})
	if cnrse.NodeInstanceID != nil {
		objectMap["NodeInstanceId"] = cnrse.NodeInstanceID
	}
	if cnrse.FaultGroupID != nil {
		objectMap["FaultGroupId"] = cnrse.FaultGroupID
	}
	if cnrse.FaultID != nil {
		objectMap["FaultId"] = cnrse.FaultID
	}
	if cnrse.NodeName != nil {
		objectMap["NodeName"] = cnrse.NodeName
	}
	if cnrse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = cnrse.EventInstanceID
	}
	if cnrse.Category != nil {
		objectMap["Category"] = cnrse.Category
	}
	if cnrse.TimeStamp != nil {
		objectMap["TimeStamp"] = cnrse.TimeStamp
	}
	if cnrse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = cnrse.HasCorrelatedEvents
	}
	if cnrse.Kind != "" {
		objectMap["Kind"] = cnrse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &cnrse, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return &cnrse, true
}

// AsFabricEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ChaosNodeRestartScheduledEvent.
func (cnrse ChaosNodeRestartScheduledEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &cnrse, true
}

// ChaosParameters defines all the parameters to configure a Chaos run.
type ChaosParameters struct {
	// TimeToRunInSeconds - Total time (in seconds) for which Chaos will run before automatically stopping. The maximum allowed value is 4,294,967,295 (System.UInt32.MaxValue).
	TimeToRunInSeconds *string `json:"TimeToRunInSeconds,omitempty"`
	// MaxClusterStabilizationTimeoutInSeconds - The maximum amount of time to wait for all cluster entities to become stable and healthy. Chaos executes in iterations and at the start of each iteration it validates the health of cluster entities.
	// During validation if a cluster entity is not stable and healthy within MaxClusterStabilizationTimeoutInSeconds, Chaos generates a validation failed event.
	MaxClusterStabilizationTimeoutInSeconds *int64 `json:"MaxClusterStabilizationTimeoutInSeconds,omitempty"`
	// MaxConcurrentFaults - MaxConcurrentFaults is the maximum number of concurrent faults induced per iteration.
	// Chaos executes in iterations and two consecutive iterations are separated by a validation phase.
	// The higher the concurrency, the more aggressive the injection of faults, leading to inducing more complex series of states to uncover bugs.
	// The recommendation is to start with a value of 2 or 3 and to exercise caution while moving up.
	MaxConcurrentFaults *int64 `json:"MaxConcurrentFaults,omitempty"`
	// EnableMoveReplicaFaults - Enables or disables the move primary and move secondary faults.
	EnableMoveReplicaFaults *bool `json:"EnableMoveReplicaFaults,omitempty"`
	// WaitTimeBetweenFaultsInSeconds - Wait time (in seconds) between consecutive faults within a single iteration.
	// The larger the value, the lower the overlapping between faults and the simpler the sequence of state transitions that the cluster goes through.
	// The recommendation is to start with a value between 1 and 5 and exercise caution while moving up.
	WaitTimeBetweenFaultsInSeconds *int64 `json:"WaitTimeBetweenFaultsInSeconds,omitempty"`
	// WaitTimeBetweenIterationsInSeconds - Time-separation (in seconds) between two consecutive iterations of Chaos.
	// The larger the value, the lower the fault injection rate.
	WaitTimeBetweenIterationsInSeconds *int64 `json:"WaitTimeBetweenIterationsInSeconds,omitempty"`
	// ClusterHealthPolicy - Passed-in cluster health policy is used to validate health of the cluster in between Chaos iterations. If the cluster health is in error or if an unexpected exception happens during fault execution--to provide the cluster with some time to recuperate--Chaos will wait for 30 minutes before the next health-check.
	ClusterHealthPolicy *ClusterHealthPolicy `json:"ClusterHealthPolicy,omitempty"`
	// Context - Describes a map, which is a collection of (string, string) type key-value pairs. The map can be used to record information about
	// the Chaos run. There cannot be more than 100 such pairs and each string (key or value) can be at most 4095 characters long.
	// This map is set by the starter of the Chaos run to optionally store the context about the specific run.
	Context *ChaosContext `json:"Context,omitempty"`
	// ChaosTargetFilter - List of cluster entities to target for Chaos faults.
	// This filter can be used to target Chaos faults only to certain node types or only to certain application instances. If ChaosTargetFilter is not used, Chaos faults all cluster entities.
	// If ChaosTargetFilter is used, Chaos faults only the entities that meet the ChaosTargetFilter specification.
	ChaosTargetFilter *ChaosTargetFilter `json:"ChaosTargetFilter,omitempty"`
}

// ChaosParametersDictionaryItem defines an item in ChaosParametersDictionary of the Chaos Schedule.
type ChaosParametersDictionaryItem struct {
	// Key - The key identifying the Chaos Parameter in the dictionary. This key is referenced by Chaos Schedule Jobs.
	Key *string `json:"Key,omitempty"`
	// Value - Defines all the parameters to configure a Chaos run.
	Value *ChaosParameters `json:"Value,omitempty"`
}

// ChaosPartitionPrimaryMoveScheduledEvent chaos Move Primary Fault Scheduled event.
type ChaosPartitionPrimaryMoveScheduledEvent struct {
	// FaultGroupID - Id of fault group.
	FaultGroupID *uuid.UUID `json:"FaultGroupId,omitempty"`
	// FaultID - Id of fault.
	FaultID *uuid.UUID `json:"FaultId,omitempty"`
	// ServiceName - Service name.
	ServiceName *string `json:"ServiceName,omitempty"`
	// NodeTo - The name of a Service Fabric node.
	NodeTo *string `json:"NodeTo,omitempty"`
	// ForcedMove - Indicates a forced move.
	ForcedMove *bool `json:"ForcedMove,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) MarshalJSON() ([]byte, error) {
	cppmse.Kind = KindChaosPartitionPrimaryMoveScheduled
	objectMap := make(map[string]interface{})
	if cppmse.FaultGroupID != nil {
		objectMap["FaultGroupId"] = cppmse.FaultGroupID
	}
	if cppmse.FaultID != nil {
		objectMap["FaultId"] = cppmse.FaultID
	}
	if cppmse.ServiceName != nil {
		objectMap["ServiceName"] = cppmse.ServiceName
	}
	if cppmse.NodeTo != nil {
		objectMap["NodeTo"] = cppmse.NodeTo
	}
	if cppmse.ForcedMove != nil {
		objectMap["ForcedMove"] = cppmse.ForcedMove
	}
	if cppmse.PartitionID != nil {
		objectMap["PartitionId"] = cppmse.PartitionID
	}
	if cppmse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = cppmse.EventInstanceID
	}
	if cppmse.Category != nil {
		objectMap["Category"] = cppmse.Category
	}
	if cppmse.TimeStamp != nil {
		objectMap["TimeStamp"] = cppmse.TimeStamp
	}
	if cppmse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = cppmse.HasCorrelatedEvents
	}
	if cppmse.Kind != "" {
		objectMap["Kind"] = cppmse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return &cppmse, true
}

// AsReplicaEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return &cppmse, true
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ChaosPartitionPrimaryMoveScheduledEvent.
func (cppmse ChaosPartitionPrimaryMoveScheduledEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &cppmse, true
}

// ChaosPartitionSecondaryMoveScheduledEvent chaos Move Secondary Fault Scheduled event.
type ChaosPartitionSecondaryMoveScheduledEvent struct {
	// FaultGroupID - Id of fault group.
	FaultGroupID *uuid.UUID `json:"FaultGroupId,omitempty"`
	// FaultID - Id of fault.
	FaultID *uuid.UUID `json:"FaultId,omitempty"`
	// ServiceName - Service name.
	ServiceName *string `json:"ServiceName,omitempty"`
	// SourceNode - The name of a Service Fabric node.
	SourceNode *string `json:"SourceNode,omitempty"`
	// DestinationNode - The name of a Service Fabric node.
	DestinationNode *string `json:"DestinationNode,omitempty"`
	// ForcedMove - Indicates a forced move.
	ForcedMove *bool `json:"ForcedMove,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) MarshalJSON() ([]byte, error) {
	cpsmse.Kind = KindChaosPartitionSecondaryMoveScheduled
	objectMap := make(map[string]interface{})
	if cpsmse.FaultGroupID != nil {
		objectMap["FaultGroupId"] = cpsmse.FaultGroupID
	}
	if cpsmse.FaultID != nil {
		objectMap["FaultId"] = cpsmse.FaultID
	}
	if cpsmse.ServiceName != nil {
		objectMap["ServiceName"] = cpsmse.ServiceName
	}
	if cpsmse.SourceNode != nil {
		objectMap["SourceNode"] = cpsmse.SourceNode
	}
	if cpsmse.DestinationNode != nil {
		objectMap["DestinationNode"] = cpsmse.DestinationNode
	}
	if cpsmse.ForcedMove != nil {
		objectMap["ForcedMove"] = cpsmse.ForcedMove
	}
	if cpsmse.PartitionID != nil {
		objectMap["PartitionId"] = cpsmse.PartitionID
	}
	if cpsmse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = cpsmse.EventInstanceID
	}
	if cpsmse.Category != nil {
		objectMap["Category"] = cpsmse.Category
	}
	if cpsmse.TimeStamp != nil {
		objectMap["TimeStamp"] = cpsmse.TimeStamp
	}
	if cpsmse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = cpsmse.HasCorrelatedEvents
	}
	if cpsmse.Kind != "" {
		objectMap["Kind"] = cpsmse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return &cpsmse, true
}

// AsReplicaEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return &cpsmse, true
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ChaosPartitionSecondaryMoveScheduledEvent.
func (cpsmse ChaosPartitionSecondaryMoveScheduledEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &cpsmse, true
}

// ChaosReplicaRemovalScheduledEvent chaos Remove Replica Fault Scheduled event.
type ChaosReplicaRemovalScheduledEvent struct {
	// FaultGroupID - Id of fault group.
	FaultGroupID *uuid.UUID `json:"FaultGroupId,omitempty"`
	// FaultID - Id of fault.
	FaultID *uuid.UUID `json:"FaultId,omitempty"`
	// ServiceURI - Service name.
	ServiceURI *string `json:"ServiceUri,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *int64 `json:"ReplicaId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) MarshalJSON() ([]byte, error) {
	crrse.Kind = KindChaosReplicaRemovalScheduled
	objectMap := make(map[string]interface{})
	if crrse.FaultGroupID != nil {
		objectMap["FaultGroupId"] = crrse.FaultGroupID
	}
	if crrse.FaultID != nil {
		objectMap["FaultId"] = crrse.FaultID
	}
	if crrse.ServiceURI != nil {
		objectMap["ServiceUri"] = crrse.ServiceURI
	}
	if crrse.PartitionID != nil {
		objectMap["PartitionId"] = crrse.PartitionID
	}
	if crrse.ReplicaID != nil {
		objectMap["ReplicaId"] = crrse.ReplicaID
	}
	if crrse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = crrse.EventInstanceID
	}
	if crrse.Category != nil {
		objectMap["Category"] = crrse.Category
	}
	if crrse.TimeStamp != nil {
		objectMap["TimeStamp"] = crrse.TimeStamp
	}
	if crrse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = crrse.HasCorrelatedEvents
	}
	if crrse.Kind != "" {
		objectMap["Kind"] = crrse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return &crrse, true
}

// AsServiceEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return &crrse, true
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ChaosReplicaRemovalScheduledEvent.
func (crrse ChaosReplicaRemovalScheduledEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &crrse, true
}

// ChaosReplicaRestartScheduledEvent chaos Restart Replica Fault Scheduled event.
type ChaosReplicaRestartScheduledEvent struct {
	// FaultGroupID - Id of fault group.
	FaultGroupID *uuid.UUID `json:"FaultGroupId,omitempty"`
	// FaultID - Id of fault.
	FaultID *uuid.UUID `json:"FaultId,omitempty"`
	// ServiceURI - Service name.
	ServiceURI *string `json:"ServiceUri,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *int64 `json:"ReplicaId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) MarshalJSON() ([]byte, error) {
	crrse.Kind = KindChaosReplicaRestartScheduled
	objectMap := make(map[string]interface{})
	if crrse.FaultGroupID != nil {
		objectMap["FaultGroupId"] = crrse.FaultGroupID
	}
	if crrse.FaultID != nil {
		objectMap["FaultId"] = crrse.FaultID
	}
	if crrse.ServiceURI != nil {
		objectMap["ServiceUri"] = crrse.ServiceURI
	}
	if crrse.PartitionID != nil {
		objectMap["PartitionId"] = crrse.PartitionID
	}
	if crrse.ReplicaID != nil {
		objectMap["ReplicaId"] = crrse.ReplicaID
	}
	if crrse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = crrse.EventInstanceID
	}
	if crrse.Category != nil {
		objectMap["Category"] = crrse.Category
	}
	if crrse.TimeStamp != nil {
		objectMap["TimeStamp"] = crrse.TimeStamp
	}
	if crrse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = crrse.HasCorrelatedEvents
	}
	if crrse.Kind != "" {
		objectMap["Kind"] = crrse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return &crrse, true
}

// AsServiceEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return &crrse, true
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ChaosReplicaRestartScheduledEvent.
func (crrse ChaosReplicaRestartScheduledEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &crrse, true
}

// ChaosSchedule defines the schedule used by Chaos.
type ChaosSchedule struct {
	// StartDate - The date and time Chaos will start using this schedule.
	StartDate *date.Time `json:"StartDate,omitempty"`
	// ExpiryDate - The date and time Chaos will continue to use this schedule until.
	ExpiryDate *date.Time `json:"ExpiryDate,omitempty"`
	// ChaosParametersDictionary - A mapping of string names to Chaos Parameters to be referenced by Chaos Schedule Jobs.
	ChaosParametersDictionary *[]ChaosParametersDictionaryItem `json:"ChaosParametersDictionary,omitempty"`
	// Jobs - A list of all Chaos Schedule Jobs that will be automated by the schedule.
	Jobs *[]ChaosScheduleJob `json:"Jobs,omitempty"`
}

// ChaosScheduleDescription defines the Chaos Schedule used by Chaos and the version of the Chaos Schedule.
// The version value wraps back to 0 after surpassing 2,147,483,647.
type ChaosScheduleDescription struct {
	autorest.Response `json:"-"`
	// Version - The version number of the Schedule.
	Version *int32 `json:"Version,omitempty"`
	// Schedule - Defines the schedule used by Chaos.
	Schedule *ChaosSchedule `json:"Schedule,omitempty"`
}

// ChaosScheduleJob defines a repetition rule and parameters of Chaos to be used with the Chaos Schedule.
type ChaosScheduleJob struct {
	// ChaosParameters - A reference to which Chaos Parameters of the Chaos Schedule to use.
	ChaosParameters *string `json:"ChaosParameters,omitempty"`
	// Days - Defines the days of the week that a Chaos Schedule Job will run for.
	Days *ChaosScheduleJobActiveDaysOfWeek `json:"Days,omitempty"`
	// Times - A list of Time Ranges that specify when during active days that this job will run. The times are interpreted as UTC.
	Times *[]TimeRange `json:"Times,omitempty"`
}

// ChaosScheduleJobActiveDaysOfWeek defines the days of the week that a Chaos Schedule Job will run for.
type ChaosScheduleJobActiveDaysOfWeek struct {
	// Sunday - Indicates if the Chaos Schedule Job will run on Sunday
	Sunday *bool `json:"Sunday,omitempty"`
	// Monday - Indicates if the Chaos Schedule Job will run on Monday
	Monday *bool `json:"Monday,omitempty"`
	// Tuesday - Indicates if the Chaos Schedule Job will run on Tuesday
	Tuesday *bool `json:"Tuesday,omitempty"`
	// Wednesday - Indicates if the Chaos Schedule Job will run on Wednesday
	Wednesday *bool `json:"Wednesday,omitempty"`
	// Thursday - Indicates if the Chaos Schedule Job will run on Thursday
	Thursday *bool `json:"Thursday,omitempty"`
	// Friday - Indicates if the Chaos Schedule Job will run on Friday
	Friday *bool `json:"Friday,omitempty"`
	// Saturday - Indicates if the Chaos Schedule Job will run on Saturday
	Saturday *bool `json:"Saturday,omitempty"`
}

// ChaosStartedEvent chaos Started event.
type ChaosStartedEvent struct {
	// MaxConcurrentFaults - Maximum number of concurrent faults.
	MaxConcurrentFaults *int64 `json:"MaxConcurrentFaults,omitempty"`
	// TimeToRunInSeconds - Time to run in seconds.
	TimeToRunInSeconds *float64 `json:"TimeToRunInSeconds,omitempty"`
	// MaxClusterStabilizationTimeoutInSeconds - Maximum timeout for cluster stabilization in seconds.
	MaxClusterStabilizationTimeoutInSeconds *float64 `json:"MaxClusterStabilizationTimeoutInSeconds,omitempty"`
	// WaitTimeBetweenIterationsInSeconds - Wait time between iterations in seconds.
	WaitTimeBetweenIterationsInSeconds *float64 `json:"WaitTimeBetweenIterationsInSeconds,omitempty"`
	// WaitTimeBetweenFaultsInSeconds - Wait time between faults in seconds.
	WaitTimeBetweenFaultsInSeconds *float64 `json:"WaitTimeBetweenFaultsInSeconds,omitempty"`
	// MoveReplicaFaultEnabled - Indicates MoveReplica fault is enabled.
	MoveReplicaFaultEnabled *bool `json:"MoveReplicaFaultEnabled,omitempty"`
	// IncludedNodeTypeList - List of included Node types.
	IncludedNodeTypeList *string `json:"IncludedNodeTypeList,omitempty"`
	// IncludedApplicationList - List of included Applications.
	IncludedApplicationList *string `json:"IncludedApplicationList,omitempty"`
	// ClusterHealthPolicy - Health policy.
	ClusterHealthPolicy *string `json:"ClusterHealthPolicy,omitempty"`
	// ChaosContext - Chaos Context.
	ChaosContext *string `json:"ChaosContext,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ChaosStartedEvent.
func (cse ChaosStartedEvent) MarshalJSON() ([]byte, error) {
	cse.Kind = KindChaosStarted
	objectMap := make(map[string]interface{})
	if cse.MaxConcurrentFaults != nil {
		objectMap["MaxConcurrentFaults"] = cse.MaxConcurrentFaults
	}
	if cse.TimeToRunInSeconds != nil {
		objectMap["TimeToRunInSeconds"] = cse.TimeToRunInSeconds
	}
	if cse.MaxClusterStabilizationTimeoutInSeconds != nil {
		objectMap["MaxClusterStabilizationTimeoutInSeconds"] = cse.MaxClusterStabilizationTimeoutInSeconds
	}
	if cse.WaitTimeBetweenIterationsInSeconds != nil {
		objectMap["WaitTimeBetweenIterationsInSeconds"] = cse.WaitTimeBetweenIterationsInSeconds
	}
	if cse.WaitTimeBetweenFaultsInSeconds != nil {
		objectMap["WaitTimeBetweenFaultsInSeconds"] = cse.WaitTimeBetweenFaultsInSeconds
	}
	if cse.MoveReplicaFaultEnabled != nil {
		objectMap["MoveReplicaFaultEnabled"] = cse.MoveReplicaFaultEnabled
	}
	if cse.IncludedNodeTypeList != nil {
		objectMap["IncludedNodeTypeList"] = cse.IncludedNodeTypeList
	}
	if cse.IncludedApplicationList != nil {
		objectMap["IncludedApplicationList"] = cse.IncludedApplicationList
	}
	if cse.ClusterHealthPolicy != nil {
		objectMap["ClusterHealthPolicy"] = cse.ClusterHealthPolicy
	}
	if cse.ChaosContext != nil {
		objectMap["ChaosContext"] = cse.ChaosContext
	}
	if cse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = cse.EventInstanceID
	}
	if cse.Category != nil {
		objectMap["Category"] = cse.Category
	}
	if cse.TimeStamp != nil {
		objectMap["TimeStamp"] = cse.TimeStamp
	}
	if cse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = cse.HasCorrelatedEvents
	}
	if cse.Kind != "" {
		objectMap["Kind"] = cse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return &cse, true
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return &cse, true
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ChaosStartedEvent.
func (cse ChaosStartedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &cse, true
}

// ChaosStoppedEvent chaos Stopped event.
type ChaosStoppedEvent struct {
	// Reason - Describes reason.
	Reason *string `json:"Reason,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) MarshalJSON() ([]byte, error) {
	cse.Kind = KindChaosStopped
	objectMap := make(map[string]interface{})
	if cse.Reason != nil {
		objectMap["Reason"] = cse.Reason
	}
	if cse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = cse.EventInstanceID
	}
	if cse.Category != nil {
		objectMap["Category"] = cse.Category
	}
	if cse.TimeStamp != nil {
		objectMap["TimeStamp"] = cse.TimeStamp
	}
	if cse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = cse.HasCorrelatedEvents
	}
	if cse.Kind != "" {
		objectMap["Kind"] = cse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return &cse, true
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return &cse, true
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ChaosStoppedEvent.
func (cse ChaosStoppedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &cse, true
}

// ChaosTargetFilter defines all filters for targeted Chaos faults, for example, faulting only certain node
// types or faulting only certain applications.
// If ChaosTargetFilter is not used, Chaos faults all cluster entities. If ChaosTargetFilter is used, Chaos
// faults only the entities that meet the ChaosTargetFilter
// specification. NodeTypeInclusionList and ApplicationInclusionList allow a union semantics only. It is
// not possible to specify an intersection
// of NodeTypeInclusionList and ApplicationInclusionList. For example, it is not possible to specify "fault
// this application only when it is on that node type."
// Once an entity is included in either NodeTypeInclusionList or ApplicationInclusionList, that entity
// cannot be excluded using ChaosTargetFilter. Even if
// applicationX does not appear in ApplicationInclusionList, in some Chaos iteration applicationX can be
// faulted because it happens to be on a node of nodeTypeY that is included
// in NodeTypeInclusionList. If both NodeTypeInclusionList and ApplicationInclusionList are null or empty,
// an ArgumentException is thrown.
type ChaosTargetFilter struct {
	// NodeTypeInclusionList - A list of node types to include in Chaos faults.
	// All types of faults (restart node, restart code package, remove replica, restart replica, move primary, and move secondary) are enabled for the nodes of these node types.
	// If a node type (say NodeTypeX) does not appear in the NodeTypeInclusionList, then node level faults (like NodeRestart) will never be enabled for the nodes of
	// NodeTypeX, but code package and replica faults can still be enabled for NodeTypeX if an application in the ApplicationInclusionList.
	// happens to reside on a node of NodeTypeX.
	// At most 100 node type names can be included in this list, to increase this number, a config upgrade is required for MaxNumberOfNodeTypesInChaosEntityFilter configuration.
	NodeTypeInclusionList *[]string `json:"NodeTypeInclusionList,omitempty"`
	// ApplicationInclusionList - A list of application URIs to include in Chaos faults.
	// All replicas belonging to services of these applications are amenable to replica faults (restart replica, remove replica, move primary, and move secondary) by Chaos.
	// Chaos may restart a code package only if the code package hosts replicas of these applications only.
	// If an application does not appear in this list, it can still be faulted in some Chaos iteration if the application ends up on a node of a node type that is included in NodeTypeInclusionList.
	// However, if applicationX is tied to nodeTypeY through placement constraints and applicationX is absent from ApplicationInclusionList and nodeTypeY is absent from NodeTypeInclusionList, then applicationX will never be faulted.
	// At most 1000 application names can be included in this list, to increase this number, a config upgrade is required for MaxNumberOfApplicationsInChaosEntityFilter configuration.
	ApplicationInclusionList *[]string `json:"ApplicationInclusionList,omitempty"`
}

// CheckExistsPropertyBatchOperation represents a PropertyBatchOperation that compares the Boolean
// existence of a property with the Exists argument.
// The PropertyBatchOperation operation fails if the property's existence is not equal to the Exists
// argument.
// The CheckExistsPropertyBatchOperation is generally used as a precondition for the write operations in
// the batch.
// Note that if one PropertyBatchOperation in a PropertyBatch fails,
// the entire batch fails and cannot be committed in a transactional manner.
type CheckExistsPropertyBatchOperation struct {
	// Exists - Whether or not the property should exist for the operation to pass.
	Exists *bool `json:"Exists,omitempty"`
	// PropertyName - The name of the Service Fabric property.
	PropertyName *string `json:"PropertyName,omitempty"`
	// Kind - Possible values include: 'KindPropertyBatchOperation', 'KindCheckExists', 'KindCheckSequence', 'KindCheckValue', 'KindDelete', 'KindGet', 'KindPut'
	Kind KindBasicPropertyBatchOperation `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for CheckExistsPropertyBatchOperation.
func (cepbo CheckExistsPropertyBatchOperation) MarshalJSON() ([]byte, error) {
	cepbo.Kind = KindCheckExists
	objectMap := make(map[string]interface{})
	if cepbo.Exists != nil {
		objectMap["Exists"] = cepbo.Exists
	}
	if cepbo.PropertyName != nil {
		objectMap["PropertyName"] = cepbo.PropertyName
	}
	if cepbo.Kind != "" {
		objectMap["Kind"] = cepbo.Kind
	}
	return json.Marshal(objectMap)
}

// AsCheckExistsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckExistsPropertyBatchOperation.
func (cepbo CheckExistsPropertyBatchOperation) AsCheckExistsPropertyBatchOperation() (*CheckExistsPropertyBatchOperation, bool) {
	return &cepbo, true
}

// AsCheckSequencePropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckExistsPropertyBatchOperation.
func (cepbo CheckExistsPropertyBatchOperation) AsCheckSequencePropertyBatchOperation() (*CheckSequencePropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckValuePropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckExistsPropertyBatchOperation.
func (cepbo CheckExistsPropertyBatchOperation) AsCheckValuePropertyBatchOperation() (*CheckValuePropertyBatchOperation, bool) {
	return nil, false
}

// AsDeletePropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckExistsPropertyBatchOperation.
func (cepbo CheckExistsPropertyBatchOperation) AsDeletePropertyBatchOperation() (*DeletePropertyBatchOperation, bool) {
	return nil, false
}

// AsGetPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckExistsPropertyBatchOperation.
func (cepbo CheckExistsPropertyBatchOperation) AsGetPropertyBatchOperation() (*GetPropertyBatchOperation, bool) {
	return nil, false
}

// AsPutPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckExistsPropertyBatchOperation.
func (cepbo CheckExistsPropertyBatchOperation) AsPutPropertyBatchOperation() (*PutPropertyBatchOperation, bool) {
	return nil, false
}

// AsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckExistsPropertyBatchOperation.
func (cepbo CheckExistsPropertyBatchOperation) AsPropertyBatchOperation() (*PropertyBatchOperation, bool) {
	return nil, false
}

// AsBasicPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckExistsPropertyBatchOperation.
func (cepbo CheckExistsPropertyBatchOperation) AsBasicPropertyBatchOperation() (BasicPropertyBatchOperation, bool) {
	return &cepbo, true
}

// CheckSequencePropertyBatchOperation compares the Sequence Number of a property with the SequenceNumber
// argument.
// A property's sequence number can be thought of as that property's version.
// Every time the property is modified, its sequence number is increased.
// The sequence number can be found in a property's metadata.
// The comparison fails if the sequence numbers are not equal.
// CheckSequencePropertyBatchOperation is generally used as a precondition for the write operations in the
// batch.
// Note that if one PropertyBatchOperation in a PropertyBatch fails,
// the entire batch fails and cannot be committed in a transactional manner.
type CheckSequencePropertyBatchOperation struct {
	// SequenceNumber - The expected sequence number.
	SequenceNumber *string `json:"SequenceNumber,omitempty"`
	// PropertyName - The name of the Service Fabric property.
	PropertyName *string `json:"PropertyName,omitempty"`
	// Kind - Possible values include: 'KindPropertyBatchOperation', 'KindCheckExists', 'KindCheckSequence', 'KindCheckValue', 'KindDelete', 'KindGet', 'KindPut'
	Kind KindBasicPropertyBatchOperation `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for CheckSequencePropertyBatchOperation.
func (cspbo CheckSequencePropertyBatchOperation) MarshalJSON() ([]byte, error) {
	cspbo.Kind = KindCheckSequence
	objectMap := make(map[string]interface{})
	if cspbo.SequenceNumber != nil {
		objectMap["SequenceNumber"] = cspbo.SequenceNumber
	}
	if cspbo.PropertyName != nil {
		objectMap["PropertyName"] = cspbo.PropertyName
	}
	if cspbo.Kind != "" {
		objectMap["Kind"] = cspbo.Kind
	}
	return json.Marshal(objectMap)
}

// AsCheckExistsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckSequencePropertyBatchOperation.
func (cspbo CheckSequencePropertyBatchOperation) AsCheckExistsPropertyBatchOperation() (*CheckExistsPropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckSequencePropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckSequencePropertyBatchOperation.
func (cspbo CheckSequencePropertyBatchOperation) AsCheckSequencePropertyBatchOperation() (*CheckSequencePropertyBatchOperation, bool) {
	return &cspbo, true
}

// AsCheckValuePropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckSequencePropertyBatchOperation.
func (cspbo CheckSequencePropertyBatchOperation) AsCheckValuePropertyBatchOperation() (*CheckValuePropertyBatchOperation, bool) {
	return nil, false
}

// AsDeletePropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckSequencePropertyBatchOperation.
func (cspbo CheckSequencePropertyBatchOperation) AsDeletePropertyBatchOperation() (*DeletePropertyBatchOperation, bool) {
	return nil, false
}

// AsGetPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckSequencePropertyBatchOperation.
func (cspbo CheckSequencePropertyBatchOperation) AsGetPropertyBatchOperation() (*GetPropertyBatchOperation, bool) {
	return nil, false
}

// AsPutPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckSequencePropertyBatchOperation.
func (cspbo CheckSequencePropertyBatchOperation) AsPutPropertyBatchOperation() (*PutPropertyBatchOperation, bool) {
	return nil, false
}

// AsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckSequencePropertyBatchOperation.
func (cspbo CheckSequencePropertyBatchOperation) AsPropertyBatchOperation() (*PropertyBatchOperation, bool) {
	return nil, false
}

// AsBasicPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckSequencePropertyBatchOperation.
func (cspbo CheckSequencePropertyBatchOperation) AsBasicPropertyBatchOperation() (BasicPropertyBatchOperation, bool) {
	return &cspbo, true
}

// CheckValuePropertyBatchOperation represents a PropertyBatchOperation that compares the value of the
// property with the expected value.
// The CheckValuePropertyBatchOperation is generally used as a precondition for the write operations in the
// batch.
// Note that if one PropertyBatchOperation in a PropertyBatch fails,
// the entire batch fails and cannot be committed in a transactional manner.
type CheckValuePropertyBatchOperation struct {
	// Value - The expected property value.
	Value BasicPropertyValue `json:"Value,omitempty"`
	// PropertyName - The name of the Service Fabric property.
	PropertyName *string `json:"PropertyName,omitempty"`
	// Kind - Possible values include: 'KindPropertyBatchOperation', 'KindCheckExists', 'KindCheckSequence', 'KindCheckValue', 'KindDelete', 'KindGet', 'KindPut'
	Kind KindBasicPropertyBatchOperation `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for CheckValuePropertyBatchOperation.
func (cvpbo CheckValuePropertyBatchOperation) MarshalJSON() ([]byte, error) {
	cvpbo.Kind = KindCheckValue
	objectMap := make(map[string]interface{})
	objectMap["Value"] = cvpbo.Value
	if cvpbo.PropertyName != nil {
		objectMap["PropertyName"] = cvpbo.PropertyName
	}
	if cvpbo.Kind != "" {
		objectMap["Kind"] = cvpbo.Kind
	}
	return json.Marshal(objectMap)
}

// AsCheckExistsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckValuePropertyBatchOperation.
func (cvpbo CheckValuePropertyBatchOperation) AsCheckExistsPropertyBatchOperation() (*CheckExistsPropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckSequencePropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckValuePropertyBatchOperation.
func (cvpbo CheckValuePropertyBatchOperation) AsCheckSequencePropertyBatchOperation() (*CheckSequencePropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckValuePropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckValuePropertyBatchOperation.
func (cvpbo CheckValuePropertyBatchOperation) AsCheckValuePropertyBatchOperation() (*CheckValuePropertyBatchOperation, bool) {
	return &cvpbo, true
}

// AsDeletePropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckValuePropertyBatchOperation.
func (cvpbo CheckValuePropertyBatchOperation) AsDeletePropertyBatchOperation() (*DeletePropertyBatchOperation, bool) {
	return nil, false
}

// AsGetPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckValuePropertyBatchOperation.
func (cvpbo CheckValuePropertyBatchOperation) AsGetPropertyBatchOperation() (*GetPropertyBatchOperation, bool) {
	return nil, false
}

// AsPutPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckValuePropertyBatchOperation.
func (cvpbo CheckValuePropertyBatchOperation) AsPutPropertyBatchOperation() (*PutPropertyBatchOperation, bool) {
	return nil, false
}

// AsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckValuePropertyBatchOperation.
func (cvpbo CheckValuePropertyBatchOperation) AsPropertyBatchOperation() (*PropertyBatchOperation, bool) {
	return nil, false
}

// AsBasicPropertyBatchOperation is the BasicPropertyBatchOperation implementation for CheckValuePropertyBatchOperation.
func (cvpbo CheckValuePropertyBatchOperation) AsBasicPropertyBatchOperation() (BasicPropertyBatchOperation, bool) {
	return &cvpbo, true
}

// UnmarshalJSON is the custom unmarshaler for CheckValuePropertyBatchOperation struct.
func (cvpbo *CheckValuePropertyBatchOperation) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "Value":
			if v != nil {
				value, err := unmarshalBasicPropertyValue(*v)
				if err != nil {
					return err
				}
				cvpbo.Value = value
			}
		case "PropertyName":
			if v != nil {
				var propertyName string
				err = json.Unmarshal(*v, &propertyName)
				if err != nil {
					return err
				}
				cvpbo.PropertyName = &propertyName
			}
		case "Kind":
			if v != nil {
				var kind KindBasicPropertyBatchOperation
				err = json.Unmarshal(*v, &kind)
				if err != nil {
					return err
				}
				cvpbo.Kind = kind
			}
		}
	}

	return nil
}

// ClusterConfiguration information about the standalone cluster configuration.
type ClusterConfiguration struct {
	autorest.Response `json:"-"`
	// ClusterConfiguration - The contents of the cluster configuration file.
	ClusterConfiguration *string `json:"ClusterConfiguration,omitempty"`
}

// ClusterConfigurationUpgradeDescription describes the parameters for a standalone cluster configuration
// upgrade.
type ClusterConfigurationUpgradeDescription struct {
	// ClusterConfig - The cluster configuration as a JSON string. For example, [this file](https://github.com/Azure-Samples/service-fabric-dotnet-standalone-cluster-configuration/blob/master/Samples/ClusterConfig.Unsecure.DevCluster.json) contains JSON describing the [nodes and other properties of the cluster](https://docs.microsoft.com/azure/service-fabric/service-fabric-cluster-manifest).
	ClusterConfig *string `json:"ClusterConfig,omitempty"`
	// HealthCheckRetryTimeout - The length of time between attempts to perform health checks if the application or cluster is not healthy.
	HealthCheckRetryTimeout *string `json:"HealthCheckRetryTimeout,omitempty"`
	// HealthCheckWaitDurationInSeconds - The length of time to wait after completing an upgrade domain before starting the health checks process.
	HealthCheckWaitDurationInSeconds *string `json:"HealthCheckWaitDurationInSeconds,omitempty"`
	// HealthCheckStableDurationInSeconds - The length of time that the application or cluster must remain healthy before the upgrade proceeds to the next upgrade domain.
	HealthCheckStableDurationInSeconds *string `json:"HealthCheckStableDurationInSeconds,omitempty"`
	// UpgradeDomainTimeoutInSeconds - The timeout for the upgrade domain.
	UpgradeDomainTimeoutInSeconds *string `json:"UpgradeDomainTimeoutInSeconds,omitempty"`
	// UpgradeTimeoutInSeconds - The upgrade timeout.
	UpgradeTimeoutInSeconds *string `json:"UpgradeTimeoutInSeconds,omitempty"`
	// MaxPercentUnhealthyApplications - The maximum allowed percentage of unhealthy applications during the upgrade. Allowed values are integer values from zero to 100.
	MaxPercentUnhealthyApplications *int32 `json:"MaxPercentUnhealthyApplications,omitempty"`
	// MaxPercentUnhealthyNodes - The maximum allowed percentage of unhealthy nodes during the upgrade. Allowed values are integer values from zero to 100.
	MaxPercentUnhealthyNodes *int32 `json:"MaxPercentUnhealthyNodes,omitempty"`
	// MaxPercentDeltaUnhealthyNodes - The maximum allowed percentage of delta health degradation during the upgrade. Allowed values are integer values from zero to 100.
	MaxPercentDeltaUnhealthyNodes *int32 `json:"MaxPercentDeltaUnhealthyNodes,omitempty"`
	// MaxPercentUpgradeDomainDeltaUnhealthyNodes - The maximum allowed percentage of upgrade domain delta health degradation during the upgrade. Allowed values are integer values from zero to 100.
	MaxPercentUpgradeDomainDeltaUnhealthyNodes *int32 `json:"MaxPercentUpgradeDomainDeltaUnhealthyNodes,omitempty"`
	// ApplicationHealthPolicies - Defines the application health policy map used to evaluate the health of an application or one of its children entities.
	ApplicationHealthPolicies *ApplicationHealthPolicies `json:"ApplicationHealthPolicies,omitempty"`
}

// ClusterConfigurationUpgradeStatusInfo information about a standalone cluster configuration upgrade
// status.
type ClusterConfigurationUpgradeStatusInfo struct {
	autorest.Response `json:"-"`
	// UpgradeState - The state of the upgrade domain. Possible values include: 'UpgradeStateInvalid', 'UpgradeStateRollingBackInProgress', 'UpgradeStateRollingBackCompleted', 'UpgradeStateRollingForwardPending', 'UpgradeStateRollingForwardInProgress', 'UpgradeStateRollingForwardCompleted', 'UpgradeStateFailed'
	UpgradeState UpgradeState `json:"UpgradeState,omitempty"`
	// ProgressStatus - The cluster manifest version.
	ProgressStatus *int32 `json:"ProgressStatus,omitempty"`
	// ConfigVersion - The cluster configuration version.
	ConfigVersion *string `json:"ConfigVersion,omitempty"`
	// Details - The cluster upgrade status details.
	Details *string `json:"Details,omitempty"`
}

// BasicClusterEvent represents the base for all Cluster Events.
type BasicClusterEvent interface {
	AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool)
	AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool)
	AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool)
	AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool)
	AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool)
	AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool)
	AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool)
	AsChaosStoppedEvent() (*ChaosStoppedEvent, bool)
	AsChaosStartedEvent() (*ChaosStartedEvent, bool)
	AsClusterEvent() (*ClusterEvent, bool)
}

// ClusterEvent represents the base for all Cluster Events.
type ClusterEvent struct {
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

func unmarshalBasicClusterEvent(body []byte) (BasicClusterEvent, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindClusterNewHealthReport):
		var cnhre ClusterNewHealthReportEvent
		err := json.Unmarshal(body, &cnhre)
		return cnhre, err
	case string(KindClusterHealthReportExpired):
		var chree ClusterHealthReportExpiredEvent
		err := json.Unmarshal(body, &chree)
		return chree, err
	case string(KindClusterUpgradeCompleted):
		var cuce ClusterUpgradeCompletedEvent
		err := json.Unmarshal(body, &cuce)
		return cuce, err
	case string(KindClusterUpgradeDomainCompleted):
		var cudce ClusterUpgradeDomainCompletedEvent
		err := json.Unmarshal(body, &cudce)
		return cudce, err
	case string(KindClusterUpgradeRollbackCompleted):
		var curce ClusterUpgradeRollbackCompletedEvent
		err := json.Unmarshal(body, &curce)
		return curce, err
	case string(KindClusterUpgradeRollbackStarted):
		var curse ClusterUpgradeRollbackStartedEvent
		err := json.Unmarshal(body, &curse)
		return curse, err
	case string(KindClusterUpgradeStarted):
		var cuse ClusterUpgradeStartedEvent
		err := json.Unmarshal(body, &cuse)
		return cuse, err
	case string(KindChaosStopped):
		var cse ChaosStoppedEvent
		err := json.Unmarshal(body, &cse)
		return cse, err
	case string(KindChaosStarted):
		var cse ChaosStartedEvent
		err := json.Unmarshal(body, &cse)
		return cse, err
	default:
		var ce ClusterEvent
		err := json.Unmarshal(body, &ce)
		return ce, err
	}
}
func unmarshalBasicClusterEventArray(body []byte) ([]BasicClusterEvent, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	ceArray := make([]BasicClusterEvent, len(rawMessages))

	for index, rawMessage := range rawMessages {
		ce, err := unmarshalBasicClusterEvent(*rawMessage)
		if err != nil {
			return nil, err
		}
		ceArray[index] = ce
	}
	return ceArray, nil
}

// MarshalJSON is the custom marshaler for ClusterEvent.
func (ce ClusterEvent) MarshalJSON() ([]byte, error) {
	ce.Kind = KindClusterEvent
	objectMap := make(map[string]interface{})
	if ce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ce.EventInstanceID
	}
	if ce.Category != nil {
		objectMap["Category"] = ce.Category
	}
	if ce.TimeStamp != nil {
		objectMap["TimeStamp"] = ce.TimeStamp
	}
	if ce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ce.HasCorrelatedEvents
	}
	if ce.Kind != "" {
		objectMap["Kind"] = ce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return &ce, true
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return &ce, true
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ClusterEvent.
func (ce ClusterEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ce, true
}

// ClusterHealth represents the health of the cluster.
// Contains the cluster aggregated health state, the cluster application and node health states as well as
// the health events and the unhealthy evaluations.
type ClusterHealth struct {
	autorest.Response `json:"-"`
	// NodeHealthStates - Cluster node health states as found in the health store.
	NodeHealthStates *[]NodeHealthState `json:"NodeHealthStates,omitempty"`
	// ApplicationHealthStates - Cluster application health states as found in the health store.
	ApplicationHealthStates *[]ApplicationHealthState `json:"ApplicationHealthStates,omitempty"`
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

// ClusterHealthChunk represents the health chunk of the cluster.
// Contains the cluster aggregated health state, and the cluster entities that respect the input filter.
type ClusterHealthChunk struct {
	autorest.Response `json:"-"`
	// HealthState - The HealthState representing the aggregated health state of the cluster computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired cluster health policy and the application health policies. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// NodeHealthStateChunks - The list of node health state chunks in the cluster that respect the filters in the cluster health chunk query description.
	NodeHealthStateChunks *NodeHealthStateChunkList `json:"NodeHealthStateChunks,omitempty"`
	// ApplicationHealthStateChunks - The list of application health state chunks in the cluster that respect the filters in the cluster health chunk query description.
	ApplicationHealthStateChunks *ApplicationHealthStateChunkList `json:"ApplicationHealthStateChunks,omitempty"`
}

// ClusterHealthChunkQueryDescription the cluster health chunk query description, which can specify the
// health policies to evaluate cluster health and very expressive filters to select which cluster entities
// to include in response.
type ClusterHealthChunkQueryDescription struct {
	// NodeFilters - Defines a list of filters that specify which nodes to be included in the returned cluster health chunk.
	// If no filters are specified, no nodes are returned. All the nodes are used to evaluate the cluster's aggregated health state, regardless of the input filters.
	// The cluster health chunk query may specify multiple node filters.
	// For example, it can specify a filter to return all nodes with health state Error and another filter to always include a node identified by its NodeName.
	NodeFilters *[]NodeHealthStateFilter `json:"NodeFilters,omitempty"`
	// ApplicationFilters - Defines a list of filters that specify which applications to be included in the returned cluster health chunk.
	// If no filters are specified, no applications are returned. All the applications are used to evaluate the cluster's aggregated health state, regardless of the input filters.
	// The cluster health chunk query may specify multiple application filters.
	// For example, it can specify a filter to return all applications with health state Error and another filter to always include applications of a specified application type.
	ApplicationFilters *[]ApplicationHealthStateFilter `json:"ApplicationFilters,omitempty"`
	// ClusterHealthPolicy - Defines a health policy used to evaluate the health of the cluster or of a cluster node.
	ClusterHealthPolicy *ClusterHealthPolicy `json:"ClusterHealthPolicy,omitempty"`
	// ApplicationHealthPolicies - Defines the application health policy map used to evaluate the health of an application or one of its children entities.
	ApplicationHealthPolicies *ApplicationHealthPolicies `json:"ApplicationHealthPolicies,omitempty"`
}

// ClusterHealthPolicies health policies to evaluate cluster health.
type ClusterHealthPolicies struct {
	// ApplicationHealthPolicyMap - Defines a map that contains specific application health policies for different applications.
	// Each entry specifies as key the application name and as value an ApplicationHealthPolicy used to evaluate the application health.
	// If an application is not specified in the map, the application health evaluation uses the ApplicationHealthPolicy found in its application manifest or the default application health policy (if no health policy is defined in the manifest).
	// The map is empty by default.
	ApplicationHealthPolicyMap *[]ApplicationHealthPolicyMapItem `json:"ApplicationHealthPolicyMap,omitempty"`
	// ClusterHealthPolicy - Defines a health policy used to evaluate the health of the cluster or of a cluster node.
	ClusterHealthPolicy *ClusterHealthPolicy `json:"ClusterHealthPolicy,omitempty"`
}

// ClusterHealthPolicy defines a health policy used to evaluate the health of the cluster or of a cluster
// node.
type ClusterHealthPolicy struct {
	// ConsiderWarningAsError - Indicates whether warnings are treated with the same severity as errors.
	ConsiderWarningAsError *bool `json:"ConsiderWarningAsError,omitempty"`
	// MaxPercentUnhealthyNodes - The maximum allowed percentage of unhealthy nodes before reporting an error. For example, to allow 10% of nodes to be unhealthy, this value would be 10.
	// The percentage represents the maximum tolerated percentage of nodes that can be unhealthy before the cluster is considered in error.
	// If the percentage is respected but there is at least one unhealthy node, the health is evaluated as Warning.
	// The percentage is calculated by dividing the number of unhealthy nodes over the total number of nodes in the cluster.
	// The computation rounds up to tolerate one failure on small numbers of nodes. Default percentage is zero.
	// In large clusters, some nodes will always be down or out for repairs, so this percentage should be configured to tolerate that.
	MaxPercentUnhealthyNodes *int32 `json:"MaxPercentUnhealthyNodes,omitempty"`
	// MaxPercentUnhealthyApplications - The maximum allowed percentage of unhealthy applications before reporting an error. For example, to allow 10% of applications to be unhealthy, this value would be 10.
	// The percentage represents the maximum tolerated percentage of applications that can be unhealthy before the cluster is considered in error.
	// If the percentage is respected but there is at least one unhealthy application, the health is evaluated as Warning.
	// This is calculated by dividing the number of unhealthy applications over the total number of application instances in the cluster, excluding applications of application types that are included in the ApplicationTypeHealthPolicyMap.
	// The computation rounds up to tolerate one failure on small numbers of applications. Default percentage is zero.
	MaxPercentUnhealthyApplications *int32 `json:"MaxPercentUnhealthyApplications,omitempty"`
	// ApplicationTypeHealthPolicyMap - Defines a map with max percentage unhealthy applications for specific application types.
	// Each entry specifies as key the application type name and as value an integer that represents the MaxPercentUnhealthyApplications percentage used to evaluate the applications of the specified application type.
	// The application type health policy map can be used during cluster health evaluation to describe special application types.
	// The application types included in the map are evaluated against the percentage specified in the map, and not with the global MaxPercentUnhealthyApplications defined in the cluster health policy.
	// The applications of application types specified in the map are not counted against the global pool of applications.
	// For example, if some applications of a type are critical, the cluster administrator can add an entry to the map for that application type
	// and assign it a value of 0% (that is, do not tolerate any failures).
	// All other applications can be evaluated with MaxPercentUnhealthyApplications set to 20% to tolerate some failures out of the thousands of application instances.
	// The application type health policy map is used only if the cluster manifest enables application type health evaluation using the configuration entry for HealthManager/EnableApplicationTypeHealthEvaluation.
	ApplicationTypeHealthPolicyMap *[]ApplicationTypeHealthPolicyMapItem `json:"ApplicationTypeHealthPolicyMap,omitempty"`
}

// ClusterHealthReportExpiredEvent cluster Health Report Expired event.
type ClusterHealthReportExpiredEvent struct {
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) MarshalJSON() ([]byte, error) {
	chree.Kind = KindClusterHealthReportExpired
	objectMap := make(map[string]interface{})
	if chree.SourceID != nil {
		objectMap["SourceId"] = chree.SourceID
	}
	if chree.Property != nil {
		objectMap["Property"] = chree.Property
	}
	if chree.HealthState != nil {
		objectMap["HealthState"] = chree.HealthState
	}
	if chree.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = chree.TimeToLiveMs
	}
	if chree.SequenceNumber != nil {
		objectMap["SequenceNumber"] = chree.SequenceNumber
	}
	if chree.Description != nil {
		objectMap["Description"] = chree.Description
	}
	if chree.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = chree.RemoveWhenExpired
	}
	if chree.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = chree.SourceUtcTimestamp
	}
	if chree.EventInstanceID != nil {
		objectMap["EventInstanceId"] = chree.EventInstanceID
	}
	if chree.Category != nil {
		objectMap["Category"] = chree.Category
	}
	if chree.TimeStamp != nil {
		objectMap["TimeStamp"] = chree.TimeStamp
	}
	if chree.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = chree.HasCorrelatedEvents
	}
	if chree.Kind != "" {
		objectMap["Kind"] = chree.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return &chree, true
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return &chree, true
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ClusterHealthReportExpiredEvent.
func (chree ClusterHealthReportExpiredEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &chree, true
}

// ClusterLoadInfo information about load in a Service Fabric cluster. It holds a summary of all metrics
// and their load in a cluster.
type ClusterLoadInfo struct {
	autorest.Response `json:"-"`
	// LastBalancingStartTimeUtc - The starting time of last resource balancing run.
	LastBalancingStartTimeUtc *date.Time `json:"LastBalancingStartTimeUtc,omitempty"`
	// LastBalancingEndTimeUtc - The end time of last resource balancing run.
	LastBalancingEndTimeUtc *date.Time `json:"LastBalancingEndTimeUtc,omitempty"`
	// LoadMetricInformation - List that contains metrics and their load information in this cluster.
	LoadMetricInformation *[]LoadMetricInformation `json:"LoadMetricInformation,omitempty"`
}

// ClusterManifest information about the cluster manifest.
type ClusterManifest struct {
	autorest.Response `json:"-"`
	// Manifest - The contents of the cluster manifest file.
	Manifest *string `json:"Manifest,omitempty"`
}

// ClusterNewHealthReportEvent cluster Health Report Created event.
type ClusterNewHealthReportEvent struct {
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) MarshalJSON() ([]byte, error) {
	cnhre.Kind = KindClusterNewHealthReport
	objectMap := make(map[string]interface{})
	if cnhre.SourceID != nil {
		objectMap["SourceId"] = cnhre.SourceID
	}
	if cnhre.Property != nil {
		objectMap["Property"] = cnhre.Property
	}
	if cnhre.HealthState != nil {
		objectMap["HealthState"] = cnhre.HealthState
	}
	if cnhre.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = cnhre.TimeToLiveMs
	}
	if cnhre.SequenceNumber != nil {
		objectMap["SequenceNumber"] = cnhre.SequenceNumber
	}
	if cnhre.Description != nil {
		objectMap["Description"] = cnhre.Description
	}
	if cnhre.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = cnhre.RemoveWhenExpired
	}
	if cnhre.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = cnhre.SourceUtcTimestamp
	}
	if cnhre.EventInstanceID != nil {
		objectMap["EventInstanceId"] = cnhre.EventInstanceID
	}
	if cnhre.Category != nil {
		objectMap["Category"] = cnhre.Category
	}
	if cnhre.TimeStamp != nil {
		objectMap["TimeStamp"] = cnhre.TimeStamp
	}
	if cnhre.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = cnhre.HasCorrelatedEvents
	}
	if cnhre.Kind != "" {
		objectMap["Kind"] = cnhre.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return &cnhre, true
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return &cnhre, true
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ClusterNewHealthReportEvent.
func (cnhre ClusterNewHealthReportEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &cnhre, true
}

// ClusterUpgradeCompletedEvent cluster Upgrade Completed event.
type ClusterUpgradeCompletedEvent struct {
	// TargetClusterVersion - Target Cluster version.
	TargetClusterVersion *string `json:"TargetClusterVersion,omitempty"`
	// OverallUpgradeElapsedTimeInMs - Overall duration of upgrade in milli-seconds.
	OverallUpgradeElapsedTimeInMs *float64 `json:"OverallUpgradeElapsedTimeInMs,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) MarshalJSON() ([]byte, error) {
	cuce.Kind = KindClusterUpgradeCompleted
	objectMap := make(map[string]interface{})
	if cuce.TargetClusterVersion != nil {
		objectMap["TargetClusterVersion"] = cuce.TargetClusterVersion
	}
	if cuce.OverallUpgradeElapsedTimeInMs != nil {
		objectMap["OverallUpgradeElapsedTimeInMs"] = cuce.OverallUpgradeElapsedTimeInMs
	}
	if cuce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = cuce.EventInstanceID
	}
	if cuce.Category != nil {
		objectMap["Category"] = cuce.Category
	}
	if cuce.TimeStamp != nil {
		objectMap["TimeStamp"] = cuce.TimeStamp
	}
	if cuce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = cuce.HasCorrelatedEvents
	}
	if cuce.Kind != "" {
		objectMap["Kind"] = cuce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return &cuce, true
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return &cuce, true
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ClusterUpgradeCompletedEvent.
func (cuce ClusterUpgradeCompletedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &cuce, true
}

// ClusterUpgradeDescriptionObject represents a ServiceFabric cluster upgrade
type ClusterUpgradeDescriptionObject struct {
	// ConfigVersion - The cluster configuration version (specified in the cluster manifest).
	ConfigVersion *string `json:"ConfigVersion,omitempty"`
	// CodeVersion - The ServiceFabric code version of the cluster.
	CodeVersion *string `json:"CodeVersion,omitempty"`
	// UpgradeKind - The kind of upgrade out of the following possible values. Possible values include: 'UpgradeKindInvalid', 'UpgradeKindRolling'
	UpgradeKind UpgradeKind `json:"UpgradeKind,omitempty"`
	// RollingUpgradeMode - The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto, UnmonitoredManual, and Monitored. Possible values include: 'UpgradeModeInvalid', 'UpgradeModeUnmonitoredAuto', 'UpgradeModeUnmonitoredManual', 'UpgradeModeMonitored'
	RollingUpgradeMode UpgradeMode `json:"RollingUpgradeMode,omitempty"`
	// UpgradeReplicaSetCheckTimeoutInSeconds - The maximum amount of time to block processing of an upgrade domain and prevent loss of availability when there are unexpected issues. When this timeout expires, processing of the upgrade domain will proceed regardless of availability loss issues. The timeout is reset at the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive. (unsigned 32-bit integer).
	UpgradeReplicaSetCheckTimeoutInSeconds *int64 `json:"UpgradeReplicaSetCheckTimeoutInSeconds,omitempty"`
	// ForceRestart - If true, then processes are forcefully restarted during upgrade even when the code version has not changed (the upgrade only changes configuration or data).
	ForceRestart *bool `json:"ForceRestart,omitempty"`
	// SortOrder - Defines the order in which an upgrade proceeds through the cluster. Possible values include: 'UpgradeSortOrderInvalid', 'UpgradeSortOrderDefault', 'UpgradeSortOrderNumeric', 'UpgradeSortOrderLexicographical', 'UpgradeSortOrderReverseNumeric', 'UpgradeSortOrderReverseLexicographical'
	SortOrder UpgradeSortOrder `json:"SortOrder,omitempty"`
	// EnableDeltaHealthEvaluation - When true, enables delta health evaluation rather than absolute health evaluation after completion of each upgrade domain.
	EnableDeltaHealthEvaluation *bool `json:"EnableDeltaHealthEvaluation,omitempty"`
	// MonitoringPolicy - Describes the parameters for monitoring an upgrade in Monitored mode.
	MonitoringPolicy *MonitoringPolicyDescription `json:"MonitoringPolicy,omitempty"`
	// ClusterHealthPolicy - Defines a health policy used to evaluate the health of the cluster or of a cluster node.
	ClusterHealthPolicy *ClusterHealthPolicy `json:"ClusterHealthPolicy,omitempty"`
	// ClusterUpgradeHealthPolicy - Defines a health policy used to evaluate the health of the cluster during a cluster upgrade.
	ClusterUpgradeHealthPolicy *ClusterUpgradeHealthPolicyObject `json:"ClusterUpgradeHealthPolicy,omitempty"`
	// ApplicationHealthPolicyMap - Defines a map that contains specific application health policies for different applications.
	// Each entry specifies as key the application name and as value an ApplicationHealthPolicy used to evaluate the application health.
	// If an application is not specified in the map, the application health evaluation uses the ApplicationHealthPolicy found in its application manifest or the default application health policy (if no health policy is defined in the manifest).
	// The map is empty by default.
	ApplicationHealthPolicyMap *[]ApplicationHealthPolicyMapItem `json:"ApplicationHealthPolicyMap,omitempty"`
}

// ClusterUpgradeDomainCompletedEvent cluster Upgrade Domain Completed event.
type ClusterUpgradeDomainCompletedEvent struct {
	// TargetClusterVersion - Target Cluster version.
	TargetClusterVersion *string `json:"TargetClusterVersion,omitempty"`
	// UpgradeState - State of upgrade.
	UpgradeState *string `json:"UpgradeState,omitempty"`
	// UpgradeDomains - Upgrade domains.
	UpgradeDomains *string `json:"UpgradeDomains,omitempty"`
	// UpgradeDomainElapsedTimeInMs - Duration of domain upgrade in milli-seconds.
	UpgradeDomainElapsedTimeInMs *float64 `json:"UpgradeDomainElapsedTimeInMs,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) MarshalJSON() ([]byte, error) {
	cudce.Kind = KindClusterUpgradeDomainCompleted
	objectMap := make(map[string]interface{})
	if cudce.TargetClusterVersion != nil {
		objectMap["TargetClusterVersion"] = cudce.TargetClusterVersion
	}
	if cudce.UpgradeState != nil {
		objectMap["UpgradeState"] = cudce.UpgradeState
	}
	if cudce.UpgradeDomains != nil {
		objectMap["UpgradeDomains"] = cudce.UpgradeDomains
	}
	if cudce.UpgradeDomainElapsedTimeInMs != nil {
		objectMap["UpgradeDomainElapsedTimeInMs"] = cudce.UpgradeDomainElapsedTimeInMs
	}
	if cudce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = cudce.EventInstanceID
	}
	if cudce.Category != nil {
		objectMap["Category"] = cudce.Category
	}
	if cudce.TimeStamp != nil {
		objectMap["TimeStamp"] = cudce.TimeStamp
	}
	if cudce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = cudce.HasCorrelatedEvents
	}
	if cudce.Kind != "" {
		objectMap["Kind"] = cudce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return &cudce, true
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return &cudce, true
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ClusterUpgradeDomainCompletedEvent.
func (cudce ClusterUpgradeDomainCompletedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &cudce, true
}

// ClusterUpgradeHealthPolicyObject defines a health policy used to evaluate the health of the cluster
// during a cluster upgrade.
type ClusterUpgradeHealthPolicyObject struct {
	// MaxPercentDeltaUnhealthyNodes - The maximum allowed percentage of nodes health degradation allowed during cluster upgrades. The delta is measured between the state of the nodes at the beginning of upgrade and the state of the nodes at the time of the health evaluation. The check is performed after every upgrade domain upgrade completion to make sure the global state of the cluster is within tolerated limits. The default value is 10%.
	MaxPercentDeltaUnhealthyNodes *int32 `json:"MaxPercentDeltaUnhealthyNodes,omitempty"`
	// MaxPercentUpgradeDomainDeltaUnhealthyNodes - The maximum allowed percentage of upgrade domain nodes health degradation allowed during cluster upgrades. The delta is measured between the state of the upgrade domain nodes at the beginning of upgrade and the state of the upgrade domain nodes at the time of the health evaluation. The check is performed after every upgrade domain upgrade completion for all completed upgrade domains to make sure the state of the upgrade domains is within tolerated limits. The default value is 15%.
	MaxPercentUpgradeDomainDeltaUnhealthyNodes *int32 `json:"MaxPercentUpgradeDomainDeltaUnhealthyNodes,omitempty"`
}

// ClusterUpgradeProgressObject information about a cluster upgrade.
type ClusterUpgradeProgressObject struct {
	autorest.Response `json:"-"`
	// CodeVersion - The ServiceFabric code version of the cluster.
	CodeVersion *string `json:"CodeVersion,omitempty"`
	// ConfigVersion - The cluster configuration version (specified in the cluster manifest).
	ConfigVersion *string `json:"ConfigVersion,omitempty"`
	// UpgradeDomains - List of upgrade domains and their statuses.
	UpgradeDomains *[]UpgradeDomainInfo `json:"UpgradeDomains,omitempty"`
	// UpgradeState - The state of the upgrade domain. Possible values include: 'UpgradeStateInvalid', 'UpgradeStateRollingBackInProgress', 'UpgradeStateRollingBackCompleted', 'UpgradeStateRollingForwardPending', 'UpgradeStateRollingForwardInProgress', 'UpgradeStateRollingForwardCompleted', 'UpgradeStateFailed'
	UpgradeState UpgradeState `json:"UpgradeState,omitempty"`
	// NextUpgradeDomain - The name of the next upgrade domain to be processed.
	NextUpgradeDomain *string `json:"NextUpgradeDomain,omitempty"`
	// RollingUpgradeMode - The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto, UnmonitoredManual, and Monitored. Possible values include: 'UpgradeModeInvalid', 'UpgradeModeUnmonitoredAuto', 'UpgradeModeUnmonitoredManual', 'UpgradeModeMonitored'
	RollingUpgradeMode UpgradeMode `json:"RollingUpgradeMode,omitempty"`
	// UpgradeDescription - Represents a ServiceFabric cluster upgrade
	UpgradeDescription *ClusterUpgradeDescriptionObject `json:"UpgradeDescription,omitempty"`
	// UpgradeDurationInMilliseconds - The estimated elapsed time spent processing the current overall upgrade.
	UpgradeDurationInMilliseconds *string `json:"UpgradeDurationInMilliseconds,omitempty"`
	// UpgradeDomainDurationInMilliseconds - The estimated elapsed time spent processing the current upgrade domain.
	UpgradeDomainDurationInMilliseconds *string `json:"UpgradeDomainDurationInMilliseconds,omitempty"`
	// UnhealthyEvaluations - List of health evaluations that resulted in the current aggregated health state.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// CurrentUpgradeDomainProgress - Information about the current in-progress upgrade domain.
	CurrentUpgradeDomainProgress *CurrentUpgradeDomainProgressInfo `json:"CurrentUpgradeDomainProgress,omitempty"`
	// StartTimestampUtc - The start time of the upgrade in UTC.
	StartTimestampUtc *string `json:"StartTimestampUtc,omitempty"`
	// FailureTimestampUtc - The failure time of the upgrade in UTC.
	FailureTimestampUtc *string `json:"FailureTimestampUtc,omitempty"`
	// FailureReason - The cause of an upgrade failure that resulted in FailureAction being executed. Possible values include: 'None', 'Interrupted', 'HealthCheck', 'UpgradeDomainTimeout', 'OverallUpgradeTimeout'
	FailureReason FailureReason `json:"FailureReason,omitempty"`
	// UpgradeDomainProgressAtFailure - The detailed upgrade progress for nodes in the current upgrade domain at the point of failure.
	UpgradeDomainProgressAtFailure *FailedUpgradeDomainProgressObject `json:"UpgradeDomainProgressAtFailure,omitempty"`
}

// ClusterUpgradeRollbackCompletedEvent cluster Upgrade Rollback Completed event.
type ClusterUpgradeRollbackCompletedEvent struct {
	// TargetClusterVersion - Target Cluster version.
	TargetClusterVersion *string `json:"TargetClusterVersion,omitempty"`
	// FailureReason - Describes failure.
	FailureReason *string `json:"FailureReason,omitempty"`
	// OverallUpgradeElapsedTimeInMs - Overall duration of upgrade in milli-seconds.
	OverallUpgradeElapsedTimeInMs *float64 `json:"OverallUpgradeElapsedTimeInMs,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) MarshalJSON() ([]byte, error) {
	curce.Kind = KindClusterUpgradeRollbackCompleted
	objectMap := make(map[string]interface{})
	if curce.TargetClusterVersion != nil {
		objectMap["TargetClusterVersion"] = curce.TargetClusterVersion
	}
	if curce.FailureReason != nil {
		objectMap["FailureReason"] = curce.FailureReason
	}
	if curce.OverallUpgradeElapsedTimeInMs != nil {
		objectMap["OverallUpgradeElapsedTimeInMs"] = curce.OverallUpgradeElapsedTimeInMs
	}
	if curce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = curce.EventInstanceID
	}
	if curce.Category != nil {
		objectMap["Category"] = curce.Category
	}
	if curce.TimeStamp != nil {
		objectMap["TimeStamp"] = curce.TimeStamp
	}
	if curce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = curce.HasCorrelatedEvents
	}
	if curce.Kind != "" {
		objectMap["Kind"] = curce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return &curce, true
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return &curce, true
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackCompletedEvent.
func (curce ClusterUpgradeRollbackCompletedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &curce, true
}

// ClusterUpgradeRollbackStartedEvent cluster Upgrade Rollback Started event.
type ClusterUpgradeRollbackStartedEvent struct {
	// TargetClusterVersion - Target Cluster version.
	TargetClusterVersion *string `json:"TargetClusterVersion,omitempty"`
	// FailureReason - Describes failure.
	FailureReason *string `json:"FailureReason,omitempty"`
	// OverallUpgradeElapsedTimeInMs - Overall duration of upgrade in milli-seconds.
	OverallUpgradeElapsedTimeInMs *float64 `json:"OverallUpgradeElapsedTimeInMs,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) MarshalJSON() ([]byte, error) {
	curse.Kind = KindClusterUpgradeRollbackStarted
	objectMap := make(map[string]interface{})
	if curse.TargetClusterVersion != nil {
		objectMap["TargetClusterVersion"] = curse.TargetClusterVersion
	}
	if curse.FailureReason != nil {
		objectMap["FailureReason"] = curse.FailureReason
	}
	if curse.OverallUpgradeElapsedTimeInMs != nil {
		objectMap["OverallUpgradeElapsedTimeInMs"] = curse.OverallUpgradeElapsedTimeInMs
	}
	if curse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = curse.EventInstanceID
	}
	if curse.Category != nil {
		objectMap["Category"] = curse.Category
	}
	if curse.TimeStamp != nil {
		objectMap["TimeStamp"] = curse.TimeStamp
	}
	if curse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = curse.HasCorrelatedEvents
	}
	if curse.Kind != "" {
		objectMap["Kind"] = curse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return &curse, true
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return &curse, true
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ClusterUpgradeRollbackStartedEvent.
func (curse ClusterUpgradeRollbackStartedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &curse, true
}

// ClusterUpgradeStartedEvent cluster Upgrade Started event.
type ClusterUpgradeStartedEvent struct {
	// CurrentClusterVersion - Current Cluster version.
	CurrentClusterVersion *string `json:"CurrentClusterVersion,omitempty"`
	// TargetClusterVersion - Target Cluster version.
	TargetClusterVersion *string `json:"TargetClusterVersion,omitempty"`
	// UpgradeType - Type of upgrade.
	UpgradeType *string `json:"UpgradeType,omitempty"`
	// RollingUpgradeMode - Mode of upgrade.
	RollingUpgradeMode *string `json:"RollingUpgradeMode,omitempty"`
	// FailureAction - Action if failed.
	FailureAction *string `json:"FailureAction,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) MarshalJSON() ([]byte, error) {
	cuse.Kind = KindClusterUpgradeStarted
	objectMap := make(map[string]interface{})
	if cuse.CurrentClusterVersion != nil {
		objectMap["CurrentClusterVersion"] = cuse.CurrentClusterVersion
	}
	if cuse.TargetClusterVersion != nil {
		objectMap["TargetClusterVersion"] = cuse.TargetClusterVersion
	}
	if cuse.UpgradeType != nil {
		objectMap["UpgradeType"] = cuse.UpgradeType
	}
	if cuse.RollingUpgradeMode != nil {
		objectMap["RollingUpgradeMode"] = cuse.RollingUpgradeMode
	}
	if cuse.FailureAction != nil {
		objectMap["FailureAction"] = cuse.FailureAction
	}
	if cuse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = cuse.EventInstanceID
	}
	if cuse.Category != nil {
		objectMap["Category"] = cuse.Category
	}
	if cuse.TimeStamp != nil {
		objectMap["TimeStamp"] = cuse.TimeStamp
	}
	if cuse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = cuse.HasCorrelatedEvents
	}
	if cuse.Kind != "" {
		objectMap["Kind"] = cuse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return &cuse, true
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return &cuse, true
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ClusterUpgradeStartedEvent.
func (cuse ClusterUpgradeStartedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &cuse, true
}

// ClusterVersion the cluster version.
type ClusterVersion struct {
	autorest.Response `json:"-"`
	// Version - The Service Fabric cluster runtime version.
	Version *string `json:"Version,omitempty"`
}

// CodePackageEntryPoint information about setup or main entry point of a code package deployed on a
// Service Fabric node.
type CodePackageEntryPoint struct {
	// EntryPointLocation - The location of entry point executable on the node.
	EntryPointLocation *string `json:"EntryPointLocation,omitempty"`
	// ProcessID - The process ID of the entry point.
	ProcessID *string `json:"ProcessId,omitempty"`
	// RunAsUserName - The user name under which entry point executable is run on the node.
	RunAsUserName *string `json:"RunAsUserName,omitempty"`
	// CodePackageEntryPointStatistics - Statistics about setup or main entry point  of a code package deployed on a Service Fabric node.
	CodePackageEntryPointStatistics *CodePackageEntryPointStatistics `json:"CodePackageEntryPointStatistics,omitempty"`
	// Status - Specifies the status of the code package entry point deployed on a Service Fabric node. Possible values include: 'EntryPointStatusInvalid', 'EntryPointStatusPending', 'EntryPointStatusStarting', 'EntryPointStatusStarted', 'EntryPointStatusStopping', 'EntryPointStatusStopped'
	Status EntryPointStatus `json:"Status,omitempty"`
	// NextActivationTime - The time (in UTC) when the entry point executable will be run next.
	NextActivationTime *date.Time `json:"NextActivationTime,omitempty"`
	// InstanceID - The instance ID for current running entry point. For a code package setup entry point (if specified) runs first and after it finishes main entry point is started. Each time entry point executable is run, its instance id will change.
	InstanceID *string `json:"InstanceId,omitempty"`
}

// CodePackageEntryPointStatistics statistics about setup or main entry point  of a code package deployed
// on a Service Fabric node.
type CodePackageEntryPointStatistics struct {
	// LastExitCode - The last exit code of the entry point.
	LastExitCode *string `json:"LastExitCode,omitempty"`
	// LastActivationTime - The last time (in UTC) when Service Fabric attempted to run the entry point.
	LastActivationTime *date.Time `json:"LastActivationTime,omitempty"`
	// LastExitTime - The last time (in UTC) when the entry point finished running.
	LastExitTime *date.Time `json:"LastExitTime,omitempty"`
	// LastSuccessfulActivationTime - The last time (in UTC) when the entry point ran successfully.
	LastSuccessfulActivationTime *date.Time `json:"LastSuccessfulActivationTime,omitempty"`
	// LastSuccessfulExitTime - The last time (in UTC) when the entry point finished running gracefully.
	LastSuccessfulExitTime *date.Time `json:"LastSuccessfulExitTime,omitempty"`
	// ActivationCount - Number of times the entry point has run.
	ActivationCount *string `json:"ActivationCount,omitempty"`
	// ActivationFailureCount - Number of times the entry point failed to run.
	ActivationFailureCount *string `json:"ActivationFailureCount,omitempty"`
	// ContinuousActivationFailureCount - Number of times the entry point continuously failed to run.
	ContinuousActivationFailureCount *string `json:"ContinuousActivationFailureCount,omitempty"`
	// ExitCount - Number of times the entry point finished running.
	ExitCount *string `json:"ExitCount,omitempty"`
	// ExitFailureCount - Number of times the entry point failed to exit gracefully.
	ExitFailureCount *string `json:"ExitFailureCount,omitempty"`
	// ContinuousExitFailureCount - Number of times the entry point continuously failed to exit gracefully.
	ContinuousExitFailureCount *string `json:"ContinuousExitFailureCount,omitempty"`
}

// ComposeDeploymentStatusInfo information about a Service Fabric compose deployment.
type ComposeDeploymentStatusInfo struct {
	autorest.Response `json:"-"`
	// Name - The name of the deployment.
	Name *string `json:"Name,omitempty"`
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// Status - The status of the compose deployment. Possible values include: 'ComposeDeploymentStatusInvalid', 'ComposeDeploymentStatusProvisioning', 'ComposeDeploymentStatusCreating', 'ComposeDeploymentStatusReady', 'ComposeDeploymentStatusUnprovisioning', 'ComposeDeploymentStatusDeleting', 'ComposeDeploymentStatusFailed', 'ComposeDeploymentStatusUpgrading'
	Status ComposeDeploymentStatus `json:"Status,omitempty"`
	// StatusDetails - The status details of compose deployment including failure message.
	StatusDetails *string `json:"StatusDetails,omitempty"`
}

// ComposeDeploymentUpgradeDescription describes the parameters for a compose deployment upgrade.
type ComposeDeploymentUpgradeDescription struct {
	// DeploymentName - The name of the deployment.
	DeploymentName *string `json:"DeploymentName,omitempty"`
	// ComposeFileContent - The content of the compose file that describes the deployment to create.
	ComposeFileContent *string `json:"ComposeFileContent,omitempty"`
	// RegistryCredential - Credential information to connect to container registry.
	RegistryCredential *RegistryCredential `json:"RegistryCredential,omitempty"`
	// UpgradeKind - The kind of upgrade out of the following possible values. Possible values include: 'UpgradeKindInvalid', 'UpgradeKindRolling'
	UpgradeKind UpgradeKind `json:"UpgradeKind,omitempty"`
	// RollingUpgradeMode - The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto, UnmonitoredManual, and Monitored. Possible values include: 'UpgradeModeInvalid', 'UpgradeModeUnmonitoredAuto', 'UpgradeModeUnmonitoredManual', 'UpgradeModeMonitored'
	RollingUpgradeMode UpgradeMode `json:"RollingUpgradeMode,omitempty"`
	// UpgradeReplicaSetCheckTimeoutInSeconds - The maximum amount of time to block processing of an upgrade domain and prevent loss of availability when there are unexpected issues. When this timeout expires, processing of the upgrade domain will proceed regardless of availability loss issues. The timeout is reset at the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive. (unsigned 32-bit integer).
	UpgradeReplicaSetCheckTimeoutInSeconds *int64 `json:"UpgradeReplicaSetCheckTimeoutInSeconds,omitempty"`
	// ForceRestart - If true, then processes are forcefully restarted during upgrade even when the code version has not changed (the upgrade only changes configuration or data).
	ForceRestart *bool `json:"ForceRestart,omitempty"`
	// MonitoringPolicy - Describes the parameters for monitoring an upgrade in Monitored mode.
	MonitoringPolicy *MonitoringPolicyDescription `json:"MonitoringPolicy,omitempty"`
	// ApplicationHealthPolicy - Defines a health policy used to evaluate the health of an application or one of its children entities.
	ApplicationHealthPolicy *ApplicationHealthPolicy `json:"ApplicationHealthPolicy,omitempty"`
}

// ComposeDeploymentUpgradeProgressInfo describes the parameters for a compose deployment upgrade.
type ComposeDeploymentUpgradeProgressInfo struct {
	autorest.Response `json:"-"`
	// DeploymentName - The name of the target deployment.
	DeploymentName *string `json:"DeploymentName,omitempty"`
	// ApplicationName - The name of the target application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// UpgradeState - The state of the compose deployment upgrade. Possible values include: 'ComposeDeploymentUpgradeStateInvalid', 'ComposeDeploymentUpgradeStateProvisioningTarget', 'ComposeDeploymentUpgradeStateRollingForwardInProgress', 'ComposeDeploymentUpgradeStateRollingForwardPending', 'ComposeDeploymentUpgradeStateUnprovisioningCurrent', 'ComposeDeploymentUpgradeStateRollingForwardCompleted', 'ComposeDeploymentUpgradeStateRollingBackInProgress', 'ComposeDeploymentUpgradeStateUnprovisioningTarget', 'ComposeDeploymentUpgradeStateRollingBackCompleted', 'ComposeDeploymentUpgradeStateFailed'
	UpgradeState ComposeDeploymentUpgradeState `json:"UpgradeState,omitempty"`
	// UpgradeStatusDetails - Additional detailed information about the status of the pending upgrade.
	UpgradeStatusDetails *string `json:"UpgradeStatusDetails,omitempty"`
	// UpgradeKind - The kind of upgrade out of the following possible values. Possible values include: 'UpgradeKindInvalid', 'UpgradeKindRolling'
	UpgradeKind UpgradeKind `json:"UpgradeKind,omitempty"`
	// RollingUpgradeMode - The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto, UnmonitoredManual, and Monitored. Possible values include: 'UpgradeModeInvalid', 'UpgradeModeUnmonitoredAuto', 'UpgradeModeUnmonitoredManual', 'UpgradeModeMonitored'
	RollingUpgradeMode UpgradeMode `json:"RollingUpgradeMode,omitempty"`
	// ForceRestart - If true, then processes are forcefully restarted during upgrade even when the code version has not changed (the upgrade only changes configuration or data).
	ForceRestart *bool `json:"ForceRestart,omitempty"`
	// UpgradeReplicaSetCheckTimeoutInSeconds - The maximum amount of time to block processing of an upgrade domain and prevent loss of availability when there are unexpected issues. When this timeout expires, processing of the upgrade domain will proceed regardless of availability loss issues. The timeout is reset at the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive. (unsigned 32-bit integer).
	UpgradeReplicaSetCheckTimeoutInSeconds *int64 `json:"UpgradeReplicaSetCheckTimeoutInSeconds,omitempty"`
	// MonitoringPolicy - Describes the parameters for monitoring an upgrade in Monitored mode.
	MonitoringPolicy *MonitoringPolicyDescription `json:"MonitoringPolicy,omitempty"`
	// ApplicationHealthPolicy - Defines a health policy used to evaluate the health of an application or one of its children entities.
	ApplicationHealthPolicy *ApplicationHealthPolicy `json:"ApplicationHealthPolicy,omitempty"`
	// TargetApplicationTypeVersion - The target application type version (found in the application manifest) for the application upgrade.
	TargetApplicationTypeVersion *string `json:"TargetApplicationTypeVersion,omitempty"`
	// UpgradeDuration - The estimated amount of time that the overall upgrade elapsed. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	UpgradeDuration *string `json:"UpgradeDuration,omitempty"`
	// CurrentUpgradeDomainDuration - The estimated amount of time spent processing current Upgrade Domain. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	CurrentUpgradeDomainDuration *string `json:"CurrentUpgradeDomainDuration,omitempty"`
	// ApplicationUnhealthyEvaluations - List of health evaluations that resulted in the current aggregated health state.
	ApplicationUnhealthyEvaluations *[]HealthEvaluationWrapper `json:"ApplicationUnhealthyEvaluations,omitempty"`
	// CurrentUpgradeDomainProgress - Information about the current in-progress upgrade domain.
	CurrentUpgradeDomainProgress *CurrentUpgradeDomainProgressInfo `json:"CurrentUpgradeDomainProgress,omitempty"`
	// StartTimestampUtc - The estimated UTC datetime when the upgrade started.
	StartTimestampUtc *string `json:"StartTimestampUtc,omitempty"`
	// FailureTimestampUtc - The estimated UTC datetime when the upgrade failed and FailureAction was executed.
	FailureTimestampUtc *string `json:"FailureTimestampUtc,omitempty"`
	// FailureReason - The cause of an upgrade failure that resulted in FailureAction being executed. Possible values include: 'None', 'Interrupted', 'HealthCheck', 'UpgradeDomainTimeout', 'OverallUpgradeTimeout'
	FailureReason FailureReason `json:"FailureReason,omitempty"`
	// UpgradeDomainProgressAtFailure - Information about the upgrade domain progress at the time of upgrade failure.
	UpgradeDomainProgressAtFailure *FailureUpgradeDomainProgressInfo `json:"UpgradeDomainProgressAtFailure,omitempty"`
	// ApplicationUpgradeStatusDetails - Additional details of application upgrade including failure message.
	ApplicationUpgradeStatusDetails *string `json:"ApplicationUpgradeStatusDetails,omitempty"`
}

// ContainerAPIRequestBody parameters for making container API call.
type ContainerAPIRequestBody struct {
	// HTTPVerb - HTTP verb of container REST API, defaults to "GET"
	HTTPVerb *string `json:"HttpVerb,omitempty"`
	// URIPath - URI path of container REST API
	URIPath *string `json:"UriPath,omitempty"`
	// ContentType - Content type of container REST API request, defaults to "application/json"
	ContentType *string `json:"Content-Type,omitempty"`
	// Body - HTTP request body of container REST API
	Body *string `json:"Body,omitempty"`
}

// ContainerAPIResponse response body that wraps container API result.
type ContainerAPIResponse struct {
	autorest.Response `json:"-"`
	// ContainerAPIResult - Container API result.
	ContainerAPIResult *ContainerAPIResult `json:"ContainerApiResult,omitempty"`
}

// ContainerAPIResult container API result.
type ContainerAPIResult struct {
	// Status - HTTP status code returned by the target container API
	Status *int32 `json:"Status,omitempty"`
	// ContentType - HTTP content type
	ContentType *string `json:"Content-Type,omitempty"`
	// ContentEncoding - HTTP content encoding
	ContentEncoding *string `json:"Content-Encoding,omitempty"`
	// Body - container API result body
	Body *string `json:"Body,omitempty"`
}

// ContainerCodePackageProperties describes a container and its runtime properties.
type ContainerCodePackageProperties struct {
	// Name - The name of the code package.
	Name *string `json:"name,omitempty"`
	// Image - The Container image to use.
	Image *string `json:"image,omitempty"`
	// ImageRegistryCredential - Image registry credential.
	ImageRegistryCredential *ImageRegistryCredential `json:"imageRegistryCredential,omitempty"`
	// Entrypoint - Override for the default entry point in the container.
	Entrypoint *string `json:"entrypoint,omitempty"`
	// Commands - Command array to execute within the container in exec form.
	Commands *[]string `json:"commands,omitempty"`
	// EnvironmentVariables - The environment variables to set in this container
	EnvironmentVariables *[]EnvironmentVariable `json:"environmentVariables,omitempty"`
	// Settings - The settings to set in this container. The setting file path can be fetched from environment variable "Fabric_SettingPath". The path for Windows container is "C:\\secrets". The path for Linux container is "/var/secrets".
	Settings *[]Setting `json:"settings,omitempty"`
	// Labels - The labels to set in this container.
	Labels *[]ContainerLabel `json:"labels,omitempty"`
	// Endpoints - The endpoints exposed by this container.
	Endpoints *[]EndpointProperties `json:"endpoints,omitempty"`
	// Resources - The resources required by this container.
	Resources *ResourceRequirements `json:"resources,omitempty"`
	// VolumeRefs - Volumes to be attached to the container. The lifetime of these volumes is independent of the application's lifetime.
	VolumeRefs *[]VolumeReference `json:"volumeRefs,omitempty"`
	// Volumes - Volumes to be attached to the container. The lifetime of these volumes is scoped to the application's lifetime.
	Volumes *[]ApplicationScopedVolume `json:"volumes,omitempty"`
	// Diagnostics - Reference to sinks in DiagnosticsDescription.
	Diagnostics *DiagnosticsRef `json:"diagnostics,omitempty"`
	// ReliableCollectionsRefs - A list of ReliableCollection resources used by this particular code package. Please refer to ReliableCollectionsRef for more details.
	ReliableCollectionsRefs *[]ReliableCollectionsRef `json:"reliableCollectionsRefs,omitempty"`
	// InstanceView - READ-ONLY; Runtime information of a container instance.
	InstanceView *ContainerInstanceView `json:"instanceView,omitempty"`
}

// MarshalJSON is the custom marshaler for ContainerCodePackageProperties.
func (ccpp ContainerCodePackageProperties) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if ccpp.Name != nil {
		objectMap["name"] = ccpp.Name
	}
	if ccpp.Image != nil {
		objectMap["image"] = ccpp.Image
	}
	if ccpp.ImageRegistryCredential != nil {
		objectMap["imageRegistryCredential"] = ccpp.ImageRegistryCredential
	}
	if ccpp.Entrypoint != nil {
		objectMap["entrypoint"] = ccpp.Entrypoint
	}
	if ccpp.Commands != nil {
		objectMap["commands"] = ccpp.Commands
	}
	if ccpp.EnvironmentVariables != nil {
		objectMap["environmentVariables"] = ccpp.EnvironmentVariables
	}
	if ccpp.Settings != nil {
		objectMap["settings"] = ccpp.Settings
	}
	if ccpp.Labels != nil {
		objectMap["labels"] = ccpp.Labels
	}
	if ccpp.Endpoints != nil {
		objectMap["endpoints"] = ccpp.Endpoints
	}
	if ccpp.Resources != nil {
		objectMap["resources"] = ccpp.Resources
	}
	if ccpp.VolumeRefs != nil {
		objectMap["volumeRefs"] = ccpp.VolumeRefs
	}
	if ccpp.Volumes != nil {
		objectMap["volumes"] = ccpp.Volumes
	}
	if ccpp.Diagnostics != nil {
		objectMap["diagnostics"] = ccpp.Diagnostics
	}
	if ccpp.ReliableCollectionsRefs != nil {
		objectMap["reliableCollectionsRefs"] = ccpp.ReliableCollectionsRefs
	}
	return json.Marshal(objectMap)
}

// ContainerEvent a container event.
type ContainerEvent struct {
	// Name - The name of the container event.
	Name *string `json:"name,omitempty"`
	// Count - The count of the event.
	Count *int32 `json:"count,omitempty"`
	// FirstTimestamp - Date/time of the first event.
	FirstTimestamp *string `json:"firstTimestamp,omitempty"`
	// LastTimestamp - Date/time of the last event.
	LastTimestamp *string `json:"lastTimestamp,omitempty"`
	// Message - The event message
	Message *string `json:"message,omitempty"`
	// Type - The event type.
	Type *string `json:"type,omitempty"`
}

// ContainerInstanceEvent represents the base for all Container Events.
type ContainerInstanceEvent struct {
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) MarshalJSON() ([]byte, error) {
	cie.Kind = KindContainerInstanceEvent
	objectMap := make(map[string]interface{})
	if cie.EventInstanceID != nil {
		objectMap["EventInstanceId"] = cie.EventInstanceID
	}
	if cie.Category != nil {
		objectMap["Category"] = cie.Category
	}
	if cie.TimeStamp != nil {
		objectMap["TimeStamp"] = cie.TimeStamp
	}
	if cie.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = cie.HasCorrelatedEvents
	}
	if cie.Kind != "" {
		objectMap["Kind"] = cie.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return &cie, true
}

// AsNodeEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ContainerInstanceEvent.
func (cie ContainerInstanceEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &cie, true
}

// ContainerInstanceView runtime information of a container instance.
type ContainerInstanceView struct {
	// RestartCount - The number of times the container has been restarted.
	RestartCount *int32 `json:"restartCount,omitempty"`
	// CurrentState - Current container instance state.
	CurrentState *ContainerState `json:"currentState,omitempty"`
	// PreviousState - Previous container instance state.
	PreviousState *ContainerState `json:"previousState,omitempty"`
	// Events - The events of this container instance.
	Events *[]ContainerEvent `json:"events,omitempty"`
}

// ContainerLabel describes a container label.
type ContainerLabel struct {
	// Name - The name of the container label.
	Name *string `json:"name,omitempty"`
	// Value - The value of the container label.
	Value *string `json:"value,omitempty"`
}

// ContainerLogs container logs.
type ContainerLogs struct {
	autorest.Response `json:"-"`
	// Content - Container logs.
	Content *string `json:"Content,omitempty"`
}

// ContainerState the container state.
type ContainerState struct {
	// State - The state of this container
	State *string `json:"state,omitempty"`
	// StartTime - Date/time when the container state started.
	StartTime *date.Time `json:"startTime,omitempty"`
	// ExitCode - The container exit code.
	ExitCode *string `json:"exitCode,omitempty"`
	// FinishTime - Date/time when the container state finished.
	FinishTime *date.Time `json:"finishTime,omitempty"`
	// DetailStatus - Human-readable status of this state.
	DetailStatus *string `json:"detailStatus,omitempty"`
}

// CreateComposeDeploymentDescription defines description for creating a Service Fabric compose deployment.
type CreateComposeDeploymentDescription struct {
	// DeploymentName - The name of the deployment.
	DeploymentName *string `json:"DeploymentName,omitempty"`
	// ComposeFileContent - The content of the compose file that describes the deployment to create.
	ComposeFileContent *string `json:"ComposeFileContent,omitempty"`
	// RegistryCredential - Credential information to connect to container registry.
	RegistryCredential *RegistryCredential `json:"RegistryCredential,omitempty"`
}

// CurrentUpgradeDomainProgressInfo information about the current in-progress upgrade domain.
type CurrentUpgradeDomainProgressInfo struct {
	// DomainName - The name of the upgrade domain
	DomainName *string `json:"DomainName,omitempty"`
	// NodeUpgradeProgressList - List of upgrading nodes and their statuses
	NodeUpgradeProgressList *[]NodeUpgradeProgressInfo `json:"NodeUpgradeProgressList,omitempty"`
}

// DeactivationIntentDescription describes the intent or reason for deactivating the node.
type DeactivationIntentDescription struct {
	// DeactivationIntent - Describes the intent or reason for deactivating the node. The possible values are following. Possible values include: 'Pause', 'Restart', 'RemoveData'
	DeactivationIntent DeactivationIntent `json:"DeactivationIntent,omitempty"`
}

// DeletePropertyBatchOperation represents a PropertyBatchOperation that deletes a specified property if it
// exists.
// Note that if one PropertyBatchOperation in a PropertyBatch fails,
// the entire batch fails and cannot be committed in a transactional manner.
type DeletePropertyBatchOperation struct {
	// PropertyName - The name of the Service Fabric property.
	PropertyName *string `json:"PropertyName,omitempty"`
	// Kind - Possible values include: 'KindPropertyBatchOperation', 'KindCheckExists', 'KindCheckSequence', 'KindCheckValue', 'KindDelete', 'KindGet', 'KindPut'
	Kind KindBasicPropertyBatchOperation `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeletePropertyBatchOperation.
func (dpbo DeletePropertyBatchOperation) MarshalJSON() ([]byte, error) {
	dpbo.Kind = KindDelete
	objectMap := make(map[string]interface{})
	if dpbo.PropertyName != nil {
		objectMap["PropertyName"] = dpbo.PropertyName
	}
	if dpbo.Kind != "" {
		objectMap["Kind"] = dpbo.Kind
	}
	return json.Marshal(objectMap)
}

// AsCheckExistsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for DeletePropertyBatchOperation.
func (dpbo DeletePropertyBatchOperation) AsCheckExistsPropertyBatchOperation() (*CheckExistsPropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckSequencePropertyBatchOperation is the BasicPropertyBatchOperation implementation for DeletePropertyBatchOperation.
func (dpbo DeletePropertyBatchOperation) AsCheckSequencePropertyBatchOperation() (*CheckSequencePropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckValuePropertyBatchOperation is the BasicPropertyBatchOperation implementation for DeletePropertyBatchOperation.
func (dpbo DeletePropertyBatchOperation) AsCheckValuePropertyBatchOperation() (*CheckValuePropertyBatchOperation, bool) {
	return nil, false
}

// AsDeletePropertyBatchOperation is the BasicPropertyBatchOperation implementation for DeletePropertyBatchOperation.
func (dpbo DeletePropertyBatchOperation) AsDeletePropertyBatchOperation() (*DeletePropertyBatchOperation, bool) {
	return &dpbo, true
}

// AsGetPropertyBatchOperation is the BasicPropertyBatchOperation implementation for DeletePropertyBatchOperation.
func (dpbo DeletePropertyBatchOperation) AsGetPropertyBatchOperation() (*GetPropertyBatchOperation, bool) {
	return nil, false
}

// AsPutPropertyBatchOperation is the BasicPropertyBatchOperation implementation for DeletePropertyBatchOperation.
func (dpbo DeletePropertyBatchOperation) AsPutPropertyBatchOperation() (*PutPropertyBatchOperation, bool) {
	return nil, false
}

// AsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for DeletePropertyBatchOperation.
func (dpbo DeletePropertyBatchOperation) AsPropertyBatchOperation() (*PropertyBatchOperation, bool) {
	return nil, false
}

// AsBasicPropertyBatchOperation is the BasicPropertyBatchOperation implementation for DeletePropertyBatchOperation.
func (dpbo DeletePropertyBatchOperation) AsBasicPropertyBatchOperation() (BasicPropertyBatchOperation, bool) {
	return &dpbo, true
}

// DeltaNodesCheckHealthEvaluation represents health evaluation for delta nodes, containing health
// evaluations for each unhealthy node that impacted current aggregated health state.
// Can be returned during cluster upgrade when the aggregated health state of the cluster is Warning or
// Error.
type DeltaNodesCheckHealthEvaluation struct {
	// BaselineErrorCount - Number of nodes with aggregated heath state Error in the health store at the beginning of the cluster upgrade.
	BaselineErrorCount *int64 `json:"BaselineErrorCount,omitempty"`
	// BaselineTotalCount - Total number of nodes in the health store at the beginning of the cluster upgrade.
	BaselineTotalCount *int64 `json:"BaselineTotalCount,omitempty"`
	// MaxPercentDeltaUnhealthyNodes - Maximum allowed percentage of delta unhealthy nodes from the ClusterUpgradeHealthPolicy.
	MaxPercentDeltaUnhealthyNodes *int32 `json:"MaxPercentDeltaUnhealthyNodes,omitempty"`
	// TotalCount - Total number of nodes in the health store.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state.
	// Includes all the unhealthy NodeHealthEvaluation that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) MarshalJSON() ([]byte, error) {
	dnche.Kind = KindDeltaNodesCheck
	objectMap := make(map[string]interface{})
	if dnche.BaselineErrorCount != nil {
		objectMap["BaselineErrorCount"] = dnche.BaselineErrorCount
	}
	if dnche.BaselineTotalCount != nil {
		objectMap["BaselineTotalCount"] = dnche.BaselineTotalCount
	}
	if dnche.MaxPercentDeltaUnhealthyNodes != nil {
		objectMap["MaxPercentDeltaUnhealthyNodes"] = dnche.MaxPercentDeltaUnhealthyNodes
	}
	if dnche.TotalCount != nil {
		objectMap["TotalCount"] = dnche.TotalCount
	}
	if dnche.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = dnche.UnhealthyEvaluations
	}
	if dnche.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = dnche.AggregatedHealthState
	}
	if dnche.Description != nil {
		objectMap["Description"] = dnche.Description
	}
	if dnche.Kind != "" {
		objectMap["Kind"] = dnche.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return &dnche, true
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for DeltaNodesCheckHealthEvaluation.
func (dnche DeltaNodesCheckHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &dnche, true
}

// DeployedApplicationHealth information about the health of an application deployed on a Service Fabric
// node.
type DeployedApplicationHealth struct {
	autorest.Response `json:"-"`
	// Name - Name of the application deployed on the node whose health information is described by this object.
	Name *string `json:"Name,omitempty"`
	// NodeName - Name of the node where this application is deployed.
	NodeName *string `json:"NodeName,omitempty"`
	// DeployedServicePackageHealthStates - Deployed service package health states for the current deployed application as found in the health store.
	DeployedServicePackageHealthStates *[]DeployedServicePackageHealthState `json:"DeployedServicePackageHealthStates,omitempty"`
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

// DeployedApplicationHealthEvaluation represents health evaluation for a deployed application, containing
// information about the data and the algorithm used by the health store to evaluate health.
type DeployedApplicationHealthEvaluation struct {
	// NodeName - Name of the node where the application is deployed to.
	NodeName *string `json:"NodeName,omitempty"`
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// UnhealthyEvaluations - List of  unhealthy evaluations that led to the current aggregated health state of the deployed application.
	// The types of the unhealthy evaluations can be DeployedServicePackagesHealthEvaluation or EventHealthEvaluation.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) MarshalJSON() ([]byte, error) {
	dahe.Kind = KindDeployedApplication
	objectMap := make(map[string]interface{})
	if dahe.NodeName != nil {
		objectMap["NodeName"] = dahe.NodeName
	}
	if dahe.ApplicationName != nil {
		objectMap["ApplicationName"] = dahe.ApplicationName
	}
	if dahe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = dahe.UnhealthyEvaluations
	}
	if dahe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = dahe.AggregatedHealthState
	}
	if dahe.Description != nil {
		objectMap["Description"] = dahe.Description
	}
	if dahe.Kind != "" {
		objectMap["Kind"] = dahe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return &dahe, true
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationHealthEvaluation.
func (dahe DeployedApplicationHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &dahe, true
}

// DeployedApplicationHealthReportExpiredEvent deployed Application Health Report Expired event.
type DeployedApplicationHealthReportExpiredEvent struct {
	// ApplicationInstanceID - Id of Application instance.
	ApplicationInstanceID *int64 `json:"ApplicationInstanceId,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) MarshalJSON() ([]byte, error) {
	dahree.Kind = KindDeployedApplicationHealthReportExpired
	objectMap := make(map[string]interface{})
	if dahree.ApplicationInstanceID != nil {
		objectMap["ApplicationInstanceId"] = dahree.ApplicationInstanceID
	}
	if dahree.NodeName != nil {
		objectMap["NodeName"] = dahree.NodeName
	}
	if dahree.SourceID != nil {
		objectMap["SourceId"] = dahree.SourceID
	}
	if dahree.Property != nil {
		objectMap["Property"] = dahree.Property
	}
	if dahree.HealthState != nil {
		objectMap["HealthState"] = dahree.HealthState
	}
	if dahree.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = dahree.TimeToLiveMs
	}
	if dahree.SequenceNumber != nil {
		objectMap["SequenceNumber"] = dahree.SequenceNumber
	}
	if dahree.Description != nil {
		objectMap["Description"] = dahree.Description
	}
	if dahree.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = dahree.RemoveWhenExpired
	}
	if dahree.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = dahree.SourceUtcTimestamp
	}
	if dahree.ApplicationID != nil {
		objectMap["ApplicationId"] = dahree.ApplicationID
	}
	if dahree.EventInstanceID != nil {
		objectMap["EventInstanceId"] = dahree.EventInstanceID
	}
	if dahree.Category != nil {
		objectMap["Category"] = dahree.Category
	}
	if dahree.TimeStamp != nil {
		objectMap["TimeStamp"] = dahree.TimeStamp
	}
	if dahree.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = dahree.HasCorrelatedEvents
	}
	if dahree.Kind != "" {
		objectMap["Kind"] = dahree.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &dahree, true
}

// AsClusterEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return &dahree, true
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for DeployedApplicationHealthReportExpiredEvent.
func (dahree DeployedApplicationHealthReportExpiredEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &dahree, true
}

// DeployedApplicationHealthState represents the health state of a deployed application, which contains the
// entity identifier and the aggregated health state.
type DeployedApplicationHealthState struct {
	// NodeName - Name of the node on which the service package is deployed.
	NodeName *string `json:"NodeName,omitempty"`
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
}

// DeployedApplicationHealthStateChunk represents the health state chunk of a deployed application, which
// contains the node where the application is deployed, the aggregated health state and any deployed
// service packages that respect the chunk query description filters.
type DeployedApplicationHealthStateChunk struct {
	// NodeName - The name of node where the application is deployed.
	NodeName *string `json:"NodeName,omitempty"`
	// DeployedServicePackageHealthStateChunks - The list of deployed service package health state chunks belonging to the deployed application that respect the filters in the cluster health chunk query description.
	DeployedServicePackageHealthStateChunks *DeployedServicePackageHealthStateChunkList `json:"DeployedServicePackageHealthStateChunks,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
}

// DeployedApplicationHealthStateChunkList the list of deployed application health state chunks that
// respect the input filters in the chunk query. Returned by get cluster health state chunks query.
type DeployedApplicationHealthStateChunkList struct {
	// Items - The list of deployed application health state chunks that respect the input filters in the chunk query.
	Items *[]DeployedApplicationHealthStateChunk `json:"Items,omitempty"`
}

// DeployedApplicationHealthStateFilter defines matching criteria to determine whether a deployed
// application should be included as a child of an application in the cluster health chunk.
// The deployed applications are only returned if the parent application matches a filter specified in the
// cluster health chunk query description.
// One filter can match zero, one or multiple deployed applications, depending on its properties.
type DeployedApplicationHealthStateFilter struct {
	// NodeNameFilter - The name of the node where the application is deployed in order to match the filter.
	// If specified, the filter is applied only to the application deployed on the specified node.
	// If the application is not deployed on the node with the specified name, no deployed application is returned in the cluster health chunk based on this filter.
	// Otherwise, the deployed application is included in the cluster health chunk if it respects the other filter properties.
	// If not specified, all deployed applications that match the parent filters (if any) are taken into consideration and matched against the other filter members, like health state filter.
	NodeNameFilter *string `json:"NodeNameFilter,omitempty"`
	// HealthStateFilter - The filter for the health state of the deployed applications. It allows selecting deployed applications if they match the desired health states.
	// The possible values are integer value of one of the following health states. Only deployed applications that match the filter are returned. All deployed applications are used to evaluate the cluster aggregated health state.
	// If not specified, default value is None, unless the node name is specified. If the filter has default value and node name is specified, the matching deployed application is returned.
	// The state values are flag-based enumeration, so the value could be a combination of these values obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6, it matches deployed applications with HealthState value of OK (2) and Warning (4).
	// - Default - Default value. Matches any HealthState. The value is zero.
	// - None - Filter that doesn't match any HealthState value. Used in order to return no results on a given collection of states. The value is 1.
	// - Ok - Filter that matches input with HealthState value Ok. The value is 2.
	// - Warning - Filter that matches input with HealthState value Warning. The value is 4.
	// - Error - Filter that matches input with HealthState value Error. The value is 8.
	// - All - Filter that matches input with any HealthState value. The value is 65535.
	HealthStateFilter *int32 `json:"HealthStateFilter,omitempty"`
	// DeployedServicePackageFilters - Defines a list of filters that specify which deployed service packages to be included in the returned cluster health chunk as children of the parent deployed application. The deployed service packages are returned only if the parent deployed application matches a filter.
	// If the list is empty, no deployed service packages are returned. All the deployed service packages are used to evaluate the parent deployed application aggregated health state, regardless of the input filters.
	// The deployed application filter may specify multiple deployed service package filters.
	// For example, it can specify a filter to return all deployed service packages with health state Error and another filter to always include a deployed service package on a node.
	DeployedServicePackageFilters *[]DeployedServicePackageHealthStateFilter `json:"DeployedServicePackageFilters,omitempty"`
}

// DeployedApplicationInfo information about application deployed on the node.
type DeployedApplicationInfo struct {
	autorest.Response `json:"-"`
	// ID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ID *string `json:"Id,omitempty"`
	// Name - The name of the application, including the 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// TypeName - The application type name as defined in the application manifest.
	TypeName *string `json:"TypeName,omitempty"`
	// Status - The status of the application deployed on the node. Following are the possible values. Possible values include: 'DeployedApplicationStatusInvalid', 'DeployedApplicationStatusDownloading', 'DeployedApplicationStatusActivating', 'DeployedApplicationStatusActive', 'DeployedApplicationStatusUpgrading', 'DeployedApplicationStatusDeactivating'
	Status DeployedApplicationStatus `json:"Status,omitempty"`
	// WorkDirectory - The work directory of the application on the node. The work directory can be used to store application data.
	WorkDirectory *string `json:"WorkDirectory,omitempty"`
	// LogDirectory - The log directory of the application on the node. The log directory can be used to store application logs.
	LogDirectory *string `json:"LogDirectory,omitempty"`
	// TempDirectory - The temp directory of the application on the node. The code packages belonging to the application are forked with this directory set as their temporary directory.
	TempDirectory *string `json:"TempDirectory,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
}

// DeployedApplicationNewHealthReportEvent deployed Application Health Report Created event.
type DeployedApplicationNewHealthReportEvent struct {
	// ApplicationInstanceID - Id of Application instance.
	ApplicationInstanceID *int64 `json:"ApplicationInstanceId,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) MarshalJSON() ([]byte, error) {
	danhre.Kind = KindDeployedApplicationNewHealthReport
	objectMap := make(map[string]interface{})
	if danhre.ApplicationInstanceID != nil {
		objectMap["ApplicationInstanceId"] = danhre.ApplicationInstanceID
	}
	if danhre.NodeName != nil {
		objectMap["NodeName"] = danhre.NodeName
	}
	if danhre.SourceID != nil {
		objectMap["SourceId"] = danhre.SourceID
	}
	if danhre.Property != nil {
		objectMap["Property"] = danhre.Property
	}
	if danhre.HealthState != nil {
		objectMap["HealthState"] = danhre.HealthState
	}
	if danhre.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = danhre.TimeToLiveMs
	}
	if danhre.SequenceNumber != nil {
		objectMap["SequenceNumber"] = danhre.SequenceNumber
	}
	if danhre.Description != nil {
		objectMap["Description"] = danhre.Description
	}
	if danhre.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = danhre.RemoveWhenExpired
	}
	if danhre.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = danhre.SourceUtcTimestamp
	}
	if danhre.ApplicationID != nil {
		objectMap["ApplicationId"] = danhre.ApplicationID
	}
	if danhre.EventInstanceID != nil {
		objectMap["EventInstanceId"] = danhre.EventInstanceID
	}
	if danhre.Category != nil {
		objectMap["Category"] = danhre.Category
	}
	if danhre.TimeStamp != nil {
		objectMap["TimeStamp"] = danhre.TimeStamp
	}
	if danhre.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = danhre.HasCorrelatedEvents
	}
	if danhre.Kind != "" {
		objectMap["Kind"] = danhre.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &danhre, true
}

// AsClusterEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return &danhre, true
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for DeployedApplicationNewHealthReportEvent.
func (danhre DeployedApplicationNewHealthReportEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &danhre, true
}

// DeployedApplicationsHealthEvaluation represents health evaluation for deployed applications, containing
// health evaluations for each unhealthy deployed application that impacted current aggregated health
// state.
// Can be returned when evaluating application health and the aggregated health state is either Error or
// Warning.
type DeployedApplicationsHealthEvaluation struct {
	// MaxPercentUnhealthyDeployedApplications - Maximum allowed percentage of unhealthy deployed applications from the ApplicationHealthPolicy.
	MaxPercentUnhealthyDeployedApplications *int32 `json:"MaxPercentUnhealthyDeployedApplications,omitempty"`
	// TotalCount - Total number of deployed applications of the application in the health store.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state. Includes all the unhealthy DeployedApplicationHealthEvaluation that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) MarshalJSON() ([]byte, error) {
	dahe.Kind = KindDeployedApplications
	objectMap := make(map[string]interface{})
	if dahe.MaxPercentUnhealthyDeployedApplications != nil {
		objectMap["MaxPercentUnhealthyDeployedApplications"] = dahe.MaxPercentUnhealthyDeployedApplications
	}
	if dahe.TotalCount != nil {
		objectMap["TotalCount"] = dahe.TotalCount
	}
	if dahe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = dahe.UnhealthyEvaluations
	}
	if dahe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = dahe.AggregatedHealthState
	}
	if dahe.Description != nil {
		objectMap["Description"] = dahe.Description
	}
	if dahe.Kind != "" {
		objectMap["Kind"] = dahe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return &dahe, true
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for DeployedApplicationsHealthEvaluation.
func (dahe DeployedApplicationsHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &dahe, true
}

// DeployedCodePackageInfo information about code package deployed on a Service Fabric node.
type DeployedCodePackageInfo struct {
	// Name - The name of the code package.
	Name *string `json:"Name,omitempty"`
	// Version - The version of the code package specified in service manifest.
	Version *string `json:"Version,omitempty"`
	// ServiceManifestName - The name of service manifest that specified this code package.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// ServicePackageActivationID - The ActivationId of a deployed service package. If ServicePackageActivationMode specified at the time of creating the service
	// is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'), then value of ServicePackageActivationId
	// is always an empty string.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// HostType - Specifies the type of host for main entry point of a code package as specified in service manifest. Possible values include: 'HostTypeInvalid', 'HostTypeExeHost', 'HostTypeContainerHost'
	HostType HostType `json:"HostType,omitempty"`
	// HostIsolationMode - Specifies the isolation mode of main entry point of a code package when it's host type is ContainerHost. This is specified as part of container host policies in application manifest while importing service manifest. Possible values include: 'HostIsolationModeNone', 'HostIsolationModeProcess', 'HostIsolationModeHyperV'
	HostIsolationMode HostIsolationMode `json:"HostIsolationMode,omitempty"`
	// Status - Specifies the status of a deployed application or service package on a Service Fabric node. Possible values include: 'DeploymentStatusInvalid', 'DeploymentStatusDownloading', 'DeploymentStatusActivating', 'DeploymentStatusActive', 'DeploymentStatusUpgrading', 'DeploymentStatusDeactivating'
	Status DeploymentStatus `json:"Status,omitempty"`
	// RunFrequencyInterval - The interval at which code package is run. This is used for periodic code package.
	RunFrequencyInterval *string `json:"RunFrequencyInterval,omitempty"`
	// SetupEntryPoint - Information about setup or main entry point of a code package deployed on a Service Fabric node.
	SetupEntryPoint *CodePackageEntryPoint `json:"SetupEntryPoint,omitempty"`
	// MainEntryPoint - Information about setup or main entry point of a code package deployed on a Service Fabric node.
	MainEntryPoint *CodePackageEntryPoint `json:"MainEntryPoint,omitempty"`
}

// DeployedServicePackageHealth information about the health of a service package for a specific
// application deployed on a Service Fabric node.
type DeployedServicePackageHealth struct {
	autorest.Response `json:"-"`
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ServiceManifestName - Name of the service manifest.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// NodeName - Name of the node where this service package is deployed.
	NodeName *string `json:"NodeName,omitempty"`
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

// DeployedServicePackageHealthEvaluation represents health evaluation for a deployed service package,
// containing information about the data and the algorithm used by health store to evaluate health. The
// evaluation is returned only when the aggregated health state is either Error or Warning.
type DeployedServicePackageHealthEvaluation struct {
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ServiceManifestName - The name of the service manifest.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the current aggregated health state. The type of the unhealthy evaluations can be EventHealthEvaluation.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) MarshalJSON() ([]byte, error) {
	dsphe.Kind = KindDeployedServicePackage
	objectMap := make(map[string]interface{})
	if dsphe.NodeName != nil {
		objectMap["NodeName"] = dsphe.NodeName
	}
	if dsphe.ApplicationName != nil {
		objectMap["ApplicationName"] = dsphe.ApplicationName
	}
	if dsphe.ServiceManifestName != nil {
		objectMap["ServiceManifestName"] = dsphe.ServiceManifestName
	}
	if dsphe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = dsphe.UnhealthyEvaluations
	}
	if dsphe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = dsphe.AggregatedHealthState
	}
	if dsphe.Description != nil {
		objectMap["Description"] = dsphe.Description
	}
	if dsphe.Kind != "" {
		objectMap["Kind"] = dsphe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return &dsphe, true
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackageHealthEvaluation.
func (dsphe DeployedServicePackageHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &dsphe, true
}

// DeployedServicePackageHealthReportExpiredEvent deployed Service Health Report Expired event.
type DeployedServicePackageHealthReportExpiredEvent struct {
	// ServiceManifest - Service manifest name.
	ServiceManifest *string `json:"ServiceManifest,omitempty"`
	// ServicePackageInstanceID - Id of Service package instance.
	ServicePackageInstanceID *int64 `json:"ServicePackageInstanceId,omitempty"`
	// ServicePackageActivationID - Id of Service package activation.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) MarshalJSON() ([]byte, error) {
	dsphree.Kind = KindDeployedServicePackageHealthReportExpired
	objectMap := make(map[string]interface{})
	if dsphree.ServiceManifest != nil {
		objectMap["ServiceManifest"] = dsphree.ServiceManifest
	}
	if dsphree.ServicePackageInstanceID != nil {
		objectMap["ServicePackageInstanceId"] = dsphree.ServicePackageInstanceID
	}
	if dsphree.ServicePackageActivationID != nil {
		objectMap["ServicePackageActivationId"] = dsphree.ServicePackageActivationID
	}
	if dsphree.NodeName != nil {
		objectMap["NodeName"] = dsphree.NodeName
	}
	if dsphree.SourceID != nil {
		objectMap["SourceId"] = dsphree.SourceID
	}
	if dsphree.Property != nil {
		objectMap["Property"] = dsphree.Property
	}
	if dsphree.HealthState != nil {
		objectMap["HealthState"] = dsphree.HealthState
	}
	if dsphree.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = dsphree.TimeToLiveMs
	}
	if dsphree.SequenceNumber != nil {
		objectMap["SequenceNumber"] = dsphree.SequenceNumber
	}
	if dsphree.Description != nil {
		objectMap["Description"] = dsphree.Description
	}
	if dsphree.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = dsphree.RemoveWhenExpired
	}
	if dsphree.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = dsphree.SourceUtcTimestamp
	}
	if dsphree.ApplicationID != nil {
		objectMap["ApplicationId"] = dsphree.ApplicationID
	}
	if dsphree.EventInstanceID != nil {
		objectMap["EventInstanceId"] = dsphree.EventInstanceID
	}
	if dsphree.Category != nil {
		objectMap["Category"] = dsphree.Category
	}
	if dsphree.TimeStamp != nil {
		objectMap["TimeStamp"] = dsphree.TimeStamp
	}
	if dsphree.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = dsphree.HasCorrelatedEvents
	}
	if dsphree.Kind != "" {
		objectMap["Kind"] = dsphree.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &dsphree, true
}

// AsClusterEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return &dsphree, true
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for DeployedServicePackageHealthReportExpiredEvent.
func (dsphree DeployedServicePackageHealthReportExpiredEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &dsphree, true
}

// DeployedServicePackageHealthState represents the health state of a deployed service package, containing
// the entity identifier and the aggregated health state.
type DeployedServicePackageHealthState struct {
	// NodeName - Name of the node on which the service package is deployed.
	NodeName *string `json:"NodeName,omitempty"`
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ServiceManifestName - Name of the manifest describing the service package.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// ServicePackageActivationID - The ActivationId of a deployed service package. If ServicePackageActivationMode specified at the time of creating the service
	// is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'), then value of ServicePackageActivationId
	// is always an empty string.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
}

// DeployedServicePackageHealthStateChunk represents the health state chunk of a deployed service package,
// which contains the service manifest name and the service package aggregated health state.
type DeployedServicePackageHealthStateChunk struct {
	// ServiceManifestName - The name of the service manifest.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// ServicePackageActivationID - The ActivationId of a deployed service package. If ServicePackageActivationMode specified at the time of creating the service
	// is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'), then value of ServicePackageActivationId
	// is always an empty string.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
}

// DeployedServicePackageHealthStateChunkList the list of deployed service package health state chunks that
// respect the input filters in the chunk query. Returned by get cluster health state chunks query.
type DeployedServicePackageHealthStateChunkList struct {
	// Items - The list of deployed service package health state chunks that respect the input filters in the chunk query.
	Items *[]DeployedServicePackageHealthStateChunk `json:"Items,omitempty"`
}

// DeployedServicePackageHealthStateFilter defines matching criteria to determine whether a deployed
// service package should be included as a child of a deployed application in the cluster health chunk.
// The deployed service packages are only returned if the parent entities match a filter specified in the
// cluster health chunk query description. The parent deployed application and its parent application must
// be included in the cluster health chunk.
// One filter can match zero, one or multiple deployed service packages, depending on its properties.
type DeployedServicePackageHealthStateFilter struct {
	// ServiceManifestNameFilter - The name of the service manifest which identifies the deployed service packages that matches the filter.
	// If specified, the filter is applied only to the specified deployed service packages, if any.
	// If no deployed service packages with specified manifest name exist, nothing is returned in the cluster health chunk based on this filter.
	// If any deployed service package exists, they are included in the cluster health chunk if it respects the other filter properties.
	// If not specified, all deployed service packages that match the parent filters (if any) are taken into consideration and matched against the other filter members, like health state filter.
	ServiceManifestNameFilter *string `json:"ServiceManifestNameFilter,omitempty"`
	// ServicePackageActivationIDFilter - The activation ID of a deployed service package that matches the filter.
	// If not specified, the filter applies to all deployed service packages that match the other parameters.
	// If specified, the filter matches only the deployed service package with the specified activation ID.
	ServicePackageActivationIDFilter *string `json:"ServicePackageActivationIdFilter,omitempty"`
	// HealthStateFilter - The filter for the health state of the deployed service packages. It allows selecting deployed service packages if they match the desired health states.
	// The possible values are integer value of one of the following health states. Only deployed service packages that match the filter are returned. All deployed service packages are used to evaluate the parent deployed application aggregated health state.
	// If not specified, default value is None, unless the deployed service package ID is specified. If the filter has default value and deployed service package ID is specified, the matching deployed service package is returned.
	// The state values are flag-based enumeration, so the value could be a combination of these values obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6, it matches deployed service packages with HealthState value of OK (2) and Warning (4).
	// - Default - Default value. Matches any HealthState. The value is zero.
	// - None - Filter that doesn't match any HealthState value. Used in order to return no results on a given collection of states. The value is 1.
	// - Ok - Filter that matches input with HealthState value Ok. The value is 2.
	// - Warning - Filter that matches input with HealthState value Warning. The value is 4.
	// - Error - Filter that matches input with HealthState value Error. The value is 8.
	// - All - Filter that matches input with any HealthState value. The value is 65535.
	HealthStateFilter *int32 `json:"HealthStateFilter,omitempty"`
}

// DeployedServicePackageInfo information about service package deployed on a Service Fabric node.
type DeployedServicePackageInfo struct {
	// Name - The name of the service package as specified in the service manifest.
	Name *string `json:"Name,omitempty"`
	// Version - The version of the service package specified in service manifest.
	Version *string `json:"Version,omitempty"`
	// Status - Specifies the status of a deployed application or service package on a Service Fabric node. Possible values include: 'DeploymentStatusInvalid', 'DeploymentStatusDownloading', 'DeploymentStatusActivating', 'DeploymentStatusActive', 'DeploymentStatusUpgrading', 'DeploymentStatusDeactivating'
	Status DeploymentStatus `json:"Status,omitempty"`
	// ServicePackageActivationID - The ActivationId of a deployed service package. If ServicePackageActivationMode specified at the time of creating the service
	// is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'), then value of ServicePackageActivationId
	// is always an empty string.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
}

// DeployedServicePackageNewHealthReportEvent deployed Service Health Report Created event.
type DeployedServicePackageNewHealthReportEvent struct {
	// ServiceManifestName - Service manifest name.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// ServicePackageInstanceID - Id of Service package instance.
	ServicePackageInstanceID *int64 `json:"ServicePackageInstanceId,omitempty"`
	// ServicePackageActivationID - Id of Service package activation.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// ApplicationID - The identity of the application. This is an encoded representation of the application name. This is used in the REST APIs to identify the application resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the application name is "fabric:/myapp/app1",
	// the application identity would be "myapp\~app1" in 6.0+ and "myapp/app1" in previous versions.
	ApplicationID *string `json:"ApplicationId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) MarshalJSON() ([]byte, error) {
	dspnhre.Kind = KindDeployedServicePackageNewHealthReport
	objectMap := make(map[string]interface{})
	if dspnhre.ServiceManifestName != nil {
		objectMap["ServiceManifestName"] = dspnhre.ServiceManifestName
	}
	if dspnhre.ServicePackageInstanceID != nil {
		objectMap["ServicePackageInstanceId"] = dspnhre.ServicePackageInstanceID
	}
	if dspnhre.ServicePackageActivationID != nil {
		objectMap["ServicePackageActivationId"] = dspnhre.ServicePackageActivationID
	}
	if dspnhre.NodeName != nil {
		objectMap["NodeName"] = dspnhre.NodeName
	}
	if dspnhre.SourceID != nil {
		objectMap["SourceId"] = dspnhre.SourceID
	}
	if dspnhre.Property != nil {
		objectMap["Property"] = dspnhre.Property
	}
	if dspnhre.HealthState != nil {
		objectMap["HealthState"] = dspnhre.HealthState
	}
	if dspnhre.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = dspnhre.TimeToLiveMs
	}
	if dspnhre.SequenceNumber != nil {
		objectMap["SequenceNumber"] = dspnhre.SequenceNumber
	}
	if dspnhre.Description != nil {
		objectMap["Description"] = dspnhre.Description
	}
	if dspnhre.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = dspnhre.RemoveWhenExpired
	}
	if dspnhre.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = dspnhre.SourceUtcTimestamp
	}
	if dspnhre.ApplicationID != nil {
		objectMap["ApplicationId"] = dspnhre.ApplicationID
	}
	if dspnhre.EventInstanceID != nil {
		objectMap["EventInstanceId"] = dspnhre.EventInstanceID
	}
	if dspnhre.Category != nil {
		objectMap["Category"] = dspnhre.Category
	}
	if dspnhre.TimeStamp != nil {
		objectMap["TimeStamp"] = dspnhre.TimeStamp
	}
	if dspnhre.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = dspnhre.HasCorrelatedEvents
	}
	if dspnhre.Kind != "" {
		objectMap["Kind"] = dspnhre.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return &dspnhre, true
}

// AsClusterEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return &dspnhre, true
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for DeployedServicePackageNewHealthReportEvent.
func (dspnhre DeployedServicePackageNewHealthReportEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &dspnhre, true
}

// DeployedServicePackagesHealthEvaluation represents health evaluation for deployed service packages,
// containing health evaluations for each unhealthy deployed service package that impacted current
// aggregated health state. Can be returned when evaluating deployed application health and the aggregated
// health state is either Error or Warning.
type DeployedServicePackagesHealthEvaluation struct {
	// TotalCount - Total number of deployed service packages of the deployed application in the health store.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state. Includes all the unhealthy DeployedServicePackageHealthEvaluation that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) MarshalJSON() ([]byte, error) {
	dsphe.Kind = KindDeployedServicePackages
	objectMap := make(map[string]interface{})
	if dsphe.TotalCount != nil {
		objectMap["TotalCount"] = dsphe.TotalCount
	}
	if dsphe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = dsphe.UnhealthyEvaluations
	}
	if dsphe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = dsphe.AggregatedHealthState
	}
	if dsphe.Description != nil {
		objectMap["Description"] = dsphe.Description
	}
	if dsphe.Kind != "" {
		objectMap["Kind"] = dsphe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return &dsphe, true
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for DeployedServicePackagesHealthEvaluation.
func (dsphe DeployedServicePackagesHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &dsphe, true
}

// BasicDeployedServiceReplicaDetailInfo information about a Service Fabric service replica deployed on a node.
type BasicDeployedServiceReplicaDetailInfo interface {
	AsDeployedStatefulServiceReplicaDetailInfo() (*DeployedStatefulServiceReplicaDetailInfo, bool)
	AsDeployedStatelessServiceInstanceDetailInfo() (*DeployedStatelessServiceInstanceDetailInfo, bool)
	AsDeployedServiceReplicaDetailInfo() (*DeployedServiceReplicaDetailInfo, bool)
}

// DeployedServiceReplicaDetailInfo information about a Service Fabric service replica deployed on a node.
type DeployedServiceReplicaDetailInfo struct {
	autorest.Response `json:"-"`
	// ServiceName - Full hierarchical name of the service in URI format starting with `fabric:`.
	ServiceName *string `json:"ServiceName,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// CurrentServiceOperation - Specifies the current active life-cycle operation on a stateful service replica or stateless service instance. Possible values include: 'ServiceOperationNameUnknown', 'ServiceOperationNameNone', 'ServiceOperationNameOpen', 'ServiceOperationNameChangeRole', 'ServiceOperationNameClose', 'ServiceOperationNameAbort'
	CurrentServiceOperation ServiceOperationName `json:"CurrentServiceOperation,omitempty"`
	// CurrentServiceOperationStartTimeUtc - The start time of the current service operation in UTC format.
	CurrentServiceOperationStartTimeUtc *date.Time `json:"CurrentServiceOperationStartTimeUtc,omitempty"`
	// ReportedLoad - List of load reported by replica.
	ReportedLoad *[]LoadMetricReportInfo `json:"ReportedLoad,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindDeployedServiceReplicaDetailInfo', 'ServiceKindStateful1', 'ServiceKindStateless1'
	ServiceKind ServiceKindBasicDeployedServiceReplicaDetailInfo `json:"ServiceKind,omitempty"`
}

func unmarshalBasicDeployedServiceReplicaDetailInfo(body []byte) (BasicDeployedServiceReplicaDetailInfo, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ServiceKind"] {
	case string(ServiceKindStateful1):
		var dssrdi DeployedStatefulServiceReplicaDetailInfo
		err := json.Unmarshal(body, &dssrdi)
		return dssrdi, err
	case string(ServiceKindStateless1):
		var dssidi DeployedStatelessServiceInstanceDetailInfo
		err := json.Unmarshal(body, &dssidi)
		return dssidi, err
	default:
		var dsrdi DeployedServiceReplicaDetailInfo
		err := json.Unmarshal(body, &dsrdi)
		return dsrdi, err
	}
}
func unmarshalBasicDeployedServiceReplicaDetailInfoArray(body []byte) ([]BasicDeployedServiceReplicaDetailInfo, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	dsrdiArray := make([]BasicDeployedServiceReplicaDetailInfo, len(rawMessages))

	for index, rawMessage := range rawMessages {
		dsrdi, err := unmarshalBasicDeployedServiceReplicaDetailInfo(*rawMessage)
		if err != nil {
			return nil, err
		}
		dsrdiArray[index] = dsrdi
	}
	return dsrdiArray, nil
}

// MarshalJSON is the custom marshaler for DeployedServiceReplicaDetailInfo.
func (dsrdi DeployedServiceReplicaDetailInfo) MarshalJSON() ([]byte, error) {
	dsrdi.ServiceKind = ServiceKindDeployedServiceReplicaDetailInfo
	objectMap := make(map[string]interface{})
	if dsrdi.ServiceName != nil {
		objectMap["ServiceName"] = dsrdi.ServiceName
	}
	if dsrdi.PartitionID != nil {
		objectMap["PartitionId"] = dsrdi.PartitionID
	}
	if dsrdi.CurrentServiceOperation != "" {
		objectMap["CurrentServiceOperation"] = dsrdi.CurrentServiceOperation
	}
	if dsrdi.CurrentServiceOperationStartTimeUtc != nil {
		objectMap["CurrentServiceOperationStartTimeUtc"] = dsrdi.CurrentServiceOperationStartTimeUtc
	}
	if dsrdi.ReportedLoad != nil {
		objectMap["ReportedLoad"] = dsrdi.ReportedLoad
	}
	if dsrdi.ServiceKind != "" {
		objectMap["ServiceKind"] = dsrdi.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsDeployedStatefulServiceReplicaDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedServiceReplicaDetailInfo.
func (dsrdi DeployedServiceReplicaDetailInfo) AsDeployedStatefulServiceReplicaDetailInfo() (*DeployedStatefulServiceReplicaDetailInfo, bool) {
	return nil, false
}

// AsDeployedStatelessServiceInstanceDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedServiceReplicaDetailInfo.
func (dsrdi DeployedServiceReplicaDetailInfo) AsDeployedStatelessServiceInstanceDetailInfo() (*DeployedStatelessServiceInstanceDetailInfo, bool) {
	return nil, false
}

// AsDeployedServiceReplicaDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedServiceReplicaDetailInfo.
func (dsrdi DeployedServiceReplicaDetailInfo) AsDeployedServiceReplicaDetailInfo() (*DeployedServiceReplicaDetailInfo, bool) {
	return &dsrdi, true
}

// AsBasicDeployedServiceReplicaDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedServiceReplicaDetailInfo.
func (dsrdi DeployedServiceReplicaDetailInfo) AsBasicDeployedServiceReplicaDetailInfo() (BasicDeployedServiceReplicaDetailInfo, bool) {
	return &dsrdi, true
}

// DeployedServiceReplicaDetailInfoModel ...
type DeployedServiceReplicaDetailInfoModel struct {
	autorest.Response `json:"-"`
	Value             BasicDeployedServiceReplicaDetailInfo `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for DeployedServiceReplicaDetailInfoModel struct.
func (dsrdim *DeployedServiceReplicaDetailInfoModel) UnmarshalJSON(body []byte) error {
	dsrdi, err := unmarshalBasicDeployedServiceReplicaDetailInfo(body)
	if err != nil {
		return err
	}
	dsrdim.Value = dsrdi

	return nil
}

// BasicDeployedServiceReplicaInfo information about a Service Fabric service replica deployed on a node.
type BasicDeployedServiceReplicaInfo interface {
	AsDeployedStatefulServiceReplicaInfo() (*DeployedStatefulServiceReplicaInfo, bool)
	AsDeployedStatelessServiceInstanceInfo() (*DeployedStatelessServiceInstanceInfo, bool)
	AsDeployedServiceReplicaInfo() (*DeployedServiceReplicaInfo, bool)
}

// DeployedServiceReplicaInfo information about a Service Fabric service replica deployed on a node.
type DeployedServiceReplicaInfo struct {
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// ServiceManifestName - The name of the service manifest in which this service type is defined.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// CodePackageName - The name of the code package that hosts this replica.
	CodePackageName *string `json:"CodePackageName,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaStatus - The status of a replica of a service. Possible values include: 'ReplicaStatusInvalid', 'ReplicaStatusInBuild', 'ReplicaStatusStandby', 'ReplicaStatusReady', 'ReplicaStatusDown', 'ReplicaStatusDropped'
	ReplicaStatus ReplicaStatus `json:"ReplicaStatus,omitempty"`
	// Address - The last address returned by the replica in Open or ChangeRole.
	Address *string `json:"Address,omitempty"`
	// ServicePackageActivationID - The ActivationId of a deployed service package. If ServicePackageActivationMode specified at the time of creating the service
	// is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'), then value of ServicePackageActivationId
	// is always an empty string.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// HostProcessID - Host process ID of the process that is hosting the replica. This will be zero if the replica is down. In hyper-v containers this host process ID will be from different kernel.
	HostProcessID *string `json:"HostProcessId,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicDeployedServiceReplicaInfoServiceKindDeployedServiceReplicaInfo', 'ServiceKindBasicDeployedServiceReplicaInfoServiceKindStateful', 'ServiceKindBasicDeployedServiceReplicaInfoServiceKindStateless'
	ServiceKind ServiceKindBasicDeployedServiceReplicaInfo `json:"ServiceKind,omitempty"`
}

func unmarshalBasicDeployedServiceReplicaInfo(body []byte) (BasicDeployedServiceReplicaInfo, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ServiceKind"] {
	case string(ServiceKindBasicDeployedServiceReplicaInfoServiceKindStateful):
		var dssri DeployedStatefulServiceReplicaInfo
		err := json.Unmarshal(body, &dssri)
		return dssri, err
	case string(ServiceKindBasicDeployedServiceReplicaInfoServiceKindStateless):
		var dssii DeployedStatelessServiceInstanceInfo
		err := json.Unmarshal(body, &dssii)
		return dssii, err
	default:
		var dsri DeployedServiceReplicaInfo
		err := json.Unmarshal(body, &dsri)
		return dsri, err
	}
}
func unmarshalBasicDeployedServiceReplicaInfoArray(body []byte) ([]BasicDeployedServiceReplicaInfo, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	dsriArray := make([]BasicDeployedServiceReplicaInfo, len(rawMessages))

	for index, rawMessage := range rawMessages {
		dsri, err := unmarshalBasicDeployedServiceReplicaInfo(*rawMessage)
		if err != nil {
			return nil, err
		}
		dsriArray[index] = dsri
	}
	return dsriArray, nil
}

// MarshalJSON is the custom marshaler for DeployedServiceReplicaInfo.
func (dsri DeployedServiceReplicaInfo) MarshalJSON() ([]byte, error) {
	dsri.ServiceKind = ServiceKindBasicDeployedServiceReplicaInfoServiceKindDeployedServiceReplicaInfo
	objectMap := make(map[string]interface{})
	if dsri.ServiceName != nil {
		objectMap["ServiceName"] = dsri.ServiceName
	}
	if dsri.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = dsri.ServiceTypeName
	}
	if dsri.ServiceManifestName != nil {
		objectMap["ServiceManifestName"] = dsri.ServiceManifestName
	}
	if dsri.CodePackageName != nil {
		objectMap["CodePackageName"] = dsri.CodePackageName
	}
	if dsri.PartitionID != nil {
		objectMap["PartitionId"] = dsri.PartitionID
	}
	if dsri.ReplicaStatus != "" {
		objectMap["ReplicaStatus"] = dsri.ReplicaStatus
	}
	if dsri.Address != nil {
		objectMap["Address"] = dsri.Address
	}
	if dsri.ServicePackageActivationID != nil {
		objectMap["ServicePackageActivationId"] = dsri.ServicePackageActivationID
	}
	if dsri.HostProcessID != nil {
		objectMap["HostProcessId"] = dsri.HostProcessID
	}
	if dsri.ServiceKind != "" {
		objectMap["ServiceKind"] = dsri.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsDeployedStatefulServiceReplicaInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedServiceReplicaInfo.
func (dsri DeployedServiceReplicaInfo) AsDeployedStatefulServiceReplicaInfo() (*DeployedStatefulServiceReplicaInfo, bool) {
	return nil, false
}

// AsDeployedStatelessServiceInstanceInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedServiceReplicaInfo.
func (dsri DeployedServiceReplicaInfo) AsDeployedStatelessServiceInstanceInfo() (*DeployedStatelessServiceInstanceInfo, bool) {
	return nil, false
}

// AsDeployedServiceReplicaInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedServiceReplicaInfo.
func (dsri DeployedServiceReplicaInfo) AsDeployedServiceReplicaInfo() (*DeployedServiceReplicaInfo, bool) {
	return &dsri, true
}

// AsBasicDeployedServiceReplicaInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedServiceReplicaInfo.
func (dsri DeployedServiceReplicaInfo) AsBasicDeployedServiceReplicaInfo() (BasicDeployedServiceReplicaInfo, bool) {
	return &dsri, true
}

// DeployedServiceTypeInfo information about service type deployed on a node, information such as the
// status of the service type registration on a node.
type DeployedServiceTypeInfo struct {
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// ServiceManifestName - The name of the service manifest in which this service type is defined.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// CodePackageName - The name of the code package that registered the service type.
	CodePackageName *string `json:"CodePackageName,omitempty"`
	// Status - The status of the service type registration on the node. Possible values include: 'ServiceTypeRegistrationStatusInvalid', 'ServiceTypeRegistrationStatusDisabled', 'ServiceTypeRegistrationStatusEnabled', 'ServiceTypeRegistrationStatusRegistered'
	Status ServiceTypeRegistrationStatus `json:"Status,omitempty"`
	// ServicePackageActivationID - The ActivationId of a deployed service package. If ServicePackageActivationMode specified at the time of creating the service
	// is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'), then value of ServicePackageActivationId
	// is always an empty string.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
}

// DeployedStatefulServiceReplicaDetailInfo information about a stateful replica running in a code package.
// Note DeployedServiceReplicaQueryResult will contain duplicate data like ServiceKind, ServiceName,
// PartitionId and replicaId.
type DeployedStatefulServiceReplicaDetailInfo struct {
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *string `json:"ReplicaId,omitempty"`
	// CurrentReplicatorOperation - Specifies the operation currently being executed by the Replicator. Possible values include: 'ReplicatorOperationNameInvalid', 'ReplicatorOperationNameNone', 'ReplicatorOperationNameOpen', 'ReplicatorOperationNameChangeRole', 'ReplicatorOperationNameUpdateEpoch', 'ReplicatorOperationNameClose', 'ReplicatorOperationNameAbort', 'ReplicatorOperationNameOnDataLoss', 'ReplicatorOperationNameWaitForCatchup', 'ReplicatorOperationNameBuild'
	CurrentReplicatorOperation ReplicatorOperationName `json:"CurrentReplicatorOperation,omitempty"`
	// ReadStatus - Specifies the access status of the partition. Possible values include: 'PartitionAccessStatusInvalid', 'PartitionAccessStatusGranted', 'PartitionAccessStatusReconfigurationPending', 'PartitionAccessStatusNotPrimary', 'PartitionAccessStatusNoWriteQuorum'
	ReadStatus PartitionAccessStatus `json:"ReadStatus,omitempty"`
	// WriteStatus - Specifies the access status of the partition. Possible values include: 'PartitionAccessStatusInvalid', 'PartitionAccessStatusGranted', 'PartitionAccessStatusReconfigurationPending', 'PartitionAccessStatusNotPrimary', 'PartitionAccessStatusNoWriteQuorum'
	WriteStatus PartitionAccessStatus `json:"WriteStatus,omitempty"`
	// ReplicatorStatus - Represents a base class for primary or secondary replicator status.
	// Contains information about the service fabric replicator like the replication/copy queue utilization, last acknowledgement received timestamp, etc.
	ReplicatorStatus BasicReplicatorStatus `json:"ReplicatorStatus,omitempty"`
	// ReplicaStatus - Key value store related information for the replica.
	ReplicaStatus *KeyValueStoreReplicaStatus `json:"ReplicaStatus,omitempty"`
	// DeployedServiceReplicaQueryResult - Information about a stateful service replica deployed on a node.
	DeployedServiceReplicaQueryResult *DeployedStatefulServiceReplicaInfo `json:"DeployedServiceReplicaQueryResult,omitempty"`
	// ServiceName - Full hierarchical name of the service in URI format starting with `fabric:`.
	ServiceName *string `json:"ServiceName,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// CurrentServiceOperation - Specifies the current active life-cycle operation on a stateful service replica or stateless service instance. Possible values include: 'ServiceOperationNameUnknown', 'ServiceOperationNameNone', 'ServiceOperationNameOpen', 'ServiceOperationNameChangeRole', 'ServiceOperationNameClose', 'ServiceOperationNameAbort'
	CurrentServiceOperation ServiceOperationName `json:"CurrentServiceOperation,omitempty"`
	// CurrentServiceOperationStartTimeUtc - The start time of the current service operation in UTC format.
	CurrentServiceOperationStartTimeUtc *date.Time `json:"CurrentServiceOperationStartTimeUtc,omitempty"`
	// ReportedLoad - List of load reported by replica.
	ReportedLoad *[]LoadMetricReportInfo `json:"ReportedLoad,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindDeployedServiceReplicaDetailInfo', 'ServiceKindStateful1', 'ServiceKindStateless1'
	ServiceKind ServiceKindBasicDeployedServiceReplicaDetailInfo `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedStatefulServiceReplicaDetailInfo.
func (dssrdi DeployedStatefulServiceReplicaDetailInfo) MarshalJSON() ([]byte, error) {
	dssrdi.ServiceKind = ServiceKindStateful1
	objectMap := make(map[string]interface{})
	if dssrdi.ReplicaID != nil {
		objectMap["ReplicaId"] = dssrdi.ReplicaID
	}
	if dssrdi.CurrentReplicatorOperation != "" {
		objectMap["CurrentReplicatorOperation"] = dssrdi.CurrentReplicatorOperation
	}
	if dssrdi.ReadStatus != "" {
		objectMap["ReadStatus"] = dssrdi.ReadStatus
	}
	if dssrdi.WriteStatus != "" {
		objectMap["WriteStatus"] = dssrdi.WriteStatus
	}
	objectMap["ReplicatorStatus"] = dssrdi.ReplicatorStatus
	if dssrdi.ReplicaStatus != nil {
		objectMap["ReplicaStatus"] = dssrdi.ReplicaStatus
	}
	if dssrdi.DeployedServiceReplicaQueryResult != nil {
		objectMap["DeployedServiceReplicaQueryResult"] = dssrdi.DeployedServiceReplicaQueryResult
	}
	if dssrdi.ServiceName != nil {
		objectMap["ServiceName"] = dssrdi.ServiceName
	}
	if dssrdi.PartitionID != nil {
		objectMap["PartitionId"] = dssrdi.PartitionID
	}
	if dssrdi.CurrentServiceOperation != "" {
		objectMap["CurrentServiceOperation"] = dssrdi.CurrentServiceOperation
	}
	if dssrdi.CurrentServiceOperationStartTimeUtc != nil {
		objectMap["CurrentServiceOperationStartTimeUtc"] = dssrdi.CurrentServiceOperationStartTimeUtc
	}
	if dssrdi.ReportedLoad != nil {
		objectMap["ReportedLoad"] = dssrdi.ReportedLoad
	}
	if dssrdi.ServiceKind != "" {
		objectMap["ServiceKind"] = dssrdi.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsDeployedStatefulServiceReplicaDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedStatefulServiceReplicaDetailInfo.
func (dssrdi DeployedStatefulServiceReplicaDetailInfo) AsDeployedStatefulServiceReplicaDetailInfo() (*DeployedStatefulServiceReplicaDetailInfo, bool) {
	return &dssrdi, true
}

// AsDeployedStatelessServiceInstanceDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedStatefulServiceReplicaDetailInfo.
func (dssrdi DeployedStatefulServiceReplicaDetailInfo) AsDeployedStatelessServiceInstanceDetailInfo() (*DeployedStatelessServiceInstanceDetailInfo, bool) {
	return nil, false
}

// AsDeployedServiceReplicaDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedStatefulServiceReplicaDetailInfo.
func (dssrdi DeployedStatefulServiceReplicaDetailInfo) AsDeployedServiceReplicaDetailInfo() (*DeployedServiceReplicaDetailInfo, bool) {
	return nil, false
}

// AsBasicDeployedServiceReplicaDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedStatefulServiceReplicaDetailInfo.
func (dssrdi DeployedStatefulServiceReplicaDetailInfo) AsBasicDeployedServiceReplicaDetailInfo() (BasicDeployedServiceReplicaDetailInfo, bool) {
	return &dssrdi, true
}

// UnmarshalJSON is the custom unmarshaler for DeployedStatefulServiceReplicaDetailInfo struct.
func (dssrdi *DeployedStatefulServiceReplicaDetailInfo) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "ReplicaId":
			if v != nil {
				var replicaID string
				err = json.Unmarshal(*v, &replicaID)
				if err != nil {
					return err
				}
				dssrdi.ReplicaID = &replicaID
			}
		case "CurrentReplicatorOperation":
			if v != nil {
				var currentReplicatorOperation ReplicatorOperationName
				err = json.Unmarshal(*v, &currentReplicatorOperation)
				if err != nil {
					return err
				}
				dssrdi.CurrentReplicatorOperation = currentReplicatorOperation
			}
		case "ReadStatus":
			if v != nil {
				var readStatus PartitionAccessStatus
				err = json.Unmarshal(*v, &readStatus)
				if err != nil {
					return err
				}
				dssrdi.ReadStatus = readStatus
			}
		case "WriteStatus":
			if v != nil {
				var writeStatus PartitionAccessStatus
				err = json.Unmarshal(*v, &writeStatus)
				if err != nil {
					return err
				}
				dssrdi.WriteStatus = writeStatus
			}
		case "ReplicatorStatus":
			if v != nil {
				replicatorStatus, err := unmarshalBasicReplicatorStatus(*v)
				if err != nil {
					return err
				}
				dssrdi.ReplicatorStatus = replicatorStatus
			}
		case "ReplicaStatus":
			if v != nil {
				var replicaStatus KeyValueStoreReplicaStatus
				err = json.Unmarshal(*v, &replicaStatus)
				if err != nil {
					return err
				}
				dssrdi.ReplicaStatus = &replicaStatus
			}
		case "DeployedServiceReplicaQueryResult":
			if v != nil {
				var deployedServiceReplicaQueryResult DeployedStatefulServiceReplicaInfo
				err = json.Unmarshal(*v, &deployedServiceReplicaQueryResult)
				if err != nil {
					return err
				}
				dssrdi.DeployedServiceReplicaQueryResult = &deployedServiceReplicaQueryResult
			}
		case "ServiceName":
			if v != nil {
				var serviceName string
				err = json.Unmarshal(*v, &serviceName)
				if err != nil {
					return err
				}
				dssrdi.ServiceName = &serviceName
			}
		case "PartitionId":
			if v != nil {
				var partitionID uuid.UUID
				err = json.Unmarshal(*v, &partitionID)
				if err != nil {
					return err
				}
				dssrdi.PartitionID = &partitionID
			}
		case "CurrentServiceOperation":
			if v != nil {
				var currentServiceOperation ServiceOperationName
				err = json.Unmarshal(*v, &currentServiceOperation)
				if err != nil {
					return err
				}
				dssrdi.CurrentServiceOperation = currentServiceOperation
			}
		case "CurrentServiceOperationStartTimeUtc":
			if v != nil {
				var currentServiceOperationStartTimeUtc date.Time
				err = json.Unmarshal(*v, &currentServiceOperationStartTimeUtc)
				if err != nil {
					return err
				}
				dssrdi.CurrentServiceOperationStartTimeUtc = &currentServiceOperationStartTimeUtc
			}
		case "ReportedLoad":
			if v != nil {
				var reportedLoad []LoadMetricReportInfo
				err = json.Unmarshal(*v, &reportedLoad)
				if err != nil {
					return err
				}
				dssrdi.ReportedLoad = &reportedLoad
			}
		case "ServiceKind":
			if v != nil {
				var serviceKind ServiceKindBasicDeployedServiceReplicaDetailInfo
				err = json.Unmarshal(*v, &serviceKind)
				if err != nil {
					return err
				}
				dssrdi.ServiceKind = serviceKind
			}
		}
	}

	return nil
}

// DeployedStatefulServiceReplicaInfo information about a stateful service replica deployed on a node.
type DeployedStatefulServiceReplicaInfo struct {
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *string `json:"ReplicaId,omitempty"`
	// ReplicaRole - The role of a replica of a stateful service. Possible values include: 'ReplicaRoleUnknown', 'ReplicaRoleNone', 'ReplicaRolePrimary', 'ReplicaRoleIdleSecondary', 'ReplicaRoleActiveSecondary'
	ReplicaRole ReplicaRole `json:"ReplicaRole,omitempty"`
	// ReconfigurationInformation - Information about current reconfiguration like phase, type, previous configuration role of replica and reconfiguration start date time.
	ReconfigurationInformation *ReconfigurationInformation `json:"ReconfigurationInformation,omitempty"`
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// ServiceManifestName - The name of the service manifest in which this service type is defined.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// CodePackageName - The name of the code package that hosts this replica.
	CodePackageName *string `json:"CodePackageName,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaStatus - The status of a replica of a service. Possible values include: 'ReplicaStatusInvalid', 'ReplicaStatusInBuild', 'ReplicaStatusStandby', 'ReplicaStatusReady', 'ReplicaStatusDown', 'ReplicaStatusDropped'
	ReplicaStatus ReplicaStatus `json:"ReplicaStatus,omitempty"`
	// Address - The last address returned by the replica in Open or ChangeRole.
	Address *string `json:"Address,omitempty"`
	// ServicePackageActivationID - The ActivationId of a deployed service package. If ServicePackageActivationMode specified at the time of creating the service
	// is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'), then value of ServicePackageActivationId
	// is always an empty string.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// HostProcessID - Host process ID of the process that is hosting the replica. This will be zero if the replica is down. In hyper-v containers this host process ID will be from different kernel.
	HostProcessID *string `json:"HostProcessId,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicDeployedServiceReplicaInfoServiceKindDeployedServiceReplicaInfo', 'ServiceKindBasicDeployedServiceReplicaInfoServiceKindStateful', 'ServiceKindBasicDeployedServiceReplicaInfoServiceKindStateless'
	ServiceKind ServiceKindBasicDeployedServiceReplicaInfo `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedStatefulServiceReplicaInfo.
func (dssri DeployedStatefulServiceReplicaInfo) MarshalJSON() ([]byte, error) {
	dssri.ServiceKind = ServiceKindBasicDeployedServiceReplicaInfoServiceKindStateful
	objectMap := make(map[string]interface{})
	if dssri.ReplicaID != nil {
		objectMap["ReplicaId"] = dssri.ReplicaID
	}
	if dssri.ReplicaRole != "" {
		objectMap["ReplicaRole"] = dssri.ReplicaRole
	}
	if dssri.ReconfigurationInformation != nil {
		objectMap["ReconfigurationInformation"] = dssri.ReconfigurationInformation
	}
	if dssri.ServiceName != nil {
		objectMap["ServiceName"] = dssri.ServiceName
	}
	if dssri.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = dssri.ServiceTypeName
	}
	if dssri.ServiceManifestName != nil {
		objectMap["ServiceManifestName"] = dssri.ServiceManifestName
	}
	if dssri.CodePackageName != nil {
		objectMap["CodePackageName"] = dssri.CodePackageName
	}
	if dssri.PartitionID != nil {
		objectMap["PartitionId"] = dssri.PartitionID
	}
	if dssri.ReplicaStatus != "" {
		objectMap["ReplicaStatus"] = dssri.ReplicaStatus
	}
	if dssri.Address != nil {
		objectMap["Address"] = dssri.Address
	}
	if dssri.ServicePackageActivationID != nil {
		objectMap["ServicePackageActivationId"] = dssri.ServicePackageActivationID
	}
	if dssri.HostProcessID != nil {
		objectMap["HostProcessId"] = dssri.HostProcessID
	}
	if dssri.ServiceKind != "" {
		objectMap["ServiceKind"] = dssri.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsDeployedStatefulServiceReplicaInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedStatefulServiceReplicaInfo.
func (dssri DeployedStatefulServiceReplicaInfo) AsDeployedStatefulServiceReplicaInfo() (*DeployedStatefulServiceReplicaInfo, bool) {
	return &dssri, true
}

// AsDeployedStatelessServiceInstanceInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedStatefulServiceReplicaInfo.
func (dssri DeployedStatefulServiceReplicaInfo) AsDeployedStatelessServiceInstanceInfo() (*DeployedStatelessServiceInstanceInfo, bool) {
	return nil, false
}

// AsDeployedServiceReplicaInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedStatefulServiceReplicaInfo.
func (dssri DeployedStatefulServiceReplicaInfo) AsDeployedServiceReplicaInfo() (*DeployedServiceReplicaInfo, bool) {
	return nil, false
}

// AsBasicDeployedServiceReplicaInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedStatefulServiceReplicaInfo.
func (dssri DeployedStatefulServiceReplicaInfo) AsBasicDeployedServiceReplicaInfo() (BasicDeployedServiceReplicaInfo, bool) {
	return &dssri, true
}

// DeployedStatelessServiceInstanceDetailInfo information about a stateless instance running in a code
// package. Note that DeployedServiceReplicaQueryResult will contain duplicate data like ServiceKind,
// ServiceName, PartitionId and InstanceId.
type DeployedStatelessServiceInstanceDetailInfo struct {
	// InstanceID - Id of a stateless service instance. InstanceId is used by Service Fabric to uniquely identify an instance of a partition of a stateless service. It is unique within a partition and does not change for the lifetime of the instance. If the instance has failed over on the same or different node, it will get a different value for the InstanceId.
	InstanceID *string `json:"InstanceId,omitempty"`
	// DeployedServiceReplicaQueryResult - Information about a stateless service instance deployed on a node.
	DeployedServiceReplicaQueryResult *DeployedStatelessServiceInstanceInfo `json:"DeployedServiceReplicaQueryResult,omitempty"`
	// ServiceName - Full hierarchical name of the service in URI format starting with `fabric:`.
	ServiceName *string `json:"ServiceName,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// CurrentServiceOperation - Specifies the current active life-cycle operation on a stateful service replica or stateless service instance. Possible values include: 'ServiceOperationNameUnknown', 'ServiceOperationNameNone', 'ServiceOperationNameOpen', 'ServiceOperationNameChangeRole', 'ServiceOperationNameClose', 'ServiceOperationNameAbort'
	CurrentServiceOperation ServiceOperationName `json:"CurrentServiceOperation,omitempty"`
	// CurrentServiceOperationStartTimeUtc - The start time of the current service operation in UTC format.
	CurrentServiceOperationStartTimeUtc *date.Time `json:"CurrentServiceOperationStartTimeUtc,omitempty"`
	// ReportedLoad - List of load reported by replica.
	ReportedLoad *[]LoadMetricReportInfo `json:"ReportedLoad,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindDeployedServiceReplicaDetailInfo', 'ServiceKindStateful1', 'ServiceKindStateless1'
	ServiceKind ServiceKindBasicDeployedServiceReplicaDetailInfo `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedStatelessServiceInstanceDetailInfo.
func (dssidi DeployedStatelessServiceInstanceDetailInfo) MarshalJSON() ([]byte, error) {
	dssidi.ServiceKind = ServiceKindStateless1
	objectMap := make(map[string]interface{})
	if dssidi.InstanceID != nil {
		objectMap["InstanceId"] = dssidi.InstanceID
	}
	if dssidi.DeployedServiceReplicaQueryResult != nil {
		objectMap["DeployedServiceReplicaQueryResult"] = dssidi.DeployedServiceReplicaQueryResult
	}
	if dssidi.ServiceName != nil {
		objectMap["ServiceName"] = dssidi.ServiceName
	}
	if dssidi.PartitionID != nil {
		objectMap["PartitionId"] = dssidi.PartitionID
	}
	if dssidi.CurrentServiceOperation != "" {
		objectMap["CurrentServiceOperation"] = dssidi.CurrentServiceOperation
	}
	if dssidi.CurrentServiceOperationStartTimeUtc != nil {
		objectMap["CurrentServiceOperationStartTimeUtc"] = dssidi.CurrentServiceOperationStartTimeUtc
	}
	if dssidi.ReportedLoad != nil {
		objectMap["ReportedLoad"] = dssidi.ReportedLoad
	}
	if dssidi.ServiceKind != "" {
		objectMap["ServiceKind"] = dssidi.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsDeployedStatefulServiceReplicaDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedStatelessServiceInstanceDetailInfo.
func (dssidi DeployedStatelessServiceInstanceDetailInfo) AsDeployedStatefulServiceReplicaDetailInfo() (*DeployedStatefulServiceReplicaDetailInfo, bool) {
	return nil, false
}

// AsDeployedStatelessServiceInstanceDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedStatelessServiceInstanceDetailInfo.
func (dssidi DeployedStatelessServiceInstanceDetailInfo) AsDeployedStatelessServiceInstanceDetailInfo() (*DeployedStatelessServiceInstanceDetailInfo, bool) {
	return &dssidi, true
}

// AsDeployedServiceReplicaDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedStatelessServiceInstanceDetailInfo.
func (dssidi DeployedStatelessServiceInstanceDetailInfo) AsDeployedServiceReplicaDetailInfo() (*DeployedServiceReplicaDetailInfo, bool) {
	return nil, false
}

// AsBasicDeployedServiceReplicaDetailInfo is the BasicDeployedServiceReplicaDetailInfo implementation for DeployedStatelessServiceInstanceDetailInfo.
func (dssidi DeployedStatelessServiceInstanceDetailInfo) AsBasicDeployedServiceReplicaDetailInfo() (BasicDeployedServiceReplicaDetailInfo, bool) {
	return &dssidi, true
}

// DeployedStatelessServiceInstanceInfo information about a stateless service instance deployed on a node.
type DeployedStatelessServiceInstanceInfo struct {
	// InstanceID - Id of a stateless service instance. InstanceId is used by Service Fabric to uniquely identify an instance of a partition of a stateless service. It is unique within a partition and does not change for the lifetime of the instance. If the instance has failed over on the same or different node, it will get a different value for the InstanceId.
	InstanceID *string `json:"InstanceId,omitempty"`
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// ServiceManifestName - The name of the service manifest in which this service type is defined.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// CodePackageName - The name of the code package that hosts this replica.
	CodePackageName *string `json:"CodePackageName,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaStatus - The status of a replica of a service. Possible values include: 'ReplicaStatusInvalid', 'ReplicaStatusInBuild', 'ReplicaStatusStandby', 'ReplicaStatusReady', 'ReplicaStatusDown', 'ReplicaStatusDropped'
	ReplicaStatus ReplicaStatus `json:"ReplicaStatus,omitempty"`
	// Address - The last address returned by the replica in Open or ChangeRole.
	Address *string `json:"Address,omitempty"`
	// ServicePackageActivationID - The ActivationId of a deployed service package. If ServicePackageActivationMode specified at the time of creating the service
	// is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'), then value of ServicePackageActivationId
	// is always an empty string.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// HostProcessID - Host process ID of the process that is hosting the replica. This will be zero if the replica is down. In hyper-v containers this host process ID will be from different kernel.
	HostProcessID *string `json:"HostProcessId,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicDeployedServiceReplicaInfoServiceKindDeployedServiceReplicaInfo', 'ServiceKindBasicDeployedServiceReplicaInfoServiceKindStateful', 'ServiceKindBasicDeployedServiceReplicaInfoServiceKindStateless'
	ServiceKind ServiceKindBasicDeployedServiceReplicaInfo `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for DeployedStatelessServiceInstanceInfo.
func (dssii DeployedStatelessServiceInstanceInfo) MarshalJSON() ([]byte, error) {
	dssii.ServiceKind = ServiceKindBasicDeployedServiceReplicaInfoServiceKindStateless
	objectMap := make(map[string]interface{})
	if dssii.InstanceID != nil {
		objectMap["InstanceId"] = dssii.InstanceID
	}
	if dssii.ServiceName != nil {
		objectMap["ServiceName"] = dssii.ServiceName
	}
	if dssii.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = dssii.ServiceTypeName
	}
	if dssii.ServiceManifestName != nil {
		objectMap["ServiceManifestName"] = dssii.ServiceManifestName
	}
	if dssii.CodePackageName != nil {
		objectMap["CodePackageName"] = dssii.CodePackageName
	}
	if dssii.PartitionID != nil {
		objectMap["PartitionId"] = dssii.PartitionID
	}
	if dssii.ReplicaStatus != "" {
		objectMap["ReplicaStatus"] = dssii.ReplicaStatus
	}
	if dssii.Address != nil {
		objectMap["Address"] = dssii.Address
	}
	if dssii.ServicePackageActivationID != nil {
		objectMap["ServicePackageActivationId"] = dssii.ServicePackageActivationID
	}
	if dssii.HostProcessID != nil {
		objectMap["HostProcessId"] = dssii.HostProcessID
	}
	if dssii.ServiceKind != "" {
		objectMap["ServiceKind"] = dssii.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsDeployedStatefulServiceReplicaInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedStatelessServiceInstanceInfo.
func (dssii DeployedStatelessServiceInstanceInfo) AsDeployedStatefulServiceReplicaInfo() (*DeployedStatefulServiceReplicaInfo, bool) {
	return nil, false
}

// AsDeployedStatelessServiceInstanceInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedStatelessServiceInstanceInfo.
func (dssii DeployedStatelessServiceInstanceInfo) AsDeployedStatelessServiceInstanceInfo() (*DeployedStatelessServiceInstanceInfo, bool) {
	return &dssii, true
}

// AsDeployedServiceReplicaInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedStatelessServiceInstanceInfo.
func (dssii DeployedStatelessServiceInstanceInfo) AsDeployedServiceReplicaInfo() (*DeployedServiceReplicaInfo, bool) {
	return nil, false
}

// AsBasicDeployedServiceReplicaInfo is the BasicDeployedServiceReplicaInfo implementation for DeployedStatelessServiceInstanceInfo.
func (dssii DeployedStatelessServiceInstanceInfo) AsBasicDeployedServiceReplicaInfo() (BasicDeployedServiceReplicaInfo, bool) {
	return &dssii, true
}

// DeployServicePackageToNodeDescription defines description for downloading packages associated with a
// service manifest to image cache on a Service Fabric node.
type DeployServicePackageToNodeDescription struct {
	// ServiceManifestName - The name of service manifest whose packages need to be downloaded.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// ApplicationTypeName - The application type name as defined in the application manifest.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// ApplicationTypeVersion - The version of the application type as defined in the application manifest.
	ApplicationTypeVersion *string `json:"ApplicationTypeVersion,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// PackageSharingPolicy - List of package sharing policy information.
	PackageSharingPolicy *[]PackageSharingPolicyInfo `json:"PackageSharingPolicy,omitempty"`
}

// DiagnosticsDescription describes the diagnostics options available
type DiagnosticsDescription struct {
	// Sinks - List of supported sinks that can be referenced.
	Sinks *[]BasicDiagnosticsSinkProperties `json:"sinks,omitempty"`
	// Enabled - Status of whether or not sinks are enabled.
	Enabled *bool `json:"enabled,omitempty"`
	// DefaultSinkRefs - The sinks to be used if diagnostics is enabled. Sink choices can be overridden at the service and code package level.
	DefaultSinkRefs *[]string `json:"defaultSinkRefs,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for DiagnosticsDescription struct.
func (dd *DiagnosticsDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "sinks":
			if v != nil {
				sinks, err := unmarshalBasicDiagnosticsSinkPropertiesArray(*v)
				if err != nil {
					return err
				}
				dd.Sinks = &sinks
			}
		case "enabled":
			if v != nil {
				var enabled bool
				err = json.Unmarshal(*v, &enabled)
				if err != nil {
					return err
				}
				dd.Enabled = &enabled
			}
		case "defaultSinkRefs":
			if v != nil {
				var defaultSinkRefs []string
				err = json.Unmarshal(*v, &defaultSinkRefs)
				if err != nil {
					return err
				}
				dd.DefaultSinkRefs = &defaultSinkRefs
			}
		}
	}

	return nil
}

// DiagnosticsRef reference to sinks in DiagnosticsDescription.
type DiagnosticsRef struct {
	// Enabled - Status of whether or not sinks are enabled.
	Enabled *bool `json:"enabled,omitempty"`
	// SinkRefs - List of sinks to be used if enabled. References the list of sinks in DiagnosticsDescription.
	SinkRefs *[]string `json:"sinkRefs,omitempty"`
}

// BasicDiagnosticsSinkProperties properties of a DiagnosticsSink.
type BasicDiagnosticsSinkProperties interface {
	AsAzureInternalMonitoringPipelineSinkDescription() (*AzureInternalMonitoringPipelineSinkDescription, bool)
	AsDiagnosticsSinkProperties() (*DiagnosticsSinkProperties, bool)
}

// DiagnosticsSinkProperties properties of a DiagnosticsSink.
type DiagnosticsSinkProperties struct {
	// Name - Name of the sink. This value is referenced by DiagnosticsReferenceDescription
	Name *string `json:"name,omitempty"`
	// Description - A description of the sink.
	Description *string `json:"description,omitempty"`
	// Kind - Possible values include: 'KindDiagnosticsSinkProperties', 'KindAzureInternalMonitoringPipeline'
	Kind KindBasicDiagnosticsSinkProperties `json:"kind,omitempty"`
}

func unmarshalBasicDiagnosticsSinkProperties(body []byte) (BasicDiagnosticsSinkProperties, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["kind"] {
	case string(KindAzureInternalMonitoringPipeline):
		var aimpsd AzureInternalMonitoringPipelineSinkDescription
		err := json.Unmarshal(body, &aimpsd)
		return aimpsd, err
	default:
		var dsp DiagnosticsSinkProperties
		err := json.Unmarshal(body, &dsp)
		return dsp, err
	}
}
func unmarshalBasicDiagnosticsSinkPropertiesArray(body []byte) ([]BasicDiagnosticsSinkProperties, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	dspArray := make([]BasicDiagnosticsSinkProperties, len(rawMessages))

	for index, rawMessage := range rawMessages {
		dsp, err := unmarshalBasicDiagnosticsSinkProperties(*rawMessage)
		if err != nil {
			return nil, err
		}
		dspArray[index] = dsp
	}
	return dspArray, nil
}

// MarshalJSON is the custom marshaler for DiagnosticsSinkProperties.
func (dsp DiagnosticsSinkProperties) MarshalJSON() ([]byte, error) {
	dsp.Kind = KindDiagnosticsSinkProperties
	objectMap := make(map[string]interface{})
	if dsp.Name != nil {
		objectMap["name"] = dsp.Name
	}
	if dsp.Description != nil {
		objectMap["description"] = dsp.Description
	}
	if dsp.Kind != "" {
		objectMap["kind"] = dsp.Kind
	}
	return json.Marshal(objectMap)
}

// AsAzureInternalMonitoringPipelineSinkDescription is the BasicDiagnosticsSinkProperties implementation for DiagnosticsSinkProperties.
func (dsp DiagnosticsSinkProperties) AsAzureInternalMonitoringPipelineSinkDescription() (*AzureInternalMonitoringPipelineSinkDescription, bool) {
	return nil, false
}

// AsDiagnosticsSinkProperties is the BasicDiagnosticsSinkProperties implementation for DiagnosticsSinkProperties.
func (dsp DiagnosticsSinkProperties) AsDiagnosticsSinkProperties() (*DiagnosticsSinkProperties, bool) {
	return &dsp, true
}

// AsBasicDiagnosticsSinkProperties is the BasicDiagnosticsSinkProperties implementation for DiagnosticsSinkProperties.
func (dsp DiagnosticsSinkProperties) AsBasicDiagnosticsSinkProperties() (BasicDiagnosticsSinkProperties, bool) {
	return &dsp, true
}

// DisableBackupDescription it describes the body parameters while disabling backup of a backup
// entity(Application/Service/Partition).
type DisableBackupDescription struct {
	// CleanBackup - Boolean flag to delete backups. It can be set to true for deleting all the backups which were created for the backup entity that is getting disabled for backup.
	CleanBackup *bool `json:"CleanBackup,omitempty"`
}

// DoublePropertyValue describes a Service Fabric property value of type Double.
type DoublePropertyValue struct {
	// Data - The data of the property value.
	Data *float64 `json:"Data,omitempty"`
	// Kind - Possible values include: 'KindPropertyValue', 'KindBinary', 'KindInt64', 'KindDouble', 'KindString', 'KindGUID'
	Kind KindBasicPropertyValue `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for DoublePropertyValue.
func (dpv DoublePropertyValue) MarshalJSON() ([]byte, error) {
	dpv.Kind = KindDouble
	objectMap := make(map[string]interface{})
	if dpv.Data != nil {
		objectMap["Data"] = dpv.Data
	}
	if dpv.Kind != "" {
		objectMap["Kind"] = dpv.Kind
	}
	return json.Marshal(objectMap)
}

// AsBinaryPropertyValue is the BasicPropertyValue implementation for DoublePropertyValue.
func (dpv DoublePropertyValue) AsBinaryPropertyValue() (*BinaryPropertyValue, bool) {
	return nil, false
}

// AsInt64PropertyValue is the BasicPropertyValue implementation for DoublePropertyValue.
func (dpv DoublePropertyValue) AsInt64PropertyValue() (*Int64PropertyValue, bool) {
	return nil, false
}

// AsDoublePropertyValue is the BasicPropertyValue implementation for DoublePropertyValue.
func (dpv DoublePropertyValue) AsDoublePropertyValue() (*DoublePropertyValue, bool) {
	return &dpv, true
}

// AsStringPropertyValue is the BasicPropertyValue implementation for DoublePropertyValue.
func (dpv DoublePropertyValue) AsStringPropertyValue() (*StringPropertyValue, bool) {
	return nil, false
}

// AsGUIDPropertyValue is the BasicPropertyValue implementation for DoublePropertyValue.
func (dpv DoublePropertyValue) AsGUIDPropertyValue() (*GUIDPropertyValue, bool) {
	return nil, false
}

// AsPropertyValue is the BasicPropertyValue implementation for DoublePropertyValue.
func (dpv DoublePropertyValue) AsPropertyValue() (*PropertyValue, bool) {
	return nil, false
}

// AsBasicPropertyValue is the BasicPropertyValue implementation for DoublePropertyValue.
func (dpv DoublePropertyValue) AsBasicPropertyValue() (BasicPropertyValue, bool) {
	return &dpv, true
}

// EnableBackupDescription specifies the parameters needed to enable periodic backup.
type EnableBackupDescription struct {
	// BackupPolicyName - Name of the backup policy to be used for enabling periodic backups.
	BackupPolicyName *string `json:"BackupPolicyName,omitempty"`
}

// EndpointProperties describes a container endpoint.
type EndpointProperties struct {
	// Name - The name of the endpoint.
	Name *string `json:"name,omitempty"`
	// Port - Port used by the container.
	Port *int32 `json:"port,omitempty"`
}

// EndpointRef describes a reference to a service endpoint.
type EndpointRef struct {
	// Name - Name of the endpoint.
	Name *string `json:"name,omitempty"`
}

// EnsureAvailabilitySafetyCheck safety check that waits to ensure the availability of the partition. It
// waits until there are replicas available such that bringing down this replica will not cause
// availability loss for the partition.
type EnsureAvailabilitySafetyCheck struct {
	// PartitionID - Id of the partition which is undergoing the safety check.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// Kind - Possible values include: 'KindSafetyCheck', 'KindPartitionSafetyCheck', 'KindEnsureAvailability', 'KindEnsurePartitionQuorum', 'KindEnsureSeedNodeQuorum', 'KindWaitForInbuildReplica', 'KindWaitForPrimaryPlacement', 'KindWaitForPrimarySwap', 'KindWaitForReconfiguration'
	Kind KindBasicSafetyCheck `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) MarshalJSON() ([]byte, error) {
	easc.Kind = KindEnsureAvailability
	objectMap := make(map[string]interface{})
	if easc.PartitionID != nil {
		objectMap["PartitionId"] = easc.PartitionID
	}
	if easc.Kind != "" {
		objectMap["Kind"] = easc.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionSafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool) {
	return nil, false
}

// AsBasicPartitionSafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsBasicPartitionSafetyCheck() (BasicPartitionSafetyCheck, bool) {
	return &easc, true
}

// AsEnsureAvailabilitySafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool) {
	return &easc, true
}

// AsEnsurePartitionQuorumSafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool) {
	return nil, false
}

// AsSeedNodeSafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsSeedNodeSafetyCheck() (*SeedNodeSafetyCheck, bool) {
	return nil, false
}

// AsWaitForInbuildReplicaSafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimaryPlacementSafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimarySwapSafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool) {
	return nil, false
}

// AsWaitForReconfigurationSafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool) {
	return nil, false
}

// AsSafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsSafetyCheck() (*SafetyCheck, bool) {
	return nil, false
}

// AsBasicSafetyCheck is the BasicSafetyCheck implementation for EnsureAvailabilitySafetyCheck.
func (easc EnsureAvailabilitySafetyCheck) AsBasicSafetyCheck() (BasicSafetyCheck, bool) {
	return &easc, true
}

// EnsurePartitionQuorumSafetyCheck safety check that ensures that a quorum of replicas are not lost for a
// partition.
type EnsurePartitionQuorumSafetyCheck struct {
	// PartitionID - Id of the partition which is undergoing the safety check.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// Kind - Possible values include: 'KindSafetyCheck', 'KindPartitionSafetyCheck', 'KindEnsureAvailability', 'KindEnsurePartitionQuorum', 'KindEnsureSeedNodeQuorum', 'KindWaitForInbuildReplica', 'KindWaitForPrimaryPlacement', 'KindWaitForPrimarySwap', 'KindWaitForReconfiguration'
	Kind KindBasicSafetyCheck `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) MarshalJSON() ([]byte, error) {
	epqsc.Kind = KindEnsurePartitionQuorum
	objectMap := make(map[string]interface{})
	if epqsc.PartitionID != nil {
		objectMap["PartitionId"] = epqsc.PartitionID
	}
	if epqsc.Kind != "" {
		objectMap["Kind"] = epqsc.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionSafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool) {
	return nil, false
}

// AsBasicPartitionSafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsBasicPartitionSafetyCheck() (BasicPartitionSafetyCheck, bool) {
	return &epqsc, true
}

// AsEnsureAvailabilitySafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool) {
	return nil, false
}

// AsEnsurePartitionQuorumSafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool) {
	return &epqsc, true
}

// AsSeedNodeSafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsSeedNodeSafetyCheck() (*SeedNodeSafetyCheck, bool) {
	return nil, false
}

// AsWaitForInbuildReplicaSafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimaryPlacementSafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimarySwapSafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool) {
	return nil, false
}

// AsWaitForReconfigurationSafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool) {
	return nil, false
}

// AsSafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsSafetyCheck() (*SafetyCheck, bool) {
	return nil, false
}

// AsBasicSafetyCheck is the BasicSafetyCheck implementation for EnsurePartitionQuorumSafetyCheck.
func (epqsc EnsurePartitionQuorumSafetyCheck) AsBasicSafetyCheck() (BasicSafetyCheck, bool) {
	return &epqsc, true
}

// EntityHealth health information common to all entities in the cluster. It contains the aggregated health
// state, health events and unhealthy evaluation.
type EntityHealth struct {
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

// EntityHealthState a base type for the health state of various entities in the cluster. It contains the
// aggregated health state.
type EntityHealthState struct {
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
}

// EntityHealthStateChunk a base type for the health state chunk of various entities in the cluster. It
// contains the aggregated health state.
type EntityHealthStateChunk struct {
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
}

// EntityHealthStateChunkList a base type for the list of health state chunks found in the cluster. It
// contains the total number of health states that match the input filters.
type EntityHealthStateChunkList struct {
	// TotalCount - Total number of entity health state objects that match the specified filters from the cluster health chunk query description.
	TotalCount *int64 `json:"TotalCount,omitempty"`
}

// EntityKindHealthStateCount represents health state count for entities of the specified entity kind.
type EntityKindHealthStateCount struct {
	// EntityKind - The entity kind for which health states are evaluated. Possible values include: 'EntityKindInvalid', 'EntityKindNode', 'EntityKindPartition', 'EntityKindService', 'EntityKindApplication', 'EntityKindReplica', 'EntityKindDeployedApplication', 'EntityKindDeployedServicePackage', 'EntityKindCluster'
	EntityKind EntityKind `json:"EntityKind,omitempty"`
	// HealthStateCount - The health state count for the entities of the specified kind.
	HealthStateCount *HealthStateCount `json:"HealthStateCount,omitempty"`
}

// EnvironmentVariable describes an environment variable for the container.
type EnvironmentVariable struct {
	// Name - The name of the environment variable.
	Name *string `json:"name,omitempty"`
	// Value - The value of the environment variable.
	Value *string `json:"value,omitempty"`
}

// Epoch an Epoch is a configuration number for the partition as a whole. When the configuration of the
// replica set changes, for example when the Primary replica changes, the operations that are replicated
// from the new Primary replica are said to be a new Epoch from the ones which were sent by the old Primary
// replica.
type Epoch struct {
	// ConfigurationVersion - The current configuration number of this Epoch. The configuration number is an increasing value that is updated whenever the configuration of this replica set changes.
	ConfigurationVersion *string `json:"ConfigurationVersion,omitempty"`
	// DataLossVersion - The current data loss number of this Epoch. The data loss number property is an increasing value which is updated whenever data loss is suspected, as when loss of a quorum of replicas in the replica set that includes the Primary replica.
	DataLossVersion *string `json:"DataLossVersion,omitempty"`
}

// EventHealthEvaluation represents health evaluation of a HealthEvent that was reported on the entity.
// The health evaluation is returned when evaluating health of an entity results in Error or Warning.
type EventHealthEvaluation struct {
	// ConsiderWarningAsError - Indicates whether warnings are treated with the same severity as errors. The field is specified in the health policy used to evaluate the entity.
	ConsiderWarningAsError *bool `json:"ConsiderWarningAsError,omitempty"`
	// UnhealthyEvent - Represents health information reported on a health entity, such as cluster, application or node, with additional metadata added by the Health Manager.
	UnhealthyEvent *HealthEvent `json:"UnhealthyEvent,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for EventHealthEvaluation.
func (ehe EventHealthEvaluation) MarshalJSON() ([]byte, error) {
	ehe.Kind = KindEvent
	objectMap := make(map[string]interface{})
	if ehe.ConsiderWarningAsError != nil {
		objectMap["ConsiderWarningAsError"] = ehe.ConsiderWarningAsError
	}
	if ehe.UnhealthyEvent != nil {
		objectMap["UnhealthyEvent"] = ehe.UnhealthyEvent
	}
	if ehe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = ehe.AggregatedHealthState
	}
	if ehe.Description != nil {
		objectMap["Description"] = ehe.Description
	}
	if ehe.Kind != "" {
		objectMap["Kind"] = ehe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return &ehe, true
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for EventHealthEvaluation.
func (ehe EventHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &ehe, true
}

// ExecutingFaultsChaosEvent describes a Chaos event that gets generated when Chaos has decided on the
// faults for an iteration. This Chaos event contains the details of the faults as a list of strings.
type ExecutingFaultsChaosEvent struct {
	// Faults - List of string description of the faults that Chaos decided to execute in an iteration.
	Faults *[]string `json:"Faults,omitempty"`
	// TimeStampUtc - The UTC timestamp when this Chaos event was generated.
	TimeStampUtc *date.Time `json:"TimeStampUtc,omitempty"`
	// Kind - Possible values include: 'KindChaosEvent', 'KindExecutingFaults', 'KindStarted', 'KindStopped', 'KindTestError', 'KindValidationFailed', 'KindWaiting'
	Kind KindBasicChaosEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ExecutingFaultsChaosEvent.
func (efce ExecutingFaultsChaosEvent) MarshalJSON() ([]byte, error) {
	efce.Kind = KindExecutingFaults
	objectMap := make(map[string]interface{})
	if efce.Faults != nil {
		objectMap["Faults"] = efce.Faults
	}
	if efce.TimeStampUtc != nil {
		objectMap["TimeStampUtc"] = efce.TimeStampUtc
	}
	if efce.Kind != "" {
		objectMap["Kind"] = efce.Kind
	}
	return json.Marshal(objectMap)
}

// AsExecutingFaultsChaosEvent is the BasicChaosEvent implementation for ExecutingFaultsChaosEvent.
func (efce ExecutingFaultsChaosEvent) AsExecutingFaultsChaosEvent() (*ExecutingFaultsChaosEvent, bool) {
	return &efce, true
}

// AsStartedChaosEvent is the BasicChaosEvent implementation for ExecutingFaultsChaosEvent.
func (efce ExecutingFaultsChaosEvent) AsStartedChaosEvent() (*StartedChaosEvent, bool) {
	return nil, false
}

// AsStoppedChaosEvent is the BasicChaosEvent implementation for ExecutingFaultsChaosEvent.
func (efce ExecutingFaultsChaosEvent) AsStoppedChaosEvent() (*StoppedChaosEvent, bool) {
	return nil, false
}

// AsTestErrorChaosEvent is the BasicChaosEvent implementation for ExecutingFaultsChaosEvent.
func (efce ExecutingFaultsChaosEvent) AsTestErrorChaosEvent() (*TestErrorChaosEvent, bool) {
	return nil, false
}

// AsValidationFailedChaosEvent is the BasicChaosEvent implementation for ExecutingFaultsChaosEvent.
func (efce ExecutingFaultsChaosEvent) AsValidationFailedChaosEvent() (*ValidationFailedChaosEvent, bool) {
	return nil, false
}

// AsWaitingChaosEvent is the BasicChaosEvent implementation for ExecutingFaultsChaosEvent.
func (efce ExecutingFaultsChaosEvent) AsWaitingChaosEvent() (*WaitingChaosEvent, bool) {
	return nil, false
}

// AsChaosEvent is the BasicChaosEvent implementation for ExecutingFaultsChaosEvent.
func (efce ExecutingFaultsChaosEvent) AsChaosEvent() (*ChaosEvent, bool) {
	return nil, false
}

// AsBasicChaosEvent is the BasicChaosEvent implementation for ExecutingFaultsChaosEvent.
func (efce ExecutingFaultsChaosEvent) AsBasicChaosEvent() (BasicChaosEvent, bool) {
	return &efce, true
}

// ExternalStoreProvisionApplicationTypeDescription describes the operation to register or provision an
// application type using an application package from an external store instead of a package uploaded to
// the Service Fabric image store.
type ExternalStoreProvisionApplicationTypeDescription struct {
	// ApplicationPackageDownloadURI - The path to the '.sfpkg' application package from where the application package can be downloaded using HTTP or HTTPS protocols. The application package can be stored in an external store that provides GET operation to download the file. Supported protocols are HTTP and HTTPS, and the path must allow READ access.
	ApplicationPackageDownloadURI *string `json:"ApplicationPackageDownloadUri,omitempty"`
	// ApplicationTypeName - The application type name represents the name of the application type found in the application manifest.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// ApplicationTypeVersion - The application type version represents the version of the application type found in the application manifest.
	ApplicationTypeVersion *string `json:"ApplicationTypeVersion,omitempty"`
	// Async - Indicates whether or not provisioning should occur asynchronously. When set to true, the provision operation returns when the request is accepted by the system, and the provision operation continues without any timeout limit. The default value is false. For large application packages, we recommend setting the value to true.
	Async *bool `json:"Async,omitempty"`
	// Kind - Possible values include: 'KindProvisionApplicationTypeDescriptionBase', 'KindImageStorePath', 'KindExternalStore'
	Kind KindBasicProvisionApplicationTypeDescriptionBase `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ExternalStoreProvisionApplicationTypeDescription.
func (espatd ExternalStoreProvisionApplicationTypeDescription) MarshalJSON() ([]byte, error) {
	espatd.Kind = KindExternalStore
	objectMap := make(map[string]interface{})
	if espatd.ApplicationPackageDownloadURI != nil {
		objectMap["ApplicationPackageDownloadUri"] = espatd.ApplicationPackageDownloadURI
	}
	if espatd.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = espatd.ApplicationTypeName
	}
	if espatd.ApplicationTypeVersion != nil {
		objectMap["ApplicationTypeVersion"] = espatd.ApplicationTypeVersion
	}
	if espatd.Async != nil {
		objectMap["Async"] = espatd.Async
	}
	if espatd.Kind != "" {
		objectMap["Kind"] = espatd.Kind
	}
	return json.Marshal(objectMap)
}

// AsProvisionApplicationTypeDescription is the BasicProvisionApplicationTypeDescriptionBase implementation for ExternalStoreProvisionApplicationTypeDescription.
func (espatd ExternalStoreProvisionApplicationTypeDescription) AsProvisionApplicationTypeDescription() (*ProvisionApplicationTypeDescription, bool) {
	return nil, false
}

// AsExternalStoreProvisionApplicationTypeDescription is the BasicProvisionApplicationTypeDescriptionBase implementation for ExternalStoreProvisionApplicationTypeDescription.
func (espatd ExternalStoreProvisionApplicationTypeDescription) AsExternalStoreProvisionApplicationTypeDescription() (*ExternalStoreProvisionApplicationTypeDescription, bool) {
	return &espatd, true
}

// AsProvisionApplicationTypeDescriptionBase is the BasicProvisionApplicationTypeDescriptionBase implementation for ExternalStoreProvisionApplicationTypeDescription.
func (espatd ExternalStoreProvisionApplicationTypeDescription) AsProvisionApplicationTypeDescriptionBase() (*ProvisionApplicationTypeDescriptionBase, bool) {
	return nil, false
}

// AsBasicProvisionApplicationTypeDescriptionBase is the BasicProvisionApplicationTypeDescriptionBase implementation for ExternalStoreProvisionApplicationTypeDescription.
func (espatd ExternalStoreProvisionApplicationTypeDescription) AsBasicProvisionApplicationTypeDescriptionBase() (BasicProvisionApplicationTypeDescriptionBase, bool) {
	return &espatd, true
}

// FabricCodeVersionInfo information about a Service Fabric code version.
type FabricCodeVersionInfo struct {
	// CodeVersion - The product version of Service Fabric.
	CodeVersion *string `json:"CodeVersion,omitempty"`
}

// FabricConfigVersionInfo information about a Service Fabric config version.
type FabricConfigVersionInfo struct {
	// ConfigVersion - The config version of Service Fabric.
	ConfigVersion *string `json:"ConfigVersion,omitempty"`
}

// FabricError the REST API operations for Service Fabric return standard HTTP status codes. This type
// defines the additional information returned from the Service Fabric API operations that are not
// successful.
type FabricError struct {
	// Error - Error object containing error code and error message.
	Error *FabricErrorError `json:"Error,omitempty"`
}

// FabricErrorError error object containing error code and error message.
type FabricErrorError struct {
	// Code - Defines the fabric error codes that be returned as part of the error object in response to Service Fabric API operations that are not successful. Following are the error code values that can be returned for a specific HTTP status code.
	//   - Possible values of the error code for HTTP status code 400 (Bad Request)
	//     - "FABRIC_E_INVALID_PARTITION_KEY"
	//     - "FABRIC_E_IMAGEBUILDER_VALIDATION_ERROR"
	//     - "FABRIC_E_INVALID_ADDRESS"
	//     - "FABRIC_E_APPLICATION_NOT_UPGRADING"
	//     - "FABRIC_E_APPLICATION_UPGRADE_VALIDATION_ERROR"
	//     - "FABRIC_E_FABRIC_NOT_UPGRADING"
	//     - "FABRIC_E_FABRIC_UPGRADE_VALIDATION_ERROR"
	//     - "FABRIC_E_INVALID_CONFIGURATION"
	//     - "FABRIC_E_INVALID_NAME_URI"
	//     - "FABRIC_E_PATH_TOO_LONG"
	//     - "FABRIC_E_KEY_TOO_LARGE"
	//     - "FABRIC_E_SERVICE_AFFINITY_CHAIN_NOT_SUPPORTED"
	//     - "FABRIC_E_INVALID_ATOMIC_GROUP"
	//     - "FABRIC_E_VALUE_EMPTY"
	//     - "FABRIC_E_BACKUP_IS_ENABLED"
	//     - "FABRIC_E_RESTORE_SOURCE_TARGET_PARTITION_MISMATCH"
	//     - "FABRIC_E_INVALID_FOR_STATELESS_SERVICES"
	//     - "FABRIC_E_INVALID_SERVICE_SCALING_POLICY"
	//     - "E_INVALIDARG"
	//   - Possible values of the error code for HTTP status code 404 (Not Found)
	//     - "FABRIC_E_NODE_NOT_FOUND"
	//     - "FABRIC_E_APPLICATION_TYPE_NOT_FOUND"
	//     - "FABRIC_E_APPLICATION_NOT_FOUND"
	//     - "FABRIC_E_SERVICE_TYPE_NOT_FOUND"
	//     - "FABRIC_E_SERVICE_DOES_NOT_EXIST"
	//     - "FABRIC_E_SERVICE_TYPE_TEMPLATE_NOT_FOUND"
	//     - "FABRIC_E_CONFIGURATION_SECTION_NOT_FOUND"
	//     - "FABRIC_E_PARTITION_NOT_FOUND"
	//     - "FABRIC_E_REPLICA_DOES_NOT_EXIST"
	//     - "FABRIC_E_SERVICE_GROUP_DOES_NOT_EXIST"
	//     - "FABRIC_E_CONFIGURATION_PARAMETER_NOT_FOUND"
	//     - "FABRIC_E_DIRECTORY_NOT_FOUND"
	//     - "FABRIC_E_FABRIC_VERSION_NOT_FOUND"
	//     - "FABRIC_E_FILE_NOT_FOUND"
	//     - "FABRIC_E_NAME_DOES_NOT_EXIST"
	//     - "FABRIC_E_PROPERTY_DOES_NOT_EXIST"
	//     - "FABRIC_E_ENUMERATION_COMPLETED"
	//     - "FABRIC_E_SERVICE_MANIFEST_NOT_FOUND"
	//     - "FABRIC_E_KEY_NOT_FOUND"
	//     - "FABRIC_E_HEALTH_ENTITY_NOT_FOUND"
	//     - "FABRIC_E_BACKUP_NOT_ENABLED"
	//     - "FABRIC_E_BACKUP_POLICY_NOT_EXISTING"
	//     - "FABRIC_E_FAULT_ANALYSIS_SERVICE_NOT_EXISTING"
	//     - "FABRIC_E_IMAGEBUILDER_RESERVED_DIRECTORY_ERROR"
	//   - Possible values of the error code for HTTP status code 409 (Conflict)
	//     - "FABRIC_E_APPLICATION_TYPE_ALREADY_EXISTS"
	//     - "FABRIC_E_APPLICATION_ALREADY_EXISTS"
	//     - "FABRIC_E_APPLICATION_ALREADY_IN_TARGET_VERSION"
	//     - "FABRIC_E_APPLICATION_TYPE_PROVISION_IN_PROGRESS"
	//     - "FABRIC_E_APPLICATION_UPGRADE_IN_PROGRESS"
	//     - "FABRIC_E_SERVICE_ALREADY_EXISTS"
	//     - "FABRIC_E_SERVICE_GROUP_ALREADY_EXISTS"
	//     - "FABRIC_E_APPLICATION_TYPE_IN_USE"
	//     - "FABRIC_E_FABRIC_ALREADY_IN_TARGET_VERSION"
	//     - "FABRIC_E_FABRIC_VERSION_ALREADY_EXISTS"
	//     - "FABRIC_E_FABRIC_VERSION_IN_USE"
	//     - "FABRIC_E_FABRIC_UPGRADE_IN_PROGRESS"
	//     - "FABRIC_E_NAME_ALREADY_EXISTS"
	//     - "FABRIC_E_NAME_NOT_EMPTY"
	//     - "FABRIC_E_PROPERTY_CHECK_FAILED"
	//     - "FABRIC_E_SERVICE_METADATA_MISMATCH"
	//     - "FABRIC_E_SERVICE_TYPE_MISMATCH"
	//     - "FABRIC_E_HEALTH_STALE_REPORT"
	//     - "FABRIC_E_SEQUENCE_NUMBER_CHECK_FAILED"
	//     - "FABRIC_E_NODE_HAS_NOT_STOPPED_YET"
	//     - "FABRIC_E_INSTANCE_ID_MISMATCH"
	//     - "FABRIC_E_BACKUP_IN_PROGRESS"
	//     - "FABRIC_E_RESTORE_IN_PROGRESS"
	//     - "FABRIC_E_BACKUP_POLICY_ALREADY_EXISTING"
	//   - Possible values of the error code for HTTP status code 413 (Request Entity Too Large)
	//     - "FABRIC_E_VALUE_TOO_LARGE"
	//   - Possible values of the error code for HTTP status code 500 (Internal Server Error)
	//     - "FABRIC_E_NODE_IS_UP"
	//     - "E_FAIL"
	//     - "FABRIC_E_SINGLE_INSTANCE_APPLICATION_ALREADY_EXISTS"
	//     - "FABRIC_E_SINGLE_INSTANCE_APPLICATION_NOT_FOUND"
	//     - "FABRIC_E_VOLUME_ALREADY_EXISTS"
	//     - "FABRIC_E_VOLUME_NOT_FOUND"
	//     - "SerializationError"
	//   - Possible values of the error code for HTTP status code 503 (Service Unavailable)
	//     - "FABRIC_E_NO_WRITE_QUORUM"
	//     - "FABRIC_E_NOT_PRIMARY"
	//     - "FABRIC_E_NOT_READY"
	//     - "FABRIC_E_RECONFIGURATION_PENDING"
	//     - "FABRIC_E_SERVICE_OFFLINE"
	//     - "E_ABORT"
	//     - "FABRIC_E_VALUE_TOO_LARGE"
	//   - Possible values of the error code for HTTP status code 504 (Gateway Timeout)
	//     - "FABRIC_E_COMMUNICATION_ERROR"
	//     - "FABRIC_E_OPERATION_NOT_COMPLETE"
	//     - "FABRIC_E_TIMEOUT". Possible values include: 'FABRICEINVALIDPARTITIONKEY', 'FABRICEIMAGEBUILDERVALIDATIONERROR', 'FABRICEINVALIDADDRESS', 'FABRICEAPPLICATIONNOTUPGRADING', 'FABRICEAPPLICATIONUPGRADEVALIDATIONERROR', 'FABRICEFABRICNOTUPGRADING', 'FABRICEFABRICUPGRADEVALIDATIONERROR', 'FABRICEINVALIDCONFIGURATION', 'FABRICEINVALIDNAMEURI', 'FABRICEPATHTOOLONG', 'FABRICEKEYTOOLARGE', 'FABRICESERVICEAFFINITYCHAINNOTSUPPORTED', 'FABRICEINVALIDATOMICGROUP', 'FABRICEVALUEEMPTY', 'FABRICENODENOTFOUND', 'FABRICEAPPLICATIONTYPENOTFOUND', 'FABRICEAPPLICATIONNOTFOUND', 'FABRICESERVICETYPENOTFOUND', 'FABRICESERVICEDOESNOTEXIST', 'FABRICESERVICETYPETEMPLATENOTFOUND', 'FABRICECONFIGURATIONSECTIONNOTFOUND', 'FABRICEPARTITIONNOTFOUND', 'FABRICEREPLICADOESNOTEXIST', 'FABRICESERVICEGROUPDOESNOTEXIST', 'FABRICECONFIGURATIONPARAMETERNOTFOUND', 'FABRICEDIRECTORYNOTFOUND', 'FABRICEFABRICVERSIONNOTFOUND', 'FABRICEFILENOTFOUND', 'FABRICENAMEDOESNOTEXIST', 'FABRICEPROPERTYDOESNOTEXIST', 'FABRICEENUMERATIONCOMPLETED', 'FABRICESERVICEMANIFESTNOTFOUND', 'FABRICEKEYNOTFOUND', 'FABRICEHEALTHENTITYNOTFOUND', 'FABRICEAPPLICATIONTYPEALREADYEXISTS', 'FABRICEAPPLICATIONALREADYEXISTS', 'FABRICEAPPLICATIONALREADYINTARGETVERSION', 'FABRICEAPPLICATIONTYPEPROVISIONINPROGRESS', 'FABRICEAPPLICATIONUPGRADEINPROGRESS', 'FABRICESERVICEALREADYEXISTS', 'FABRICESERVICEGROUPALREADYEXISTS', 'FABRICEAPPLICATIONTYPEINUSE', 'FABRICEFABRICALREADYINTARGETVERSION', 'FABRICEFABRICVERSIONALREADYEXISTS', 'FABRICEFABRICVERSIONINUSE', 'FABRICEFABRICUPGRADEINPROGRESS', 'FABRICENAMEALREADYEXISTS', 'FABRICENAMENOTEMPTY', 'FABRICEPROPERTYCHECKFAILED', 'FABRICESERVICEMETADATAMISMATCH', 'FABRICESERVICETYPEMISMATCH', 'FABRICEHEALTHSTALEREPORT', 'FABRICESEQUENCENUMBERCHECKFAILED', 'FABRICENODEHASNOTSTOPPEDYET', 'FABRICEINSTANCEIDMISMATCH', 'FABRICEVALUETOOLARGE', 'FABRICENOWRITEQUORUM', 'FABRICENOTPRIMARY', 'FABRICENOTREADY', 'FABRICERECONFIGURATIONPENDING', 'FABRICESERVICEOFFLINE', 'EABORT', 'FABRICECOMMUNICATIONERROR', 'FABRICEOPERATIONNOTCOMPLETE', 'FABRICETIMEOUT', 'FABRICENODEISUP', 'EFAIL', 'FABRICEBACKUPISENABLED', 'FABRICERESTORESOURCETARGETPARTITIONMISMATCH', 'FABRICEINVALIDFORSTATELESSSERVICES', 'FABRICEBACKUPNOTENABLED', 'FABRICEBACKUPPOLICYNOTEXISTING', 'FABRICEFAULTANALYSISSERVICENOTEXISTING', 'FABRICEBACKUPINPROGRESS', 'FABRICERESTOREINPROGRESS', 'FABRICEBACKUPPOLICYALREADYEXISTING', 'FABRICEINVALIDSERVICESCALINGPOLICY', 'EINVALIDARG', 'FABRICESINGLEINSTANCEAPPLICATIONALREADYEXISTS', 'FABRICESINGLEINSTANCEAPPLICATIONNOTFOUND', 'FABRICEVOLUMEALREADYEXISTS', 'FABRICEVOLUMENOTFOUND', 'SerializationError', 'FABRICEIMAGEBUILDERRESERVEDDIRECTORYERROR'
	Code FabricErrorCodes `json:"Code,omitempty"`
	// Message - Error message.
	Message *string `json:"Message,omitempty"`
}

// BasicFabricEvent represents the base for all Fabric Events.
type BasicFabricEvent interface {
	AsApplicationEvent() (*ApplicationEvent, bool)
	AsBasicApplicationEvent() (BasicApplicationEvent, bool)
	AsClusterEvent() (*ClusterEvent, bool)
	AsBasicClusterEvent() (BasicClusterEvent, bool)
	AsContainerInstanceEvent() (*ContainerInstanceEvent, bool)
	AsNodeEvent() (*NodeEvent, bool)
	AsBasicNodeEvent() (BasicNodeEvent, bool)
	AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool)
	AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool)
	AsPartitionEvent() (*PartitionEvent, bool)
	AsBasicPartitionEvent() (BasicPartitionEvent, bool)
	AsReplicaEvent() (*ReplicaEvent, bool)
	AsBasicReplicaEvent() (BasicReplicaEvent, bool)
	AsServiceEvent() (*ServiceEvent, bool)
	AsBasicServiceEvent() (BasicServiceEvent, bool)
	AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool)
	AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool)
	AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool)
	AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool)
	AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool)
	AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool)
	AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool)
	AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool)
	AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool)
	AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool)
	AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool)
	AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool)
	AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool)
	AsNodeAbortedEvent() (*NodeAbortedEvent, bool)
	AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool)
	AsNodeClosedEvent() (*NodeClosedEvent, bool)
	AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool)
	AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool)
	AsNodeDownEvent() (*NodeDownEvent, bool)
	AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool)
	AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool)
	AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool)
	AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool)
	AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool)
	AsNodeUpEvent() (*NodeUpEvent, bool)
	AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool)
	AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool)
	AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool)
	AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool)
	AsServiceCreatedEvent() (*ServiceCreatedEvent, bool)
	AsServiceDeletedEvent() (*ServiceDeletedEvent, bool)
	AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool)
	AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool)
	AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool)
	AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool)
	AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool)
	AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool)
	AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool)
	AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool)
	AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool)
	AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool)
	AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool)
	AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool)
	AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool)
	AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool)
	AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool)
	AsChaosStoppedEvent() (*ChaosStoppedEvent, bool)
	AsChaosStartedEvent() (*ChaosStartedEvent, bool)
	AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool)
	AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool)
	AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool)
	AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool)
	AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool)
	AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool)
	AsFabricEvent() (*FabricEvent, bool)
}

// FabricEvent represents the base for all Fabric Events.
type FabricEvent struct {
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

func unmarshalBasicFabricEvent(body []byte) (BasicFabricEvent, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindApplicationEvent):
		var ae ApplicationEvent
		err := json.Unmarshal(body, &ae)
		return ae, err
	case string(KindClusterEvent):
		var ce ClusterEvent
		err := json.Unmarshal(body, &ce)
		return ce, err
	case string(KindContainerInstanceEvent):
		var cie ContainerInstanceEvent
		err := json.Unmarshal(body, &cie)
		return cie, err
	case string(KindNodeEvent):
		var ne NodeEvent
		err := json.Unmarshal(body, &ne)
		return ne, err
	case string(KindPartitionAnalysisEvent):
		var pae PartitionAnalysisEvent
		err := json.Unmarshal(body, &pae)
		return pae, err
	case string(KindPartitionEvent):
		var peVar PartitionEvent
		err := json.Unmarshal(body, &peVar)
		return peVar, err
	case string(KindReplicaEvent):
		var re ReplicaEvent
		err := json.Unmarshal(body, &re)
		return re, err
	case string(KindServiceEvent):
		var se ServiceEvent
		err := json.Unmarshal(body, &se)
		return se, err
	case string(KindApplicationCreated):
		var ace ApplicationCreatedEvent
		err := json.Unmarshal(body, &ace)
		return ace, err
	case string(KindApplicationDeleted):
		var ade ApplicationDeletedEvent
		err := json.Unmarshal(body, &ade)
		return ade, err
	case string(KindApplicationNewHealthReport):
		var anhre ApplicationNewHealthReportEvent
		err := json.Unmarshal(body, &anhre)
		return anhre, err
	case string(KindApplicationHealthReportExpired):
		var ahree ApplicationHealthReportExpiredEvent
		err := json.Unmarshal(body, &ahree)
		return ahree, err
	case string(KindApplicationUpgradeCompleted):
		var auce ApplicationUpgradeCompletedEvent
		err := json.Unmarshal(body, &auce)
		return auce, err
	case string(KindApplicationUpgradeDomainCompleted):
		var audce ApplicationUpgradeDomainCompletedEvent
		err := json.Unmarshal(body, &audce)
		return audce, err
	case string(KindApplicationUpgradeRollbackCompleted):
		var aurce ApplicationUpgradeRollbackCompletedEvent
		err := json.Unmarshal(body, &aurce)
		return aurce, err
	case string(KindApplicationUpgradeRollbackStarted):
		var aurse ApplicationUpgradeRollbackStartedEvent
		err := json.Unmarshal(body, &aurse)
		return aurse, err
	case string(KindApplicationUpgradeStarted):
		var ause ApplicationUpgradeStartedEvent
		err := json.Unmarshal(body, &ause)
		return ause, err
	case string(KindDeployedApplicationNewHealthReport):
		var danhre DeployedApplicationNewHealthReportEvent
		err := json.Unmarshal(body, &danhre)
		return danhre, err
	case string(KindDeployedApplicationHealthReportExpired):
		var dahree DeployedApplicationHealthReportExpiredEvent
		err := json.Unmarshal(body, &dahree)
		return dahree, err
	case string(KindApplicationProcessExited):
		var apee ApplicationProcessExitedEvent
		err := json.Unmarshal(body, &apee)
		return apee, err
	case string(KindApplicationContainerInstanceExited):
		var aciee ApplicationContainerInstanceExitedEvent
		err := json.Unmarshal(body, &aciee)
		return aciee, err
	case string(KindNodeAborted):
		var nae NodeAbortedEvent
		err := json.Unmarshal(body, &nae)
		return nae, err
	case string(KindNodeAddedToCluster):
		var natce NodeAddedToClusterEvent
		err := json.Unmarshal(body, &natce)
		return natce, err
	case string(KindNodeClosed):
		var nce NodeClosedEvent
		err := json.Unmarshal(body, &nce)
		return nce, err
	case string(KindNodeDeactivateCompleted):
		var ndce NodeDeactivateCompletedEvent
		err := json.Unmarshal(body, &ndce)
		return ndce, err
	case string(KindNodeDeactivateStarted):
		var ndse NodeDeactivateStartedEvent
		err := json.Unmarshal(body, &ndse)
		return ndse, err
	case string(KindNodeDown):
		var nde NodeDownEvent
		err := json.Unmarshal(body, &nde)
		return nde, err
	case string(KindNodeNewHealthReport):
		var nnhre NodeNewHealthReportEvent
		err := json.Unmarshal(body, &nnhre)
		return nnhre, err
	case string(KindNodeHealthReportExpired):
		var nhree NodeHealthReportExpiredEvent
		err := json.Unmarshal(body, &nhree)
		return nhree, err
	case string(KindNodeOpenSucceeded):
		var nose NodeOpenSucceededEvent
		err := json.Unmarshal(body, &nose)
		return nose, err
	case string(KindNodeOpenFailed):
		var nofe NodeOpenFailedEvent
		err := json.Unmarshal(body, &nofe)
		return nofe, err
	case string(KindNodeRemovedFromCluster):
		var nrfce NodeRemovedFromClusterEvent
		err := json.Unmarshal(body, &nrfce)
		return nrfce, err
	case string(KindNodeUp):
		var nue NodeUpEvent
		err := json.Unmarshal(body, &nue)
		return nue, err
	case string(KindPartitionNewHealthReport):
		var pnhre PartitionNewHealthReportEvent
		err := json.Unmarshal(body, &pnhre)
		return pnhre, err
	case string(KindPartitionHealthReportExpired):
		var phree PartitionHealthReportExpiredEvent
		err := json.Unmarshal(body, &phree)
		return phree, err
	case string(KindPartitionReconfigured):
		var pre PartitionReconfiguredEvent
		err := json.Unmarshal(body, &pre)
		return pre, err
	case string(KindPartitionPrimaryMoveAnalysis):
		var ppmae PartitionPrimaryMoveAnalysisEvent
		err := json.Unmarshal(body, &ppmae)
		return ppmae, err
	case string(KindServiceCreated):
		var sce ServiceCreatedEvent
		err := json.Unmarshal(body, &sce)
		return sce, err
	case string(KindServiceDeleted):
		var sde ServiceDeletedEvent
		err := json.Unmarshal(body, &sde)
		return sde, err
	case string(KindServiceNewHealthReport):
		var snhre ServiceNewHealthReportEvent
		err := json.Unmarshal(body, &snhre)
		return snhre, err
	case string(KindServiceHealthReportExpired):
		var shree ServiceHealthReportExpiredEvent
		err := json.Unmarshal(body, &shree)
		return shree, err
	case string(KindDeployedServicePackageNewHealthReport):
		var dspnhre DeployedServicePackageNewHealthReportEvent
		err := json.Unmarshal(body, &dspnhre)
		return dspnhre, err
	case string(KindDeployedServicePackageHealthReportExpired):
		var dsphree DeployedServicePackageHealthReportExpiredEvent
		err := json.Unmarshal(body, &dsphree)
		return dsphree, err
	case string(KindStatefulReplicaNewHealthReport):
		var srnhre StatefulReplicaNewHealthReportEvent
		err := json.Unmarshal(body, &srnhre)
		return srnhre, err
	case string(KindStatefulReplicaHealthReportExpired):
		var srhree StatefulReplicaHealthReportExpiredEvent
		err := json.Unmarshal(body, &srhree)
		return srhree, err
	case string(KindStatelessReplicaNewHealthReport):
		var srnhre StatelessReplicaNewHealthReportEvent
		err := json.Unmarshal(body, &srnhre)
		return srnhre, err
	case string(KindStatelessReplicaHealthReportExpired):
		var srhree StatelessReplicaHealthReportExpiredEvent
		err := json.Unmarshal(body, &srhree)
		return srhree, err
	case string(KindClusterNewHealthReport):
		var cnhre ClusterNewHealthReportEvent
		err := json.Unmarshal(body, &cnhre)
		return cnhre, err
	case string(KindClusterHealthReportExpired):
		var chree ClusterHealthReportExpiredEvent
		err := json.Unmarshal(body, &chree)
		return chree, err
	case string(KindClusterUpgradeCompleted):
		var cuce ClusterUpgradeCompletedEvent
		err := json.Unmarshal(body, &cuce)
		return cuce, err
	case string(KindClusterUpgradeDomainCompleted):
		var cudce ClusterUpgradeDomainCompletedEvent
		err := json.Unmarshal(body, &cudce)
		return cudce, err
	case string(KindClusterUpgradeRollbackCompleted):
		var curce ClusterUpgradeRollbackCompletedEvent
		err := json.Unmarshal(body, &curce)
		return curce, err
	case string(KindClusterUpgradeRollbackStarted):
		var curse ClusterUpgradeRollbackStartedEvent
		err := json.Unmarshal(body, &curse)
		return curse, err
	case string(KindClusterUpgradeStarted):
		var cuse ClusterUpgradeStartedEvent
		err := json.Unmarshal(body, &cuse)
		return cuse, err
	case string(KindChaosStopped):
		var cse ChaosStoppedEvent
		err := json.Unmarshal(body, &cse)
		return cse, err
	case string(KindChaosStarted):
		var cse ChaosStartedEvent
		err := json.Unmarshal(body, &cse)
		return cse, err
	case string(KindChaosCodePackageRestartScheduled):
		var ccprse ChaosCodePackageRestartScheduledEvent
		err := json.Unmarshal(body, &ccprse)
		return ccprse, err
	case string(KindChaosReplicaRemovalScheduled):
		var crrse ChaosReplicaRemovalScheduledEvent
		err := json.Unmarshal(body, &crrse)
		return crrse, err
	case string(KindChaosPartitionSecondaryMoveScheduled):
		var cpsmse ChaosPartitionSecondaryMoveScheduledEvent
		err := json.Unmarshal(body, &cpsmse)
		return cpsmse, err
	case string(KindChaosPartitionPrimaryMoveScheduled):
		var cppmse ChaosPartitionPrimaryMoveScheduledEvent
		err := json.Unmarshal(body, &cppmse)
		return cppmse, err
	case string(KindChaosReplicaRestartScheduled):
		var crrse ChaosReplicaRestartScheduledEvent
		err := json.Unmarshal(body, &crrse)
		return crrse, err
	case string(KindChaosNodeRestartScheduled):
		var cnrse ChaosNodeRestartScheduledEvent
		err := json.Unmarshal(body, &cnrse)
		return cnrse, err
	default:
		var fe FabricEvent
		err := json.Unmarshal(body, &fe)
		return fe, err
	}
}
func unmarshalBasicFabricEventArray(body []byte) ([]BasicFabricEvent, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	feArray := make([]BasicFabricEvent, len(rawMessages))

	for index, rawMessage := range rawMessages {
		fe, err := unmarshalBasicFabricEvent(*rawMessage)
		if err != nil {
			return nil, err
		}
		feArray[index] = fe
	}
	return feArray, nil
}

// MarshalJSON is the custom marshaler for FabricEvent.
func (fe FabricEvent) MarshalJSON() ([]byte, error) {
	fe.Kind = KindFabricEvent
	objectMap := make(map[string]interface{})
	if fe.EventInstanceID != nil {
		objectMap["EventInstanceId"] = fe.EventInstanceID
	}
	if fe.Category != nil {
		objectMap["Category"] = fe.Category
	}
	if fe.TimeStamp != nil {
		objectMap["TimeStamp"] = fe.TimeStamp
	}
	if fe.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = fe.HasCorrelatedEvents
	}
	if fe.Kind != "" {
		objectMap["Kind"] = fe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsFabricEvent() (*FabricEvent, bool) {
	return &fe, true
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for FabricEvent.
func (fe FabricEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &fe, true
}

// FailedPropertyBatchInfo derived from PropertyBatchInfo. Represents the property batch failing. Contains
// information about the specific batch failure.
type FailedPropertyBatchInfo struct {
	// ErrorMessage - The error message of the failed operation. Describes the exception thrown due to the first unsuccessful operation in the property batch.
	ErrorMessage *string `json:"ErrorMessage,omitempty"`
	// OperationIndex - The index of the unsuccessful operation in the property batch.
	OperationIndex *int32 `json:"OperationIndex,omitempty"`
	// Kind - Possible values include: 'KindPropertyBatchInfo', 'KindSuccessful', 'KindFailed'
	Kind KindBasicPropertyBatchInfo `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for FailedPropertyBatchInfo.
func (fpbi FailedPropertyBatchInfo) MarshalJSON() ([]byte, error) {
	fpbi.Kind = KindFailed
	objectMap := make(map[string]interface{})
	if fpbi.ErrorMessage != nil {
		objectMap["ErrorMessage"] = fpbi.ErrorMessage
	}
	if fpbi.OperationIndex != nil {
		objectMap["OperationIndex"] = fpbi.OperationIndex
	}
	if fpbi.Kind != "" {
		objectMap["Kind"] = fpbi.Kind
	}
	return json.Marshal(objectMap)
}

// AsSuccessfulPropertyBatchInfo is the BasicPropertyBatchInfo implementation for FailedPropertyBatchInfo.
func (fpbi FailedPropertyBatchInfo) AsSuccessfulPropertyBatchInfo() (*SuccessfulPropertyBatchInfo, bool) {
	return nil, false
}

// AsFailedPropertyBatchInfo is the BasicPropertyBatchInfo implementation for FailedPropertyBatchInfo.
func (fpbi FailedPropertyBatchInfo) AsFailedPropertyBatchInfo() (*FailedPropertyBatchInfo, bool) {
	return &fpbi, true
}

// AsPropertyBatchInfo is the BasicPropertyBatchInfo implementation for FailedPropertyBatchInfo.
func (fpbi FailedPropertyBatchInfo) AsPropertyBatchInfo() (*PropertyBatchInfo, bool) {
	return nil, false
}

// AsBasicPropertyBatchInfo is the BasicPropertyBatchInfo implementation for FailedPropertyBatchInfo.
func (fpbi FailedPropertyBatchInfo) AsBasicPropertyBatchInfo() (BasicPropertyBatchInfo, bool) {
	return &fpbi, true
}

// FailedUpgradeDomainProgressObject the detailed upgrade progress for nodes in the current upgrade domain
// at the point of failure.
type FailedUpgradeDomainProgressObject struct {
	// DomainName - The name of the upgrade domain
	DomainName *string `json:"DomainName,omitempty"`
	// NodeUpgradeProgressList - List of upgrading nodes and their statuses
	NodeUpgradeProgressList *[]NodeUpgradeProgressInfo `json:"NodeUpgradeProgressList,omitempty"`
}

// FailureUpgradeDomainProgressInfo information about the upgrade domain progress at the time of upgrade
// failure.
type FailureUpgradeDomainProgressInfo struct {
	// DomainName - The name of the upgrade domain
	DomainName *string `json:"DomainName,omitempty"`
	// NodeUpgradeProgressList - List of upgrading nodes and their statuses
	NodeUpgradeProgressList *[]NodeUpgradeProgressInfo `json:"NodeUpgradeProgressList,omitempty"`
}

// FileInfo information about a image store file.
type FileInfo struct {
	// FileSize - The size of file in bytes.
	FileSize *string `json:"FileSize,omitempty"`
	// FileVersion - Information about the version of image store file.
	FileVersion *FileVersion `json:"FileVersion,omitempty"`
	// ModifiedDate - The date and time when the image store file was last modified.
	ModifiedDate *date.Time `json:"ModifiedDate,omitempty"`
	// StoreRelativePath - The file path relative to the image store root path.
	StoreRelativePath *string `json:"StoreRelativePath,omitempty"`
}

// FileShareBackupStorageDescription describes the parameters for file share storage used for storing or
// enumerating backups.
type FileShareBackupStorageDescription struct {
	// Path - UNC path of the file share where to store or enumerate backups from.
	Path *string `json:"Path,omitempty"`
	// PrimaryUserName - Primary user name to access the file share.
	PrimaryUserName *string `json:"PrimaryUserName,omitempty"`
	// PrimaryPassword - Primary password to access the share location.
	PrimaryPassword *string `json:"PrimaryPassword,omitempty"`
	// SecondaryUserName - Secondary user name to access the file share.
	SecondaryUserName *string `json:"SecondaryUserName,omitempty"`
	// SecondaryPassword - Secondary password to access the share location
	SecondaryPassword *string `json:"SecondaryPassword,omitempty"`
	// FriendlyName - Friendly name for this backup storage.
	FriendlyName *string `json:"FriendlyName,omitempty"`
	// StorageKind - Possible values include: 'StorageKindBackupStorageDescription', 'StorageKindAzureBlobStore', 'StorageKindFileShare'
	StorageKind StorageKind `json:"StorageKind,omitempty"`
}

// MarshalJSON is the custom marshaler for FileShareBackupStorageDescription.
func (fsbsd FileShareBackupStorageDescription) MarshalJSON() ([]byte, error) {
	fsbsd.StorageKind = StorageKindFileShare
	objectMap := make(map[string]interface{})
	if fsbsd.Path != nil {
		objectMap["Path"] = fsbsd.Path
	}
	if fsbsd.PrimaryUserName != nil {
		objectMap["PrimaryUserName"] = fsbsd.PrimaryUserName
	}
	if fsbsd.PrimaryPassword != nil {
		objectMap["PrimaryPassword"] = fsbsd.PrimaryPassword
	}
	if fsbsd.SecondaryUserName != nil {
		objectMap["SecondaryUserName"] = fsbsd.SecondaryUserName
	}
	if fsbsd.SecondaryPassword != nil {
		objectMap["SecondaryPassword"] = fsbsd.SecondaryPassword
	}
	if fsbsd.FriendlyName != nil {
		objectMap["FriendlyName"] = fsbsd.FriendlyName
	}
	if fsbsd.StorageKind != "" {
		objectMap["StorageKind"] = fsbsd.StorageKind
	}
	return json.Marshal(objectMap)
}

// AsAzureBlobBackupStorageDescription is the BasicBackupStorageDescription implementation for FileShareBackupStorageDescription.
func (fsbsd FileShareBackupStorageDescription) AsAzureBlobBackupStorageDescription() (*AzureBlobBackupStorageDescription, bool) {
	return nil, false
}

// AsFileShareBackupStorageDescription is the BasicBackupStorageDescription implementation for FileShareBackupStorageDescription.
func (fsbsd FileShareBackupStorageDescription) AsFileShareBackupStorageDescription() (*FileShareBackupStorageDescription, bool) {
	return &fsbsd, true
}

// AsBackupStorageDescription is the BasicBackupStorageDescription implementation for FileShareBackupStorageDescription.
func (fsbsd FileShareBackupStorageDescription) AsBackupStorageDescription() (*BackupStorageDescription, bool) {
	return nil, false
}

// AsBasicBackupStorageDescription is the BasicBackupStorageDescription implementation for FileShareBackupStorageDescription.
func (fsbsd FileShareBackupStorageDescription) AsBasicBackupStorageDescription() (BasicBackupStorageDescription, bool) {
	return &fsbsd, true
}

// FileVersion information about the version of image store file.
type FileVersion struct {
	// VersionNumber - The current image store version number for the file is used in image store for checking whether it need to be updated.
	VersionNumber *string `json:"VersionNumber,omitempty"`
	// EpochDataLossNumber - The epoch data loss number of image store replica when this file entry was updated or created.
	EpochDataLossNumber *string `json:"EpochDataLossNumber,omitempty"`
	// EpochConfigurationNumber - The epoch configuration version number of the image store replica when this file entry was created or updated.
	EpochConfigurationNumber *string `json:"EpochConfigurationNumber,omitempty"`
}

// FolderInfo information about a image store folder. It includes how many files this folder contains and
// its image store relative path.
type FolderInfo struct {
	// StoreRelativePath - The remote location within image store. This path is relative to the image store root.
	StoreRelativePath *string `json:"StoreRelativePath,omitempty"`
	// FileCount - The number of files from within the image store folder.
	FileCount *string `json:"FileCount,omitempty"`
}

// FolderSizeInfo information of a image store folder size
type FolderSizeInfo struct {
	autorest.Response `json:"-"`
	// StoreRelativePath - The remote location within image store. This path is relative to the image store root.
	StoreRelativePath *string `json:"StoreRelativePath,omitempty"`
	// FolderSize - The size of folder in bytes.
	FolderSize *string `json:"FolderSize,omitempty"`
}

// FrequencyBasedBackupScheduleDescription describes the frequency based backup schedule.
type FrequencyBasedBackupScheduleDescription struct {
	// Interval - Defines the interval with which backups are periodically taken. It should be specified in ISO8601 format. Timespan in seconds is not supported and will be ignored while creating the policy.
	Interval *string `json:"Interval,omitempty"`
	// ScheduleKind - Possible values include: 'ScheduleKindBackupScheduleDescription', 'ScheduleKindFrequencyBased', 'ScheduleKindTimeBased'
	ScheduleKind ScheduleKind `json:"ScheduleKind,omitempty"`
}

// MarshalJSON is the custom marshaler for FrequencyBasedBackupScheduleDescription.
func (fbbsd FrequencyBasedBackupScheduleDescription) MarshalJSON() ([]byte, error) {
	fbbsd.ScheduleKind = ScheduleKindFrequencyBased
	objectMap := make(map[string]interface{})
	if fbbsd.Interval != nil {
		objectMap["Interval"] = fbbsd.Interval
	}
	if fbbsd.ScheduleKind != "" {
		objectMap["ScheduleKind"] = fbbsd.ScheduleKind
	}
	return json.Marshal(objectMap)
}

// AsFrequencyBasedBackupScheduleDescription is the BasicBackupScheduleDescription implementation for FrequencyBasedBackupScheduleDescription.
func (fbbsd FrequencyBasedBackupScheduleDescription) AsFrequencyBasedBackupScheduleDescription() (*FrequencyBasedBackupScheduleDescription, bool) {
	return &fbbsd, true
}

// AsTimeBasedBackupScheduleDescription is the BasicBackupScheduleDescription implementation for FrequencyBasedBackupScheduleDescription.
func (fbbsd FrequencyBasedBackupScheduleDescription) AsTimeBasedBackupScheduleDescription() (*TimeBasedBackupScheduleDescription, bool) {
	return nil, false
}

// AsBackupScheduleDescription is the BasicBackupScheduleDescription implementation for FrequencyBasedBackupScheduleDescription.
func (fbbsd FrequencyBasedBackupScheduleDescription) AsBackupScheduleDescription() (*BackupScheduleDescription, bool) {
	return nil, false
}

// AsBasicBackupScheduleDescription is the BasicBackupScheduleDescription implementation for FrequencyBasedBackupScheduleDescription.
func (fbbsd FrequencyBasedBackupScheduleDescription) AsBasicBackupScheduleDescription() (BasicBackupScheduleDescription, bool) {
	return &fbbsd, true
}

// GatewayDestination describes destination endpoint for routing traffic.
type GatewayDestination struct {
	// ApplicationName - Name of the service fabric Mesh application.
	ApplicationName *string `json:"applicationName,omitempty"`
	// ServiceName - service that contains the endpoint.
	ServiceName *string `json:"serviceName,omitempty"`
	// EndpointName - name of the endpoint in the service.
	EndpointName *string `json:"endpointName,omitempty"`
}

// GatewayProperties describes properties of a gateway resource.
type GatewayProperties struct {
	// Description - User readable description of the gateway.
	Description *string `json:"description,omitempty"`
	// SourceNetwork - Network the gateway should listen on for requests.
	SourceNetwork *NetworkRef `json:"sourceNetwork,omitempty"`
	// DestinationNetwork - Network that the Application is using.
	DestinationNetwork *NetworkRef `json:"destinationNetwork,omitempty"`
	// TCP - Configuration for tcp connectivity for this gateway.
	TCP *[]TCPConfig `json:"tcp,omitempty"`
	// HTTP - Configuration for http connectivity for this gateway.
	HTTP *[]HTTPConfig `json:"http,omitempty"`
	// Status - READ-ONLY; Status of the resource. Possible values include: 'ResourceStatusUnknown', 'ResourceStatusReady', 'ResourceStatusUpgrading', 'ResourceStatusCreating', 'ResourceStatusDeleting', 'ResourceStatusFailed'
	Status ResourceStatus `json:"status,omitempty"`
	// StatusDetails - READ-ONLY; Gives additional information about the current status of the gateway.
	StatusDetails *string `json:"statusDetails,omitempty"`
	// IPAddress - READ-ONLY; IP address of the gateway. This is populated in the response and is ignored for incoming requests.
	IPAddress *string `json:"ipAddress,omitempty"`
}

// MarshalJSON is the custom marshaler for GatewayProperties.
func (gp GatewayProperties) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if gp.Description != nil {
		objectMap["description"] = gp.Description
	}
	if gp.SourceNetwork != nil {
		objectMap["sourceNetwork"] = gp.SourceNetwork
	}
	if gp.DestinationNetwork != nil {
		objectMap["destinationNetwork"] = gp.DestinationNetwork
	}
	if gp.TCP != nil {
		objectMap["tcp"] = gp.TCP
	}
	if gp.HTTP != nil {
		objectMap["http"] = gp.HTTP
	}
	return json.Marshal(objectMap)
}

// GatewayResourceDescription this type describes a gateway resource.
type GatewayResourceDescription struct {
	autorest.Response `json:"-"`
	// Name - Name of the Gateway resource.
	Name *string `json:"name,omitempty"`
	// GatewayProperties - Describes properties of a gateway resource.
	*GatewayProperties `json:"properties,omitempty"`
}

// MarshalJSON is the custom marshaler for GatewayResourceDescription.
func (grd GatewayResourceDescription) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if grd.Name != nil {
		objectMap["name"] = grd.Name
	}
	if grd.GatewayProperties != nil {
		objectMap["properties"] = grd.GatewayProperties
	}
	return json.Marshal(objectMap)
}

// UnmarshalJSON is the custom unmarshaler for GatewayResourceDescription struct.
func (grd *GatewayResourceDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				grd.Name = &name
			}
		case "properties":
			if v != nil {
				var gatewayProperties GatewayProperties
				err = json.Unmarshal(*v, &gatewayProperties)
				if err != nil {
					return err
				}
				grd.GatewayProperties = &gatewayProperties
			}
		}
	}

	return nil
}

// GetBackupByStorageQueryDescription describes additional filters to be applied, while listing backups,
// and backup storage details from where to fetch the backups.
type GetBackupByStorageQueryDescription struct {
	// StartDateTimeFilter - Specifies the start date time in ISO8601 from which to enumerate backups. If not specified, backups are enumerated from the beginning.
	StartDateTimeFilter *date.Time `json:"StartDateTimeFilter,omitempty"`
	// EndDateTimeFilter - Specifies the end date time in ISO8601 till which to enumerate backups. If not specified, backups are enumerated till the end.
	EndDateTimeFilter *date.Time `json:"EndDateTimeFilter,omitempty"`
	// Latest - If specified as true, gets the most recent backup (within the specified time range) for every partition under the specified backup entity.
	Latest *bool `json:"Latest,omitempty"`
	// Storage - Describes the parameters for the backup storage from where to enumerate backups. This is optional and by default backups are enumerated from the backup storage where this backup entity is currently being backed up (as specified in backup policy). This parameter is useful to be able to enumerate backups from another cluster where you may intend to restore.
	Storage BasicBackupStorageDescription `json:"Storage,omitempty"`
	// BackupEntity - Indicates the entity for which to enumerate backups.
	BackupEntity BasicBackupEntity `json:"BackupEntity,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for GetBackupByStorageQueryDescription struct.
func (gbbsqd *GetBackupByStorageQueryDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "StartDateTimeFilter":
			if v != nil {
				var startDateTimeFilter date.Time
				err = json.Unmarshal(*v, &startDateTimeFilter)
				if err != nil {
					return err
				}
				gbbsqd.StartDateTimeFilter = &startDateTimeFilter
			}
		case "EndDateTimeFilter":
			if v != nil {
				var endDateTimeFilter date.Time
				err = json.Unmarshal(*v, &endDateTimeFilter)
				if err != nil {
					return err
				}
				gbbsqd.EndDateTimeFilter = &endDateTimeFilter
			}
		case "Latest":
			if v != nil {
				var latest bool
				err = json.Unmarshal(*v, &latest)
				if err != nil {
					return err
				}
				gbbsqd.Latest = &latest
			}
		case "Storage":
			if v != nil {
				storage, err := unmarshalBasicBackupStorageDescription(*v)
				if err != nil {
					return err
				}
				gbbsqd.Storage = storage
			}
		case "BackupEntity":
			if v != nil {
				backupEntity, err := unmarshalBasicBackupEntity(*v)
				if err != nil {
					return err
				}
				gbbsqd.BackupEntity = backupEntity
			}
		}
	}

	return nil
}

// GetPropertyBatchOperation represents a PropertyBatchOperation that gets the specified property if it
// exists.
// Note that if one PropertyBatchOperation in a PropertyBatch fails,
// the entire batch fails and cannot be committed in a transactional manner.
type GetPropertyBatchOperation struct {
	// IncludeValue - Whether or not to return the property value with the metadata.
	// True if values should be returned with the metadata; False to return only property metadata.
	IncludeValue *bool `json:"IncludeValue,omitempty"`
	// PropertyName - The name of the Service Fabric property.
	PropertyName *string `json:"PropertyName,omitempty"`
	// Kind - Possible values include: 'KindPropertyBatchOperation', 'KindCheckExists', 'KindCheckSequence', 'KindCheckValue', 'KindDelete', 'KindGet', 'KindPut'
	Kind KindBasicPropertyBatchOperation `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for GetPropertyBatchOperation.
func (gpbo GetPropertyBatchOperation) MarshalJSON() ([]byte, error) {
	gpbo.Kind = KindGet
	objectMap := make(map[string]interface{})
	if gpbo.IncludeValue != nil {
		objectMap["IncludeValue"] = gpbo.IncludeValue
	}
	if gpbo.PropertyName != nil {
		objectMap["PropertyName"] = gpbo.PropertyName
	}
	if gpbo.Kind != "" {
		objectMap["Kind"] = gpbo.Kind
	}
	return json.Marshal(objectMap)
}

// AsCheckExistsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for GetPropertyBatchOperation.
func (gpbo GetPropertyBatchOperation) AsCheckExistsPropertyBatchOperation() (*CheckExistsPropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckSequencePropertyBatchOperation is the BasicPropertyBatchOperation implementation for GetPropertyBatchOperation.
func (gpbo GetPropertyBatchOperation) AsCheckSequencePropertyBatchOperation() (*CheckSequencePropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckValuePropertyBatchOperation is the BasicPropertyBatchOperation implementation for GetPropertyBatchOperation.
func (gpbo GetPropertyBatchOperation) AsCheckValuePropertyBatchOperation() (*CheckValuePropertyBatchOperation, bool) {
	return nil, false
}

// AsDeletePropertyBatchOperation is the BasicPropertyBatchOperation implementation for GetPropertyBatchOperation.
func (gpbo GetPropertyBatchOperation) AsDeletePropertyBatchOperation() (*DeletePropertyBatchOperation, bool) {
	return nil, false
}

// AsGetPropertyBatchOperation is the BasicPropertyBatchOperation implementation for GetPropertyBatchOperation.
func (gpbo GetPropertyBatchOperation) AsGetPropertyBatchOperation() (*GetPropertyBatchOperation, bool) {
	return &gpbo, true
}

// AsPutPropertyBatchOperation is the BasicPropertyBatchOperation implementation for GetPropertyBatchOperation.
func (gpbo GetPropertyBatchOperation) AsPutPropertyBatchOperation() (*PutPropertyBatchOperation, bool) {
	return nil, false
}

// AsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for GetPropertyBatchOperation.
func (gpbo GetPropertyBatchOperation) AsPropertyBatchOperation() (*PropertyBatchOperation, bool) {
	return nil, false
}

// AsBasicPropertyBatchOperation is the BasicPropertyBatchOperation implementation for GetPropertyBatchOperation.
func (gpbo GetPropertyBatchOperation) AsBasicPropertyBatchOperation() (BasicPropertyBatchOperation, bool) {
	return &gpbo, true
}

// GUIDPropertyValue describes a Service Fabric property value of type Guid.
type GUIDPropertyValue struct {
	// Data - The data of the property value.
	Data *uuid.UUID `json:"Data,omitempty"`
	// Kind - Possible values include: 'KindPropertyValue', 'KindBinary', 'KindInt64', 'KindDouble', 'KindString', 'KindGUID'
	Kind KindBasicPropertyValue `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for GUIDPropertyValue.
func (gpv GUIDPropertyValue) MarshalJSON() ([]byte, error) {
	gpv.Kind = KindGUID
	objectMap := make(map[string]interface{})
	if gpv.Data != nil {
		objectMap["Data"] = gpv.Data
	}
	if gpv.Kind != "" {
		objectMap["Kind"] = gpv.Kind
	}
	return json.Marshal(objectMap)
}

// AsBinaryPropertyValue is the BasicPropertyValue implementation for GUIDPropertyValue.
func (gpv GUIDPropertyValue) AsBinaryPropertyValue() (*BinaryPropertyValue, bool) {
	return nil, false
}

// AsInt64PropertyValue is the BasicPropertyValue implementation for GUIDPropertyValue.
func (gpv GUIDPropertyValue) AsInt64PropertyValue() (*Int64PropertyValue, bool) {
	return nil, false
}

// AsDoublePropertyValue is the BasicPropertyValue implementation for GUIDPropertyValue.
func (gpv GUIDPropertyValue) AsDoublePropertyValue() (*DoublePropertyValue, bool) {
	return nil, false
}

// AsStringPropertyValue is the BasicPropertyValue implementation for GUIDPropertyValue.
func (gpv GUIDPropertyValue) AsStringPropertyValue() (*StringPropertyValue, bool) {
	return nil, false
}

// AsGUIDPropertyValue is the BasicPropertyValue implementation for GUIDPropertyValue.
func (gpv GUIDPropertyValue) AsGUIDPropertyValue() (*GUIDPropertyValue, bool) {
	return &gpv, true
}

// AsPropertyValue is the BasicPropertyValue implementation for GUIDPropertyValue.
func (gpv GUIDPropertyValue) AsPropertyValue() (*PropertyValue, bool) {
	return nil, false
}

// AsBasicPropertyValue is the BasicPropertyValue implementation for GUIDPropertyValue.
func (gpv GUIDPropertyValue) AsBasicPropertyValue() (BasicPropertyValue, bool) {
	return &gpv, true
}

// BasicHealthEvaluation represents a health evaluation which describes the data and the algorithm used by health
// manager to evaluate the health of an entity.
type BasicHealthEvaluation interface {
	AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool)
	AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool)
	AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool)
	AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool)
	AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool)
	AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool)
	AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool)
	AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool)
	AsEventHealthEvaluation() (*EventHealthEvaluation, bool)
	AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool)
	AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool)
	AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool)
	AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool)
	AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool)
	AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool)
	AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool)
	AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool)
	AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool)
	AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool)
	AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool)
	AsHealthEvaluation() (*HealthEvaluation, bool)
}

// HealthEvaluation represents a health evaluation which describes the data and the algorithm used by health
// manager to evaluate the health of an entity.
type HealthEvaluation struct {
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

func unmarshalBasicHealthEvaluation(body []byte) (BasicHealthEvaluation, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindApplication):
		var ahe ApplicationHealthEvaluation
		err := json.Unmarshal(body, &ahe)
		return ahe, err
	case string(KindApplications):
		var ahe ApplicationsHealthEvaluation
		err := json.Unmarshal(body, &ahe)
		return ahe, err
	case string(KindApplicationTypeApplications):
		var atahe ApplicationTypeApplicationsHealthEvaluation
		err := json.Unmarshal(body, &atahe)
		return atahe, err
	case string(KindDeltaNodesCheck):
		var dnche DeltaNodesCheckHealthEvaluation
		err := json.Unmarshal(body, &dnche)
		return dnche, err
	case string(KindDeployedApplication):
		var dahe DeployedApplicationHealthEvaluation
		err := json.Unmarshal(body, &dahe)
		return dahe, err
	case string(KindDeployedApplications):
		var dahe DeployedApplicationsHealthEvaluation
		err := json.Unmarshal(body, &dahe)
		return dahe, err
	case string(KindDeployedServicePackage):
		var dsphe DeployedServicePackageHealthEvaluation
		err := json.Unmarshal(body, &dsphe)
		return dsphe, err
	case string(KindDeployedServicePackages):
		var dsphe DeployedServicePackagesHealthEvaluation
		err := json.Unmarshal(body, &dsphe)
		return dsphe, err
	case string(KindEvent):
		var ehe EventHealthEvaluation
		err := json.Unmarshal(body, &ehe)
		return ehe, err
	case string(KindNode):
		var nhe NodeHealthEvaluation
		err := json.Unmarshal(body, &nhe)
		return nhe, err
	case string(KindNodes):
		var nhe NodesHealthEvaluation
		err := json.Unmarshal(body, &nhe)
		return nhe, err
	case string(KindPartition):
		var phe PartitionHealthEvaluation
		err := json.Unmarshal(body, &phe)
		return phe, err
	case string(KindPartitions):
		var phe PartitionsHealthEvaluation
		err := json.Unmarshal(body, &phe)
		return phe, err
	case string(KindReplica):
		var rhe ReplicaHealthEvaluation
		err := json.Unmarshal(body, &rhe)
		return rhe, err
	case string(KindReplicas):
		var rhe ReplicasHealthEvaluation
		err := json.Unmarshal(body, &rhe)
		return rhe, err
	case string(KindService):
		var she ServiceHealthEvaluation
		err := json.Unmarshal(body, &she)
		return she, err
	case string(KindServices):
		var she ServicesHealthEvaluation
		err := json.Unmarshal(body, &she)
		return she, err
	case string(KindSystemApplication):
		var sahe SystemApplicationHealthEvaluation
		err := json.Unmarshal(body, &sahe)
		return sahe, err
	case string(KindUpgradeDomainDeltaNodesCheck):
		var uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation
		err := json.Unmarshal(body, &uddnche)
		return uddnche, err
	case string(KindUpgradeDomainNodes):
		var udnhe UpgradeDomainNodesHealthEvaluation
		err := json.Unmarshal(body, &udnhe)
		return udnhe, err
	default:
		var he HealthEvaluation
		err := json.Unmarshal(body, &he)
		return he, err
	}
}
func unmarshalBasicHealthEvaluationArray(body []byte) ([]BasicHealthEvaluation, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	heArray := make([]BasicHealthEvaluation, len(rawMessages))

	for index, rawMessage := range rawMessages {
		he, err := unmarshalBasicHealthEvaluation(*rawMessage)
		if err != nil {
			return nil, err
		}
		heArray[index] = he
	}
	return heArray, nil
}

// MarshalJSON is the custom marshaler for HealthEvaluation.
func (he HealthEvaluation) MarshalJSON() ([]byte, error) {
	he.Kind = KindHealthEvaluation
	objectMap := make(map[string]interface{})
	if he.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = he.AggregatedHealthState
	}
	if he.Description != nil {
		objectMap["Description"] = he.Description
	}
	if he.Kind != "" {
		objectMap["Kind"] = he.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return &he, true
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for HealthEvaluation.
func (he HealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &he, true
}

// HealthEvaluationWrapper wrapper object for health evaluation.
type HealthEvaluationWrapper struct {
	// HealthEvaluation - Represents a health evaluation which describes the data and the algorithm used by health manager to evaluate the health of an entity.
	HealthEvaluation BasicHealthEvaluation `json:"HealthEvaluation,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for HealthEvaluationWrapper struct.
func (hew *HealthEvaluationWrapper) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "HealthEvaluation":
			if v != nil {
				healthEvaluation, err := unmarshalBasicHealthEvaluation(*v)
				if err != nil {
					return err
				}
				hew.HealthEvaluation = healthEvaluation
			}
		}
	}

	return nil
}

// HealthEvent represents health information reported on a health entity, such as cluster, application or
// node, with additional metadata added by the Health Manager.
type HealthEvent struct {
	// IsExpired - Returns true if the health event is expired, otherwise false.
	IsExpired *bool `json:"IsExpired,omitempty"`
	// SourceUtcTimestamp - The date and time when the health report was sent by the source.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// LastModifiedUtcTimestamp - The date and time when the health report was last modified by the health store.
	LastModifiedUtcTimestamp *date.Time `json:"LastModifiedUtcTimestamp,omitempty"`
	// LastOkTransitionAt - If the current health state is 'Ok', this property returns the time at which the health report was first reported with 'Ok'.
	// For periodic reporting, many reports with the same state may have been generated.
	// This property returns the date and time when the first 'Ok' health report was received.
	// If the current health state is 'Error' or 'Warning', returns the date and time at which the health state was last in 'Ok', before transitioning to a different state.
	// If the health state was never 'Ok', the value will be zero date-time.
	LastOkTransitionAt *date.Time `json:"LastOkTransitionAt,omitempty"`
	// LastWarningTransitionAt - If the current health state is 'Warning', this property returns the time at which the health report was first reported with 'Warning'. For periodic reporting, many reports with the same state may have been generated however, this property returns only the date and time at the first 'Warning' health report was received.
	// If the current health state is 'Ok' or 'Error', returns the date and time at which the health state was last in 'Warning', before transitioning to a different state.
	// If the health state was never 'Warning', the value will be zero date-time.
	LastWarningTransitionAt *date.Time `json:"LastWarningTransitionAt,omitempty"`
	// LastErrorTransitionAt - If the current health state is 'Error', this property returns the time at which the health report was first reported with 'Error'. For periodic reporting, many reports with the same state may have been generated however, this property returns only the date and time at the first 'Error' health report was received.
	// If the current health state is 'Ok' or 'Warning', returns the date and time at which the health state was last in 'Error', before transitioning to a different state.
	// If the health state was never 'Error', the value will be zero date-time.
	LastErrorTransitionAt *date.Time `json:"LastErrorTransitionAt,omitempty"`
	// SourceID - The source name that identifies the client/watchdog/system component that generated the health information.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - The property of the health information. An entity can have health reports for different properties.
	// The property is a string and not a fixed enumeration to allow the reporter flexibility to categorize the state condition that triggers the report.
	// For example, a reporter with SourceId "LocalWatchdog" can monitor the state of the available disk on a node,
	// so it can report "AvailableDisk" property on that node.
	// The same reporter can monitor the node connectivity, so it can report a property "Connectivity" on the same node.
	// In the health store, these reports are treated as separate health events for the specified node.
	// Together with the SourceId, the property uniquely identifies the health information.
	Property *string `json:"Property,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// TimeToLiveInMilliSeconds - The duration for which this health report is valid. This field uses ISO8601 format for specifying the duration.
	// When clients report periodically, they should send reports with higher frequency than time to live.
	// If clients report on transition, they can set the time to live to infinite.
	// When time to live expires, the health event that contains the health information
	// is either removed from health store, if RemoveWhenExpired is true, or evaluated at error, if RemoveWhenExpired false.
	// If not specified, time to live defaults to infinite value.
	TimeToLiveInMilliSeconds *string `json:"TimeToLiveInMilliSeconds,omitempty"`
	// Description - The description of the health information. It represents free text used to add human readable information about the report.
	// The maximum string length for the description is 4096 characters.
	// If the provided string is longer, it will be automatically truncated.
	// When truncated, the last characters of the description contain a marker "[Truncated]", and total string size is 4096 characters.
	// The presence of the marker indicates to users that truncation occurred.
	// Note that when truncated, the description has less than 4096 characters from the original string.
	Description *string `json:"Description,omitempty"`
	// SequenceNumber - The sequence number for this health report as a numeric string.
	// The report sequence number is used by the health store to detect stale reports.
	// If not specified, a sequence number is auto-generated by the health client when a report is added.
	SequenceNumber *string `json:"SequenceNumber,omitempty"`
	// RemoveWhenExpired - Value that indicates whether the report is removed from health store when it expires.
	// If set to true, the report is removed from the health store after it expires.
	// If set to false, the report is treated as an error when expired. The value of this property is false by default.
	// When clients report periodically, they should set RemoveWhenExpired false (default).
	// This way, if the reporter has issues (e.g. deadlock) and can't report, the entity is evaluated at error when the health report expires.
	// This flags the entity as being in Error health state.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
}

// HealthInformation represents common health report information. It is included in all health reports sent
// to health store and in all health events returned by health queries.
type HealthInformation struct {
	// SourceID - The source name that identifies the client/watchdog/system component that generated the health information.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - The property of the health information. An entity can have health reports for different properties.
	// The property is a string and not a fixed enumeration to allow the reporter flexibility to categorize the state condition that triggers the report.
	// For example, a reporter with SourceId "LocalWatchdog" can monitor the state of the available disk on a node,
	// so it can report "AvailableDisk" property on that node.
	// The same reporter can monitor the node connectivity, so it can report a property "Connectivity" on the same node.
	// In the health store, these reports are treated as separate health events for the specified node.
	// Together with the SourceId, the property uniquely identifies the health information.
	Property *string `json:"Property,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// TimeToLiveInMilliSeconds - The duration for which this health report is valid. This field uses ISO8601 format for specifying the duration.
	// When clients report periodically, they should send reports with higher frequency than time to live.
	// If clients report on transition, they can set the time to live to infinite.
	// When time to live expires, the health event that contains the health information
	// is either removed from health store, if RemoveWhenExpired is true, or evaluated at error, if RemoveWhenExpired false.
	// If not specified, time to live defaults to infinite value.
	TimeToLiveInMilliSeconds *string `json:"TimeToLiveInMilliSeconds,omitempty"`
	// Description - The description of the health information. It represents free text used to add human readable information about the report.
	// The maximum string length for the description is 4096 characters.
	// If the provided string is longer, it will be automatically truncated.
	// When truncated, the last characters of the description contain a marker "[Truncated]", and total string size is 4096 characters.
	// The presence of the marker indicates to users that truncation occurred.
	// Note that when truncated, the description has less than 4096 characters from the original string.
	Description *string `json:"Description,omitempty"`
	// SequenceNumber - The sequence number for this health report as a numeric string.
	// The report sequence number is used by the health store to detect stale reports.
	// If not specified, a sequence number is auto-generated by the health client when a report is added.
	SequenceNumber *string `json:"SequenceNumber,omitempty"`
	// RemoveWhenExpired - Value that indicates whether the report is removed from health store when it expires.
	// If set to true, the report is removed from the health store after it expires.
	// If set to false, the report is treated as an error when expired. The value of this property is false by default.
	// When clients report periodically, they should set RemoveWhenExpired false (default).
	// This way, if the reporter has issues (e.g. deadlock) and can't report, the entity is evaluated at error when the health report expires.
	// This flags the entity as being in Error health state.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
}

// HealthStateCount represents information about how many health entities are in Ok, Warning and Error
// health state.
type HealthStateCount struct {
	// OkCount - The number of health entities with aggregated health state Ok.
	OkCount *int64 `json:"OkCount,omitempty"`
	// WarningCount - The number of health entities with aggregated health state Warning.
	WarningCount *int64 `json:"WarningCount,omitempty"`
	// ErrorCount - The number of health entities with aggregated health state Error.
	ErrorCount *int64 `json:"ErrorCount,omitempty"`
}

// HealthStatistics the health statistics of an entity, returned as part of the health query result when
// the query description is configured to include statistics.
// The statistics include health state counts for all children types of the current entity.
// For example, for cluster, the health statistics include health state counts for nodes, applications,
// services, partitions, replicas, deployed applications and deployed service packages.
// For partition, the health statistics include health counts for replicas.
type HealthStatistics struct {
	// HealthStateCountList - List of health state counts per entity kind, which keeps track of how many children of the queried entity are in Ok, Warning and Error state.
	HealthStateCountList *[]EntityKindHealthStateCount `json:"HealthStateCountList,omitempty"`
}

// HTTPConfig describes the http configuration for external connectivity for this network.
type HTTPConfig struct {
	// Name - http gateway config name.
	Name *string `json:"name,omitempty"`
	// Port - Specifies the port at which the service endpoint below needs to be exposed.
	Port *int32 `json:"port,omitempty"`
	// Hosts - description for routing.
	Hosts *[]HTTPHostConfig `json:"hosts,omitempty"`
}

// HTTPHostConfig describes the hostname properties for http routing.
type HTTPHostConfig struct {
	// Name - http hostname config name.
	Name *string `json:"name,omitempty"`
	// Routes - Route information to use for routing. Routes are processed in the order they are specified. Specify routes that are more specific before routes that can handle general cases.
	Routes *[]HTTPRouteConfig `json:"routes,omitempty"`
}

// HTTPRouteConfig describes the hostname properties for http routing.
type HTTPRouteConfig struct {
	// Name - http route name.
	Name *string `json:"name,omitempty"`
	// Match - Describes a rule for http route matching.
	Match *HTTPRouteMatchRule `json:"match,omitempty"`
	// Destination - Describes destination endpoint for routing traffic.
	Destination *GatewayDestination `json:"destination,omitempty"`
}

// HTTPRouteMatchHeader describes header information for http route matching.
type HTTPRouteMatchHeader struct {
	// Name - Name of header to match in request.
	Name *string `json:"name,omitempty"`
	// Value - Value of header to match in request.
	Value *string `json:"value,omitempty"`
	// Type - how to match header value. Possible values include: 'Exact'
	Type HeaderMatchType `json:"type,omitempty"`
}

// HTTPRouteMatchPath path to match for routing.
type HTTPRouteMatchPath struct {
	// Value - Uri path to match for request.
	Value *string `json:"value,omitempty"`
	// Rewrite - replacement string for matched part of the Uri.
	Rewrite *string `json:"rewrite,omitempty"`
	// Type - how to match value in the Uri
	Type *string `json:"type,omitempty"`
}

// HTTPRouteMatchRule describes a rule for http route matching.
type HTTPRouteMatchRule struct {
	// Path - Path to match for routing.
	Path *HTTPRouteMatchPath `json:"path,omitempty"`
	// Headers - headers and their values to match in request.
	Headers *[]HTTPRouteMatchHeader `json:"headers,omitempty"`
}

// IdentityDescription information describing the identities associated with this application.
type IdentityDescription struct {
	// TokenServiceEndpoint - the endpoint for the token service managing this identity
	TokenServiceEndpoint *string `json:"tokenServiceEndpoint,omitempty"`
	// Type - the types of identities associated with this resource; currently restricted to 'SystemAssigned and UserAssigned'
	Type *string `json:"type,omitempty"`
	// TenantID - the identifier of the tenant containing the application's identity.
	TenantID *string `json:"tenantId,omitempty"`
	// PrincipalID - the object identifier of the Service Principal of the identity associated with this resource.
	PrincipalID *string `json:"principalId,omitempty"`
	// UserAssignedIdentities - represents user assigned identities map.
	UserAssignedIdentities map[string]*IdentityItemDescription `json:"userAssignedIdentities"`
}

// MarshalJSON is the custom marshaler for IdentityDescription.
func (ID IdentityDescription) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if ID.TokenServiceEndpoint != nil {
		objectMap["tokenServiceEndpoint"] = ID.TokenServiceEndpoint
	}
	if ID.Type != nil {
		objectMap["type"] = ID.Type
	}
	if ID.TenantID != nil {
		objectMap["tenantId"] = ID.TenantID
	}
	if ID.PrincipalID != nil {
		objectMap["principalId"] = ID.PrincipalID
	}
	if ID.UserAssignedIdentities != nil {
		objectMap["userAssignedIdentities"] = ID.UserAssignedIdentities
	}
	return json.Marshal(objectMap)
}

// IdentityItemDescription describes a single user-assigned identity associated with the application.
type IdentityItemDescription struct {
	// PrincipalID - the object identifier of the Service Principal which this identity represents.
	PrincipalID *string `json:"principalId,omitempty"`
	// ClientID - the client identifier of the Service Principal which this identity represents.
	ClientID *string `json:"clientId,omitempty"`
}

// ImageRegistryCredential image registry credential.
type ImageRegistryCredential struct {
	// Server - Docker image registry server, without protocol such as `http` and `https`.
	Server *string `json:"server,omitempty"`
	// Username - The username for the private registry.
	Username *string `json:"username,omitempty"`
	// Password - The password for the private registry. The password is required for create or update operations, however it is not returned in the get or list operations.
	Password *string `json:"password,omitempty"`
}

// ImageStoreContent information about the image store content.
type ImageStoreContent struct {
	autorest.Response `json:"-"`
	// StoreFiles - The list of image store file info objects represents files found under the given image store relative path.
	StoreFiles *[]FileInfo `json:"StoreFiles,omitempty"`
	// StoreFolders - The list of image store folder info objects represents subfolders found under the given image store relative path.
	StoreFolders *[]FolderInfo `json:"StoreFolders,omitempty"`
}

// ImageStoreCopyDescription information about how to copy image store content from one image store
// relative path to another image store relative path.
type ImageStoreCopyDescription struct {
	// RemoteSource - The relative path of source image store content to be copied from.
	RemoteSource *string `json:"RemoteSource,omitempty"`
	// RemoteDestination - The relative path of destination image store content to be copied to.
	RemoteDestination *string `json:"RemoteDestination,omitempty"`
	// SkipFiles - The list of the file names to be skipped for copying.
	SkipFiles *[]string `json:"SkipFiles,omitempty"`
	// CheckMarkFile - Indicates whether to check mark file during copying. The property is true if checking mark file is required, false otherwise. The mark file is used to check whether the folder is well constructed. If the property is true and mark file does not exist, the copy is skipped.
	CheckMarkFile *bool `json:"CheckMarkFile,omitempty"`
}

// InlinedValueSecretResourceProperties describes the properties of a secret resource whose value is
// provided explicitly as plaintext. The secret resource may have multiple values, each being uniquely
// versioned. The secret value of each version is stored encrypted, and delivered as plaintext into the
// context of applications referencing it.
type InlinedValueSecretResourceProperties struct {
	// Description - User readable description of the secret.
	Description *string `json:"description,omitempty"`
	// Status - READ-ONLY; Status of the resource. Possible values include: 'ResourceStatusUnknown', 'ResourceStatusReady', 'ResourceStatusUpgrading', 'ResourceStatusCreating', 'ResourceStatusDeleting', 'ResourceStatusFailed'
	Status ResourceStatus `json:"status,omitempty"`
	// StatusDetails - READ-ONLY; Gives additional information about the current status of the secret.
	StatusDetails *string `json:"statusDetails,omitempty"`
	// ContentType - The type of the content stored in the secret value. The value of this property is opaque to Service Fabric. Once set, the value of this property cannot be changed.
	ContentType *string `json:"contentType,omitempty"`
	// Kind - Possible values include: 'KindSecretResourcePropertiesBase', 'KindSecretResourceProperties', 'KindInlinedValue'
	Kind KindBasicSecretResourcePropertiesBase `json:"kind,omitempty"`
}

// MarshalJSON is the custom marshaler for InlinedValueSecretResourceProperties.
func (ivsrp InlinedValueSecretResourceProperties) MarshalJSON() ([]byte, error) {
	ivsrp.Kind = KindInlinedValue
	objectMap := make(map[string]interface{})
	if ivsrp.Description != nil {
		objectMap["description"] = ivsrp.Description
	}
	if ivsrp.ContentType != nil {
		objectMap["contentType"] = ivsrp.ContentType
	}
	if ivsrp.Kind != "" {
		objectMap["kind"] = ivsrp.Kind
	}
	return json.Marshal(objectMap)
}

// AsSecretResourceProperties is the BasicSecretResourcePropertiesBase implementation for InlinedValueSecretResourceProperties.
func (ivsrp InlinedValueSecretResourceProperties) AsSecretResourceProperties() (*SecretResourceProperties, bool) {
	return nil, false
}

// AsBasicSecretResourceProperties is the BasicSecretResourcePropertiesBase implementation for InlinedValueSecretResourceProperties.
func (ivsrp InlinedValueSecretResourceProperties) AsBasicSecretResourceProperties() (BasicSecretResourceProperties, bool) {
	return &ivsrp, true
}

// AsInlinedValueSecretResourceProperties is the BasicSecretResourcePropertiesBase implementation for InlinedValueSecretResourceProperties.
func (ivsrp InlinedValueSecretResourceProperties) AsInlinedValueSecretResourceProperties() (*InlinedValueSecretResourceProperties, bool) {
	return &ivsrp, true
}

// AsSecretResourcePropertiesBase is the BasicSecretResourcePropertiesBase implementation for InlinedValueSecretResourceProperties.
func (ivsrp InlinedValueSecretResourceProperties) AsSecretResourcePropertiesBase() (*SecretResourcePropertiesBase, bool) {
	return nil, false
}

// AsBasicSecretResourcePropertiesBase is the BasicSecretResourcePropertiesBase implementation for InlinedValueSecretResourceProperties.
func (ivsrp InlinedValueSecretResourceProperties) AsBasicSecretResourcePropertiesBase() (BasicSecretResourcePropertiesBase, bool) {
	return &ivsrp, true
}

// Int64PropertyValue describes a Service Fabric property value of type Int64.
type Int64PropertyValue struct {
	// Data - The data of the property value.
	Data *string `json:"Data,omitempty"`
	// Kind - Possible values include: 'KindPropertyValue', 'KindBinary', 'KindInt64', 'KindDouble', 'KindString', 'KindGUID'
	Kind KindBasicPropertyValue `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for Int64PropertyValue.
func (i6pv Int64PropertyValue) MarshalJSON() ([]byte, error) {
	i6pv.Kind = KindInt64
	objectMap := make(map[string]interface{})
	if i6pv.Data != nil {
		objectMap["Data"] = i6pv.Data
	}
	if i6pv.Kind != "" {
		objectMap["Kind"] = i6pv.Kind
	}
	return json.Marshal(objectMap)
}

// AsBinaryPropertyValue is the BasicPropertyValue implementation for Int64PropertyValue.
func (i6pv Int64PropertyValue) AsBinaryPropertyValue() (*BinaryPropertyValue, bool) {
	return nil, false
}

// AsInt64PropertyValue is the BasicPropertyValue implementation for Int64PropertyValue.
func (i6pv Int64PropertyValue) AsInt64PropertyValue() (*Int64PropertyValue, bool) {
	return &i6pv, true
}

// AsDoublePropertyValue is the BasicPropertyValue implementation for Int64PropertyValue.
func (i6pv Int64PropertyValue) AsDoublePropertyValue() (*DoublePropertyValue, bool) {
	return nil, false
}

// AsStringPropertyValue is the BasicPropertyValue implementation for Int64PropertyValue.
func (i6pv Int64PropertyValue) AsStringPropertyValue() (*StringPropertyValue, bool) {
	return nil, false
}

// AsGUIDPropertyValue is the BasicPropertyValue implementation for Int64PropertyValue.
func (i6pv Int64PropertyValue) AsGUIDPropertyValue() (*GUIDPropertyValue, bool) {
	return nil, false
}

// AsPropertyValue is the BasicPropertyValue implementation for Int64PropertyValue.
func (i6pv Int64PropertyValue) AsPropertyValue() (*PropertyValue, bool) {
	return nil, false
}

// AsBasicPropertyValue is the BasicPropertyValue implementation for Int64PropertyValue.
func (i6pv Int64PropertyValue) AsBasicPropertyValue() (BasicPropertyValue, bool) {
	return &i6pv, true
}

// Int64RangePartitionInformation describes the partition information for the integer range that is based
// on partition schemes.
type Int64RangePartitionInformation struct {
	// LowKey - Specifies the minimum key value handled by this partition.
	LowKey *string `json:"LowKey,omitempty"`
	// HighKey - Specifies the maximum key value handled by this partition.
	HighKey *string `json:"HighKey,omitempty"`
	// ID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	ID *uuid.UUID `json:"Id,omitempty"`
	// ServicePartitionKind - Possible values include: 'ServicePartitionKindPartitionInformation', 'ServicePartitionKindInt64Range1', 'ServicePartitionKindNamed1', 'ServicePartitionKindSingleton1'
	ServicePartitionKind ServicePartitionKindBasicPartitionInformation `json:"ServicePartitionKind,omitempty"`
}

// MarshalJSON is the custom marshaler for Int64RangePartitionInformation.
func (i6rpi Int64RangePartitionInformation) MarshalJSON() ([]byte, error) {
	i6rpi.ServicePartitionKind = ServicePartitionKindInt64Range1
	objectMap := make(map[string]interface{})
	if i6rpi.LowKey != nil {
		objectMap["LowKey"] = i6rpi.LowKey
	}
	if i6rpi.HighKey != nil {
		objectMap["HighKey"] = i6rpi.HighKey
	}
	if i6rpi.ID != nil {
		objectMap["Id"] = i6rpi.ID
	}
	if i6rpi.ServicePartitionKind != "" {
		objectMap["ServicePartitionKind"] = i6rpi.ServicePartitionKind
	}
	return json.Marshal(objectMap)
}

// AsInt64RangePartitionInformation is the BasicPartitionInformation implementation for Int64RangePartitionInformation.
func (i6rpi Int64RangePartitionInformation) AsInt64RangePartitionInformation() (*Int64RangePartitionInformation, bool) {
	return &i6rpi, true
}

// AsNamedPartitionInformation is the BasicPartitionInformation implementation for Int64RangePartitionInformation.
func (i6rpi Int64RangePartitionInformation) AsNamedPartitionInformation() (*NamedPartitionInformation, bool) {
	return nil, false
}

// AsSingletonPartitionInformation is the BasicPartitionInformation implementation for Int64RangePartitionInformation.
func (i6rpi Int64RangePartitionInformation) AsSingletonPartitionInformation() (*SingletonPartitionInformation, bool) {
	return nil, false
}

// AsPartitionInformation is the BasicPartitionInformation implementation for Int64RangePartitionInformation.
func (i6rpi Int64RangePartitionInformation) AsPartitionInformation() (*PartitionInformation, bool) {
	return nil, false
}

// AsBasicPartitionInformation is the BasicPartitionInformation implementation for Int64RangePartitionInformation.
func (i6rpi Int64RangePartitionInformation) AsBasicPartitionInformation() (BasicPartitionInformation, bool) {
	return &i6rpi, true
}

// InvokeDataLossResult represents information about an operation in a terminal state (Completed or
// Faulted).
type InvokeDataLossResult struct {
	// ErrorCode - If OperationState is Completed, this is 0.  If OperationState is Faulted, this is an error code indicating the reason.
	ErrorCode *int32 `json:"ErrorCode,omitempty"`
	// SelectedPartition - This class returns information about the partition that the user-induced operation acted upon.
	SelectedPartition *SelectedPartition `json:"SelectedPartition,omitempty"`
}

// InvokeQuorumLossResult represents information about an operation in a terminal state (Completed or
// Faulted).
type InvokeQuorumLossResult struct {
	// ErrorCode - If OperationState is Completed, this is 0.  If OperationState is Faulted, this is an error code indicating the reason.
	ErrorCode *int32 `json:"ErrorCode,omitempty"`
	// SelectedPartition - This class returns information about the partition that the user-induced operation acted upon.
	SelectedPartition *SelectedPartition `json:"SelectedPartition,omitempty"`
}

// KeyValueStoreReplicaStatus key value store related information for the replica.
type KeyValueStoreReplicaStatus struct {
	// DatabaseRowCountEstimate - Value indicating the estimated number of rows in the underlying database.
	DatabaseRowCountEstimate *string `json:"DatabaseRowCountEstimate,omitempty"`
	// DatabaseLogicalSizeEstimate - Value indicating the estimated size of the underlying database.
	DatabaseLogicalSizeEstimate *string `json:"DatabaseLogicalSizeEstimate,omitempty"`
	// CopyNotificationCurrentKeyFilter - Value indicating the latest key-prefix filter applied to enumeration during the callback. Null if there is no pending callback.
	CopyNotificationCurrentKeyFilter *string `json:"CopyNotificationCurrentKeyFilter,omitempty"`
	// CopyNotificationCurrentProgress - Value indicating the latest number of keys enumerated during the callback. 0 if there is no pending callback.
	CopyNotificationCurrentProgress *string `json:"CopyNotificationCurrentProgress,omitempty"`
	// StatusDetails - Value indicating the current status details of the replica.
	StatusDetails *string `json:"StatusDetails,omitempty"`
	// Kind - Possible values include: 'KindReplicaStatusBase', 'KindKeyValueStore'
	Kind KindBasicReplicaStatusBase `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for KeyValueStoreReplicaStatus.
func (kvsrs KeyValueStoreReplicaStatus) MarshalJSON() ([]byte, error) {
	kvsrs.Kind = KindKeyValueStore
	objectMap := make(map[string]interface{})
	if kvsrs.DatabaseRowCountEstimate != nil {
		objectMap["DatabaseRowCountEstimate"] = kvsrs.DatabaseRowCountEstimate
	}
	if kvsrs.DatabaseLogicalSizeEstimate != nil {
		objectMap["DatabaseLogicalSizeEstimate"] = kvsrs.DatabaseLogicalSizeEstimate
	}
	if kvsrs.CopyNotificationCurrentKeyFilter != nil {
		objectMap["CopyNotificationCurrentKeyFilter"] = kvsrs.CopyNotificationCurrentKeyFilter
	}
	if kvsrs.CopyNotificationCurrentProgress != nil {
		objectMap["CopyNotificationCurrentProgress"] = kvsrs.CopyNotificationCurrentProgress
	}
	if kvsrs.StatusDetails != nil {
		objectMap["StatusDetails"] = kvsrs.StatusDetails
	}
	if kvsrs.Kind != "" {
		objectMap["Kind"] = kvsrs.Kind
	}
	return json.Marshal(objectMap)
}

// AsKeyValueStoreReplicaStatus is the BasicReplicaStatusBase implementation for KeyValueStoreReplicaStatus.
func (kvsrs KeyValueStoreReplicaStatus) AsKeyValueStoreReplicaStatus() (*KeyValueStoreReplicaStatus, bool) {
	return &kvsrs, true
}

// AsReplicaStatusBase is the BasicReplicaStatusBase implementation for KeyValueStoreReplicaStatus.
func (kvsrs KeyValueStoreReplicaStatus) AsReplicaStatusBase() (*ReplicaStatusBase, bool) {
	return nil, false
}

// AsBasicReplicaStatusBase is the BasicReplicaStatusBase implementation for KeyValueStoreReplicaStatus.
func (kvsrs KeyValueStoreReplicaStatus) AsBasicReplicaStatusBase() (BasicReplicaStatusBase, bool) {
	return &kvsrs, true
}

// ListApplicationEvent ...
type ListApplicationEvent struct {
	autorest.Response `json:"-"`
	Value             *[]BasicApplicationEvent `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ListApplicationEvent struct.
func (lae *ListApplicationEvent) UnmarshalJSON(body []byte) error {
	ae, err := unmarshalBasicApplicationEventArray(body)
	if err != nil {
		return err
	}
	lae.Value = &ae

	return nil
}

// ListClusterEvent ...
type ListClusterEvent struct {
	autorest.Response `json:"-"`
	Value             *[]BasicClusterEvent `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ListClusterEvent struct.
func (lce *ListClusterEvent) UnmarshalJSON(body []byte) error {
	ce, err := unmarshalBasicClusterEventArray(body)
	if err != nil {
		return err
	}
	lce.Value = &ce

	return nil
}

// ListContainerInstanceEvent ...
type ListContainerInstanceEvent struct {
	autorest.Response `json:"-"`
	Value             *[]ContainerInstanceEvent `json:"value,omitempty"`
}

// ListDeployedCodePackageInfo ...
type ListDeployedCodePackageInfo struct {
	autorest.Response `json:"-"`
	Value             *[]DeployedCodePackageInfo `json:"value,omitempty"`
}

// ListDeployedServicePackageInfo ...
type ListDeployedServicePackageInfo struct {
	autorest.Response `json:"-"`
	Value             *[]DeployedServicePackageInfo `json:"value,omitempty"`
}

// ListDeployedServiceReplicaInfo ...
type ListDeployedServiceReplicaInfo struct {
	autorest.Response `json:"-"`
	Value             *[]BasicDeployedServiceReplicaInfo `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ListDeployedServiceReplicaInfo struct.
func (ldsri *ListDeployedServiceReplicaInfo) UnmarshalJSON(body []byte) error {
	dsri, err := unmarshalBasicDeployedServiceReplicaInfoArray(body)
	if err != nil {
		return err
	}
	ldsri.Value = &dsri

	return nil
}

// ListDeployedServiceTypeInfo ...
type ListDeployedServiceTypeInfo struct {
	autorest.Response `json:"-"`
	Value             *[]DeployedServiceTypeInfo `json:"value,omitempty"`
}

// ListFabricCodeVersionInfo ...
type ListFabricCodeVersionInfo struct {
	autorest.Response `json:"-"`
	Value             *[]FabricCodeVersionInfo `json:"value,omitempty"`
}

// ListFabricConfigVersionInfo ...
type ListFabricConfigVersionInfo struct {
	autorest.Response `json:"-"`
	Value             *[]FabricConfigVersionInfo `json:"value,omitempty"`
}

// ListFabricEvent ...
type ListFabricEvent struct {
	autorest.Response `json:"-"`
	Value             *[]BasicFabricEvent `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ListFabricEvent struct.
func (lfe *ListFabricEvent) UnmarshalJSON(body []byte) error {
	fe, err := unmarshalBasicFabricEventArray(body)
	if err != nil {
		return err
	}
	lfe.Value = &fe

	return nil
}

// ListNodeEvent ...
type ListNodeEvent struct {
	autorest.Response `json:"-"`
	Value             *[]BasicNodeEvent `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ListNodeEvent struct.
func (lne *ListNodeEvent) UnmarshalJSON(body []byte) error {
	ne, err := unmarshalBasicNodeEventArray(body)
	if err != nil {
		return err
	}
	lne.Value = &ne

	return nil
}

// ListOperationStatus ...
type ListOperationStatus struct {
	autorest.Response `json:"-"`
	Value             *[]OperationStatus `json:"value,omitempty"`
}

// ListPartitionEvent ...
type ListPartitionEvent struct {
	autorest.Response `json:"-"`
	Value             *[]BasicPartitionEvent `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ListPartitionEvent struct.
func (lpe *ListPartitionEvent) UnmarshalJSON(body []byte) error {
	peVar, err := unmarshalBasicPartitionEventArray(body)
	if err != nil {
		return err
	}
	lpe.Value = &peVar

	return nil
}

// ListRepairTask ...
type ListRepairTask struct {
	autorest.Response `json:"-"`
	Value             *[]RepairTask `json:"value,omitempty"`
}

// ListReplicaEvent ...
type ListReplicaEvent struct {
	autorest.Response `json:"-"`
	Value             *[]BasicReplicaEvent `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ListReplicaEvent struct.
func (lre *ListReplicaEvent) UnmarshalJSON(body []byte) error {
	re, err := unmarshalBasicReplicaEventArray(body)
	if err != nil {
		return err
	}
	lre.Value = &re

	return nil
}

// ListServiceEvent ...
type ListServiceEvent struct {
	autorest.Response `json:"-"`
	Value             *[]BasicServiceEvent `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ListServiceEvent struct.
func (lse *ListServiceEvent) UnmarshalJSON(body []byte) error {
	se, err := unmarshalBasicServiceEventArray(body)
	if err != nil {
		return err
	}
	lse.Value = &se

	return nil
}

// ListServiceTypeInfo ...
type ListServiceTypeInfo struct {
	autorest.Response `json:"-"`
	Value             *[]ServiceTypeInfo `json:"value,omitempty"`
}

// LoadMetricInformation represents data structure that contains load information for a certain metric in a
// cluster.
type LoadMetricInformation struct {
	// Name - Name of the metric for which this load information is provided.
	Name *string `json:"Name,omitempty"`
	// IsBalancedBefore - Value that indicates whether the metrics is balanced or not before resource balancer run
	IsBalancedBefore *bool `json:"IsBalancedBefore,omitempty"`
	// IsBalancedAfter - Value that indicates whether the metrics is balanced or not after resource balancer run.
	IsBalancedAfter *bool `json:"IsBalancedAfter,omitempty"`
	// DeviationBefore - The standard average deviation of the metrics before resource balancer run.
	DeviationBefore *string `json:"DeviationBefore,omitempty"`
	// DeviationAfter - The standard average deviation of the metrics after resource balancer run.
	DeviationAfter *string `json:"DeviationAfter,omitempty"`
	// BalancingThreshold - The balancing threshold for a certain metric.
	BalancingThreshold *string `json:"BalancingThreshold,omitempty"`
	// Action - The current action being taken with regard to this metric
	Action *string `json:"Action,omitempty"`
	// ActivityThreshold - The Activity Threshold specified for this metric in the system Cluster Manifest.
	ActivityThreshold *string `json:"ActivityThreshold,omitempty"`
	// ClusterCapacity - The total cluster capacity for a given metric
	ClusterCapacity *string `json:"ClusterCapacity,omitempty"`
	// ClusterLoad - The total cluster load. In future releases of Service Fabric this parameter will be deprecated in favor of CurrentClusterLoad.
	ClusterLoad *string `json:"ClusterLoad,omitempty"`
	// CurrentClusterLoad - The total cluster load.
	CurrentClusterLoad *string `json:"CurrentClusterLoad,omitempty"`
	// ClusterRemainingCapacity - The remaining capacity for the metric in the cluster. In future releases of Service Fabric this parameter will be deprecated in favor of ClusterCapacityRemaining.
	ClusterRemainingCapacity *string `json:"ClusterRemainingCapacity,omitempty"`
	// ClusterCapacityRemaining - The remaining capacity for the metric in the cluster.
	ClusterCapacityRemaining *string `json:"ClusterCapacityRemaining,omitempty"`
	// IsClusterCapacityViolation - Indicates that the metric is currently over capacity in the cluster.
	IsClusterCapacityViolation *bool `json:"IsClusterCapacityViolation,omitempty"`
	// NodeBufferPercentage - The reserved percentage of total node capacity for this metric.
	NodeBufferPercentage *string `json:"NodeBufferPercentage,omitempty"`
	// ClusterBufferedCapacity - Remaining capacity in the cluster excluding the reserved space. In future releases of Service Fabric this parameter will be deprecated in favor of BufferedClusterCapacityRemaining.
	ClusterBufferedCapacity *string `json:"ClusterBufferedCapacity,omitempty"`
	// BufferedClusterCapacityRemaining - Remaining capacity in the cluster excluding the reserved space.
	BufferedClusterCapacityRemaining *string `json:"BufferedClusterCapacityRemaining,omitempty"`
	// ClusterRemainingBufferedCapacity - The remaining percentage of cluster total capacity for this metric.
	ClusterRemainingBufferedCapacity *string `json:"ClusterRemainingBufferedCapacity,omitempty"`
	// MinNodeLoadValue - The minimum load on any node for this metric. In future releases of Service Fabric this parameter will be deprecated in favor of MinimumNodeLoad.
	MinNodeLoadValue *string `json:"MinNodeLoadValue,omitempty"`
	// MinimumNodeLoad - The minimum load on any node for this metric.
	MinimumNodeLoad *string `json:"MinimumNodeLoad,omitempty"`
	// MinNodeLoadNodeID - The node id of the node with the minimum load for this metric.
	MinNodeLoadNodeID *NodeID `json:"MinNodeLoadNodeId,omitempty"`
	// MaxNodeLoadValue - The maximum load on any node for this metric. In future releases of Service Fabric this parameter will be deprecated in favor of MaximumNodeLoad.
	MaxNodeLoadValue *string `json:"MaxNodeLoadValue,omitempty"`
	// MaximumNodeLoad - The maximum load on any node for this metric.
	MaximumNodeLoad *string `json:"MaximumNodeLoad,omitempty"`
	// MaxNodeLoadNodeID - The node id of the node with the maximum load for this metric.
	MaxNodeLoadNodeID *NodeID `json:"MaxNodeLoadNodeId,omitempty"`
	// PlannedLoadRemoval - This value represents the load of the replicas that are planned to be removed in the future within the cluster.
	// This kind of load is reported for replicas that are currently being moving to other nodes and for replicas that are currently being dropped but still use the load on the source node.
	PlannedLoadRemoval *string `json:"PlannedLoadRemoval,omitempty"`
}

// LoadMetricReport represents the load metric report which contains the time metric was reported, its name
// and value.
type LoadMetricReport struct {
	// LastReportedUtc - Gets the UTC time when the load was reported.
	LastReportedUtc *date.Time `json:"LastReportedUtc,omitempty"`
	// Name - The name of the load metric.
	Name *string `json:"Name,omitempty"`
	// Value - The value of the load metric. In future releases of Service Fabric this parameter will be deprecated in favor of CurrentValue.
	Value *string `json:"Value,omitempty"`
	// CurrentValue - The value of the load metric.
	CurrentValue *string `json:"CurrentValue,omitempty"`
}

// LoadMetricReportInfo information about load reported by replica.
type LoadMetricReportInfo struct {
	// Name - The name of the metric.
	Name *string `json:"Name,omitempty"`
	// Value - The value of the load for the metric. In future releases of Service Fabric this parameter will be deprecated in favor of CurrentValue.
	Value *int32 `json:"Value,omitempty"`
	// CurrentValue - The double value of the load for the metric.
	CurrentValue *string `json:"CurrentValue,omitempty"`
	// LastReportedUtc - The UTC time when the load is reported.
	LastReportedUtc *date.Time `json:"LastReportedUtc,omitempty"`
}

// LocalNetworkResourceProperties information about a Service Fabric container network local to a single
// Service Fabric cluster.
type LocalNetworkResourceProperties struct {
	// NetworkAddressPrefix - Address space for the local container network.
	NetworkAddressPrefix *string `json:"networkAddressPrefix,omitempty"`
	// Description - User readable description of the network.
	Description *string `json:"description,omitempty"`
	// Status - READ-ONLY; Status of the network. Possible values include: 'ResourceStatusUnknown', 'ResourceStatusReady', 'ResourceStatusUpgrading', 'ResourceStatusCreating', 'ResourceStatusDeleting', 'ResourceStatusFailed'
	Status ResourceStatus `json:"status,omitempty"`
	// StatusDetails - READ-ONLY; Gives additional information about the current status of the network.
	StatusDetails *string `json:"statusDetails,omitempty"`
	// Kind - Possible values include: 'KindNetworkResourcePropertiesBase', 'KindNetworkResourceProperties', 'KindLocal'
	Kind KindBasicNetworkResourcePropertiesBase `json:"kind,omitempty"`
}

// MarshalJSON is the custom marshaler for LocalNetworkResourceProperties.
func (lnrp LocalNetworkResourceProperties) MarshalJSON() ([]byte, error) {
	lnrp.Kind = KindLocal
	objectMap := make(map[string]interface{})
	if lnrp.NetworkAddressPrefix != nil {
		objectMap["networkAddressPrefix"] = lnrp.NetworkAddressPrefix
	}
	if lnrp.Description != nil {
		objectMap["description"] = lnrp.Description
	}
	if lnrp.Kind != "" {
		objectMap["kind"] = lnrp.Kind
	}
	return json.Marshal(objectMap)
}

// AsNetworkResourceProperties is the BasicNetworkResourcePropertiesBase implementation for LocalNetworkResourceProperties.
func (lnrp LocalNetworkResourceProperties) AsNetworkResourceProperties() (*NetworkResourceProperties, bool) {
	return nil, false
}

// AsBasicNetworkResourceProperties is the BasicNetworkResourcePropertiesBase implementation for LocalNetworkResourceProperties.
func (lnrp LocalNetworkResourceProperties) AsBasicNetworkResourceProperties() (BasicNetworkResourceProperties, bool) {
	return &lnrp, true
}

// AsLocalNetworkResourceProperties is the BasicNetworkResourcePropertiesBase implementation for LocalNetworkResourceProperties.
func (lnrp LocalNetworkResourceProperties) AsLocalNetworkResourceProperties() (*LocalNetworkResourceProperties, bool) {
	return &lnrp, true
}

// AsNetworkResourcePropertiesBase is the BasicNetworkResourcePropertiesBase implementation for LocalNetworkResourceProperties.
func (lnrp LocalNetworkResourceProperties) AsNetworkResourcePropertiesBase() (*NetworkResourcePropertiesBase, bool) {
	return nil, false
}

// AsBasicNetworkResourcePropertiesBase is the BasicNetworkResourcePropertiesBase implementation for LocalNetworkResourceProperties.
func (lnrp LocalNetworkResourceProperties) AsBasicNetworkResourcePropertiesBase() (BasicNetworkResourcePropertiesBase, bool) {
	return &lnrp, true
}

// ManagedApplicationIdentity describes a managed application identity.
type ManagedApplicationIdentity struct {
	// Name - The name of the identity.
	Name *string `json:"Name,omitempty"`
	// PrincipalID - The identity's PrincipalId.
	PrincipalID *string `json:"PrincipalId,omitempty"`
}

// ManagedApplicationIdentityDescription managed application identity description.
type ManagedApplicationIdentityDescription struct {
	// TokenServiceEndpoint - Token service endpoint.
	TokenServiceEndpoint *string `json:"TokenServiceEndpoint,omitempty"`
	// ManagedIdentities - A list of managed application identity objects.
	ManagedIdentities *[]ManagedApplicationIdentity `json:"ManagedIdentities,omitempty"`
}

// MonitoringPolicyDescription describes the parameters for monitoring an upgrade in Monitored mode.
type MonitoringPolicyDescription struct {
	// FailureAction - The compensating action to perform when a Monitored upgrade encounters monitoring policy or health policy violations.
	// Invalid indicates the failure action is invalid. Rollback specifies that the upgrade will start rolling back automatically.
	// Manual indicates that the upgrade will switch to UnmonitoredManual upgrade mode. Possible values include: 'FailureActionInvalid', 'FailureActionRollback', 'FailureActionManual'
	FailureAction FailureAction `json:"FailureAction,omitempty"`
	// HealthCheckWaitDurationInMilliseconds - The amount of time to wait after completing an upgrade domain before applying health policies. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	HealthCheckWaitDurationInMilliseconds *string `json:"HealthCheckWaitDurationInMilliseconds,omitempty"`
	// HealthCheckStableDurationInMilliseconds - The amount of time that the application or cluster must remain healthy before the upgrade proceeds to the next upgrade domain. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	HealthCheckStableDurationInMilliseconds *string `json:"HealthCheckStableDurationInMilliseconds,omitempty"`
	// HealthCheckRetryTimeoutInMilliseconds - The amount of time to retry health evaluation when the application or cluster is unhealthy before FailureAction is executed. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	HealthCheckRetryTimeoutInMilliseconds *string `json:"HealthCheckRetryTimeoutInMilliseconds,omitempty"`
	// UpgradeTimeoutInMilliseconds - The amount of time the overall upgrade has to complete before FailureAction is executed. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	UpgradeTimeoutInMilliseconds *string `json:"UpgradeTimeoutInMilliseconds,omitempty"`
	// UpgradeDomainTimeoutInMilliseconds - The amount of time each upgrade domain has to complete before FailureAction is executed. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	UpgradeDomainTimeoutInMilliseconds *string `json:"UpgradeDomainTimeoutInMilliseconds,omitempty"`
}

// NameDescription describes a Service Fabric name.
type NameDescription struct {
	// Name - The Service Fabric name, including the 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
}

// NamedPartitionInformation describes the partition information for the name as a string that is based on
// partition schemes.
type NamedPartitionInformation struct {
	// Name - Name of the partition.
	Name *string `json:"Name,omitempty"`
	// ID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	ID *uuid.UUID `json:"Id,omitempty"`
	// ServicePartitionKind - Possible values include: 'ServicePartitionKindPartitionInformation', 'ServicePartitionKindInt64Range1', 'ServicePartitionKindNamed1', 'ServicePartitionKindSingleton1'
	ServicePartitionKind ServicePartitionKindBasicPartitionInformation `json:"ServicePartitionKind,omitempty"`
}

// MarshalJSON is the custom marshaler for NamedPartitionInformation.
func (npi NamedPartitionInformation) MarshalJSON() ([]byte, error) {
	npi.ServicePartitionKind = ServicePartitionKindNamed1
	objectMap := make(map[string]interface{})
	if npi.Name != nil {
		objectMap["Name"] = npi.Name
	}
	if npi.ID != nil {
		objectMap["Id"] = npi.ID
	}
	if npi.ServicePartitionKind != "" {
		objectMap["ServicePartitionKind"] = npi.ServicePartitionKind
	}
	return json.Marshal(objectMap)
}

// AsInt64RangePartitionInformation is the BasicPartitionInformation implementation for NamedPartitionInformation.
func (npi NamedPartitionInformation) AsInt64RangePartitionInformation() (*Int64RangePartitionInformation, bool) {
	return nil, false
}

// AsNamedPartitionInformation is the BasicPartitionInformation implementation for NamedPartitionInformation.
func (npi NamedPartitionInformation) AsNamedPartitionInformation() (*NamedPartitionInformation, bool) {
	return &npi, true
}

// AsSingletonPartitionInformation is the BasicPartitionInformation implementation for NamedPartitionInformation.
func (npi NamedPartitionInformation) AsSingletonPartitionInformation() (*SingletonPartitionInformation, bool) {
	return nil, false
}

// AsPartitionInformation is the BasicPartitionInformation implementation for NamedPartitionInformation.
func (npi NamedPartitionInformation) AsPartitionInformation() (*PartitionInformation, bool) {
	return nil, false
}

// AsBasicPartitionInformation is the BasicPartitionInformation implementation for NamedPartitionInformation.
func (npi NamedPartitionInformation) AsBasicPartitionInformation() (BasicPartitionInformation, bool) {
	return &npi, true
}

// NamedPartitionSchemeDescription describes the named partition scheme of the service.
type NamedPartitionSchemeDescription struct {
	// Count - The number of partitions.
	Count *int32 `json:"Count,omitempty"`
	// Names - Array of size specified by the ‘Count’ parameter, for the names of the partitions.
	Names *[]string `json:"Names,omitempty"`
	// PartitionScheme - Possible values include: 'PartitionSchemePartitionSchemeDescription', 'PartitionSchemeNamed1', 'PartitionSchemeSingleton1', 'PartitionSchemeUniformInt64Range1'
	PartitionScheme PartitionSchemeBasicPartitionSchemeDescription `json:"PartitionScheme,omitempty"`
}

// MarshalJSON is the custom marshaler for NamedPartitionSchemeDescription.
func (npsd NamedPartitionSchemeDescription) MarshalJSON() ([]byte, error) {
	npsd.PartitionScheme = PartitionSchemeNamed1
	objectMap := make(map[string]interface{})
	if npsd.Count != nil {
		objectMap["Count"] = npsd.Count
	}
	if npsd.Names != nil {
		objectMap["Names"] = npsd.Names
	}
	if npsd.PartitionScheme != "" {
		objectMap["PartitionScheme"] = npsd.PartitionScheme
	}
	return json.Marshal(objectMap)
}

// AsNamedPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for NamedPartitionSchemeDescription.
func (npsd NamedPartitionSchemeDescription) AsNamedPartitionSchemeDescription() (*NamedPartitionSchemeDescription, bool) {
	return &npsd, true
}

// AsSingletonPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for NamedPartitionSchemeDescription.
func (npsd NamedPartitionSchemeDescription) AsSingletonPartitionSchemeDescription() (*SingletonPartitionSchemeDescription, bool) {
	return nil, false
}

// AsUniformInt64RangePartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for NamedPartitionSchemeDescription.
func (npsd NamedPartitionSchemeDescription) AsUniformInt64RangePartitionSchemeDescription() (*UniformInt64RangePartitionSchemeDescription, bool) {
	return nil, false
}

// AsPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for NamedPartitionSchemeDescription.
func (npsd NamedPartitionSchemeDescription) AsPartitionSchemeDescription() (*PartitionSchemeDescription, bool) {
	return nil, false
}

// AsBasicPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for NamedPartitionSchemeDescription.
func (npsd NamedPartitionSchemeDescription) AsBasicPartitionSchemeDescription() (BasicPartitionSchemeDescription, bool) {
	return &npsd, true
}

// NetworkRef describes a network reference in a service.
type NetworkRef struct {
	// Name - Name of the network
	Name *string `json:"name,omitempty"`
	// EndpointRefs - A list of endpoints that are exposed on this network.
	EndpointRefs *[]EndpointRef `json:"endpointRefs,omitempty"`
}

// NetworkResourceDescription this type describes a network resource.
type NetworkResourceDescription struct {
	autorest.Response `json:"-"`
	// Name - Name of the Network resource.
	Name *string `json:"name,omitempty"`
	// Properties - Describes properties of a network resource.
	Properties BasicNetworkResourceProperties `json:"properties,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for NetworkResourceDescription struct.
func (nrd *NetworkResourceDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				nrd.Name = &name
			}
		case "properties":
			if v != nil {
				properties, err := unmarshalBasicNetworkResourceProperties(*v)
				if err != nil {
					return err
				}
				nrd.Properties = properties
			}
		}
	}

	return nil
}

// BasicNetworkResourceProperties describes properties of a network resource.
type BasicNetworkResourceProperties interface {
	AsLocalNetworkResourceProperties() (*LocalNetworkResourceProperties, bool)
	AsNetworkResourceProperties() (*NetworkResourceProperties, bool)
}

// NetworkResourceProperties describes properties of a network resource.
type NetworkResourceProperties struct {
	// Description - User readable description of the network.
	Description *string `json:"description,omitempty"`
	// Status - READ-ONLY; Status of the network. Possible values include: 'ResourceStatusUnknown', 'ResourceStatusReady', 'ResourceStatusUpgrading', 'ResourceStatusCreating', 'ResourceStatusDeleting', 'ResourceStatusFailed'
	Status ResourceStatus `json:"status,omitempty"`
	// StatusDetails - READ-ONLY; Gives additional information about the current status of the network.
	StatusDetails *string `json:"statusDetails,omitempty"`
	// Kind - Possible values include: 'KindNetworkResourcePropertiesBase', 'KindNetworkResourceProperties', 'KindLocal'
	Kind KindBasicNetworkResourcePropertiesBase `json:"kind,omitempty"`
}

func unmarshalBasicNetworkResourceProperties(body []byte) (BasicNetworkResourceProperties, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["kind"] {
	case string(KindLocal):
		var lnrp LocalNetworkResourceProperties
		err := json.Unmarshal(body, &lnrp)
		return lnrp, err
	default:
		var nrp NetworkResourceProperties
		err := json.Unmarshal(body, &nrp)
		return nrp, err
	}
}
func unmarshalBasicNetworkResourcePropertiesArray(body []byte) ([]BasicNetworkResourceProperties, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	nrpArray := make([]BasicNetworkResourceProperties, len(rawMessages))

	for index, rawMessage := range rawMessages {
		nrp, err := unmarshalBasicNetworkResourceProperties(*rawMessage)
		if err != nil {
			return nil, err
		}
		nrpArray[index] = nrp
	}
	return nrpArray, nil
}

// MarshalJSON is the custom marshaler for NetworkResourceProperties.
func (nrp NetworkResourceProperties) MarshalJSON() ([]byte, error) {
	nrp.Kind = KindNetworkResourceProperties
	objectMap := make(map[string]interface{})
	if nrp.Description != nil {
		objectMap["description"] = nrp.Description
	}
	if nrp.Kind != "" {
		objectMap["kind"] = nrp.Kind
	}
	return json.Marshal(objectMap)
}

// AsNetworkResourceProperties is the BasicNetworkResourcePropertiesBase implementation for NetworkResourceProperties.
func (nrp NetworkResourceProperties) AsNetworkResourceProperties() (*NetworkResourceProperties, bool) {
	return &nrp, true
}

// AsBasicNetworkResourceProperties is the BasicNetworkResourcePropertiesBase implementation for NetworkResourceProperties.
func (nrp NetworkResourceProperties) AsBasicNetworkResourceProperties() (BasicNetworkResourceProperties, bool) {
	return &nrp, true
}

// AsLocalNetworkResourceProperties is the BasicNetworkResourcePropertiesBase implementation for NetworkResourceProperties.
func (nrp NetworkResourceProperties) AsLocalNetworkResourceProperties() (*LocalNetworkResourceProperties, bool) {
	return nil, false
}

// AsNetworkResourcePropertiesBase is the BasicNetworkResourcePropertiesBase implementation for NetworkResourceProperties.
func (nrp NetworkResourceProperties) AsNetworkResourcePropertiesBase() (*NetworkResourcePropertiesBase, bool) {
	return nil, false
}

// AsBasicNetworkResourcePropertiesBase is the BasicNetworkResourcePropertiesBase implementation for NetworkResourceProperties.
func (nrp NetworkResourceProperties) AsBasicNetworkResourcePropertiesBase() (BasicNetworkResourcePropertiesBase, bool) {
	return &nrp, true
}

// BasicNetworkResourcePropertiesBase this type describes the properties of a network resource, including its kind.
type BasicNetworkResourcePropertiesBase interface {
	AsNetworkResourceProperties() (*NetworkResourceProperties, bool)
	AsBasicNetworkResourceProperties() (BasicNetworkResourceProperties, bool)
	AsLocalNetworkResourceProperties() (*LocalNetworkResourceProperties, bool)
	AsNetworkResourcePropertiesBase() (*NetworkResourcePropertiesBase, bool)
}

// NetworkResourcePropertiesBase this type describes the properties of a network resource, including its kind.
type NetworkResourcePropertiesBase struct {
	// Kind - Possible values include: 'KindNetworkResourcePropertiesBase', 'KindNetworkResourceProperties', 'KindLocal'
	Kind KindBasicNetworkResourcePropertiesBase `json:"kind,omitempty"`
}

func unmarshalBasicNetworkResourcePropertiesBase(body []byte) (BasicNetworkResourcePropertiesBase, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["kind"] {
	case string(KindNetworkResourceProperties):
		var nrp NetworkResourceProperties
		err := json.Unmarshal(body, &nrp)
		return nrp, err
	case string(KindLocal):
		var lnrp LocalNetworkResourceProperties
		err := json.Unmarshal(body, &lnrp)
		return lnrp, err
	default:
		var nrpb NetworkResourcePropertiesBase
		err := json.Unmarshal(body, &nrpb)
		return nrpb, err
	}
}
func unmarshalBasicNetworkResourcePropertiesBaseArray(body []byte) ([]BasicNetworkResourcePropertiesBase, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	nrpbArray := make([]BasicNetworkResourcePropertiesBase, len(rawMessages))

	for index, rawMessage := range rawMessages {
		nrpb, err := unmarshalBasicNetworkResourcePropertiesBase(*rawMessage)
		if err != nil {
			return nil, err
		}
		nrpbArray[index] = nrpb
	}
	return nrpbArray, nil
}

// MarshalJSON is the custom marshaler for NetworkResourcePropertiesBase.
func (nrpb NetworkResourcePropertiesBase) MarshalJSON() ([]byte, error) {
	nrpb.Kind = KindNetworkResourcePropertiesBase
	objectMap := make(map[string]interface{})
	if nrpb.Kind != "" {
		objectMap["kind"] = nrpb.Kind
	}
	return json.Marshal(objectMap)
}

// AsNetworkResourceProperties is the BasicNetworkResourcePropertiesBase implementation for NetworkResourcePropertiesBase.
func (nrpb NetworkResourcePropertiesBase) AsNetworkResourceProperties() (*NetworkResourceProperties, bool) {
	return nil, false
}

// AsBasicNetworkResourceProperties is the BasicNetworkResourcePropertiesBase implementation for NetworkResourcePropertiesBase.
func (nrpb NetworkResourcePropertiesBase) AsBasicNetworkResourceProperties() (BasicNetworkResourceProperties, bool) {
	return nil, false
}

// AsLocalNetworkResourceProperties is the BasicNetworkResourcePropertiesBase implementation for NetworkResourcePropertiesBase.
func (nrpb NetworkResourcePropertiesBase) AsLocalNetworkResourceProperties() (*LocalNetworkResourceProperties, bool) {
	return nil, false
}

// AsNetworkResourcePropertiesBase is the BasicNetworkResourcePropertiesBase implementation for NetworkResourcePropertiesBase.
func (nrpb NetworkResourcePropertiesBase) AsNetworkResourcePropertiesBase() (*NetworkResourcePropertiesBase, bool) {
	return &nrpb, true
}

// AsBasicNetworkResourcePropertiesBase is the BasicNetworkResourcePropertiesBase implementation for NetworkResourcePropertiesBase.
func (nrpb NetworkResourcePropertiesBase) AsBasicNetworkResourcePropertiesBase() (BasicNetworkResourcePropertiesBase, bool) {
	return &nrpb, true
}

// NodeAbortedEvent node Aborted event.
type NodeAbortedEvent struct {
	// NodeInstance - Id of Node instance.
	NodeInstance *int64 `json:"NodeInstance,omitempty"`
	// NodeID - Id of Node.
	NodeID *string `json:"NodeId,omitempty"`
	// UpgradeDomain - Upgrade domain of Node.
	UpgradeDomain *string `json:"UpgradeDomain,omitempty"`
	// FaultDomain - Fault domain of Node.
	FaultDomain *string `json:"FaultDomain,omitempty"`
	// IPAddressOrFQDN - IP address or FQDN.
	IPAddressOrFQDN *string `json:"IpAddressOrFQDN,omitempty"`
	// Hostname - Name of Host.
	Hostname *string `json:"Hostname,omitempty"`
	// IsSeedNode - Indicates if it is seed node.
	IsSeedNode *bool `json:"IsSeedNode,omitempty"`
	// NodeVersion - Version of Node.
	NodeVersion *string `json:"NodeVersion,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeAbortedEvent.
func (nae NodeAbortedEvent) MarshalJSON() ([]byte, error) {
	nae.Kind = KindNodeAborted
	objectMap := make(map[string]interface{})
	if nae.NodeInstance != nil {
		objectMap["NodeInstance"] = nae.NodeInstance
	}
	if nae.NodeID != nil {
		objectMap["NodeId"] = nae.NodeID
	}
	if nae.UpgradeDomain != nil {
		objectMap["UpgradeDomain"] = nae.UpgradeDomain
	}
	if nae.FaultDomain != nil {
		objectMap["FaultDomain"] = nae.FaultDomain
	}
	if nae.IPAddressOrFQDN != nil {
		objectMap["IpAddressOrFQDN"] = nae.IPAddressOrFQDN
	}
	if nae.Hostname != nil {
		objectMap["Hostname"] = nae.Hostname
	}
	if nae.IsSeedNode != nil {
		objectMap["IsSeedNode"] = nae.IsSeedNode
	}
	if nae.NodeVersion != nil {
		objectMap["NodeVersion"] = nae.NodeVersion
	}
	if nae.NodeName != nil {
		objectMap["NodeName"] = nae.NodeName
	}
	if nae.EventInstanceID != nil {
		objectMap["EventInstanceId"] = nae.EventInstanceID
	}
	if nae.Category != nil {
		objectMap["Category"] = nae.Category
	}
	if nae.TimeStamp != nil {
		objectMap["TimeStamp"] = nae.TimeStamp
	}
	if nae.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = nae.HasCorrelatedEvents
	}
	if nae.Kind != "" {
		objectMap["Kind"] = nae.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &nae, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return &nae, true
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeAbortedEvent.
func (nae NodeAbortedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &nae, true
}

// NodeAddedToClusterEvent node Added event.
type NodeAddedToClusterEvent struct {
	// NodeID - Id of Node.
	NodeID *string `json:"NodeId,omitempty"`
	// NodeInstance - Id of Node instance.
	NodeInstance *int64 `json:"NodeInstance,omitempty"`
	// NodeType - Type of Node.
	NodeType *string `json:"NodeType,omitempty"`
	// FabricVersion - Fabric version.
	FabricVersion *string `json:"FabricVersion,omitempty"`
	// IPAddressOrFQDN - IP address or FQDN.
	IPAddressOrFQDN *string `json:"IpAddressOrFQDN,omitempty"`
	// NodeCapacities - Capacities.
	NodeCapacities *string `json:"NodeCapacities,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) MarshalJSON() ([]byte, error) {
	natce.Kind = KindNodeAddedToCluster
	objectMap := make(map[string]interface{})
	if natce.NodeID != nil {
		objectMap["NodeId"] = natce.NodeID
	}
	if natce.NodeInstance != nil {
		objectMap["NodeInstance"] = natce.NodeInstance
	}
	if natce.NodeType != nil {
		objectMap["NodeType"] = natce.NodeType
	}
	if natce.FabricVersion != nil {
		objectMap["FabricVersion"] = natce.FabricVersion
	}
	if natce.IPAddressOrFQDN != nil {
		objectMap["IpAddressOrFQDN"] = natce.IPAddressOrFQDN
	}
	if natce.NodeCapacities != nil {
		objectMap["NodeCapacities"] = natce.NodeCapacities
	}
	if natce.NodeName != nil {
		objectMap["NodeName"] = natce.NodeName
	}
	if natce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = natce.EventInstanceID
	}
	if natce.Category != nil {
		objectMap["Category"] = natce.Category
	}
	if natce.TimeStamp != nil {
		objectMap["TimeStamp"] = natce.TimeStamp
	}
	if natce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = natce.HasCorrelatedEvents
	}
	if natce.Kind != "" {
		objectMap["Kind"] = natce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &natce, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return &natce, true
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeAddedToClusterEvent.
func (natce NodeAddedToClusterEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &natce, true
}

// NodeClosedEvent node Closed event.
type NodeClosedEvent struct {
	// NodeID - Id of Node.
	NodeID *string `json:"NodeId,omitempty"`
	// NodeInstance - Id of Node instance.
	NodeInstance *int64 `json:"NodeInstance,omitempty"`
	// Error - Describes error.
	Error *string `json:"Error,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeClosedEvent.
func (nce NodeClosedEvent) MarshalJSON() ([]byte, error) {
	nce.Kind = KindNodeClosed
	objectMap := make(map[string]interface{})
	if nce.NodeID != nil {
		objectMap["NodeId"] = nce.NodeID
	}
	if nce.NodeInstance != nil {
		objectMap["NodeInstance"] = nce.NodeInstance
	}
	if nce.Error != nil {
		objectMap["Error"] = nce.Error
	}
	if nce.NodeName != nil {
		objectMap["NodeName"] = nce.NodeName
	}
	if nce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = nce.EventInstanceID
	}
	if nce.Category != nil {
		objectMap["Category"] = nce.Category
	}
	if nce.TimeStamp != nil {
		objectMap["TimeStamp"] = nce.TimeStamp
	}
	if nce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = nce.HasCorrelatedEvents
	}
	if nce.Kind != "" {
		objectMap["Kind"] = nce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &nce, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return &nce, true
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeClosedEvent.
func (nce NodeClosedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &nce, true
}

// NodeDeactivateCompletedEvent node Deactivate Completed event.
type NodeDeactivateCompletedEvent struct {
	// NodeInstance - Id of Node instance.
	NodeInstance *int64 `json:"NodeInstance,omitempty"`
	// EffectiveDeactivateIntent - Describes deactivate intent.
	EffectiveDeactivateIntent *string `json:"EffectiveDeactivateIntent,omitempty"`
	// BatchIdsWithDeactivateIntent - Batch Ids.
	BatchIdsWithDeactivateIntent *string `json:"BatchIdsWithDeactivateIntent,omitempty"`
	// StartTime - Start time.
	StartTime *date.Time `json:"StartTime,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) MarshalJSON() ([]byte, error) {
	ndce.Kind = KindNodeDeactivateCompleted
	objectMap := make(map[string]interface{})
	if ndce.NodeInstance != nil {
		objectMap["NodeInstance"] = ndce.NodeInstance
	}
	if ndce.EffectiveDeactivateIntent != nil {
		objectMap["EffectiveDeactivateIntent"] = ndce.EffectiveDeactivateIntent
	}
	if ndce.BatchIdsWithDeactivateIntent != nil {
		objectMap["BatchIdsWithDeactivateIntent"] = ndce.BatchIdsWithDeactivateIntent
	}
	if ndce.StartTime != nil {
		objectMap["StartTime"] = ndce.StartTime
	}
	if ndce.NodeName != nil {
		objectMap["NodeName"] = ndce.NodeName
	}
	if ndce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ndce.EventInstanceID
	}
	if ndce.Category != nil {
		objectMap["Category"] = ndce.Category
	}
	if ndce.TimeStamp != nil {
		objectMap["TimeStamp"] = ndce.TimeStamp
	}
	if ndce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ndce.HasCorrelatedEvents
	}
	if ndce.Kind != "" {
		objectMap["Kind"] = ndce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &ndce, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return &ndce, true
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeDeactivateCompletedEvent.
func (ndce NodeDeactivateCompletedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ndce, true
}

// NodeDeactivateStartedEvent node Deactivate Started event.
type NodeDeactivateStartedEvent struct {
	// NodeInstance - Id of Node instance.
	NodeInstance *int64 `json:"NodeInstance,omitempty"`
	// BatchID - Batch Id.
	BatchID *string `json:"BatchId,omitempty"`
	// DeactivateIntent - Describes deactivate intent.
	DeactivateIntent *string `json:"DeactivateIntent,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) MarshalJSON() ([]byte, error) {
	ndse.Kind = KindNodeDeactivateStarted
	objectMap := make(map[string]interface{})
	if ndse.NodeInstance != nil {
		objectMap["NodeInstance"] = ndse.NodeInstance
	}
	if ndse.BatchID != nil {
		objectMap["BatchId"] = ndse.BatchID
	}
	if ndse.DeactivateIntent != nil {
		objectMap["DeactivateIntent"] = ndse.DeactivateIntent
	}
	if ndse.NodeName != nil {
		objectMap["NodeName"] = ndse.NodeName
	}
	if ndse.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ndse.EventInstanceID
	}
	if ndse.Category != nil {
		objectMap["Category"] = ndse.Category
	}
	if ndse.TimeStamp != nil {
		objectMap["TimeStamp"] = ndse.TimeStamp
	}
	if ndse.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ndse.HasCorrelatedEvents
	}
	if ndse.Kind != "" {
		objectMap["Kind"] = ndse.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &ndse, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return &ndse, true
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeDeactivateStartedEvent.
func (ndse NodeDeactivateStartedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ndse, true
}

// NodeDeactivationInfo information about the node deactivation. This information is valid for a node that
// is undergoing deactivation or has already been deactivated.
type NodeDeactivationInfo struct {
	// NodeDeactivationIntent - The intent or the reason for deactivating the node. Following are the possible values for it. Possible values include: 'NodeDeactivationIntentInvalid', 'NodeDeactivationIntentPause', 'NodeDeactivationIntentRestart', 'NodeDeactivationIntentRemoveData', 'NodeDeactivationIntentRemoveNode'
	NodeDeactivationIntent NodeDeactivationIntent `json:"NodeDeactivationIntent,omitempty"`
	// NodeDeactivationStatus - The status of node deactivation operation. Following are the possible values. Possible values include: 'NodeDeactivationStatusNone', 'NodeDeactivationStatusSafetyCheckInProgress', 'NodeDeactivationStatusSafetyCheckComplete', 'NodeDeactivationStatusCompleted'
	NodeDeactivationStatus NodeDeactivationStatus `json:"NodeDeactivationStatus,omitempty"`
	// NodeDeactivationTask - List of tasks representing the deactivation operation on the node.
	NodeDeactivationTask *[]NodeDeactivationTask `json:"NodeDeactivationTask,omitempty"`
	// PendingSafetyChecks - List of pending safety checks
	PendingSafetyChecks *[]SafetyCheckWrapper `json:"PendingSafetyChecks,omitempty"`
}

// NodeDeactivationTask the task representing the deactivation operation on the node.
type NodeDeactivationTask struct {
	// NodeDeactivationTaskID - Identity of the task related to deactivation operation on the node.
	NodeDeactivationTaskID *NodeDeactivationTaskID `json:"NodeDeactivationTaskId,omitempty"`
	// NodeDeactivationIntent - The intent or the reason for deactivating the node. Following are the possible values for it. Possible values include: 'NodeDeactivationIntentInvalid', 'NodeDeactivationIntentPause', 'NodeDeactivationIntentRestart', 'NodeDeactivationIntentRemoveData', 'NodeDeactivationIntentRemoveNode'
	NodeDeactivationIntent NodeDeactivationIntent `json:"NodeDeactivationIntent,omitempty"`
}

// NodeDeactivationTaskID identity of the task related to deactivation operation on the node.
type NodeDeactivationTaskID struct {
	// ID - Value of the task id.
	ID *string `json:"Id,omitempty"`
	// NodeDeactivationTaskType - The type of the task that performed the node deactivation. Following are the possible values. Possible values include: 'NodeDeactivationTaskTypeInvalid', 'NodeDeactivationTaskTypeInfrastructure', 'NodeDeactivationTaskTypeRepair', 'NodeDeactivationTaskTypeClient'
	NodeDeactivationTaskType NodeDeactivationTaskType `json:"NodeDeactivationTaskType,omitempty"`
}

// NodeDownEvent node Down event.
type NodeDownEvent struct {
	// NodeInstance - Id of Node instance.
	NodeInstance *int64 `json:"NodeInstance,omitempty"`
	// LastNodeUpAt - Time when Node was last up.
	LastNodeUpAt *date.Time `json:"LastNodeUpAt,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeDownEvent.
func (nde NodeDownEvent) MarshalJSON() ([]byte, error) {
	nde.Kind = KindNodeDown
	objectMap := make(map[string]interface{})
	if nde.NodeInstance != nil {
		objectMap["NodeInstance"] = nde.NodeInstance
	}
	if nde.LastNodeUpAt != nil {
		objectMap["LastNodeUpAt"] = nde.LastNodeUpAt
	}
	if nde.NodeName != nil {
		objectMap["NodeName"] = nde.NodeName
	}
	if nde.EventInstanceID != nil {
		objectMap["EventInstanceId"] = nde.EventInstanceID
	}
	if nde.Category != nil {
		objectMap["Category"] = nde.Category
	}
	if nde.TimeStamp != nil {
		objectMap["TimeStamp"] = nde.TimeStamp
	}
	if nde.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = nde.HasCorrelatedEvents
	}
	if nde.Kind != "" {
		objectMap["Kind"] = nde.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &nde, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return &nde, true
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeDownEvent.
func (nde NodeDownEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &nde, true
}

// BasicNodeEvent represents the base for all Node Events.
type BasicNodeEvent interface {
	AsNodeAbortedEvent() (*NodeAbortedEvent, bool)
	AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool)
	AsNodeClosedEvent() (*NodeClosedEvent, bool)
	AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool)
	AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool)
	AsNodeDownEvent() (*NodeDownEvent, bool)
	AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool)
	AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool)
	AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool)
	AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool)
	AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool)
	AsNodeUpEvent() (*NodeUpEvent, bool)
	AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool)
	AsNodeEvent() (*NodeEvent, bool)
}

// NodeEvent represents the base for all Node Events.
type NodeEvent struct {
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

func unmarshalBasicNodeEvent(body []byte) (BasicNodeEvent, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindNodeAborted):
		var nae NodeAbortedEvent
		err := json.Unmarshal(body, &nae)
		return nae, err
	case string(KindNodeAddedToCluster):
		var natce NodeAddedToClusterEvent
		err := json.Unmarshal(body, &natce)
		return natce, err
	case string(KindNodeClosed):
		var nce NodeClosedEvent
		err := json.Unmarshal(body, &nce)
		return nce, err
	case string(KindNodeDeactivateCompleted):
		var ndce NodeDeactivateCompletedEvent
		err := json.Unmarshal(body, &ndce)
		return ndce, err
	case string(KindNodeDeactivateStarted):
		var ndse NodeDeactivateStartedEvent
		err := json.Unmarshal(body, &ndse)
		return ndse, err
	case string(KindNodeDown):
		var nde NodeDownEvent
		err := json.Unmarshal(body, &nde)
		return nde, err
	case string(KindNodeNewHealthReport):
		var nnhre NodeNewHealthReportEvent
		err := json.Unmarshal(body, &nnhre)
		return nnhre, err
	case string(KindNodeHealthReportExpired):
		var nhree NodeHealthReportExpiredEvent
		err := json.Unmarshal(body, &nhree)
		return nhree, err
	case string(KindNodeOpenSucceeded):
		var nose NodeOpenSucceededEvent
		err := json.Unmarshal(body, &nose)
		return nose, err
	case string(KindNodeOpenFailed):
		var nofe NodeOpenFailedEvent
		err := json.Unmarshal(body, &nofe)
		return nofe, err
	case string(KindNodeRemovedFromCluster):
		var nrfce NodeRemovedFromClusterEvent
		err := json.Unmarshal(body, &nrfce)
		return nrfce, err
	case string(KindNodeUp):
		var nue NodeUpEvent
		err := json.Unmarshal(body, &nue)
		return nue, err
	case string(KindChaosNodeRestartScheduled):
		var cnrse ChaosNodeRestartScheduledEvent
		err := json.Unmarshal(body, &cnrse)
		return cnrse, err
	default:
		var ne NodeEvent
		err := json.Unmarshal(body, &ne)
		return ne, err
	}
}
func unmarshalBasicNodeEventArray(body []byte) ([]BasicNodeEvent, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	neArray := make([]BasicNodeEvent, len(rawMessages))

	for index, rawMessage := range rawMessages {
		ne, err := unmarshalBasicNodeEvent(*rawMessage)
		if err != nil {
			return nil, err
		}
		neArray[index] = ne
	}
	return neArray, nil
}

// MarshalJSON is the custom marshaler for NodeEvent.
func (ne NodeEvent) MarshalJSON() ([]byte, error) {
	ne.Kind = KindNodeEvent
	objectMap := make(map[string]interface{})
	if ne.NodeName != nil {
		objectMap["NodeName"] = ne.NodeName
	}
	if ne.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ne.EventInstanceID
	}
	if ne.Category != nil {
		objectMap["Category"] = ne.Category
	}
	if ne.TimeStamp != nil {
		objectMap["TimeStamp"] = ne.TimeStamp
	}
	if ne.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ne.HasCorrelatedEvents
	}
	if ne.Kind != "" {
		objectMap["Kind"] = ne.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeEvent() (*NodeEvent, bool) {
	return &ne, true
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &ne, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeEvent.
func (ne NodeEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ne, true
}

// NodeHealth information about the health of a Service Fabric node.
type NodeHealth struct {
	autorest.Response `json:"-"`
	// Name - Name of the node whose health information is described by this object.
	Name *string `json:"Name,omitempty"`
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

// NodeHealthEvaluation represents health evaluation for a node, containing information about the data and
// the algorithm used by health store to evaluate health. The evaluation is returned only when the
// aggregated health state is either Error or Warning.
type NodeHealthEvaluation struct {
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the current aggregated health state of the node. The types of the unhealthy evaluations can be EventHealthEvaluation.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) MarshalJSON() ([]byte, error) {
	nhe.Kind = KindNode
	objectMap := make(map[string]interface{})
	if nhe.NodeName != nil {
		objectMap["NodeName"] = nhe.NodeName
	}
	if nhe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = nhe.UnhealthyEvaluations
	}
	if nhe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = nhe.AggregatedHealthState
	}
	if nhe.Description != nil {
		objectMap["Description"] = nhe.Description
	}
	if nhe.Kind != "" {
		objectMap["Kind"] = nhe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return &nhe, true
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for NodeHealthEvaluation.
func (nhe NodeHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &nhe, true
}

// NodeHealthReportExpiredEvent node Health Report Expired event.
type NodeHealthReportExpiredEvent struct {
	// NodeInstanceID - Id of Node instance.
	NodeInstanceID *int64 `json:"NodeInstanceId,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) MarshalJSON() ([]byte, error) {
	nhree.Kind = KindNodeHealthReportExpired
	objectMap := make(map[string]interface{})
	if nhree.NodeInstanceID != nil {
		objectMap["NodeInstanceId"] = nhree.NodeInstanceID
	}
	if nhree.SourceID != nil {
		objectMap["SourceId"] = nhree.SourceID
	}
	if nhree.Property != nil {
		objectMap["Property"] = nhree.Property
	}
	if nhree.HealthState != nil {
		objectMap["HealthState"] = nhree.HealthState
	}
	if nhree.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = nhree.TimeToLiveMs
	}
	if nhree.SequenceNumber != nil {
		objectMap["SequenceNumber"] = nhree.SequenceNumber
	}
	if nhree.Description != nil {
		objectMap["Description"] = nhree.Description
	}
	if nhree.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = nhree.RemoveWhenExpired
	}
	if nhree.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = nhree.SourceUtcTimestamp
	}
	if nhree.NodeName != nil {
		objectMap["NodeName"] = nhree.NodeName
	}
	if nhree.EventInstanceID != nil {
		objectMap["EventInstanceId"] = nhree.EventInstanceID
	}
	if nhree.Category != nil {
		objectMap["Category"] = nhree.Category
	}
	if nhree.TimeStamp != nil {
		objectMap["TimeStamp"] = nhree.TimeStamp
	}
	if nhree.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = nhree.HasCorrelatedEvents
	}
	if nhree.Kind != "" {
		objectMap["Kind"] = nhree.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &nhree, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return &nhree, true
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeHealthReportExpiredEvent.
func (nhree NodeHealthReportExpiredEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &nhree, true
}

// NodeHealthState represents the health state of a node, which contains the node identifier and its
// aggregated health state.
type NodeHealthState struct {
	// Name - The name of a Service Fabric node.
	Name *string `json:"Name,omitempty"`
	// ID - An internal ID used by Service Fabric to uniquely identify a node. Node Id is deterministically generated from node name.
	ID *NodeID `json:"Id,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
}

// NodeHealthStateChunk represents the health state chunk of a node, which contains the node name and its
// aggregated health state.
type NodeHealthStateChunk struct {
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
}

// NodeHealthStateChunkList the list of node health state chunks in the cluster that respect the input
// filters in the chunk query. Returned by get cluster health state chunks query.
type NodeHealthStateChunkList struct {
	// Items - The list of node health state chunks that respect the input filters in the chunk query.
	Items *[]NodeHealthStateChunk `json:"Items,omitempty"`
	// TotalCount - Total number of entity health state objects that match the specified filters from the cluster health chunk query description.
	TotalCount *int64 `json:"TotalCount,omitempty"`
}

// NodeHealthStateFilter defines matching criteria to determine whether a node should be included in the
// returned cluster health chunk.
// One filter can match zero, one or multiple nodes, depending on its properties.
// Can be specified in the cluster health chunk query description.
type NodeHealthStateFilter struct {
	// NodeNameFilter - Name of the node that matches the filter. The filter is applied only to the specified node, if it exists.
	// If the node doesn't exist, no node is returned in the cluster health chunk based on this filter.
	// If the node exists, it is included in the cluster health chunk if the health state matches the other filter properties.
	// If not specified, all nodes that match the parent filters (if any) are taken into consideration and matched against the other filter members, like health state filter.
	NodeNameFilter *string `json:"NodeNameFilter,omitempty"`
	// HealthStateFilter - The filter for the health state of the nodes. It allows selecting nodes if they match the desired health states.
	// The possible values are integer value of one of the following health states. Only nodes that match the filter are returned. All nodes are used to evaluate the cluster aggregated health state.
	// If not specified, default value is None, unless the node name is specified. If the filter has default value and node name is specified, the matching node is returned.
	// The state values are flag-based enumeration, so the value could be a combination of these values obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6, it matches nodes with HealthState value of OK (2) and Warning (4).
	// - Default - Default value. Matches any HealthState. The value is zero.
	// - None - Filter that doesn't match any HealthState value. Used in order to return no results on a given collection of states. The value is 1.
	// - Ok - Filter that matches input with HealthState value Ok. The value is 2.
	// - Warning - Filter that matches input with HealthState value Warning. The value is 4.
	// - Error - Filter that matches input with HealthState value Error. The value is 8.
	// - All - Filter that matches input with any HealthState value. The value is 65535.
	HealthStateFilter *int32 `json:"HealthStateFilter,omitempty"`
}

// NodeID an internal ID used by Service Fabric to uniquely identify a node. Node Id is deterministically
// generated from node name.
type NodeID struct {
	// ID - Value of the node Id. This is a 128 bit integer.
	ID *string `json:"Id,omitempty"`
}

// NodeImpact describes the expected impact of a repair to a particular node.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type NodeImpact struct {
	// NodeName - The name of the impacted node.
	NodeName *string `json:"NodeName,omitempty"`
	// ImpactLevel - The level of impact expected. Possible values include: 'ImpactLevelInvalid', 'ImpactLevelNone', 'ImpactLevelRestart', 'ImpactLevelRemoveData', 'ImpactLevelRemoveNode'
	ImpactLevel ImpactLevel `json:"ImpactLevel,omitempty"`
}

// NodeInfo information about a node in Service Fabric cluster.
type NodeInfo struct {
	autorest.Response `json:"-"`
	// Name - The name of a Service Fabric node.
	Name *string `json:"Name,omitempty"`
	// IPAddressOrFQDN - The IP address or fully qualified domain name of the node.
	IPAddressOrFQDN *string `json:"IpAddressOrFQDN,omitempty"`
	// Type - The type of the node.
	Type *string `json:"Type,omitempty"`
	// CodeVersion - The version of Service Fabric binaries that the node is running.
	CodeVersion *string `json:"CodeVersion,omitempty"`
	// ConfigVersion - The version of Service Fabric cluster manifest that the node is using.
	ConfigVersion *string `json:"ConfigVersion,omitempty"`
	// NodeStatus - The status of the node. Possible values include: 'NodeStatusInvalid', 'NodeStatusUp', 'NodeStatusDown', 'NodeStatusEnabling', 'NodeStatusDisabling', 'NodeStatusDisabled', 'NodeStatusUnknown', 'NodeStatusRemoved'
	NodeStatus NodeStatus `json:"NodeStatus,omitempty"`
	// NodeUpTimeInSeconds - Time in seconds since the node has been in NodeStatus Up. Value zero indicates that the node is not Up.
	NodeUpTimeInSeconds *string `json:"NodeUpTimeInSeconds,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// IsSeedNode - Indicates if the node is a seed node or not. Returns true if the node is a seed node, otherwise false. A quorum of seed nodes are required for proper operation of Service Fabric cluster.
	IsSeedNode *bool `json:"IsSeedNode,omitempty"`
	// UpgradeDomain - The upgrade domain of the node.
	UpgradeDomain *string `json:"UpgradeDomain,omitempty"`
	// FaultDomain - The fault domain of the node.
	FaultDomain *string `json:"FaultDomain,omitempty"`
	// ID - An internal ID used by Service Fabric to uniquely identify a node. Node Id is deterministically generated from node name.
	ID *NodeID `json:"Id,omitempty"`
	// InstanceID - The ID representing the node instance. While the ID of the node is deterministically generated from the node name and remains same across restarts, the InstanceId changes every time node restarts.
	InstanceID *string `json:"InstanceId,omitempty"`
	// NodeDeactivationInfo - Information about the node deactivation. This information is valid for a node that is undergoing deactivation or has already been deactivated.
	NodeDeactivationInfo *NodeDeactivationInfo `json:"NodeDeactivationInfo,omitempty"`
	// IsStopped - Indicates if the node is stopped by calling stop node API or not. Returns true if the node is stopped, otherwise false.
	IsStopped *bool `json:"IsStopped,omitempty"`
	// NodeDownTimeInSeconds - Time in seconds since the node has been in NodeStatus Down. Value zero indicates node is not NodeStatus Down.
	NodeDownTimeInSeconds *string `json:"NodeDownTimeInSeconds,omitempty"`
	// NodeUpAt - Date time in UTC when the node came up. If the node has never been up then this value will be zero date time.
	NodeUpAt *date.Time `json:"NodeUpAt,omitempty"`
	// NodeDownAt - Date time in UTC when the node went down. If node has never been down then this value will be zero date time.
	NodeDownAt *date.Time `json:"NodeDownAt,omitempty"`
}

// NodeLoadInfo information about load on a Service Fabric node. It holds a summary of all metrics and
// their load on a node.
type NodeLoadInfo struct {
	autorest.Response `json:"-"`
	// NodeName - Name of the node for which the load information is provided by this object.
	NodeName *string `json:"NodeName,omitempty"`
	// NodeLoadMetricInformation - List that contains metrics and their load information on this node.
	NodeLoadMetricInformation *[]NodeLoadMetricInformation `json:"NodeLoadMetricInformation,omitempty"`
}

// NodeLoadMetricInformation represents data structure that contains load information for a certain metric
// on a node.
type NodeLoadMetricInformation struct {
	// Name - Name of the metric for which this load information is provided.
	Name *string `json:"Name,omitempty"`
	// NodeCapacity - Total capacity on the node for this metric.
	NodeCapacity *string `json:"NodeCapacity,omitempty"`
	// NodeLoad - Current load on the node for this metric. In future releases of Service Fabric this parameter will be deprecated in favor of CurrentNodeLoad.
	NodeLoad *string `json:"NodeLoad,omitempty"`
	// NodeRemainingCapacity - The remaining capacity on the node for this metric. In future releases of Service Fabric this parameter will be deprecated in favor of NodeCapacityRemaining.
	NodeRemainingCapacity *string `json:"NodeRemainingCapacity,omitempty"`
	// IsCapacityViolation - Indicates if there is a capacity violation for this metric on the node.
	IsCapacityViolation *bool `json:"IsCapacityViolation,omitempty"`
	// NodeBufferedCapacity - The value that indicates the reserved capacity for this metric on the node.
	NodeBufferedCapacity *string `json:"NodeBufferedCapacity,omitempty"`
	// NodeRemainingBufferedCapacity - The remaining reserved capacity for this metric on the node. In future releases of Service Fabric this parameter will be deprecated in favor of BufferedNodeCapacityRemaining.
	NodeRemainingBufferedCapacity *string `json:"NodeRemainingBufferedCapacity,omitempty"`
	// CurrentNodeLoad - Current load on the node for this metric.
	CurrentNodeLoad *string `json:"CurrentNodeLoad,omitempty"`
	// NodeCapacityRemaining - The remaining capacity on the node for the metric.
	NodeCapacityRemaining *string `json:"NodeCapacityRemaining,omitempty"`
	// BufferedNodeCapacityRemaining - The remaining capacity which is not reserved by NodeBufferPercentage for this metric on the node.
	BufferedNodeCapacityRemaining *string `json:"BufferedNodeCapacityRemaining,omitempty"`
	// PlannedNodeLoadRemoval - This value represents the load of the replicas that are planned to be removed in the future.
	// This kind of load is reported for replicas that are currently being moving to other nodes and for replicas that are currently being dropped but still use the load on the source node.
	PlannedNodeLoadRemoval *string `json:"PlannedNodeLoadRemoval,omitempty"`
}

// NodeNewHealthReportEvent node Health Report Created event.
type NodeNewHealthReportEvent struct {
	// NodeInstanceID - Id of Node instance.
	NodeInstanceID *int64 `json:"NodeInstanceId,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) MarshalJSON() ([]byte, error) {
	nnhre.Kind = KindNodeNewHealthReport
	objectMap := make(map[string]interface{})
	if nnhre.NodeInstanceID != nil {
		objectMap["NodeInstanceId"] = nnhre.NodeInstanceID
	}
	if nnhre.SourceID != nil {
		objectMap["SourceId"] = nnhre.SourceID
	}
	if nnhre.Property != nil {
		objectMap["Property"] = nnhre.Property
	}
	if nnhre.HealthState != nil {
		objectMap["HealthState"] = nnhre.HealthState
	}
	if nnhre.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = nnhre.TimeToLiveMs
	}
	if nnhre.SequenceNumber != nil {
		objectMap["SequenceNumber"] = nnhre.SequenceNumber
	}
	if nnhre.Description != nil {
		objectMap["Description"] = nnhre.Description
	}
	if nnhre.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = nnhre.RemoveWhenExpired
	}
	if nnhre.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = nnhre.SourceUtcTimestamp
	}
	if nnhre.NodeName != nil {
		objectMap["NodeName"] = nnhre.NodeName
	}
	if nnhre.EventInstanceID != nil {
		objectMap["EventInstanceId"] = nnhre.EventInstanceID
	}
	if nnhre.Category != nil {
		objectMap["Category"] = nnhre.Category
	}
	if nnhre.TimeStamp != nil {
		objectMap["TimeStamp"] = nnhre.TimeStamp
	}
	if nnhre.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = nnhre.HasCorrelatedEvents
	}
	if nnhre.Kind != "" {
		objectMap["Kind"] = nnhre.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &nnhre, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return &nnhre, true
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeNewHealthReportEvent.
func (nnhre NodeNewHealthReportEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &nnhre, true
}

// NodeOpenFailedEvent node Open Failed event.
type NodeOpenFailedEvent struct {
	// NodeInstance - Id of Node instance.
	NodeInstance *int64 `json:"NodeInstance,omitempty"`
	// NodeID - Id of Node.
	NodeID *string `json:"NodeId,omitempty"`
	// UpgradeDomain - Upgrade domain of Node.
	UpgradeDomain *string `json:"UpgradeDomain,omitempty"`
	// FaultDomain - Fault domain of Node.
	FaultDomain *string `json:"FaultDomain,omitempty"`
	// IPAddressOrFQDN - IP address or FQDN.
	IPAddressOrFQDN *string `json:"IpAddressOrFQDN,omitempty"`
	// Hostname - Name of Host.
	Hostname *string `json:"Hostname,omitempty"`
	// IsSeedNode - Indicates if it is seed node.
	IsSeedNode *bool `json:"IsSeedNode,omitempty"`
	// NodeVersion - Version of Node.
	NodeVersion *string `json:"NodeVersion,omitempty"`
	// Error - Describes the error.
	Error *string `json:"Error,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) MarshalJSON() ([]byte, error) {
	nofe.Kind = KindNodeOpenFailed
	objectMap := make(map[string]interface{})
	if nofe.NodeInstance != nil {
		objectMap["NodeInstance"] = nofe.NodeInstance
	}
	if nofe.NodeID != nil {
		objectMap["NodeId"] = nofe.NodeID
	}
	if nofe.UpgradeDomain != nil {
		objectMap["UpgradeDomain"] = nofe.UpgradeDomain
	}
	if nofe.FaultDomain != nil {
		objectMap["FaultDomain"] = nofe.FaultDomain
	}
	if nofe.IPAddressOrFQDN != nil {
		objectMap["IpAddressOrFQDN"] = nofe.IPAddressOrFQDN
	}
	if nofe.Hostname != nil {
		objectMap["Hostname"] = nofe.Hostname
	}
	if nofe.IsSeedNode != nil {
		objectMap["IsSeedNode"] = nofe.IsSeedNode
	}
	if nofe.NodeVersion != nil {
		objectMap["NodeVersion"] = nofe.NodeVersion
	}
	if nofe.Error != nil {
		objectMap["Error"] = nofe.Error
	}
	if nofe.NodeName != nil {
		objectMap["NodeName"] = nofe.NodeName
	}
	if nofe.EventInstanceID != nil {
		objectMap["EventInstanceId"] = nofe.EventInstanceID
	}
	if nofe.Category != nil {
		objectMap["Category"] = nofe.Category
	}
	if nofe.TimeStamp != nil {
		objectMap["TimeStamp"] = nofe.TimeStamp
	}
	if nofe.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = nofe.HasCorrelatedEvents
	}
	if nofe.Kind != "" {
		objectMap["Kind"] = nofe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &nofe, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return &nofe, true
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeOpenFailedEvent.
func (nofe NodeOpenFailedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &nofe, true
}

// NodeOpenSucceededEvent node Opened Succeeded event.
type NodeOpenSucceededEvent struct {
	// NodeInstance - Id of Node instance.
	NodeInstance *int64 `json:"NodeInstance,omitempty"`
	// NodeID - Id of Node.
	NodeID *string `json:"NodeId,omitempty"`
	// UpgradeDomain - Upgrade domain of Node.
	UpgradeDomain *string `json:"UpgradeDomain,omitempty"`
	// FaultDomain - Fault domain of Node.
	FaultDomain *string `json:"FaultDomain,omitempty"`
	// IPAddressOrFQDN - IP address or FQDN.
	IPAddressOrFQDN *string `json:"IpAddressOrFQDN,omitempty"`
	// Hostname - Name of Host.
	Hostname *string `json:"Hostname,omitempty"`
	// IsSeedNode - Indicates if it is seed node.
	IsSeedNode *bool `json:"IsSeedNode,omitempty"`
	// NodeVersion - Version of Node.
	NodeVersion *string `json:"NodeVersion,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) MarshalJSON() ([]byte, error) {
	nose.Kind = KindNodeOpenSucceeded
	objectMap := make(map[string]interface{})
	if nose.NodeInstance != nil {
		objectMap["NodeInstance"] = nose.NodeInstance
	}
	if nose.NodeID != nil {
		objectMap["NodeId"] = nose.NodeID
	}
	if nose.UpgradeDomain != nil {
		objectMap["UpgradeDomain"] = nose.UpgradeDomain
	}
	if nose.FaultDomain != nil {
		objectMap["FaultDomain"] = nose.FaultDomain
	}
	if nose.IPAddressOrFQDN != nil {
		objectMap["IpAddressOrFQDN"] = nose.IPAddressOrFQDN
	}
	if nose.Hostname != nil {
		objectMap["Hostname"] = nose.Hostname
	}
	if nose.IsSeedNode != nil {
		objectMap["IsSeedNode"] = nose.IsSeedNode
	}
	if nose.NodeVersion != nil {
		objectMap["NodeVersion"] = nose.NodeVersion
	}
	if nose.NodeName != nil {
		objectMap["NodeName"] = nose.NodeName
	}
	if nose.EventInstanceID != nil {
		objectMap["EventInstanceId"] = nose.EventInstanceID
	}
	if nose.Category != nil {
		objectMap["Category"] = nose.Category
	}
	if nose.TimeStamp != nil {
		objectMap["TimeStamp"] = nose.TimeStamp
	}
	if nose.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = nose.HasCorrelatedEvents
	}
	if nose.Kind != "" {
		objectMap["Kind"] = nose.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &nose, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return &nose, true
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeOpenSucceededEvent.
func (nose NodeOpenSucceededEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &nose, true
}

// NodeRemovedFromClusterEvent node Removed event.
type NodeRemovedFromClusterEvent struct {
	// NodeID - Id of Node.
	NodeID *string `json:"NodeId,omitempty"`
	// NodeInstance - Id of Node instance.
	NodeInstance *int64 `json:"NodeInstance,omitempty"`
	// NodeType - Type of Node.
	NodeType *string `json:"NodeType,omitempty"`
	// FabricVersion - Fabric version.
	FabricVersion *string `json:"FabricVersion,omitempty"`
	// IPAddressOrFQDN - IP address or FQDN.
	IPAddressOrFQDN *string `json:"IpAddressOrFQDN,omitempty"`
	// NodeCapacities - Capacities.
	NodeCapacities *string `json:"NodeCapacities,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) MarshalJSON() ([]byte, error) {
	nrfce.Kind = KindNodeRemovedFromCluster
	objectMap := make(map[string]interface{})
	if nrfce.NodeID != nil {
		objectMap["NodeId"] = nrfce.NodeID
	}
	if nrfce.NodeInstance != nil {
		objectMap["NodeInstance"] = nrfce.NodeInstance
	}
	if nrfce.NodeType != nil {
		objectMap["NodeType"] = nrfce.NodeType
	}
	if nrfce.FabricVersion != nil {
		objectMap["FabricVersion"] = nrfce.FabricVersion
	}
	if nrfce.IPAddressOrFQDN != nil {
		objectMap["IpAddressOrFQDN"] = nrfce.IPAddressOrFQDN
	}
	if nrfce.NodeCapacities != nil {
		objectMap["NodeCapacities"] = nrfce.NodeCapacities
	}
	if nrfce.NodeName != nil {
		objectMap["NodeName"] = nrfce.NodeName
	}
	if nrfce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = nrfce.EventInstanceID
	}
	if nrfce.Category != nil {
		objectMap["Category"] = nrfce.Category
	}
	if nrfce.TimeStamp != nil {
		objectMap["TimeStamp"] = nrfce.TimeStamp
	}
	if nrfce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = nrfce.HasCorrelatedEvents
	}
	if nrfce.Kind != "" {
		objectMap["Kind"] = nrfce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &nrfce, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return &nrfce, true
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeRemovedFromClusterEvent.
func (nrfce NodeRemovedFromClusterEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &nrfce, true
}

// NodeRepairImpactDescription describes the expected impact of a repair on a set of nodes.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type NodeRepairImpactDescription struct {
	// NodeImpactList - The list of nodes impacted by a repair action and their respective expected impact.
	NodeImpactList *[]NodeImpact `json:"NodeImpactList,omitempty"`
	// Kind - Possible values include: 'KindBasicRepairImpactDescriptionBaseKindRepairImpactDescriptionBase', 'KindBasicRepairImpactDescriptionBaseKindNode'
	Kind KindBasicRepairImpactDescriptionBase `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeRepairImpactDescription.
func (nrid NodeRepairImpactDescription) MarshalJSON() ([]byte, error) {
	nrid.Kind = KindBasicRepairImpactDescriptionBaseKindNode
	objectMap := make(map[string]interface{})
	if nrid.NodeImpactList != nil {
		objectMap["NodeImpactList"] = nrid.NodeImpactList
	}
	if nrid.Kind != "" {
		objectMap["Kind"] = nrid.Kind
	}
	return json.Marshal(objectMap)
}

// AsNodeRepairImpactDescription is the BasicRepairImpactDescriptionBase implementation for NodeRepairImpactDescription.
func (nrid NodeRepairImpactDescription) AsNodeRepairImpactDescription() (*NodeRepairImpactDescription, bool) {
	return &nrid, true
}

// AsRepairImpactDescriptionBase is the BasicRepairImpactDescriptionBase implementation for NodeRepairImpactDescription.
func (nrid NodeRepairImpactDescription) AsRepairImpactDescriptionBase() (*RepairImpactDescriptionBase, bool) {
	return nil, false
}

// AsBasicRepairImpactDescriptionBase is the BasicRepairImpactDescriptionBase implementation for NodeRepairImpactDescription.
func (nrid NodeRepairImpactDescription) AsBasicRepairImpactDescriptionBase() (BasicRepairImpactDescriptionBase, bool) {
	return &nrid, true
}

// NodeRepairTargetDescription describes the list of nodes targeted by a repair action.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type NodeRepairTargetDescription struct {
	// NodeNames - The list of nodes targeted by a repair action.
	NodeNames *[]string `json:"NodeNames,omitempty"`
	// Kind - Possible values include: 'KindBasicRepairTargetDescriptionBaseKindRepairTargetDescriptionBase', 'KindBasicRepairTargetDescriptionBaseKindNode'
	Kind KindBasicRepairTargetDescriptionBase `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeRepairTargetDescription.
func (nrtd NodeRepairTargetDescription) MarshalJSON() ([]byte, error) {
	nrtd.Kind = KindBasicRepairTargetDescriptionBaseKindNode
	objectMap := make(map[string]interface{})
	if nrtd.NodeNames != nil {
		objectMap["NodeNames"] = nrtd.NodeNames
	}
	if nrtd.Kind != "" {
		objectMap["Kind"] = nrtd.Kind
	}
	return json.Marshal(objectMap)
}

// AsNodeRepairTargetDescription is the BasicRepairTargetDescriptionBase implementation for NodeRepairTargetDescription.
func (nrtd NodeRepairTargetDescription) AsNodeRepairTargetDescription() (*NodeRepairTargetDescription, bool) {
	return &nrtd, true
}

// AsRepairTargetDescriptionBase is the BasicRepairTargetDescriptionBase implementation for NodeRepairTargetDescription.
func (nrtd NodeRepairTargetDescription) AsRepairTargetDescriptionBase() (*RepairTargetDescriptionBase, bool) {
	return nil, false
}

// AsBasicRepairTargetDescriptionBase is the BasicRepairTargetDescriptionBase implementation for NodeRepairTargetDescription.
func (nrtd NodeRepairTargetDescription) AsBasicRepairTargetDescriptionBase() (BasicRepairTargetDescriptionBase, bool) {
	return &nrtd, true
}

// NodeResult contains information about a node that was targeted by a user-induced operation.
type NodeResult struct {
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// NodeInstanceID - The node instance id.
	NodeInstanceID *string `json:"NodeInstanceId,omitempty"`
}

// NodesHealthEvaluation represents health evaluation for nodes, containing health evaluations for each
// unhealthy node that impacted current aggregated health state. Can be returned when evaluating cluster
// health and the aggregated health state is either Error or Warning.
type NodesHealthEvaluation struct {
	// MaxPercentUnhealthyNodes - Maximum allowed percentage of unhealthy nodes from the ClusterHealthPolicy.
	MaxPercentUnhealthyNodes *int32 `json:"MaxPercentUnhealthyNodes,omitempty"`
	// TotalCount - Total number of nodes found in the health store.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state. Includes all the unhealthy NodeHealthEvaluation that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) MarshalJSON() ([]byte, error) {
	nhe.Kind = KindNodes
	objectMap := make(map[string]interface{})
	if nhe.MaxPercentUnhealthyNodes != nil {
		objectMap["MaxPercentUnhealthyNodes"] = nhe.MaxPercentUnhealthyNodes
	}
	if nhe.TotalCount != nil {
		objectMap["TotalCount"] = nhe.TotalCount
	}
	if nhe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = nhe.UnhealthyEvaluations
	}
	if nhe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = nhe.AggregatedHealthState
	}
	if nhe.Description != nil {
		objectMap["Description"] = nhe.Description
	}
	if nhe.Kind != "" {
		objectMap["Kind"] = nhe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return &nhe, true
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for NodesHealthEvaluation.
func (nhe NodesHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &nhe, true
}

// NodeTransitionProgress information about an NodeTransition operation.  This class contains an
// OperationState and a NodeTransitionResult.  The NodeTransitionResult is not valid until OperationState
// is Completed or Faulted.
type NodeTransitionProgress struct {
	autorest.Response `json:"-"`
	// State - The state of the operation. Possible values include: 'OperationStateInvalid', 'OperationStateRunning', 'OperationStateRollingBack', 'OperationStateCompleted', 'OperationStateFaulted', 'OperationStateCancelled', 'OperationStateForceCancelled'
	State OperationState `json:"State,omitempty"`
	// NodeTransitionResult - Represents information about an operation in a terminal state (Completed or Faulted).
	NodeTransitionResult *NodeTransitionResult `json:"NodeTransitionResult,omitempty"`
}

// NodeTransitionResult represents information about an operation in a terminal state (Completed or
// Faulted).
type NodeTransitionResult struct {
	// ErrorCode - If OperationState is Completed, this is 0.  If OperationState is Faulted, this is an error code indicating the reason.
	ErrorCode *int32 `json:"ErrorCode,omitempty"`
	// NodeResult - Contains information about a node that was targeted by a user-induced operation.
	NodeResult *NodeResult `json:"NodeResult,omitempty"`
}

// NodeUpEvent node Up event.
type NodeUpEvent struct {
	// NodeInstance - Id of Node instance.
	NodeInstance *int64 `json:"NodeInstance,omitempty"`
	// LastNodeDownAt - Time when Node was last down.
	LastNodeDownAt *date.Time `json:"LastNodeDownAt,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for NodeUpEvent.
func (nue NodeUpEvent) MarshalJSON() ([]byte, error) {
	nue.Kind = KindNodeUp
	objectMap := make(map[string]interface{})
	if nue.NodeInstance != nil {
		objectMap["NodeInstance"] = nue.NodeInstance
	}
	if nue.LastNodeDownAt != nil {
		objectMap["LastNodeDownAt"] = nue.LastNodeDownAt
	}
	if nue.NodeName != nil {
		objectMap["NodeName"] = nue.NodeName
	}
	if nue.EventInstanceID != nil {
		objectMap["EventInstanceId"] = nue.EventInstanceID
	}
	if nue.Category != nil {
		objectMap["Category"] = nue.Category
	}
	if nue.TimeStamp != nil {
		objectMap["TimeStamp"] = nue.TimeStamp
	}
	if nue.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = nue.HasCorrelatedEvents
	}
	if nue.Kind != "" {
		objectMap["Kind"] = nue.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return &nue, true
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return &nue, true
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for NodeUpEvent.
func (nue NodeUpEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &nue, true
}

// NodeUpgradeProgressInfo information about the upgrading node and its status
type NodeUpgradeProgressInfo struct {
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// UpgradePhase - The state of the upgrading node. Possible values include: 'NodeUpgradePhaseInvalid', 'NodeUpgradePhasePreUpgradeSafetyCheck', 'NodeUpgradePhaseUpgrading', 'NodeUpgradePhasePostUpgradeSafetyCheck'
	UpgradePhase NodeUpgradePhase `json:"UpgradePhase,omitempty"`
	// PendingSafetyChecks - List of pending safety checks
	PendingSafetyChecks *[]SafetyCheckWrapper `json:"PendingSafetyChecks,omitempty"`
}

// OperationStatus contains the OperationId, OperationState, and OperationType for user-induced operations.
type OperationStatus struct {
	// OperationID - A GUID that identifies a call to this API.  This is also passed into the corresponding GetProgress API.
	OperationID *uuid.UUID `json:"OperationId,omitempty"`
	// State - The state of the operation. Possible values include: 'OperationStateInvalid', 'OperationStateRunning', 'OperationStateRollingBack', 'OperationStateCompleted', 'OperationStateFaulted', 'OperationStateCancelled', 'OperationStateForceCancelled'
	State OperationState `json:"State,omitempty"`
	// Type - The type of the operation. Possible values include: 'OperationTypeInvalid', 'OperationTypePartitionDataLoss', 'OperationTypePartitionQuorumLoss', 'OperationTypePartitionRestart', 'OperationTypeNodeTransition'
	Type OperationType `json:"Type,omitempty"`
}

// PackageSharingPolicyInfo represents a policy for the package sharing.
type PackageSharingPolicyInfo struct {
	// SharedPackageName - The name of code, configuration or data package that should be shared.
	SharedPackageName *string `json:"SharedPackageName,omitempty"`
	// PackageSharingScope - Represents the scope for PackageSharingPolicy. This is specified during DeployServicePackageToNode operation. Possible values include: 'PackageSharingPolicyScopeNone', 'PackageSharingPolicyScopeAll', 'PackageSharingPolicyScopeCode', 'PackageSharingPolicyScopeConfig', 'PackageSharingPolicyScopeData'
	PackageSharingScope PackageSharingPolicyScope `json:"PackageSharingScope,omitempty"`
}

// PagedApplicationInfoList the list of applications in the cluster. The list is paged when all of the
// results cannot fit in a single message. The next set of results can be obtained by executing the same
// query with the continuation token provided in this list.
type PagedApplicationInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of application information.
	Items *[]ApplicationInfo `json:"Items,omitempty"`
}

// PagedApplicationResourceDescriptionList the list of application resources. The list is paged when all of
// the results cannot fit in a single message. The next set of results can be obtained by executing the
// same query with the continuation token provided in this list.
type PagedApplicationResourceDescriptionList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - One page of the list.
	Items *[]ApplicationResourceDescription `json:"Items,omitempty"`
}

// PagedApplicationTypeInfoList the list of application types that are provisioned or being provisioned in
// the cluster. The list is paged when all of the results cannot fit in a single message. The next set of
// results can be obtained by executing the same query with the continuation token provided in this list.
type PagedApplicationTypeInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of application type information.
	Items *[]ApplicationTypeInfo `json:"Items,omitempty"`
}

// PagedBackupConfigurationInfoList the list of backup configuration information. The list is paged when
// all of the results cannot fit in a single message. The next set of results can be obtained by executing
// the same query with the continuation token provided in this list.
type PagedBackupConfigurationInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of backup configuration information.
	Items *[]BasicBackupConfigurationInfo `json:"Items,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for PagedBackupConfigurationInfoList struct.
func (pbcil *PagedBackupConfigurationInfoList) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "ContinuationToken":
			if v != nil {
				var continuationToken string
				err = json.Unmarshal(*v, &continuationToken)
				if err != nil {
					return err
				}
				pbcil.ContinuationToken = &continuationToken
			}
		case "Items":
			if v != nil {
				items, err := unmarshalBasicBackupConfigurationInfoArray(*v)
				if err != nil {
					return err
				}
				pbcil.Items = &items
			}
		}
	}

	return nil
}

// PagedBackupEntityList the list of backup entities that are being periodically backed. The list is paged
// when all of the results cannot fit in a single message. The next set of results can be obtained by
// executing the same query with the continuation token provided in this list.
type PagedBackupEntityList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of backup entity information.
	Items *[]BasicBackupEntity `json:"Items,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for PagedBackupEntityList struct.
func (pbel *PagedBackupEntityList) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "ContinuationToken":
			if v != nil {
				var continuationToken string
				err = json.Unmarshal(*v, &continuationToken)
				if err != nil {
					return err
				}
				pbel.ContinuationToken = &continuationToken
			}
		case "Items":
			if v != nil {
				items, err := unmarshalBasicBackupEntityArray(*v)
				if err != nil {
					return err
				}
				pbel.Items = &items
			}
		}
	}

	return nil
}

// PagedBackupInfoList the list of backups. The list is paged when all of the results cannot fit in a
// single message. The next set of results can be obtained by executing the same query with the
// continuation token provided in this list.
type PagedBackupInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of backup information.
	Items *[]BackupInfo `json:"Items,omitempty"`
}

// PagedBackupPolicyDescriptionList the list of backup policies configured in the cluster. The list is
// paged when all of the results cannot fit in a single message. The next set of results can be obtained by
// executing the same query with the continuation token provided in this list.
type PagedBackupPolicyDescriptionList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - The list of backup policies information.
	Items *[]BackupPolicyDescription `json:"Items,omitempty"`
}

// PagedComposeDeploymentStatusInfoList the list of compose deployments in the cluster. The list is paged
// when all of the results cannot fit in a single message. The next set of results can be obtained by
// executing the same query with the continuation token provided in this list.
type PagedComposeDeploymentStatusInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of compose deployment status information.
	Items *[]ComposeDeploymentStatusInfo `json:"Items,omitempty"`
}

// PagedDeployedApplicationInfoList the list of deployed applications in activating, downloading, or active
// states on a node.
// The list is paged when all of the results cannot fit in a single message.
// The next set of results can be obtained by executing the same query with the continuation token provided
// in this list.
type PagedDeployedApplicationInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of deployed application information.
	Items *[]DeployedApplicationInfo `json:"Items,omitempty"`
}

// PagedGatewayResourceDescriptionList the list of gateway resources. The list is paged when all of the
// results cannot fit in a single message. The next set of results can be obtained by executing the same
// query with the continuation token provided in this list.
type PagedGatewayResourceDescriptionList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - One page of the list.
	Items *[]GatewayResourceDescription `json:"Items,omitempty"`
}

// PagedNetworkResourceDescriptionList the list of network resources. The list is paged when all of the
// results cannot fit in a single message. The next set of results can be obtained by executing the same
// query with the continuation token provided in this list.
type PagedNetworkResourceDescriptionList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - One page of the list.
	Items *[]NetworkResourceDescription `json:"Items,omitempty"`
}

// PagedNodeInfoList the list of nodes in the cluster. The list is paged when all of the results cannot fit
// in a single message. The next set of results can be obtained by executing the same query with the
// continuation token provided in this list.
type PagedNodeInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of node information.
	Items *[]NodeInfo `json:"Items,omitempty"`
}

// PagedPropertyInfoList the paged list of Service Fabric properties under a given name. The list is paged
// when all of the results cannot fit in a single message. The next set of results can be obtained by
// executing the same query with the continuation token provided in this list.
type PagedPropertyInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// IsConsistent - Indicates whether any property under the given name has been modified during the enumeration. If there was a modification, this property value is false.
	IsConsistent *bool `json:"IsConsistent,omitempty"`
	// Properties - List of property information.
	Properties *[]PropertyInfo `json:"Properties,omitempty"`
}

// PagedReplicaInfoList the list of replicas in the cluster for a given partition. The list is paged when
// all of the results cannot fit in a single message. The next set of results can be obtained by executing
// the same query with the continuation token provided in this list.
type PagedReplicaInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of replica information.
	Items *[]BasicReplicaInfo `json:"Items,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for PagedReplicaInfoList struct.
func (pril *PagedReplicaInfoList) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "ContinuationToken":
			if v != nil {
				var continuationToken string
				err = json.Unmarshal(*v, &continuationToken)
				if err != nil {
					return err
				}
				pril.ContinuationToken = &continuationToken
			}
		case "Items":
			if v != nil {
				items, err := unmarshalBasicReplicaInfoArray(*v)
				if err != nil {
					return err
				}
				pril.Items = &items
			}
		}
	}

	return nil
}

// PagedSecretResourceDescriptionList the list of secret resources. The list is paged when all of the
// results cannot fit in a single message. The next set of results can be obtained by executing the same
// query with the continuation token provided in this list.
type PagedSecretResourceDescriptionList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - One page of the list.
	Items *[]SecretResourceDescription `json:"Items,omitempty"`
}

// PagedSecretValueResourceDescriptionList the list of values of a secret resource, paged if the number of
// results exceeds the limits of a single message. The next set of results can be obtained by executing the
// same query with the continuation token provided in the previous page.
type PagedSecretValueResourceDescriptionList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - One page of the list.
	Items *[]SecretValueResourceDescription `json:"Items,omitempty"`
}

// PagedServiceInfoList the list of services in the cluster for an application. The list is paged when all
// of the results cannot fit in a single message. The next set of results can be obtained by executing the
// same query with the continuation token provided in this list.
type PagedServiceInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of service information.
	Items *[]BasicServiceInfo `json:"Items,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for PagedServiceInfoList struct.
func (psil *PagedServiceInfoList) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "ContinuationToken":
			if v != nil {
				var continuationToken string
				err = json.Unmarshal(*v, &continuationToken)
				if err != nil {
					return err
				}
				psil.ContinuationToken = &continuationToken
			}
		case "Items":
			if v != nil {
				items, err := unmarshalBasicServiceInfoArray(*v)
				if err != nil {
					return err
				}
				psil.Items = &items
			}
		}
	}

	return nil
}

// PagedServicePartitionInfoList the list of partition in the cluster for a service. The list is paged when
// all of the results cannot fit in a single message. The next set of results can be obtained by executing
// the same query with the continuation token provided in this list.
type PagedServicePartitionInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of service partition information.
	Items *[]BasicServicePartitionInfo `json:"Items,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for PagedServicePartitionInfoList struct.
func (pspil *PagedServicePartitionInfoList) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "ContinuationToken":
			if v != nil {
				var continuationToken string
				err = json.Unmarshal(*v, &continuationToken)
				if err != nil {
					return err
				}
				pspil.ContinuationToken = &continuationToken
			}
		case "Items":
			if v != nil {
				items, err := unmarshalBasicServicePartitionInfoArray(*v)
				if err != nil {
					return err
				}
				pspil.Items = &items
			}
		}
	}

	return nil
}

// PagedServiceReplicaDescriptionList the list of service resource replicas in the cluster. The list is
// paged when all of the results cannot fit in a single message. The next set of results can be obtained by
// executing the same query with the continuation token provided in this list.
type PagedServiceReplicaDescriptionList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - List of service resource replica description.
	Items *[]ServiceReplicaDescription `json:"Items,omitempty"`
}

// PagedServiceResourceDescriptionList the list of service resources. The list is paged when all of the
// results cannot fit in a single message. The next set of results can be obtained by executing the same
// query with the continuation token provided in this list.
type PagedServiceResourceDescriptionList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - One page of the list.
	Items *[]ServiceResourceDescription `json:"Items,omitempty"`
}

// PagedSubNameInfoList a paged list of Service Fabric names. The list is paged when all of the results
// cannot fit in a single message. The next set of results can be obtained by executing the same query with
// the continuation token provided in this list.
type PagedSubNameInfoList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// IsConsistent - Indicates whether any name under the given name has been modified during the enumeration. If there was a modification, this property value is false.
	IsConsistent *bool `json:"IsConsistent,omitempty"`
	// SubNames - List of the child names.
	SubNames *[]string `json:"SubNames,omitempty"`
}

// PagedVolumeResourceDescriptionList the list of volume resources. The list is paged when all of the
// results cannot fit in a single message. The next set of results can be obtained by executing the same
// query with the continuation token provided in this list.
type PagedVolumeResourceDescriptionList struct {
	autorest.Response `json:"-"`
	// ContinuationToken - The continuation token parameter is used to obtain next set of results. The continuation token is included in the response of the API when the results from the system do not fit in a single response. When this value is passed to the next API call, the API returns next set of results. If there are no further results, then the continuation token is not included in the response.
	ContinuationToken *string `json:"ContinuationToken,omitempty"`
	// Items - One page of the list.
	Items *[]VolumeResourceDescription `json:"Items,omitempty"`
}

// BasicPartitionAnalysisEvent represents the base for all Partition Analysis Events.
type BasicPartitionAnalysisEvent interface {
	AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool)
	AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool)
}

// PartitionAnalysisEvent represents the base for all Partition Analysis Events.
type PartitionAnalysisEvent struct {
	// Metadata - Metadata about an Analysis Event.
	Metadata *AnalysisEventMetadata `json:"Metadata,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

func unmarshalBasicPartitionAnalysisEvent(body []byte) (BasicPartitionAnalysisEvent, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindPartitionPrimaryMoveAnalysis):
		var ppmae PartitionPrimaryMoveAnalysisEvent
		err := json.Unmarshal(body, &ppmae)
		return ppmae, err
	default:
		var pae PartitionAnalysisEvent
		err := json.Unmarshal(body, &pae)
		return pae, err
	}
}
func unmarshalBasicPartitionAnalysisEventArray(body []byte) ([]BasicPartitionAnalysisEvent, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	paeArray := make([]BasicPartitionAnalysisEvent, len(rawMessages))

	for index, rawMessage := range rawMessages {
		pae, err := unmarshalBasicPartitionAnalysisEvent(*rawMessage)
		if err != nil {
			return nil, err
		}
		paeArray[index] = pae
	}
	return paeArray, nil
}

// MarshalJSON is the custom marshaler for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) MarshalJSON() ([]byte, error) {
	pae.Kind = KindPartitionAnalysisEvent
	objectMap := make(map[string]interface{})
	if pae.Metadata != nil {
		objectMap["Metadata"] = pae.Metadata
	}
	if pae.PartitionID != nil {
		objectMap["PartitionId"] = pae.PartitionID
	}
	if pae.EventInstanceID != nil {
		objectMap["EventInstanceId"] = pae.EventInstanceID
	}
	if pae.Category != nil {
		objectMap["Category"] = pae.Category
	}
	if pae.TimeStamp != nil {
		objectMap["TimeStamp"] = pae.TimeStamp
	}
	if pae.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = pae.HasCorrelatedEvents
	}
	if pae.Kind != "" {
		objectMap["Kind"] = pae.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return &pae, true
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return &pae, true
}

// AsPartitionEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return &pae, true
}

// AsReplicaEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for PartitionAnalysisEvent.
func (pae PartitionAnalysisEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &pae, true
}

// PartitionBackupConfigurationInfo backup configuration information, for a specific partition, specifying
// what backup policy is being applied and suspend description, if any.
type PartitionBackupConfigurationInfo struct {
	autorest.Response `json:"-"`
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// PartitionID - The partition ID identifying the partition.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// PolicyName - The name of the backup policy which is applicable to this Service Fabric application or service or partition.
	PolicyName *string `json:"PolicyName,omitempty"`
	// PolicyInheritedFrom - Specifies the scope at which the backup policy is applied. Possible values include: 'BackupPolicyScopeInvalid', 'BackupPolicyScopePartition', 'BackupPolicyScopeService', 'BackupPolicyScopeApplication'
	PolicyInheritedFrom BackupPolicyScope `json:"PolicyInheritedFrom,omitempty"`
	// SuspensionInfo - Describes the backup suspension details.
	SuspensionInfo *BackupSuspensionInfo `json:"SuspensionInfo,omitempty"`
	// Kind - Possible values include: 'KindBasicBackupConfigurationInfoKindBackupConfigurationInfo', 'KindBasicBackupConfigurationInfoKindApplication', 'KindBasicBackupConfigurationInfoKindService', 'KindBasicBackupConfigurationInfoKindPartition'
	Kind KindBasicBackupConfigurationInfo `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for PartitionBackupConfigurationInfo.
func (pbci PartitionBackupConfigurationInfo) MarshalJSON() ([]byte, error) {
	pbci.Kind = KindBasicBackupConfigurationInfoKindPartition
	objectMap := make(map[string]interface{})
	if pbci.ServiceName != nil {
		objectMap["ServiceName"] = pbci.ServiceName
	}
	if pbci.PartitionID != nil {
		objectMap["PartitionId"] = pbci.PartitionID
	}
	if pbci.PolicyName != nil {
		objectMap["PolicyName"] = pbci.PolicyName
	}
	if pbci.PolicyInheritedFrom != "" {
		objectMap["PolicyInheritedFrom"] = pbci.PolicyInheritedFrom
	}
	if pbci.SuspensionInfo != nil {
		objectMap["SuspensionInfo"] = pbci.SuspensionInfo
	}
	if pbci.Kind != "" {
		objectMap["Kind"] = pbci.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for PartitionBackupConfigurationInfo.
func (pbci PartitionBackupConfigurationInfo) AsApplicationBackupConfigurationInfo() (*ApplicationBackupConfigurationInfo, bool) {
	return nil, false
}

// AsServiceBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for PartitionBackupConfigurationInfo.
func (pbci PartitionBackupConfigurationInfo) AsServiceBackupConfigurationInfo() (*ServiceBackupConfigurationInfo, bool) {
	return nil, false
}

// AsPartitionBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for PartitionBackupConfigurationInfo.
func (pbci PartitionBackupConfigurationInfo) AsPartitionBackupConfigurationInfo() (*PartitionBackupConfigurationInfo, bool) {
	return &pbci, true
}

// AsBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for PartitionBackupConfigurationInfo.
func (pbci PartitionBackupConfigurationInfo) AsBackupConfigurationInfo() (*BackupConfigurationInfo, bool) {
	return nil, false
}

// AsBasicBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for PartitionBackupConfigurationInfo.
func (pbci PartitionBackupConfigurationInfo) AsBasicBackupConfigurationInfo() (BasicBackupConfigurationInfo, bool) {
	return &pbci, true
}

// PartitionBackupEntity identifies the Service Fabric stateful partition which is being backed up.
type PartitionBackupEntity struct {
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// PartitionID - The partition ID identifying the partition.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// EntityKind - Possible values include: 'EntityKindBackupEntity', 'EntityKindApplication1', 'EntityKindService1', 'EntityKindPartition1'
	EntityKind EntityKindBasicBackupEntity `json:"EntityKind,omitempty"`
}

// MarshalJSON is the custom marshaler for PartitionBackupEntity.
func (pbe PartitionBackupEntity) MarshalJSON() ([]byte, error) {
	pbe.EntityKind = EntityKindPartition1
	objectMap := make(map[string]interface{})
	if pbe.ServiceName != nil {
		objectMap["ServiceName"] = pbe.ServiceName
	}
	if pbe.PartitionID != nil {
		objectMap["PartitionId"] = pbe.PartitionID
	}
	if pbe.EntityKind != "" {
		objectMap["EntityKind"] = pbe.EntityKind
	}
	return json.Marshal(objectMap)
}

// AsApplicationBackupEntity is the BasicBackupEntity implementation for PartitionBackupEntity.
func (pbe PartitionBackupEntity) AsApplicationBackupEntity() (*ApplicationBackupEntity, bool) {
	return nil, false
}

// AsServiceBackupEntity is the BasicBackupEntity implementation for PartitionBackupEntity.
func (pbe PartitionBackupEntity) AsServiceBackupEntity() (*ServiceBackupEntity, bool) {
	return nil, false
}

// AsPartitionBackupEntity is the BasicBackupEntity implementation for PartitionBackupEntity.
func (pbe PartitionBackupEntity) AsPartitionBackupEntity() (*PartitionBackupEntity, bool) {
	return &pbe, true
}

// AsBackupEntity is the BasicBackupEntity implementation for PartitionBackupEntity.
func (pbe PartitionBackupEntity) AsBackupEntity() (*BackupEntity, bool) {
	return nil, false
}

// AsBasicBackupEntity is the BasicBackupEntity implementation for PartitionBackupEntity.
func (pbe PartitionBackupEntity) AsBasicBackupEntity() (BasicBackupEntity, bool) {
	return &pbe, true
}

// PartitionDataLossProgress information about a partition data loss user-induced operation.
type PartitionDataLossProgress struct {
	autorest.Response `json:"-"`
	// State - The state of the operation. Possible values include: 'OperationStateInvalid', 'OperationStateRunning', 'OperationStateRollingBack', 'OperationStateCompleted', 'OperationStateFaulted', 'OperationStateCancelled', 'OperationStateForceCancelled'
	State OperationState `json:"State,omitempty"`
	// InvokeDataLossResult - Represents information about an operation in a terminal state (Completed or Faulted).
	InvokeDataLossResult *InvokeDataLossResult `json:"InvokeDataLossResult,omitempty"`
}

// BasicPartitionEvent represents the base for all Partition Events.
type BasicPartitionEvent interface {
	AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool)
	AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool)
	AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool)
	AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool)
	AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool)
	AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool)
	AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool)
	AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool)
	AsPartitionEvent() (*PartitionEvent, bool)
}

// PartitionEvent represents the base for all Partition Events.
type PartitionEvent struct {
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

func unmarshalBasicPartitionEvent(body []byte) (BasicPartitionEvent, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindPartitionAnalysisEvent):
		var pae PartitionAnalysisEvent
		err := json.Unmarshal(body, &pae)
		return pae, err
	case string(KindPartitionNewHealthReport):
		var pnhre PartitionNewHealthReportEvent
		err := json.Unmarshal(body, &pnhre)
		return pnhre, err
	case string(KindPartitionHealthReportExpired):
		var phree PartitionHealthReportExpiredEvent
		err := json.Unmarshal(body, &phree)
		return phree, err
	case string(KindPartitionReconfigured):
		var pre PartitionReconfiguredEvent
		err := json.Unmarshal(body, &pre)
		return pre, err
	case string(KindPartitionPrimaryMoveAnalysis):
		var ppmae PartitionPrimaryMoveAnalysisEvent
		err := json.Unmarshal(body, &ppmae)
		return ppmae, err
	case string(KindChaosPartitionSecondaryMoveScheduled):
		var cpsmse ChaosPartitionSecondaryMoveScheduledEvent
		err := json.Unmarshal(body, &cpsmse)
		return cpsmse, err
	case string(KindChaosPartitionPrimaryMoveScheduled):
		var cppmse ChaosPartitionPrimaryMoveScheduledEvent
		err := json.Unmarshal(body, &cppmse)
		return cppmse, err
	default:
		var peVar PartitionEvent
		err := json.Unmarshal(body, &peVar)
		return peVar, err
	}
}
func unmarshalBasicPartitionEventArray(body []byte) ([]BasicPartitionEvent, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	peVarArray := make([]BasicPartitionEvent, len(rawMessages))

	for index, rawMessage := range rawMessages {
		peVar, err := unmarshalBasicPartitionEvent(*rawMessage)
		if err != nil {
			return nil, err
		}
		peVarArray[index] = peVar
	}
	return peVarArray, nil
}

// MarshalJSON is the custom marshaler for PartitionEvent.
func (peVar PartitionEvent) MarshalJSON() ([]byte, error) {
	peVar.Kind = KindPartitionEvent
	objectMap := make(map[string]interface{})
	if peVar.PartitionID != nil {
		objectMap["PartitionId"] = peVar.PartitionID
	}
	if peVar.EventInstanceID != nil {
		objectMap["EventInstanceId"] = peVar.EventInstanceID
	}
	if peVar.Category != nil {
		objectMap["Category"] = peVar.Category
	}
	if peVar.TimeStamp != nil {
		objectMap["TimeStamp"] = peVar.TimeStamp
	}
	if peVar.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = peVar.HasCorrelatedEvents
	}
	if peVar.Kind != "" {
		objectMap["Kind"] = peVar.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return &peVar, true
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return &peVar, true
}

// AsReplicaEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for PartitionEvent.
func (peVar PartitionEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &peVar, true
}

// PartitionHealth information about the health of a Service Fabric partition.
type PartitionHealth struct {
	autorest.Response `json:"-"`
	// PartitionID - ID of the partition whose health information is described by this object.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaHealthStates - The list of replica health states associated with the partition.
	ReplicaHealthStates *[]BasicReplicaHealthState `json:"ReplicaHealthStates,omitempty"`
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for PartitionHealth struct.
func (ph *PartitionHealth) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "PartitionId":
			if v != nil {
				var partitionID uuid.UUID
				err = json.Unmarshal(*v, &partitionID)
				if err != nil {
					return err
				}
				ph.PartitionID = &partitionID
			}
		case "ReplicaHealthStates":
			if v != nil {
				replicaHealthStates, err := unmarshalBasicReplicaHealthStateArray(*v)
				if err != nil {
					return err
				}
				ph.ReplicaHealthStates = &replicaHealthStates
			}
		case "AggregatedHealthState":
			if v != nil {
				var aggregatedHealthState HealthState
				err = json.Unmarshal(*v, &aggregatedHealthState)
				if err != nil {
					return err
				}
				ph.AggregatedHealthState = aggregatedHealthState
			}
		case "HealthEvents":
			if v != nil {
				var healthEvents []HealthEvent
				err = json.Unmarshal(*v, &healthEvents)
				if err != nil {
					return err
				}
				ph.HealthEvents = &healthEvents
			}
		case "UnhealthyEvaluations":
			if v != nil {
				var unhealthyEvaluations []HealthEvaluationWrapper
				err = json.Unmarshal(*v, &unhealthyEvaluations)
				if err != nil {
					return err
				}
				ph.UnhealthyEvaluations = &unhealthyEvaluations
			}
		case "HealthStatistics":
			if v != nil {
				var healthStatistics HealthStatistics
				err = json.Unmarshal(*v, &healthStatistics)
				if err != nil {
					return err
				}
				ph.HealthStatistics = &healthStatistics
			}
		}
	}

	return nil
}

// PartitionHealthEvaluation represents health evaluation for a partition, containing information about the
// data and the algorithm used by health store to evaluate health. The evaluation is returned only when the
// aggregated health state is either Error or Warning.
type PartitionHealthEvaluation struct {
	// PartitionID - Id of the partition whose health evaluation is described by this object.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the current aggregated health state of the partition. The types of the unhealthy evaluations can be ReplicasHealthEvaluation or EventHealthEvaluation.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) MarshalJSON() ([]byte, error) {
	phe.Kind = KindPartition
	objectMap := make(map[string]interface{})
	if phe.PartitionID != nil {
		objectMap["PartitionId"] = phe.PartitionID
	}
	if phe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = phe.UnhealthyEvaluations
	}
	if phe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = phe.AggregatedHealthState
	}
	if phe.Description != nil {
		objectMap["Description"] = phe.Description
	}
	if phe.Kind != "" {
		objectMap["Kind"] = phe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return &phe, true
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for PartitionHealthEvaluation.
func (phe PartitionHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &phe, true
}

// PartitionHealthReportExpiredEvent partition Health Report Expired event.
type PartitionHealthReportExpiredEvent struct {
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) MarshalJSON() ([]byte, error) {
	phree.Kind = KindPartitionHealthReportExpired
	objectMap := make(map[string]interface{})
	if phree.SourceID != nil {
		objectMap["SourceId"] = phree.SourceID
	}
	if phree.Property != nil {
		objectMap["Property"] = phree.Property
	}
	if phree.HealthState != nil {
		objectMap["HealthState"] = phree.HealthState
	}
	if phree.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = phree.TimeToLiveMs
	}
	if phree.SequenceNumber != nil {
		objectMap["SequenceNumber"] = phree.SequenceNumber
	}
	if phree.Description != nil {
		objectMap["Description"] = phree.Description
	}
	if phree.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = phree.RemoveWhenExpired
	}
	if phree.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = phree.SourceUtcTimestamp
	}
	if phree.PartitionID != nil {
		objectMap["PartitionId"] = phree.PartitionID
	}
	if phree.EventInstanceID != nil {
		objectMap["EventInstanceId"] = phree.EventInstanceID
	}
	if phree.Category != nil {
		objectMap["Category"] = phree.Category
	}
	if phree.TimeStamp != nil {
		objectMap["TimeStamp"] = phree.TimeStamp
	}
	if phree.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = phree.HasCorrelatedEvents
	}
	if phree.Kind != "" {
		objectMap["Kind"] = phree.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return &phree, true
}

// AsReplicaEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return &phree, true
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for PartitionHealthReportExpiredEvent.
func (phree PartitionHealthReportExpiredEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &phree, true
}

// PartitionHealthState represents the health state of a partition, which contains the partition identifier
// and its aggregated health state.
type PartitionHealthState struct {
	// PartitionID - Id of the partition whose health state is described by this object.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
}

// PartitionHealthStateChunk represents the health state chunk of a partition, which contains the partition
// ID, its aggregated health state and any replicas that respect the filters in the cluster health chunk
// query description.
type PartitionHealthStateChunk struct {
	// PartitionID - The Id of the partition.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaHealthStateChunks - The list of replica health state chunks belonging to the partition that respect the filters in the cluster health chunk query description.
	ReplicaHealthStateChunks *ReplicaHealthStateChunkList `json:"ReplicaHealthStateChunks,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
}

// PartitionHealthStateChunkList the list of partition health state chunks that respect the input filters
// in the chunk query description.
// Returned by get cluster health state chunks query as part of the parent application hierarchy.
type PartitionHealthStateChunkList struct {
	// Items - The list of partition health state chunks that respect the input filters in the chunk query.
	Items *[]PartitionHealthStateChunk `json:"Items,omitempty"`
}

// PartitionHealthStateFilter defines matching criteria to determine whether a partition should be included
// as a child of a service in the cluster health chunk.
// The partitions are only returned if the parent entities match a filter specified in the cluster health
// chunk query description. The parent service and application must be included in the cluster health
// chunk.
// One filter can match zero, one or multiple partitions, depending on its properties.
type PartitionHealthStateFilter struct {
	// PartitionIDFilter - ID of the partition that matches the filter. The filter is applied only to the specified partition, if it exists.
	// If the partition doesn't exist, no partition is returned in the cluster health chunk based on this filter.
	// If the partition exists, it is included in the cluster health chunk if it respects the other filter properties.
	// If not specified, all partitions that match the parent filters (if any) are taken into consideration and matched against the other filter members, like health state filter.
	PartitionIDFilter *uuid.UUID `json:"PartitionIdFilter,omitempty"`
	// HealthStateFilter - The filter for the health state of the partitions. It allows selecting partitions if they match the desired health states.
	// The possible values are integer value of one of the following health states. Only partitions that match the filter are returned. All partitions are used to evaluate the cluster aggregated health state.
	// If not specified, default value is None, unless the partition ID is specified. If the filter has default value and partition ID is specified, the matching partition is returned.
	// The state values are flag-based enumeration, so the value could be a combination of these values obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6, it matches partitions with HealthState value of OK (2) and Warning (4).
	// - Default - Default value. Matches any HealthState. The value is zero.
	// - None - Filter that doesn't match any HealthState value. Used in order to return no results on a given collection of states. The value is 1.
	// - Ok - Filter that matches input with HealthState value Ok. The value is 2.
	// - Warning - Filter that matches input with HealthState value Warning. The value is 4.
	// - Error - Filter that matches input with HealthState value Error. The value is 8.
	// - All - Filter that matches input with any HealthState value. The value is 65535.
	HealthStateFilter *int32 `json:"HealthStateFilter,omitempty"`
	// ReplicaFilters - Defines a list of filters that specify which replicas to be included in the returned cluster health chunk as children of the parent partition. The replicas are returned only if the parent partition matches a filter.
	// If the list is empty, no replicas are returned. All the replicas are used to evaluate the parent partition aggregated health state, regardless of the input filters.
	// The partition filter may specify multiple replica filters.
	// For example, it can specify a filter to return all replicas with health state Error and another filter to always include a replica identified by its replica id.
	ReplicaFilters *[]ReplicaHealthStateFilter `json:"ReplicaFilters,omitempty"`
}

// BasicPartitionInformation information about the partition identity, partitioning scheme and keys supported by it.
type BasicPartitionInformation interface {
	AsInt64RangePartitionInformation() (*Int64RangePartitionInformation, bool)
	AsNamedPartitionInformation() (*NamedPartitionInformation, bool)
	AsSingletonPartitionInformation() (*SingletonPartitionInformation, bool)
	AsPartitionInformation() (*PartitionInformation, bool)
}

// PartitionInformation information about the partition identity, partitioning scheme and keys supported by it.
type PartitionInformation struct {
	// ID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	ID *uuid.UUID `json:"Id,omitempty"`
	// ServicePartitionKind - Possible values include: 'ServicePartitionKindPartitionInformation', 'ServicePartitionKindInt64Range1', 'ServicePartitionKindNamed1', 'ServicePartitionKindSingleton1'
	ServicePartitionKind ServicePartitionKindBasicPartitionInformation `json:"ServicePartitionKind,omitempty"`
}

func unmarshalBasicPartitionInformation(body []byte) (BasicPartitionInformation, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ServicePartitionKind"] {
	case string(ServicePartitionKindInt64Range1):
		var i6rpi Int64RangePartitionInformation
		err := json.Unmarshal(body, &i6rpi)
		return i6rpi, err
	case string(ServicePartitionKindNamed1):
		var npi NamedPartitionInformation
		err := json.Unmarshal(body, &npi)
		return npi, err
	case string(ServicePartitionKindSingleton1):
		var spi SingletonPartitionInformation
		err := json.Unmarshal(body, &spi)
		return spi, err
	default:
		var pi PartitionInformation
		err := json.Unmarshal(body, &pi)
		return pi, err
	}
}
func unmarshalBasicPartitionInformationArray(body []byte) ([]BasicPartitionInformation, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	piArray := make([]BasicPartitionInformation, len(rawMessages))

	for index, rawMessage := range rawMessages {
		pi, err := unmarshalBasicPartitionInformation(*rawMessage)
		if err != nil {
			return nil, err
		}
		piArray[index] = pi
	}
	return piArray, nil
}

// MarshalJSON is the custom marshaler for PartitionInformation.
func (pi PartitionInformation) MarshalJSON() ([]byte, error) {
	pi.ServicePartitionKind = ServicePartitionKindPartitionInformation
	objectMap := make(map[string]interface{})
	if pi.ID != nil {
		objectMap["Id"] = pi.ID
	}
	if pi.ServicePartitionKind != "" {
		objectMap["ServicePartitionKind"] = pi.ServicePartitionKind
	}
	return json.Marshal(objectMap)
}

// AsInt64RangePartitionInformation is the BasicPartitionInformation implementation for PartitionInformation.
func (pi PartitionInformation) AsInt64RangePartitionInformation() (*Int64RangePartitionInformation, bool) {
	return nil, false
}

// AsNamedPartitionInformation is the BasicPartitionInformation implementation for PartitionInformation.
func (pi PartitionInformation) AsNamedPartitionInformation() (*NamedPartitionInformation, bool) {
	return nil, false
}

// AsSingletonPartitionInformation is the BasicPartitionInformation implementation for PartitionInformation.
func (pi PartitionInformation) AsSingletonPartitionInformation() (*SingletonPartitionInformation, bool) {
	return nil, false
}

// AsPartitionInformation is the BasicPartitionInformation implementation for PartitionInformation.
func (pi PartitionInformation) AsPartitionInformation() (*PartitionInformation, bool) {
	return &pi, true
}

// AsBasicPartitionInformation is the BasicPartitionInformation implementation for PartitionInformation.
func (pi PartitionInformation) AsBasicPartitionInformation() (BasicPartitionInformation, bool) {
	return &pi, true
}

// PartitionInstanceCountScaleMechanism represents a scaling mechanism for adding or removing instances of
// stateless service partition.
type PartitionInstanceCountScaleMechanism struct {
	// MinInstanceCount - Minimum number of instances of the partition.
	MinInstanceCount *int32 `json:"MinInstanceCount,omitempty"`
	// MaxInstanceCount - Maximum number of instances of the partition.
	MaxInstanceCount *int32 `json:"MaxInstanceCount,omitempty"`
	// ScaleIncrement - The number of instances to add or remove during a scaling operation.
	ScaleIncrement *int32 `json:"ScaleIncrement,omitempty"`
	// Kind - Possible values include: 'KindScalingMechanismDescription', 'KindPartitionInstanceCount', 'KindAddRemoveIncrementalNamedPartition'
	Kind KindBasicScalingMechanismDescription `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for PartitionInstanceCountScaleMechanism.
func (picsm PartitionInstanceCountScaleMechanism) MarshalJSON() ([]byte, error) {
	picsm.Kind = KindPartitionInstanceCount
	objectMap := make(map[string]interface{})
	if picsm.MinInstanceCount != nil {
		objectMap["MinInstanceCount"] = picsm.MinInstanceCount
	}
	if picsm.MaxInstanceCount != nil {
		objectMap["MaxInstanceCount"] = picsm.MaxInstanceCount
	}
	if picsm.ScaleIncrement != nil {
		objectMap["ScaleIncrement"] = picsm.ScaleIncrement
	}
	if picsm.Kind != "" {
		objectMap["Kind"] = picsm.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionInstanceCountScaleMechanism is the BasicScalingMechanismDescription implementation for PartitionInstanceCountScaleMechanism.
func (picsm PartitionInstanceCountScaleMechanism) AsPartitionInstanceCountScaleMechanism() (*PartitionInstanceCountScaleMechanism, bool) {
	return &picsm, true
}

// AsAddRemoveIncrementalNamedPartitionScalingMechanism is the BasicScalingMechanismDescription implementation for PartitionInstanceCountScaleMechanism.
func (picsm PartitionInstanceCountScaleMechanism) AsAddRemoveIncrementalNamedPartitionScalingMechanism() (*AddRemoveIncrementalNamedPartitionScalingMechanism, bool) {
	return nil, false
}

// AsScalingMechanismDescription is the BasicScalingMechanismDescription implementation for PartitionInstanceCountScaleMechanism.
func (picsm PartitionInstanceCountScaleMechanism) AsScalingMechanismDescription() (*ScalingMechanismDescription, bool) {
	return nil, false
}

// AsBasicScalingMechanismDescription is the BasicScalingMechanismDescription implementation for PartitionInstanceCountScaleMechanism.
func (picsm PartitionInstanceCountScaleMechanism) AsBasicScalingMechanismDescription() (BasicScalingMechanismDescription, bool) {
	return &picsm, true
}

// PartitionLoadInformation represents load information for a partition, which contains the primary and
// secondary reported load metrics.
// In case there is no load reported, PartitionLoadInformation will contain the default load for the
// service of the partition.
// For default loads, LoadMetricReport's LastReportedUtc is set to 0.
type PartitionLoadInformation struct {
	autorest.Response `json:"-"`
	// PartitionID - Id of the partition.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// PrimaryLoadMetricReports - Array of load reports from the primary replica for this partition.
	PrimaryLoadMetricReports *[]LoadMetricReport `json:"PrimaryLoadMetricReports,omitempty"`
	// SecondaryLoadMetricReports - Array of aggregated load reports from all secondary replicas for this partition.
	// Array only contains the latest reported load for each metric.
	SecondaryLoadMetricReports *[]LoadMetricReport `json:"SecondaryLoadMetricReports,omitempty"`
}

// PartitionNewHealthReportEvent partition Health Report Created event.
type PartitionNewHealthReportEvent struct {
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) MarshalJSON() ([]byte, error) {
	pnhre.Kind = KindPartitionNewHealthReport
	objectMap := make(map[string]interface{})
	if pnhre.SourceID != nil {
		objectMap["SourceId"] = pnhre.SourceID
	}
	if pnhre.Property != nil {
		objectMap["Property"] = pnhre.Property
	}
	if pnhre.HealthState != nil {
		objectMap["HealthState"] = pnhre.HealthState
	}
	if pnhre.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = pnhre.TimeToLiveMs
	}
	if pnhre.SequenceNumber != nil {
		objectMap["SequenceNumber"] = pnhre.SequenceNumber
	}
	if pnhre.Description != nil {
		objectMap["Description"] = pnhre.Description
	}
	if pnhre.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = pnhre.RemoveWhenExpired
	}
	if pnhre.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = pnhre.SourceUtcTimestamp
	}
	if pnhre.PartitionID != nil {
		objectMap["PartitionId"] = pnhre.PartitionID
	}
	if pnhre.EventInstanceID != nil {
		objectMap["EventInstanceId"] = pnhre.EventInstanceID
	}
	if pnhre.Category != nil {
		objectMap["Category"] = pnhre.Category
	}
	if pnhre.TimeStamp != nil {
		objectMap["TimeStamp"] = pnhre.TimeStamp
	}
	if pnhre.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = pnhre.HasCorrelatedEvents
	}
	if pnhre.Kind != "" {
		objectMap["Kind"] = pnhre.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return &pnhre, true
}

// AsReplicaEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return &pnhre, true
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for PartitionNewHealthReportEvent.
func (pnhre PartitionNewHealthReportEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &pnhre, true
}

// PartitionPrimaryMoveAnalysisEvent partition Primary Move Analysis event.
type PartitionPrimaryMoveAnalysisEvent struct {
	// WhenMoveCompleted - Time when the move was completed.
	WhenMoveCompleted *date.Time `json:"WhenMoveCompleted,omitempty"`
	// PreviousNode - The name of a Service Fabric node.
	PreviousNode *string `json:"PreviousNode,omitempty"`
	// CurrentNode - The name of a Service Fabric node.
	CurrentNode *string `json:"CurrentNode,omitempty"`
	// MoveReason - Move reason.
	MoveReason *string `json:"MoveReason,omitempty"`
	// RelevantTraces - Relevant traces.
	RelevantTraces *string `json:"RelevantTraces,omitempty"`
	// Metadata - Metadata about an Analysis Event.
	Metadata *AnalysisEventMetadata `json:"Metadata,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) MarshalJSON() ([]byte, error) {
	ppmae.Kind = KindPartitionPrimaryMoveAnalysis
	objectMap := make(map[string]interface{})
	if ppmae.WhenMoveCompleted != nil {
		objectMap["WhenMoveCompleted"] = ppmae.WhenMoveCompleted
	}
	if ppmae.PreviousNode != nil {
		objectMap["PreviousNode"] = ppmae.PreviousNode
	}
	if ppmae.CurrentNode != nil {
		objectMap["CurrentNode"] = ppmae.CurrentNode
	}
	if ppmae.MoveReason != nil {
		objectMap["MoveReason"] = ppmae.MoveReason
	}
	if ppmae.RelevantTraces != nil {
		objectMap["RelevantTraces"] = ppmae.RelevantTraces
	}
	if ppmae.Metadata != nil {
		objectMap["Metadata"] = ppmae.Metadata
	}
	if ppmae.PartitionID != nil {
		objectMap["PartitionId"] = ppmae.PartitionID
	}
	if ppmae.EventInstanceID != nil {
		objectMap["EventInstanceId"] = ppmae.EventInstanceID
	}
	if ppmae.Category != nil {
		objectMap["Category"] = ppmae.Category
	}
	if ppmae.TimeStamp != nil {
		objectMap["TimeStamp"] = ppmae.TimeStamp
	}
	if ppmae.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = ppmae.HasCorrelatedEvents
	}
	if ppmae.Kind != "" {
		objectMap["Kind"] = ppmae.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return &ppmae, true
}

// AsPartitionEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return &ppmae, true
}

// AsReplicaEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return &ppmae, true
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for PartitionPrimaryMoveAnalysisEvent.
func (ppmae PartitionPrimaryMoveAnalysisEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &ppmae, true
}

// PartitionQuorumLossProgress information about a partition quorum loss user-induced operation.
type PartitionQuorumLossProgress struct {
	autorest.Response `json:"-"`
	// State - The state of the operation. Possible values include: 'OperationStateInvalid', 'OperationStateRunning', 'OperationStateRollingBack', 'OperationStateCompleted', 'OperationStateFaulted', 'OperationStateCancelled', 'OperationStateForceCancelled'
	State OperationState `json:"State,omitempty"`
	// InvokeQuorumLossResult - Represents information about an operation in a terminal state (Completed or Faulted).
	InvokeQuorumLossResult *InvokeQuorumLossResult `json:"InvokeQuorumLossResult,omitempty"`
}

// PartitionReconfiguredEvent partition Reconfiguration event.
type PartitionReconfiguredEvent struct {
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// NodeInstanceID - Id of Node instance.
	NodeInstanceID *string `json:"NodeInstanceId,omitempty"`
	// ServiceType - Type of Service.
	ServiceType *string `json:"ServiceType,omitempty"`
	// CcEpochDataLossVersion - CcEpochDataLoss version.
	CcEpochDataLossVersion *int64 `json:"CcEpochDataLossVersion,omitempty"`
	// CcEpochConfigVersion - CcEpochConfig version.
	CcEpochConfigVersion *int64 `json:"CcEpochConfigVersion,omitempty"`
	// ReconfigType - Type of reconfiguration.
	ReconfigType *string `json:"ReconfigType,omitempty"`
	// Result - Describes reconfiguration result.
	Result *string `json:"Result,omitempty"`
	// Phase0DurationMs - Duration of Phase0 in milli-seconds.
	Phase0DurationMs *float64 `json:"Phase0DurationMs,omitempty"`
	// Phase1DurationMs - Duration of Phase1 in milli-seconds.
	Phase1DurationMs *float64 `json:"Phase1DurationMs,omitempty"`
	// Phase2DurationMs - Duration of Phase2 in milli-seconds.
	Phase2DurationMs *float64 `json:"Phase2DurationMs,omitempty"`
	// Phase3DurationMs - Duration of Phase3 in milli-seconds.
	Phase3DurationMs *float64 `json:"Phase3DurationMs,omitempty"`
	// Phase4DurationMs - Duration of Phase4 in milli-seconds.
	Phase4DurationMs *float64 `json:"Phase4DurationMs,omitempty"`
	// TotalDurationMs - Total duration in milli-seconds.
	TotalDurationMs *float64 `json:"TotalDurationMs,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) MarshalJSON() ([]byte, error) {
	pre.Kind = KindPartitionReconfigured
	objectMap := make(map[string]interface{})
	if pre.NodeName != nil {
		objectMap["NodeName"] = pre.NodeName
	}
	if pre.NodeInstanceID != nil {
		objectMap["NodeInstanceId"] = pre.NodeInstanceID
	}
	if pre.ServiceType != nil {
		objectMap["ServiceType"] = pre.ServiceType
	}
	if pre.CcEpochDataLossVersion != nil {
		objectMap["CcEpochDataLossVersion"] = pre.CcEpochDataLossVersion
	}
	if pre.CcEpochConfigVersion != nil {
		objectMap["CcEpochConfigVersion"] = pre.CcEpochConfigVersion
	}
	if pre.ReconfigType != nil {
		objectMap["ReconfigType"] = pre.ReconfigType
	}
	if pre.Result != nil {
		objectMap["Result"] = pre.Result
	}
	if pre.Phase0DurationMs != nil {
		objectMap["Phase0DurationMs"] = pre.Phase0DurationMs
	}
	if pre.Phase1DurationMs != nil {
		objectMap["Phase1DurationMs"] = pre.Phase1DurationMs
	}
	if pre.Phase2DurationMs != nil {
		objectMap["Phase2DurationMs"] = pre.Phase2DurationMs
	}
	if pre.Phase3DurationMs != nil {
		objectMap["Phase3DurationMs"] = pre.Phase3DurationMs
	}
	if pre.Phase4DurationMs != nil {
		objectMap["Phase4DurationMs"] = pre.Phase4DurationMs
	}
	if pre.TotalDurationMs != nil {
		objectMap["TotalDurationMs"] = pre.TotalDurationMs
	}
	if pre.PartitionID != nil {
		objectMap["PartitionId"] = pre.PartitionID
	}
	if pre.EventInstanceID != nil {
		objectMap["EventInstanceId"] = pre.EventInstanceID
	}
	if pre.Category != nil {
		objectMap["Category"] = pre.Category
	}
	if pre.TimeStamp != nil {
		objectMap["TimeStamp"] = pre.TimeStamp
	}
	if pre.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = pre.HasCorrelatedEvents
	}
	if pre.Kind != "" {
		objectMap["Kind"] = pre.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return &pre, true
}

// AsReplicaEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return &pre, true
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for PartitionReconfiguredEvent.
func (pre PartitionReconfiguredEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &pre, true
}

// PartitionRestartProgress information about a partition restart user-induced operation.
type PartitionRestartProgress struct {
	autorest.Response `json:"-"`
	// State - The state of the operation. Possible values include: 'OperationStateInvalid', 'OperationStateRunning', 'OperationStateRollingBack', 'OperationStateCompleted', 'OperationStateFaulted', 'OperationStateCancelled', 'OperationStateForceCancelled'
	State OperationState `json:"State,omitempty"`
	// RestartPartitionResult - Represents information about an operation in a terminal state (Completed or Faulted).
	RestartPartitionResult *RestartPartitionResult `json:"RestartPartitionResult,omitempty"`
}

// BasicPartitionSafetyCheck represents a safety check for the service partition being performed by service fabric
// before continuing with operations.
type BasicPartitionSafetyCheck interface {
	AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool)
	AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool)
	AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool)
	AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool)
	AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool)
	AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool)
	AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool)
}

// PartitionSafetyCheck represents a safety check for the service partition being performed by service fabric
// before continuing with operations.
type PartitionSafetyCheck struct {
	// PartitionID - Id of the partition which is undergoing the safety check.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// Kind - Possible values include: 'KindSafetyCheck', 'KindPartitionSafetyCheck', 'KindEnsureAvailability', 'KindEnsurePartitionQuorum', 'KindEnsureSeedNodeQuorum', 'KindWaitForInbuildReplica', 'KindWaitForPrimaryPlacement', 'KindWaitForPrimarySwap', 'KindWaitForReconfiguration'
	Kind KindBasicSafetyCheck `json:"Kind,omitempty"`
}

func unmarshalBasicPartitionSafetyCheck(body []byte) (BasicPartitionSafetyCheck, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindEnsureAvailability):
		var easc EnsureAvailabilitySafetyCheck
		err := json.Unmarshal(body, &easc)
		return easc, err
	case string(KindEnsurePartitionQuorum):
		var epqsc EnsurePartitionQuorumSafetyCheck
		err := json.Unmarshal(body, &epqsc)
		return epqsc, err
	case string(KindWaitForInbuildReplica):
		var wfirsc WaitForInbuildReplicaSafetyCheck
		err := json.Unmarshal(body, &wfirsc)
		return wfirsc, err
	case string(KindWaitForPrimaryPlacement):
		var wfppsc WaitForPrimaryPlacementSafetyCheck
		err := json.Unmarshal(body, &wfppsc)
		return wfppsc, err
	case string(KindWaitForPrimarySwap):
		var wfpssc WaitForPrimarySwapSafetyCheck
		err := json.Unmarshal(body, &wfpssc)
		return wfpssc, err
	case string(KindWaitForReconfiguration):
		var wfrsc WaitForReconfigurationSafetyCheck
		err := json.Unmarshal(body, &wfrsc)
		return wfrsc, err
	default:
		var psc PartitionSafetyCheck
		err := json.Unmarshal(body, &psc)
		return psc, err
	}
}
func unmarshalBasicPartitionSafetyCheckArray(body []byte) ([]BasicPartitionSafetyCheck, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	pscArray := make([]BasicPartitionSafetyCheck, len(rawMessages))

	for index, rawMessage := range rawMessages {
		psc, err := unmarshalBasicPartitionSafetyCheck(*rawMessage)
		if err != nil {
			return nil, err
		}
		pscArray[index] = psc
	}
	return pscArray, nil
}

// MarshalJSON is the custom marshaler for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) MarshalJSON() ([]byte, error) {
	psc.Kind = KindPartitionSafetyCheck
	objectMap := make(map[string]interface{})
	if psc.PartitionID != nil {
		objectMap["PartitionId"] = psc.PartitionID
	}
	if psc.Kind != "" {
		objectMap["Kind"] = psc.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionSafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool) {
	return &psc, true
}

// AsBasicPartitionSafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsBasicPartitionSafetyCheck() (BasicPartitionSafetyCheck, bool) {
	return &psc, true
}

// AsEnsureAvailabilitySafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool) {
	return nil, false
}

// AsEnsurePartitionQuorumSafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool) {
	return nil, false
}

// AsSeedNodeSafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsSeedNodeSafetyCheck() (*SeedNodeSafetyCheck, bool) {
	return nil, false
}

// AsWaitForInbuildReplicaSafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimaryPlacementSafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimarySwapSafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool) {
	return nil, false
}

// AsWaitForReconfigurationSafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool) {
	return nil, false
}

// AsSafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsSafetyCheck() (*SafetyCheck, bool) {
	return nil, false
}

// AsBasicSafetyCheck is the BasicSafetyCheck implementation for PartitionSafetyCheck.
func (psc PartitionSafetyCheck) AsBasicSafetyCheck() (BasicSafetyCheck, bool) {
	return &psc, true
}

// BasicPartitionSchemeDescription describes how the service is partitioned.
type BasicPartitionSchemeDescription interface {
	AsNamedPartitionSchemeDescription() (*NamedPartitionSchemeDescription, bool)
	AsSingletonPartitionSchemeDescription() (*SingletonPartitionSchemeDescription, bool)
	AsUniformInt64RangePartitionSchemeDescription() (*UniformInt64RangePartitionSchemeDescription, bool)
	AsPartitionSchemeDescription() (*PartitionSchemeDescription, bool)
}

// PartitionSchemeDescription describes how the service is partitioned.
type PartitionSchemeDescription struct {
	// PartitionScheme - Possible values include: 'PartitionSchemePartitionSchemeDescription', 'PartitionSchemeNamed1', 'PartitionSchemeSingleton1', 'PartitionSchemeUniformInt64Range1'
	PartitionScheme PartitionSchemeBasicPartitionSchemeDescription `json:"PartitionScheme,omitempty"`
}

func unmarshalBasicPartitionSchemeDescription(body []byte) (BasicPartitionSchemeDescription, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["PartitionScheme"] {
	case string(PartitionSchemeNamed1):
		var npsd NamedPartitionSchemeDescription
		err := json.Unmarshal(body, &npsd)
		return npsd, err
	case string(PartitionSchemeSingleton1):
		var spsd SingletonPartitionSchemeDescription
		err := json.Unmarshal(body, &spsd)
		return spsd, err
	case string(PartitionSchemeUniformInt64Range1):
		var ui6rpsd UniformInt64RangePartitionSchemeDescription
		err := json.Unmarshal(body, &ui6rpsd)
		return ui6rpsd, err
	default:
		var psd PartitionSchemeDescription
		err := json.Unmarshal(body, &psd)
		return psd, err
	}
}
func unmarshalBasicPartitionSchemeDescriptionArray(body []byte) ([]BasicPartitionSchemeDescription, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	psdArray := make([]BasicPartitionSchemeDescription, len(rawMessages))

	for index, rawMessage := range rawMessages {
		psd, err := unmarshalBasicPartitionSchemeDescription(*rawMessage)
		if err != nil {
			return nil, err
		}
		psdArray[index] = psd
	}
	return psdArray, nil
}

// MarshalJSON is the custom marshaler for PartitionSchemeDescription.
func (psd PartitionSchemeDescription) MarshalJSON() ([]byte, error) {
	psd.PartitionScheme = PartitionSchemePartitionSchemeDescription
	objectMap := make(map[string]interface{})
	if psd.PartitionScheme != "" {
		objectMap["PartitionScheme"] = psd.PartitionScheme
	}
	return json.Marshal(objectMap)
}

// AsNamedPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for PartitionSchemeDescription.
func (psd PartitionSchemeDescription) AsNamedPartitionSchemeDescription() (*NamedPartitionSchemeDescription, bool) {
	return nil, false
}

// AsSingletonPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for PartitionSchemeDescription.
func (psd PartitionSchemeDescription) AsSingletonPartitionSchemeDescription() (*SingletonPartitionSchemeDescription, bool) {
	return nil, false
}

// AsUniformInt64RangePartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for PartitionSchemeDescription.
func (psd PartitionSchemeDescription) AsUniformInt64RangePartitionSchemeDescription() (*UniformInt64RangePartitionSchemeDescription, bool) {
	return nil, false
}

// AsPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for PartitionSchemeDescription.
func (psd PartitionSchemeDescription) AsPartitionSchemeDescription() (*PartitionSchemeDescription, bool) {
	return &psd, true
}

// AsBasicPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for PartitionSchemeDescription.
func (psd PartitionSchemeDescription) AsBasicPartitionSchemeDescription() (BasicPartitionSchemeDescription, bool) {
	return &psd, true
}

// PartitionsHealthEvaluation represents health evaluation for the partitions of a service, containing
// health evaluations for each unhealthy partition that impacts current aggregated health state. Can be
// returned when evaluating service health and the aggregated health state is either Error or Warning.
type PartitionsHealthEvaluation struct {
	// MaxPercentUnhealthyPartitionsPerService - Maximum allowed percentage of unhealthy partitions per service from the ServiceTypeHealthPolicy.
	MaxPercentUnhealthyPartitionsPerService *int32 `json:"MaxPercentUnhealthyPartitionsPerService,omitempty"`
	// TotalCount - Total number of partitions of the service from the health store.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state. Includes all the unhealthy PartitionHealthEvaluation that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) MarshalJSON() ([]byte, error) {
	phe.Kind = KindPartitions
	objectMap := make(map[string]interface{})
	if phe.MaxPercentUnhealthyPartitionsPerService != nil {
		objectMap["MaxPercentUnhealthyPartitionsPerService"] = phe.MaxPercentUnhealthyPartitionsPerService
	}
	if phe.TotalCount != nil {
		objectMap["TotalCount"] = phe.TotalCount
	}
	if phe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = phe.UnhealthyEvaluations
	}
	if phe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = phe.AggregatedHealthState
	}
	if phe.Description != nil {
		objectMap["Description"] = phe.Description
	}
	if phe.Kind != "" {
		objectMap["Kind"] = phe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return &phe, true
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for PartitionsHealthEvaluation.
func (phe PartitionsHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &phe, true
}

// PrimaryReplicatorStatus provides statistics about the Service Fabric Replicator, when it is functioning
// in a Primary role.
type PrimaryReplicatorStatus struct {
	// ReplicationQueueStatus - Details about the replication queue on the primary replicator.
	ReplicationQueueStatus *ReplicatorQueueStatus `json:"ReplicationQueueStatus,omitempty"`
	// RemoteReplicators - The status of all the active and idle secondary replicators that the primary is aware of.
	RemoteReplicators *[]RemoteReplicatorStatus `json:"RemoteReplicators,omitempty"`
	// Kind - Possible values include: 'KindReplicatorStatus', 'KindPrimary', 'KindSecondaryReplicatorStatus', 'KindActiveSecondary', 'KindIdleSecondary'
	Kind KindBasicReplicatorStatus `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for PrimaryReplicatorStatus.
func (prs PrimaryReplicatorStatus) MarshalJSON() ([]byte, error) {
	prs.Kind = KindPrimary
	objectMap := make(map[string]interface{})
	if prs.ReplicationQueueStatus != nil {
		objectMap["ReplicationQueueStatus"] = prs.ReplicationQueueStatus
	}
	if prs.RemoteReplicators != nil {
		objectMap["RemoteReplicators"] = prs.RemoteReplicators
	}
	if prs.Kind != "" {
		objectMap["Kind"] = prs.Kind
	}
	return json.Marshal(objectMap)
}

// AsPrimaryReplicatorStatus is the BasicReplicatorStatus implementation for PrimaryReplicatorStatus.
func (prs PrimaryReplicatorStatus) AsPrimaryReplicatorStatus() (*PrimaryReplicatorStatus, bool) {
	return &prs, true
}

// AsSecondaryReplicatorStatus is the BasicReplicatorStatus implementation for PrimaryReplicatorStatus.
func (prs PrimaryReplicatorStatus) AsSecondaryReplicatorStatus() (*SecondaryReplicatorStatus, bool) {
	return nil, false
}

// AsBasicSecondaryReplicatorStatus is the BasicReplicatorStatus implementation for PrimaryReplicatorStatus.
func (prs PrimaryReplicatorStatus) AsBasicSecondaryReplicatorStatus() (BasicSecondaryReplicatorStatus, bool) {
	return nil, false
}

// AsSecondaryActiveReplicatorStatus is the BasicReplicatorStatus implementation for PrimaryReplicatorStatus.
func (prs PrimaryReplicatorStatus) AsSecondaryActiveReplicatorStatus() (*SecondaryActiveReplicatorStatus, bool) {
	return nil, false
}

// AsSecondaryIdleReplicatorStatus is the BasicReplicatorStatus implementation for PrimaryReplicatorStatus.
func (prs PrimaryReplicatorStatus) AsSecondaryIdleReplicatorStatus() (*SecondaryIdleReplicatorStatus, bool) {
	return nil, false
}

// AsReplicatorStatus is the BasicReplicatorStatus implementation for PrimaryReplicatorStatus.
func (prs PrimaryReplicatorStatus) AsReplicatorStatus() (*ReplicatorStatus, bool) {
	return nil, false
}

// AsBasicReplicatorStatus is the BasicReplicatorStatus implementation for PrimaryReplicatorStatus.
func (prs PrimaryReplicatorStatus) AsBasicReplicatorStatus() (BasicReplicatorStatus, bool) {
	return &prs, true
}

// PropertyBatchDescriptionList describes a list of property batch operations to be executed. Either all or
// none of the operations will be committed.
type PropertyBatchDescriptionList struct {
	// Operations - A list of the property batch operations to be executed.
	Operations *[]BasicPropertyBatchOperation `json:"Operations,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for PropertyBatchDescriptionList struct.
func (pbdl *PropertyBatchDescriptionList) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "Operations":
			if v != nil {
				operations, err := unmarshalBasicPropertyBatchOperationArray(*v)
				if err != nil {
					return err
				}
				pbdl.Operations = &operations
			}
		}
	}

	return nil
}

// BasicPropertyBatchInfo information about the results of a property batch.
type BasicPropertyBatchInfo interface {
	AsSuccessfulPropertyBatchInfo() (*SuccessfulPropertyBatchInfo, bool)
	AsFailedPropertyBatchInfo() (*FailedPropertyBatchInfo, bool)
	AsPropertyBatchInfo() (*PropertyBatchInfo, bool)
}

// PropertyBatchInfo information about the results of a property batch.
type PropertyBatchInfo struct {
	autorest.Response `json:"-"`
	// Kind - Possible values include: 'KindPropertyBatchInfo', 'KindSuccessful', 'KindFailed'
	Kind KindBasicPropertyBatchInfo `json:"Kind,omitempty"`
}

func unmarshalBasicPropertyBatchInfo(body []byte) (BasicPropertyBatchInfo, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindSuccessful):
		var spbi SuccessfulPropertyBatchInfo
		err := json.Unmarshal(body, &spbi)
		return spbi, err
	case string(KindFailed):
		var fpbi FailedPropertyBatchInfo
		err := json.Unmarshal(body, &fpbi)
		return fpbi, err
	default:
		var pbi PropertyBatchInfo
		err := json.Unmarshal(body, &pbi)
		return pbi, err
	}
}
func unmarshalBasicPropertyBatchInfoArray(body []byte) ([]BasicPropertyBatchInfo, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	pbiArray := make([]BasicPropertyBatchInfo, len(rawMessages))

	for index, rawMessage := range rawMessages {
		pbi, err := unmarshalBasicPropertyBatchInfo(*rawMessage)
		if err != nil {
			return nil, err
		}
		pbiArray[index] = pbi
	}
	return pbiArray, nil
}

// MarshalJSON is the custom marshaler for PropertyBatchInfo.
func (pbi PropertyBatchInfo) MarshalJSON() ([]byte, error) {
	pbi.Kind = KindPropertyBatchInfo
	objectMap := make(map[string]interface{})
	if pbi.Kind != "" {
		objectMap["Kind"] = pbi.Kind
	}
	return json.Marshal(objectMap)
}

// AsSuccessfulPropertyBatchInfo is the BasicPropertyBatchInfo implementation for PropertyBatchInfo.
func (pbi PropertyBatchInfo) AsSuccessfulPropertyBatchInfo() (*SuccessfulPropertyBatchInfo, bool) {
	return nil, false
}

// AsFailedPropertyBatchInfo is the BasicPropertyBatchInfo implementation for PropertyBatchInfo.
func (pbi PropertyBatchInfo) AsFailedPropertyBatchInfo() (*FailedPropertyBatchInfo, bool) {
	return nil, false
}

// AsPropertyBatchInfo is the BasicPropertyBatchInfo implementation for PropertyBatchInfo.
func (pbi PropertyBatchInfo) AsPropertyBatchInfo() (*PropertyBatchInfo, bool) {
	return &pbi, true
}

// AsBasicPropertyBatchInfo is the BasicPropertyBatchInfo implementation for PropertyBatchInfo.
func (pbi PropertyBatchInfo) AsBasicPropertyBatchInfo() (BasicPropertyBatchInfo, bool) {
	return &pbi, true
}

// PropertyBatchInfoModel ...
type PropertyBatchInfoModel struct {
	autorest.Response `json:"-"`
	Value             BasicPropertyBatchInfo `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for PropertyBatchInfoModel struct.
func (pbim *PropertyBatchInfoModel) UnmarshalJSON(body []byte) error {
	pbi, err := unmarshalBasicPropertyBatchInfo(body)
	if err != nil {
		return err
	}
	pbim.Value = pbi

	return nil
}

// BasicPropertyBatchOperation represents the base type for property operations that can be put into a batch and
// submitted.
type BasicPropertyBatchOperation interface {
	AsCheckExistsPropertyBatchOperation() (*CheckExistsPropertyBatchOperation, bool)
	AsCheckSequencePropertyBatchOperation() (*CheckSequencePropertyBatchOperation, bool)
	AsCheckValuePropertyBatchOperation() (*CheckValuePropertyBatchOperation, bool)
	AsDeletePropertyBatchOperation() (*DeletePropertyBatchOperation, bool)
	AsGetPropertyBatchOperation() (*GetPropertyBatchOperation, bool)
	AsPutPropertyBatchOperation() (*PutPropertyBatchOperation, bool)
	AsPropertyBatchOperation() (*PropertyBatchOperation, bool)
}

// PropertyBatchOperation represents the base type for property operations that can be put into a batch and
// submitted.
type PropertyBatchOperation struct {
	// PropertyName - The name of the Service Fabric property.
	PropertyName *string `json:"PropertyName,omitempty"`
	// Kind - Possible values include: 'KindPropertyBatchOperation', 'KindCheckExists', 'KindCheckSequence', 'KindCheckValue', 'KindDelete', 'KindGet', 'KindPut'
	Kind KindBasicPropertyBatchOperation `json:"Kind,omitempty"`
}

func unmarshalBasicPropertyBatchOperation(body []byte) (BasicPropertyBatchOperation, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindCheckExists):
		var cepbo CheckExistsPropertyBatchOperation
		err := json.Unmarshal(body, &cepbo)
		return cepbo, err
	case string(KindCheckSequence):
		var cspbo CheckSequencePropertyBatchOperation
		err := json.Unmarshal(body, &cspbo)
		return cspbo, err
	case string(KindCheckValue):
		var cvpbo CheckValuePropertyBatchOperation
		err := json.Unmarshal(body, &cvpbo)
		return cvpbo, err
	case string(KindDelete):
		var dpbo DeletePropertyBatchOperation
		err := json.Unmarshal(body, &dpbo)
		return dpbo, err
	case string(KindGet):
		var gpbo GetPropertyBatchOperation
		err := json.Unmarshal(body, &gpbo)
		return gpbo, err
	case string(KindPut):
		var ppbo PutPropertyBatchOperation
		err := json.Unmarshal(body, &ppbo)
		return ppbo, err
	default:
		var pbo PropertyBatchOperation
		err := json.Unmarshal(body, &pbo)
		return pbo, err
	}
}
func unmarshalBasicPropertyBatchOperationArray(body []byte) ([]BasicPropertyBatchOperation, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	pboArray := make([]BasicPropertyBatchOperation, len(rawMessages))

	for index, rawMessage := range rawMessages {
		pbo, err := unmarshalBasicPropertyBatchOperation(*rawMessage)
		if err != nil {
			return nil, err
		}
		pboArray[index] = pbo
	}
	return pboArray, nil
}

// MarshalJSON is the custom marshaler for PropertyBatchOperation.
func (pbo PropertyBatchOperation) MarshalJSON() ([]byte, error) {
	pbo.Kind = KindPropertyBatchOperation
	objectMap := make(map[string]interface{})
	if pbo.PropertyName != nil {
		objectMap["PropertyName"] = pbo.PropertyName
	}
	if pbo.Kind != "" {
		objectMap["Kind"] = pbo.Kind
	}
	return json.Marshal(objectMap)
}

// AsCheckExistsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for PropertyBatchOperation.
func (pbo PropertyBatchOperation) AsCheckExistsPropertyBatchOperation() (*CheckExistsPropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckSequencePropertyBatchOperation is the BasicPropertyBatchOperation implementation for PropertyBatchOperation.
func (pbo PropertyBatchOperation) AsCheckSequencePropertyBatchOperation() (*CheckSequencePropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckValuePropertyBatchOperation is the BasicPropertyBatchOperation implementation for PropertyBatchOperation.
func (pbo PropertyBatchOperation) AsCheckValuePropertyBatchOperation() (*CheckValuePropertyBatchOperation, bool) {
	return nil, false
}

// AsDeletePropertyBatchOperation is the BasicPropertyBatchOperation implementation for PropertyBatchOperation.
func (pbo PropertyBatchOperation) AsDeletePropertyBatchOperation() (*DeletePropertyBatchOperation, bool) {
	return nil, false
}

// AsGetPropertyBatchOperation is the BasicPropertyBatchOperation implementation for PropertyBatchOperation.
func (pbo PropertyBatchOperation) AsGetPropertyBatchOperation() (*GetPropertyBatchOperation, bool) {
	return nil, false
}

// AsPutPropertyBatchOperation is the BasicPropertyBatchOperation implementation for PropertyBatchOperation.
func (pbo PropertyBatchOperation) AsPutPropertyBatchOperation() (*PutPropertyBatchOperation, bool) {
	return nil, false
}

// AsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for PropertyBatchOperation.
func (pbo PropertyBatchOperation) AsPropertyBatchOperation() (*PropertyBatchOperation, bool) {
	return &pbo, true
}

// AsBasicPropertyBatchOperation is the BasicPropertyBatchOperation implementation for PropertyBatchOperation.
func (pbo PropertyBatchOperation) AsBasicPropertyBatchOperation() (BasicPropertyBatchOperation, bool) {
	return &pbo, true
}

// PropertyDescription description of a Service Fabric property.
type PropertyDescription struct {
	// PropertyName - The name of the Service Fabric property.
	PropertyName *string `json:"PropertyName,omitempty"`
	// CustomTypeID - The property's custom type ID. Using this property, the user is able to tag the type of the value of the property.
	CustomTypeID *string `json:"CustomTypeId,omitempty"`
	// Value - Describes a Service Fabric property value.
	Value BasicPropertyValue `json:"Value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for PropertyDescription struct.
func (pd *PropertyDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "PropertyName":
			if v != nil {
				var propertyName string
				err = json.Unmarshal(*v, &propertyName)
				if err != nil {
					return err
				}
				pd.PropertyName = &propertyName
			}
		case "CustomTypeId":
			if v != nil {
				var customTypeID string
				err = json.Unmarshal(*v, &customTypeID)
				if err != nil {
					return err
				}
				pd.CustomTypeID = &customTypeID
			}
		case "Value":
			if v != nil {
				value, err := unmarshalBasicPropertyValue(*v)
				if err != nil {
					return err
				}
				pd.Value = value
			}
		}
	}

	return nil
}

// PropertyInfo information about a Service Fabric property.
type PropertyInfo struct {
	autorest.Response `json:"-"`
	// Name - The name of the Service Fabric property.
	Name *string `json:"Name,omitempty"`
	// Value - Describes a Service Fabric property value.
	Value BasicPropertyValue `json:"Value,omitempty"`
	// Metadata - The metadata associated with a property, including the property's name.
	Metadata *PropertyMetadata `json:"Metadata,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for PropertyInfo struct.
func (pi *PropertyInfo) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "Name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				pi.Name = &name
			}
		case "Value":
			if v != nil {
				value, err := unmarshalBasicPropertyValue(*v)
				if err != nil {
					return err
				}
				pi.Value = value
			}
		case "Metadata":
			if v != nil {
				var metadata PropertyMetadata
				err = json.Unmarshal(*v, &metadata)
				if err != nil {
					return err
				}
				pi.Metadata = &metadata
			}
		}
	}

	return nil
}

// PropertyMetadata the metadata associated with a property, including the property's name.
type PropertyMetadata struct {
	// TypeID - The kind of property, determined by the type of data. Following are the possible values. Possible values include: 'PropertyValueKindInvalid', 'PropertyValueKindBinary', 'PropertyValueKindInt64', 'PropertyValueKindDouble', 'PropertyValueKindString', 'PropertyValueKindGUID'
	TypeID PropertyValueKind `json:"TypeId,omitempty"`
	// CustomTypeID - The property's custom type ID.
	CustomTypeID *string `json:"CustomTypeId,omitempty"`
	// Parent - The name of the parent Service Fabric Name for the property. It could be thought of as the name-space/table under which the property exists.
	Parent *string `json:"Parent,omitempty"`
	// SizeInBytes - The length of the serialized property value.
	SizeInBytes *int32 `json:"SizeInBytes,omitempty"`
	// LastModifiedUtcTimestamp - Represents when the Property was last modified. Only write operations will cause this field to be updated.
	LastModifiedUtcTimestamp *date.Time `json:"LastModifiedUtcTimestamp,omitempty"`
	// SequenceNumber - The version of the property. Every time a property is modified, its sequence number is increased.
	SequenceNumber *string `json:"SequenceNumber,omitempty"`
}

// BasicPropertyValue describes a Service Fabric property value.
type BasicPropertyValue interface {
	AsBinaryPropertyValue() (*BinaryPropertyValue, bool)
	AsInt64PropertyValue() (*Int64PropertyValue, bool)
	AsDoublePropertyValue() (*DoublePropertyValue, bool)
	AsStringPropertyValue() (*StringPropertyValue, bool)
	AsGUIDPropertyValue() (*GUIDPropertyValue, bool)
	AsPropertyValue() (*PropertyValue, bool)
}

// PropertyValue describes a Service Fabric property value.
type PropertyValue struct {
	// Kind - Possible values include: 'KindPropertyValue', 'KindBinary', 'KindInt64', 'KindDouble', 'KindString', 'KindGUID'
	Kind KindBasicPropertyValue `json:"Kind,omitempty"`
}

func unmarshalBasicPropertyValue(body []byte) (BasicPropertyValue, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindBinary):
		var bpv BinaryPropertyValue
		err := json.Unmarshal(body, &bpv)
		return bpv, err
	case string(KindInt64):
		var i6pv Int64PropertyValue
		err := json.Unmarshal(body, &i6pv)
		return i6pv, err
	case string(KindDouble):
		var dpv DoublePropertyValue
		err := json.Unmarshal(body, &dpv)
		return dpv, err
	case string(KindString):
		var spv StringPropertyValue
		err := json.Unmarshal(body, &spv)
		return spv, err
	case string(KindGUID):
		var gpv GUIDPropertyValue
		err := json.Unmarshal(body, &gpv)
		return gpv, err
	default:
		var pv PropertyValue
		err := json.Unmarshal(body, &pv)
		return pv, err
	}
}
func unmarshalBasicPropertyValueArray(body []byte) ([]BasicPropertyValue, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	pvArray := make([]BasicPropertyValue, len(rawMessages))

	for index, rawMessage := range rawMessages {
		pv, err := unmarshalBasicPropertyValue(*rawMessage)
		if err != nil {
			return nil, err
		}
		pvArray[index] = pv
	}
	return pvArray, nil
}

// MarshalJSON is the custom marshaler for PropertyValue.
func (pv PropertyValue) MarshalJSON() ([]byte, error) {
	pv.Kind = KindPropertyValue
	objectMap := make(map[string]interface{})
	if pv.Kind != "" {
		objectMap["Kind"] = pv.Kind
	}
	return json.Marshal(objectMap)
}

// AsBinaryPropertyValue is the BasicPropertyValue implementation for PropertyValue.
func (pv PropertyValue) AsBinaryPropertyValue() (*BinaryPropertyValue, bool) {
	return nil, false
}

// AsInt64PropertyValue is the BasicPropertyValue implementation for PropertyValue.
func (pv PropertyValue) AsInt64PropertyValue() (*Int64PropertyValue, bool) {
	return nil, false
}

// AsDoublePropertyValue is the BasicPropertyValue implementation for PropertyValue.
func (pv PropertyValue) AsDoublePropertyValue() (*DoublePropertyValue, bool) {
	return nil, false
}

// AsStringPropertyValue is the BasicPropertyValue implementation for PropertyValue.
func (pv PropertyValue) AsStringPropertyValue() (*StringPropertyValue, bool) {
	return nil, false
}

// AsGUIDPropertyValue is the BasicPropertyValue implementation for PropertyValue.
func (pv PropertyValue) AsGUIDPropertyValue() (*GUIDPropertyValue, bool) {
	return nil, false
}

// AsPropertyValue is the BasicPropertyValue implementation for PropertyValue.
func (pv PropertyValue) AsPropertyValue() (*PropertyValue, bool) {
	return &pv, true
}

// AsBasicPropertyValue is the BasicPropertyValue implementation for PropertyValue.
func (pv PropertyValue) AsBasicPropertyValue() (BasicPropertyValue, bool) {
	return &pv, true
}

// ProvisionApplicationTypeDescription describes the operation to register or provision an application type
// using an application package uploaded to the Service Fabric image store.
type ProvisionApplicationTypeDescription struct {
	// ApplicationTypeBuildPath - The relative path for the application package in the image store specified during the prior upload operation.
	ApplicationTypeBuildPath *string `json:"ApplicationTypeBuildPath,omitempty"`
	// ApplicationPackageCleanupPolicy - The kind of action that needs to be taken for cleaning up the application package after successful provision. Possible values include: 'ApplicationPackageCleanupPolicyInvalid', 'ApplicationPackageCleanupPolicyDefault', 'ApplicationPackageCleanupPolicyAutomatic', 'ApplicationPackageCleanupPolicyManual'
	ApplicationPackageCleanupPolicy ApplicationPackageCleanupPolicy `json:"ApplicationPackageCleanupPolicy,omitempty"`
	// Async - Indicates whether or not provisioning should occur asynchronously. When set to true, the provision operation returns when the request is accepted by the system, and the provision operation continues without any timeout limit. The default value is false. For large application packages, we recommend setting the value to true.
	Async *bool `json:"Async,omitempty"`
	// Kind - Possible values include: 'KindProvisionApplicationTypeDescriptionBase', 'KindImageStorePath', 'KindExternalStore'
	Kind KindBasicProvisionApplicationTypeDescriptionBase `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ProvisionApplicationTypeDescription.
func (patd ProvisionApplicationTypeDescription) MarshalJSON() ([]byte, error) {
	patd.Kind = KindImageStorePath
	objectMap := make(map[string]interface{})
	if patd.ApplicationTypeBuildPath != nil {
		objectMap["ApplicationTypeBuildPath"] = patd.ApplicationTypeBuildPath
	}
	if patd.ApplicationPackageCleanupPolicy != "" {
		objectMap["ApplicationPackageCleanupPolicy"] = patd.ApplicationPackageCleanupPolicy
	}
	if patd.Async != nil {
		objectMap["Async"] = patd.Async
	}
	if patd.Kind != "" {
		objectMap["Kind"] = patd.Kind
	}
	return json.Marshal(objectMap)
}

// AsProvisionApplicationTypeDescription is the BasicProvisionApplicationTypeDescriptionBase implementation for ProvisionApplicationTypeDescription.
func (patd ProvisionApplicationTypeDescription) AsProvisionApplicationTypeDescription() (*ProvisionApplicationTypeDescription, bool) {
	return &patd, true
}

// AsExternalStoreProvisionApplicationTypeDescription is the BasicProvisionApplicationTypeDescriptionBase implementation for ProvisionApplicationTypeDescription.
func (patd ProvisionApplicationTypeDescription) AsExternalStoreProvisionApplicationTypeDescription() (*ExternalStoreProvisionApplicationTypeDescription, bool) {
	return nil, false
}

// AsProvisionApplicationTypeDescriptionBase is the BasicProvisionApplicationTypeDescriptionBase implementation for ProvisionApplicationTypeDescription.
func (patd ProvisionApplicationTypeDescription) AsProvisionApplicationTypeDescriptionBase() (*ProvisionApplicationTypeDescriptionBase, bool) {
	return nil, false
}

// AsBasicProvisionApplicationTypeDescriptionBase is the BasicProvisionApplicationTypeDescriptionBase implementation for ProvisionApplicationTypeDescription.
func (patd ProvisionApplicationTypeDescription) AsBasicProvisionApplicationTypeDescriptionBase() (BasicProvisionApplicationTypeDescriptionBase, bool) {
	return &patd, true
}

// BasicProvisionApplicationTypeDescriptionBase represents the type of registration or provision requested, and if the
// operation needs to be asynchronous or not. Supported types of provision operations are from either image store or
// external store.
type BasicProvisionApplicationTypeDescriptionBase interface {
	AsProvisionApplicationTypeDescription() (*ProvisionApplicationTypeDescription, bool)
	AsExternalStoreProvisionApplicationTypeDescription() (*ExternalStoreProvisionApplicationTypeDescription, bool)
	AsProvisionApplicationTypeDescriptionBase() (*ProvisionApplicationTypeDescriptionBase, bool)
}

// ProvisionApplicationTypeDescriptionBase represents the type of registration or provision requested, and if
// the operation needs to be asynchronous or not. Supported types of provision operations are from either image
// store or external store.
type ProvisionApplicationTypeDescriptionBase struct {
	// Async - Indicates whether or not provisioning should occur asynchronously. When set to true, the provision operation returns when the request is accepted by the system, and the provision operation continues without any timeout limit. The default value is false. For large application packages, we recommend setting the value to true.
	Async *bool `json:"Async,omitempty"`
	// Kind - Possible values include: 'KindProvisionApplicationTypeDescriptionBase', 'KindImageStorePath', 'KindExternalStore'
	Kind KindBasicProvisionApplicationTypeDescriptionBase `json:"Kind,omitempty"`
}

func unmarshalBasicProvisionApplicationTypeDescriptionBase(body []byte) (BasicProvisionApplicationTypeDescriptionBase, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindImageStorePath):
		var patd ProvisionApplicationTypeDescription
		err := json.Unmarshal(body, &patd)
		return patd, err
	case string(KindExternalStore):
		var espatd ExternalStoreProvisionApplicationTypeDescription
		err := json.Unmarshal(body, &espatd)
		return espatd, err
	default:
		var patdb ProvisionApplicationTypeDescriptionBase
		err := json.Unmarshal(body, &patdb)
		return patdb, err
	}
}
func unmarshalBasicProvisionApplicationTypeDescriptionBaseArray(body []byte) ([]BasicProvisionApplicationTypeDescriptionBase, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	patdbArray := make([]BasicProvisionApplicationTypeDescriptionBase, len(rawMessages))

	for index, rawMessage := range rawMessages {
		patdb, err := unmarshalBasicProvisionApplicationTypeDescriptionBase(*rawMessage)
		if err != nil {
			return nil, err
		}
		patdbArray[index] = patdb
	}
	return patdbArray, nil
}

// MarshalJSON is the custom marshaler for ProvisionApplicationTypeDescriptionBase.
func (patdb ProvisionApplicationTypeDescriptionBase) MarshalJSON() ([]byte, error) {
	patdb.Kind = KindProvisionApplicationTypeDescriptionBase
	objectMap := make(map[string]interface{})
	if patdb.Async != nil {
		objectMap["Async"] = patdb.Async
	}
	if patdb.Kind != "" {
		objectMap["Kind"] = patdb.Kind
	}
	return json.Marshal(objectMap)
}

// AsProvisionApplicationTypeDescription is the BasicProvisionApplicationTypeDescriptionBase implementation for ProvisionApplicationTypeDescriptionBase.
func (patdb ProvisionApplicationTypeDescriptionBase) AsProvisionApplicationTypeDescription() (*ProvisionApplicationTypeDescription, bool) {
	return nil, false
}

// AsExternalStoreProvisionApplicationTypeDescription is the BasicProvisionApplicationTypeDescriptionBase implementation for ProvisionApplicationTypeDescriptionBase.
func (patdb ProvisionApplicationTypeDescriptionBase) AsExternalStoreProvisionApplicationTypeDescription() (*ExternalStoreProvisionApplicationTypeDescription, bool) {
	return nil, false
}

// AsProvisionApplicationTypeDescriptionBase is the BasicProvisionApplicationTypeDescriptionBase implementation for ProvisionApplicationTypeDescriptionBase.
func (patdb ProvisionApplicationTypeDescriptionBase) AsProvisionApplicationTypeDescriptionBase() (*ProvisionApplicationTypeDescriptionBase, bool) {
	return &patdb, true
}

// AsBasicProvisionApplicationTypeDescriptionBase is the BasicProvisionApplicationTypeDescriptionBase implementation for ProvisionApplicationTypeDescriptionBase.
func (patdb ProvisionApplicationTypeDescriptionBase) AsBasicProvisionApplicationTypeDescriptionBase() (BasicProvisionApplicationTypeDescriptionBase, bool) {
	return &patdb, true
}

// ProvisionFabricDescription describes the parameters for provisioning a cluster.
type ProvisionFabricDescription struct {
	// CodeFilePath - The cluster code package file path.
	CodeFilePath *string `json:"CodeFilePath,omitempty"`
	// ClusterManifestFilePath - The cluster manifest file path.
	ClusterManifestFilePath *string `json:"ClusterManifestFilePath,omitempty"`
}

// PutPropertyBatchOperation puts the specified property under the specified name.
// Note that if one PropertyBatchOperation in a PropertyBatch fails,
// the entire batch fails and cannot be committed in a transactional manner.
type PutPropertyBatchOperation struct {
	// Value - Describes a Service Fabric property value.
	Value BasicPropertyValue `json:"Value,omitempty"`
	// CustomTypeID - The property's custom type ID. Using this property, the user is able to tag the type of the value of the property.
	CustomTypeID *string `json:"CustomTypeId,omitempty"`
	// PropertyName - The name of the Service Fabric property.
	PropertyName *string `json:"PropertyName,omitempty"`
	// Kind - Possible values include: 'KindPropertyBatchOperation', 'KindCheckExists', 'KindCheckSequence', 'KindCheckValue', 'KindDelete', 'KindGet', 'KindPut'
	Kind KindBasicPropertyBatchOperation `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for PutPropertyBatchOperation.
func (ppbo PutPropertyBatchOperation) MarshalJSON() ([]byte, error) {
	ppbo.Kind = KindPut
	objectMap := make(map[string]interface{})
	objectMap["Value"] = ppbo.Value
	if ppbo.CustomTypeID != nil {
		objectMap["CustomTypeId"] = ppbo.CustomTypeID
	}
	if ppbo.PropertyName != nil {
		objectMap["PropertyName"] = ppbo.PropertyName
	}
	if ppbo.Kind != "" {
		objectMap["Kind"] = ppbo.Kind
	}
	return json.Marshal(objectMap)
}

// AsCheckExistsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for PutPropertyBatchOperation.
func (ppbo PutPropertyBatchOperation) AsCheckExistsPropertyBatchOperation() (*CheckExistsPropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckSequencePropertyBatchOperation is the BasicPropertyBatchOperation implementation for PutPropertyBatchOperation.
func (ppbo PutPropertyBatchOperation) AsCheckSequencePropertyBatchOperation() (*CheckSequencePropertyBatchOperation, bool) {
	return nil, false
}

// AsCheckValuePropertyBatchOperation is the BasicPropertyBatchOperation implementation for PutPropertyBatchOperation.
func (ppbo PutPropertyBatchOperation) AsCheckValuePropertyBatchOperation() (*CheckValuePropertyBatchOperation, bool) {
	return nil, false
}

// AsDeletePropertyBatchOperation is the BasicPropertyBatchOperation implementation for PutPropertyBatchOperation.
func (ppbo PutPropertyBatchOperation) AsDeletePropertyBatchOperation() (*DeletePropertyBatchOperation, bool) {
	return nil, false
}

// AsGetPropertyBatchOperation is the BasicPropertyBatchOperation implementation for PutPropertyBatchOperation.
func (ppbo PutPropertyBatchOperation) AsGetPropertyBatchOperation() (*GetPropertyBatchOperation, bool) {
	return nil, false
}

// AsPutPropertyBatchOperation is the BasicPropertyBatchOperation implementation for PutPropertyBatchOperation.
func (ppbo PutPropertyBatchOperation) AsPutPropertyBatchOperation() (*PutPropertyBatchOperation, bool) {
	return &ppbo, true
}

// AsPropertyBatchOperation is the BasicPropertyBatchOperation implementation for PutPropertyBatchOperation.
func (ppbo PutPropertyBatchOperation) AsPropertyBatchOperation() (*PropertyBatchOperation, bool) {
	return nil, false
}

// AsBasicPropertyBatchOperation is the BasicPropertyBatchOperation implementation for PutPropertyBatchOperation.
func (ppbo PutPropertyBatchOperation) AsBasicPropertyBatchOperation() (BasicPropertyBatchOperation, bool) {
	return &ppbo, true
}

// UnmarshalJSON is the custom unmarshaler for PutPropertyBatchOperation struct.
func (ppbo *PutPropertyBatchOperation) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "Value":
			if v != nil {
				value, err := unmarshalBasicPropertyValue(*v)
				if err != nil {
					return err
				}
				ppbo.Value = value
			}
		case "CustomTypeId":
			if v != nil {
				var customTypeID string
				err = json.Unmarshal(*v, &customTypeID)
				if err != nil {
					return err
				}
				ppbo.CustomTypeID = &customTypeID
			}
		case "PropertyName":
			if v != nil {
				var propertyName string
				err = json.Unmarshal(*v, &propertyName)
				if err != nil {
					return err
				}
				ppbo.PropertyName = &propertyName
			}
		case "Kind":
			if v != nil {
				var kind KindBasicPropertyBatchOperation
				err = json.Unmarshal(*v, &kind)
				if err != nil {
					return err
				}
				ppbo.Kind = kind
			}
		}
	}

	return nil
}

// ReconfigurationInformation information about current reconfiguration like phase, type, previous
// configuration role of replica and reconfiguration start date time.
type ReconfigurationInformation struct {
	// PreviousConfigurationRole - Replica role before reconfiguration started. Possible values include: 'ReplicaRoleUnknown', 'ReplicaRoleNone', 'ReplicaRolePrimary', 'ReplicaRoleIdleSecondary', 'ReplicaRoleActiveSecondary'
	PreviousConfigurationRole ReplicaRole `json:"PreviousConfigurationRole,omitempty"`
	// ReconfigurationPhase - Current phase of ongoing reconfiguration. If no reconfiguration is taking place then this value will be "None". Possible values include: 'ReconfigurationPhaseUnknown', 'ReconfigurationPhaseNone', 'ReconfigurationPhasePhase0', 'ReconfigurationPhasePhase1', 'ReconfigurationPhasePhase2', 'ReconfigurationPhasePhase3', 'ReconfigurationPhasePhase4', 'ReconfigurationPhaseAbortPhaseZero'
	ReconfigurationPhase ReconfigurationPhase `json:"ReconfigurationPhase,omitempty"`
	// ReconfigurationType - Type of current ongoing reconfiguration. If no reconfiguration is taking place then this value will be "None". Possible values include: 'ReconfigurationTypeUnknown', 'ReconfigurationTypeSwapPrimary', 'ReconfigurationTypeFailover', 'ReconfigurationTypeOther'
	ReconfigurationType ReconfigurationType `json:"ReconfigurationType,omitempty"`
	// ReconfigurationStartTimeUtc - Start time (in UTC) of the ongoing reconfiguration. If no reconfiguration is taking place then this value will be zero date-time.
	ReconfigurationStartTimeUtc *date.Time `json:"ReconfigurationStartTimeUtc,omitempty"`
}

// RegistryCredential credential information to connect to container registry.
type RegistryCredential struct {
	// RegistryUserName - The user name to connect to container registry.
	RegistryUserName *string `json:"RegistryUserName,omitempty"`
	// RegistryPassword - The password for supplied username to connect to container registry.
	RegistryPassword *string `json:"RegistryPassword,omitempty"`
	// PasswordEncrypted - Indicates that supplied container registry password is encrypted.
	PasswordEncrypted *bool `json:"PasswordEncrypted,omitempty"`
}

// ReliableCollectionsRef specifying this parameter adds support for reliable collections
type ReliableCollectionsRef struct {
	// Name - Name of ReliableCollection resource. Right now it's not used and you can use any string.
	Name *string `json:"name,omitempty"`
	// DoNotPersistState - False (the default) if ReliableCollections state is persisted to disk as usual. True if you do not want to persist state, in which case replication is still enabled and you can use ReliableCollections as distributed cache.
	DoNotPersistState *bool `json:"doNotPersistState,omitempty"`
}

// RemoteReplicatorAcknowledgementDetail provides various statistics of the acknowledgements that are being
// received from the remote replicator.
type RemoteReplicatorAcknowledgementDetail struct {
	// AverageReceiveDuration - Represents the average duration it takes for the remote replicator to receive an operation.
	AverageReceiveDuration *string `json:"AverageReceiveDuration,omitempty"`
	// AverageApplyDuration - Represents the average duration it takes for the remote replicator to apply an operation. This usually entails writing the operation to disk.
	AverageApplyDuration *string `json:"AverageApplyDuration,omitempty"`
	// NotReceivedCount - Represents the number of operations not yet received by a remote replicator.
	NotReceivedCount *string `json:"NotReceivedCount,omitempty"`
	// ReceivedAndNotAppliedCount - Represents the number of operations received and not yet applied by a remote replicator.
	ReceivedAndNotAppliedCount *string `json:"ReceivedAndNotAppliedCount,omitempty"`
}

// RemoteReplicatorAcknowledgementStatus provides details about the remote replicators from the primary
// replicator's point of view.
type RemoteReplicatorAcknowledgementStatus struct {
	// ReplicationStreamAcknowledgementDetail - Details about the acknowledgements for operations that are part of the replication stream data.
	ReplicationStreamAcknowledgementDetail *RemoteReplicatorAcknowledgementDetail `json:"ReplicationStreamAcknowledgementDetail,omitempty"`
	// CopyStreamAcknowledgementDetail - Details about the acknowledgements for operations that are part of the copy stream data.
	CopyStreamAcknowledgementDetail *RemoteReplicatorAcknowledgementDetail `json:"CopyStreamAcknowledgementDetail,omitempty"`
}

// RemoteReplicatorStatus represents the state of the secondary replicator from the primary replicator’s
// point of view.
type RemoteReplicatorStatus struct {
	// ReplicaID - Represents the replica ID of the remote secondary replicator.
	ReplicaID *string `json:"ReplicaId,omitempty"`
	// LastAcknowledgementProcessedTimeUtc - The last timestamp (in UTC) when an acknowledgement from the secondary replicator was processed on the primary.
	// UTC 0 represents an invalid value, indicating that no acknowledgement messages were ever processed.
	LastAcknowledgementProcessedTimeUtc *date.Time `json:"LastAcknowledgementProcessedTimeUtc,omitempty"`
	// LastReceivedReplicationSequenceNumber - The highest replication operation sequence number that the secondary has received from the primary.
	LastReceivedReplicationSequenceNumber *string `json:"LastReceivedReplicationSequenceNumber,omitempty"`
	// LastAppliedReplicationSequenceNumber - The highest replication operation sequence number that the secondary has applied to its state.
	LastAppliedReplicationSequenceNumber *string `json:"LastAppliedReplicationSequenceNumber,omitempty"`
	// IsInBuild - A value that indicates whether the secondary replica is in the process of being built.
	IsInBuild *bool `json:"IsInBuild,omitempty"`
	// LastReceivedCopySequenceNumber - The highest copy operation sequence number that the secondary has received from the primary.
	// A value of -1 implies that the secondary has received all copy operations.
	LastReceivedCopySequenceNumber *string `json:"LastReceivedCopySequenceNumber,omitempty"`
	// LastAppliedCopySequenceNumber - The highest copy operation sequence number that the secondary has applied to its state.
	// A value of -1 implies that the secondary has applied all copy operations and the copy process is complete.
	LastAppliedCopySequenceNumber *string `json:"LastAppliedCopySequenceNumber,omitempty"`
	// RemoteReplicatorAcknowledgementStatus - Represents the acknowledgment status for the remote secondary replicator.
	RemoteReplicatorAcknowledgementStatus *RemoteReplicatorAcknowledgementStatus `json:"RemoteReplicatorAcknowledgementStatus,omitempty"`
}

// BasicRepairImpactDescriptionBase describes the expected impact of executing a repair task.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type BasicRepairImpactDescriptionBase interface {
	AsNodeRepairImpactDescription() (*NodeRepairImpactDescription, bool)
	AsRepairImpactDescriptionBase() (*RepairImpactDescriptionBase, bool)
}

// RepairImpactDescriptionBase describes the expected impact of executing a repair task.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type RepairImpactDescriptionBase struct {
	// Kind - Possible values include: 'KindBasicRepairImpactDescriptionBaseKindRepairImpactDescriptionBase', 'KindBasicRepairImpactDescriptionBaseKindNode'
	Kind KindBasicRepairImpactDescriptionBase `json:"Kind,omitempty"`
}

func unmarshalBasicRepairImpactDescriptionBase(body []byte) (BasicRepairImpactDescriptionBase, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindBasicRepairImpactDescriptionBaseKindNode):
		var nrid NodeRepairImpactDescription
		err := json.Unmarshal(body, &nrid)
		return nrid, err
	default:
		var ridb RepairImpactDescriptionBase
		err := json.Unmarshal(body, &ridb)
		return ridb, err
	}
}
func unmarshalBasicRepairImpactDescriptionBaseArray(body []byte) ([]BasicRepairImpactDescriptionBase, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	ridbArray := make([]BasicRepairImpactDescriptionBase, len(rawMessages))

	for index, rawMessage := range rawMessages {
		ridb, err := unmarshalBasicRepairImpactDescriptionBase(*rawMessage)
		if err != nil {
			return nil, err
		}
		ridbArray[index] = ridb
	}
	return ridbArray, nil
}

// MarshalJSON is the custom marshaler for RepairImpactDescriptionBase.
func (ridb RepairImpactDescriptionBase) MarshalJSON() ([]byte, error) {
	ridb.Kind = KindBasicRepairImpactDescriptionBaseKindRepairImpactDescriptionBase
	objectMap := make(map[string]interface{})
	if ridb.Kind != "" {
		objectMap["Kind"] = ridb.Kind
	}
	return json.Marshal(objectMap)
}

// AsNodeRepairImpactDescription is the BasicRepairImpactDescriptionBase implementation for RepairImpactDescriptionBase.
func (ridb RepairImpactDescriptionBase) AsNodeRepairImpactDescription() (*NodeRepairImpactDescription, bool) {
	return nil, false
}

// AsRepairImpactDescriptionBase is the BasicRepairImpactDescriptionBase implementation for RepairImpactDescriptionBase.
func (ridb RepairImpactDescriptionBase) AsRepairImpactDescriptionBase() (*RepairImpactDescriptionBase, bool) {
	return &ridb, true
}

// AsBasicRepairImpactDescriptionBase is the BasicRepairImpactDescriptionBase implementation for RepairImpactDescriptionBase.
func (ridb RepairImpactDescriptionBase) AsBasicRepairImpactDescriptionBase() (BasicRepairImpactDescriptionBase, bool) {
	return &ridb, true
}

// BasicRepairTargetDescriptionBase describes the entities targeted by a repair action.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type BasicRepairTargetDescriptionBase interface {
	AsNodeRepairTargetDescription() (*NodeRepairTargetDescription, bool)
	AsRepairTargetDescriptionBase() (*RepairTargetDescriptionBase, bool)
}

// RepairTargetDescriptionBase describes the entities targeted by a repair action.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type RepairTargetDescriptionBase struct {
	// Kind - Possible values include: 'KindBasicRepairTargetDescriptionBaseKindRepairTargetDescriptionBase', 'KindBasicRepairTargetDescriptionBaseKindNode'
	Kind KindBasicRepairTargetDescriptionBase `json:"Kind,omitempty"`
}

func unmarshalBasicRepairTargetDescriptionBase(body []byte) (BasicRepairTargetDescriptionBase, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindBasicRepairTargetDescriptionBaseKindNode):
		var nrtd NodeRepairTargetDescription
		err := json.Unmarshal(body, &nrtd)
		return nrtd, err
	default:
		var rtdb RepairTargetDescriptionBase
		err := json.Unmarshal(body, &rtdb)
		return rtdb, err
	}
}
func unmarshalBasicRepairTargetDescriptionBaseArray(body []byte) ([]BasicRepairTargetDescriptionBase, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	rtdbArray := make([]BasicRepairTargetDescriptionBase, len(rawMessages))

	for index, rawMessage := range rawMessages {
		rtdb, err := unmarshalBasicRepairTargetDescriptionBase(*rawMessage)
		if err != nil {
			return nil, err
		}
		rtdbArray[index] = rtdb
	}
	return rtdbArray, nil
}

// MarshalJSON is the custom marshaler for RepairTargetDescriptionBase.
func (rtdb RepairTargetDescriptionBase) MarshalJSON() ([]byte, error) {
	rtdb.Kind = KindBasicRepairTargetDescriptionBaseKindRepairTargetDescriptionBase
	objectMap := make(map[string]interface{})
	if rtdb.Kind != "" {
		objectMap["Kind"] = rtdb.Kind
	}
	return json.Marshal(objectMap)
}

// AsNodeRepairTargetDescription is the BasicRepairTargetDescriptionBase implementation for RepairTargetDescriptionBase.
func (rtdb RepairTargetDescriptionBase) AsNodeRepairTargetDescription() (*NodeRepairTargetDescription, bool) {
	return nil, false
}

// AsRepairTargetDescriptionBase is the BasicRepairTargetDescriptionBase implementation for RepairTargetDescriptionBase.
func (rtdb RepairTargetDescriptionBase) AsRepairTargetDescriptionBase() (*RepairTargetDescriptionBase, bool) {
	return &rtdb, true
}

// AsBasicRepairTargetDescriptionBase is the BasicRepairTargetDescriptionBase implementation for RepairTargetDescriptionBase.
func (rtdb RepairTargetDescriptionBase) AsBasicRepairTargetDescriptionBase() (BasicRepairTargetDescriptionBase, bool) {
	return &rtdb, true
}

// RepairTask represents a repair task, which includes information about what kind of repair was requested,
// what its progress is, and what its final result was.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type RepairTask struct {
	// TaskID - The ID of the repair task.
	TaskID *string `json:"TaskId,omitempty"`
	// Version - The version of the repair task.
	// When creating a new repair task, the version must be set to zero.  When updating a repair task,
	// the version is used for optimistic concurrency checks.  If the version is
	// set to zero, the update will not check for write conflicts.  If the version is set to a non-zero value, then the
	// update will only succeed if the actual current version of the repair task matches this value.
	Version *string `json:"Version,omitempty"`
	// Description - A description of the purpose of the repair task, or other informational details.
	// May be set when the repair task is created, and is immutable once set.
	Description *string `json:"Description,omitempty"`
	// State - The workflow state of the repair task. Valid initial states are Created, Claimed, and Preparing. Possible values include: 'StateInvalid', 'StateCreated', 'StateClaimed', 'StatePreparing', 'StateApproved', 'StateExecuting', 'StateRestoring', 'StateCompleted'
	State State `json:"State,omitempty"`
	// Flags - A bitwise-OR of the following values, which gives additional details about the status of the repair task.
	// - 1 - Cancellation of the repair has been requested
	// - 2 - Abort of the repair has been requested
	// - 4 - Approval of the repair was forced via client request
	Flags *int32 `json:"Flags,omitempty"`
	// Action - The requested repair action. Must be specified when the repair task is created, and is immutable once set.
	Action *string `json:"Action,omitempty"`
	// Target - The target object determines what actions the system will take to prepare for the impact of the repair, prior to approving execution of the repair.
	// May be set when the repair task is created, and is immutable once set.
	Target BasicRepairTargetDescriptionBase `json:"Target,omitempty"`
	// Executor - The name of the repair executor. Must be specified in Claimed and later states, and is immutable once set.
	Executor *string `json:"Executor,omitempty"`
	// ExecutorData - A data string that the repair executor can use to store its internal state.
	ExecutorData *string `json:"ExecutorData,omitempty"`
	// Impact - The impact object determines what actions the system will take to prepare for the impact of the repair, prior to approving execution of the repair.
	// Impact must be specified by the repair executor when transitioning to the Preparing state, and is immutable once set.
	Impact BasicRepairImpactDescriptionBase `json:"Impact,omitempty"`
	// ResultStatus - A value describing the overall result of the repair task execution. Must be specified in the Restoring and later states, and is immutable once set. Possible values include: 'ResultStatusInvalid', 'ResultStatusSucceeded', 'ResultStatusCancelled', 'ResultStatusInterrupted', 'ResultStatusFailed', 'ResultStatusPending'
	ResultStatus ResultStatus `json:"ResultStatus,omitempty"`
	// ResultCode - A numeric value providing additional details about the result of the repair task execution.
	// May be specified in the Restoring and later states, and is immutable once set.
	ResultCode *int32 `json:"ResultCode,omitempty"`
	// ResultDetails - A string providing additional details about the result of the repair task execution.
	// May be specified in the Restoring and later states, and is immutable once set.
	ResultDetails *string `json:"ResultDetails,omitempty"`
	// History - An object that contains timestamps of the repair task's state transitions.
	// These timestamps are updated by the system, and cannot be directly modified.
	History *RepairTaskHistory `json:"History,omitempty"`
	// PreparingHealthCheckState - The workflow state of the health check when the repair task is in the Preparing state. Possible values include: 'NotStarted', 'InProgress', 'Succeeded', 'Skipped', 'TimedOut'
	PreparingHealthCheckState RepairTaskHealthCheckState `json:"PreparingHealthCheckState,omitempty"`
	// RestoringHealthCheckState - The workflow state of the health check when the repair task is in the Restoring state. Possible values include: 'NotStarted', 'InProgress', 'Succeeded', 'Skipped', 'TimedOut'
	RestoringHealthCheckState RepairTaskHealthCheckState `json:"RestoringHealthCheckState,omitempty"`
	// PerformPreparingHealthCheck - A value to determine if health checks will be performed when the repair task enters the Preparing state.
	PerformPreparingHealthCheck *bool `json:"PerformPreparingHealthCheck,omitempty"`
	// PerformRestoringHealthCheck - A value to determine if health checks will be performed when the repair task enters the Restoring state.
	PerformRestoringHealthCheck *bool `json:"PerformRestoringHealthCheck,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for RepairTask struct.
func (rt *RepairTask) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "TaskId":
			if v != nil {
				var taskID string
				err = json.Unmarshal(*v, &taskID)
				if err != nil {
					return err
				}
				rt.TaskID = &taskID
			}
		case "Version":
			if v != nil {
				var version string
				err = json.Unmarshal(*v, &version)
				if err != nil {
					return err
				}
				rt.Version = &version
			}
		case "Description":
			if v != nil {
				var description string
				err = json.Unmarshal(*v, &description)
				if err != nil {
					return err
				}
				rt.Description = &description
			}
		case "State":
			if v != nil {
				var state State
				err = json.Unmarshal(*v, &state)
				if err != nil {
					return err
				}
				rt.State = state
			}
		case "Flags":
			if v != nil {
				var flags int32
				err = json.Unmarshal(*v, &flags)
				if err != nil {
					return err
				}
				rt.Flags = &flags
			}
		case "Action":
			if v != nil {
				var action string
				err = json.Unmarshal(*v, &action)
				if err != nil {
					return err
				}
				rt.Action = &action
			}
		case "Target":
			if v != nil {
				target, err := unmarshalBasicRepairTargetDescriptionBase(*v)
				if err != nil {
					return err
				}
				rt.Target = target
			}
		case "Executor":
			if v != nil {
				var executor string
				err = json.Unmarshal(*v, &executor)
				if err != nil {
					return err
				}
				rt.Executor = &executor
			}
		case "ExecutorData":
			if v != nil {
				var executorData string
				err = json.Unmarshal(*v, &executorData)
				if err != nil {
					return err
				}
				rt.ExecutorData = &executorData
			}
		case "Impact":
			if v != nil {
				impact, err := unmarshalBasicRepairImpactDescriptionBase(*v)
				if err != nil {
					return err
				}
				rt.Impact = impact
			}
		case "ResultStatus":
			if v != nil {
				var resultStatus ResultStatus
				err = json.Unmarshal(*v, &resultStatus)
				if err != nil {
					return err
				}
				rt.ResultStatus = resultStatus
			}
		case "ResultCode":
			if v != nil {
				var resultCode int32
				err = json.Unmarshal(*v, &resultCode)
				if err != nil {
					return err
				}
				rt.ResultCode = &resultCode
			}
		case "ResultDetails":
			if v != nil {
				var resultDetails string
				err = json.Unmarshal(*v, &resultDetails)
				if err != nil {
					return err
				}
				rt.ResultDetails = &resultDetails
			}
		case "History":
			if v != nil {
				var history RepairTaskHistory
				err = json.Unmarshal(*v, &history)
				if err != nil {
					return err
				}
				rt.History = &history
			}
		case "PreparingHealthCheckState":
			if v != nil {
				var preparingHealthCheckState RepairTaskHealthCheckState
				err = json.Unmarshal(*v, &preparingHealthCheckState)
				if err != nil {
					return err
				}
				rt.PreparingHealthCheckState = preparingHealthCheckState
			}
		case "RestoringHealthCheckState":
			if v != nil {
				var restoringHealthCheckState RepairTaskHealthCheckState
				err = json.Unmarshal(*v, &restoringHealthCheckState)
				if err != nil {
					return err
				}
				rt.RestoringHealthCheckState = restoringHealthCheckState
			}
		case "PerformPreparingHealthCheck":
			if v != nil {
				var performPreparingHealthCheck bool
				err = json.Unmarshal(*v, &performPreparingHealthCheck)
				if err != nil {
					return err
				}
				rt.PerformPreparingHealthCheck = &performPreparingHealthCheck
			}
		case "PerformRestoringHealthCheck":
			if v != nil {
				var performRestoringHealthCheck bool
				err = json.Unmarshal(*v, &performRestoringHealthCheck)
				if err != nil {
					return err
				}
				rt.PerformRestoringHealthCheck = &performRestoringHealthCheck
			}
		}
	}

	return nil
}

// RepairTaskApproveDescription describes a request for forced approval of a repair task.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type RepairTaskApproveDescription struct {
	// TaskID - The ID of the repair task.
	TaskID *string `json:"TaskId,omitempty"`
	// Version - The current version number of the repair task. If non-zero, then the request will only succeed if this value matches the actual current version of the repair task. If zero, then no version check is performed.
	Version *string `json:"Version,omitempty"`
}

// RepairTaskCancelDescription describes a request to cancel a repair task.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type RepairTaskCancelDescription struct {
	// TaskID - The ID of the repair task.
	TaskID *string `json:"TaskId,omitempty"`
	// Version - The current version number of the repair task. If non-zero, then the request will only succeed if this value matches the actual current version of the repair task. If zero, then no version check is performed.
	Version *string `json:"Version,omitempty"`
	// RequestAbort - _True_ if the repair should be stopped as soon as possible even if it has already started executing. _False_ if the repair should be cancelled only if execution has not yet started.
	RequestAbort *bool `json:"RequestAbort,omitempty"`
}

// RepairTaskDeleteDescription describes a request to delete a completed repair task.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type RepairTaskDeleteDescription struct {
	// TaskID - The ID of the completed repair task to be deleted.
	TaskID *string `json:"TaskId,omitempty"`
	// Version - The current version number of the repair task. If non-zero, then the request will only succeed if this value matches the actual current version of the repair task. If zero, then no version check is performed.
	Version *string `json:"Version,omitempty"`
}

// RepairTaskHistory a record of the times when the repair task entered each state.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type RepairTaskHistory struct {
	// CreatedUtcTimestamp - The time when the repair task entered the Created state.
	CreatedUtcTimestamp *date.Time `json:"CreatedUtcTimestamp,omitempty"`
	// ClaimedUtcTimestamp - The time when the repair task entered the Claimed state.
	ClaimedUtcTimestamp *date.Time `json:"ClaimedUtcTimestamp,omitempty"`
	// PreparingUtcTimestamp - The time when the repair task entered the Preparing state.
	PreparingUtcTimestamp *date.Time `json:"PreparingUtcTimestamp,omitempty"`
	// ApprovedUtcTimestamp - The time when the repair task entered the Approved state
	ApprovedUtcTimestamp *date.Time `json:"ApprovedUtcTimestamp,omitempty"`
	// ExecutingUtcTimestamp - The time when the repair task entered the Executing state
	ExecutingUtcTimestamp *date.Time `json:"ExecutingUtcTimestamp,omitempty"`
	// RestoringUtcTimestamp - The time when the repair task entered the Restoring state
	RestoringUtcTimestamp *date.Time `json:"RestoringUtcTimestamp,omitempty"`
	// CompletedUtcTimestamp - The time when the repair task entered the Completed state
	CompletedUtcTimestamp *date.Time `json:"CompletedUtcTimestamp,omitempty"`
	// PreparingHealthCheckStartUtcTimestamp - The time when the repair task started the health check in the Preparing state.
	PreparingHealthCheckStartUtcTimestamp *date.Time `json:"PreparingHealthCheckStartUtcTimestamp,omitempty"`
	// PreparingHealthCheckEndUtcTimestamp - The time when the repair task completed the health check in the Preparing state.
	PreparingHealthCheckEndUtcTimestamp *date.Time `json:"PreparingHealthCheckEndUtcTimestamp,omitempty"`
	// RestoringHealthCheckStartUtcTimestamp - The time when the repair task started the health check in the Restoring state.
	RestoringHealthCheckStartUtcTimestamp *date.Time `json:"RestoringHealthCheckStartUtcTimestamp,omitempty"`
	// RestoringHealthCheckEndUtcTimestamp - The time when the repair task completed the health check in the Restoring state.
	RestoringHealthCheckEndUtcTimestamp *date.Time `json:"RestoringHealthCheckEndUtcTimestamp,omitempty"`
}

// RepairTaskUpdateHealthPolicyDescription describes a request to update the health policy of a repair
// task.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type RepairTaskUpdateHealthPolicyDescription struct {
	// TaskID - The ID of the repair task to be updated.
	TaskID *string `json:"TaskId,omitempty"`
	// Version - The current version number of the repair task. If non-zero, then the request will only succeed if this value matches the actual current value of the repair task. If zero, then no version check is performed.
	Version *string `json:"Version,omitempty"`
	// PerformPreparingHealthCheck - A boolean indicating if health check is to be performed in the Preparing stage of the repair task. If not specified the existing value should not be altered. Otherwise, specify the desired new value.
	PerformPreparingHealthCheck *bool `json:"PerformPreparingHealthCheck,omitempty"`
	// PerformRestoringHealthCheck - A boolean indicating if health check is to be performed in the Restoring stage of the repair task. If not specified the existing value should not be altered. Otherwise, specify the desired new value.
	PerformRestoringHealthCheck *bool `json:"PerformRestoringHealthCheck,omitempty"`
}

// RepairTaskUpdateInfo describes the result of an operation that created or updated a repair task.
//
// This type supports the Service Fabric platform; it is not meant to be used directly from your code.
type RepairTaskUpdateInfo struct {
	autorest.Response `json:"-"`
	// Version - The new version of the repair task.
	Version *string `json:"Version,omitempty"`
}

// BasicReplicaEvent represents the base for all Replica Events.
type BasicReplicaEvent interface {
	AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool)
	AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool)
	AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool)
	AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool)
	AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool)
	AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool)
	AsReplicaEvent() (*ReplicaEvent, bool)
}

// ReplicaEvent represents the base for all Replica Events.
type ReplicaEvent struct {
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *int64 `json:"ReplicaId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

func unmarshalBasicReplicaEvent(body []byte) (BasicReplicaEvent, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindStatefulReplicaNewHealthReport):
		var srnhre StatefulReplicaNewHealthReportEvent
		err := json.Unmarshal(body, &srnhre)
		return srnhre, err
	case string(KindStatefulReplicaHealthReportExpired):
		var srhree StatefulReplicaHealthReportExpiredEvent
		err := json.Unmarshal(body, &srhree)
		return srhree, err
	case string(KindStatelessReplicaNewHealthReport):
		var srnhre StatelessReplicaNewHealthReportEvent
		err := json.Unmarshal(body, &srnhre)
		return srnhre, err
	case string(KindStatelessReplicaHealthReportExpired):
		var srhree StatelessReplicaHealthReportExpiredEvent
		err := json.Unmarshal(body, &srhree)
		return srhree, err
	case string(KindChaosReplicaRemovalScheduled):
		var crrse ChaosReplicaRemovalScheduledEvent
		err := json.Unmarshal(body, &crrse)
		return crrse, err
	case string(KindChaosReplicaRestartScheduled):
		var crrse ChaosReplicaRestartScheduledEvent
		err := json.Unmarshal(body, &crrse)
		return crrse, err
	default:
		var re ReplicaEvent
		err := json.Unmarshal(body, &re)
		return re, err
	}
}
func unmarshalBasicReplicaEventArray(body []byte) ([]BasicReplicaEvent, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	reArray := make([]BasicReplicaEvent, len(rawMessages))

	for index, rawMessage := range rawMessages {
		re, err := unmarshalBasicReplicaEvent(*rawMessage)
		if err != nil {
			return nil, err
		}
		reArray[index] = re
	}
	return reArray, nil
}

// MarshalJSON is the custom marshaler for ReplicaEvent.
func (re ReplicaEvent) MarshalJSON() ([]byte, error) {
	re.Kind = KindReplicaEvent
	objectMap := make(map[string]interface{})
	if re.PartitionID != nil {
		objectMap["PartitionId"] = re.PartitionID
	}
	if re.ReplicaID != nil {
		objectMap["ReplicaId"] = re.ReplicaID
	}
	if re.EventInstanceID != nil {
		objectMap["EventInstanceId"] = re.EventInstanceID
	}
	if re.Category != nil {
		objectMap["Category"] = re.Category
	}
	if re.TimeStamp != nil {
		objectMap["TimeStamp"] = re.TimeStamp
	}
	if re.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = re.HasCorrelatedEvents
	}
	if re.Kind != "" {
		objectMap["Kind"] = re.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return &re, true
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return &re, true
}

// AsServiceEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ReplicaEvent.
func (re ReplicaEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &re, true
}

// BasicReplicaHealth represents a base class for stateful service replica or stateless service instance health.
// Contains the replica aggregated health state, the health events and the unhealthy evaluations.
type BasicReplicaHealth interface {
	AsStatefulServiceReplicaHealth() (*StatefulServiceReplicaHealth, bool)
	AsStatelessServiceInstanceHealth() (*StatelessServiceInstanceHealth, bool)
	AsReplicaHealth() (*ReplicaHealth, bool)
}

// ReplicaHealth represents a base class for stateful service replica or stateless service instance health.
// Contains the replica aggregated health state, the health events and the unhealthy evaluations.
type ReplicaHealth struct {
	autorest.Response `json:"-"`
	// PartitionID - Id of the partition to which this replica belongs.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicReplicaHealthServiceKindReplicaHealth', 'ServiceKindBasicReplicaHealthServiceKindStateful', 'ServiceKindBasicReplicaHealthServiceKindStateless'
	ServiceKind ServiceKindBasicReplicaHealth `json:"ServiceKind,omitempty"`
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

func unmarshalBasicReplicaHealth(body []byte) (BasicReplicaHealth, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ServiceKind"] {
	case string(ServiceKindBasicReplicaHealthServiceKindStateful):
		var ssrh StatefulServiceReplicaHealth
		err := json.Unmarshal(body, &ssrh)
		return ssrh, err
	case string(ServiceKindBasicReplicaHealthServiceKindStateless):
		var ssih StatelessServiceInstanceHealth
		err := json.Unmarshal(body, &ssih)
		return ssih, err
	default:
		var rh ReplicaHealth
		err := json.Unmarshal(body, &rh)
		return rh, err
	}
}
func unmarshalBasicReplicaHealthArray(body []byte) ([]BasicReplicaHealth, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	rhArray := make([]BasicReplicaHealth, len(rawMessages))

	for index, rawMessage := range rawMessages {
		rh, err := unmarshalBasicReplicaHealth(*rawMessage)
		if err != nil {
			return nil, err
		}
		rhArray[index] = rh
	}
	return rhArray, nil
}

// MarshalJSON is the custom marshaler for ReplicaHealth.
func (rh ReplicaHealth) MarshalJSON() ([]byte, error) {
	rh.ServiceKind = ServiceKindBasicReplicaHealthServiceKindReplicaHealth
	objectMap := make(map[string]interface{})
	if rh.PartitionID != nil {
		objectMap["PartitionId"] = rh.PartitionID
	}
	if rh.ServiceKind != "" {
		objectMap["ServiceKind"] = rh.ServiceKind
	}
	if rh.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = rh.AggregatedHealthState
	}
	if rh.HealthEvents != nil {
		objectMap["HealthEvents"] = rh.HealthEvents
	}
	if rh.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = rh.UnhealthyEvaluations
	}
	if rh.HealthStatistics != nil {
		objectMap["HealthStatistics"] = rh.HealthStatistics
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceReplicaHealth is the BasicReplicaHealth implementation for ReplicaHealth.
func (rh ReplicaHealth) AsStatefulServiceReplicaHealth() (*StatefulServiceReplicaHealth, bool) {
	return nil, false
}

// AsStatelessServiceInstanceHealth is the BasicReplicaHealth implementation for ReplicaHealth.
func (rh ReplicaHealth) AsStatelessServiceInstanceHealth() (*StatelessServiceInstanceHealth, bool) {
	return nil, false
}

// AsReplicaHealth is the BasicReplicaHealth implementation for ReplicaHealth.
func (rh ReplicaHealth) AsReplicaHealth() (*ReplicaHealth, bool) {
	return &rh, true
}

// AsBasicReplicaHealth is the BasicReplicaHealth implementation for ReplicaHealth.
func (rh ReplicaHealth) AsBasicReplicaHealth() (BasicReplicaHealth, bool) {
	return &rh, true
}

// ReplicaHealthEvaluation represents health evaluation for a replica, containing information about the
// data and the algorithm used by health store to evaluate health. The evaluation is returned only when the
// aggregated health state is either Error or Warning.
type ReplicaHealthEvaluation struct {
	// PartitionID - Id of the partition to which the replica belongs.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaOrInstanceID - Id of a stateful service replica or a stateless service instance. This ID is used in the queries that apply to both stateful and stateless services. It is used by Service Fabric to uniquely identify a replica of a partition of a stateful service or an instance of a stateless service partition. It is unique within a partition and does not change for the lifetime of the replica or the instance. If a stateful replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the ID. If a stateless instance is failed over on the same or different node it will get a different value for the ID.
	ReplicaOrInstanceID *string `json:"ReplicaOrInstanceId,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the current aggregated health state of the replica. The types of the unhealthy evaluations can be EventHealthEvaluation.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) MarshalJSON() ([]byte, error) {
	rhe.Kind = KindReplica
	objectMap := make(map[string]interface{})
	if rhe.PartitionID != nil {
		objectMap["PartitionId"] = rhe.PartitionID
	}
	if rhe.ReplicaOrInstanceID != nil {
		objectMap["ReplicaOrInstanceId"] = rhe.ReplicaOrInstanceID
	}
	if rhe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = rhe.UnhealthyEvaluations
	}
	if rhe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = rhe.AggregatedHealthState
	}
	if rhe.Description != nil {
		objectMap["Description"] = rhe.Description
	}
	if rhe.Kind != "" {
		objectMap["Kind"] = rhe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return &rhe, true
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for ReplicaHealthEvaluation.
func (rhe ReplicaHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &rhe, true
}

// ReplicaHealthModel ...
type ReplicaHealthModel struct {
	autorest.Response `json:"-"`
	Value             BasicReplicaHealth `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ReplicaHealthModel struct.
func (rhm *ReplicaHealthModel) UnmarshalJSON(body []byte) error {
	rh, err := unmarshalBasicReplicaHealth(body)
	if err != nil {
		return err
	}
	rhm.Value = rh

	return nil
}

// BasicReplicaHealthState represents a base class for stateful service replica or stateless service instance health
// state.
type BasicReplicaHealthState interface {
	AsStatefulServiceReplicaHealthState() (*StatefulServiceReplicaHealthState, bool)
	AsStatelessServiceInstanceHealthState() (*StatelessServiceInstanceHealthState, bool)
	AsReplicaHealthState() (*ReplicaHealthState, bool)
}

// ReplicaHealthState represents a base class for stateful service replica or stateless service instance health
// state.
type ReplicaHealthState struct {
	// PartitionID - The ID of the partition to which this replica belongs.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicReplicaHealthStateServiceKindReplicaHealthState', 'ServiceKindBasicReplicaHealthStateServiceKindStateful', 'ServiceKindBasicReplicaHealthStateServiceKindStateless'
	ServiceKind ServiceKindBasicReplicaHealthState `json:"ServiceKind,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
}

func unmarshalBasicReplicaHealthState(body []byte) (BasicReplicaHealthState, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ServiceKind"] {
	case string(ServiceKindBasicReplicaHealthStateServiceKindStateful):
		var ssrhs StatefulServiceReplicaHealthState
		err := json.Unmarshal(body, &ssrhs)
		return ssrhs, err
	case string(ServiceKindBasicReplicaHealthStateServiceKindStateless):
		var ssihs StatelessServiceInstanceHealthState
		err := json.Unmarshal(body, &ssihs)
		return ssihs, err
	default:
		var RHS ReplicaHealthState
		err := json.Unmarshal(body, &RHS)
		return RHS, err
	}
}
func unmarshalBasicReplicaHealthStateArray(body []byte) ([]BasicReplicaHealthState, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	RHSArray := make([]BasicReplicaHealthState, len(rawMessages))

	for index, rawMessage := range rawMessages {
		RHS, err := unmarshalBasicReplicaHealthState(*rawMessage)
		if err != nil {
			return nil, err
		}
		RHSArray[index] = RHS
	}
	return RHSArray, nil
}

// MarshalJSON is the custom marshaler for ReplicaHealthState.
func (RHS ReplicaHealthState) MarshalJSON() ([]byte, error) {
	RHS.ServiceKind = ServiceKindBasicReplicaHealthStateServiceKindReplicaHealthState
	objectMap := make(map[string]interface{})
	if RHS.PartitionID != nil {
		objectMap["PartitionId"] = RHS.PartitionID
	}
	if RHS.ServiceKind != "" {
		objectMap["ServiceKind"] = RHS.ServiceKind
	}
	if RHS.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = RHS.AggregatedHealthState
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceReplicaHealthState is the BasicReplicaHealthState implementation for ReplicaHealthState.
func (RHS ReplicaHealthState) AsStatefulServiceReplicaHealthState() (*StatefulServiceReplicaHealthState, bool) {
	return nil, false
}

// AsStatelessServiceInstanceHealthState is the BasicReplicaHealthState implementation for ReplicaHealthState.
func (RHS ReplicaHealthState) AsStatelessServiceInstanceHealthState() (*StatelessServiceInstanceHealthState, bool) {
	return nil, false
}

// AsReplicaHealthState is the BasicReplicaHealthState implementation for ReplicaHealthState.
func (RHS ReplicaHealthState) AsReplicaHealthState() (*ReplicaHealthState, bool) {
	return &RHS, true
}

// AsBasicReplicaHealthState is the BasicReplicaHealthState implementation for ReplicaHealthState.
func (RHS ReplicaHealthState) AsBasicReplicaHealthState() (BasicReplicaHealthState, bool) {
	return &RHS, true
}

// ReplicaHealthStateChunk represents the health state chunk of a stateful service replica or a stateless
// service instance.
// The replica health state contains the replica ID and its aggregated health state.
type ReplicaHealthStateChunk struct {
	// ReplicaOrInstanceID - Id of a stateful service replica or a stateless service instance. This ID is used in the queries that apply to both stateful and stateless services. It is used by Service Fabric to uniquely identify a replica of a partition of a stateful service or an instance of a stateless service partition. It is unique within a partition and does not change for the lifetime of the replica or the instance. If a stateful replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the ID. If a stateless instance is failed over on the same or different node it will get a different value for the ID.
	ReplicaOrInstanceID *string `json:"ReplicaOrInstanceId,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
}

// ReplicaHealthStateChunkList the list of replica health state chunks that respect the input filters in
// the chunk query. Returned by get cluster health state chunks query.
type ReplicaHealthStateChunkList struct {
	// Items - The list of replica health state chunks that respect the input filters in the chunk query.
	Items *[]ReplicaHealthStateChunk `json:"Items,omitempty"`
}

// ReplicaHealthStateFilter defines matching criteria to determine whether a replica should be included as
// a child of a partition in the cluster health chunk.
// The replicas are only returned if the parent entities match a filter specified in the cluster health
// chunk query description. The parent partition, service and application must be included in the cluster
// health chunk.
// One filter can match zero, one or multiple replicas, depending on its properties.
type ReplicaHealthStateFilter struct {
	// ReplicaOrInstanceIDFilter - Id of the stateful service replica or stateless service instance that matches the filter. The filter is applied only to the specified replica, if it exists.
	// If the replica doesn't exist, no replica is returned in the cluster health chunk based on this filter.
	// If the replica exists, it is included in the cluster health chunk if it respects the other filter properties.
	// If not specified, all replicas that match the parent filters (if any) are taken into consideration and matched against the other filter members, like health state filter.
	ReplicaOrInstanceIDFilter *string `json:"ReplicaOrInstanceIdFilter,omitempty"`
	// HealthStateFilter - The filter for the health state of the replicas. It allows selecting replicas if they match the desired health states.
	// The possible values are integer value of one of the following health states. Only replicas that match the filter are returned. All replicas are used to evaluate the parent partition aggregated health state.
	// If not specified, default value is None, unless the replica ID is specified. If the filter has default value and replica ID is specified, the matching replica is returned.
	// The state values are flag-based enumeration, so the value could be a combination of these values obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6, it matches replicas with HealthState value of OK (2) and Warning (4).
	// - Default - Default value. Matches any HealthState. The value is zero.
	// - None - Filter that doesn't match any HealthState value. Used in order to return no results on a given collection of states. The value is 1.
	// - Ok - Filter that matches input with HealthState value Ok. The value is 2.
	// - Warning - Filter that matches input with HealthState value Warning. The value is 4.
	// - Error - Filter that matches input with HealthState value Error. The value is 8.
	// - All - Filter that matches input with any HealthState value. The value is 65535.
	HealthStateFilter *int32 `json:"HealthStateFilter,omitempty"`
}

// BasicReplicaInfo information about the identity, status, health, node name, uptime, and other details about the
// replica.
type BasicReplicaInfo interface {
	AsStatefulServiceReplicaInfo() (*StatefulServiceReplicaInfo, bool)
	AsStatelessServiceInstanceInfo() (*StatelessServiceInstanceInfo, bool)
	AsReplicaInfo() (*ReplicaInfo, bool)
}

// ReplicaInfo information about the identity, status, health, node name, uptime, and other details about the
// replica.
type ReplicaInfo struct {
	autorest.Response `json:"-"`
	// ReplicaStatus - The status of a replica of a service. Possible values include: 'ReplicaStatusInvalid', 'ReplicaStatusInBuild', 'ReplicaStatusStandby', 'ReplicaStatusReady', 'ReplicaStatusDown', 'ReplicaStatusDropped'
	ReplicaStatus ReplicaStatus `json:"ReplicaStatus,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// Address - The address the replica is listening on.
	Address *string `json:"Address,omitempty"`
	// LastInBuildDurationInSeconds - The last in build duration of the replica in seconds.
	LastInBuildDurationInSeconds *string `json:"LastInBuildDurationInSeconds,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicReplicaInfoServiceKindReplicaInfo', 'ServiceKindBasicReplicaInfoServiceKindStateful', 'ServiceKindBasicReplicaInfoServiceKindStateless'
	ServiceKind ServiceKindBasicReplicaInfo `json:"ServiceKind,omitempty"`
}

func unmarshalBasicReplicaInfo(body []byte) (BasicReplicaInfo, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ServiceKind"] {
	case string(ServiceKindBasicReplicaInfoServiceKindStateful):
		var ssri StatefulServiceReplicaInfo
		err := json.Unmarshal(body, &ssri)
		return ssri, err
	case string(ServiceKindBasicReplicaInfoServiceKindStateless):
		var ssii StatelessServiceInstanceInfo
		err := json.Unmarshal(body, &ssii)
		return ssii, err
	default:
		var ri ReplicaInfo
		err := json.Unmarshal(body, &ri)
		return ri, err
	}
}
func unmarshalBasicReplicaInfoArray(body []byte) ([]BasicReplicaInfo, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	riArray := make([]BasicReplicaInfo, len(rawMessages))

	for index, rawMessage := range rawMessages {
		ri, err := unmarshalBasicReplicaInfo(*rawMessage)
		if err != nil {
			return nil, err
		}
		riArray[index] = ri
	}
	return riArray, nil
}

// MarshalJSON is the custom marshaler for ReplicaInfo.
func (ri ReplicaInfo) MarshalJSON() ([]byte, error) {
	ri.ServiceKind = ServiceKindBasicReplicaInfoServiceKindReplicaInfo
	objectMap := make(map[string]interface{})
	if ri.ReplicaStatus != "" {
		objectMap["ReplicaStatus"] = ri.ReplicaStatus
	}
	if ri.HealthState != "" {
		objectMap["HealthState"] = ri.HealthState
	}
	if ri.NodeName != nil {
		objectMap["NodeName"] = ri.NodeName
	}
	if ri.Address != nil {
		objectMap["Address"] = ri.Address
	}
	if ri.LastInBuildDurationInSeconds != nil {
		objectMap["LastInBuildDurationInSeconds"] = ri.LastInBuildDurationInSeconds
	}
	if ri.ServiceKind != "" {
		objectMap["ServiceKind"] = ri.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceReplicaInfo is the BasicReplicaInfo implementation for ReplicaInfo.
func (ri ReplicaInfo) AsStatefulServiceReplicaInfo() (*StatefulServiceReplicaInfo, bool) {
	return nil, false
}

// AsStatelessServiceInstanceInfo is the BasicReplicaInfo implementation for ReplicaInfo.
func (ri ReplicaInfo) AsStatelessServiceInstanceInfo() (*StatelessServiceInstanceInfo, bool) {
	return nil, false
}

// AsReplicaInfo is the BasicReplicaInfo implementation for ReplicaInfo.
func (ri ReplicaInfo) AsReplicaInfo() (*ReplicaInfo, bool) {
	return &ri, true
}

// AsBasicReplicaInfo is the BasicReplicaInfo implementation for ReplicaInfo.
func (ri ReplicaInfo) AsBasicReplicaInfo() (BasicReplicaInfo, bool) {
	return &ri, true
}

// ReplicaInfoModel ...
type ReplicaInfoModel struct {
	autorest.Response `json:"-"`
	Value             BasicReplicaInfo `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ReplicaInfoModel struct.
func (rim *ReplicaInfoModel) UnmarshalJSON(body []byte) error {
	ri, err := unmarshalBasicReplicaInfo(body)
	if err != nil {
		return err
	}
	rim.Value = ri

	return nil
}

// ReplicasHealthEvaluation represents health evaluation for replicas, containing health evaluations for
// each unhealthy replica that impacted current aggregated health state. Can be returned when evaluating
// partition health and the aggregated health state is either Error or Warning.
type ReplicasHealthEvaluation struct {
	// MaxPercentUnhealthyReplicasPerPartition - Maximum allowed percentage of unhealthy replicas per partition from the ApplicationHealthPolicy.
	MaxPercentUnhealthyReplicasPerPartition *int32 `json:"MaxPercentUnhealthyReplicasPerPartition,omitempty"`
	// TotalCount - Total number of replicas in the partition from the health store.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state. Includes all the unhealthy ReplicaHealthEvaluation that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) MarshalJSON() ([]byte, error) {
	rhe.Kind = KindReplicas
	objectMap := make(map[string]interface{})
	if rhe.MaxPercentUnhealthyReplicasPerPartition != nil {
		objectMap["MaxPercentUnhealthyReplicasPerPartition"] = rhe.MaxPercentUnhealthyReplicasPerPartition
	}
	if rhe.TotalCount != nil {
		objectMap["TotalCount"] = rhe.TotalCount
	}
	if rhe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = rhe.UnhealthyEvaluations
	}
	if rhe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = rhe.AggregatedHealthState
	}
	if rhe.Description != nil {
		objectMap["Description"] = rhe.Description
	}
	if rhe.Kind != "" {
		objectMap["Kind"] = rhe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return &rhe, true
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for ReplicasHealthEvaluation.
func (rhe ReplicasHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &rhe, true
}

// BasicReplicaStatusBase information about the replica.
type BasicReplicaStatusBase interface {
	AsKeyValueStoreReplicaStatus() (*KeyValueStoreReplicaStatus, bool)
	AsReplicaStatusBase() (*ReplicaStatusBase, bool)
}

// ReplicaStatusBase information about the replica.
type ReplicaStatusBase struct {
	// Kind - Possible values include: 'KindReplicaStatusBase', 'KindKeyValueStore'
	Kind KindBasicReplicaStatusBase `json:"Kind,omitempty"`
}

func unmarshalBasicReplicaStatusBase(body []byte) (BasicReplicaStatusBase, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindKeyValueStore):
		var kvsrs KeyValueStoreReplicaStatus
		err := json.Unmarshal(body, &kvsrs)
		return kvsrs, err
	default:
		var rsb ReplicaStatusBase
		err := json.Unmarshal(body, &rsb)
		return rsb, err
	}
}
func unmarshalBasicReplicaStatusBaseArray(body []byte) ([]BasicReplicaStatusBase, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	rsbArray := make([]BasicReplicaStatusBase, len(rawMessages))

	for index, rawMessage := range rawMessages {
		rsb, err := unmarshalBasicReplicaStatusBase(*rawMessage)
		if err != nil {
			return nil, err
		}
		rsbArray[index] = rsb
	}
	return rsbArray, nil
}

// MarshalJSON is the custom marshaler for ReplicaStatusBase.
func (rsb ReplicaStatusBase) MarshalJSON() ([]byte, error) {
	rsb.Kind = KindReplicaStatusBase
	objectMap := make(map[string]interface{})
	if rsb.Kind != "" {
		objectMap["Kind"] = rsb.Kind
	}
	return json.Marshal(objectMap)
}

// AsKeyValueStoreReplicaStatus is the BasicReplicaStatusBase implementation for ReplicaStatusBase.
func (rsb ReplicaStatusBase) AsKeyValueStoreReplicaStatus() (*KeyValueStoreReplicaStatus, bool) {
	return nil, false
}

// AsReplicaStatusBase is the BasicReplicaStatusBase implementation for ReplicaStatusBase.
func (rsb ReplicaStatusBase) AsReplicaStatusBase() (*ReplicaStatusBase, bool) {
	return &rsb, true
}

// AsBasicReplicaStatusBase is the BasicReplicaStatusBase implementation for ReplicaStatusBase.
func (rsb ReplicaStatusBase) AsBasicReplicaStatusBase() (BasicReplicaStatusBase, bool) {
	return &rsb, true
}

// ReplicatorQueueStatus provides various statistics of the queue used in the service fabric replicator.
// Contains information about the service fabric replicator like the replication/copy queue utilization,
// last acknowledgement received timestamp, etc.
// Depending on the role of the replicator, the properties in this type imply different meanings.
type ReplicatorQueueStatus struct {
	// QueueUtilizationPercentage - Represents the utilization of the queue. A value of 0 indicates that the queue is empty and a value of 100 indicates the queue is full.
	QueueUtilizationPercentage *int32 `json:"QueueUtilizationPercentage,omitempty"`
	// QueueMemorySize - Represents the virtual memory consumed by the queue in bytes.
	QueueMemorySize *string `json:"QueueMemorySize,omitempty"`
	// FirstSequenceNumber - On a primary replicator, this is semantically the sequence number of the operation for which all the secondary replicas have sent an acknowledgement.
	// On a secondary replicator, this is the smallest sequence number of the operation that is present in the queue.
	FirstSequenceNumber *string `json:"FirstSequenceNumber,omitempty"`
	// CompletedSequenceNumber - On a primary replicator, this is semantically the highest sequence number of the operation for which all the secondary replicas have sent an acknowledgement.
	// On a secondary replicator, this is semantically the highest sequence number that has been applied to the persistent state.
	CompletedSequenceNumber *string `json:"CompletedSequenceNumber,omitempty"`
	// CommittedSequenceNumber - On a primary replicator, this is semantically the highest sequence number of the operation for which a write quorum of the secondary replicas have sent an acknowledgement.
	// On a secondary replicator, this is semantically the highest sequence number of the in-order operation received from the primary.
	CommittedSequenceNumber *string `json:"CommittedSequenceNumber,omitempty"`
	// LastSequenceNumber - Represents the latest sequence number of the operation that is available in the queue.
	LastSequenceNumber *string `json:"LastSequenceNumber,omitempty"`
}

// BasicReplicatorStatus represents a base class for primary or secondary replicator status.
// Contains information about the service fabric replicator like the replication/copy queue utilization, last
// acknowledgement received timestamp, etc.
type BasicReplicatorStatus interface {
	AsPrimaryReplicatorStatus() (*PrimaryReplicatorStatus, bool)
	AsSecondaryReplicatorStatus() (*SecondaryReplicatorStatus, bool)
	AsBasicSecondaryReplicatorStatus() (BasicSecondaryReplicatorStatus, bool)
	AsSecondaryActiveReplicatorStatus() (*SecondaryActiveReplicatorStatus, bool)
	AsSecondaryIdleReplicatorStatus() (*SecondaryIdleReplicatorStatus, bool)
	AsReplicatorStatus() (*ReplicatorStatus, bool)
}

// ReplicatorStatus represents a base class for primary or secondary replicator status.
// Contains information about the service fabric replicator like the replication/copy queue utilization, last
// acknowledgement received timestamp, etc.
type ReplicatorStatus struct {
	// Kind - Possible values include: 'KindReplicatorStatus', 'KindPrimary', 'KindSecondaryReplicatorStatus', 'KindActiveSecondary', 'KindIdleSecondary'
	Kind KindBasicReplicatorStatus `json:"Kind,omitempty"`
}

func unmarshalBasicReplicatorStatus(body []byte) (BasicReplicatorStatus, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindPrimary):
		var prs PrimaryReplicatorStatus
		err := json.Unmarshal(body, &prs)
		return prs, err
	case string(KindSecondaryReplicatorStatus):
		var srs SecondaryReplicatorStatus
		err := json.Unmarshal(body, &srs)
		return srs, err
	case string(KindActiveSecondary):
		var sars SecondaryActiveReplicatorStatus
		err := json.Unmarshal(body, &sars)
		return sars, err
	case string(KindIdleSecondary):
		var sirs SecondaryIdleReplicatorStatus
		err := json.Unmarshal(body, &sirs)
		return sirs, err
	default:
		var rs ReplicatorStatus
		err := json.Unmarshal(body, &rs)
		return rs, err
	}
}
func unmarshalBasicReplicatorStatusArray(body []byte) ([]BasicReplicatorStatus, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	rsArray := make([]BasicReplicatorStatus, len(rawMessages))

	for index, rawMessage := range rawMessages {
		rs, err := unmarshalBasicReplicatorStatus(*rawMessage)
		if err != nil {
			return nil, err
		}
		rsArray[index] = rs
	}
	return rsArray, nil
}

// MarshalJSON is the custom marshaler for ReplicatorStatus.
func (rs ReplicatorStatus) MarshalJSON() ([]byte, error) {
	rs.Kind = KindReplicatorStatus
	objectMap := make(map[string]interface{})
	if rs.Kind != "" {
		objectMap["Kind"] = rs.Kind
	}
	return json.Marshal(objectMap)
}

// AsPrimaryReplicatorStatus is the BasicReplicatorStatus implementation for ReplicatorStatus.
func (rs ReplicatorStatus) AsPrimaryReplicatorStatus() (*PrimaryReplicatorStatus, bool) {
	return nil, false
}

// AsSecondaryReplicatorStatus is the BasicReplicatorStatus implementation for ReplicatorStatus.
func (rs ReplicatorStatus) AsSecondaryReplicatorStatus() (*SecondaryReplicatorStatus, bool) {
	return nil, false
}

// AsBasicSecondaryReplicatorStatus is the BasicReplicatorStatus implementation for ReplicatorStatus.
func (rs ReplicatorStatus) AsBasicSecondaryReplicatorStatus() (BasicSecondaryReplicatorStatus, bool) {
	return nil, false
}

// AsSecondaryActiveReplicatorStatus is the BasicReplicatorStatus implementation for ReplicatorStatus.
func (rs ReplicatorStatus) AsSecondaryActiveReplicatorStatus() (*SecondaryActiveReplicatorStatus, bool) {
	return nil, false
}

// AsSecondaryIdleReplicatorStatus is the BasicReplicatorStatus implementation for ReplicatorStatus.
func (rs ReplicatorStatus) AsSecondaryIdleReplicatorStatus() (*SecondaryIdleReplicatorStatus, bool) {
	return nil, false
}

// AsReplicatorStatus is the BasicReplicatorStatus implementation for ReplicatorStatus.
func (rs ReplicatorStatus) AsReplicatorStatus() (*ReplicatorStatus, bool) {
	return &rs, true
}

// AsBasicReplicatorStatus is the BasicReplicatorStatus implementation for ReplicatorStatus.
func (rs ReplicatorStatus) AsBasicReplicatorStatus() (BasicReplicatorStatus, bool) {
	return &rs, true
}

// ResolvedServiceEndpoint endpoint of a resolved service partition.
type ResolvedServiceEndpoint struct {
	// Kind - The role of the replica where the endpoint is reported. Possible values include: 'ServiceEndpointRoleInvalid', 'ServiceEndpointRoleStateless', 'ServiceEndpointRoleStatefulPrimary', 'ServiceEndpointRoleStatefulSecondary'
	Kind ServiceEndpointRole `json:"Kind,omitempty"`
	// Address - The address of the endpoint. If the endpoint has multiple listeners the address is a JSON object with one property per listener with the value as the address of that listener.
	Address *string `json:"Address,omitempty"`
}

// ResolvedServicePartition information about a service partition and its associated endpoints.
type ResolvedServicePartition struct {
	autorest.Response `json:"-"`
	// Name - The full name of the service with 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// PartitionInformation - A representation of the resolved partition.
	PartitionInformation BasicPartitionInformation `json:"PartitionInformation,omitempty"`
	// Endpoints - List of resolved service endpoints of a service partition.
	Endpoints *[]ResolvedServiceEndpoint `json:"Endpoints,omitempty"`
	// Version - The version of this resolved service partition result. This version should be passed in the next time the ResolveService call is made via the PreviousRspVersion query parameter.
	Version *string `json:"Version,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ResolvedServicePartition struct.
func (rsp *ResolvedServicePartition) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "Name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				rsp.Name = &name
			}
		case "PartitionInformation":
			if v != nil {
				partitionInformation, err := unmarshalBasicPartitionInformation(*v)
				if err != nil {
					return err
				}
				rsp.PartitionInformation = partitionInformation
			}
		case "Endpoints":
			if v != nil {
				var endpoints []ResolvedServiceEndpoint
				err = json.Unmarshal(*v, &endpoints)
				if err != nil {
					return err
				}
				rsp.Endpoints = &endpoints
			}
		case "Version":
			if v != nil {
				var version string
				err = json.Unmarshal(*v, &version)
				if err != nil {
					return err
				}
				rsp.Version = &version
			}
		}
	}

	return nil
}

// ResourceLimits this type describes the resource limits for a given container. It describes the most
// amount of resources a container is allowed to use before being restarted.
type ResourceLimits struct {
	// MemoryInGB - The memory limit in GB.
	MemoryInGB *float64 `json:"memoryInGB,omitempty"`
	// CPU - CPU limits in cores. At present, only full cores are supported.
	CPU *float64 `json:"cpu,omitempty"`
}

// ResourceRequests this type describes the requested resources for a given container. It describes the
// least amount of resources required for the container. A container can consume more than requested
// resources up to the specified limits before being restarted. Currently, the requested resources are
// treated as limits.
type ResourceRequests struct {
	// MemoryInGB - The memory request in GB for this container.
	MemoryInGB *float64 `json:"memoryInGB,omitempty"`
	// CPU - Requested number of CPU cores. At present, only full cores are supported.
	CPU *float64 `json:"cpu,omitempty"`
}

// ResourceRequirements this type describes the resource requirements for a container or a service.
type ResourceRequirements struct {
	// Requests - Describes the requested resources for a given container.
	Requests *ResourceRequests `json:"requests,omitempty"`
	// Limits - Describes the maximum limits on the resources for a given container.
	Limits *ResourceLimits `json:"limits,omitempty"`
}

// RestartDeployedCodePackageDescription defines description for restarting a deployed code package on
// Service Fabric node.
type RestartDeployedCodePackageDescription struct {
	// ServiceManifestName - The name of service manifest that specified this code package.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// ServicePackageActivationID - The ActivationId of a deployed service package. If ServicePackageActivationMode specified at the time of creating the service
	// is 'SharedProcess' (or if it is not specified, in which case it defaults to 'SharedProcess'), then value of ServicePackageActivationId
	// is always an empty string.
	ServicePackageActivationID *string `json:"ServicePackageActivationId,omitempty"`
	// CodePackageName - The name of the code package defined in the service manifest.
	CodePackageName *string `json:"CodePackageName,omitempty"`
	// CodePackageInstanceID - The instance ID for currently running entry point. For a code package setup entry point (if specified) runs first and after it finishes main entry point is started.
	// Each time entry point executable is run, its instance ID will change. If 0 is passed in as the code package instance ID, the API will restart the code package with whatever instance ID it is currently running.
	// If an instance ID other than 0 is passed in, the API will restart the code package only if the current Instance ID matches the passed in instance ID.
	// Note, passing in the exact instance ID (not 0) in the API is safer, because if ensures at most one restart of the code package.
	CodePackageInstanceID *string `json:"CodePackageInstanceId,omitempty"`
}

// RestartNodeDescription describes the parameters to restart a Service Fabric node.
type RestartNodeDescription struct {
	// NodeInstanceID - The instance ID of the target node. If instance ID is specified the node is restarted only if it matches with the current instance of the node. A default value of "0" would match any instance ID. The instance ID can be obtained using get node query.
	NodeInstanceID *string `json:"NodeInstanceId,omitempty"`
	// CreateFabricDump - Specify True to create a dump of the fabric node process. This is case-sensitive. Possible values include: 'False', 'True'
	CreateFabricDump CreateFabricDump `json:"CreateFabricDump,omitempty"`
}

// RestartPartitionResult represents information about an operation in a terminal state (Completed or
// Faulted).
type RestartPartitionResult struct {
	// ErrorCode - If OperationState is Completed, this is 0.  If OperationState is Faulted, this is an error code indicating the reason.
	ErrorCode *int32 `json:"ErrorCode,omitempty"`
	// SelectedPartition - This class returns information about the partition that the user-induced operation acted upon.
	SelectedPartition *SelectedPartition `json:"SelectedPartition,omitempty"`
}

// RestorePartitionDescription specifies the parameters needed to trigger a restore of a specific
// partition.
type RestorePartitionDescription struct {
	// BackupID - Unique backup ID.
	BackupID *uuid.UUID `json:"BackupId,omitempty"`
	// BackupLocation - Location of the backup relative to the backup storage specified/ configured.
	BackupLocation *string `json:"BackupLocation,omitempty"`
	// BackupStorage - Location of the backup from where the partition will be restored.
	BackupStorage BasicBackupStorageDescription `json:"BackupStorage,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for RestorePartitionDescription struct.
func (rpd *RestorePartitionDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "BackupId":
			if v != nil {
				var backupID uuid.UUID
				err = json.Unmarshal(*v, &backupID)
				if err != nil {
					return err
				}
				rpd.BackupID = &backupID
			}
		case "BackupLocation":
			if v != nil {
				var backupLocation string
				err = json.Unmarshal(*v, &backupLocation)
				if err != nil {
					return err
				}
				rpd.BackupLocation = &backupLocation
			}
		case "BackupStorage":
			if v != nil {
				backupStorage, err := unmarshalBasicBackupStorageDescription(*v)
				if err != nil {
					return err
				}
				rpd.BackupStorage = backupStorage
			}
		}
	}

	return nil
}

// RestoreProgressInfo describes the progress of a restore operation on a partition.
type RestoreProgressInfo struct {
	autorest.Response `json:"-"`
	// RestoreState - Represents the current state of the partition restore operation. Possible values include: 'RestoreStateInvalid', 'RestoreStateAccepted', 'RestoreStateRestoreInProgress', 'RestoreStateSuccess', 'RestoreStateFailure', 'RestoreStateTimeout'
	RestoreState RestoreState `json:"RestoreState,omitempty"`
	// TimeStampUtc - Timestamp when operation succeeded or failed.
	TimeStampUtc *date.Time `json:"TimeStampUtc,omitempty"`
	// RestoredEpoch - Describes the epoch at which the partition is restored.
	RestoredEpoch *Epoch `json:"RestoredEpoch,omitempty"`
	// RestoredLsn - Restored LSN.
	RestoredLsn *string `json:"RestoredLsn,omitempty"`
	// FailureError - Denotes the failure encountered in performing restore operation.
	FailureError *FabricErrorError `json:"FailureError,omitempty"`
}

// ResumeApplicationUpgradeDescription describes the parameters for resuming an unmonitored manual Service
// Fabric application upgrade
type ResumeApplicationUpgradeDescription struct {
	// UpgradeDomainName - The name of the upgrade domain in which to resume the upgrade.
	UpgradeDomainName *string `json:"UpgradeDomainName,omitempty"`
}

// ResumeClusterUpgradeDescription describes the parameters for resuming a cluster upgrade.
type ResumeClusterUpgradeDescription struct {
	// UpgradeDomain - The next upgrade domain for this cluster upgrade.
	UpgradeDomain *string `json:"UpgradeDomain,omitempty"`
}

// BasicBasicRetentionPolicyDescription describes the retention policy configured.
type BasicBasicRetentionPolicyDescription interface {
	AsBasicRetentionPolicyDescription() (*BasicRetentionPolicyDescription, bool)
	AsRetentionPolicyDescription() (*RetentionPolicyDescription, bool)
}

// RetentionPolicyDescription describes the retention policy configured.
type RetentionPolicyDescription struct {
	// RetentionPolicyType - Possible values include: 'RetentionPolicyTypeRetentionPolicyDescription', 'RetentionPolicyTypeBasic1'
	RetentionPolicyType RetentionPolicyTypeBasicBasicRetentionPolicyDescription `json:"RetentionPolicyType,omitempty"`
}

func unmarshalBasicBasicRetentionPolicyDescription(body []byte) (BasicBasicRetentionPolicyDescription, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["RetentionPolicyType"] {
	case string(RetentionPolicyTypeBasic1):
		var brpd BasicRetentionPolicyDescription
		err := json.Unmarshal(body, &brpd)
		return brpd, err
	default:
		var rpd RetentionPolicyDescription
		err := json.Unmarshal(body, &rpd)
		return rpd, err
	}
}
func unmarshalBasicBasicRetentionPolicyDescriptionArray(body []byte) ([]BasicBasicRetentionPolicyDescription, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	rpdArray := make([]BasicBasicRetentionPolicyDescription, len(rawMessages))

	for index, rawMessage := range rawMessages {
		rpd, err := unmarshalBasicBasicRetentionPolicyDescription(*rawMessage)
		if err != nil {
			return nil, err
		}
		rpdArray[index] = rpd
	}
	return rpdArray, nil
}

// MarshalJSON is the custom marshaler for RetentionPolicyDescription.
func (rpd RetentionPolicyDescription) MarshalJSON() ([]byte, error) {
	rpd.RetentionPolicyType = RetentionPolicyTypeRetentionPolicyDescription
	objectMap := make(map[string]interface{})
	if rpd.RetentionPolicyType != "" {
		objectMap["RetentionPolicyType"] = rpd.RetentionPolicyType
	}
	return json.Marshal(objectMap)
}

// AsBasicRetentionPolicyDescription is the BasicBasicRetentionPolicyDescription implementation for RetentionPolicyDescription.
func (rpd RetentionPolicyDescription) AsBasicRetentionPolicyDescription() (*BasicRetentionPolicyDescription, bool) {
	return nil, false
}

// AsRetentionPolicyDescription is the BasicBasicRetentionPolicyDescription implementation for RetentionPolicyDescription.
func (rpd RetentionPolicyDescription) AsRetentionPolicyDescription() (*RetentionPolicyDescription, bool) {
	return &rpd, true
}

// AsBasicBasicRetentionPolicyDescription is the BasicBasicRetentionPolicyDescription implementation for RetentionPolicyDescription.
func (rpd RetentionPolicyDescription) AsBasicBasicRetentionPolicyDescription() (BasicBasicRetentionPolicyDescription, bool) {
	return &rpd, true
}

// RollingUpgradeUpdateDescription describes the parameters for updating a rolling upgrade of application
// or cluster.
type RollingUpgradeUpdateDescription struct {
	// RollingUpgradeMode - The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto, UnmonitoredManual, and Monitored. Possible values include: 'UpgradeModeInvalid', 'UpgradeModeUnmonitoredAuto', 'UpgradeModeUnmonitoredManual', 'UpgradeModeMonitored'
	RollingUpgradeMode UpgradeMode `json:"RollingUpgradeMode,omitempty"`
	// ForceRestart - If true, then processes are forcefully restarted during upgrade even when the code version has not changed (the upgrade only changes configuration or data).
	ForceRestart *bool `json:"ForceRestart,omitempty"`
	// ReplicaSetCheckTimeoutInMilliseconds - The maximum amount of time to block processing of an upgrade domain and prevent loss of availability when there are unexpected issues. When this timeout expires, processing of the upgrade domain will proceed regardless of availability loss issues. The timeout is reset at the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive. (unsigned 32-bit integer).
	ReplicaSetCheckTimeoutInMilliseconds *int64 `json:"ReplicaSetCheckTimeoutInMilliseconds,omitempty"`
	// FailureAction - The compensating action to perform when a Monitored upgrade encounters monitoring policy or health policy violations.
	// Invalid indicates the failure action is invalid. Rollback specifies that the upgrade will start rolling back automatically.
	// Manual indicates that the upgrade will switch to UnmonitoredManual upgrade mode. Possible values include: 'FailureActionInvalid', 'FailureActionRollback', 'FailureActionManual'
	FailureAction FailureAction `json:"FailureAction,omitempty"`
	// HealthCheckWaitDurationInMilliseconds - The amount of time to wait after completing an upgrade domain before applying health policies. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	HealthCheckWaitDurationInMilliseconds *string `json:"HealthCheckWaitDurationInMilliseconds,omitempty"`
	// HealthCheckStableDurationInMilliseconds - The amount of time that the application or cluster must remain healthy before the upgrade proceeds to the next upgrade domain. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	HealthCheckStableDurationInMilliseconds *string `json:"HealthCheckStableDurationInMilliseconds,omitempty"`
	// HealthCheckRetryTimeoutInMilliseconds - The amount of time to retry health evaluation when the application or cluster is unhealthy before FailureAction is executed. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	HealthCheckRetryTimeoutInMilliseconds *string `json:"HealthCheckRetryTimeoutInMilliseconds,omitempty"`
	// UpgradeTimeoutInMilliseconds - The amount of time the overall upgrade has to complete before FailureAction is executed. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	UpgradeTimeoutInMilliseconds *string `json:"UpgradeTimeoutInMilliseconds,omitempty"`
	// UpgradeDomainTimeoutInMilliseconds - The amount of time each upgrade domain has to complete before FailureAction is executed. It is first interpreted as a string representing an ISO 8601 duration. If that fails, then it is interpreted as a number representing the total number of milliseconds.
	UpgradeDomainTimeoutInMilliseconds *string `json:"UpgradeDomainTimeoutInMilliseconds,omitempty"`
}

// BasicSafetyCheck represents a safety check performed by service fabric before continuing with the operations. These
// checks ensure the availability of the service and the reliability of the state.
type BasicSafetyCheck interface {
	AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool)
	AsBasicPartitionSafetyCheck() (BasicPartitionSafetyCheck, bool)
	AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool)
	AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool)
	AsSeedNodeSafetyCheck() (*SeedNodeSafetyCheck, bool)
	AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool)
	AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool)
	AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool)
	AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool)
	AsSafetyCheck() (*SafetyCheck, bool)
}

// SafetyCheck represents a safety check performed by service fabric before continuing with the operations.
// These checks ensure the availability of the service and the reliability of the state.
type SafetyCheck struct {
	// Kind - Possible values include: 'KindSafetyCheck', 'KindPartitionSafetyCheck', 'KindEnsureAvailability', 'KindEnsurePartitionQuorum', 'KindEnsureSeedNodeQuorum', 'KindWaitForInbuildReplica', 'KindWaitForPrimaryPlacement', 'KindWaitForPrimarySwap', 'KindWaitForReconfiguration'
	Kind KindBasicSafetyCheck `json:"Kind,omitempty"`
}

func unmarshalBasicSafetyCheck(body []byte) (BasicSafetyCheck, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindPartitionSafetyCheck):
		var psc PartitionSafetyCheck
		err := json.Unmarshal(body, &psc)
		return psc, err
	case string(KindEnsureAvailability):
		var easc EnsureAvailabilitySafetyCheck
		err := json.Unmarshal(body, &easc)
		return easc, err
	case string(KindEnsurePartitionQuorum):
		var epqsc EnsurePartitionQuorumSafetyCheck
		err := json.Unmarshal(body, &epqsc)
		return epqsc, err
	case string(KindEnsureSeedNodeQuorum):
		var snsc SeedNodeSafetyCheck
		err := json.Unmarshal(body, &snsc)
		return snsc, err
	case string(KindWaitForInbuildReplica):
		var wfirsc WaitForInbuildReplicaSafetyCheck
		err := json.Unmarshal(body, &wfirsc)
		return wfirsc, err
	case string(KindWaitForPrimaryPlacement):
		var wfppsc WaitForPrimaryPlacementSafetyCheck
		err := json.Unmarshal(body, &wfppsc)
		return wfppsc, err
	case string(KindWaitForPrimarySwap):
		var wfpssc WaitForPrimarySwapSafetyCheck
		err := json.Unmarshal(body, &wfpssc)
		return wfpssc, err
	case string(KindWaitForReconfiguration):
		var wfrsc WaitForReconfigurationSafetyCheck
		err := json.Unmarshal(body, &wfrsc)
		return wfrsc, err
	default:
		var sc SafetyCheck
		err := json.Unmarshal(body, &sc)
		return sc, err
	}
}
func unmarshalBasicSafetyCheckArray(body []byte) ([]BasicSafetyCheck, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	scArray := make([]BasicSafetyCheck, len(rawMessages))

	for index, rawMessage := range rawMessages {
		sc, err := unmarshalBasicSafetyCheck(*rawMessage)
		if err != nil {
			return nil, err
		}
		scArray[index] = sc
	}
	return scArray, nil
}

// MarshalJSON is the custom marshaler for SafetyCheck.
func (sc SafetyCheck) MarshalJSON() ([]byte, error) {
	sc.Kind = KindSafetyCheck
	objectMap := make(map[string]interface{})
	if sc.Kind != "" {
		objectMap["Kind"] = sc.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionSafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool) {
	return nil, false
}

// AsBasicPartitionSafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsBasicPartitionSafetyCheck() (BasicPartitionSafetyCheck, bool) {
	return nil, false
}

// AsEnsureAvailabilitySafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool) {
	return nil, false
}

// AsEnsurePartitionQuorumSafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool) {
	return nil, false
}

// AsSeedNodeSafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsSeedNodeSafetyCheck() (*SeedNodeSafetyCheck, bool) {
	return nil, false
}

// AsWaitForInbuildReplicaSafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimaryPlacementSafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimarySwapSafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool) {
	return nil, false
}

// AsWaitForReconfigurationSafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool) {
	return nil, false
}

// AsSafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsSafetyCheck() (*SafetyCheck, bool) {
	return &sc, true
}

// AsBasicSafetyCheck is the BasicSafetyCheck implementation for SafetyCheck.
func (sc SafetyCheck) AsBasicSafetyCheck() (BasicSafetyCheck, bool) {
	return &sc, true
}

// SafetyCheckWrapper a wrapper for the safety check object. Safety checks are performed by service fabric
// before continuing with the operations. These checks ensure the availability of the service and the
// reliability of the state.
type SafetyCheckWrapper struct {
	// SafetyCheck - Represents a safety check performed by service fabric before continuing with the operations. These checks ensure the availability of the service and the reliability of the state.
	SafetyCheck BasicSafetyCheck `json:"SafetyCheck,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for SafetyCheckWrapper struct.
func (scw *SafetyCheckWrapper) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "SafetyCheck":
			if v != nil {
				safetyCheck, err := unmarshalBasicSafetyCheck(*v)
				if err != nil {
					return err
				}
				scw.SafetyCheck = safetyCheck
			}
		}
	}

	return nil
}

// BasicScalingMechanismDescription describes the mechanism for performing a scaling operation.
type BasicScalingMechanismDescription interface {
	AsPartitionInstanceCountScaleMechanism() (*PartitionInstanceCountScaleMechanism, bool)
	AsAddRemoveIncrementalNamedPartitionScalingMechanism() (*AddRemoveIncrementalNamedPartitionScalingMechanism, bool)
	AsScalingMechanismDescription() (*ScalingMechanismDescription, bool)
}

// ScalingMechanismDescription describes the mechanism for performing a scaling operation.
type ScalingMechanismDescription struct {
	// Kind - Possible values include: 'KindScalingMechanismDescription', 'KindPartitionInstanceCount', 'KindAddRemoveIncrementalNamedPartition'
	Kind KindBasicScalingMechanismDescription `json:"Kind,omitempty"`
}

func unmarshalBasicScalingMechanismDescription(body []byte) (BasicScalingMechanismDescription, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindPartitionInstanceCount):
		var picsm PartitionInstanceCountScaleMechanism
		err := json.Unmarshal(body, &picsm)
		return picsm, err
	case string(KindAddRemoveIncrementalNamedPartition):
		var arinpsm AddRemoveIncrementalNamedPartitionScalingMechanism
		err := json.Unmarshal(body, &arinpsm)
		return arinpsm, err
	default:
		var smd ScalingMechanismDescription
		err := json.Unmarshal(body, &smd)
		return smd, err
	}
}
func unmarshalBasicScalingMechanismDescriptionArray(body []byte) ([]BasicScalingMechanismDescription, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	smdArray := make([]BasicScalingMechanismDescription, len(rawMessages))

	for index, rawMessage := range rawMessages {
		smd, err := unmarshalBasicScalingMechanismDescription(*rawMessage)
		if err != nil {
			return nil, err
		}
		smdArray[index] = smd
	}
	return smdArray, nil
}

// MarshalJSON is the custom marshaler for ScalingMechanismDescription.
func (smd ScalingMechanismDescription) MarshalJSON() ([]byte, error) {
	smd.Kind = KindScalingMechanismDescription
	objectMap := make(map[string]interface{})
	if smd.Kind != "" {
		objectMap["Kind"] = smd.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionInstanceCountScaleMechanism is the BasicScalingMechanismDescription implementation for ScalingMechanismDescription.
func (smd ScalingMechanismDescription) AsPartitionInstanceCountScaleMechanism() (*PartitionInstanceCountScaleMechanism, bool) {
	return nil, false
}

// AsAddRemoveIncrementalNamedPartitionScalingMechanism is the BasicScalingMechanismDescription implementation for ScalingMechanismDescription.
func (smd ScalingMechanismDescription) AsAddRemoveIncrementalNamedPartitionScalingMechanism() (*AddRemoveIncrementalNamedPartitionScalingMechanism, bool) {
	return nil, false
}

// AsScalingMechanismDescription is the BasicScalingMechanismDescription implementation for ScalingMechanismDescription.
func (smd ScalingMechanismDescription) AsScalingMechanismDescription() (*ScalingMechanismDescription, bool) {
	return &smd, true
}

// AsBasicScalingMechanismDescription is the BasicScalingMechanismDescription implementation for ScalingMechanismDescription.
func (smd ScalingMechanismDescription) AsBasicScalingMechanismDescription() (BasicScalingMechanismDescription, bool) {
	return &smd, true
}

// ScalingPolicyDescription describes how the scaling should be performed
type ScalingPolicyDescription struct {
	// ScalingTrigger - Specifies the trigger associated with this scaling policy
	ScalingTrigger BasicScalingTriggerDescription `json:"ScalingTrigger,omitempty"`
	// ScalingMechanism - Specifies the mechanism associated with this scaling policy
	ScalingMechanism BasicScalingMechanismDescription `json:"ScalingMechanism,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ScalingPolicyDescription struct.
func (spd *ScalingPolicyDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "ScalingTrigger":
			if v != nil {
				scalingTrigger, err := unmarshalBasicScalingTriggerDescription(*v)
				if err != nil {
					return err
				}
				spd.ScalingTrigger = scalingTrigger
			}
		case "ScalingMechanism":
			if v != nil {
				scalingMechanism, err := unmarshalBasicScalingMechanismDescription(*v)
				if err != nil {
					return err
				}
				spd.ScalingMechanism = scalingMechanism
			}
		}
	}

	return nil
}

// BasicScalingTriggerDescription describes the trigger for performing a scaling operation.
type BasicScalingTriggerDescription interface {
	AsAveragePartitionLoadScalingTrigger() (*AveragePartitionLoadScalingTrigger, bool)
	AsAverageServiceLoadScalingTrigger() (*AverageServiceLoadScalingTrigger, bool)
	AsScalingTriggerDescription() (*ScalingTriggerDescription, bool)
}

// ScalingTriggerDescription describes the trigger for performing a scaling operation.
type ScalingTriggerDescription struct {
	// Kind - Possible values include: 'KindScalingTriggerDescription', 'KindAveragePartitionLoad', 'KindAverageServiceLoad'
	Kind KindBasicScalingTriggerDescription `json:"Kind,omitempty"`
}

func unmarshalBasicScalingTriggerDescription(body []byte) (BasicScalingTriggerDescription, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindAveragePartitionLoad):
		var aplst AveragePartitionLoadScalingTrigger
		err := json.Unmarshal(body, &aplst)
		return aplst, err
	case string(KindAverageServiceLoad):
		var aslst AverageServiceLoadScalingTrigger
		err := json.Unmarshal(body, &aslst)
		return aslst, err
	default:
		var std ScalingTriggerDescription
		err := json.Unmarshal(body, &std)
		return std, err
	}
}
func unmarshalBasicScalingTriggerDescriptionArray(body []byte) ([]BasicScalingTriggerDescription, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	stdArray := make([]BasicScalingTriggerDescription, len(rawMessages))

	for index, rawMessage := range rawMessages {
		std, err := unmarshalBasicScalingTriggerDescription(*rawMessage)
		if err != nil {
			return nil, err
		}
		stdArray[index] = std
	}
	return stdArray, nil
}

// MarshalJSON is the custom marshaler for ScalingTriggerDescription.
func (std ScalingTriggerDescription) MarshalJSON() ([]byte, error) {
	std.Kind = KindScalingTriggerDescription
	objectMap := make(map[string]interface{})
	if std.Kind != "" {
		objectMap["Kind"] = std.Kind
	}
	return json.Marshal(objectMap)
}

// AsAveragePartitionLoadScalingTrigger is the BasicScalingTriggerDescription implementation for ScalingTriggerDescription.
func (std ScalingTriggerDescription) AsAveragePartitionLoadScalingTrigger() (*AveragePartitionLoadScalingTrigger, bool) {
	return nil, false
}

// AsAverageServiceLoadScalingTrigger is the BasicScalingTriggerDescription implementation for ScalingTriggerDescription.
func (std ScalingTriggerDescription) AsAverageServiceLoadScalingTrigger() (*AverageServiceLoadScalingTrigger, bool) {
	return nil, false
}

// AsScalingTriggerDescription is the BasicScalingTriggerDescription implementation for ScalingTriggerDescription.
func (std ScalingTriggerDescription) AsScalingTriggerDescription() (*ScalingTriggerDescription, bool) {
	return &std, true
}

// AsBasicScalingTriggerDescription is the BasicScalingTriggerDescription implementation for ScalingTriggerDescription.
func (std ScalingTriggerDescription) AsBasicScalingTriggerDescription() (BasicScalingTriggerDescription, bool) {
	return &std, true
}

// SecondaryActiveReplicatorStatus status of the secondary replicator when it is in active mode and is part
// of the replica set.
type SecondaryActiveReplicatorStatus struct {
	// ReplicationQueueStatus - Details about the replication queue on the secondary replicator.
	ReplicationQueueStatus *ReplicatorQueueStatus `json:"ReplicationQueueStatus,omitempty"`
	// LastReplicationOperationReceivedTimeUtc - The last time-stamp (UTC) at which a replication operation was received from the primary.
	// UTC 0 represents an invalid value, indicating that a replication operation message was never received.
	LastReplicationOperationReceivedTimeUtc *date.Time `json:"LastReplicationOperationReceivedTimeUtc,omitempty"`
	// IsInBuild - Value that indicates whether the replica is currently being built.
	IsInBuild *bool `json:"IsInBuild,omitempty"`
	// CopyQueueStatus - Details about the copy queue on the secondary replicator.
	CopyQueueStatus *ReplicatorQueueStatus `json:"CopyQueueStatus,omitempty"`
	// LastCopyOperationReceivedTimeUtc - The last time-stamp (UTC) at which a copy operation was received from the primary.
	// UTC 0 represents an invalid value, indicating that a copy operation message was never received.
	LastCopyOperationReceivedTimeUtc *date.Time `json:"LastCopyOperationReceivedTimeUtc,omitempty"`
	// LastAcknowledgementSentTimeUtc - The last time-stamp (UTC) at which an acknowledgment was sent to the primary replicator.
	// UTC 0 represents an invalid value, indicating that an acknowledgment message was never sent.
	LastAcknowledgementSentTimeUtc *date.Time `json:"LastAcknowledgementSentTimeUtc,omitempty"`
	// Kind - Possible values include: 'KindReplicatorStatus', 'KindPrimary', 'KindSecondaryReplicatorStatus', 'KindActiveSecondary', 'KindIdleSecondary'
	Kind KindBasicReplicatorStatus `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for SecondaryActiveReplicatorStatus.
func (sars SecondaryActiveReplicatorStatus) MarshalJSON() ([]byte, error) {
	sars.Kind = KindActiveSecondary
	objectMap := make(map[string]interface{})
	if sars.ReplicationQueueStatus != nil {
		objectMap["ReplicationQueueStatus"] = sars.ReplicationQueueStatus
	}
	if sars.LastReplicationOperationReceivedTimeUtc != nil {
		objectMap["LastReplicationOperationReceivedTimeUtc"] = sars.LastReplicationOperationReceivedTimeUtc
	}
	if sars.IsInBuild != nil {
		objectMap["IsInBuild"] = sars.IsInBuild
	}
	if sars.CopyQueueStatus != nil {
		objectMap["CopyQueueStatus"] = sars.CopyQueueStatus
	}
	if sars.LastCopyOperationReceivedTimeUtc != nil {
		objectMap["LastCopyOperationReceivedTimeUtc"] = sars.LastCopyOperationReceivedTimeUtc
	}
	if sars.LastAcknowledgementSentTimeUtc != nil {
		objectMap["LastAcknowledgementSentTimeUtc"] = sars.LastAcknowledgementSentTimeUtc
	}
	if sars.Kind != "" {
		objectMap["Kind"] = sars.Kind
	}
	return json.Marshal(objectMap)
}

// AsPrimaryReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryActiveReplicatorStatus.
func (sars SecondaryActiveReplicatorStatus) AsPrimaryReplicatorStatus() (*PrimaryReplicatorStatus, bool) {
	return nil, false
}

// AsSecondaryReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryActiveReplicatorStatus.
func (sars SecondaryActiveReplicatorStatus) AsSecondaryReplicatorStatus() (*SecondaryReplicatorStatus, bool) {
	return nil, false
}

// AsBasicSecondaryReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryActiveReplicatorStatus.
func (sars SecondaryActiveReplicatorStatus) AsBasicSecondaryReplicatorStatus() (BasicSecondaryReplicatorStatus, bool) {
	return &sars, true
}

// AsSecondaryActiveReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryActiveReplicatorStatus.
func (sars SecondaryActiveReplicatorStatus) AsSecondaryActiveReplicatorStatus() (*SecondaryActiveReplicatorStatus, bool) {
	return &sars, true
}

// AsSecondaryIdleReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryActiveReplicatorStatus.
func (sars SecondaryActiveReplicatorStatus) AsSecondaryIdleReplicatorStatus() (*SecondaryIdleReplicatorStatus, bool) {
	return nil, false
}

// AsReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryActiveReplicatorStatus.
func (sars SecondaryActiveReplicatorStatus) AsReplicatorStatus() (*ReplicatorStatus, bool) {
	return nil, false
}

// AsBasicReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryActiveReplicatorStatus.
func (sars SecondaryActiveReplicatorStatus) AsBasicReplicatorStatus() (BasicReplicatorStatus, bool) {
	return &sars, true
}

// SecondaryIdleReplicatorStatus status of the secondary replicator when it is in idle mode and is being
// built by the primary.
type SecondaryIdleReplicatorStatus struct {
	// ReplicationQueueStatus - Details about the replication queue on the secondary replicator.
	ReplicationQueueStatus *ReplicatorQueueStatus `json:"ReplicationQueueStatus,omitempty"`
	// LastReplicationOperationReceivedTimeUtc - The last time-stamp (UTC) at which a replication operation was received from the primary.
	// UTC 0 represents an invalid value, indicating that a replication operation message was never received.
	LastReplicationOperationReceivedTimeUtc *date.Time `json:"LastReplicationOperationReceivedTimeUtc,omitempty"`
	// IsInBuild - Value that indicates whether the replica is currently being built.
	IsInBuild *bool `json:"IsInBuild,omitempty"`
	// CopyQueueStatus - Details about the copy queue on the secondary replicator.
	CopyQueueStatus *ReplicatorQueueStatus `json:"CopyQueueStatus,omitempty"`
	// LastCopyOperationReceivedTimeUtc - The last time-stamp (UTC) at which a copy operation was received from the primary.
	// UTC 0 represents an invalid value, indicating that a copy operation message was never received.
	LastCopyOperationReceivedTimeUtc *date.Time `json:"LastCopyOperationReceivedTimeUtc,omitempty"`
	// LastAcknowledgementSentTimeUtc - The last time-stamp (UTC) at which an acknowledgment was sent to the primary replicator.
	// UTC 0 represents an invalid value, indicating that an acknowledgment message was never sent.
	LastAcknowledgementSentTimeUtc *date.Time `json:"LastAcknowledgementSentTimeUtc,omitempty"`
	// Kind - Possible values include: 'KindReplicatorStatus', 'KindPrimary', 'KindSecondaryReplicatorStatus', 'KindActiveSecondary', 'KindIdleSecondary'
	Kind KindBasicReplicatorStatus `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for SecondaryIdleReplicatorStatus.
func (sirs SecondaryIdleReplicatorStatus) MarshalJSON() ([]byte, error) {
	sirs.Kind = KindIdleSecondary
	objectMap := make(map[string]interface{})
	if sirs.ReplicationQueueStatus != nil {
		objectMap["ReplicationQueueStatus"] = sirs.ReplicationQueueStatus
	}
	if sirs.LastReplicationOperationReceivedTimeUtc != nil {
		objectMap["LastReplicationOperationReceivedTimeUtc"] = sirs.LastReplicationOperationReceivedTimeUtc
	}
	if sirs.IsInBuild != nil {
		objectMap["IsInBuild"] = sirs.IsInBuild
	}
	if sirs.CopyQueueStatus != nil {
		objectMap["CopyQueueStatus"] = sirs.CopyQueueStatus
	}
	if sirs.LastCopyOperationReceivedTimeUtc != nil {
		objectMap["LastCopyOperationReceivedTimeUtc"] = sirs.LastCopyOperationReceivedTimeUtc
	}
	if sirs.LastAcknowledgementSentTimeUtc != nil {
		objectMap["LastAcknowledgementSentTimeUtc"] = sirs.LastAcknowledgementSentTimeUtc
	}
	if sirs.Kind != "" {
		objectMap["Kind"] = sirs.Kind
	}
	return json.Marshal(objectMap)
}

// AsPrimaryReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryIdleReplicatorStatus.
func (sirs SecondaryIdleReplicatorStatus) AsPrimaryReplicatorStatus() (*PrimaryReplicatorStatus, bool) {
	return nil, false
}

// AsSecondaryReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryIdleReplicatorStatus.
func (sirs SecondaryIdleReplicatorStatus) AsSecondaryReplicatorStatus() (*SecondaryReplicatorStatus, bool) {
	return nil, false
}

// AsBasicSecondaryReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryIdleReplicatorStatus.
func (sirs SecondaryIdleReplicatorStatus) AsBasicSecondaryReplicatorStatus() (BasicSecondaryReplicatorStatus, bool) {
	return &sirs, true
}

// AsSecondaryActiveReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryIdleReplicatorStatus.
func (sirs SecondaryIdleReplicatorStatus) AsSecondaryActiveReplicatorStatus() (*SecondaryActiveReplicatorStatus, bool) {
	return nil, false
}

// AsSecondaryIdleReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryIdleReplicatorStatus.
func (sirs SecondaryIdleReplicatorStatus) AsSecondaryIdleReplicatorStatus() (*SecondaryIdleReplicatorStatus, bool) {
	return &sirs, true
}

// AsReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryIdleReplicatorStatus.
func (sirs SecondaryIdleReplicatorStatus) AsReplicatorStatus() (*ReplicatorStatus, bool) {
	return nil, false
}

// AsBasicReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryIdleReplicatorStatus.
func (sirs SecondaryIdleReplicatorStatus) AsBasicReplicatorStatus() (BasicReplicatorStatus, bool) {
	return &sirs, true
}

// BasicSecondaryReplicatorStatus provides statistics about the Service Fabric Replicator, when it is functioning in a
// ActiveSecondary role.
type BasicSecondaryReplicatorStatus interface {
	AsSecondaryActiveReplicatorStatus() (*SecondaryActiveReplicatorStatus, bool)
	AsSecondaryIdleReplicatorStatus() (*SecondaryIdleReplicatorStatus, bool)
	AsSecondaryReplicatorStatus() (*SecondaryReplicatorStatus, bool)
}

// SecondaryReplicatorStatus provides statistics about the Service Fabric Replicator, when it is functioning in
// a ActiveSecondary role.
type SecondaryReplicatorStatus struct {
	// ReplicationQueueStatus - Details about the replication queue on the secondary replicator.
	ReplicationQueueStatus *ReplicatorQueueStatus `json:"ReplicationQueueStatus,omitempty"`
	// LastReplicationOperationReceivedTimeUtc - The last time-stamp (UTC) at which a replication operation was received from the primary.
	// UTC 0 represents an invalid value, indicating that a replication operation message was never received.
	LastReplicationOperationReceivedTimeUtc *date.Time `json:"LastReplicationOperationReceivedTimeUtc,omitempty"`
	// IsInBuild - Value that indicates whether the replica is currently being built.
	IsInBuild *bool `json:"IsInBuild,omitempty"`
	// CopyQueueStatus - Details about the copy queue on the secondary replicator.
	CopyQueueStatus *ReplicatorQueueStatus `json:"CopyQueueStatus,omitempty"`
	// LastCopyOperationReceivedTimeUtc - The last time-stamp (UTC) at which a copy operation was received from the primary.
	// UTC 0 represents an invalid value, indicating that a copy operation message was never received.
	LastCopyOperationReceivedTimeUtc *date.Time `json:"LastCopyOperationReceivedTimeUtc,omitempty"`
	// LastAcknowledgementSentTimeUtc - The last time-stamp (UTC) at which an acknowledgment was sent to the primary replicator.
	// UTC 0 represents an invalid value, indicating that an acknowledgment message was never sent.
	LastAcknowledgementSentTimeUtc *date.Time `json:"LastAcknowledgementSentTimeUtc,omitempty"`
	// Kind - Possible values include: 'KindReplicatorStatus', 'KindPrimary', 'KindSecondaryReplicatorStatus', 'KindActiveSecondary', 'KindIdleSecondary'
	Kind KindBasicReplicatorStatus `json:"Kind,omitempty"`
}

func unmarshalBasicSecondaryReplicatorStatus(body []byte) (BasicSecondaryReplicatorStatus, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindActiveSecondary):
		var sars SecondaryActiveReplicatorStatus
		err := json.Unmarshal(body, &sars)
		return sars, err
	case string(KindIdleSecondary):
		var sirs SecondaryIdleReplicatorStatus
		err := json.Unmarshal(body, &sirs)
		return sirs, err
	default:
		var srs SecondaryReplicatorStatus
		err := json.Unmarshal(body, &srs)
		return srs, err
	}
}
func unmarshalBasicSecondaryReplicatorStatusArray(body []byte) ([]BasicSecondaryReplicatorStatus, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	srsArray := make([]BasicSecondaryReplicatorStatus, len(rawMessages))

	for index, rawMessage := range rawMessages {
		srs, err := unmarshalBasicSecondaryReplicatorStatus(*rawMessage)
		if err != nil {
			return nil, err
		}
		srsArray[index] = srs
	}
	return srsArray, nil
}

// MarshalJSON is the custom marshaler for SecondaryReplicatorStatus.
func (srs SecondaryReplicatorStatus) MarshalJSON() ([]byte, error) {
	srs.Kind = KindSecondaryReplicatorStatus
	objectMap := make(map[string]interface{})
	if srs.ReplicationQueueStatus != nil {
		objectMap["ReplicationQueueStatus"] = srs.ReplicationQueueStatus
	}
	if srs.LastReplicationOperationReceivedTimeUtc != nil {
		objectMap["LastReplicationOperationReceivedTimeUtc"] = srs.LastReplicationOperationReceivedTimeUtc
	}
	if srs.IsInBuild != nil {
		objectMap["IsInBuild"] = srs.IsInBuild
	}
	if srs.CopyQueueStatus != nil {
		objectMap["CopyQueueStatus"] = srs.CopyQueueStatus
	}
	if srs.LastCopyOperationReceivedTimeUtc != nil {
		objectMap["LastCopyOperationReceivedTimeUtc"] = srs.LastCopyOperationReceivedTimeUtc
	}
	if srs.LastAcknowledgementSentTimeUtc != nil {
		objectMap["LastAcknowledgementSentTimeUtc"] = srs.LastAcknowledgementSentTimeUtc
	}
	if srs.Kind != "" {
		objectMap["Kind"] = srs.Kind
	}
	return json.Marshal(objectMap)
}

// AsPrimaryReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryReplicatorStatus.
func (srs SecondaryReplicatorStatus) AsPrimaryReplicatorStatus() (*PrimaryReplicatorStatus, bool) {
	return nil, false
}

// AsSecondaryReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryReplicatorStatus.
func (srs SecondaryReplicatorStatus) AsSecondaryReplicatorStatus() (*SecondaryReplicatorStatus, bool) {
	return &srs, true
}

// AsBasicSecondaryReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryReplicatorStatus.
func (srs SecondaryReplicatorStatus) AsBasicSecondaryReplicatorStatus() (BasicSecondaryReplicatorStatus, bool) {
	return &srs, true
}

// AsSecondaryActiveReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryReplicatorStatus.
func (srs SecondaryReplicatorStatus) AsSecondaryActiveReplicatorStatus() (*SecondaryActiveReplicatorStatus, bool) {
	return nil, false
}

// AsSecondaryIdleReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryReplicatorStatus.
func (srs SecondaryReplicatorStatus) AsSecondaryIdleReplicatorStatus() (*SecondaryIdleReplicatorStatus, bool) {
	return nil, false
}

// AsReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryReplicatorStatus.
func (srs SecondaryReplicatorStatus) AsReplicatorStatus() (*ReplicatorStatus, bool) {
	return nil, false
}

// AsBasicReplicatorStatus is the BasicReplicatorStatus implementation for SecondaryReplicatorStatus.
func (srs SecondaryReplicatorStatus) AsBasicReplicatorStatus() (BasicReplicatorStatus, bool) {
	return &srs, true
}

// SecretResourceDescription this type describes a secret resource.
type SecretResourceDescription struct {
	autorest.Response `json:"-"`
	// Properties - Describes the properties of a secret resource.
	Properties BasicSecretResourceProperties `json:"properties,omitempty"`
	// Name - Name of the Secret resource.
	Name *string `json:"name,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for SecretResourceDescription struct.
func (srd *SecretResourceDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "properties":
			if v != nil {
				properties, err := unmarshalBasicSecretResourceProperties(*v)
				if err != nil {
					return err
				}
				srd.Properties = properties
			}
		case "name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				srd.Name = &name
			}
		}
	}

	return nil
}

// BasicSecretResourceProperties describes the properties of a secret resource.
type BasicSecretResourceProperties interface {
	AsInlinedValueSecretResourceProperties() (*InlinedValueSecretResourceProperties, bool)
	AsSecretResourceProperties() (*SecretResourceProperties, bool)
}

// SecretResourceProperties describes the properties of a secret resource.
type SecretResourceProperties struct {
	// Description - User readable description of the secret.
	Description *string `json:"description,omitempty"`
	// Status - READ-ONLY; Status of the resource. Possible values include: 'ResourceStatusUnknown', 'ResourceStatusReady', 'ResourceStatusUpgrading', 'ResourceStatusCreating', 'ResourceStatusDeleting', 'ResourceStatusFailed'
	Status ResourceStatus `json:"status,omitempty"`
	// StatusDetails - READ-ONLY; Gives additional information about the current status of the secret.
	StatusDetails *string `json:"statusDetails,omitempty"`
	// ContentType - The type of the content stored in the secret value. The value of this property is opaque to Service Fabric. Once set, the value of this property cannot be changed.
	ContentType *string `json:"contentType,omitempty"`
	// Kind - Possible values include: 'KindSecretResourcePropertiesBase', 'KindSecretResourceProperties', 'KindInlinedValue'
	Kind KindBasicSecretResourcePropertiesBase `json:"kind,omitempty"`
}

func unmarshalBasicSecretResourceProperties(body []byte) (BasicSecretResourceProperties, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["kind"] {
	case string(KindInlinedValue):
		var ivsrp InlinedValueSecretResourceProperties
		err := json.Unmarshal(body, &ivsrp)
		return ivsrp, err
	default:
		var srp SecretResourceProperties
		err := json.Unmarshal(body, &srp)
		return srp, err
	}
}
func unmarshalBasicSecretResourcePropertiesArray(body []byte) ([]BasicSecretResourceProperties, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	srpArray := make([]BasicSecretResourceProperties, len(rawMessages))

	for index, rawMessage := range rawMessages {
		srp, err := unmarshalBasicSecretResourceProperties(*rawMessage)
		if err != nil {
			return nil, err
		}
		srpArray[index] = srp
	}
	return srpArray, nil
}

// MarshalJSON is the custom marshaler for SecretResourceProperties.
func (srp SecretResourceProperties) MarshalJSON() ([]byte, error) {
	srp.Kind = KindSecretResourceProperties
	objectMap := make(map[string]interface{})
	if srp.Description != nil {
		objectMap["description"] = srp.Description
	}
	if srp.ContentType != nil {
		objectMap["contentType"] = srp.ContentType
	}
	if srp.Kind != "" {
		objectMap["kind"] = srp.Kind
	}
	return json.Marshal(objectMap)
}

// AsSecretResourceProperties is the BasicSecretResourcePropertiesBase implementation for SecretResourceProperties.
func (srp SecretResourceProperties) AsSecretResourceProperties() (*SecretResourceProperties, bool) {
	return &srp, true
}

// AsBasicSecretResourceProperties is the BasicSecretResourcePropertiesBase implementation for SecretResourceProperties.
func (srp SecretResourceProperties) AsBasicSecretResourceProperties() (BasicSecretResourceProperties, bool) {
	return &srp, true
}

// AsInlinedValueSecretResourceProperties is the BasicSecretResourcePropertiesBase implementation for SecretResourceProperties.
func (srp SecretResourceProperties) AsInlinedValueSecretResourceProperties() (*InlinedValueSecretResourceProperties, bool) {
	return nil, false
}

// AsSecretResourcePropertiesBase is the BasicSecretResourcePropertiesBase implementation for SecretResourceProperties.
func (srp SecretResourceProperties) AsSecretResourcePropertiesBase() (*SecretResourcePropertiesBase, bool) {
	return nil, false
}

// AsBasicSecretResourcePropertiesBase is the BasicSecretResourcePropertiesBase implementation for SecretResourceProperties.
func (srp SecretResourceProperties) AsBasicSecretResourcePropertiesBase() (BasicSecretResourcePropertiesBase, bool) {
	return &srp, true
}

// BasicSecretResourcePropertiesBase this type describes the properties of a secret resource, including its kind.
type BasicSecretResourcePropertiesBase interface {
	AsSecretResourceProperties() (*SecretResourceProperties, bool)
	AsBasicSecretResourceProperties() (BasicSecretResourceProperties, bool)
	AsInlinedValueSecretResourceProperties() (*InlinedValueSecretResourceProperties, bool)
	AsSecretResourcePropertiesBase() (*SecretResourcePropertiesBase, bool)
}

// SecretResourcePropertiesBase this type describes the properties of a secret resource, including its kind.
type SecretResourcePropertiesBase struct {
	// Kind - Possible values include: 'KindSecretResourcePropertiesBase', 'KindSecretResourceProperties', 'KindInlinedValue'
	Kind KindBasicSecretResourcePropertiesBase `json:"kind,omitempty"`
}

func unmarshalBasicSecretResourcePropertiesBase(body []byte) (BasicSecretResourcePropertiesBase, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["kind"] {
	case string(KindSecretResourceProperties):
		var srp SecretResourceProperties
		err := json.Unmarshal(body, &srp)
		return srp, err
	case string(KindInlinedValue):
		var ivsrp InlinedValueSecretResourceProperties
		err := json.Unmarshal(body, &ivsrp)
		return ivsrp, err
	default:
		var srpb SecretResourcePropertiesBase
		err := json.Unmarshal(body, &srpb)
		return srpb, err
	}
}
func unmarshalBasicSecretResourcePropertiesBaseArray(body []byte) ([]BasicSecretResourcePropertiesBase, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	srpbArray := make([]BasicSecretResourcePropertiesBase, len(rawMessages))

	for index, rawMessage := range rawMessages {
		srpb, err := unmarshalBasicSecretResourcePropertiesBase(*rawMessage)
		if err != nil {
			return nil, err
		}
		srpbArray[index] = srpb
	}
	return srpbArray, nil
}

// MarshalJSON is the custom marshaler for SecretResourcePropertiesBase.
func (srpb SecretResourcePropertiesBase) MarshalJSON() ([]byte, error) {
	srpb.Kind = KindSecretResourcePropertiesBase
	objectMap := make(map[string]interface{})
	if srpb.Kind != "" {
		objectMap["kind"] = srpb.Kind
	}
	return json.Marshal(objectMap)
}

// AsSecretResourceProperties is the BasicSecretResourcePropertiesBase implementation for SecretResourcePropertiesBase.
func (srpb SecretResourcePropertiesBase) AsSecretResourceProperties() (*SecretResourceProperties, bool) {
	return nil, false
}

// AsBasicSecretResourceProperties is the BasicSecretResourcePropertiesBase implementation for SecretResourcePropertiesBase.
func (srpb SecretResourcePropertiesBase) AsBasicSecretResourceProperties() (BasicSecretResourceProperties, bool) {
	return nil, false
}

// AsInlinedValueSecretResourceProperties is the BasicSecretResourcePropertiesBase implementation for SecretResourcePropertiesBase.
func (srpb SecretResourcePropertiesBase) AsInlinedValueSecretResourceProperties() (*InlinedValueSecretResourceProperties, bool) {
	return nil, false
}

// AsSecretResourcePropertiesBase is the BasicSecretResourcePropertiesBase implementation for SecretResourcePropertiesBase.
func (srpb SecretResourcePropertiesBase) AsSecretResourcePropertiesBase() (*SecretResourcePropertiesBase, bool) {
	return &srpb, true
}

// AsBasicSecretResourcePropertiesBase is the BasicSecretResourcePropertiesBase implementation for SecretResourcePropertiesBase.
func (srpb SecretResourcePropertiesBase) AsBasicSecretResourcePropertiesBase() (BasicSecretResourcePropertiesBase, bool) {
	return &srpb, true
}

// SecretValue this type represents the unencrypted value of the secret.
type SecretValue struct {
	autorest.Response `json:"-"`
	// Value - The actual value of the secret.
	Value *string `json:"value,omitempty"`
}

// SecretValueProperties this type describes properties of secret value resource.
type SecretValueProperties struct {
	// Value - The actual value of the secret.
	Value *string `json:"value,omitempty"`
}

// SecretValueResourceDescription this type describes a value of a secret resource. The name of this
// resource is the version identifier corresponding to this secret value.
type SecretValueResourceDescription struct {
	autorest.Response `json:"-"`
	// Name - Version identifier of the secret value.
	Name *string `json:"name,omitempty"`
	// SecretValueResourceProperties - This type describes properties of a secret value resource.
	*SecretValueResourceProperties `json:"properties,omitempty"`
}

// MarshalJSON is the custom marshaler for SecretValueResourceDescription.
func (svrd SecretValueResourceDescription) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if svrd.Name != nil {
		objectMap["name"] = svrd.Name
	}
	if svrd.SecretValueResourceProperties != nil {
		objectMap["properties"] = svrd.SecretValueResourceProperties
	}
	return json.Marshal(objectMap)
}

// UnmarshalJSON is the custom unmarshaler for SecretValueResourceDescription struct.
func (svrd *SecretValueResourceDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				svrd.Name = &name
			}
		case "properties":
			if v != nil {
				var secretValueResourceProperties SecretValueResourceProperties
				err = json.Unmarshal(*v, &secretValueResourceProperties)
				if err != nil {
					return err
				}
				svrd.SecretValueResourceProperties = &secretValueResourceProperties
			}
		}
	}

	return nil
}

// SecretValueResourceProperties this type describes properties of a secret value resource.
type SecretValueResourceProperties struct {
	// Value - The actual value of the secret.
	Value *string `json:"value,omitempty"`
}

// SeedNodeSafetyCheck represents a safety check for the seed nodes being performed by service fabric
// before continuing with node level operations.
type SeedNodeSafetyCheck struct {
	// Kind - Possible values include: 'KindSafetyCheck', 'KindPartitionSafetyCheck', 'KindEnsureAvailability', 'KindEnsurePartitionQuorum', 'KindEnsureSeedNodeQuorum', 'KindWaitForInbuildReplica', 'KindWaitForPrimaryPlacement', 'KindWaitForPrimarySwap', 'KindWaitForReconfiguration'
	Kind KindBasicSafetyCheck `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) MarshalJSON() ([]byte, error) {
	snsc.Kind = KindEnsureSeedNodeQuorum
	objectMap := make(map[string]interface{})
	if snsc.Kind != "" {
		objectMap["Kind"] = snsc.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionSafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool) {
	return nil, false
}

// AsBasicPartitionSafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsBasicPartitionSafetyCheck() (BasicPartitionSafetyCheck, bool) {
	return nil, false
}

// AsEnsureAvailabilitySafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool) {
	return nil, false
}

// AsEnsurePartitionQuorumSafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool) {
	return nil, false
}

// AsSeedNodeSafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsSeedNodeSafetyCheck() (*SeedNodeSafetyCheck, bool) {
	return &snsc, true
}

// AsWaitForInbuildReplicaSafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimaryPlacementSafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimarySwapSafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool) {
	return nil, false
}

// AsWaitForReconfigurationSafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool) {
	return nil, false
}

// AsSafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsSafetyCheck() (*SafetyCheck, bool) {
	return nil, false
}

// AsBasicSafetyCheck is the BasicSafetyCheck implementation for SeedNodeSafetyCheck.
func (snsc SeedNodeSafetyCheck) AsBasicSafetyCheck() (BasicSafetyCheck, bool) {
	return &snsc, true
}

// SelectedPartition this class returns information about the partition that the user-induced operation
// acted upon.
type SelectedPartition struct {
	// ServiceName - The name of the service the partition belongs to.
	ServiceName *string `json:"ServiceName,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
}

// ServiceBackupConfigurationInfo backup configuration information for a specific Service Fabric service
// specifying what backup policy is being applied and suspend description, if any.
type ServiceBackupConfigurationInfo struct {
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// PolicyName - The name of the backup policy which is applicable to this Service Fabric application or service or partition.
	PolicyName *string `json:"PolicyName,omitempty"`
	// PolicyInheritedFrom - Specifies the scope at which the backup policy is applied. Possible values include: 'BackupPolicyScopeInvalid', 'BackupPolicyScopePartition', 'BackupPolicyScopeService', 'BackupPolicyScopeApplication'
	PolicyInheritedFrom BackupPolicyScope `json:"PolicyInheritedFrom,omitempty"`
	// SuspensionInfo - Describes the backup suspension details.
	SuspensionInfo *BackupSuspensionInfo `json:"SuspensionInfo,omitempty"`
	// Kind - Possible values include: 'KindBasicBackupConfigurationInfoKindBackupConfigurationInfo', 'KindBasicBackupConfigurationInfoKindApplication', 'KindBasicBackupConfigurationInfoKindService', 'KindBasicBackupConfigurationInfoKindPartition'
	Kind KindBasicBackupConfigurationInfo `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ServiceBackupConfigurationInfo.
func (sbci ServiceBackupConfigurationInfo) MarshalJSON() ([]byte, error) {
	sbci.Kind = KindBasicBackupConfigurationInfoKindService
	objectMap := make(map[string]interface{})
	if sbci.ServiceName != nil {
		objectMap["ServiceName"] = sbci.ServiceName
	}
	if sbci.PolicyName != nil {
		objectMap["PolicyName"] = sbci.PolicyName
	}
	if sbci.PolicyInheritedFrom != "" {
		objectMap["PolicyInheritedFrom"] = sbci.PolicyInheritedFrom
	}
	if sbci.SuspensionInfo != nil {
		objectMap["SuspensionInfo"] = sbci.SuspensionInfo
	}
	if sbci.Kind != "" {
		objectMap["Kind"] = sbci.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for ServiceBackupConfigurationInfo.
func (sbci ServiceBackupConfigurationInfo) AsApplicationBackupConfigurationInfo() (*ApplicationBackupConfigurationInfo, bool) {
	return nil, false
}

// AsServiceBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for ServiceBackupConfigurationInfo.
func (sbci ServiceBackupConfigurationInfo) AsServiceBackupConfigurationInfo() (*ServiceBackupConfigurationInfo, bool) {
	return &sbci, true
}

// AsPartitionBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for ServiceBackupConfigurationInfo.
func (sbci ServiceBackupConfigurationInfo) AsPartitionBackupConfigurationInfo() (*PartitionBackupConfigurationInfo, bool) {
	return nil, false
}

// AsBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for ServiceBackupConfigurationInfo.
func (sbci ServiceBackupConfigurationInfo) AsBackupConfigurationInfo() (*BackupConfigurationInfo, bool) {
	return nil, false
}

// AsBasicBackupConfigurationInfo is the BasicBackupConfigurationInfo implementation for ServiceBackupConfigurationInfo.
func (sbci ServiceBackupConfigurationInfo) AsBasicBackupConfigurationInfo() (BasicBackupConfigurationInfo, bool) {
	return &sbci, true
}

// ServiceBackupEntity identifies the Service Fabric stateful service which is being backed up.
type ServiceBackupEntity struct {
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// EntityKind - Possible values include: 'EntityKindBackupEntity', 'EntityKindApplication1', 'EntityKindService1', 'EntityKindPartition1'
	EntityKind EntityKindBasicBackupEntity `json:"EntityKind,omitempty"`
}

// MarshalJSON is the custom marshaler for ServiceBackupEntity.
func (sbe ServiceBackupEntity) MarshalJSON() ([]byte, error) {
	sbe.EntityKind = EntityKindService1
	objectMap := make(map[string]interface{})
	if sbe.ServiceName != nil {
		objectMap["ServiceName"] = sbe.ServiceName
	}
	if sbe.EntityKind != "" {
		objectMap["EntityKind"] = sbe.EntityKind
	}
	return json.Marshal(objectMap)
}

// AsApplicationBackupEntity is the BasicBackupEntity implementation for ServiceBackupEntity.
func (sbe ServiceBackupEntity) AsApplicationBackupEntity() (*ApplicationBackupEntity, bool) {
	return nil, false
}

// AsServiceBackupEntity is the BasicBackupEntity implementation for ServiceBackupEntity.
func (sbe ServiceBackupEntity) AsServiceBackupEntity() (*ServiceBackupEntity, bool) {
	return &sbe, true
}

// AsPartitionBackupEntity is the BasicBackupEntity implementation for ServiceBackupEntity.
func (sbe ServiceBackupEntity) AsPartitionBackupEntity() (*PartitionBackupEntity, bool) {
	return nil, false
}

// AsBackupEntity is the BasicBackupEntity implementation for ServiceBackupEntity.
func (sbe ServiceBackupEntity) AsBackupEntity() (*BackupEntity, bool) {
	return nil, false
}

// AsBasicBackupEntity is the BasicBackupEntity implementation for ServiceBackupEntity.
func (sbe ServiceBackupEntity) AsBasicBackupEntity() (BasicBackupEntity, bool) {
	return &sbe, true
}

// ServiceCorrelationDescription creates a particular correlation between services.
type ServiceCorrelationDescription struct {
	// Scheme - The ServiceCorrelationScheme which describes the relationship between this service and the service specified via ServiceName. Possible values include: 'ServiceCorrelationSchemeInvalid', 'ServiceCorrelationSchemeAffinity', 'ServiceCorrelationSchemeAlignedAffinity', 'ServiceCorrelationSchemeNonAlignedAffinity'
	Scheme ServiceCorrelationScheme `json:"Scheme,omitempty"`
	// ServiceName - The name of the service that the correlation relationship is established with.
	ServiceName *string `json:"ServiceName,omitempty"`
}

// ServiceCreatedEvent service Created event.
type ServiceCreatedEvent struct {
	// ServiceTypeName - Service type name.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// ApplicationName - Application name.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ApplicationTypeName - Application type name.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// ServiceInstance - Id of Service instance.
	ServiceInstance *int64 `json:"ServiceInstance,omitempty"`
	// IsStateful - Indicates if Service is stateful.
	IsStateful *bool `json:"IsStateful,omitempty"`
	// PartitionCount - Number of partitions.
	PartitionCount *int32 `json:"PartitionCount,omitempty"`
	// TargetReplicaSetSize - Size of target replicas set.
	TargetReplicaSetSize *int32 `json:"TargetReplicaSetSize,omitempty"`
	// MinReplicaSetSize - Minimum size of replicas set.
	MinReplicaSetSize *int32 `json:"MinReplicaSetSize,omitempty"`
	// ServicePackageVersion - Version of Service package.
	ServicePackageVersion *string `json:"ServicePackageVersion,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ServiceID - The identity of the service. This ID is an encoded representation of the service name. This is used in the REST APIs to identify the service resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the service name is "fabric:/myapp/app1/svc1",
	// the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous versions.
	ServiceID *string `json:"ServiceId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) MarshalJSON() ([]byte, error) {
	sce.Kind = KindServiceCreated
	objectMap := make(map[string]interface{})
	if sce.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = sce.ServiceTypeName
	}
	if sce.ApplicationName != nil {
		objectMap["ApplicationName"] = sce.ApplicationName
	}
	if sce.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = sce.ApplicationTypeName
	}
	if sce.ServiceInstance != nil {
		objectMap["ServiceInstance"] = sce.ServiceInstance
	}
	if sce.IsStateful != nil {
		objectMap["IsStateful"] = sce.IsStateful
	}
	if sce.PartitionCount != nil {
		objectMap["PartitionCount"] = sce.PartitionCount
	}
	if sce.TargetReplicaSetSize != nil {
		objectMap["TargetReplicaSetSize"] = sce.TargetReplicaSetSize
	}
	if sce.MinReplicaSetSize != nil {
		objectMap["MinReplicaSetSize"] = sce.MinReplicaSetSize
	}
	if sce.ServicePackageVersion != nil {
		objectMap["ServicePackageVersion"] = sce.ServicePackageVersion
	}
	if sce.PartitionID != nil {
		objectMap["PartitionId"] = sce.PartitionID
	}
	if sce.ServiceID != nil {
		objectMap["ServiceId"] = sce.ServiceID
	}
	if sce.EventInstanceID != nil {
		objectMap["EventInstanceId"] = sce.EventInstanceID
	}
	if sce.Category != nil {
		objectMap["Category"] = sce.Category
	}
	if sce.TimeStamp != nil {
		objectMap["TimeStamp"] = sce.TimeStamp
	}
	if sce.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = sce.HasCorrelatedEvents
	}
	if sce.Kind != "" {
		objectMap["Kind"] = sce.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return &sce, true
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return &sce, true
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ServiceCreatedEvent.
func (sce ServiceCreatedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &sce, true
}

// ServiceDeletedEvent service Deleted event.
type ServiceDeletedEvent struct {
	// ServiceTypeName - Service type name.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// ApplicationName - Application name.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ApplicationTypeName - Application type name.
	ApplicationTypeName *string `json:"ApplicationTypeName,omitempty"`
	// ServiceInstance - Id of Service instance.
	ServiceInstance *int64 `json:"ServiceInstance,omitempty"`
	// IsStateful - Indicates if Service is stateful.
	IsStateful *bool `json:"IsStateful,omitempty"`
	// PartitionCount - Number of partitions.
	PartitionCount *int32 `json:"PartitionCount,omitempty"`
	// TargetReplicaSetSize - Size of target replicas set.
	TargetReplicaSetSize *int32 `json:"TargetReplicaSetSize,omitempty"`
	// MinReplicaSetSize - Minimum size of replicas set.
	MinReplicaSetSize *int32 `json:"MinReplicaSetSize,omitempty"`
	// ServicePackageVersion - Version of Service package.
	ServicePackageVersion *string `json:"ServicePackageVersion,omitempty"`
	// ServiceID - The identity of the service. This ID is an encoded representation of the service name. This is used in the REST APIs to identify the service resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the service name is "fabric:/myapp/app1/svc1",
	// the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous versions.
	ServiceID *string `json:"ServiceId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) MarshalJSON() ([]byte, error) {
	sde.Kind = KindServiceDeleted
	objectMap := make(map[string]interface{})
	if sde.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = sde.ServiceTypeName
	}
	if sde.ApplicationName != nil {
		objectMap["ApplicationName"] = sde.ApplicationName
	}
	if sde.ApplicationTypeName != nil {
		objectMap["ApplicationTypeName"] = sde.ApplicationTypeName
	}
	if sde.ServiceInstance != nil {
		objectMap["ServiceInstance"] = sde.ServiceInstance
	}
	if sde.IsStateful != nil {
		objectMap["IsStateful"] = sde.IsStateful
	}
	if sde.PartitionCount != nil {
		objectMap["PartitionCount"] = sde.PartitionCount
	}
	if sde.TargetReplicaSetSize != nil {
		objectMap["TargetReplicaSetSize"] = sde.TargetReplicaSetSize
	}
	if sde.MinReplicaSetSize != nil {
		objectMap["MinReplicaSetSize"] = sde.MinReplicaSetSize
	}
	if sde.ServicePackageVersion != nil {
		objectMap["ServicePackageVersion"] = sde.ServicePackageVersion
	}
	if sde.ServiceID != nil {
		objectMap["ServiceId"] = sde.ServiceID
	}
	if sde.EventInstanceID != nil {
		objectMap["EventInstanceId"] = sde.EventInstanceID
	}
	if sde.Category != nil {
		objectMap["Category"] = sde.Category
	}
	if sde.TimeStamp != nil {
		objectMap["TimeStamp"] = sde.TimeStamp
	}
	if sde.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = sde.HasCorrelatedEvents
	}
	if sde.Kind != "" {
		objectMap["Kind"] = sde.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return &sde, true
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return &sde, true
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ServiceDeletedEvent.
func (sde ServiceDeletedEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &sde, true
}

// BasicServiceDescription a ServiceDescription contains all of the information necessary to create a service.
type BasicServiceDescription interface {
	AsStatefulServiceDescription() (*StatefulServiceDescription, bool)
	AsStatelessServiceDescription() (*StatelessServiceDescription, bool)
	AsServiceDescription() (*ServiceDescription, bool)
}

// ServiceDescription a ServiceDescription contains all of the information necessary to create a service.
type ServiceDescription struct {
	autorest.Response `json:"-"`
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// InitializationData - The initialization data as an array of bytes. Initialization data is passed to service instances or replicas when they are created.
	InitializationData *[]int32 `json:"InitializationData,omitempty"`
	// PartitionDescription - The partition description as an object.
	PartitionDescription BasicPartitionSchemeDescription `json:"PartitionDescription,omitempty"`
	// PlacementConstraints - The placement constraints as a string. Placement constraints are boolean expressions on node properties and allow for restricting a service to particular nodes based on the service requirements. For example, to place a service on nodes where NodeType is blue specify the following: "NodeColor == blue)".
	PlacementConstraints *string `json:"PlacementConstraints,omitempty"`
	// CorrelationScheme - The correlation scheme.
	CorrelationScheme *[]ServiceCorrelationDescription `json:"CorrelationScheme,omitempty"`
	// ServiceLoadMetrics - The service load metrics.
	ServiceLoadMetrics *[]ServiceLoadMetricDescription `json:"ServiceLoadMetrics,omitempty"`
	// ServicePlacementPolicies - The service placement policies.
	ServicePlacementPolicies *[]BasicServicePlacementPolicyDescription `json:"ServicePlacementPolicies,omitempty"`
	// DefaultMoveCost - The move cost for the service. Possible values include: 'Zero', 'Low', 'Medium', 'High'
	DefaultMoveCost MoveCost `json:"DefaultMoveCost,omitempty"`
	// IsDefaultMoveCostSpecified - Indicates if the DefaultMoveCost property is specified.
	IsDefaultMoveCostSpecified *bool `json:"IsDefaultMoveCostSpecified,omitempty"`
	// ServicePackageActivationMode - The activation mode of service package to be used for a service. Possible values include: 'SharedProcess', 'ExclusiveProcess'
	ServicePackageActivationMode ServicePackageActivationMode `json:"ServicePackageActivationMode,omitempty"`
	// ServiceDNSName - The DNS name of the service. It requires the DNS system service to be enabled in Service Fabric cluster.
	ServiceDNSName *string `json:"ServiceDnsName,omitempty"`
	// ScalingPolicies - Scaling policies for this service.
	ScalingPolicies *[]ScalingPolicyDescription `json:"ScalingPolicies,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServiceDescriptionServiceKindServiceDescription', 'ServiceKindBasicServiceDescriptionServiceKindStateful', 'ServiceKindBasicServiceDescriptionServiceKindStateless'
	ServiceKind ServiceKindBasicServiceDescription `json:"ServiceKind,omitempty"`
}

func unmarshalBasicServiceDescription(body []byte) (BasicServiceDescription, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ServiceKind"] {
	case string(ServiceKindBasicServiceDescriptionServiceKindStateful):
		var ssd StatefulServiceDescription
		err := json.Unmarshal(body, &ssd)
		return ssd, err
	case string(ServiceKindBasicServiceDescriptionServiceKindStateless):
		var ssd StatelessServiceDescription
		err := json.Unmarshal(body, &ssd)
		return ssd, err
	default:
		var sd ServiceDescription
		err := json.Unmarshal(body, &sd)
		return sd, err
	}
}
func unmarshalBasicServiceDescriptionArray(body []byte) ([]BasicServiceDescription, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	sdArray := make([]BasicServiceDescription, len(rawMessages))

	for index, rawMessage := range rawMessages {
		sd, err := unmarshalBasicServiceDescription(*rawMessage)
		if err != nil {
			return nil, err
		}
		sdArray[index] = sd
	}
	return sdArray, nil
}

// MarshalJSON is the custom marshaler for ServiceDescription.
func (sd ServiceDescription) MarshalJSON() ([]byte, error) {
	sd.ServiceKind = ServiceKindBasicServiceDescriptionServiceKindServiceDescription
	objectMap := make(map[string]interface{})
	if sd.ApplicationName != nil {
		objectMap["ApplicationName"] = sd.ApplicationName
	}
	if sd.ServiceName != nil {
		objectMap["ServiceName"] = sd.ServiceName
	}
	if sd.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = sd.ServiceTypeName
	}
	if sd.InitializationData != nil {
		objectMap["InitializationData"] = sd.InitializationData
	}
	objectMap["PartitionDescription"] = sd.PartitionDescription
	if sd.PlacementConstraints != nil {
		objectMap["PlacementConstraints"] = sd.PlacementConstraints
	}
	if sd.CorrelationScheme != nil {
		objectMap["CorrelationScheme"] = sd.CorrelationScheme
	}
	if sd.ServiceLoadMetrics != nil {
		objectMap["ServiceLoadMetrics"] = sd.ServiceLoadMetrics
	}
	if sd.ServicePlacementPolicies != nil {
		objectMap["ServicePlacementPolicies"] = sd.ServicePlacementPolicies
	}
	if sd.DefaultMoveCost != "" {
		objectMap["DefaultMoveCost"] = sd.DefaultMoveCost
	}
	if sd.IsDefaultMoveCostSpecified != nil {
		objectMap["IsDefaultMoveCostSpecified"] = sd.IsDefaultMoveCostSpecified
	}
	if sd.ServicePackageActivationMode != "" {
		objectMap["ServicePackageActivationMode"] = sd.ServicePackageActivationMode
	}
	if sd.ServiceDNSName != nil {
		objectMap["ServiceDnsName"] = sd.ServiceDNSName
	}
	if sd.ScalingPolicies != nil {
		objectMap["ScalingPolicies"] = sd.ScalingPolicies
	}
	if sd.ServiceKind != "" {
		objectMap["ServiceKind"] = sd.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceDescription is the BasicServiceDescription implementation for ServiceDescription.
func (sd ServiceDescription) AsStatefulServiceDescription() (*StatefulServiceDescription, bool) {
	return nil, false
}

// AsStatelessServiceDescription is the BasicServiceDescription implementation for ServiceDescription.
func (sd ServiceDescription) AsStatelessServiceDescription() (*StatelessServiceDescription, bool) {
	return nil, false
}

// AsServiceDescription is the BasicServiceDescription implementation for ServiceDescription.
func (sd ServiceDescription) AsServiceDescription() (*ServiceDescription, bool) {
	return &sd, true
}

// AsBasicServiceDescription is the BasicServiceDescription implementation for ServiceDescription.
func (sd ServiceDescription) AsBasicServiceDescription() (BasicServiceDescription, bool) {
	return &sd, true
}

// UnmarshalJSON is the custom unmarshaler for ServiceDescription struct.
func (sd *ServiceDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "ApplicationName":
			if v != nil {
				var applicationName string
				err = json.Unmarshal(*v, &applicationName)
				if err != nil {
					return err
				}
				sd.ApplicationName = &applicationName
			}
		case "ServiceName":
			if v != nil {
				var serviceName string
				err = json.Unmarshal(*v, &serviceName)
				if err != nil {
					return err
				}
				sd.ServiceName = &serviceName
			}
		case "ServiceTypeName":
			if v != nil {
				var serviceTypeName string
				err = json.Unmarshal(*v, &serviceTypeName)
				if err != nil {
					return err
				}
				sd.ServiceTypeName = &serviceTypeName
			}
		case "InitializationData":
			if v != nil {
				var initializationData []int32
				err = json.Unmarshal(*v, &initializationData)
				if err != nil {
					return err
				}
				sd.InitializationData = &initializationData
			}
		case "PartitionDescription":
			if v != nil {
				partitionDescription, err := unmarshalBasicPartitionSchemeDescription(*v)
				if err != nil {
					return err
				}
				sd.PartitionDescription = partitionDescription
			}
		case "PlacementConstraints":
			if v != nil {
				var placementConstraints string
				err = json.Unmarshal(*v, &placementConstraints)
				if err != nil {
					return err
				}
				sd.PlacementConstraints = &placementConstraints
			}
		case "CorrelationScheme":
			if v != nil {
				var correlationScheme []ServiceCorrelationDescription
				err = json.Unmarshal(*v, &correlationScheme)
				if err != nil {
					return err
				}
				sd.CorrelationScheme = &correlationScheme
			}
		case "ServiceLoadMetrics":
			if v != nil {
				var serviceLoadMetrics []ServiceLoadMetricDescription
				err = json.Unmarshal(*v, &serviceLoadMetrics)
				if err != nil {
					return err
				}
				sd.ServiceLoadMetrics = &serviceLoadMetrics
			}
		case "ServicePlacementPolicies":
			if v != nil {
				servicePlacementPolicies, err := unmarshalBasicServicePlacementPolicyDescriptionArray(*v)
				if err != nil {
					return err
				}
				sd.ServicePlacementPolicies = &servicePlacementPolicies
			}
		case "DefaultMoveCost":
			if v != nil {
				var defaultMoveCost MoveCost
				err = json.Unmarshal(*v, &defaultMoveCost)
				if err != nil {
					return err
				}
				sd.DefaultMoveCost = defaultMoveCost
			}
		case "IsDefaultMoveCostSpecified":
			if v != nil {
				var isDefaultMoveCostSpecified bool
				err = json.Unmarshal(*v, &isDefaultMoveCostSpecified)
				if err != nil {
					return err
				}
				sd.IsDefaultMoveCostSpecified = &isDefaultMoveCostSpecified
			}
		case "ServicePackageActivationMode":
			if v != nil {
				var servicePackageActivationMode ServicePackageActivationMode
				err = json.Unmarshal(*v, &servicePackageActivationMode)
				if err != nil {
					return err
				}
				sd.ServicePackageActivationMode = servicePackageActivationMode
			}
		case "ServiceDnsName":
			if v != nil {
				var serviceDNSName string
				err = json.Unmarshal(*v, &serviceDNSName)
				if err != nil {
					return err
				}
				sd.ServiceDNSName = &serviceDNSName
			}
		case "ScalingPolicies":
			if v != nil {
				var scalingPolicies []ScalingPolicyDescription
				err = json.Unmarshal(*v, &scalingPolicies)
				if err != nil {
					return err
				}
				sd.ScalingPolicies = &scalingPolicies
			}
		case "ServiceKind":
			if v != nil {
				var serviceKind ServiceKindBasicServiceDescription
				err = json.Unmarshal(*v, &serviceKind)
				if err != nil {
					return err
				}
				sd.ServiceKind = serviceKind
			}
		}
	}

	return nil
}

// ServiceDescriptionModel ...
type ServiceDescriptionModel struct {
	autorest.Response `json:"-"`
	Value             BasicServiceDescription `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ServiceDescriptionModel struct.
func (sdm *ServiceDescriptionModel) UnmarshalJSON(body []byte) error {
	sd, err := unmarshalBasicServiceDescription(body)
	if err != nil {
		return err
	}
	sdm.Value = sd

	return nil
}

// BasicServiceEvent represents the base for all Service Events.
type BasicServiceEvent interface {
	AsServiceCreatedEvent() (*ServiceCreatedEvent, bool)
	AsServiceDeletedEvent() (*ServiceDeletedEvent, bool)
	AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool)
	AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool)
	AsServiceEvent() (*ServiceEvent, bool)
}

// ServiceEvent represents the base for all Service Events.
type ServiceEvent struct {
	// ServiceID - The identity of the service. This ID is an encoded representation of the service name. This is used in the REST APIs to identify the service resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the service name is "fabric:/myapp/app1/svc1",
	// the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous versions.
	ServiceID *string `json:"ServiceId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

func unmarshalBasicServiceEvent(body []byte) (BasicServiceEvent, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindServiceCreated):
		var sce ServiceCreatedEvent
		err := json.Unmarshal(body, &sce)
		return sce, err
	case string(KindServiceDeleted):
		var sde ServiceDeletedEvent
		err := json.Unmarshal(body, &sde)
		return sde, err
	case string(KindServiceNewHealthReport):
		var snhre ServiceNewHealthReportEvent
		err := json.Unmarshal(body, &snhre)
		return snhre, err
	case string(KindServiceHealthReportExpired):
		var shree ServiceHealthReportExpiredEvent
		err := json.Unmarshal(body, &shree)
		return shree, err
	default:
		var se ServiceEvent
		err := json.Unmarshal(body, &se)
		return se, err
	}
}
func unmarshalBasicServiceEventArray(body []byte) ([]BasicServiceEvent, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	seArray := make([]BasicServiceEvent, len(rawMessages))

	for index, rawMessage := range rawMessages {
		se, err := unmarshalBasicServiceEvent(*rawMessage)
		if err != nil {
			return nil, err
		}
		seArray[index] = se
	}
	return seArray, nil
}

// MarshalJSON is the custom marshaler for ServiceEvent.
func (se ServiceEvent) MarshalJSON() ([]byte, error) {
	se.Kind = KindServiceEvent
	objectMap := make(map[string]interface{})
	if se.ServiceID != nil {
		objectMap["ServiceId"] = se.ServiceID
	}
	if se.EventInstanceID != nil {
		objectMap["EventInstanceId"] = se.EventInstanceID
	}
	if se.Category != nil {
		objectMap["Category"] = se.Category
	}
	if se.TimeStamp != nil {
		objectMap["TimeStamp"] = se.TimeStamp
	}
	if se.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = se.HasCorrelatedEvents
	}
	if se.Kind != "" {
		objectMap["Kind"] = se.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return &se, true
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return &se, true
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ServiceEvent.
func (se ServiceEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &se, true
}

// ServiceFromTemplateDescription defines description for creating a Service Fabric service from a template
// defined in the application manifest.
type ServiceFromTemplateDescription struct {
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// InitializationData - The initialization data for the newly created service instance.
	InitializationData *[]int32 `json:"InitializationData,omitempty"`
	// ServicePackageActivationMode - The activation mode of service package to be used for a service. Possible values include: 'SharedProcess', 'ExclusiveProcess'
	ServicePackageActivationMode ServicePackageActivationMode `json:"ServicePackageActivationMode,omitempty"`
	// ServiceDNSName - The DNS name of the service. It requires the DNS system service to be enabled in Service Fabric cluster.
	ServiceDNSName *string `json:"ServiceDnsName,omitempty"`
}

// ServiceHealth information about the health of a Service Fabric service.
type ServiceHealth struct {
	autorest.Response `json:"-"`
	// Name - The name of the service whose health information is described by this object.
	Name *string `json:"Name,omitempty"`
	// PartitionHealthStates - The list of partition health states associated with the service.
	PartitionHealthStates *[]PartitionHealthState `json:"PartitionHealthStates,omitempty"`
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

// ServiceHealthEvaluation represents health evaluation for a service, containing information about the
// data and the algorithm used by health store to evaluate health. The evaluation is returned only when the
// aggregated health state is either Error or Warning.
type ServiceHealthEvaluation struct {
	// ServiceName - Name of the service whose health evaluation is described by this object.
	ServiceName *string `json:"ServiceName,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the current aggregated health state of the service. The types of the unhealthy evaluations can be PartitionsHealthEvaluation or EventHealthEvaluation.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) MarshalJSON() ([]byte, error) {
	she.Kind = KindService
	objectMap := make(map[string]interface{})
	if she.ServiceName != nil {
		objectMap["ServiceName"] = she.ServiceName
	}
	if she.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = she.UnhealthyEvaluations
	}
	if she.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = she.AggregatedHealthState
	}
	if she.Description != nil {
		objectMap["Description"] = she.Description
	}
	if she.Kind != "" {
		objectMap["Kind"] = she.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return &she, true
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for ServiceHealthEvaluation.
func (she ServiceHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &she, true
}

// ServiceHealthReportExpiredEvent service Health Report Expired event.
type ServiceHealthReportExpiredEvent struct {
	// InstanceID - Id of Service instance.
	InstanceID *int64 `json:"InstanceId,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// ServiceID - The identity of the service. This ID is an encoded representation of the service name. This is used in the REST APIs to identify the service resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the service name is "fabric:/myapp/app1/svc1",
	// the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous versions.
	ServiceID *string `json:"ServiceId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) MarshalJSON() ([]byte, error) {
	shree.Kind = KindServiceHealthReportExpired
	objectMap := make(map[string]interface{})
	if shree.InstanceID != nil {
		objectMap["InstanceId"] = shree.InstanceID
	}
	if shree.SourceID != nil {
		objectMap["SourceId"] = shree.SourceID
	}
	if shree.Property != nil {
		objectMap["Property"] = shree.Property
	}
	if shree.HealthState != nil {
		objectMap["HealthState"] = shree.HealthState
	}
	if shree.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = shree.TimeToLiveMs
	}
	if shree.SequenceNumber != nil {
		objectMap["SequenceNumber"] = shree.SequenceNumber
	}
	if shree.Description != nil {
		objectMap["Description"] = shree.Description
	}
	if shree.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = shree.RemoveWhenExpired
	}
	if shree.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = shree.SourceUtcTimestamp
	}
	if shree.ServiceID != nil {
		objectMap["ServiceId"] = shree.ServiceID
	}
	if shree.EventInstanceID != nil {
		objectMap["EventInstanceId"] = shree.EventInstanceID
	}
	if shree.Category != nil {
		objectMap["Category"] = shree.Category
	}
	if shree.TimeStamp != nil {
		objectMap["TimeStamp"] = shree.TimeStamp
	}
	if shree.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = shree.HasCorrelatedEvents
	}
	if shree.Kind != "" {
		objectMap["Kind"] = shree.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return &shree, true
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return &shree, true
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ServiceHealthReportExpiredEvent.
func (shree ServiceHealthReportExpiredEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &shree, true
}

// ServiceHealthState represents the health state of a service, which contains the service identifier and
// its aggregated health state.
type ServiceHealthState struct {
	// ServiceName - Name of the service whose health state is represented by this object.
	ServiceName *string `json:"ServiceName,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
}

// ServiceHealthStateChunk represents the health state chunk of a service, which contains the service name,
// its aggregated health state and any partitions that respect the filters in the cluster health chunk
// query description.
type ServiceHealthStateChunk struct {
	// ServiceName - The name of the service whose health state chunk is provided in this object.
	ServiceName *string `json:"ServiceName,omitempty"`
	// PartitionHealthStateChunks - The list of partition health state chunks belonging to the service that respect the filters in the cluster health chunk query description.
	PartitionHealthStateChunks *PartitionHealthStateChunkList `json:"PartitionHealthStateChunks,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
}

// ServiceHealthStateChunkList the list of service health state chunks that respect the input filters in
// the chunk query. Returned by get cluster health state chunks query.
type ServiceHealthStateChunkList struct {
	// Items - The list of service health state chunks that respect the input filters in the chunk query.
	Items *[]ServiceHealthStateChunk `json:"Items,omitempty"`
}

// ServiceHealthStateFilter defines matching criteria to determine whether a service should be included as
// a child of an application in the cluster health chunk.
// The services are only returned if the parent application matches a filter specified in the cluster
// health chunk query description.
// One filter can match zero, one or multiple services, depending on its properties.
type ServiceHealthStateFilter struct {
	// ServiceNameFilter - The name of the service that matches the filter. The filter is applied only to the specified service, if it exists.
	// If the service doesn't exist, no service is returned in the cluster health chunk based on this filter.
	// If the service exists, it is included as the application's child if the health state matches the other filter properties.
	// If not specified, all services that match the parent filters (if any) are taken into consideration and matched against the other filter members, like health state filter.
	ServiceNameFilter *string `json:"ServiceNameFilter,omitempty"`
	// HealthStateFilter - The filter for the health state of the services. It allows selecting services if they match the desired health states.
	// The possible values are integer value of one of the following health states. Only services that match the filter are returned. All services are used to evaluate the cluster aggregated health state.
	// If not specified, default value is None, unless the service name is specified. If the filter has default value and service name is specified, the matching service is returned.
	// The state values are flag-based enumeration, so the value could be a combination of these values obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6, it matches services with HealthState value of OK (2) and Warning (4).
	// - Default - Default value. Matches any HealthState. The value is zero.
	// - None - Filter that doesn't match any HealthState value. Used in order to return no results on a given collection of states. The value is 1.
	// - Ok - Filter that matches input with HealthState value Ok. The value is 2.
	// - Warning - Filter that matches input with HealthState value Warning. The value is 4.
	// - Error - Filter that matches input with HealthState value Error. The value is 8.
	// - All - Filter that matches input with any HealthState value. The value is 65535.
	HealthStateFilter *int32 `json:"HealthStateFilter,omitempty"`
	// PartitionFilters - Defines a list of filters that specify which partitions to be included in the returned cluster health chunk as children of the service. The partitions are returned only if the parent service matches a filter.
	// If the list is empty, no partitions are returned. All the partitions are used to evaluate the parent service aggregated health state, regardless of the input filters.
	// The service filter may specify multiple partition filters.
	// For example, it can specify a filter to return all partitions with health state Error and another filter to always include a partition identified by its partition ID.
	PartitionFilters *[]PartitionHealthStateFilter `json:"PartitionFilters,omitempty"`
}

// ServiceIdentity map service identity friendly name to an application identity.
type ServiceIdentity struct {
	// Name - The identity friendly name.
	Name *string `json:"name,omitempty"`
	// IdentityRef - The application identity name.
	IdentityRef *string `json:"identityRef,omitempty"`
}

// BasicServiceInfo information about a Service Fabric service.
type BasicServiceInfo interface {
	AsStatefulServiceInfo() (*StatefulServiceInfo, bool)
	AsStatelessServiceInfo() (*StatelessServiceInfo, bool)
	AsServiceInfo() (*ServiceInfo, bool)
}

// ServiceInfo information about a Service Fabric service.
type ServiceInfo struct {
	autorest.Response `json:"-"`
	// ID - The identity of the service. This ID is an encoded representation of the service name. This is used in the REST APIs to identify the service resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the service name is "fabric:/myapp/app1/svc1",
	// the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous versions.
	ID *string `json:"Id,omitempty"`
	// Name - The full name of the service with 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// TypeName - Name of the service type as specified in the service manifest.
	TypeName *string `json:"TypeName,omitempty"`
	// ManifestVersion - The version of the service manifest.
	ManifestVersion *string `json:"ManifestVersion,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// ServiceStatus - The status of the application. Possible values include: 'ServiceStatusUnknown', 'ServiceStatusActive', 'ServiceStatusUpgrading', 'ServiceStatusDeleting', 'ServiceStatusCreating', 'ServiceStatusFailed'
	ServiceStatus ServiceStatus `json:"ServiceStatus,omitempty"`
	// IsServiceGroup - Whether the service is in a service group.
	IsServiceGroup *bool `json:"IsServiceGroup,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServiceInfoServiceKindServiceInfo', 'ServiceKindBasicServiceInfoServiceKindStateful', 'ServiceKindBasicServiceInfoServiceKindStateless'
	ServiceKind ServiceKindBasicServiceInfo `json:"ServiceKind,omitempty"`
}

func unmarshalBasicServiceInfo(body []byte) (BasicServiceInfo, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ServiceKind"] {
	case string(ServiceKindBasicServiceInfoServiceKindStateful):
		var ssi StatefulServiceInfo
		err := json.Unmarshal(body, &ssi)
		return ssi, err
	case string(ServiceKindBasicServiceInfoServiceKindStateless):
		var ssi StatelessServiceInfo
		err := json.Unmarshal(body, &ssi)
		return ssi, err
	default:
		var si ServiceInfo
		err := json.Unmarshal(body, &si)
		return si, err
	}
}
func unmarshalBasicServiceInfoArray(body []byte) ([]BasicServiceInfo, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	siArray := make([]BasicServiceInfo, len(rawMessages))

	for index, rawMessage := range rawMessages {
		si, err := unmarshalBasicServiceInfo(*rawMessage)
		if err != nil {
			return nil, err
		}
		siArray[index] = si
	}
	return siArray, nil
}

// MarshalJSON is the custom marshaler for ServiceInfo.
func (si ServiceInfo) MarshalJSON() ([]byte, error) {
	si.ServiceKind = ServiceKindBasicServiceInfoServiceKindServiceInfo
	objectMap := make(map[string]interface{})
	if si.ID != nil {
		objectMap["Id"] = si.ID
	}
	if si.Name != nil {
		objectMap["Name"] = si.Name
	}
	if si.TypeName != nil {
		objectMap["TypeName"] = si.TypeName
	}
	if si.ManifestVersion != nil {
		objectMap["ManifestVersion"] = si.ManifestVersion
	}
	if si.HealthState != "" {
		objectMap["HealthState"] = si.HealthState
	}
	if si.ServiceStatus != "" {
		objectMap["ServiceStatus"] = si.ServiceStatus
	}
	if si.IsServiceGroup != nil {
		objectMap["IsServiceGroup"] = si.IsServiceGroup
	}
	if si.ServiceKind != "" {
		objectMap["ServiceKind"] = si.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceInfo is the BasicServiceInfo implementation for ServiceInfo.
func (si ServiceInfo) AsStatefulServiceInfo() (*StatefulServiceInfo, bool) {
	return nil, false
}

// AsStatelessServiceInfo is the BasicServiceInfo implementation for ServiceInfo.
func (si ServiceInfo) AsStatelessServiceInfo() (*StatelessServiceInfo, bool) {
	return nil, false
}

// AsServiceInfo is the BasicServiceInfo implementation for ServiceInfo.
func (si ServiceInfo) AsServiceInfo() (*ServiceInfo, bool) {
	return &si, true
}

// AsBasicServiceInfo is the BasicServiceInfo implementation for ServiceInfo.
func (si ServiceInfo) AsBasicServiceInfo() (BasicServiceInfo, bool) {
	return &si, true
}

// ServiceInfoModel ...
type ServiceInfoModel struct {
	autorest.Response `json:"-"`
	Value             BasicServiceInfo `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ServiceInfoModel struct.
func (sim *ServiceInfoModel) UnmarshalJSON(body []byte) error {
	si, err := unmarshalBasicServiceInfo(body)
	if err != nil {
		return err
	}
	sim.Value = si

	return nil
}

// ServiceLoadMetricDescription specifies a metric to load balance a service during runtime.
type ServiceLoadMetricDescription struct {
	// Name - The name of the metric. If the service chooses to report load during runtime, the load metric name should match the name that is specified in Name exactly. Note that metric names are case-sensitive.
	Name *string `json:"Name,omitempty"`
	// Weight - The service load metric relative weight, compared to other metrics configured for this service, as a number. Possible values include: 'ServiceLoadMetricWeightZero', 'ServiceLoadMetricWeightLow', 'ServiceLoadMetricWeightMedium', 'ServiceLoadMetricWeightHigh'
	Weight ServiceLoadMetricWeight `json:"Weight,omitempty"`
	// PrimaryDefaultLoad - Used only for Stateful services. The default amount of load, as a number, that this service creates for this metric when it is a Primary replica.
	PrimaryDefaultLoad *int32 `json:"PrimaryDefaultLoad,omitempty"`
	// SecondaryDefaultLoad - Used only for Stateful services. The default amount of load, as a number, that this service creates for this metric when it is a Secondary replica.
	SecondaryDefaultLoad *int32 `json:"SecondaryDefaultLoad,omitempty"`
	// DefaultLoad - Used only for Stateless services. The default amount of load, as a number, that this service creates for this metric.
	DefaultLoad *int32 `json:"DefaultLoad,omitempty"`
}

// ServiceNameInfo information about the service name.
type ServiceNameInfo struct {
	autorest.Response `json:"-"`
	// ID - The identity of the service. This ID is an encoded representation of the service name. This is used in the REST APIs to identify the service resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the service name is "fabric:/myapp/app1/svc1",
	// the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous versions.
	ID *string `json:"Id,omitempty"`
	// Name - The full name of the service with 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
}

// ServiceNewHealthReportEvent service Health Report Created event.
type ServiceNewHealthReportEvent struct {
	// InstanceID - Id of Service instance.
	InstanceID *int64 `json:"InstanceId,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// ServiceID - The identity of the service. This ID is an encoded representation of the service name. This is used in the REST APIs to identify the service resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the service name is "fabric:/myapp/app1/svc1",
	// the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous versions.
	ServiceID *string `json:"ServiceId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) MarshalJSON() ([]byte, error) {
	snhre.Kind = KindServiceNewHealthReport
	objectMap := make(map[string]interface{})
	if snhre.InstanceID != nil {
		objectMap["InstanceId"] = snhre.InstanceID
	}
	if snhre.SourceID != nil {
		objectMap["SourceId"] = snhre.SourceID
	}
	if snhre.Property != nil {
		objectMap["Property"] = snhre.Property
	}
	if snhre.HealthState != nil {
		objectMap["HealthState"] = snhre.HealthState
	}
	if snhre.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = snhre.TimeToLiveMs
	}
	if snhre.SequenceNumber != nil {
		objectMap["SequenceNumber"] = snhre.SequenceNumber
	}
	if snhre.Description != nil {
		objectMap["Description"] = snhre.Description
	}
	if snhre.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = snhre.RemoveWhenExpired
	}
	if snhre.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = snhre.SourceUtcTimestamp
	}
	if snhre.ServiceID != nil {
		objectMap["ServiceId"] = snhre.ServiceID
	}
	if snhre.EventInstanceID != nil {
		objectMap["EventInstanceId"] = snhre.EventInstanceID
	}
	if snhre.Category != nil {
		objectMap["Category"] = snhre.Category
	}
	if snhre.TimeStamp != nil {
		objectMap["TimeStamp"] = snhre.TimeStamp
	}
	if snhre.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = snhre.HasCorrelatedEvents
	}
	if snhre.Kind != "" {
		objectMap["Kind"] = snhre.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return nil, false
}

// AsServiceEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return &snhre, true
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return &snhre, true
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for ServiceNewHealthReportEvent.
func (snhre ServiceNewHealthReportEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &snhre, true
}

// BasicServicePartitionInfo information about a partition of a Service Fabric service.
type BasicServicePartitionInfo interface {
	AsStatefulServicePartitionInfo() (*StatefulServicePartitionInfo, bool)
	AsStatelessServicePartitionInfo() (*StatelessServicePartitionInfo, bool)
	AsServicePartitionInfo() (*ServicePartitionInfo, bool)
}

// ServicePartitionInfo information about a partition of a Service Fabric service.
type ServicePartitionInfo struct {
	autorest.Response `json:"-"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// PartitionStatus - The status of the service fabric service partition. Possible values include: 'ServicePartitionStatusInvalid', 'ServicePartitionStatusReady', 'ServicePartitionStatusNotReady', 'ServicePartitionStatusInQuorumLoss', 'ServicePartitionStatusReconfiguring', 'ServicePartitionStatusDeleting'
	PartitionStatus ServicePartitionStatus `json:"PartitionStatus,omitempty"`
	// PartitionInformation - Information about the partition identity, partitioning scheme and keys supported by it.
	PartitionInformation BasicPartitionInformation `json:"PartitionInformation,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServicePartitionInfoServiceKindServicePartitionInfo', 'ServiceKindBasicServicePartitionInfoServiceKindStateful', 'ServiceKindBasicServicePartitionInfoServiceKindStateless'
	ServiceKind ServiceKindBasicServicePartitionInfo `json:"ServiceKind,omitempty"`
}

func unmarshalBasicServicePartitionInfo(body []byte) (BasicServicePartitionInfo, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ServiceKind"] {
	case string(ServiceKindBasicServicePartitionInfoServiceKindStateful):
		var sspi StatefulServicePartitionInfo
		err := json.Unmarshal(body, &sspi)
		return sspi, err
	case string(ServiceKindBasicServicePartitionInfoServiceKindStateless):
		var sspi StatelessServicePartitionInfo
		err := json.Unmarshal(body, &sspi)
		return sspi, err
	default:
		var spi ServicePartitionInfo
		err := json.Unmarshal(body, &spi)
		return spi, err
	}
}
func unmarshalBasicServicePartitionInfoArray(body []byte) ([]BasicServicePartitionInfo, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	spiArray := make([]BasicServicePartitionInfo, len(rawMessages))

	for index, rawMessage := range rawMessages {
		spi, err := unmarshalBasicServicePartitionInfo(*rawMessage)
		if err != nil {
			return nil, err
		}
		spiArray[index] = spi
	}
	return spiArray, nil
}

// MarshalJSON is the custom marshaler for ServicePartitionInfo.
func (spi ServicePartitionInfo) MarshalJSON() ([]byte, error) {
	spi.ServiceKind = ServiceKindBasicServicePartitionInfoServiceKindServicePartitionInfo
	objectMap := make(map[string]interface{})
	if spi.HealthState != "" {
		objectMap["HealthState"] = spi.HealthState
	}
	if spi.PartitionStatus != "" {
		objectMap["PartitionStatus"] = spi.PartitionStatus
	}
	objectMap["PartitionInformation"] = spi.PartitionInformation
	if spi.ServiceKind != "" {
		objectMap["ServiceKind"] = spi.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServicePartitionInfo is the BasicServicePartitionInfo implementation for ServicePartitionInfo.
func (spi ServicePartitionInfo) AsStatefulServicePartitionInfo() (*StatefulServicePartitionInfo, bool) {
	return nil, false
}

// AsStatelessServicePartitionInfo is the BasicServicePartitionInfo implementation for ServicePartitionInfo.
func (spi ServicePartitionInfo) AsStatelessServicePartitionInfo() (*StatelessServicePartitionInfo, bool) {
	return nil, false
}

// AsServicePartitionInfo is the BasicServicePartitionInfo implementation for ServicePartitionInfo.
func (spi ServicePartitionInfo) AsServicePartitionInfo() (*ServicePartitionInfo, bool) {
	return &spi, true
}

// AsBasicServicePartitionInfo is the BasicServicePartitionInfo implementation for ServicePartitionInfo.
func (spi ServicePartitionInfo) AsBasicServicePartitionInfo() (BasicServicePartitionInfo, bool) {
	return &spi, true
}

// UnmarshalJSON is the custom unmarshaler for ServicePartitionInfo struct.
func (spi *ServicePartitionInfo) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "HealthState":
			if v != nil {
				var healthState HealthState
				err = json.Unmarshal(*v, &healthState)
				if err != nil {
					return err
				}
				spi.HealthState = healthState
			}
		case "PartitionStatus":
			if v != nil {
				var partitionStatus ServicePartitionStatus
				err = json.Unmarshal(*v, &partitionStatus)
				if err != nil {
					return err
				}
				spi.PartitionStatus = partitionStatus
			}
		case "PartitionInformation":
			if v != nil {
				partitionInformation, err := unmarshalBasicPartitionInformation(*v)
				if err != nil {
					return err
				}
				spi.PartitionInformation = partitionInformation
			}
		case "ServiceKind":
			if v != nil {
				var serviceKind ServiceKindBasicServicePartitionInfo
				err = json.Unmarshal(*v, &serviceKind)
				if err != nil {
					return err
				}
				spi.ServiceKind = serviceKind
			}
		}
	}

	return nil
}

// ServicePartitionInfoModel ...
type ServicePartitionInfoModel struct {
	autorest.Response `json:"-"`
	Value             BasicServicePartitionInfo `json:"value,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ServicePartitionInfoModel struct.
func (spim *ServicePartitionInfoModel) UnmarshalJSON(body []byte) error {
	spi, err := unmarshalBasicServicePartitionInfo(body)
	if err != nil {
		return err
	}
	spim.Value = spi

	return nil
}

// ServicePlacementInvalidDomainPolicyDescription describes the policy to be used for placement of a
// Service Fabric service where a particular fault or upgrade domain should not be used for placement of
// the instances or replicas of that service.
type ServicePlacementInvalidDomainPolicyDescription struct {
	// DomainName - The name of the domain that should not be used for placement.
	DomainName *string `json:"DomainName,omitempty"`
	// Type - Possible values include: 'TypeServicePlacementPolicyDescription', 'TypeInvalidDomain', 'TypeNonPartiallyPlaceService', 'TypePreferPrimaryDomain', 'TypeRequireDomain', 'TypeRequireDomainDistribution'
	Type Type `json:"Type,omitempty"`
}

// MarshalJSON is the custom marshaler for ServicePlacementInvalidDomainPolicyDescription.
func (spidpd ServicePlacementInvalidDomainPolicyDescription) MarshalJSON() ([]byte, error) {
	spidpd.Type = TypeInvalidDomain
	objectMap := make(map[string]interface{})
	if spidpd.DomainName != nil {
		objectMap["DomainName"] = spidpd.DomainName
	}
	if spidpd.Type != "" {
		objectMap["Type"] = spidpd.Type
	}
	return json.Marshal(objectMap)
}

// AsServicePlacementInvalidDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementInvalidDomainPolicyDescription.
func (spidpd ServicePlacementInvalidDomainPolicyDescription) AsServicePlacementInvalidDomainPolicyDescription() (*ServicePlacementInvalidDomainPolicyDescription, bool) {
	return &spidpd, true
}

// AsServicePlacementNonPartiallyPlaceServicePolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementInvalidDomainPolicyDescription.
func (spidpd ServicePlacementInvalidDomainPolicyDescription) AsServicePlacementNonPartiallyPlaceServicePolicyDescription() (*ServicePlacementNonPartiallyPlaceServicePolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementPreferPrimaryDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementInvalidDomainPolicyDescription.
func (spidpd ServicePlacementInvalidDomainPolicyDescription) AsServicePlacementPreferPrimaryDomainPolicyDescription() (*ServicePlacementPreferPrimaryDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementRequiredDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementInvalidDomainPolicyDescription.
func (spidpd ServicePlacementInvalidDomainPolicyDescription) AsServicePlacementRequiredDomainPolicyDescription() (*ServicePlacementRequiredDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementRequireDomainDistributionPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementInvalidDomainPolicyDescription.
func (spidpd ServicePlacementInvalidDomainPolicyDescription) AsServicePlacementRequireDomainDistributionPolicyDescription() (*ServicePlacementRequireDomainDistributionPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementInvalidDomainPolicyDescription.
func (spidpd ServicePlacementInvalidDomainPolicyDescription) AsServicePlacementPolicyDescription() (*ServicePlacementPolicyDescription, bool) {
	return nil, false
}

// AsBasicServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementInvalidDomainPolicyDescription.
func (spidpd ServicePlacementInvalidDomainPolicyDescription) AsBasicServicePlacementPolicyDescription() (BasicServicePlacementPolicyDescription, bool) {
	return &spidpd, true
}

// ServicePlacementNonPartiallyPlaceServicePolicyDescription describes the policy to be used for placement
// of a Service Fabric service where all replicas must be able to be placed in order for any replicas to be
// created.
type ServicePlacementNonPartiallyPlaceServicePolicyDescription struct {
	// Type - Possible values include: 'TypeServicePlacementPolicyDescription', 'TypeInvalidDomain', 'TypeNonPartiallyPlaceService', 'TypePreferPrimaryDomain', 'TypeRequireDomain', 'TypeRequireDomainDistribution'
	Type Type `json:"Type,omitempty"`
}

// MarshalJSON is the custom marshaler for ServicePlacementNonPartiallyPlaceServicePolicyDescription.
func (spnppspd ServicePlacementNonPartiallyPlaceServicePolicyDescription) MarshalJSON() ([]byte, error) {
	spnppspd.Type = TypeNonPartiallyPlaceService
	objectMap := make(map[string]interface{})
	if spnppspd.Type != "" {
		objectMap["Type"] = spnppspd.Type
	}
	return json.Marshal(objectMap)
}

// AsServicePlacementInvalidDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementNonPartiallyPlaceServicePolicyDescription.
func (spnppspd ServicePlacementNonPartiallyPlaceServicePolicyDescription) AsServicePlacementInvalidDomainPolicyDescription() (*ServicePlacementInvalidDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementNonPartiallyPlaceServicePolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementNonPartiallyPlaceServicePolicyDescription.
func (spnppspd ServicePlacementNonPartiallyPlaceServicePolicyDescription) AsServicePlacementNonPartiallyPlaceServicePolicyDescription() (*ServicePlacementNonPartiallyPlaceServicePolicyDescription, bool) {
	return &spnppspd, true
}

// AsServicePlacementPreferPrimaryDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementNonPartiallyPlaceServicePolicyDescription.
func (spnppspd ServicePlacementNonPartiallyPlaceServicePolicyDescription) AsServicePlacementPreferPrimaryDomainPolicyDescription() (*ServicePlacementPreferPrimaryDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementRequiredDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementNonPartiallyPlaceServicePolicyDescription.
func (spnppspd ServicePlacementNonPartiallyPlaceServicePolicyDescription) AsServicePlacementRequiredDomainPolicyDescription() (*ServicePlacementRequiredDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementRequireDomainDistributionPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementNonPartiallyPlaceServicePolicyDescription.
func (spnppspd ServicePlacementNonPartiallyPlaceServicePolicyDescription) AsServicePlacementRequireDomainDistributionPolicyDescription() (*ServicePlacementRequireDomainDistributionPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementNonPartiallyPlaceServicePolicyDescription.
func (spnppspd ServicePlacementNonPartiallyPlaceServicePolicyDescription) AsServicePlacementPolicyDescription() (*ServicePlacementPolicyDescription, bool) {
	return nil, false
}

// AsBasicServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementNonPartiallyPlaceServicePolicyDescription.
func (spnppspd ServicePlacementNonPartiallyPlaceServicePolicyDescription) AsBasicServicePlacementPolicyDescription() (BasicServicePlacementPolicyDescription, bool) {
	return &spnppspd, true
}

// BasicServicePlacementPolicyDescription describes the policy to be used for placement of a Service Fabric service.
type BasicServicePlacementPolicyDescription interface {
	AsServicePlacementInvalidDomainPolicyDescription() (*ServicePlacementInvalidDomainPolicyDescription, bool)
	AsServicePlacementNonPartiallyPlaceServicePolicyDescription() (*ServicePlacementNonPartiallyPlaceServicePolicyDescription, bool)
	AsServicePlacementPreferPrimaryDomainPolicyDescription() (*ServicePlacementPreferPrimaryDomainPolicyDescription, bool)
	AsServicePlacementRequiredDomainPolicyDescription() (*ServicePlacementRequiredDomainPolicyDescription, bool)
	AsServicePlacementRequireDomainDistributionPolicyDescription() (*ServicePlacementRequireDomainDistributionPolicyDescription, bool)
	AsServicePlacementPolicyDescription() (*ServicePlacementPolicyDescription, bool)
}

// ServicePlacementPolicyDescription describes the policy to be used for placement of a Service Fabric service.
type ServicePlacementPolicyDescription struct {
	// Type - Possible values include: 'TypeServicePlacementPolicyDescription', 'TypeInvalidDomain', 'TypeNonPartiallyPlaceService', 'TypePreferPrimaryDomain', 'TypeRequireDomain', 'TypeRequireDomainDistribution'
	Type Type `json:"Type,omitempty"`
}

func unmarshalBasicServicePlacementPolicyDescription(body []byte) (BasicServicePlacementPolicyDescription, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Type"] {
	case string(TypeInvalidDomain):
		var spidpd ServicePlacementInvalidDomainPolicyDescription
		err := json.Unmarshal(body, &spidpd)
		return spidpd, err
	case string(TypeNonPartiallyPlaceService):
		var spnppspd ServicePlacementNonPartiallyPlaceServicePolicyDescription
		err := json.Unmarshal(body, &spnppspd)
		return spnppspd, err
	case string(TypePreferPrimaryDomain):
		var spppdpd ServicePlacementPreferPrimaryDomainPolicyDescription
		err := json.Unmarshal(body, &spppdpd)
		return spppdpd, err
	case string(TypeRequireDomain):
		var sprdpd ServicePlacementRequiredDomainPolicyDescription
		err := json.Unmarshal(body, &sprdpd)
		return sprdpd, err
	case string(TypeRequireDomainDistribution):
		var sprddpd ServicePlacementRequireDomainDistributionPolicyDescription
		err := json.Unmarshal(body, &sprddpd)
		return sprddpd, err
	default:
		var sppd ServicePlacementPolicyDescription
		err := json.Unmarshal(body, &sppd)
		return sppd, err
	}
}
func unmarshalBasicServicePlacementPolicyDescriptionArray(body []byte) ([]BasicServicePlacementPolicyDescription, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	sppdArray := make([]BasicServicePlacementPolicyDescription, len(rawMessages))

	for index, rawMessage := range rawMessages {
		sppd, err := unmarshalBasicServicePlacementPolicyDescription(*rawMessage)
		if err != nil {
			return nil, err
		}
		sppdArray[index] = sppd
	}
	return sppdArray, nil
}

// MarshalJSON is the custom marshaler for ServicePlacementPolicyDescription.
func (sppd ServicePlacementPolicyDescription) MarshalJSON() ([]byte, error) {
	sppd.Type = TypeServicePlacementPolicyDescription
	objectMap := make(map[string]interface{})
	if sppd.Type != "" {
		objectMap["Type"] = sppd.Type
	}
	return json.Marshal(objectMap)
}

// AsServicePlacementInvalidDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPolicyDescription.
func (sppd ServicePlacementPolicyDescription) AsServicePlacementInvalidDomainPolicyDescription() (*ServicePlacementInvalidDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementNonPartiallyPlaceServicePolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPolicyDescription.
func (sppd ServicePlacementPolicyDescription) AsServicePlacementNonPartiallyPlaceServicePolicyDescription() (*ServicePlacementNonPartiallyPlaceServicePolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementPreferPrimaryDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPolicyDescription.
func (sppd ServicePlacementPolicyDescription) AsServicePlacementPreferPrimaryDomainPolicyDescription() (*ServicePlacementPreferPrimaryDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementRequiredDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPolicyDescription.
func (sppd ServicePlacementPolicyDescription) AsServicePlacementRequiredDomainPolicyDescription() (*ServicePlacementRequiredDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementRequireDomainDistributionPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPolicyDescription.
func (sppd ServicePlacementPolicyDescription) AsServicePlacementRequireDomainDistributionPolicyDescription() (*ServicePlacementRequireDomainDistributionPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPolicyDescription.
func (sppd ServicePlacementPolicyDescription) AsServicePlacementPolicyDescription() (*ServicePlacementPolicyDescription, bool) {
	return &sppd, true
}

// AsBasicServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPolicyDescription.
func (sppd ServicePlacementPolicyDescription) AsBasicServicePlacementPolicyDescription() (BasicServicePlacementPolicyDescription, bool) {
	return &sppd, true
}

// ServicePlacementPreferPrimaryDomainPolicyDescription describes the policy to be used for placement of a
// Service Fabric service where the service's Primary replicas should optimally be placed in a particular
// domain.
//
// This placement policy is usually used with fault domains in scenarios where the Service Fabric cluster
// is geographically distributed in order to indicate that a service's primary replica should be located in
// a particular fault domain, which in geo-distributed scenarios usually aligns with regional or datacenter
// boundaries. Note that since this is an optimization it is possible that the Primary replica may not end
// up located in this domain due to failures, capacity limits, or other constraints.
type ServicePlacementPreferPrimaryDomainPolicyDescription struct {
	// DomainName - The name of the domain that should used for placement as per this policy.
	DomainName *string `json:"DomainName,omitempty"`
	// Type - Possible values include: 'TypeServicePlacementPolicyDescription', 'TypeInvalidDomain', 'TypeNonPartiallyPlaceService', 'TypePreferPrimaryDomain', 'TypeRequireDomain', 'TypeRequireDomainDistribution'
	Type Type `json:"Type,omitempty"`
}

// MarshalJSON is the custom marshaler for ServicePlacementPreferPrimaryDomainPolicyDescription.
func (spppdpd ServicePlacementPreferPrimaryDomainPolicyDescription) MarshalJSON() ([]byte, error) {
	spppdpd.Type = TypePreferPrimaryDomain
	objectMap := make(map[string]interface{})
	if spppdpd.DomainName != nil {
		objectMap["DomainName"] = spppdpd.DomainName
	}
	if spppdpd.Type != "" {
		objectMap["Type"] = spppdpd.Type
	}
	return json.Marshal(objectMap)
}

// AsServicePlacementInvalidDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPreferPrimaryDomainPolicyDescription.
func (spppdpd ServicePlacementPreferPrimaryDomainPolicyDescription) AsServicePlacementInvalidDomainPolicyDescription() (*ServicePlacementInvalidDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementNonPartiallyPlaceServicePolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPreferPrimaryDomainPolicyDescription.
func (spppdpd ServicePlacementPreferPrimaryDomainPolicyDescription) AsServicePlacementNonPartiallyPlaceServicePolicyDescription() (*ServicePlacementNonPartiallyPlaceServicePolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementPreferPrimaryDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPreferPrimaryDomainPolicyDescription.
func (spppdpd ServicePlacementPreferPrimaryDomainPolicyDescription) AsServicePlacementPreferPrimaryDomainPolicyDescription() (*ServicePlacementPreferPrimaryDomainPolicyDescription, bool) {
	return &spppdpd, true
}

// AsServicePlacementRequiredDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPreferPrimaryDomainPolicyDescription.
func (spppdpd ServicePlacementPreferPrimaryDomainPolicyDescription) AsServicePlacementRequiredDomainPolicyDescription() (*ServicePlacementRequiredDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementRequireDomainDistributionPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPreferPrimaryDomainPolicyDescription.
func (spppdpd ServicePlacementPreferPrimaryDomainPolicyDescription) AsServicePlacementRequireDomainDistributionPolicyDescription() (*ServicePlacementRequireDomainDistributionPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPreferPrimaryDomainPolicyDescription.
func (spppdpd ServicePlacementPreferPrimaryDomainPolicyDescription) AsServicePlacementPolicyDescription() (*ServicePlacementPolicyDescription, bool) {
	return nil, false
}

// AsBasicServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementPreferPrimaryDomainPolicyDescription.
func (spppdpd ServicePlacementPreferPrimaryDomainPolicyDescription) AsBasicServicePlacementPolicyDescription() (BasicServicePlacementPolicyDescription, bool) {
	return &spppdpd, true
}

// ServicePlacementRequiredDomainPolicyDescription describes the policy to be used for placement of a
// Service Fabric service where the instances or replicas of that service must be placed in a particular
// domain
type ServicePlacementRequiredDomainPolicyDescription struct {
	// DomainName - The name of the domain that should used for placement as per this policy.
	DomainName *string `json:"DomainName,omitempty"`
	// Type - Possible values include: 'TypeServicePlacementPolicyDescription', 'TypeInvalidDomain', 'TypeNonPartiallyPlaceService', 'TypePreferPrimaryDomain', 'TypeRequireDomain', 'TypeRequireDomainDistribution'
	Type Type `json:"Type,omitempty"`
}

// MarshalJSON is the custom marshaler for ServicePlacementRequiredDomainPolicyDescription.
func (sprdpd ServicePlacementRequiredDomainPolicyDescription) MarshalJSON() ([]byte, error) {
	sprdpd.Type = TypeRequireDomain
	objectMap := make(map[string]interface{})
	if sprdpd.DomainName != nil {
		objectMap["DomainName"] = sprdpd.DomainName
	}
	if sprdpd.Type != "" {
		objectMap["Type"] = sprdpd.Type
	}
	return json.Marshal(objectMap)
}

// AsServicePlacementInvalidDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequiredDomainPolicyDescription.
func (sprdpd ServicePlacementRequiredDomainPolicyDescription) AsServicePlacementInvalidDomainPolicyDescription() (*ServicePlacementInvalidDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementNonPartiallyPlaceServicePolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequiredDomainPolicyDescription.
func (sprdpd ServicePlacementRequiredDomainPolicyDescription) AsServicePlacementNonPartiallyPlaceServicePolicyDescription() (*ServicePlacementNonPartiallyPlaceServicePolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementPreferPrimaryDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequiredDomainPolicyDescription.
func (sprdpd ServicePlacementRequiredDomainPolicyDescription) AsServicePlacementPreferPrimaryDomainPolicyDescription() (*ServicePlacementPreferPrimaryDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementRequiredDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequiredDomainPolicyDescription.
func (sprdpd ServicePlacementRequiredDomainPolicyDescription) AsServicePlacementRequiredDomainPolicyDescription() (*ServicePlacementRequiredDomainPolicyDescription, bool) {
	return &sprdpd, true
}

// AsServicePlacementRequireDomainDistributionPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequiredDomainPolicyDescription.
func (sprdpd ServicePlacementRequiredDomainPolicyDescription) AsServicePlacementRequireDomainDistributionPolicyDescription() (*ServicePlacementRequireDomainDistributionPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequiredDomainPolicyDescription.
func (sprdpd ServicePlacementRequiredDomainPolicyDescription) AsServicePlacementPolicyDescription() (*ServicePlacementPolicyDescription, bool) {
	return nil, false
}

// AsBasicServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequiredDomainPolicyDescription.
func (sprdpd ServicePlacementRequiredDomainPolicyDescription) AsBasicServicePlacementPolicyDescription() (BasicServicePlacementPolicyDescription, bool) {
	return &sprdpd, true
}

// ServicePlacementRequireDomainDistributionPolicyDescription describes the policy to be used for placement
// of a Service Fabric service where two replicas from the same partition should never be placed in the
// same fault or upgrade domain.
//
// While this is not common it can expose the service to an increased risk of concurrent failures due to
// unplanned outages or other cases of subsequent/concurrent failures. As an example, consider a case where
// replicas are deployed across different data center, with one replica per location. In the event that one
// of the datacenters goes offline, normally the replica that was placed in that datacenter will be packed
// into one of the remaining datacenters. If this is not desirable then this policy should be set.
type ServicePlacementRequireDomainDistributionPolicyDescription struct {
	// DomainName - The name of the domain that should used for placement as per this policy.
	DomainName *string `json:"DomainName,omitempty"`
	// Type - Possible values include: 'TypeServicePlacementPolicyDescription', 'TypeInvalidDomain', 'TypeNonPartiallyPlaceService', 'TypePreferPrimaryDomain', 'TypeRequireDomain', 'TypeRequireDomainDistribution'
	Type Type `json:"Type,omitempty"`
}

// MarshalJSON is the custom marshaler for ServicePlacementRequireDomainDistributionPolicyDescription.
func (sprddpd ServicePlacementRequireDomainDistributionPolicyDescription) MarshalJSON() ([]byte, error) {
	sprddpd.Type = TypeRequireDomainDistribution
	objectMap := make(map[string]interface{})
	if sprddpd.DomainName != nil {
		objectMap["DomainName"] = sprddpd.DomainName
	}
	if sprddpd.Type != "" {
		objectMap["Type"] = sprddpd.Type
	}
	return json.Marshal(objectMap)
}

// AsServicePlacementInvalidDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequireDomainDistributionPolicyDescription.
func (sprddpd ServicePlacementRequireDomainDistributionPolicyDescription) AsServicePlacementInvalidDomainPolicyDescription() (*ServicePlacementInvalidDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementNonPartiallyPlaceServicePolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequireDomainDistributionPolicyDescription.
func (sprddpd ServicePlacementRequireDomainDistributionPolicyDescription) AsServicePlacementNonPartiallyPlaceServicePolicyDescription() (*ServicePlacementNonPartiallyPlaceServicePolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementPreferPrimaryDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequireDomainDistributionPolicyDescription.
func (sprddpd ServicePlacementRequireDomainDistributionPolicyDescription) AsServicePlacementPreferPrimaryDomainPolicyDescription() (*ServicePlacementPreferPrimaryDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementRequiredDomainPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequireDomainDistributionPolicyDescription.
func (sprddpd ServicePlacementRequireDomainDistributionPolicyDescription) AsServicePlacementRequiredDomainPolicyDescription() (*ServicePlacementRequiredDomainPolicyDescription, bool) {
	return nil, false
}

// AsServicePlacementRequireDomainDistributionPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequireDomainDistributionPolicyDescription.
func (sprddpd ServicePlacementRequireDomainDistributionPolicyDescription) AsServicePlacementRequireDomainDistributionPolicyDescription() (*ServicePlacementRequireDomainDistributionPolicyDescription, bool) {
	return &sprddpd, true
}

// AsServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequireDomainDistributionPolicyDescription.
func (sprddpd ServicePlacementRequireDomainDistributionPolicyDescription) AsServicePlacementPolicyDescription() (*ServicePlacementPolicyDescription, bool) {
	return nil, false
}

// AsBasicServicePlacementPolicyDescription is the BasicServicePlacementPolicyDescription implementation for ServicePlacementRequireDomainDistributionPolicyDescription.
func (sprddpd ServicePlacementRequireDomainDistributionPolicyDescription) AsBasicServicePlacementPolicyDescription() (BasicServicePlacementPolicyDescription, bool) {
	return &sprddpd, true
}

// ServiceProperties describes properties of a service resource.
type ServiceProperties struct {
	// Description - User readable description of the service.
	Description *string `json:"description,omitempty"`
	// ReplicaCount - The number of replicas of the service to create. Defaults to 1 if not specified.
	ReplicaCount *int32 `json:"replicaCount,omitempty"`
	// AutoScalingPolicies - Auto scaling policies
	AutoScalingPolicies *[]AutoScalingPolicy `json:"autoScalingPolicies,omitempty"`
	// Status - READ-ONLY; Status of the service. Possible values include: 'ResourceStatusUnknown', 'ResourceStatusReady', 'ResourceStatusUpgrading', 'ResourceStatusCreating', 'ResourceStatusDeleting', 'ResourceStatusFailed'
	Status ResourceStatus `json:"status,omitempty"`
	// StatusDetails - READ-ONLY; Gives additional information about the current status of the service.
	StatusDetails *string `json:"statusDetails,omitempty"`
	// HealthState - READ-ONLY; Describes the health state of an application resource. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"healthState,omitempty"`
	// UnhealthyEvaluation - READ-ONLY; When the service's health state is not 'Ok', this additional details from service fabric Health Manager for the user to know why the service is marked unhealthy.
	UnhealthyEvaluation *string `json:"unhealthyEvaluation,omitempty"`
	// IdentityRefs - The service identity list.
	IdentityRefs *[]ServiceIdentity `json:"identityRefs,omitempty"`
}

// MarshalJSON is the custom marshaler for ServiceProperties.
func (sp ServiceProperties) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if sp.Description != nil {
		objectMap["description"] = sp.Description
	}
	if sp.ReplicaCount != nil {
		objectMap["replicaCount"] = sp.ReplicaCount
	}
	if sp.AutoScalingPolicies != nil {
		objectMap["autoScalingPolicies"] = sp.AutoScalingPolicies
	}
	if sp.IdentityRefs != nil {
		objectMap["identityRefs"] = sp.IdentityRefs
	}
	return json.Marshal(objectMap)
}

// ServiceReplicaDescription describes a replica of a service resource.
type ServiceReplicaDescription struct {
	autorest.Response `json:"-"`
	// ReplicaName - Name of the replica.
	ReplicaName *string `json:"replicaName,omitempty"`
	// OsType - The operation system required by the code in service. Possible values include: 'Linux', 'Windows'
	OsType OperatingSystemType `json:"osType,omitempty"`
	// CodePackages - Describes the set of code packages that forms the service. A code package describes the container and the properties for running it. All the code packages are started together on the same host and share the same context (network, process etc.).
	CodePackages *[]ContainerCodePackageProperties `json:"codePackages,omitempty"`
	// NetworkRefs - The names of the private networks that this service needs to be part of.
	NetworkRefs *[]NetworkRef `json:"networkRefs,omitempty"`
	// Diagnostics - Reference to sinks in DiagnosticsDescription.
	Diagnostics *DiagnosticsRef `json:"diagnostics,omitempty"`
}

// ServiceReplicaProperties describes the properties of a service replica.
type ServiceReplicaProperties struct {
	// OsType - The operation system required by the code in service. Possible values include: 'Linux', 'Windows'
	OsType OperatingSystemType `json:"osType,omitempty"`
	// CodePackages - Describes the set of code packages that forms the service. A code package describes the container and the properties for running it. All the code packages are started together on the same host and share the same context (network, process etc.).
	CodePackages *[]ContainerCodePackageProperties `json:"codePackages,omitempty"`
	// NetworkRefs - The names of the private networks that this service needs to be part of.
	NetworkRefs *[]NetworkRef `json:"networkRefs,omitempty"`
	// Diagnostics - Reference to sinks in DiagnosticsDescription.
	Diagnostics *DiagnosticsRef `json:"diagnostics,omitempty"`
}

// ServiceResourceDescription this type describes a service resource.
type ServiceResourceDescription struct {
	autorest.Response `json:"-"`
	// Name - Name of the Service resource.
	Name *string `json:"name,omitempty"`
	// ServiceResourceProperties - This type describes properties of a service resource.
	*ServiceResourceProperties `json:"properties,omitempty"`
}

// MarshalJSON is the custom marshaler for ServiceResourceDescription.
func (srd ServiceResourceDescription) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if srd.Name != nil {
		objectMap["name"] = srd.Name
	}
	if srd.ServiceResourceProperties != nil {
		objectMap["properties"] = srd.ServiceResourceProperties
	}
	return json.Marshal(objectMap)
}

// UnmarshalJSON is the custom unmarshaler for ServiceResourceDescription struct.
func (srd *ServiceResourceDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				srd.Name = &name
			}
		case "properties":
			if v != nil {
				var serviceResourceProperties ServiceResourceProperties
				err = json.Unmarshal(*v, &serviceResourceProperties)
				if err != nil {
					return err
				}
				srd.ServiceResourceProperties = &serviceResourceProperties
			}
		}
	}

	return nil
}

// ServiceResourceProperties this type describes properties of a service resource.
type ServiceResourceProperties struct {
	// OsType - The operation system required by the code in service. Possible values include: 'Linux', 'Windows'
	OsType OperatingSystemType `json:"osType,omitempty"`
	// CodePackages - Describes the set of code packages that forms the service. A code package describes the container and the properties for running it. All the code packages are started together on the same host and share the same context (network, process etc.).
	CodePackages *[]ContainerCodePackageProperties `json:"codePackages,omitempty"`
	// NetworkRefs - The names of the private networks that this service needs to be part of.
	NetworkRefs *[]NetworkRef `json:"networkRefs,omitempty"`
	// Diagnostics - Reference to sinks in DiagnosticsDescription.
	Diagnostics *DiagnosticsRef `json:"diagnostics,omitempty"`
	// Description - User readable description of the service.
	Description *string `json:"description,omitempty"`
	// ReplicaCount - The number of replicas of the service to create. Defaults to 1 if not specified.
	ReplicaCount *int32 `json:"replicaCount,omitempty"`
	// AutoScalingPolicies - Auto scaling policies
	AutoScalingPolicies *[]AutoScalingPolicy `json:"autoScalingPolicies,omitempty"`
	// Status - READ-ONLY; Status of the service. Possible values include: 'ResourceStatusUnknown', 'ResourceStatusReady', 'ResourceStatusUpgrading', 'ResourceStatusCreating', 'ResourceStatusDeleting', 'ResourceStatusFailed'
	Status ResourceStatus `json:"status,omitempty"`
	// StatusDetails - READ-ONLY; Gives additional information about the current status of the service.
	StatusDetails *string `json:"statusDetails,omitempty"`
	// HealthState - READ-ONLY; Describes the health state of an application resource. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"healthState,omitempty"`
	// UnhealthyEvaluation - READ-ONLY; When the service's health state is not 'Ok', this additional details from service fabric Health Manager for the user to know why the service is marked unhealthy.
	UnhealthyEvaluation *string `json:"unhealthyEvaluation,omitempty"`
	// IdentityRefs - The service identity list.
	IdentityRefs *[]ServiceIdentity `json:"identityRefs,omitempty"`
}

// MarshalJSON is the custom marshaler for ServiceResourceProperties.
func (srp ServiceResourceProperties) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if srp.OsType != "" {
		objectMap["osType"] = srp.OsType
	}
	if srp.CodePackages != nil {
		objectMap["codePackages"] = srp.CodePackages
	}
	if srp.NetworkRefs != nil {
		objectMap["networkRefs"] = srp.NetworkRefs
	}
	if srp.Diagnostics != nil {
		objectMap["diagnostics"] = srp.Diagnostics
	}
	if srp.Description != nil {
		objectMap["description"] = srp.Description
	}
	if srp.ReplicaCount != nil {
		objectMap["replicaCount"] = srp.ReplicaCount
	}
	if srp.AutoScalingPolicies != nil {
		objectMap["autoScalingPolicies"] = srp.AutoScalingPolicies
	}
	if srp.IdentityRefs != nil {
		objectMap["identityRefs"] = srp.IdentityRefs
	}
	return json.Marshal(objectMap)
}

// ServicesHealthEvaluation represents health evaluation for services of a certain service type belonging
// to an application, containing health evaluations for each unhealthy service that impacted current
// aggregated health state. Can be returned when evaluating application health and the aggregated health
// state is either Error or Warning.
type ServicesHealthEvaluation struct {
	// ServiceTypeName - Name of the service type of the services.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// MaxPercentUnhealthyServices - Maximum allowed percentage of unhealthy services from the ServiceTypeHealthPolicy.
	MaxPercentUnhealthyServices *int32 `json:"MaxPercentUnhealthyServices,omitempty"`
	// TotalCount - Total number of services of the current service type in the application from the health store.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state. Includes all the unhealthy ServiceHealthEvaluation that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) MarshalJSON() ([]byte, error) {
	she.Kind = KindServices
	objectMap := make(map[string]interface{})
	if she.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = she.ServiceTypeName
	}
	if she.MaxPercentUnhealthyServices != nil {
		objectMap["MaxPercentUnhealthyServices"] = she.MaxPercentUnhealthyServices
	}
	if she.TotalCount != nil {
		objectMap["TotalCount"] = she.TotalCount
	}
	if she.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = she.UnhealthyEvaluations
	}
	if she.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = she.AggregatedHealthState
	}
	if she.Description != nil {
		objectMap["Description"] = she.Description
	}
	if she.Kind != "" {
		objectMap["Kind"] = she.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return &she, true
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for ServicesHealthEvaluation.
func (she ServicesHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &she, true
}

// BasicServiceTypeDescription describes a service type defined in the service manifest of a provisioned application
// type. The properties the ones defined in the service manifest.
type BasicServiceTypeDescription interface {
	AsStatefulServiceTypeDescription() (*StatefulServiceTypeDescription, bool)
	AsStatelessServiceTypeDescription() (*StatelessServiceTypeDescription, bool)
	AsServiceTypeDescription() (*ServiceTypeDescription, bool)
}

// ServiceTypeDescription describes a service type defined in the service manifest of a provisioned application
// type. The properties the ones defined in the service manifest.
type ServiceTypeDescription struct {
	// IsStateful - Indicates whether the service type is a stateful service type or a stateless service type. This property is true if the service type is a stateful service type, false otherwise.
	IsStateful *bool `json:"IsStateful,omitempty"`
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// PlacementConstraints - The placement constraint to be used when instantiating this service in a Service Fabric cluster.
	PlacementConstraints *string `json:"PlacementConstraints,omitempty"`
	// LoadMetrics - The service load metrics is given as an array of ServiceLoadMetricDescription objects.
	LoadMetrics *[]ServiceLoadMetricDescription `json:"LoadMetrics,omitempty"`
	// ServicePlacementPolicies - List of service placement policy descriptions.
	ServicePlacementPolicies *[]BasicServicePlacementPolicyDescription `json:"ServicePlacementPolicies,omitempty"`
	// Extensions - List of service type extensions.
	Extensions *[]ServiceTypeExtensionDescription `json:"Extensions,omitempty"`
	// Kind - Possible values include: 'KindServiceTypeDescription', 'KindStateful', 'KindStateless'
	Kind KindBasicServiceTypeDescription `json:"Kind,omitempty"`
}

func unmarshalBasicServiceTypeDescription(body []byte) (BasicServiceTypeDescription, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["Kind"] {
	case string(KindStateful):
		var sstd StatefulServiceTypeDescription
		err := json.Unmarshal(body, &sstd)
		return sstd, err
	case string(KindStateless):
		var sstd StatelessServiceTypeDescription
		err := json.Unmarshal(body, &sstd)
		return sstd, err
	default:
		var std ServiceTypeDescription
		err := json.Unmarshal(body, &std)
		return std, err
	}
}
func unmarshalBasicServiceTypeDescriptionArray(body []byte) ([]BasicServiceTypeDescription, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	stdArray := make([]BasicServiceTypeDescription, len(rawMessages))

	for index, rawMessage := range rawMessages {
		std, err := unmarshalBasicServiceTypeDescription(*rawMessage)
		if err != nil {
			return nil, err
		}
		stdArray[index] = std
	}
	return stdArray, nil
}

// MarshalJSON is the custom marshaler for ServiceTypeDescription.
func (std ServiceTypeDescription) MarshalJSON() ([]byte, error) {
	std.Kind = KindServiceTypeDescription
	objectMap := make(map[string]interface{})
	if std.IsStateful != nil {
		objectMap["IsStateful"] = std.IsStateful
	}
	if std.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = std.ServiceTypeName
	}
	if std.PlacementConstraints != nil {
		objectMap["PlacementConstraints"] = std.PlacementConstraints
	}
	if std.LoadMetrics != nil {
		objectMap["LoadMetrics"] = std.LoadMetrics
	}
	if std.ServicePlacementPolicies != nil {
		objectMap["ServicePlacementPolicies"] = std.ServicePlacementPolicies
	}
	if std.Extensions != nil {
		objectMap["Extensions"] = std.Extensions
	}
	if std.Kind != "" {
		objectMap["Kind"] = std.Kind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceTypeDescription is the BasicServiceTypeDescription implementation for ServiceTypeDescription.
func (std ServiceTypeDescription) AsStatefulServiceTypeDescription() (*StatefulServiceTypeDescription, bool) {
	return nil, false
}

// AsStatelessServiceTypeDescription is the BasicServiceTypeDescription implementation for ServiceTypeDescription.
func (std ServiceTypeDescription) AsStatelessServiceTypeDescription() (*StatelessServiceTypeDescription, bool) {
	return nil, false
}

// AsServiceTypeDescription is the BasicServiceTypeDescription implementation for ServiceTypeDescription.
func (std ServiceTypeDescription) AsServiceTypeDescription() (*ServiceTypeDescription, bool) {
	return &std, true
}

// AsBasicServiceTypeDescription is the BasicServiceTypeDescription implementation for ServiceTypeDescription.
func (std ServiceTypeDescription) AsBasicServiceTypeDescription() (BasicServiceTypeDescription, bool) {
	return &std, true
}

// UnmarshalJSON is the custom unmarshaler for ServiceTypeDescription struct.
func (std *ServiceTypeDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "IsStateful":
			if v != nil {
				var isStateful bool
				err = json.Unmarshal(*v, &isStateful)
				if err != nil {
					return err
				}
				std.IsStateful = &isStateful
			}
		case "ServiceTypeName":
			if v != nil {
				var serviceTypeName string
				err = json.Unmarshal(*v, &serviceTypeName)
				if err != nil {
					return err
				}
				std.ServiceTypeName = &serviceTypeName
			}
		case "PlacementConstraints":
			if v != nil {
				var placementConstraints string
				err = json.Unmarshal(*v, &placementConstraints)
				if err != nil {
					return err
				}
				std.PlacementConstraints = &placementConstraints
			}
		case "LoadMetrics":
			if v != nil {
				var loadMetrics []ServiceLoadMetricDescription
				err = json.Unmarshal(*v, &loadMetrics)
				if err != nil {
					return err
				}
				std.LoadMetrics = &loadMetrics
			}
		case "ServicePlacementPolicies":
			if v != nil {
				servicePlacementPolicies, err := unmarshalBasicServicePlacementPolicyDescriptionArray(*v)
				if err != nil {
					return err
				}
				std.ServicePlacementPolicies = &servicePlacementPolicies
			}
		case "Extensions":
			if v != nil {
				var extensions []ServiceTypeExtensionDescription
				err = json.Unmarshal(*v, &extensions)
				if err != nil {
					return err
				}
				std.Extensions = &extensions
			}
		case "Kind":
			if v != nil {
				var kind KindBasicServiceTypeDescription
				err = json.Unmarshal(*v, &kind)
				if err != nil {
					return err
				}
				std.Kind = kind
			}
		}
	}

	return nil
}

// ServiceTypeExtensionDescription describes extension of a service type defined in the service manifest.
type ServiceTypeExtensionDescription struct {
	// Key - The name of the extension.
	Key *string `json:"Key,omitempty"`
	// Value - The extension value.
	Value *string `json:"Value,omitempty"`
}

// ServiceTypeHealthPolicy represents the health policy used to evaluate the health of services belonging
// to a service type.
type ServiceTypeHealthPolicy struct {
	// MaxPercentUnhealthyPartitionsPerService - The maximum allowed percentage of unhealthy partitions per service. Allowed values are Byte values from zero to 100
	// The percentage represents the maximum tolerated percentage of partitions that can be unhealthy before the service is considered in error.
	// If the percentage is respected but there is at least one unhealthy partition, the health is evaluated as Warning.
	// The percentage is calculated by dividing the number of unhealthy partitions over the total number of partitions in the service.
	// The computation rounds up to tolerate one failure on small numbers of partitions. Default percentage is zero.
	MaxPercentUnhealthyPartitionsPerService *int32 `json:"MaxPercentUnhealthyPartitionsPerService,omitempty"`
	// MaxPercentUnhealthyReplicasPerPartition - The maximum allowed percentage of unhealthy replicas per partition. Allowed values are Byte values from zero to 100.
	// The percentage represents the maximum tolerated percentage of replicas that can be unhealthy before the partition is considered in error.
	// If the percentage is respected but there is at least one unhealthy replica, the health is evaluated as Warning.
	// The percentage is calculated by dividing the number of unhealthy replicas over the total number of replicas in the partition.
	// The computation rounds up to tolerate one failure on small numbers of replicas. Default percentage is zero.
	MaxPercentUnhealthyReplicasPerPartition *int32 `json:"MaxPercentUnhealthyReplicasPerPartition,omitempty"`
	// MaxPercentUnhealthyServices - The maximum allowed percentage of unhealthy services. Allowed values are Byte values from zero to 100.
	// The percentage represents the maximum tolerated percentage of services that can be unhealthy before the application is considered in error.
	// If the percentage is respected but there is at least one unhealthy service, the health is evaluated as Warning.
	// This is calculated by dividing the number of unhealthy services of the specific service type over the total number of services of the specific service type.
	// The computation rounds up to tolerate one failure on small numbers of services. Default percentage is zero.
	MaxPercentUnhealthyServices *int32 `json:"MaxPercentUnhealthyServices,omitempty"`
}

// ServiceTypeHealthPolicyMapItem defines an item in ServiceTypeHealthPolicyMap.
type ServiceTypeHealthPolicyMapItem struct {
	// Key - The key of the service type health policy map item. This is the name of the service type.
	Key *string `json:"Key,omitempty"`
	// Value - The value of the service type health policy map item. This is the ServiceTypeHealthPolicy for this service type.
	Value *ServiceTypeHealthPolicy `json:"Value,omitempty"`
}

// ServiceTypeInfo information about a service type that is defined in a service manifest of a provisioned
// application type.
type ServiceTypeInfo struct {
	autorest.Response `json:"-"`
	// ServiceTypeDescription - Describes a service type defined in the service manifest of a provisioned application type. The properties the ones defined in the service manifest.
	ServiceTypeDescription BasicServiceTypeDescription `json:"ServiceTypeDescription,omitempty"`
	// ServiceManifestName - The name of the service manifest in which this service type is defined.
	ServiceManifestName *string `json:"ServiceManifestName,omitempty"`
	// ServiceManifestVersion - The version of the service manifest in which this service type is defined.
	ServiceManifestVersion *string `json:"ServiceManifestVersion,omitempty"`
	// IsServiceGroup - Indicates whether the service is a service group. If it is, the property value is true otherwise false.
	IsServiceGroup *bool `json:"IsServiceGroup,omitempty"`
}

// UnmarshalJSON is the custom unmarshaler for ServiceTypeInfo struct.
func (sti *ServiceTypeInfo) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "ServiceTypeDescription":
			if v != nil {
				serviceTypeDescription, err := unmarshalBasicServiceTypeDescription(*v)
				if err != nil {
					return err
				}
				sti.ServiceTypeDescription = serviceTypeDescription
			}
		case "ServiceManifestName":
			if v != nil {
				var serviceManifestName string
				err = json.Unmarshal(*v, &serviceManifestName)
				if err != nil {
					return err
				}
				sti.ServiceManifestName = &serviceManifestName
			}
		case "ServiceManifestVersion":
			if v != nil {
				var serviceManifestVersion string
				err = json.Unmarshal(*v, &serviceManifestVersion)
				if err != nil {
					return err
				}
				sti.ServiceManifestVersion = &serviceManifestVersion
			}
		case "IsServiceGroup":
			if v != nil {
				var isServiceGroup bool
				err = json.Unmarshal(*v, &isServiceGroup)
				if err != nil {
					return err
				}
				sti.IsServiceGroup = &isServiceGroup
			}
		}
	}

	return nil
}

// ServiceTypeManifest contains the manifest describing a service type registered as part of an application
// in a Service Fabric cluster.
type ServiceTypeManifest struct {
	autorest.Response `json:"-"`
	// Manifest - The XML manifest as a string.
	Manifest *string `json:"Manifest,omitempty"`
}

// BasicServiceUpdateDescription a ServiceUpdateDescription contains all of the information necessary to update a
// service.
type BasicServiceUpdateDescription interface {
	AsStatefulServiceUpdateDescription() (*StatefulServiceUpdateDescription, bool)
	AsStatelessServiceUpdateDescription() (*StatelessServiceUpdateDescription, bool)
	AsServiceUpdateDescription() (*ServiceUpdateDescription, bool)
}

// ServiceUpdateDescription a ServiceUpdateDescription contains all of the information necessary to update a
// service.
type ServiceUpdateDescription struct {
	// Flags - Flags indicating whether other properties are set. Each of the associated properties corresponds to a flag, specified below, which, if set, indicate that the property is specified.
	// This property can be a combination of those flags obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6 then the flags for ReplicaRestartWaitDuration (2) and QuorumLossWaitDuration (4) are set.
	// - None - Does not indicate any other properties are set. The value is zero.
	// - TargetReplicaSetSize/InstanceCount - Indicates whether the TargetReplicaSetSize property (for Stateful services) or the InstanceCount property (for Stateless services) is set. The value is 1.
	// - ReplicaRestartWaitDuration - Indicates the ReplicaRestartWaitDuration property is set. The value is  2.
	// - QuorumLossWaitDuration - Indicates the QuorumLossWaitDuration property is set. The value is 4.
	// - StandByReplicaKeepDuration - Indicates the StandByReplicaKeepDuration property is set. The value is 8.
	// - MinReplicaSetSize - Indicates the MinReplicaSetSize property is set. The value is 16.
	// - PlacementConstraints - Indicates the PlacementConstraints property is set. The value is 32.
	// - PlacementPolicyList - Indicates the ServicePlacementPolicies property is set. The value is 64.
	// - Correlation - Indicates the CorrelationScheme property is set. The value is 128.
	// - Metrics - Indicates the ServiceLoadMetrics property is set. The value is 256.
	// - DefaultMoveCost - Indicates the DefaultMoveCost property is set. The value is 512.
	// - ScalingPolicy - Indicates the ScalingPolicies property is set. The value is 1024.
	Flags *string `json:"Flags,omitempty"`
	// PlacementConstraints - The placement constraints as a string. Placement constraints are boolean expressions on node properties and allow for restricting a service to particular nodes based on the service requirements. For example, to place a service on nodes where NodeType is blue specify the following: "NodeColor == blue)".
	PlacementConstraints *string `json:"PlacementConstraints,omitempty"`
	// CorrelationScheme - The correlation scheme.
	CorrelationScheme *[]ServiceCorrelationDescription `json:"CorrelationScheme,omitempty"`
	// LoadMetrics - The service load metrics.
	LoadMetrics *[]ServiceLoadMetricDescription `json:"LoadMetrics,omitempty"`
	// ServicePlacementPolicies - The service placement policies.
	ServicePlacementPolicies *[]BasicServicePlacementPolicyDescription `json:"ServicePlacementPolicies,omitempty"`
	// DefaultMoveCost - The move cost for the service. Possible values include: 'Zero', 'Low', 'Medium', 'High'
	DefaultMoveCost MoveCost `json:"DefaultMoveCost,omitempty"`
	// ScalingPolicies - Scaling policies for this service.
	ScalingPolicies *[]ScalingPolicyDescription `json:"ScalingPolicies,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServiceUpdateDescriptionServiceKindServiceUpdateDescription', 'ServiceKindBasicServiceUpdateDescriptionServiceKindStateful', 'ServiceKindBasicServiceUpdateDescriptionServiceKindStateless'
	ServiceKind ServiceKindBasicServiceUpdateDescription `json:"ServiceKind,omitempty"`
}

func unmarshalBasicServiceUpdateDescription(body []byte) (BasicServiceUpdateDescription, error) {
	var m map[string]interface{}
	err := json.Unmarshal(body, &m)
	if err != nil {
		return nil, err
	}

	switch m["ServiceKind"] {
	case string(ServiceKindBasicServiceUpdateDescriptionServiceKindStateful):
		var ssud StatefulServiceUpdateDescription
		err := json.Unmarshal(body, &ssud)
		return ssud, err
	case string(ServiceKindBasicServiceUpdateDescriptionServiceKindStateless):
		var ssud StatelessServiceUpdateDescription
		err := json.Unmarshal(body, &ssud)
		return ssud, err
	default:
		var sud ServiceUpdateDescription
		err := json.Unmarshal(body, &sud)
		return sud, err
	}
}
func unmarshalBasicServiceUpdateDescriptionArray(body []byte) ([]BasicServiceUpdateDescription, error) {
	var rawMessages []*json.RawMessage
	err := json.Unmarshal(body, &rawMessages)
	if err != nil {
		return nil, err
	}

	sudArray := make([]BasicServiceUpdateDescription, len(rawMessages))

	for index, rawMessage := range rawMessages {
		sud, err := unmarshalBasicServiceUpdateDescription(*rawMessage)
		if err != nil {
			return nil, err
		}
		sudArray[index] = sud
	}
	return sudArray, nil
}

// MarshalJSON is the custom marshaler for ServiceUpdateDescription.
func (sud ServiceUpdateDescription) MarshalJSON() ([]byte, error) {
	sud.ServiceKind = ServiceKindBasicServiceUpdateDescriptionServiceKindServiceUpdateDescription
	objectMap := make(map[string]interface{})
	if sud.Flags != nil {
		objectMap["Flags"] = sud.Flags
	}
	if sud.PlacementConstraints != nil {
		objectMap["PlacementConstraints"] = sud.PlacementConstraints
	}
	if sud.CorrelationScheme != nil {
		objectMap["CorrelationScheme"] = sud.CorrelationScheme
	}
	if sud.LoadMetrics != nil {
		objectMap["LoadMetrics"] = sud.LoadMetrics
	}
	if sud.ServicePlacementPolicies != nil {
		objectMap["ServicePlacementPolicies"] = sud.ServicePlacementPolicies
	}
	if sud.DefaultMoveCost != "" {
		objectMap["DefaultMoveCost"] = sud.DefaultMoveCost
	}
	if sud.ScalingPolicies != nil {
		objectMap["ScalingPolicies"] = sud.ScalingPolicies
	}
	if sud.ServiceKind != "" {
		objectMap["ServiceKind"] = sud.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceUpdateDescription is the BasicServiceUpdateDescription implementation for ServiceUpdateDescription.
func (sud ServiceUpdateDescription) AsStatefulServiceUpdateDescription() (*StatefulServiceUpdateDescription, bool) {
	return nil, false
}

// AsStatelessServiceUpdateDescription is the BasicServiceUpdateDescription implementation for ServiceUpdateDescription.
func (sud ServiceUpdateDescription) AsStatelessServiceUpdateDescription() (*StatelessServiceUpdateDescription, bool) {
	return nil, false
}

// AsServiceUpdateDescription is the BasicServiceUpdateDescription implementation for ServiceUpdateDescription.
func (sud ServiceUpdateDescription) AsServiceUpdateDescription() (*ServiceUpdateDescription, bool) {
	return &sud, true
}

// AsBasicServiceUpdateDescription is the BasicServiceUpdateDescription implementation for ServiceUpdateDescription.
func (sud ServiceUpdateDescription) AsBasicServiceUpdateDescription() (BasicServiceUpdateDescription, bool) {
	return &sud, true
}

// UnmarshalJSON is the custom unmarshaler for ServiceUpdateDescription struct.
func (sud *ServiceUpdateDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "Flags":
			if v != nil {
				var flags string
				err = json.Unmarshal(*v, &flags)
				if err != nil {
					return err
				}
				sud.Flags = &flags
			}
		case "PlacementConstraints":
			if v != nil {
				var placementConstraints string
				err = json.Unmarshal(*v, &placementConstraints)
				if err != nil {
					return err
				}
				sud.PlacementConstraints = &placementConstraints
			}
		case "CorrelationScheme":
			if v != nil {
				var correlationScheme []ServiceCorrelationDescription
				err = json.Unmarshal(*v, &correlationScheme)
				if err != nil {
					return err
				}
				sud.CorrelationScheme = &correlationScheme
			}
		case "LoadMetrics":
			if v != nil {
				var loadMetrics []ServiceLoadMetricDescription
				err = json.Unmarshal(*v, &loadMetrics)
				if err != nil {
					return err
				}
				sud.LoadMetrics = &loadMetrics
			}
		case "ServicePlacementPolicies":
			if v != nil {
				servicePlacementPolicies, err := unmarshalBasicServicePlacementPolicyDescriptionArray(*v)
				if err != nil {
					return err
				}
				sud.ServicePlacementPolicies = &servicePlacementPolicies
			}
		case "DefaultMoveCost":
			if v != nil {
				var defaultMoveCost MoveCost
				err = json.Unmarshal(*v, &defaultMoveCost)
				if err != nil {
					return err
				}
				sud.DefaultMoveCost = defaultMoveCost
			}
		case "ScalingPolicies":
			if v != nil {
				var scalingPolicies []ScalingPolicyDescription
				err = json.Unmarshal(*v, &scalingPolicies)
				if err != nil {
					return err
				}
				sud.ScalingPolicies = &scalingPolicies
			}
		case "ServiceKind":
			if v != nil {
				var serviceKind ServiceKindBasicServiceUpdateDescription
				err = json.Unmarshal(*v, &serviceKind)
				if err != nil {
					return err
				}
				sud.ServiceKind = serviceKind
			}
		}
	}

	return nil
}

// Setting describes a setting for the container. The setting file path can be fetched from environment
// variable "Fabric_SettingPath". The path for Windows container is "C:\\secrets". The path for Linux
// container is "/var/secrets".
type Setting struct {
	// Name - The name of the setting.
	Name *string `json:"name,omitempty"`
	// Value - The value of the setting.
	Value *string `json:"value,omitempty"`
}

// SingletonPartitionInformation information about a partition that is singleton. The services with
// singleton partitioning scheme are effectively non-partitioned. They only have one partition.
type SingletonPartitionInformation struct {
	// ID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	ID *uuid.UUID `json:"Id,omitempty"`
	// ServicePartitionKind - Possible values include: 'ServicePartitionKindPartitionInformation', 'ServicePartitionKindInt64Range1', 'ServicePartitionKindNamed1', 'ServicePartitionKindSingleton1'
	ServicePartitionKind ServicePartitionKindBasicPartitionInformation `json:"ServicePartitionKind,omitempty"`
}

// MarshalJSON is the custom marshaler for SingletonPartitionInformation.
func (spi SingletonPartitionInformation) MarshalJSON() ([]byte, error) {
	spi.ServicePartitionKind = ServicePartitionKindSingleton1
	objectMap := make(map[string]interface{})
	if spi.ID != nil {
		objectMap["Id"] = spi.ID
	}
	if spi.ServicePartitionKind != "" {
		objectMap["ServicePartitionKind"] = spi.ServicePartitionKind
	}
	return json.Marshal(objectMap)
}

// AsInt64RangePartitionInformation is the BasicPartitionInformation implementation for SingletonPartitionInformation.
func (spi SingletonPartitionInformation) AsInt64RangePartitionInformation() (*Int64RangePartitionInformation, bool) {
	return nil, false
}

// AsNamedPartitionInformation is the BasicPartitionInformation implementation for SingletonPartitionInformation.
func (spi SingletonPartitionInformation) AsNamedPartitionInformation() (*NamedPartitionInformation, bool) {
	return nil, false
}

// AsSingletonPartitionInformation is the BasicPartitionInformation implementation for SingletonPartitionInformation.
func (spi SingletonPartitionInformation) AsSingletonPartitionInformation() (*SingletonPartitionInformation, bool) {
	return &spi, true
}

// AsPartitionInformation is the BasicPartitionInformation implementation for SingletonPartitionInformation.
func (spi SingletonPartitionInformation) AsPartitionInformation() (*PartitionInformation, bool) {
	return nil, false
}

// AsBasicPartitionInformation is the BasicPartitionInformation implementation for SingletonPartitionInformation.
func (spi SingletonPartitionInformation) AsBasicPartitionInformation() (BasicPartitionInformation, bool) {
	return &spi, true
}

// SingletonPartitionSchemeDescription describes the partition scheme of a singleton-partitioned, or
// non-partitioned service.
type SingletonPartitionSchemeDescription struct {
	// PartitionScheme - Possible values include: 'PartitionSchemePartitionSchemeDescription', 'PartitionSchemeNamed1', 'PartitionSchemeSingleton1', 'PartitionSchemeUniformInt64Range1'
	PartitionScheme PartitionSchemeBasicPartitionSchemeDescription `json:"PartitionScheme,omitempty"`
}

// MarshalJSON is the custom marshaler for SingletonPartitionSchemeDescription.
func (spsd SingletonPartitionSchemeDescription) MarshalJSON() ([]byte, error) {
	spsd.PartitionScheme = PartitionSchemeSingleton1
	objectMap := make(map[string]interface{})
	if spsd.PartitionScheme != "" {
		objectMap["PartitionScheme"] = spsd.PartitionScheme
	}
	return json.Marshal(objectMap)
}

// AsNamedPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for SingletonPartitionSchemeDescription.
func (spsd SingletonPartitionSchemeDescription) AsNamedPartitionSchemeDescription() (*NamedPartitionSchemeDescription, bool) {
	return nil, false
}

// AsSingletonPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for SingletonPartitionSchemeDescription.
func (spsd SingletonPartitionSchemeDescription) AsSingletonPartitionSchemeDescription() (*SingletonPartitionSchemeDescription, bool) {
	return &spsd, true
}

// AsUniformInt64RangePartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for SingletonPartitionSchemeDescription.
func (spsd SingletonPartitionSchemeDescription) AsUniformInt64RangePartitionSchemeDescription() (*UniformInt64RangePartitionSchemeDescription, bool) {
	return nil, false
}

// AsPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for SingletonPartitionSchemeDescription.
func (spsd SingletonPartitionSchemeDescription) AsPartitionSchemeDescription() (*PartitionSchemeDescription, bool) {
	return nil, false
}

// AsBasicPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for SingletonPartitionSchemeDescription.
func (spsd SingletonPartitionSchemeDescription) AsBasicPartitionSchemeDescription() (BasicPartitionSchemeDescription, bool) {
	return &spsd, true
}

// StartClusterUpgradeDescription describes the parameters for starting a cluster upgrade.
type StartClusterUpgradeDescription struct {
	// CodeVersion - The cluster code version.
	CodeVersion *string `json:"CodeVersion,omitempty"`
	// ConfigVersion - The cluster configuration version.
	ConfigVersion *string `json:"ConfigVersion,omitempty"`
	// UpgradeKind - The kind of upgrade out of the following possible values. Possible values include: 'UpgradeKindInvalid', 'UpgradeKindRolling'
	UpgradeKind UpgradeKind `json:"UpgradeKind,omitempty"`
	// RollingUpgradeMode - The mode used to monitor health during a rolling upgrade. The values are UnmonitoredAuto, UnmonitoredManual, and Monitored. Possible values include: 'UpgradeModeInvalid', 'UpgradeModeUnmonitoredAuto', 'UpgradeModeUnmonitoredManual', 'UpgradeModeMonitored'
	RollingUpgradeMode UpgradeMode `json:"RollingUpgradeMode,omitempty"`
	// UpgradeReplicaSetCheckTimeoutInSeconds - The maximum amount of time to block processing of an upgrade domain and prevent loss of availability when there are unexpected issues. When this timeout expires, processing of the upgrade domain will proceed regardless of availability loss issues. The timeout is reset at the start of each upgrade domain. Valid values are between 0 and 42949672925 inclusive. (unsigned 32-bit integer).
	UpgradeReplicaSetCheckTimeoutInSeconds *int64 `json:"UpgradeReplicaSetCheckTimeoutInSeconds,omitempty"`
	// ForceRestart - If true, then processes are forcefully restarted during upgrade even when the code version has not changed (the upgrade only changes configuration or data).
	ForceRestart *bool `json:"ForceRestart,omitempty"`
	// SortOrder - Defines the order in which an upgrade proceeds through the cluster. Possible values include: 'UpgradeSortOrderInvalid', 'UpgradeSortOrderDefault', 'UpgradeSortOrderNumeric', 'UpgradeSortOrderLexicographical', 'UpgradeSortOrderReverseNumeric', 'UpgradeSortOrderReverseLexicographical'
	SortOrder UpgradeSortOrder `json:"SortOrder,omitempty"`
	// MonitoringPolicy - Describes the parameters for monitoring an upgrade in Monitored mode.
	MonitoringPolicy *MonitoringPolicyDescription `json:"MonitoringPolicy,omitempty"`
	// ClusterHealthPolicy - Defines a health policy used to evaluate the health of the cluster or of a cluster node.
	ClusterHealthPolicy *ClusterHealthPolicy `json:"ClusterHealthPolicy,omitempty"`
	// EnableDeltaHealthEvaluation - When true, enables delta health evaluation rather than absolute health evaluation after completion of each upgrade domain.
	EnableDeltaHealthEvaluation *bool `json:"EnableDeltaHealthEvaluation,omitempty"`
	// ClusterUpgradeHealthPolicy - Defines a health policy used to evaluate the health of the cluster during a cluster upgrade.
	ClusterUpgradeHealthPolicy *ClusterUpgradeHealthPolicyObject `json:"ClusterUpgradeHealthPolicy,omitempty"`
	// ApplicationHealthPolicyMap - Defines the application health policy map used to evaluate the health of an application or one of its children entities.
	ApplicationHealthPolicyMap *ApplicationHealthPolicies `json:"ApplicationHealthPolicyMap,omitempty"`
}

// StartedChaosEvent describes a Chaos event that gets generated when Chaos is started.
type StartedChaosEvent struct {
	// ChaosParameters - Defines all the parameters to configure a Chaos run.
	ChaosParameters *ChaosParameters `json:"ChaosParameters,omitempty"`
	// TimeStampUtc - The UTC timestamp when this Chaos event was generated.
	TimeStampUtc *date.Time `json:"TimeStampUtc,omitempty"`
	// Kind - Possible values include: 'KindChaosEvent', 'KindExecutingFaults', 'KindStarted', 'KindStopped', 'KindTestError', 'KindValidationFailed', 'KindWaiting'
	Kind KindBasicChaosEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for StartedChaosEvent.
func (sce StartedChaosEvent) MarshalJSON() ([]byte, error) {
	sce.Kind = KindStarted
	objectMap := make(map[string]interface{})
	if sce.ChaosParameters != nil {
		objectMap["ChaosParameters"] = sce.ChaosParameters
	}
	if sce.TimeStampUtc != nil {
		objectMap["TimeStampUtc"] = sce.TimeStampUtc
	}
	if sce.Kind != "" {
		objectMap["Kind"] = sce.Kind
	}
	return json.Marshal(objectMap)
}

// AsExecutingFaultsChaosEvent is the BasicChaosEvent implementation for StartedChaosEvent.
func (sce StartedChaosEvent) AsExecutingFaultsChaosEvent() (*ExecutingFaultsChaosEvent, bool) {
	return nil, false
}

// AsStartedChaosEvent is the BasicChaosEvent implementation for StartedChaosEvent.
func (sce StartedChaosEvent) AsStartedChaosEvent() (*StartedChaosEvent, bool) {
	return &sce, true
}

// AsStoppedChaosEvent is the BasicChaosEvent implementation for StartedChaosEvent.
func (sce StartedChaosEvent) AsStoppedChaosEvent() (*StoppedChaosEvent, bool) {
	return nil, false
}

// AsTestErrorChaosEvent is the BasicChaosEvent implementation for StartedChaosEvent.
func (sce StartedChaosEvent) AsTestErrorChaosEvent() (*TestErrorChaosEvent, bool) {
	return nil, false
}

// AsValidationFailedChaosEvent is the BasicChaosEvent implementation for StartedChaosEvent.
func (sce StartedChaosEvent) AsValidationFailedChaosEvent() (*ValidationFailedChaosEvent, bool) {
	return nil, false
}

// AsWaitingChaosEvent is the BasicChaosEvent implementation for StartedChaosEvent.
func (sce StartedChaosEvent) AsWaitingChaosEvent() (*WaitingChaosEvent, bool) {
	return nil, false
}

// AsChaosEvent is the BasicChaosEvent implementation for StartedChaosEvent.
func (sce StartedChaosEvent) AsChaosEvent() (*ChaosEvent, bool) {
	return nil, false
}

// AsBasicChaosEvent is the BasicChaosEvent implementation for StartedChaosEvent.
func (sce StartedChaosEvent) AsBasicChaosEvent() (BasicChaosEvent, bool) {
	return &sce, true
}

// StatefulReplicaHealthReportExpiredEvent stateful Replica Health Report Expired event.
type StatefulReplicaHealthReportExpiredEvent struct {
	// ReplicaInstanceID - Id of Replica instance.
	ReplicaInstanceID *int64 `json:"ReplicaInstanceId,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *int64 `json:"ReplicaId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) MarshalJSON() ([]byte, error) {
	srhree.Kind = KindStatefulReplicaHealthReportExpired
	objectMap := make(map[string]interface{})
	if srhree.ReplicaInstanceID != nil {
		objectMap["ReplicaInstanceId"] = srhree.ReplicaInstanceID
	}
	if srhree.SourceID != nil {
		objectMap["SourceId"] = srhree.SourceID
	}
	if srhree.Property != nil {
		objectMap["Property"] = srhree.Property
	}
	if srhree.HealthState != nil {
		objectMap["HealthState"] = srhree.HealthState
	}
	if srhree.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = srhree.TimeToLiveMs
	}
	if srhree.SequenceNumber != nil {
		objectMap["SequenceNumber"] = srhree.SequenceNumber
	}
	if srhree.Description != nil {
		objectMap["Description"] = srhree.Description
	}
	if srhree.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = srhree.RemoveWhenExpired
	}
	if srhree.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = srhree.SourceUtcTimestamp
	}
	if srhree.PartitionID != nil {
		objectMap["PartitionId"] = srhree.PartitionID
	}
	if srhree.ReplicaID != nil {
		objectMap["ReplicaId"] = srhree.ReplicaID
	}
	if srhree.EventInstanceID != nil {
		objectMap["EventInstanceId"] = srhree.EventInstanceID
	}
	if srhree.Category != nil {
		objectMap["Category"] = srhree.Category
	}
	if srhree.TimeStamp != nil {
		objectMap["TimeStamp"] = srhree.TimeStamp
	}
	if srhree.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = srhree.HasCorrelatedEvents
	}
	if srhree.Kind != "" {
		objectMap["Kind"] = srhree.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return &srhree, true
}

// AsServiceEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return &srhree, true
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for StatefulReplicaHealthReportExpiredEvent.
func (srhree StatefulReplicaHealthReportExpiredEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &srhree, true
}

// StatefulReplicaNewHealthReportEvent stateful Replica Health Report Created event.
type StatefulReplicaNewHealthReportEvent struct {
	// ReplicaInstanceID - Id of Replica instance.
	ReplicaInstanceID *int64 `json:"ReplicaInstanceId,omitempty"`
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *int64 `json:"ReplicaId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) MarshalJSON() ([]byte, error) {
	srnhre.Kind = KindStatefulReplicaNewHealthReport
	objectMap := make(map[string]interface{})
	if srnhre.ReplicaInstanceID != nil {
		objectMap["ReplicaInstanceId"] = srnhre.ReplicaInstanceID
	}
	if srnhre.SourceID != nil {
		objectMap["SourceId"] = srnhre.SourceID
	}
	if srnhre.Property != nil {
		objectMap["Property"] = srnhre.Property
	}
	if srnhre.HealthState != nil {
		objectMap["HealthState"] = srnhre.HealthState
	}
	if srnhre.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = srnhre.TimeToLiveMs
	}
	if srnhre.SequenceNumber != nil {
		objectMap["SequenceNumber"] = srnhre.SequenceNumber
	}
	if srnhre.Description != nil {
		objectMap["Description"] = srnhre.Description
	}
	if srnhre.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = srnhre.RemoveWhenExpired
	}
	if srnhre.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = srnhre.SourceUtcTimestamp
	}
	if srnhre.PartitionID != nil {
		objectMap["PartitionId"] = srnhre.PartitionID
	}
	if srnhre.ReplicaID != nil {
		objectMap["ReplicaId"] = srnhre.ReplicaID
	}
	if srnhre.EventInstanceID != nil {
		objectMap["EventInstanceId"] = srnhre.EventInstanceID
	}
	if srnhre.Category != nil {
		objectMap["Category"] = srnhre.Category
	}
	if srnhre.TimeStamp != nil {
		objectMap["TimeStamp"] = srnhre.TimeStamp
	}
	if srnhre.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = srnhre.HasCorrelatedEvents
	}
	if srnhre.Kind != "" {
		objectMap["Kind"] = srnhre.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return &srnhre, true
}

// AsServiceEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return &srnhre, true
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for StatefulReplicaNewHealthReportEvent.
func (srnhre StatefulReplicaNewHealthReportEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &srnhre, true
}

// StatefulServiceDescription describes a stateful service.
type StatefulServiceDescription struct {
	// TargetReplicaSetSize - The target replica set size as a number.
	TargetReplicaSetSize *int32 `json:"TargetReplicaSetSize,omitempty"`
	// MinReplicaSetSize - The minimum replica set size as a number.
	MinReplicaSetSize *int32 `json:"MinReplicaSetSize,omitempty"`
	// HasPersistedState - A flag indicating whether this is a persistent service which stores states on the local disk. If it is then the value of this property is true, if not it is false.
	HasPersistedState *bool `json:"HasPersistedState,omitempty"`
	// Flags - Flags indicating whether other properties are set. Each of the associated properties corresponds to a flag, specified below, which, if set, indicate that the property is specified.
	// This property can be a combination of those flags obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6 then the flags for QuorumLossWaitDuration (2) and StandByReplicaKeepDuration(4) are set.
	// - None - Does not indicate any other properties are set. The value is zero.
	// - ReplicaRestartWaitDuration - Indicates the ReplicaRestartWaitDuration property is set. The value is 1.
	// - QuorumLossWaitDuration - Indicates the QuorumLossWaitDuration property is set. The value is 2.
	// - StandByReplicaKeepDuration - Indicates the StandByReplicaKeepDuration property is set. The value is 4.
	Flags *int32 `json:"Flags,omitempty"`
	// ReplicaRestartWaitDurationSeconds - The duration, in seconds, between when a replica goes down and when a new replica is created.
	ReplicaRestartWaitDurationSeconds *int64 `json:"ReplicaRestartWaitDurationSeconds,omitempty"`
	// QuorumLossWaitDurationSeconds - The maximum duration, in seconds, for which a partition is allowed to be in a state of quorum loss.
	QuorumLossWaitDurationSeconds *int64 `json:"QuorumLossWaitDurationSeconds,omitempty"`
	// StandByReplicaKeepDurationSeconds - The definition on how long StandBy replicas should be maintained before being removed.
	StandByReplicaKeepDurationSeconds *int64 `json:"StandByReplicaKeepDurationSeconds,omitempty"`
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// InitializationData - The initialization data as an array of bytes. Initialization data is passed to service instances or replicas when they are created.
	InitializationData *[]int32 `json:"InitializationData,omitempty"`
	// PartitionDescription - The partition description as an object.
	PartitionDescription BasicPartitionSchemeDescription `json:"PartitionDescription,omitempty"`
	// PlacementConstraints - The placement constraints as a string. Placement constraints are boolean expressions on node properties and allow for restricting a service to particular nodes based on the service requirements. For example, to place a service on nodes where NodeType is blue specify the following: "NodeColor == blue)".
	PlacementConstraints *string `json:"PlacementConstraints,omitempty"`
	// CorrelationScheme - The correlation scheme.
	CorrelationScheme *[]ServiceCorrelationDescription `json:"CorrelationScheme,omitempty"`
	// ServiceLoadMetrics - The service load metrics.
	ServiceLoadMetrics *[]ServiceLoadMetricDescription `json:"ServiceLoadMetrics,omitempty"`
	// ServicePlacementPolicies - The service placement policies.
	ServicePlacementPolicies *[]BasicServicePlacementPolicyDescription `json:"ServicePlacementPolicies,omitempty"`
	// DefaultMoveCost - The move cost for the service. Possible values include: 'Zero', 'Low', 'Medium', 'High'
	DefaultMoveCost MoveCost `json:"DefaultMoveCost,omitempty"`
	// IsDefaultMoveCostSpecified - Indicates if the DefaultMoveCost property is specified.
	IsDefaultMoveCostSpecified *bool `json:"IsDefaultMoveCostSpecified,omitempty"`
	// ServicePackageActivationMode - The activation mode of service package to be used for a service. Possible values include: 'SharedProcess', 'ExclusiveProcess'
	ServicePackageActivationMode ServicePackageActivationMode `json:"ServicePackageActivationMode,omitempty"`
	// ServiceDNSName - The DNS name of the service. It requires the DNS system service to be enabled in Service Fabric cluster.
	ServiceDNSName *string `json:"ServiceDnsName,omitempty"`
	// ScalingPolicies - Scaling policies for this service.
	ScalingPolicies *[]ScalingPolicyDescription `json:"ScalingPolicies,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServiceDescriptionServiceKindServiceDescription', 'ServiceKindBasicServiceDescriptionServiceKindStateful', 'ServiceKindBasicServiceDescriptionServiceKindStateless'
	ServiceKind ServiceKindBasicServiceDescription `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatefulServiceDescription.
func (ssd StatefulServiceDescription) MarshalJSON() ([]byte, error) {
	ssd.ServiceKind = ServiceKindBasicServiceDescriptionServiceKindStateful
	objectMap := make(map[string]interface{})
	if ssd.TargetReplicaSetSize != nil {
		objectMap["TargetReplicaSetSize"] = ssd.TargetReplicaSetSize
	}
	if ssd.MinReplicaSetSize != nil {
		objectMap["MinReplicaSetSize"] = ssd.MinReplicaSetSize
	}
	if ssd.HasPersistedState != nil {
		objectMap["HasPersistedState"] = ssd.HasPersistedState
	}
	if ssd.Flags != nil {
		objectMap["Flags"] = ssd.Flags
	}
	if ssd.ReplicaRestartWaitDurationSeconds != nil {
		objectMap["ReplicaRestartWaitDurationSeconds"] = ssd.ReplicaRestartWaitDurationSeconds
	}
	if ssd.QuorumLossWaitDurationSeconds != nil {
		objectMap["QuorumLossWaitDurationSeconds"] = ssd.QuorumLossWaitDurationSeconds
	}
	if ssd.StandByReplicaKeepDurationSeconds != nil {
		objectMap["StandByReplicaKeepDurationSeconds"] = ssd.StandByReplicaKeepDurationSeconds
	}
	if ssd.ApplicationName != nil {
		objectMap["ApplicationName"] = ssd.ApplicationName
	}
	if ssd.ServiceName != nil {
		objectMap["ServiceName"] = ssd.ServiceName
	}
	if ssd.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = ssd.ServiceTypeName
	}
	if ssd.InitializationData != nil {
		objectMap["InitializationData"] = ssd.InitializationData
	}
	objectMap["PartitionDescription"] = ssd.PartitionDescription
	if ssd.PlacementConstraints != nil {
		objectMap["PlacementConstraints"] = ssd.PlacementConstraints
	}
	if ssd.CorrelationScheme != nil {
		objectMap["CorrelationScheme"] = ssd.CorrelationScheme
	}
	if ssd.ServiceLoadMetrics != nil {
		objectMap["ServiceLoadMetrics"] = ssd.ServiceLoadMetrics
	}
	if ssd.ServicePlacementPolicies != nil {
		objectMap["ServicePlacementPolicies"] = ssd.ServicePlacementPolicies
	}
	if ssd.DefaultMoveCost != "" {
		objectMap["DefaultMoveCost"] = ssd.DefaultMoveCost
	}
	if ssd.IsDefaultMoveCostSpecified != nil {
		objectMap["IsDefaultMoveCostSpecified"] = ssd.IsDefaultMoveCostSpecified
	}
	if ssd.ServicePackageActivationMode != "" {
		objectMap["ServicePackageActivationMode"] = ssd.ServicePackageActivationMode
	}
	if ssd.ServiceDNSName != nil {
		objectMap["ServiceDnsName"] = ssd.ServiceDNSName
	}
	if ssd.ScalingPolicies != nil {
		objectMap["ScalingPolicies"] = ssd.ScalingPolicies
	}
	if ssd.ServiceKind != "" {
		objectMap["ServiceKind"] = ssd.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceDescription is the BasicServiceDescription implementation for StatefulServiceDescription.
func (ssd StatefulServiceDescription) AsStatefulServiceDescription() (*StatefulServiceDescription, bool) {
	return &ssd, true
}

// AsStatelessServiceDescription is the BasicServiceDescription implementation for StatefulServiceDescription.
func (ssd StatefulServiceDescription) AsStatelessServiceDescription() (*StatelessServiceDescription, bool) {
	return nil, false
}

// AsServiceDescription is the BasicServiceDescription implementation for StatefulServiceDescription.
func (ssd StatefulServiceDescription) AsServiceDescription() (*ServiceDescription, bool) {
	return nil, false
}

// AsBasicServiceDescription is the BasicServiceDescription implementation for StatefulServiceDescription.
func (ssd StatefulServiceDescription) AsBasicServiceDescription() (BasicServiceDescription, bool) {
	return &ssd, true
}

// UnmarshalJSON is the custom unmarshaler for StatefulServiceDescription struct.
func (ssd *StatefulServiceDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "TargetReplicaSetSize":
			if v != nil {
				var targetReplicaSetSize int32
				err = json.Unmarshal(*v, &targetReplicaSetSize)
				if err != nil {
					return err
				}
				ssd.TargetReplicaSetSize = &targetReplicaSetSize
			}
		case "MinReplicaSetSize":
			if v != nil {
				var minReplicaSetSize int32
				err = json.Unmarshal(*v, &minReplicaSetSize)
				if err != nil {
					return err
				}
				ssd.MinReplicaSetSize = &minReplicaSetSize
			}
		case "HasPersistedState":
			if v != nil {
				var hasPersistedState bool
				err = json.Unmarshal(*v, &hasPersistedState)
				if err != nil {
					return err
				}
				ssd.HasPersistedState = &hasPersistedState
			}
		case "Flags":
			if v != nil {
				var flags int32
				err = json.Unmarshal(*v, &flags)
				if err != nil {
					return err
				}
				ssd.Flags = &flags
			}
		case "ReplicaRestartWaitDurationSeconds":
			if v != nil {
				var replicaRestartWaitDurationSeconds int64
				err = json.Unmarshal(*v, &replicaRestartWaitDurationSeconds)
				if err != nil {
					return err
				}
				ssd.ReplicaRestartWaitDurationSeconds = &replicaRestartWaitDurationSeconds
			}
		case "QuorumLossWaitDurationSeconds":
			if v != nil {
				var quorumLossWaitDurationSeconds int64
				err = json.Unmarshal(*v, &quorumLossWaitDurationSeconds)
				if err != nil {
					return err
				}
				ssd.QuorumLossWaitDurationSeconds = &quorumLossWaitDurationSeconds
			}
		case "StandByReplicaKeepDurationSeconds":
			if v != nil {
				var standByReplicaKeepDurationSeconds int64
				err = json.Unmarshal(*v, &standByReplicaKeepDurationSeconds)
				if err != nil {
					return err
				}
				ssd.StandByReplicaKeepDurationSeconds = &standByReplicaKeepDurationSeconds
			}
		case "ApplicationName":
			if v != nil {
				var applicationName string
				err = json.Unmarshal(*v, &applicationName)
				if err != nil {
					return err
				}
				ssd.ApplicationName = &applicationName
			}
		case "ServiceName":
			if v != nil {
				var serviceName string
				err = json.Unmarshal(*v, &serviceName)
				if err != nil {
					return err
				}
				ssd.ServiceName = &serviceName
			}
		case "ServiceTypeName":
			if v != nil {
				var serviceTypeName string
				err = json.Unmarshal(*v, &serviceTypeName)
				if err != nil {
					return err
				}
				ssd.ServiceTypeName = &serviceTypeName
			}
		case "InitializationData":
			if v != nil {
				var initializationData []int32
				err = json.Unmarshal(*v, &initializationData)
				if err != nil {
					return err
				}
				ssd.InitializationData = &initializationData
			}
		case "PartitionDescription":
			if v != nil {
				partitionDescription, err := unmarshalBasicPartitionSchemeDescription(*v)
				if err != nil {
					return err
				}
				ssd.PartitionDescription = partitionDescription
			}
		case "PlacementConstraints":
			if v != nil {
				var placementConstraints string
				err = json.Unmarshal(*v, &placementConstraints)
				if err != nil {
					return err
				}
				ssd.PlacementConstraints = &placementConstraints
			}
		case "CorrelationScheme":
			if v != nil {
				var correlationScheme []ServiceCorrelationDescription
				err = json.Unmarshal(*v, &correlationScheme)
				if err != nil {
					return err
				}
				ssd.CorrelationScheme = &correlationScheme
			}
		case "ServiceLoadMetrics":
			if v != nil {
				var serviceLoadMetrics []ServiceLoadMetricDescription
				err = json.Unmarshal(*v, &serviceLoadMetrics)
				if err != nil {
					return err
				}
				ssd.ServiceLoadMetrics = &serviceLoadMetrics
			}
		case "ServicePlacementPolicies":
			if v != nil {
				servicePlacementPolicies, err := unmarshalBasicServicePlacementPolicyDescriptionArray(*v)
				if err != nil {
					return err
				}
				ssd.ServicePlacementPolicies = &servicePlacementPolicies
			}
		case "DefaultMoveCost":
			if v != nil {
				var defaultMoveCost MoveCost
				err = json.Unmarshal(*v, &defaultMoveCost)
				if err != nil {
					return err
				}
				ssd.DefaultMoveCost = defaultMoveCost
			}
		case "IsDefaultMoveCostSpecified":
			if v != nil {
				var isDefaultMoveCostSpecified bool
				err = json.Unmarshal(*v, &isDefaultMoveCostSpecified)
				if err != nil {
					return err
				}
				ssd.IsDefaultMoveCostSpecified = &isDefaultMoveCostSpecified
			}
		case "ServicePackageActivationMode":
			if v != nil {
				var servicePackageActivationMode ServicePackageActivationMode
				err = json.Unmarshal(*v, &servicePackageActivationMode)
				if err != nil {
					return err
				}
				ssd.ServicePackageActivationMode = servicePackageActivationMode
			}
		case "ServiceDnsName":
			if v != nil {
				var serviceDNSName string
				err = json.Unmarshal(*v, &serviceDNSName)
				if err != nil {
					return err
				}
				ssd.ServiceDNSName = &serviceDNSName
			}
		case "ScalingPolicies":
			if v != nil {
				var scalingPolicies []ScalingPolicyDescription
				err = json.Unmarshal(*v, &scalingPolicies)
				if err != nil {
					return err
				}
				ssd.ScalingPolicies = &scalingPolicies
			}
		case "ServiceKind":
			if v != nil {
				var serviceKind ServiceKindBasicServiceDescription
				err = json.Unmarshal(*v, &serviceKind)
				if err != nil {
					return err
				}
				ssd.ServiceKind = serviceKind
			}
		}
	}

	return nil
}

// StatefulServiceInfo information about a stateful Service Fabric service.
type StatefulServiceInfo struct {
	// HasPersistedState - Whether the service has persisted state.
	HasPersistedState *bool `json:"HasPersistedState,omitempty"`
	// ID - The identity of the service. This ID is an encoded representation of the service name. This is used in the REST APIs to identify the service resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the service name is "fabric:/myapp/app1/svc1",
	// the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous versions.
	ID *string `json:"Id,omitempty"`
	// Name - The full name of the service with 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// TypeName - Name of the service type as specified in the service manifest.
	TypeName *string `json:"TypeName,omitempty"`
	// ManifestVersion - The version of the service manifest.
	ManifestVersion *string `json:"ManifestVersion,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// ServiceStatus - The status of the application. Possible values include: 'ServiceStatusUnknown', 'ServiceStatusActive', 'ServiceStatusUpgrading', 'ServiceStatusDeleting', 'ServiceStatusCreating', 'ServiceStatusFailed'
	ServiceStatus ServiceStatus `json:"ServiceStatus,omitempty"`
	// IsServiceGroup - Whether the service is in a service group.
	IsServiceGroup *bool `json:"IsServiceGroup,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServiceInfoServiceKindServiceInfo', 'ServiceKindBasicServiceInfoServiceKindStateful', 'ServiceKindBasicServiceInfoServiceKindStateless'
	ServiceKind ServiceKindBasicServiceInfo `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatefulServiceInfo.
func (ssi StatefulServiceInfo) MarshalJSON() ([]byte, error) {
	ssi.ServiceKind = ServiceKindBasicServiceInfoServiceKindStateful
	objectMap := make(map[string]interface{})
	if ssi.HasPersistedState != nil {
		objectMap["HasPersistedState"] = ssi.HasPersistedState
	}
	if ssi.ID != nil {
		objectMap["Id"] = ssi.ID
	}
	if ssi.Name != nil {
		objectMap["Name"] = ssi.Name
	}
	if ssi.TypeName != nil {
		objectMap["TypeName"] = ssi.TypeName
	}
	if ssi.ManifestVersion != nil {
		objectMap["ManifestVersion"] = ssi.ManifestVersion
	}
	if ssi.HealthState != "" {
		objectMap["HealthState"] = ssi.HealthState
	}
	if ssi.ServiceStatus != "" {
		objectMap["ServiceStatus"] = ssi.ServiceStatus
	}
	if ssi.IsServiceGroup != nil {
		objectMap["IsServiceGroup"] = ssi.IsServiceGroup
	}
	if ssi.ServiceKind != "" {
		objectMap["ServiceKind"] = ssi.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceInfo is the BasicServiceInfo implementation for StatefulServiceInfo.
func (ssi StatefulServiceInfo) AsStatefulServiceInfo() (*StatefulServiceInfo, bool) {
	return &ssi, true
}

// AsStatelessServiceInfo is the BasicServiceInfo implementation for StatefulServiceInfo.
func (ssi StatefulServiceInfo) AsStatelessServiceInfo() (*StatelessServiceInfo, bool) {
	return nil, false
}

// AsServiceInfo is the BasicServiceInfo implementation for StatefulServiceInfo.
func (ssi StatefulServiceInfo) AsServiceInfo() (*ServiceInfo, bool) {
	return nil, false
}

// AsBasicServiceInfo is the BasicServiceInfo implementation for StatefulServiceInfo.
func (ssi StatefulServiceInfo) AsBasicServiceInfo() (BasicServiceInfo, bool) {
	return &ssi, true
}

// StatefulServicePartitionInfo information about a partition of a stateful Service Fabric service..
type StatefulServicePartitionInfo struct {
	// TargetReplicaSetSize - The target replica set size as a number.
	TargetReplicaSetSize *int64 `json:"TargetReplicaSetSize,omitempty"`
	// MinReplicaSetSize - The minimum replica set size as a number.
	MinReplicaSetSize *int64 `json:"MinReplicaSetSize,omitempty"`
	// LastQuorumLossDuration - The duration for which this partition was in quorum loss. If the partition is currently in quorum loss, it returns the duration since it has been in that state. This field is using ISO8601 format for specifying the duration.
	LastQuorumLossDuration *string `json:"LastQuorumLossDuration,omitempty"`
	// PrimaryEpoch - An Epoch is a configuration number for the partition as a whole. When the configuration of the replica set changes, for example when the Primary replica changes, the operations that are replicated from the new Primary replica are said to be a new Epoch from the ones which were sent by the old Primary replica.
	PrimaryEpoch *Epoch `json:"PrimaryEpoch,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// PartitionStatus - The status of the service fabric service partition. Possible values include: 'ServicePartitionStatusInvalid', 'ServicePartitionStatusReady', 'ServicePartitionStatusNotReady', 'ServicePartitionStatusInQuorumLoss', 'ServicePartitionStatusReconfiguring', 'ServicePartitionStatusDeleting'
	PartitionStatus ServicePartitionStatus `json:"PartitionStatus,omitempty"`
	// PartitionInformation - Information about the partition identity, partitioning scheme and keys supported by it.
	PartitionInformation BasicPartitionInformation `json:"PartitionInformation,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServicePartitionInfoServiceKindServicePartitionInfo', 'ServiceKindBasicServicePartitionInfoServiceKindStateful', 'ServiceKindBasicServicePartitionInfoServiceKindStateless'
	ServiceKind ServiceKindBasicServicePartitionInfo `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatefulServicePartitionInfo.
func (sspi StatefulServicePartitionInfo) MarshalJSON() ([]byte, error) {
	sspi.ServiceKind = ServiceKindBasicServicePartitionInfoServiceKindStateful
	objectMap := make(map[string]interface{})
	if sspi.TargetReplicaSetSize != nil {
		objectMap["TargetReplicaSetSize"] = sspi.TargetReplicaSetSize
	}
	if sspi.MinReplicaSetSize != nil {
		objectMap["MinReplicaSetSize"] = sspi.MinReplicaSetSize
	}
	if sspi.LastQuorumLossDuration != nil {
		objectMap["LastQuorumLossDuration"] = sspi.LastQuorumLossDuration
	}
	if sspi.PrimaryEpoch != nil {
		objectMap["PrimaryEpoch"] = sspi.PrimaryEpoch
	}
	if sspi.HealthState != "" {
		objectMap["HealthState"] = sspi.HealthState
	}
	if sspi.PartitionStatus != "" {
		objectMap["PartitionStatus"] = sspi.PartitionStatus
	}
	objectMap["PartitionInformation"] = sspi.PartitionInformation
	if sspi.ServiceKind != "" {
		objectMap["ServiceKind"] = sspi.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServicePartitionInfo is the BasicServicePartitionInfo implementation for StatefulServicePartitionInfo.
func (sspi StatefulServicePartitionInfo) AsStatefulServicePartitionInfo() (*StatefulServicePartitionInfo, bool) {
	return &sspi, true
}

// AsStatelessServicePartitionInfo is the BasicServicePartitionInfo implementation for StatefulServicePartitionInfo.
func (sspi StatefulServicePartitionInfo) AsStatelessServicePartitionInfo() (*StatelessServicePartitionInfo, bool) {
	return nil, false
}

// AsServicePartitionInfo is the BasicServicePartitionInfo implementation for StatefulServicePartitionInfo.
func (sspi StatefulServicePartitionInfo) AsServicePartitionInfo() (*ServicePartitionInfo, bool) {
	return nil, false
}

// AsBasicServicePartitionInfo is the BasicServicePartitionInfo implementation for StatefulServicePartitionInfo.
func (sspi StatefulServicePartitionInfo) AsBasicServicePartitionInfo() (BasicServicePartitionInfo, bool) {
	return &sspi, true
}

// UnmarshalJSON is the custom unmarshaler for StatefulServicePartitionInfo struct.
func (sspi *StatefulServicePartitionInfo) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "TargetReplicaSetSize":
			if v != nil {
				var targetReplicaSetSize int64
				err = json.Unmarshal(*v, &targetReplicaSetSize)
				if err != nil {
					return err
				}
				sspi.TargetReplicaSetSize = &targetReplicaSetSize
			}
		case "MinReplicaSetSize":
			if v != nil {
				var minReplicaSetSize int64
				err = json.Unmarshal(*v, &minReplicaSetSize)
				if err != nil {
					return err
				}
				sspi.MinReplicaSetSize = &minReplicaSetSize
			}
		case "LastQuorumLossDuration":
			if v != nil {
				var lastQuorumLossDuration string
				err = json.Unmarshal(*v, &lastQuorumLossDuration)
				if err != nil {
					return err
				}
				sspi.LastQuorumLossDuration = &lastQuorumLossDuration
			}
		case "PrimaryEpoch":
			if v != nil {
				var primaryEpoch Epoch
				err = json.Unmarshal(*v, &primaryEpoch)
				if err != nil {
					return err
				}
				sspi.PrimaryEpoch = &primaryEpoch
			}
		case "HealthState":
			if v != nil {
				var healthState HealthState
				err = json.Unmarshal(*v, &healthState)
				if err != nil {
					return err
				}
				sspi.HealthState = healthState
			}
		case "PartitionStatus":
			if v != nil {
				var partitionStatus ServicePartitionStatus
				err = json.Unmarshal(*v, &partitionStatus)
				if err != nil {
					return err
				}
				sspi.PartitionStatus = partitionStatus
			}
		case "PartitionInformation":
			if v != nil {
				partitionInformation, err := unmarshalBasicPartitionInformation(*v)
				if err != nil {
					return err
				}
				sspi.PartitionInformation = partitionInformation
			}
		case "ServiceKind":
			if v != nil {
				var serviceKind ServiceKindBasicServicePartitionInfo
				err = json.Unmarshal(*v, &serviceKind)
				if err != nil {
					return err
				}
				sspi.ServiceKind = serviceKind
			}
		}
	}

	return nil
}

// StatefulServiceReplicaHealth represents the health of the stateful service replica.
// Contains the replica aggregated health state, the health events and the unhealthy evaluations.
type StatefulServiceReplicaHealth struct {
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *string `json:"ReplicaId,omitempty"`
	// PartitionID - Id of the partition to which this replica belongs.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicReplicaHealthServiceKindReplicaHealth', 'ServiceKindBasicReplicaHealthServiceKindStateful', 'ServiceKindBasicReplicaHealthServiceKindStateless'
	ServiceKind ServiceKindBasicReplicaHealth `json:"ServiceKind,omitempty"`
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

// MarshalJSON is the custom marshaler for StatefulServiceReplicaHealth.
func (ssrh StatefulServiceReplicaHealth) MarshalJSON() ([]byte, error) {
	ssrh.ServiceKind = ServiceKindBasicReplicaHealthServiceKindStateful
	objectMap := make(map[string]interface{})
	if ssrh.ReplicaID != nil {
		objectMap["ReplicaId"] = ssrh.ReplicaID
	}
	if ssrh.PartitionID != nil {
		objectMap["PartitionId"] = ssrh.PartitionID
	}
	if ssrh.ServiceKind != "" {
		objectMap["ServiceKind"] = ssrh.ServiceKind
	}
	if ssrh.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = ssrh.AggregatedHealthState
	}
	if ssrh.HealthEvents != nil {
		objectMap["HealthEvents"] = ssrh.HealthEvents
	}
	if ssrh.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = ssrh.UnhealthyEvaluations
	}
	if ssrh.HealthStatistics != nil {
		objectMap["HealthStatistics"] = ssrh.HealthStatistics
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceReplicaHealth is the BasicReplicaHealth implementation for StatefulServiceReplicaHealth.
func (ssrh StatefulServiceReplicaHealth) AsStatefulServiceReplicaHealth() (*StatefulServiceReplicaHealth, bool) {
	return &ssrh, true
}

// AsStatelessServiceInstanceHealth is the BasicReplicaHealth implementation for StatefulServiceReplicaHealth.
func (ssrh StatefulServiceReplicaHealth) AsStatelessServiceInstanceHealth() (*StatelessServiceInstanceHealth, bool) {
	return nil, false
}

// AsReplicaHealth is the BasicReplicaHealth implementation for StatefulServiceReplicaHealth.
func (ssrh StatefulServiceReplicaHealth) AsReplicaHealth() (*ReplicaHealth, bool) {
	return nil, false
}

// AsBasicReplicaHealth is the BasicReplicaHealth implementation for StatefulServiceReplicaHealth.
func (ssrh StatefulServiceReplicaHealth) AsBasicReplicaHealth() (BasicReplicaHealth, bool) {
	return &ssrh, true
}

// StatefulServiceReplicaHealthState represents the health state of the stateful service replica, which
// contains the replica ID and the aggregated health state.
type StatefulServiceReplicaHealthState struct {
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *string `json:"ReplicaId,omitempty"`
	// PartitionID - The ID of the partition to which this replica belongs.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicReplicaHealthStateServiceKindReplicaHealthState', 'ServiceKindBasicReplicaHealthStateServiceKindStateful', 'ServiceKindBasicReplicaHealthStateServiceKindStateless'
	ServiceKind ServiceKindBasicReplicaHealthState `json:"ServiceKind,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
}

// MarshalJSON is the custom marshaler for StatefulServiceReplicaHealthState.
func (ssrhs StatefulServiceReplicaHealthState) MarshalJSON() ([]byte, error) {
	ssrhs.ServiceKind = ServiceKindBasicReplicaHealthStateServiceKindStateful
	objectMap := make(map[string]interface{})
	if ssrhs.ReplicaID != nil {
		objectMap["ReplicaId"] = ssrhs.ReplicaID
	}
	if ssrhs.PartitionID != nil {
		objectMap["PartitionId"] = ssrhs.PartitionID
	}
	if ssrhs.ServiceKind != "" {
		objectMap["ServiceKind"] = ssrhs.ServiceKind
	}
	if ssrhs.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = ssrhs.AggregatedHealthState
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceReplicaHealthState is the BasicReplicaHealthState implementation for StatefulServiceReplicaHealthState.
func (ssrhs StatefulServiceReplicaHealthState) AsStatefulServiceReplicaHealthState() (*StatefulServiceReplicaHealthState, bool) {
	return &ssrhs, true
}

// AsStatelessServiceInstanceHealthState is the BasicReplicaHealthState implementation for StatefulServiceReplicaHealthState.
func (ssrhs StatefulServiceReplicaHealthState) AsStatelessServiceInstanceHealthState() (*StatelessServiceInstanceHealthState, bool) {
	return nil, false
}

// AsReplicaHealthState is the BasicReplicaHealthState implementation for StatefulServiceReplicaHealthState.
func (ssrhs StatefulServiceReplicaHealthState) AsReplicaHealthState() (*ReplicaHealthState, bool) {
	return nil, false
}

// AsBasicReplicaHealthState is the BasicReplicaHealthState implementation for StatefulServiceReplicaHealthState.
func (ssrhs StatefulServiceReplicaHealthState) AsBasicReplicaHealthState() (BasicReplicaHealthState, bool) {
	return &ssrhs, true
}

// StatefulServiceReplicaInfo represents a stateful service replica. This includes information about the
// identity, role, status, health, node name, uptime, and other details about the replica.
type StatefulServiceReplicaInfo struct {
	// ReplicaRole - The role of a replica of a stateful service. Possible values include: 'ReplicaRoleUnknown', 'ReplicaRoleNone', 'ReplicaRolePrimary', 'ReplicaRoleIdleSecondary', 'ReplicaRoleActiveSecondary'
	ReplicaRole ReplicaRole `json:"ReplicaRole,omitempty"`
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *string `json:"ReplicaId,omitempty"`
	// ReplicaStatus - The status of a replica of a service. Possible values include: 'ReplicaStatusInvalid', 'ReplicaStatusInBuild', 'ReplicaStatusStandby', 'ReplicaStatusReady', 'ReplicaStatusDown', 'ReplicaStatusDropped'
	ReplicaStatus ReplicaStatus `json:"ReplicaStatus,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// Address - The address the replica is listening on.
	Address *string `json:"Address,omitempty"`
	// LastInBuildDurationInSeconds - The last in build duration of the replica in seconds.
	LastInBuildDurationInSeconds *string `json:"LastInBuildDurationInSeconds,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicReplicaInfoServiceKindReplicaInfo', 'ServiceKindBasicReplicaInfoServiceKindStateful', 'ServiceKindBasicReplicaInfoServiceKindStateless'
	ServiceKind ServiceKindBasicReplicaInfo `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatefulServiceReplicaInfo.
func (ssri StatefulServiceReplicaInfo) MarshalJSON() ([]byte, error) {
	ssri.ServiceKind = ServiceKindBasicReplicaInfoServiceKindStateful
	objectMap := make(map[string]interface{})
	if ssri.ReplicaRole != "" {
		objectMap["ReplicaRole"] = ssri.ReplicaRole
	}
	if ssri.ReplicaID != nil {
		objectMap["ReplicaId"] = ssri.ReplicaID
	}
	if ssri.ReplicaStatus != "" {
		objectMap["ReplicaStatus"] = ssri.ReplicaStatus
	}
	if ssri.HealthState != "" {
		objectMap["HealthState"] = ssri.HealthState
	}
	if ssri.NodeName != nil {
		objectMap["NodeName"] = ssri.NodeName
	}
	if ssri.Address != nil {
		objectMap["Address"] = ssri.Address
	}
	if ssri.LastInBuildDurationInSeconds != nil {
		objectMap["LastInBuildDurationInSeconds"] = ssri.LastInBuildDurationInSeconds
	}
	if ssri.ServiceKind != "" {
		objectMap["ServiceKind"] = ssri.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceReplicaInfo is the BasicReplicaInfo implementation for StatefulServiceReplicaInfo.
func (ssri StatefulServiceReplicaInfo) AsStatefulServiceReplicaInfo() (*StatefulServiceReplicaInfo, bool) {
	return &ssri, true
}

// AsStatelessServiceInstanceInfo is the BasicReplicaInfo implementation for StatefulServiceReplicaInfo.
func (ssri StatefulServiceReplicaInfo) AsStatelessServiceInstanceInfo() (*StatelessServiceInstanceInfo, bool) {
	return nil, false
}

// AsReplicaInfo is the BasicReplicaInfo implementation for StatefulServiceReplicaInfo.
func (ssri StatefulServiceReplicaInfo) AsReplicaInfo() (*ReplicaInfo, bool) {
	return nil, false
}

// AsBasicReplicaInfo is the BasicReplicaInfo implementation for StatefulServiceReplicaInfo.
func (ssri StatefulServiceReplicaInfo) AsBasicReplicaInfo() (BasicReplicaInfo, bool) {
	return &ssri, true
}

// StatefulServiceTypeDescription describes a stateful service type defined in the service manifest of a
// provisioned application type.
type StatefulServiceTypeDescription struct {
	// HasPersistedState - A flag indicating whether this is a persistent service which stores states on the local disk. If it is then the value of this property is true, if not it is false.
	HasPersistedState *bool `json:"HasPersistedState,omitempty"`
	// IsStateful - Indicates whether the service type is a stateful service type or a stateless service type. This property is true if the service type is a stateful service type, false otherwise.
	IsStateful *bool `json:"IsStateful,omitempty"`
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// PlacementConstraints - The placement constraint to be used when instantiating this service in a Service Fabric cluster.
	PlacementConstraints *string `json:"PlacementConstraints,omitempty"`
	// LoadMetrics - The service load metrics is given as an array of ServiceLoadMetricDescription objects.
	LoadMetrics *[]ServiceLoadMetricDescription `json:"LoadMetrics,omitempty"`
	// ServicePlacementPolicies - List of service placement policy descriptions.
	ServicePlacementPolicies *[]BasicServicePlacementPolicyDescription `json:"ServicePlacementPolicies,omitempty"`
	// Extensions - List of service type extensions.
	Extensions *[]ServiceTypeExtensionDescription `json:"Extensions,omitempty"`
	// Kind - Possible values include: 'KindServiceTypeDescription', 'KindStateful', 'KindStateless'
	Kind KindBasicServiceTypeDescription `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatefulServiceTypeDescription.
func (sstd StatefulServiceTypeDescription) MarshalJSON() ([]byte, error) {
	sstd.Kind = KindStateful
	objectMap := make(map[string]interface{})
	if sstd.HasPersistedState != nil {
		objectMap["HasPersistedState"] = sstd.HasPersistedState
	}
	if sstd.IsStateful != nil {
		objectMap["IsStateful"] = sstd.IsStateful
	}
	if sstd.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = sstd.ServiceTypeName
	}
	if sstd.PlacementConstraints != nil {
		objectMap["PlacementConstraints"] = sstd.PlacementConstraints
	}
	if sstd.LoadMetrics != nil {
		objectMap["LoadMetrics"] = sstd.LoadMetrics
	}
	if sstd.ServicePlacementPolicies != nil {
		objectMap["ServicePlacementPolicies"] = sstd.ServicePlacementPolicies
	}
	if sstd.Extensions != nil {
		objectMap["Extensions"] = sstd.Extensions
	}
	if sstd.Kind != "" {
		objectMap["Kind"] = sstd.Kind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceTypeDescription is the BasicServiceTypeDescription implementation for StatefulServiceTypeDescription.
func (sstd StatefulServiceTypeDescription) AsStatefulServiceTypeDescription() (*StatefulServiceTypeDescription, bool) {
	return &sstd, true
}

// AsStatelessServiceTypeDescription is the BasicServiceTypeDescription implementation for StatefulServiceTypeDescription.
func (sstd StatefulServiceTypeDescription) AsStatelessServiceTypeDescription() (*StatelessServiceTypeDescription, bool) {
	return nil, false
}

// AsServiceTypeDescription is the BasicServiceTypeDescription implementation for StatefulServiceTypeDescription.
func (sstd StatefulServiceTypeDescription) AsServiceTypeDescription() (*ServiceTypeDescription, bool) {
	return nil, false
}

// AsBasicServiceTypeDescription is the BasicServiceTypeDescription implementation for StatefulServiceTypeDescription.
func (sstd StatefulServiceTypeDescription) AsBasicServiceTypeDescription() (BasicServiceTypeDescription, bool) {
	return &sstd, true
}

// UnmarshalJSON is the custom unmarshaler for StatefulServiceTypeDescription struct.
func (sstd *StatefulServiceTypeDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "HasPersistedState":
			if v != nil {
				var hasPersistedState bool
				err = json.Unmarshal(*v, &hasPersistedState)
				if err != nil {
					return err
				}
				sstd.HasPersistedState = &hasPersistedState
			}
		case "IsStateful":
			if v != nil {
				var isStateful bool
				err = json.Unmarshal(*v, &isStateful)
				if err != nil {
					return err
				}
				sstd.IsStateful = &isStateful
			}
		case "ServiceTypeName":
			if v != nil {
				var serviceTypeName string
				err = json.Unmarshal(*v, &serviceTypeName)
				if err != nil {
					return err
				}
				sstd.ServiceTypeName = &serviceTypeName
			}
		case "PlacementConstraints":
			if v != nil {
				var placementConstraints string
				err = json.Unmarshal(*v, &placementConstraints)
				if err != nil {
					return err
				}
				sstd.PlacementConstraints = &placementConstraints
			}
		case "LoadMetrics":
			if v != nil {
				var loadMetrics []ServiceLoadMetricDescription
				err = json.Unmarshal(*v, &loadMetrics)
				if err != nil {
					return err
				}
				sstd.LoadMetrics = &loadMetrics
			}
		case "ServicePlacementPolicies":
			if v != nil {
				servicePlacementPolicies, err := unmarshalBasicServicePlacementPolicyDescriptionArray(*v)
				if err != nil {
					return err
				}
				sstd.ServicePlacementPolicies = &servicePlacementPolicies
			}
		case "Extensions":
			if v != nil {
				var extensions []ServiceTypeExtensionDescription
				err = json.Unmarshal(*v, &extensions)
				if err != nil {
					return err
				}
				sstd.Extensions = &extensions
			}
		case "Kind":
			if v != nil {
				var kind KindBasicServiceTypeDescription
				err = json.Unmarshal(*v, &kind)
				if err != nil {
					return err
				}
				sstd.Kind = kind
			}
		}
	}

	return nil
}

// StatefulServiceUpdateDescription describes an update for a stateful service.
type StatefulServiceUpdateDescription struct {
	// TargetReplicaSetSize - The target replica set size as a number.
	TargetReplicaSetSize *int32 `json:"TargetReplicaSetSize,omitempty"`
	// MinReplicaSetSize - The minimum replica set size as a number.
	MinReplicaSetSize *int32 `json:"MinReplicaSetSize,omitempty"`
	// ReplicaRestartWaitDurationSeconds - The duration, in seconds, between when a replica goes down and when a new replica is created.
	ReplicaRestartWaitDurationSeconds *string `json:"ReplicaRestartWaitDurationSeconds,omitempty"`
	// QuorumLossWaitDurationSeconds - The maximum duration, in seconds, for which a partition is allowed to be in a state of quorum loss.
	QuorumLossWaitDurationSeconds *string `json:"QuorumLossWaitDurationSeconds,omitempty"`
	// StandByReplicaKeepDurationSeconds - The definition on how long StandBy replicas should be maintained before being removed.
	StandByReplicaKeepDurationSeconds *string `json:"StandByReplicaKeepDurationSeconds,omitempty"`
	// Flags - Flags indicating whether other properties are set. Each of the associated properties corresponds to a flag, specified below, which, if set, indicate that the property is specified.
	// This property can be a combination of those flags obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6 then the flags for ReplicaRestartWaitDuration (2) and QuorumLossWaitDuration (4) are set.
	// - None - Does not indicate any other properties are set. The value is zero.
	// - TargetReplicaSetSize/InstanceCount - Indicates whether the TargetReplicaSetSize property (for Stateful services) or the InstanceCount property (for Stateless services) is set. The value is 1.
	// - ReplicaRestartWaitDuration - Indicates the ReplicaRestartWaitDuration property is set. The value is  2.
	// - QuorumLossWaitDuration - Indicates the QuorumLossWaitDuration property is set. The value is 4.
	// - StandByReplicaKeepDuration - Indicates the StandByReplicaKeepDuration property is set. The value is 8.
	// - MinReplicaSetSize - Indicates the MinReplicaSetSize property is set. The value is 16.
	// - PlacementConstraints - Indicates the PlacementConstraints property is set. The value is 32.
	// - PlacementPolicyList - Indicates the ServicePlacementPolicies property is set. The value is 64.
	// - Correlation - Indicates the CorrelationScheme property is set. The value is 128.
	// - Metrics - Indicates the ServiceLoadMetrics property is set. The value is 256.
	// - DefaultMoveCost - Indicates the DefaultMoveCost property is set. The value is 512.
	// - ScalingPolicy - Indicates the ScalingPolicies property is set. The value is 1024.
	Flags *string `json:"Flags,omitempty"`
	// PlacementConstraints - The placement constraints as a string. Placement constraints are boolean expressions on node properties and allow for restricting a service to particular nodes based on the service requirements. For example, to place a service on nodes where NodeType is blue specify the following: "NodeColor == blue)".
	PlacementConstraints *string `json:"PlacementConstraints,omitempty"`
	// CorrelationScheme - The correlation scheme.
	CorrelationScheme *[]ServiceCorrelationDescription `json:"CorrelationScheme,omitempty"`
	// LoadMetrics - The service load metrics.
	LoadMetrics *[]ServiceLoadMetricDescription `json:"LoadMetrics,omitempty"`
	// ServicePlacementPolicies - The service placement policies.
	ServicePlacementPolicies *[]BasicServicePlacementPolicyDescription `json:"ServicePlacementPolicies,omitempty"`
	// DefaultMoveCost - The move cost for the service. Possible values include: 'Zero', 'Low', 'Medium', 'High'
	DefaultMoveCost MoveCost `json:"DefaultMoveCost,omitempty"`
	// ScalingPolicies - Scaling policies for this service.
	ScalingPolicies *[]ScalingPolicyDescription `json:"ScalingPolicies,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServiceUpdateDescriptionServiceKindServiceUpdateDescription', 'ServiceKindBasicServiceUpdateDescriptionServiceKindStateful', 'ServiceKindBasicServiceUpdateDescriptionServiceKindStateless'
	ServiceKind ServiceKindBasicServiceUpdateDescription `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatefulServiceUpdateDescription.
func (ssud StatefulServiceUpdateDescription) MarshalJSON() ([]byte, error) {
	ssud.ServiceKind = ServiceKindBasicServiceUpdateDescriptionServiceKindStateful
	objectMap := make(map[string]interface{})
	if ssud.TargetReplicaSetSize != nil {
		objectMap["TargetReplicaSetSize"] = ssud.TargetReplicaSetSize
	}
	if ssud.MinReplicaSetSize != nil {
		objectMap["MinReplicaSetSize"] = ssud.MinReplicaSetSize
	}
	if ssud.ReplicaRestartWaitDurationSeconds != nil {
		objectMap["ReplicaRestartWaitDurationSeconds"] = ssud.ReplicaRestartWaitDurationSeconds
	}
	if ssud.QuorumLossWaitDurationSeconds != nil {
		objectMap["QuorumLossWaitDurationSeconds"] = ssud.QuorumLossWaitDurationSeconds
	}
	if ssud.StandByReplicaKeepDurationSeconds != nil {
		objectMap["StandByReplicaKeepDurationSeconds"] = ssud.StandByReplicaKeepDurationSeconds
	}
	if ssud.Flags != nil {
		objectMap["Flags"] = ssud.Flags
	}
	if ssud.PlacementConstraints != nil {
		objectMap["PlacementConstraints"] = ssud.PlacementConstraints
	}
	if ssud.CorrelationScheme != nil {
		objectMap["CorrelationScheme"] = ssud.CorrelationScheme
	}
	if ssud.LoadMetrics != nil {
		objectMap["LoadMetrics"] = ssud.LoadMetrics
	}
	if ssud.ServicePlacementPolicies != nil {
		objectMap["ServicePlacementPolicies"] = ssud.ServicePlacementPolicies
	}
	if ssud.DefaultMoveCost != "" {
		objectMap["DefaultMoveCost"] = ssud.DefaultMoveCost
	}
	if ssud.ScalingPolicies != nil {
		objectMap["ScalingPolicies"] = ssud.ScalingPolicies
	}
	if ssud.ServiceKind != "" {
		objectMap["ServiceKind"] = ssud.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceUpdateDescription is the BasicServiceUpdateDescription implementation for StatefulServiceUpdateDescription.
func (ssud StatefulServiceUpdateDescription) AsStatefulServiceUpdateDescription() (*StatefulServiceUpdateDescription, bool) {
	return &ssud, true
}

// AsStatelessServiceUpdateDescription is the BasicServiceUpdateDescription implementation for StatefulServiceUpdateDescription.
func (ssud StatefulServiceUpdateDescription) AsStatelessServiceUpdateDescription() (*StatelessServiceUpdateDescription, bool) {
	return nil, false
}

// AsServiceUpdateDescription is the BasicServiceUpdateDescription implementation for StatefulServiceUpdateDescription.
func (ssud StatefulServiceUpdateDescription) AsServiceUpdateDescription() (*ServiceUpdateDescription, bool) {
	return nil, false
}

// AsBasicServiceUpdateDescription is the BasicServiceUpdateDescription implementation for StatefulServiceUpdateDescription.
func (ssud StatefulServiceUpdateDescription) AsBasicServiceUpdateDescription() (BasicServiceUpdateDescription, bool) {
	return &ssud, true
}

// UnmarshalJSON is the custom unmarshaler for StatefulServiceUpdateDescription struct.
func (ssud *StatefulServiceUpdateDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "TargetReplicaSetSize":
			if v != nil {
				var targetReplicaSetSize int32
				err = json.Unmarshal(*v, &targetReplicaSetSize)
				if err != nil {
					return err
				}
				ssud.TargetReplicaSetSize = &targetReplicaSetSize
			}
		case "MinReplicaSetSize":
			if v != nil {
				var minReplicaSetSize int32
				err = json.Unmarshal(*v, &minReplicaSetSize)
				if err != nil {
					return err
				}
				ssud.MinReplicaSetSize = &minReplicaSetSize
			}
		case "ReplicaRestartWaitDurationSeconds":
			if v != nil {
				var replicaRestartWaitDurationSeconds string
				err = json.Unmarshal(*v, &replicaRestartWaitDurationSeconds)
				if err != nil {
					return err
				}
				ssud.ReplicaRestartWaitDurationSeconds = &replicaRestartWaitDurationSeconds
			}
		case "QuorumLossWaitDurationSeconds":
			if v != nil {
				var quorumLossWaitDurationSeconds string
				err = json.Unmarshal(*v, &quorumLossWaitDurationSeconds)
				if err != nil {
					return err
				}
				ssud.QuorumLossWaitDurationSeconds = &quorumLossWaitDurationSeconds
			}
		case "StandByReplicaKeepDurationSeconds":
			if v != nil {
				var standByReplicaKeepDurationSeconds string
				err = json.Unmarshal(*v, &standByReplicaKeepDurationSeconds)
				if err != nil {
					return err
				}
				ssud.StandByReplicaKeepDurationSeconds = &standByReplicaKeepDurationSeconds
			}
		case "Flags":
			if v != nil {
				var flags string
				err = json.Unmarshal(*v, &flags)
				if err != nil {
					return err
				}
				ssud.Flags = &flags
			}
		case "PlacementConstraints":
			if v != nil {
				var placementConstraints string
				err = json.Unmarshal(*v, &placementConstraints)
				if err != nil {
					return err
				}
				ssud.PlacementConstraints = &placementConstraints
			}
		case "CorrelationScheme":
			if v != nil {
				var correlationScheme []ServiceCorrelationDescription
				err = json.Unmarshal(*v, &correlationScheme)
				if err != nil {
					return err
				}
				ssud.CorrelationScheme = &correlationScheme
			}
		case "LoadMetrics":
			if v != nil {
				var loadMetrics []ServiceLoadMetricDescription
				err = json.Unmarshal(*v, &loadMetrics)
				if err != nil {
					return err
				}
				ssud.LoadMetrics = &loadMetrics
			}
		case "ServicePlacementPolicies":
			if v != nil {
				servicePlacementPolicies, err := unmarshalBasicServicePlacementPolicyDescriptionArray(*v)
				if err != nil {
					return err
				}
				ssud.ServicePlacementPolicies = &servicePlacementPolicies
			}
		case "DefaultMoveCost":
			if v != nil {
				var defaultMoveCost MoveCost
				err = json.Unmarshal(*v, &defaultMoveCost)
				if err != nil {
					return err
				}
				ssud.DefaultMoveCost = defaultMoveCost
			}
		case "ScalingPolicies":
			if v != nil {
				var scalingPolicies []ScalingPolicyDescription
				err = json.Unmarshal(*v, &scalingPolicies)
				if err != nil {
					return err
				}
				ssud.ScalingPolicies = &scalingPolicies
			}
		case "ServiceKind":
			if v != nil {
				var serviceKind ServiceKindBasicServiceUpdateDescription
				err = json.Unmarshal(*v, &serviceKind)
				if err != nil {
					return err
				}
				ssud.ServiceKind = serviceKind
			}
		}
	}

	return nil
}

// StatelessReplicaHealthReportExpiredEvent stateless Replica Health Report Expired event.
type StatelessReplicaHealthReportExpiredEvent struct {
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *int64 `json:"ReplicaId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) MarshalJSON() ([]byte, error) {
	srhree.Kind = KindStatelessReplicaHealthReportExpired
	objectMap := make(map[string]interface{})
	if srhree.SourceID != nil {
		objectMap["SourceId"] = srhree.SourceID
	}
	if srhree.Property != nil {
		objectMap["Property"] = srhree.Property
	}
	if srhree.HealthState != nil {
		objectMap["HealthState"] = srhree.HealthState
	}
	if srhree.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = srhree.TimeToLiveMs
	}
	if srhree.SequenceNumber != nil {
		objectMap["SequenceNumber"] = srhree.SequenceNumber
	}
	if srhree.Description != nil {
		objectMap["Description"] = srhree.Description
	}
	if srhree.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = srhree.RemoveWhenExpired
	}
	if srhree.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = srhree.SourceUtcTimestamp
	}
	if srhree.PartitionID != nil {
		objectMap["PartitionId"] = srhree.PartitionID
	}
	if srhree.ReplicaID != nil {
		objectMap["ReplicaId"] = srhree.ReplicaID
	}
	if srhree.EventInstanceID != nil {
		objectMap["EventInstanceId"] = srhree.EventInstanceID
	}
	if srhree.Category != nil {
		objectMap["Category"] = srhree.Category
	}
	if srhree.TimeStamp != nil {
		objectMap["TimeStamp"] = srhree.TimeStamp
	}
	if srhree.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = srhree.HasCorrelatedEvents
	}
	if srhree.Kind != "" {
		objectMap["Kind"] = srhree.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return &srhree, true
}

// AsServiceEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return &srhree, true
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for StatelessReplicaHealthReportExpiredEvent.
func (srhree StatelessReplicaHealthReportExpiredEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &srhree, true
}

// StatelessReplicaNewHealthReportEvent stateless Replica Health Report Created event.
type StatelessReplicaNewHealthReportEvent struct {
	// SourceID - Id of report source.
	SourceID *string `json:"SourceId,omitempty"`
	// Property - Describes the property.
	Property *string `json:"Property,omitempty"`
	// HealthState - Describes the property health state.
	HealthState *string `json:"HealthState,omitempty"`
	// TimeToLiveMs - Time to live in milli-seconds.
	TimeToLiveMs *int64 `json:"TimeToLiveMs,omitempty"`
	// SequenceNumber - Sequence number of report.
	SequenceNumber *int64 `json:"SequenceNumber,omitempty"`
	// Description - Description of report.
	Description *string `json:"Description,omitempty"`
	// RemoveWhenExpired - Indicates the removal when it expires.
	RemoveWhenExpired *bool `json:"RemoveWhenExpired,omitempty"`
	// SourceUtcTimestamp - Source time.
	SourceUtcTimestamp *date.Time `json:"SourceUtcTimestamp,omitempty"`
	// PartitionID - An internal ID used by Service Fabric to uniquely identify a partition. This is a randomly generated GUID when the service was created. The partition ID is unique and does not change for the lifetime of the service. If the same service was deleted and recreated the IDs of its partitions would be different.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ReplicaID - Id of a stateful service replica. ReplicaId is used by Service Fabric to uniquely identify a replica of a partition. It is unique within a partition and does not change for the lifetime of the replica. If a replica gets dropped and another replica gets created on the same node for the same partition, it will get a different value for the id. Sometimes the id of a stateless service instance is also referred as a replica id.
	ReplicaID *int64 `json:"ReplicaId,omitempty"`
	// EventInstanceID - The identifier for the FabricEvent instance.
	EventInstanceID *uuid.UUID `json:"EventInstanceId,omitempty"`
	// Category - The category of event.
	Category *string `json:"Category,omitempty"`
	// TimeStamp - The time event was logged.
	TimeStamp *date.Time `json:"TimeStamp,omitempty"`
	// HasCorrelatedEvents - Shows there is existing related events available.
	HasCorrelatedEvents *bool `json:"HasCorrelatedEvents,omitempty"`
	// Kind - Possible values include: 'KindFabricEvent', 'KindApplicationEvent', 'KindClusterEvent', 'KindContainerInstanceEvent', 'KindNodeEvent', 'KindPartitionAnalysisEvent', 'KindPartitionEvent', 'KindReplicaEvent', 'KindServiceEvent', 'KindApplicationCreated', 'KindApplicationDeleted', 'KindApplicationNewHealthReport', 'KindApplicationHealthReportExpired', 'KindApplicationUpgradeCompleted', 'KindApplicationUpgradeDomainCompleted', 'KindApplicationUpgradeRollbackCompleted', 'KindApplicationUpgradeRollbackStarted', 'KindApplicationUpgradeStarted', 'KindDeployedApplicationNewHealthReport', 'KindDeployedApplicationHealthReportExpired', 'KindApplicationProcessExited', 'KindApplicationContainerInstanceExited', 'KindNodeAborted', 'KindNodeAddedToCluster', 'KindNodeClosed', 'KindNodeDeactivateCompleted', 'KindNodeDeactivateStarted', 'KindNodeDown', 'KindNodeNewHealthReport', 'KindNodeHealthReportExpired', 'KindNodeOpenSucceeded', 'KindNodeOpenFailed', 'KindNodeRemovedFromCluster', 'KindNodeUp', 'KindPartitionNewHealthReport', 'KindPartitionHealthReportExpired', 'KindPartitionReconfigured', 'KindPartitionPrimaryMoveAnalysis', 'KindServiceCreated', 'KindServiceDeleted', 'KindServiceNewHealthReport', 'KindServiceHealthReportExpired', 'KindDeployedServicePackageNewHealthReport', 'KindDeployedServicePackageHealthReportExpired', 'KindStatefulReplicaNewHealthReport', 'KindStatefulReplicaHealthReportExpired', 'KindStatelessReplicaNewHealthReport', 'KindStatelessReplicaHealthReportExpired', 'KindClusterNewHealthReport', 'KindClusterHealthReportExpired', 'KindClusterUpgradeCompleted', 'KindClusterUpgradeDomainCompleted', 'KindClusterUpgradeRollbackCompleted', 'KindClusterUpgradeRollbackStarted', 'KindClusterUpgradeStarted', 'KindChaosStopped', 'KindChaosStarted', 'KindChaosCodePackageRestartScheduled', 'KindChaosReplicaRemovalScheduled', 'KindChaosPartitionSecondaryMoveScheduled', 'KindChaosPartitionPrimaryMoveScheduled', 'KindChaosReplicaRestartScheduled', 'KindChaosNodeRestartScheduled'
	Kind KindBasicFabricEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) MarshalJSON() ([]byte, error) {
	srnhre.Kind = KindStatelessReplicaNewHealthReport
	objectMap := make(map[string]interface{})
	if srnhre.SourceID != nil {
		objectMap["SourceId"] = srnhre.SourceID
	}
	if srnhre.Property != nil {
		objectMap["Property"] = srnhre.Property
	}
	if srnhre.HealthState != nil {
		objectMap["HealthState"] = srnhre.HealthState
	}
	if srnhre.TimeToLiveMs != nil {
		objectMap["TimeToLiveMs"] = srnhre.TimeToLiveMs
	}
	if srnhre.SequenceNumber != nil {
		objectMap["SequenceNumber"] = srnhre.SequenceNumber
	}
	if srnhre.Description != nil {
		objectMap["Description"] = srnhre.Description
	}
	if srnhre.RemoveWhenExpired != nil {
		objectMap["RemoveWhenExpired"] = srnhre.RemoveWhenExpired
	}
	if srnhre.SourceUtcTimestamp != nil {
		objectMap["SourceUtcTimestamp"] = srnhre.SourceUtcTimestamp
	}
	if srnhre.PartitionID != nil {
		objectMap["PartitionId"] = srnhre.PartitionID
	}
	if srnhre.ReplicaID != nil {
		objectMap["ReplicaId"] = srnhre.ReplicaID
	}
	if srnhre.EventInstanceID != nil {
		objectMap["EventInstanceId"] = srnhre.EventInstanceID
	}
	if srnhre.Category != nil {
		objectMap["Category"] = srnhre.Category
	}
	if srnhre.TimeStamp != nil {
		objectMap["TimeStamp"] = srnhre.TimeStamp
	}
	if srnhre.HasCorrelatedEvents != nil {
		objectMap["HasCorrelatedEvents"] = srnhre.HasCorrelatedEvents
	}
	if srnhre.Kind != "" {
		objectMap["Kind"] = srnhre.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationEvent() (*ApplicationEvent, bool) {
	return nil, false
}

// AsBasicApplicationEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsBasicApplicationEvent() (BasicApplicationEvent, bool) {
	return nil, false
}

// AsClusterEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsClusterEvent() (*ClusterEvent, bool) {
	return nil, false
}

// AsBasicClusterEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsBasicClusterEvent() (BasicClusterEvent, bool) {
	return nil, false
}

// AsContainerInstanceEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsContainerInstanceEvent() (*ContainerInstanceEvent, bool) {
	return nil, false
}

// AsNodeEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeEvent() (*NodeEvent, bool) {
	return nil, false
}

// AsBasicNodeEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsBasicNodeEvent() (BasicNodeEvent, bool) {
	return nil, false
}

// AsPartitionAnalysisEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsPartitionAnalysisEvent() (*PartitionAnalysisEvent, bool) {
	return nil, false
}

// AsBasicPartitionAnalysisEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsBasicPartitionAnalysisEvent() (BasicPartitionAnalysisEvent, bool) {
	return nil, false
}

// AsPartitionEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsPartitionEvent() (*PartitionEvent, bool) {
	return nil, false
}

// AsBasicPartitionEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsBasicPartitionEvent() (BasicPartitionEvent, bool) {
	return nil, false
}

// AsReplicaEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsReplicaEvent() (*ReplicaEvent, bool) {
	return nil, false
}

// AsBasicReplicaEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsBasicReplicaEvent() (BasicReplicaEvent, bool) {
	return &srnhre, true
}

// AsServiceEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsServiceEvent() (*ServiceEvent, bool) {
	return nil, false
}

// AsBasicServiceEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsBasicServiceEvent() (BasicServiceEvent, bool) {
	return nil, false
}

// AsApplicationCreatedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationCreatedEvent() (*ApplicationCreatedEvent, bool) {
	return nil, false
}

// AsApplicationDeletedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationDeletedEvent() (*ApplicationDeletedEvent, bool) {
	return nil, false
}

// AsApplicationNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationNewHealthReportEvent() (*ApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationHealthReportExpiredEvent() (*ApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationUpgradeCompletedEvent() (*ApplicationUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationUpgradeDomainCompletedEvent() (*ApplicationUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationUpgradeRollbackCompletedEvent() (*ApplicationUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationUpgradeRollbackStartedEvent() (*ApplicationUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsApplicationUpgradeStartedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationUpgradeStartedEvent() (*ApplicationUpgradeStartedEvent, bool) {
	return nil, false
}

// AsDeployedApplicationNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsDeployedApplicationNewHealthReportEvent() (*DeployedApplicationNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedApplicationHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsDeployedApplicationHealthReportExpiredEvent() (*DeployedApplicationHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsApplicationProcessExitedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationProcessExitedEvent() (*ApplicationProcessExitedEvent, bool) {
	return nil, false
}

// AsApplicationContainerInstanceExitedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsApplicationContainerInstanceExitedEvent() (*ApplicationContainerInstanceExitedEvent, bool) {
	return nil, false
}

// AsNodeAbortedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeAbortedEvent() (*NodeAbortedEvent, bool) {
	return nil, false
}

// AsNodeAddedToClusterEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeAddedToClusterEvent() (*NodeAddedToClusterEvent, bool) {
	return nil, false
}

// AsNodeClosedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeClosedEvent() (*NodeClosedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeDeactivateCompletedEvent() (*NodeDeactivateCompletedEvent, bool) {
	return nil, false
}

// AsNodeDeactivateStartedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeDeactivateStartedEvent() (*NodeDeactivateStartedEvent, bool) {
	return nil, false
}

// AsNodeDownEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeDownEvent() (*NodeDownEvent, bool) {
	return nil, false
}

// AsNodeNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeNewHealthReportEvent() (*NodeNewHealthReportEvent, bool) {
	return nil, false
}

// AsNodeHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeHealthReportExpiredEvent() (*NodeHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsNodeOpenSucceededEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeOpenSucceededEvent() (*NodeOpenSucceededEvent, bool) {
	return nil, false
}

// AsNodeOpenFailedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeOpenFailedEvent() (*NodeOpenFailedEvent, bool) {
	return nil, false
}

// AsNodeRemovedFromClusterEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeRemovedFromClusterEvent() (*NodeRemovedFromClusterEvent, bool) {
	return nil, false
}

// AsNodeUpEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsNodeUpEvent() (*NodeUpEvent, bool) {
	return nil, false
}

// AsPartitionNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsPartitionNewHealthReportEvent() (*PartitionNewHealthReportEvent, bool) {
	return nil, false
}

// AsPartitionHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsPartitionHealthReportExpiredEvent() (*PartitionHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsPartitionReconfiguredEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsPartitionReconfiguredEvent() (*PartitionReconfiguredEvent, bool) {
	return nil, false
}

// AsPartitionPrimaryMoveAnalysisEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsPartitionPrimaryMoveAnalysisEvent() (*PartitionPrimaryMoveAnalysisEvent, bool) {
	return nil, false
}

// AsServiceCreatedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsServiceCreatedEvent() (*ServiceCreatedEvent, bool) {
	return nil, false
}

// AsServiceDeletedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsServiceDeletedEvent() (*ServiceDeletedEvent, bool) {
	return nil, false
}

// AsServiceNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsServiceNewHealthReportEvent() (*ServiceNewHealthReportEvent, bool) {
	return nil, false
}

// AsServiceHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsServiceHealthReportExpiredEvent() (*ServiceHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsDeployedServicePackageNewHealthReportEvent() (*DeployedServicePackageNewHealthReportEvent, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsDeployedServicePackageHealthReportExpiredEvent() (*DeployedServicePackageHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatefulReplicaNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsStatefulReplicaNewHealthReportEvent() (*StatefulReplicaNewHealthReportEvent, bool) {
	return nil, false
}

// AsStatefulReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsStatefulReplicaHealthReportExpiredEvent() (*StatefulReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsStatelessReplicaNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsStatelessReplicaNewHealthReportEvent() (*StatelessReplicaNewHealthReportEvent, bool) {
	return &srnhre, true
}

// AsStatelessReplicaHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsStatelessReplicaHealthReportExpiredEvent() (*StatelessReplicaHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterNewHealthReportEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsClusterNewHealthReportEvent() (*ClusterNewHealthReportEvent, bool) {
	return nil, false
}

// AsClusterHealthReportExpiredEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsClusterHealthReportExpiredEvent() (*ClusterHealthReportExpiredEvent, bool) {
	return nil, false
}

// AsClusterUpgradeCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsClusterUpgradeCompletedEvent() (*ClusterUpgradeCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeDomainCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsClusterUpgradeDomainCompletedEvent() (*ClusterUpgradeDomainCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackCompletedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsClusterUpgradeRollbackCompletedEvent() (*ClusterUpgradeRollbackCompletedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeRollbackStartedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsClusterUpgradeRollbackStartedEvent() (*ClusterUpgradeRollbackStartedEvent, bool) {
	return nil, false
}

// AsClusterUpgradeStartedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsClusterUpgradeStartedEvent() (*ClusterUpgradeStartedEvent, bool) {
	return nil, false
}

// AsChaosStoppedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsChaosStoppedEvent() (*ChaosStoppedEvent, bool) {
	return nil, false
}

// AsChaosStartedEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsChaosStartedEvent() (*ChaosStartedEvent, bool) {
	return nil, false
}

// AsChaosCodePackageRestartScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsChaosCodePackageRestartScheduledEvent() (*ChaosCodePackageRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRemovalScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsChaosReplicaRemovalScheduledEvent() (*ChaosReplicaRemovalScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionSecondaryMoveScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsChaosPartitionSecondaryMoveScheduledEvent() (*ChaosPartitionSecondaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosPartitionPrimaryMoveScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsChaosPartitionPrimaryMoveScheduledEvent() (*ChaosPartitionPrimaryMoveScheduledEvent, bool) {
	return nil, false
}

// AsChaosReplicaRestartScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsChaosReplicaRestartScheduledEvent() (*ChaosReplicaRestartScheduledEvent, bool) {
	return nil, false
}

// AsChaosNodeRestartScheduledEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsChaosNodeRestartScheduledEvent() (*ChaosNodeRestartScheduledEvent, bool) {
	return nil, false
}

// AsFabricEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsFabricEvent() (*FabricEvent, bool) {
	return nil, false
}

// AsBasicFabricEvent is the BasicFabricEvent implementation for StatelessReplicaNewHealthReportEvent.
func (srnhre StatelessReplicaNewHealthReportEvent) AsBasicFabricEvent() (BasicFabricEvent, bool) {
	return &srnhre, true
}

// StatelessServiceDescription describes a stateless service.
type StatelessServiceDescription struct {
	// InstanceCount - The instance count.
	InstanceCount *int32 `json:"InstanceCount,omitempty"`
	// ApplicationName - The name of the application, including the 'fabric:' URI scheme.
	ApplicationName *string `json:"ApplicationName,omitempty"`
	// ServiceName - The full name of the service with 'fabric:' URI scheme.
	ServiceName *string `json:"ServiceName,omitempty"`
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// InitializationData - The initialization data as an array of bytes. Initialization data is passed to service instances or replicas when they are created.
	InitializationData *[]int32 `json:"InitializationData,omitempty"`
	// PartitionDescription - The partition description as an object.
	PartitionDescription BasicPartitionSchemeDescription `json:"PartitionDescription,omitempty"`
	// PlacementConstraints - The placement constraints as a string. Placement constraints are boolean expressions on node properties and allow for restricting a service to particular nodes based on the service requirements. For example, to place a service on nodes where NodeType is blue specify the following: "NodeColor == blue)".
	PlacementConstraints *string `json:"PlacementConstraints,omitempty"`
	// CorrelationScheme - The correlation scheme.
	CorrelationScheme *[]ServiceCorrelationDescription `json:"CorrelationScheme,omitempty"`
	// ServiceLoadMetrics - The service load metrics.
	ServiceLoadMetrics *[]ServiceLoadMetricDescription `json:"ServiceLoadMetrics,omitempty"`
	// ServicePlacementPolicies - The service placement policies.
	ServicePlacementPolicies *[]BasicServicePlacementPolicyDescription `json:"ServicePlacementPolicies,omitempty"`
	// DefaultMoveCost - The move cost for the service. Possible values include: 'Zero', 'Low', 'Medium', 'High'
	DefaultMoveCost MoveCost `json:"DefaultMoveCost,omitempty"`
	// IsDefaultMoveCostSpecified - Indicates if the DefaultMoveCost property is specified.
	IsDefaultMoveCostSpecified *bool `json:"IsDefaultMoveCostSpecified,omitempty"`
	// ServicePackageActivationMode - The activation mode of service package to be used for a service. Possible values include: 'SharedProcess', 'ExclusiveProcess'
	ServicePackageActivationMode ServicePackageActivationMode `json:"ServicePackageActivationMode,omitempty"`
	// ServiceDNSName - The DNS name of the service. It requires the DNS system service to be enabled in Service Fabric cluster.
	ServiceDNSName *string `json:"ServiceDnsName,omitempty"`
	// ScalingPolicies - Scaling policies for this service.
	ScalingPolicies *[]ScalingPolicyDescription `json:"ScalingPolicies,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServiceDescriptionServiceKindServiceDescription', 'ServiceKindBasicServiceDescriptionServiceKindStateful', 'ServiceKindBasicServiceDescriptionServiceKindStateless'
	ServiceKind ServiceKindBasicServiceDescription `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatelessServiceDescription.
func (ssd StatelessServiceDescription) MarshalJSON() ([]byte, error) {
	ssd.ServiceKind = ServiceKindBasicServiceDescriptionServiceKindStateless
	objectMap := make(map[string]interface{})
	if ssd.InstanceCount != nil {
		objectMap["InstanceCount"] = ssd.InstanceCount
	}
	if ssd.ApplicationName != nil {
		objectMap["ApplicationName"] = ssd.ApplicationName
	}
	if ssd.ServiceName != nil {
		objectMap["ServiceName"] = ssd.ServiceName
	}
	if ssd.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = ssd.ServiceTypeName
	}
	if ssd.InitializationData != nil {
		objectMap["InitializationData"] = ssd.InitializationData
	}
	objectMap["PartitionDescription"] = ssd.PartitionDescription
	if ssd.PlacementConstraints != nil {
		objectMap["PlacementConstraints"] = ssd.PlacementConstraints
	}
	if ssd.CorrelationScheme != nil {
		objectMap["CorrelationScheme"] = ssd.CorrelationScheme
	}
	if ssd.ServiceLoadMetrics != nil {
		objectMap["ServiceLoadMetrics"] = ssd.ServiceLoadMetrics
	}
	if ssd.ServicePlacementPolicies != nil {
		objectMap["ServicePlacementPolicies"] = ssd.ServicePlacementPolicies
	}
	if ssd.DefaultMoveCost != "" {
		objectMap["DefaultMoveCost"] = ssd.DefaultMoveCost
	}
	if ssd.IsDefaultMoveCostSpecified != nil {
		objectMap["IsDefaultMoveCostSpecified"] = ssd.IsDefaultMoveCostSpecified
	}
	if ssd.ServicePackageActivationMode != "" {
		objectMap["ServicePackageActivationMode"] = ssd.ServicePackageActivationMode
	}
	if ssd.ServiceDNSName != nil {
		objectMap["ServiceDnsName"] = ssd.ServiceDNSName
	}
	if ssd.ScalingPolicies != nil {
		objectMap["ScalingPolicies"] = ssd.ScalingPolicies
	}
	if ssd.ServiceKind != "" {
		objectMap["ServiceKind"] = ssd.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceDescription is the BasicServiceDescription implementation for StatelessServiceDescription.
func (ssd StatelessServiceDescription) AsStatefulServiceDescription() (*StatefulServiceDescription, bool) {
	return nil, false
}

// AsStatelessServiceDescription is the BasicServiceDescription implementation for StatelessServiceDescription.
func (ssd StatelessServiceDescription) AsStatelessServiceDescription() (*StatelessServiceDescription, bool) {
	return &ssd, true
}

// AsServiceDescription is the BasicServiceDescription implementation for StatelessServiceDescription.
func (ssd StatelessServiceDescription) AsServiceDescription() (*ServiceDescription, bool) {
	return nil, false
}

// AsBasicServiceDescription is the BasicServiceDescription implementation for StatelessServiceDescription.
func (ssd StatelessServiceDescription) AsBasicServiceDescription() (BasicServiceDescription, bool) {
	return &ssd, true
}

// UnmarshalJSON is the custom unmarshaler for StatelessServiceDescription struct.
func (ssd *StatelessServiceDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "InstanceCount":
			if v != nil {
				var instanceCount int32
				err = json.Unmarshal(*v, &instanceCount)
				if err != nil {
					return err
				}
				ssd.InstanceCount = &instanceCount
			}
		case "ApplicationName":
			if v != nil {
				var applicationName string
				err = json.Unmarshal(*v, &applicationName)
				if err != nil {
					return err
				}
				ssd.ApplicationName = &applicationName
			}
		case "ServiceName":
			if v != nil {
				var serviceName string
				err = json.Unmarshal(*v, &serviceName)
				if err != nil {
					return err
				}
				ssd.ServiceName = &serviceName
			}
		case "ServiceTypeName":
			if v != nil {
				var serviceTypeName string
				err = json.Unmarshal(*v, &serviceTypeName)
				if err != nil {
					return err
				}
				ssd.ServiceTypeName = &serviceTypeName
			}
		case "InitializationData":
			if v != nil {
				var initializationData []int32
				err = json.Unmarshal(*v, &initializationData)
				if err != nil {
					return err
				}
				ssd.InitializationData = &initializationData
			}
		case "PartitionDescription":
			if v != nil {
				partitionDescription, err := unmarshalBasicPartitionSchemeDescription(*v)
				if err != nil {
					return err
				}
				ssd.PartitionDescription = partitionDescription
			}
		case "PlacementConstraints":
			if v != nil {
				var placementConstraints string
				err = json.Unmarshal(*v, &placementConstraints)
				if err != nil {
					return err
				}
				ssd.PlacementConstraints = &placementConstraints
			}
		case "CorrelationScheme":
			if v != nil {
				var correlationScheme []ServiceCorrelationDescription
				err = json.Unmarshal(*v, &correlationScheme)
				if err != nil {
					return err
				}
				ssd.CorrelationScheme = &correlationScheme
			}
		case "ServiceLoadMetrics":
			if v != nil {
				var serviceLoadMetrics []ServiceLoadMetricDescription
				err = json.Unmarshal(*v, &serviceLoadMetrics)
				if err != nil {
					return err
				}
				ssd.ServiceLoadMetrics = &serviceLoadMetrics
			}
		case "ServicePlacementPolicies":
			if v != nil {
				servicePlacementPolicies, err := unmarshalBasicServicePlacementPolicyDescriptionArray(*v)
				if err != nil {
					return err
				}
				ssd.ServicePlacementPolicies = &servicePlacementPolicies
			}
		case "DefaultMoveCost":
			if v != nil {
				var defaultMoveCost MoveCost
				err = json.Unmarshal(*v, &defaultMoveCost)
				if err != nil {
					return err
				}
				ssd.DefaultMoveCost = defaultMoveCost
			}
		case "IsDefaultMoveCostSpecified":
			if v != nil {
				var isDefaultMoveCostSpecified bool
				err = json.Unmarshal(*v, &isDefaultMoveCostSpecified)
				if err != nil {
					return err
				}
				ssd.IsDefaultMoveCostSpecified = &isDefaultMoveCostSpecified
			}
		case "ServicePackageActivationMode":
			if v != nil {
				var servicePackageActivationMode ServicePackageActivationMode
				err = json.Unmarshal(*v, &servicePackageActivationMode)
				if err != nil {
					return err
				}
				ssd.ServicePackageActivationMode = servicePackageActivationMode
			}
		case "ServiceDnsName":
			if v != nil {
				var serviceDNSName string
				err = json.Unmarshal(*v, &serviceDNSName)
				if err != nil {
					return err
				}
				ssd.ServiceDNSName = &serviceDNSName
			}
		case "ScalingPolicies":
			if v != nil {
				var scalingPolicies []ScalingPolicyDescription
				err = json.Unmarshal(*v, &scalingPolicies)
				if err != nil {
					return err
				}
				ssd.ScalingPolicies = &scalingPolicies
			}
		case "ServiceKind":
			if v != nil {
				var serviceKind ServiceKindBasicServiceDescription
				err = json.Unmarshal(*v, &serviceKind)
				if err != nil {
					return err
				}
				ssd.ServiceKind = serviceKind
			}
		}
	}

	return nil
}

// StatelessServiceInfo information about a stateless Service Fabric service.
type StatelessServiceInfo struct {
	// ID - The identity of the service. This ID is an encoded representation of the service name. This is used in the REST APIs to identify the service resource.
	// Starting in version 6.0, hierarchical names are delimited with the "\~" character. For example, if the service name is "fabric:/myapp/app1/svc1",
	// the service identity would be "myapp~app1\~svc1" in 6.0+ and "myapp/app1/svc1" in previous versions.
	ID *string `json:"Id,omitempty"`
	// Name - The full name of the service with 'fabric:' URI scheme.
	Name *string `json:"Name,omitempty"`
	// TypeName - Name of the service type as specified in the service manifest.
	TypeName *string `json:"TypeName,omitempty"`
	// ManifestVersion - The version of the service manifest.
	ManifestVersion *string `json:"ManifestVersion,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// ServiceStatus - The status of the application. Possible values include: 'ServiceStatusUnknown', 'ServiceStatusActive', 'ServiceStatusUpgrading', 'ServiceStatusDeleting', 'ServiceStatusCreating', 'ServiceStatusFailed'
	ServiceStatus ServiceStatus `json:"ServiceStatus,omitempty"`
	// IsServiceGroup - Whether the service is in a service group.
	IsServiceGroup *bool `json:"IsServiceGroup,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServiceInfoServiceKindServiceInfo', 'ServiceKindBasicServiceInfoServiceKindStateful', 'ServiceKindBasicServiceInfoServiceKindStateless'
	ServiceKind ServiceKindBasicServiceInfo `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatelessServiceInfo.
func (ssi StatelessServiceInfo) MarshalJSON() ([]byte, error) {
	ssi.ServiceKind = ServiceKindBasicServiceInfoServiceKindStateless
	objectMap := make(map[string]interface{})
	if ssi.ID != nil {
		objectMap["Id"] = ssi.ID
	}
	if ssi.Name != nil {
		objectMap["Name"] = ssi.Name
	}
	if ssi.TypeName != nil {
		objectMap["TypeName"] = ssi.TypeName
	}
	if ssi.ManifestVersion != nil {
		objectMap["ManifestVersion"] = ssi.ManifestVersion
	}
	if ssi.HealthState != "" {
		objectMap["HealthState"] = ssi.HealthState
	}
	if ssi.ServiceStatus != "" {
		objectMap["ServiceStatus"] = ssi.ServiceStatus
	}
	if ssi.IsServiceGroup != nil {
		objectMap["IsServiceGroup"] = ssi.IsServiceGroup
	}
	if ssi.ServiceKind != "" {
		objectMap["ServiceKind"] = ssi.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceInfo is the BasicServiceInfo implementation for StatelessServiceInfo.
func (ssi StatelessServiceInfo) AsStatefulServiceInfo() (*StatefulServiceInfo, bool) {
	return nil, false
}

// AsStatelessServiceInfo is the BasicServiceInfo implementation for StatelessServiceInfo.
func (ssi StatelessServiceInfo) AsStatelessServiceInfo() (*StatelessServiceInfo, bool) {
	return &ssi, true
}

// AsServiceInfo is the BasicServiceInfo implementation for StatelessServiceInfo.
func (ssi StatelessServiceInfo) AsServiceInfo() (*ServiceInfo, bool) {
	return nil, false
}

// AsBasicServiceInfo is the BasicServiceInfo implementation for StatelessServiceInfo.
func (ssi StatelessServiceInfo) AsBasicServiceInfo() (BasicServiceInfo, bool) {
	return &ssi, true
}

// StatelessServiceInstanceHealth represents the health of the stateless service instance.
// Contains the instance aggregated health state, the health events and the unhealthy evaluations.
type StatelessServiceInstanceHealth struct {
	// InstanceID - Id of a stateless service instance. InstanceId is used by Service Fabric to uniquely identify an instance of a partition of a stateless service. It is unique within a partition and does not change for the lifetime of the instance. If the instance has failed over on the same or different node, it will get a different value for the InstanceId.
	InstanceID *string `json:"InstanceId,omitempty"`
	// PartitionID - Id of the partition to which this replica belongs.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicReplicaHealthServiceKindReplicaHealth', 'ServiceKindBasicReplicaHealthServiceKindStateful', 'ServiceKindBasicReplicaHealthServiceKindStateless'
	ServiceKind ServiceKindBasicReplicaHealth `json:"ServiceKind,omitempty"`
	// AggregatedHealthState - The HealthState representing the aggregated health state of the entity computed by Health Manager.
	// The health evaluation of the entity reflects all events reported on the entity and its children (if any).
	// The aggregation is done by applying the desired health policy. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// HealthEvents - The list of health events reported on the entity.
	HealthEvents *[]HealthEvent `json:"HealthEvents,omitempty"`
	// UnhealthyEvaluations - The unhealthy evaluations that show why the current aggregated health state was returned by Health Manager.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// HealthStatistics - Shows the health statistics for all children types of the queried entity.
	HealthStatistics *HealthStatistics `json:"HealthStatistics,omitempty"`
}

// MarshalJSON is the custom marshaler for StatelessServiceInstanceHealth.
func (ssih StatelessServiceInstanceHealth) MarshalJSON() ([]byte, error) {
	ssih.ServiceKind = ServiceKindBasicReplicaHealthServiceKindStateless
	objectMap := make(map[string]interface{})
	if ssih.InstanceID != nil {
		objectMap["InstanceId"] = ssih.InstanceID
	}
	if ssih.PartitionID != nil {
		objectMap["PartitionId"] = ssih.PartitionID
	}
	if ssih.ServiceKind != "" {
		objectMap["ServiceKind"] = ssih.ServiceKind
	}
	if ssih.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = ssih.AggregatedHealthState
	}
	if ssih.HealthEvents != nil {
		objectMap["HealthEvents"] = ssih.HealthEvents
	}
	if ssih.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = ssih.UnhealthyEvaluations
	}
	if ssih.HealthStatistics != nil {
		objectMap["HealthStatistics"] = ssih.HealthStatistics
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceReplicaHealth is the BasicReplicaHealth implementation for StatelessServiceInstanceHealth.
func (ssih StatelessServiceInstanceHealth) AsStatefulServiceReplicaHealth() (*StatefulServiceReplicaHealth, bool) {
	return nil, false
}

// AsStatelessServiceInstanceHealth is the BasicReplicaHealth implementation for StatelessServiceInstanceHealth.
func (ssih StatelessServiceInstanceHealth) AsStatelessServiceInstanceHealth() (*StatelessServiceInstanceHealth, bool) {
	return &ssih, true
}

// AsReplicaHealth is the BasicReplicaHealth implementation for StatelessServiceInstanceHealth.
func (ssih StatelessServiceInstanceHealth) AsReplicaHealth() (*ReplicaHealth, bool) {
	return nil, false
}

// AsBasicReplicaHealth is the BasicReplicaHealth implementation for StatelessServiceInstanceHealth.
func (ssih StatelessServiceInstanceHealth) AsBasicReplicaHealth() (BasicReplicaHealth, bool) {
	return &ssih, true
}

// StatelessServiceInstanceHealthState represents the health state of the stateless service instance, which
// contains the instance ID and the aggregated health state.
type StatelessServiceInstanceHealthState struct {
	// ReplicaID - Id of the stateless service instance on the wire this field is called ReplicaId.
	ReplicaID *string `json:"ReplicaId,omitempty"`
	// PartitionID - The ID of the partition to which this replica belongs.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicReplicaHealthStateServiceKindReplicaHealthState', 'ServiceKindBasicReplicaHealthStateServiceKindStateful', 'ServiceKindBasicReplicaHealthStateServiceKindStateless'
	ServiceKind ServiceKindBasicReplicaHealthState `json:"ServiceKind,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
}

// MarshalJSON is the custom marshaler for StatelessServiceInstanceHealthState.
func (ssihs StatelessServiceInstanceHealthState) MarshalJSON() ([]byte, error) {
	ssihs.ServiceKind = ServiceKindBasicReplicaHealthStateServiceKindStateless
	objectMap := make(map[string]interface{})
	if ssihs.ReplicaID != nil {
		objectMap["ReplicaId"] = ssihs.ReplicaID
	}
	if ssihs.PartitionID != nil {
		objectMap["PartitionId"] = ssihs.PartitionID
	}
	if ssihs.ServiceKind != "" {
		objectMap["ServiceKind"] = ssihs.ServiceKind
	}
	if ssihs.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = ssihs.AggregatedHealthState
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceReplicaHealthState is the BasicReplicaHealthState implementation for StatelessServiceInstanceHealthState.
func (ssihs StatelessServiceInstanceHealthState) AsStatefulServiceReplicaHealthState() (*StatefulServiceReplicaHealthState, bool) {
	return nil, false
}

// AsStatelessServiceInstanceHealthState is the BasicReplicaHealthState implementation for StatelessServiceInstanceHealthState.
func (ssihs StatelessServiceInstanceHealthState) AsStatelessServiceInstanceHealthState() (*StatelessServiceInstanceHealthState, bool) {
	return &ssihs, true
}

// AsReplicaHealthState is the BasicReplicaHealthState implementation for StatelessServiceInstanceHealthState.
func (ssihs StatelessServiceInstanceHealthState) AsReplicaHealthState() (*ReplicaHealthState, bool) {
	return nil, false
}

// AsBasicReplicaHealthState is the BasicReplicaHealthState implementation for StatelessServiceInstanceHealthState.
func (ssihs StatelessServiceInstanceHealthState) AsBasicReplicaHealthState() (BasicReplicaHealthState, bool) {
	return &ssihs, true
}

// StatelessServiceInstanceInfo represents a stateless service instance. This includes information about
// the identity, status, health, node name, uptime, and other details about the instance.
type StatelessServiceInstanceInfo struct {
	// InstanceID - Id of a stateless service instance. InstanceId is used by Service Fabric to uniquely identify an instance of a partition of a stateless service. It is unique within a partition and does not change for the lifetime of the instance. If the instance has failed over on the same or different node, it will get a different value for the InstanceId.
	InstanceID *string `json:"InstanceId,omitempty"`
	// ReplicaStatus - The status of a replica of a service. Possible values include: 'ReplicaStatusInvalid', 'ReplicaStatusInBuild', 'ReplicaStatusStandby', 'ReplicaStatusReady', 'ReplicaStatusDown', 'ReplicaStatusDropped'
	ReplicaStatus ReplicaStatus `json:"ReplicaStatus,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// NodeName - The name of a Service Fabric node.
	NodeName *string `json:"NodeName,omitempty"`
	// Address - The address the replica is listening on.
	Address *string `json:"Address,omitempty"`
	// LastInBuildDurationInSeconds - The last in build duration of the replica in seconds.
	LastInBuildDurationInSeconds *string `json:"LastInBuildDurationInSeconds,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicReplicaInfoServiceKindReplicaInfo', 'ServiceKindBasicReplicaInfoServiceKindStateful', 'ServiceKindBasicReplicaInfoServiceKindStateless'
	ServiceKind ServiceKindBasicReplicaInfo `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatelessServiceInstanceInfo.
func (ssii StatelessServiceInstanceInfo) MarshalJSON() ([]byte, error) {
	ssii.ServiceKind = ServiceKindBasicReplicaInfoServiceKindStateless
	objectMap := make(map[string]interface{})
	if ssii.InstanceID != nil {
		objectMap["InstanceId"] = ssii.InstanceID
	}
	if ssii.ReplicaStatus != "" {
		objectMap["ReplicaStatus"] = ssii.ReplicaStatus
	}
	if ssii.HealthState != "" {
		objectMap["HealthState"] = ssii.HealthState
	}
	if ssii.NodeName != nil {
		objectMap["NodeName"] = ssii.NodeName
	}
	if ssii.Address != nil {
		objectMap["Address"] = ssii.Address
	}
	if ssii.LastInBuildDurationInSeconds != nil {
		objectMap["LastInBuildDurationInSeconds"] = ssii.LastInBuildDurationInSeconds
	}
	if ssii.ServiceKind != "" {
		objectMap["ServiceKind"] = ssii.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceReplicaInfo is the BasicReplicaInfo implementation for StatelessServiceInstanceInfo.
func (ssii StatelessServiceInstanceInfo) AsStatefulServiceReplicaInfo() (*StatefulServiceReplicaInfo, bool) {
	return nil, false
}

// AsStatelessServiceInstanceInfo is the BasicReplicaInfo implementation for StatelessServiceInstanceInfo.
func (ssii StatelessServiceInstanceInfo) AsStatelessServiceInstanceInfo() (*StatelessServiceInstanceInfo, bool) {
	return &ssii, true
}

// AsReplicaInfo is the BasicReplicaInfo implementation for StatelessServiceInstanceInfo.
func (ssii StatelessServiceInstanceInfo) AsReplicaInfo() (*ReplicaInfo, bool) {
	return nil, false
}

// AsBasicReplicaInfo is the BasicReplicaInfo implementation for StatelessServiceInstanceInfo.
func (ssii StatelessServiceInstanceInfo) AsBasicReplicaInfo() (BasicReplicaInfo, bool) {
	return &ssii, true
}

// StatelessServicePartitionInfo information about a partition of a stateless Service Fabric service.
type StatelessServicePartitionInfo struct {
	// InstanceCount - Number of instances of this partition.
	InstanceCount *int64 `json:"InstanceCount,omitempty"`
	// HealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	HealthState HealthState `json:"HealthState,omitempty"`
	// PartitionStatus - The status of the service fabric service partition. Possible values include: 'ServicePartitionStatusInvalid', 'ServicePartitionStatusReady', 'ServicePartitionStatusNotReady', 'ServicePartitionStatusInQuorumLoss', 'ServicePartitionStatusReconfiguring', 'ServicePartitionStatusDeleting'
	PartitionStatus ServicePartitionStatus `json:"PartitionStatus,omitempty"`
	// PartitionInformation - Information about the partition identity, partitioning scheme and keys supported by it.
	PartitionInformation BasicPartitionInformation `json:"PartitionInformation,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServicePartitionInfoServiceKindServicePartitionInfo', 'ServiceKindBasicServicePartitionInfoServiceKindStateful', 'ServiceKindBasicServicePartitionInfoServiceKindStateless'
	ServiceKind ServiceKindBasicServicePartitionInfo `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatelessServicePartitionInfo.
func (sspi StatelessServicePartitionInfo) MarshalJSON() ([]byte, error) {
	sspi.ServiceKind = ServiceKindBasicServicePartitionInfoServiceKindStateless
	objectMap := make(map[string]interface{})
	if sspi.InstanceCount != nil {
		objectMap["InstanceCount"] = sspi.InstanceCount
	}
	if sspi.HealthState != "" {
		objectMap["HealthState"] = sspi.HealthState
	}
	if sspi.PartitionStatus != "" {
		objectMap["PartitionStatus"] = sspi.PartitionStatus
	}
	objectMap["PartitionInformation"] = sspi.PartitionInformation
	if sspi.ServiceKind != "" {
		objectMap["ServiceKind"] = sspi.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServicePartitionInfo is the BasicServicePartitionInfo implementation for StatelessServicePartitionInfo.
func (sspi StatelessServicePartitionInfo) AsStatefulServicePartitionInfo() (*StatefulServicePartitionInfo, bool) {
	return nil, false
}

// AsStatelessServicePartitionInfo is the BasicServicePartitionInfo implementation for StatelessServicePartitionInfo.
func (sspi StatelessServicePartitionInfo) AsStatelessServicePartitionInfo() (*StatelessServicePartitionInfo, bool) {
	return &sspi, true
}

// AsServicePartitionInfo is the BasicServicePartitionInfo implementation for StatelessServicePartitionInfo.
func (sspi StatelessServicePartitionInfo) AsServicePartitionInfo() (*ServicePartitionInfo, bool) {
	return nil, false
}

// AsBasicServicePartitionInfo is the BasicServicePartitionInfo implementation for StatelessServicePartitionInfo.
func (sspi StatelessServicePartitionInfo) AsBasicServicePartitionInfo() (BasicServicePartitionInfo, bool) {
	return &sspi, true
}

// UnmarshalJSON is the custom unmarshaler for StatelessServicePartitionInfo struct.
func (sspi *StatelessServicePartitionInfo) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "InstanceCount":
			if v != nil {
				var instanceCount int64
				err = json.Unmarshal(*v, &instanceCount)
				if err != nil {
					return err
				}
				sspi.InstanceCount = &instanceCount
			}
		case "HealthState":
			if v != nil {
				var healthState HealthState
				err = json.Unmarshal(*v, &healthState)
				if err != nil {
					return err
				}
				sspi.HealthState = healthState
			}
		case "PartitionStatus":
			if v != nil {
				var partitionStatus ServicePartitionStatus
				err = json.Unmarshal(*v, &partitionStatus)
				if err != nil {
					return err
				}
				sspi.PartitionStatus = partitionStatus
			}
		case "PartitionInformation":
			if v != nil {
				partitionInformation, err := unmarshalBasicPartitionInformation(*v)
				if err != nil {
					return err
				}
				sspi.PartitionInformation = partitionInformation
			}
		case "ServiceKind":
			if v != nil {
				var serviceKind ServiceKindBasicServicePartitionInfo
				err = json.Unmarshal(*v, &serviceKind)
				if err != nil {
					return err
				}
				sspi.ServiceKind = serviceKind
			}
		}
	}

	return nil
}

// StatelessServiceTypeDescription describes a stateless service type defined in the service manifest of a
// provisioned application type.
type StatelessServiceTypeDescription struct {
	// UseImplicitHost - A flag indicating if this type is not implemented and hosted by a user service process, but is implicitly hosted by a system created process. This value is true for services using the guest executable services, false otherwise.
	UseImplicitHost *bool `json:"UseImplicitHost,omitempty"`
	// IsStateful - Indicates whether the service type is a stateful service type or a stateless service type. This property is true if the service type is a stateful service type, false otherwise.
	IsStateful *bool `json:"IsStateful,omitempty"`
	// ServiceTypeName - Name of the service type as specified in the service manifest.
	ServiceTypeName *string `json:"ServiceTypeName,omitempty"`
	// PlacementConstraints - The placement constraint to be used when instantiating this service in a Service Fabric cluster.
	PlacementConstraints *string `json:"PlacementConstraints,omitempty"`
	// LoadMetrics - The service load metrics is given as an array of ServiceLoadMetricDescription objects.
	LoadMetrics *[]ServiceLoadMetricDescription `json:"LoadMetrics,omitempty"`
	// ServicePlacementPolicies - List of service placement policy descriptions.
	ServicePlacementPolicies *[]BasicServicePlacementPolicyDescription `json:"ServicePlacementPolicies,omitempty"`
	// Extensions - List of service type extensions.
	Extensions *[]ServiceTypeExtensionDescription `json:"Extensions,omitempty"`
	// Kind - Possible values include: 'KindServiceTypeDescription', 'KindStateful', 'KindStateless'
	Kind KindBasicServiceTypeDescription `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatelessServiceTypeDescription.
func (sstd StatelessServiceTypeDescription) MarshalJSON() ([]byte, error) {
	sstd.Kind = KindStateless
	objectMap := make(map[string]interface{})
	if sstd.UseImplicitHost != nil {
		objectMap["UseImplicitHost"] = sstd.UseImplicitHost
	}
	if sstd.IsStateful != nil {
		objectMap["IsStateful"] = sstd.IsStateful
	}
	if sstd.ServiceTypeName != nil {
		objectMap["ServiceTypeName"] = sstd.ServiceTypeName
	}
	if sstd.PlacementConstraints != nil {
		objectMap["PlacementConstraints"] = sstd.PlacementConstraints
	}
	if sstd.LoadMetrics != nil {
		objectMap["LoadMetrics"] = sstd.LoadMetrics
	}
	if sstd.ServicePlacementPolicies != nil {
		objectMap["ServicePlacementPolicies"] = sstd.ServicePlacementPolicies
	}
	if sstd.Extensions != nil {
		objectMap["Extensions"] = sstd.Extensions
	}
	if sstd.Kind != "" {
		objectMap["Kind"] = sstd.Kind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceTypeDescription is the BasicServiceTypeDescription implementation for StatelessServiceTypeDescription.
func (sstd StatelessServiceTypeDescription) AsStatefulServiceTypeDescription() (*StatefulServiceTypeDescription, bool) {
	return nil, false
}

// AsStatelessServiceTypeDescription is the BasicServiceTypeDescription implementation for StatelessServiceTypeDescription.
func (sstd StatelessServiceTypeDescription) AsStatelessServiceTypeDescription() (*StatelessServiceTypeDescription, bool) {
	return &sstd, true
}

// AsServiceTypeDescription is the BasicServiceTypeDescription implementation for StatelessServiceTypeDescription.
func (sstd StatelessServiceTypeDescription) AsServiceTypeDescription() (*ServiceTypeDescription, bool) {
	return nil, false
}

// AsBasicServiceTypeDescription is the BasicServiceTypeDescription implementation for StatelessServiceTypeDescription.
func (sstd StatelessServiceTypeDescription) AsBasicServiceTypeDescription() (BasicServiceTypeDescription, bool) {
	return &sstd, true
}

// UnmarshalJSON is the custom unmarshaler for StatelessServiceTypeDescription struct.
func (sstd *StatelessServiceTypeDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "UseImplicitHost":
			if v != nil {
				var useImplicitHost bool
				err = json.Unmarshal(*v, &useImplicitHost)
				if err != nil {
					return err
				}
				sstd.UseImplicitHost = &useImplicitHost
			}
		case "IsStateful":
			if v != nil {
				var isStateful bool
				err = json.Unmarshal(*v, &isStateful)
				if err != nil {
					return err
				}
				sstd.IsStateful = &isStateful
			}
		case "ServiceTypeName":
			if v != nil {
				var serviceTypeName string
				err = json.Unmarshal(*v, &serviceTypeName)
				if err != nil {
					return err
				}
				sstd.ServiceTypeName = &serviceTypeName
			}
		case "PlacementConstraints":
			if v != nil {
				var placementConstraints string
				err = json.Unmarshal(*v, &placementConstraints)
				if err != nil {
					return err
				}
				sstd.PlacementConstraints = &placementConstraints
			}
		case "LoadMetrics":
			if v != nil {
				var loadMetrics []ServiceLoadMetricDescription
				err = json.Unmarshal(*v, &loadMetrics)
				if err != nil {
					return err
				}
				sstd.LoadMetrics = &loadMetrics
			}
		case "ServicePlacementPolicies":
			if v != nil {
				servicePlacementPolicies, err := unmarshalBasicServicePlacementPolicyDescriptionArray(*v)
				if err != nil {
					return err
				}
				sstd.ServicePlacementPolicies = &servicePlacementPolicies
			}
		case "Extensions":
			if v != nil {
				var extensions []ServiceTypeExtensionDescription
				err = json.Unmarshal(*v, &extensions)
				if err != nil {
					return err
				}
				sstd.Extensions = &extensions
			}
		case "Kind":
			if v != nil {
				var kind KindBasicServiceTypeDescription
				err = json.Unmarshal(*v, &kind)
				if err != nil {
					return err
				}
				sstd.Kind = kind
			}
		}
	}

	return nil
}

// StatelessServiceUpdateDescription describes an update for a stateless service.
type StatelessServiceUpdateDescription struct {
	// InstanceCount - The instance count.
	InstanceCount *int32 `json:"InstanceCount,omitempty"`
	// Flags - Flags indicating whether other properties are set. Each of the associated properties corresponds to a flag, specified below, which, if set, indicate that the property is specified.
	// This property can be a combination of those flags obtained using bitwise 'OR' operator.
	// For example, if the provided value is 6 then the flags for ReplicaRestartWaitDuration (2) and QuorumLossWaitDuration (4) are set.
	// - None - Does not indicate any other properties are set. The value is zero.
	// - TargetReplicaSetSize/InstanceCount - Indicates whether the TargetReplicaSetSize property (for Stateful services) or the InstanceCount property (for Stateless services) is set. The value is 1.
	// - ReplicaRestartWaitDuration - Indicates the ReplicaRestartWaitDuration property is set. The value is  2.
	// - QuorumLossWaitDuration - Indicates the QuorumLossWaitDuration property is set. The value is 4.
	// - StandByReplicaKeepDuration - Indicates the StandByReplicaKeepDuration property is set. The value is 8.
	// - MinReplicaSetSize - Indicates the MinReplicaSetSize property is set. The value is 16.
	// - PlacementConstraints - Indicates the PlacementConstraints property is set. The value is 32.
	// - PlacementPolicyList - Indicates the ServicePlacementPolicies property is set. The value is 64.
	// - Correlation - Indicates the CorrelationScheme property is set. The value is 128.
	// - Metrics - Indicates the ServiceLoadMetrics property is set. The value is 256.
	// - DefaultMoveCost - Indicates the DefaultMoveCost property is set. The value is 512.
	// - ScalingPolicy - Indicates the ScalingPolicies property is set. The value is 1024.
	Flags *string `json:"Flags,omitempty"`
	// PlacementConstraints - The placement constraints as a string. Placement constraints are boolean expressions on node properties and allow for restricting a service to particular nodes based on the service requirements. For example, to place a service on nodes where NodeType is blue specify the following: "NodeColor == blue)".
	PlacementConstraints *string `json:"PlacementConstraints,omitempty"`
	// CorrelationScheme - The correlation scheme.
	CorrelationScheme *[]ServiceCorrelationDescription `json:"CorrelationScheme,omitempty"`
	// LoadMetrics - The service load metrics.
	LoadMetrics *[]ServiceLoadMetricDescription `json:"LoadMetrics,omitempty"`
	// ServicePlacementPolicies - The service placement policies.
	ServicePlacementPolicies *[]BasicServicePlacementPolicyDescription `json:"ServicePlacementPolicies,omitempty"`
	// DefaultMoveCost - The move cost for the service. Possible values include: 'Zero', 'Low', 'Medium', 'High'
	DefaultMoveCost MoveCost `json:"DefaultMoveCost,omitempty"`
	// ScalingPolicies - Scaling policies for this service.
	ScalingPolicies *[]ScalingPolicyDescription `json:"ScalingPolicies,omitempty"`
	// ServiceKind - Possible values include: 'ServiceKindBasicServiceUpdateDescriptionServiceKindServiceUpdateDescription', 'ServiceKindBasicServiceUpdateDescriptionServiceKindStateful', 'ServiceKindBasicServiceUpdateDescriptionServiceKindStateless'
	ServiceKind ServiceKindBasicServiceUpdateDescription `json:"ServiceKind,omitempty"`
}

// MarshalJSON is the custom marshaler for StatelessServiceUpdateDescription.
func (ssud StatelessServiceUpdateDescription) MarshalJSON() ([]byte, error) {
	ssud.ServiceKind = ServiceKindBasicServiceUpdateDescriptionServiceKindStateless
	objectMap := make(map[string]interface{})
	if ssud.InstanceCount != nil {
		objectMap["InstanceCount"] = ssud.InstanceCount
	}
	if ssud.Flags != nil {
		objectMap["Flags"] = ssud.Flags
	}
	if ssud.PlacementConstraints != nil {
		objectMap["PlacementConstraints"] = ssud.PlacementConstraints
	}
	if ssud.CorrelationScheme != nil {
		objectMap["CorrelationScheme"] = ssud.CorrelationScheme
	}
	if ssud.LoadMetrics != nil {
		objectMap["LoadMetrics"] = ssud.LoadMetrics
	}
	if ssud.ServicePlacementPolicies != nil {
		objectMap["ServicePlacementPolicies"] = ssud.ServicePlacementPolicies
	}
	if ssud.DefaultMoveCost != "" {
		objectMap["DefaultMoveCost"] = ssud.DefaultMoveCost
	}
	if ssud.ScalingPolicies != nil {
		objectMap["ScalingPolicies"] = ssud.ScalingPolicies
	}
	if ssud.ServiceKind != "" {
		objectMap["ServiceKind"] = ssud.ServiceKind
	}
	return json.Marshal(objectMap)
}

// AsStatefulServiceUpdateDescription is the BasicServiceUpdateDescription implementation for StatelessServiceUpdateDescription.
func (ssud StatelessServiceUpdateDescription) AsStatefulServiceUpdateDescription() (*StatefulServiceUpdateDescription, bool) {
	return nil, false
}

// AsStatelessServiceUpdateDescription is the BasicServiceUpdateDescription implementation for StatelessServiceUpdateDescription.
func (ssud StatelessServiceUpdateDescription) AsStatelessServiceUpdateDescription() (*StatelessServiceUpdateDescription, bool) {
	return &ssud, true
}

// AsServiceUpdateDescription is the BasicServiceUpdateDescription implementation for StatelessServiceUpdateDescription.
func (ssud StatelessServiceUpdateDescription) AsServiceUpdateDescription() (*ServiceUpdateDescription, bool) {
	return nil, false
}

// AsBasicServiceUpdateDescription is the BasicServiceUpdateDescription implementation for StatelessServiceUpdateDescription.
func (ssud StatelessServiceUpdateDescription) AsBasicServiceUpdateDescription() (BasicServiceUpdateDescription, bool) {
	return &ssud, true
}

// UnmarshalJSON is the custom unmarshaler for StatelessServiceUpdateDescription struct.
func (ssud *StatelessServiceUpdateDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "InstanceCount":
			if v != nil {
				var instanceCount int32
				err = json.Unmarshal(*v, &instanceCount)
				if err != nil {
					return err
				}
				ssud.InstanceCount = &instanceCount
			}
		case "Flags":
			if v != nil {
				var flags string
				err = json.Unmarshal(*v, &flags)
				if err != nil {
					return err
				}
				ssud.Flags = &flags
			}
		case "PlacementConstraints":
			if v != nil {
				var placementConstraints string
				err = json.Unmarshal(*v, &placementConstraints)
				if err != nil {
					return err
				}
				ssud.PlacementConstraints = &placementConstraints
			}
		case "CorrelationScheme":
			if v != nil {
				var correlationScheme []ServiceCorrelationDescription
				err = json.Unmarshal(*v, &correlationScheme)
				if err != nil {
					return err
				}
				ssud.CorrelationScheme = &correlationScheme
			}
		case "LoadMetrics":
			if v != nil {
				var loadMetrics []ServiceLoadMetricDescription
				err = json.Unmarshal(*v, &loadMetrics)
				if err != nil {
					return err
				}
				ssud.LoadMetrics = &loadMetrics
			}
		case "ServicePlacementPolicies":
			if v != nil {
				servicePlacementPolicies, err := unmarshalBasicServicePlacementPolicyDescriptionArray(*v)
				if err != nil {
					return err
				}
				ssud.ServicePlacementPolicies = &servicePlacementPolicies
			}
		case "DefaultMoveCost":
			if v != nil {
				var defaultMoveCost MoveCost
				err = json.Unmarshal(*v, &defaultMoveCost)
				if err != nil {
					return err
				}
				ssud.DefaultMoveCost = defaultMoveCost
			}
		case "ScalingPolicies":
			if v != nil {
				var scalingPolicies []ScalingPolicyDescription
				err = json.Unmarshal(*v, &scalingPolicies)
				if err != nil {
					return err
				}
				ssud.ScalingPolicies = &scalingPolicies
			}
		case "ServiceKind":
			if v != nil {
				var serviceKind ServiceKindBasicServiceUpdateDescription
				err = json.Unmarshal(*v, &serviceKind)
				if err != nil {
					return err
				}
				ssud.ServiceKind = serviceKind
			}
		}
	}

	return nil
}

// StoppedChaosEvent describes a Chaos event that gets generated when Chaos stops because either the user
// issued a stop or the time to run was up.
type StoppedChaosEvent struct {
	// Reason - Describes why Chaos stopped. Chaos can stop because of StopChaos API call or the timeToRun provided in ChaosParameters is over.
	Reason *string `json:"Reason,omitempty"`
	// TimeStampUtc - The UTC timestamp when this Chaos event was generated.
	TimeStampUtc *date.Time `json:"TimeStampUtc,omitempty"`
	// Kind - Possible values include: 'KindChaosEvent', 'KindExecutingFaults', 'KindStarted', 'KindStopped', 'KindTestError', 'KindValidationFailed', 'KindWaiting'
	Kind KindBasicChaosEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for StoppedChaosEvent.
func (sce StoppedChaosEvent) MarshalJSON() ([]byte, error) {
	sce.Kind = KindStopped
	objectMap := make(map[string]interface{})
	if sce.Reason != nil {
		objectMap["Reason"] = sce.Reason
	}
	if sce.TimeStampUtc != nil {
		objectMap["TimeStampUtc"] = sce.TimeStampUtc
	}
	if sce.Kind != "" {
		objectMap["Kind"] = sce.Kind
	}
	return json.Marshal(objectMap)
}

// AsExecutingFaultsChaosEvent is the BasicChaosEvent implementation for StoppedChaosEvent.
func (sce StoppedChaosEvent) AsExecutingFaultsChaosEvent() (*ExecutingFaultsChaosEvent, bool) {
	return nil, false
}

// AsStartedChaosEvent is the BasicChaosEvent implementation for StoppedChaosEvent.
func (sce StoppedChaosEvent) AsStartedChaosEvent() (*StartedChaosEvent, bool) {
	return nil, false
}

// AsStoppedChaosEvent is the BasicChaosEvent implementation for StoppedChaosEvent.
func (sce StoppedChaosEvent) AsStoppedChaosEvent() (*StoppedChaosEvent, bool) {
	return &sce, true
}

// AsTestErrorChaosEvent is the BasicChaosEvent implementation for StoppedChaosEvent.
func (sce StoppedChaosEvent) AsTestErrorChaosEvent() (*TestErrorChaosEvent, bool) {
	return nil, false
}

// AsValidationFailedChaosEvent is the BasicChaosEvent implementation for StoppedChaosEvent.
func (sce StoppedChaosEvent) AsValidationFailedChaosEvent() (*ValidationFailedChaosEvent, bool) {
	return nil, false
}

// AsWaitingChaosEvent is the BasicChaosEvent implementation for StoppedChaosEvent.
func (sce StoppedChaosEvent) AsWaitingChaosEvent() (*WaitingChaosEvent, bool) {
	return nil, false
}

// AsChaosEvent is the BasicChaosEvent implementation for StoppedChaosEvent.
func (sce StoppedChaosEvent) AsChaosEvent() (*ChaosEvent, bool) {
	return nil, false
}

// AsBasicChaosEvent is the BasicChaosEvent implementation for StoppedChaosEvent.
func (sce StoppedChaosEvent) AsBasicChaosEvent() (BasicChaosEvent, bool) {
	return &sce, true
}

// String ...
type String struct {
	autorest.Response `json:"-"`
	Value             *string `json:"value,omitempty"`
}

// StringPropertyValue describes a Service Fabric property value of type String.
type StringPropertyValue struct {
	// Data - The data of the property value.
	Data *string `json:"Data,omitempty"`
	// Kind - Possible values include: 'KindPropertyValue', 'KindBinary', 'KindInt64', 'KindDouble', 'KindString', 'KindGUID'
	Kind KindBasicPropertyValue `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for StringPropertyValue.
func (spv StringPropertyValue) MarshalJSON() ([]byte, error) {
	spv.Kind = KindString
	objectMap := make(map[string]interface{})
	if spv.Data != nil {
		objectMap["Data"] = spv.Data
	}
	if spv.Kind != "" {
		objectMap["Kind"] = spv.Kind
	}
	return json.Marshal(objectMap)
}

// AsBinaryPropertyValue is the BasicPropertyValue implementation for StringPropertyValue.
func (spv StringPropertyValue) AsBinaryPropertyValue() (*BinaryPropertyValue, bool) {
	return nil, false
}

// AsInt64PropertyValue is the BasicPropertyValue implementation for StringPropertyValue.
func (spv StringPropertyValue) AsInt64PropertyValue() (*Int64PropertyValue, bool) {
	return nil, false
}

// AsDoublePropertyValue is the BasicPropertyValue implementation for StringPropertyValue.
func (spv StringPropertyValue) AsDoublePropertyValue() (*DoublePropertyValue, bool) {
	return nil, false
}

// AsStringPropertyValue is the BasicPropertyValue implementation for StringPropertyValue.
func (spv StringPropertyValue) AsStringPropertyValue() (*StringPropertyValue, bool) {
	return &spv, true
}

// AsGUIDPropertyValue is the BasicPropertyValue implementation for StringPropertyValue.
func (spv StringPropertyValue) AsGUIDPropertyValue() (*GUIDPropertyValue, bool) {
	return nil, false
}

// AsPropertyValue is the BasicPropertyValue implementation for StringPropertyValue.
func (spv StringPropertyValue) AsPropertyValue() (*PropertyValue, bool) {
	return nil, false
}

// AsBasicPropertyValue is the BasicPropertyValue implementation for StringPropertyValue.
func (spv StringPropertyValue) AsBasicPropertyValue() (BasicPropertyValue, bool) {
	return &spv, true
}

// SuccessfulPropertyBatchInfo derived from PropertyBatchInfo. Represents the property batch succeeding.
// Contains the results of any "Get" operations in the batch.
type SuccessfulPropertyBatchInfo struct {
	// Properties - A map containing the properties that were requested through any "Get" property batch operations. The key represents the index of the "Get" operation in the original request, in string form. The value is the property. If a property is not found, it will not be in the map.
	Properties map[string]*PropertyInfo `json:"Properties"`
	// Kind - Possible values include: 'KindPropertyBatchInfo', 'KindSuccessful', 'KindFailed'
	Kind KindBasicPropertyBatchInfo `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for SuccessfulPropertyBatchInfo.
func (spbi SuccessfulPropertyBatchInfo) MarshalJSON() ([]byte, error) {
	spbi.Kind = KindSuccessful
	objectMap := make(map[string]interface{})
	if spbi.Properties != nil {
		objectMap["Properties"] = spbi.Properties
	}
	if spbi.Kind != "" {
		objectMap["Kind"] = spbi.Kind
	}
	return json.Marshal(objectMap)
}

// AsSuccessfulPropertyBatchInfo is the BasicPropertyBatchInfo implementation for SuccessfulPropertyBatchInfo.
func (spbi SuccessfulPropertyBatchInfo) AsSuccessfulPropertyBatchInfo() (*SuccessfulPropertyBatchInfo, bool) {
	return &spbi, true
}

// AsFailedPropertyBatchInfo is the BasicPropertyBatchInfo implementation for SuccessfulPropertyBatchInfo.
func (spbi SuccessfulPropertyBatchInfo) AsFailedPropertyBatchInfo() (*FailedPropertyBatchInfo, bool) {
	return nil, false
}

// AsPropertyBatchInfo is the BasicPropertyBatchInfo implementation for SuccessfulPropertyBatchInfo.
func (spbi SuccessfulPropertyBatchInfo) AsPropertyBatchInfo() (*PropertyBatchInfo, bool) {
	return nil, false
}

// AsBasicPropertyBatchInfo is the BasicPropertyBatchInfo implementation for SuccessfulPropertyBatchInfo.
func (spbi SuccessfulPropertyBatchInfo) AsBasicPropertyBatchInfo() (BasicPropertyBatchInfo, bool) {
	return &spbi, true
}

// SystemApplicationHealthEvaluation represents health evaluation for the fabric:/System application,
// containing information about the data and the algorithm used by health store to evaluate health. The
// evaluation is returned only when the aggregated health state of the cluster is either Error or Warning.
type SystemApplicationHealthEvaluation struct {
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the current aggregated health state of the system application. The types of the unhealthy evaluations can be DeployedApplicationsHealthEvaluation, ServicesHealthEvaluation or EventHealthEvaluation.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) MarshalJSON() ([]byte, error) {
	sahe.Kind = KindSystemApplication
	objectMap := make(map[string]interface{})
	if sahe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = sahe.UnhealthyEvaluations
	}
	if sahe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = sahe.AggregatedHealthState
	}
	if sahe.Description != nil {
		objectMap["Description"] = sahe.Description
	}
	if sahe.Kind != "" {
		objectMap["Kind"] = sahe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return &sahe, true
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for SystemApplicationHealthEvaluation.
func (sahe SystemApplicationHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &sahe, true
}

// TCPConfig describes the tcp configuration for external connectivity for this network.
type TCPConfig struct {
	// Name - tcp gateway config name.
	Name *string `json:"name,omitempty"`
	// Port - Specifies the port at which the service endpoint below needs to be exposed.
	Port *int32 `json:"port,omitempty"`
	// Destination - Describes destination endpoint for routing traffic.
	Destination *GatewayDestination `json:"destination,omitempty"`
}

// TestErrorChaosEvent describes a Chaos event that gets generated when an unexpected event occurs in the
// Chaos engine.
// For example, due to the cluster snapshot being inconsistent, while faulting an entity, Chaos found that
// the entity was already faulted -- which would be an unexpected event.
type TestErrorChaosEvent struct {
	// Reason - Describes why TestErrorChaosEvent was generated. For example, Chaos tries to fault a partition but finds that the partition is no longer fault tolerant, then a TestErrorEvent gets generated with the reason stating that the partition is not fault tolerant.
	Reason *string `json:"Reason,omitempty"`
	// TimeStampUtc - The UTC timestamp when this Chaos event was generated.
	TimeStampUtc *date.Time `json:"TimeStampUtc,omitempty"`
	// Kind - Possible values include: 'KindChaosEvent', 'KindExecutingFaults', 'KindStarted', 'KindStopped', 'KindTestError', 'KindValidationFailed', 'KindWaiting'
	Kind KindBasicChaosEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for TestErrorChaosEvent.
func (tece TestErrorChaosEvent) MarshalJSON() ([]byte, error) {
	tece.Kind = KindTestError
	objectMap := make(map[string]interface{})
	if tece.Reason != nil {
		objectMap["Reason"] = tece.Reason
	}
	if tece.TimeStampUtc != nil {
		objectMap["TimeStampUtc"] = tece.TimeStampUtc
	}
	if tece.Kind != "" {
		objectMap["Kind"] = tece.Kind
	}
	return json.Marshal(objectMap)
}

// AsExecutingFaultsChaosEvent is the BasicChaosEvent implementation for TestErrorChaosEvent.
func (tece TestErrorChaosEvent) AsExecutingFaultsChaosEvent() (*ExecutingFaultsChaosEvent, bool) {
	return nil, false
}

// AsStartedChaosEvent is the BasicChaosEvent implementation for TestErrorChaosEvent.
func (tece TestErrorChaosEvent) AsStartedChaosEvent() (*StartedChaosEvent, bool) {
	return nil, false
}

// AsStoppedChaosEvent is the BasicChaosEvent implementation for TestErrorChaosEvent.
func (tece TestErrorChaosEvent) AsStoppedChaosEvent() (*StoppedChaosEvent, bool) {
	return nil, false
}

// AsTestErrorChaosEvent is the BasicChaosEvent implementation for TestErrorChaosEvent.
func (tece TestErrorChaosEvent) AsTestErrorChaosEvent() (*TestErrorChaosEvent, bool) {
	return &tece, true
}

// AsValidationFailedChaosEvent is the BasicChaosEvent implementation for TestErrorChaosEvent.
func (tece TestErrorChaosEvent) AsValidationFailedChaosEvent() (*ValidationFailedChaosEvent, bool) {
	return nil, false
}

// AsWaitingChaosEvent is the BasicChaosEvent implementation for TestErrorChaosEvent.
func (tece TestErrorChaosEvent) AsWaitingChaosEvent() (*WaitingChaosEvent, bool) {
	return nil, false
}

// AsChaosEvent is the BasicChaosEvent implementation for TestErrorChaosEvent.
func (tece TestErrorChaosEvent) AsChaosEvent() (*ChaosEvent, bool) {
	return nil, false
}

// AsBasicChaosEvent is the BasicChaosEvent implementation for TestErrorChaosEvent.
func (tece TestErrorChaosEvent) AsBasicChaosEvent() (BasicChaosEvent, bool) {
	return &tece, true
}

// TimeBasedBackupScheduleDescription describes the time based backup schedule.
type TimeBasedBackupScheduleDescription struct {
	// ScheduleFrequencyType - Describes the frequency with which to run the time based backup schedule. Possible values include: 'BackupScheduleFrequencyTypeInvalid', 'BackupScheduleFrequencyTypeDaily', 'BackupScheduleFrequencyTypeWeekly'
	ScheduleFrequencyType BackupScheduleFrequencyType `json:"ScheduleFrequencyType,omitempty"`
	// RunDays - List of days of a week when to trigger the periodic backup. This is valid only when the backup schedule frequency type is weekly.
	RunDays *[]DayOfWeek `json:"RunDays,omitempty"`
	// RunTimes - Represents the list of exact time during the day in ISO8601 format. Like '19:00:00' will represent '7PM' during the day. Date specified along with time will be ignored.
	RunTimes *[]date.Time `json:"RunTimes,omitempty"`
	// ScheduleKind - Possible values include: 'ScheduleKindBackupScheduleDescription', 'ScheduleKindFrequencyBased', 'ScheduleKindTimeBased'
	ScheduleKind ScheduleKind `json:"ScheduleKind,omitempty"`
}

// MarshalJSON is the custom marshaler for TimeBasedBackupScheduleDescription.
func (tbbsd TimeBasedBackupScheduleDescription) MarshalJSON() ([]byte, error) {
	tbbsd.ScheduleKind = ScheduleKindTimeBased
	objectMap := make(map[string]interface{})
	if tbbsd.ScheduleFrequencyType != "" {
		objectMap["ScheduleFrequencyType"] = tbbsd.ScheduleFrequencyType
	}
	if tbbsd.RunDays != nil {
		objectMap["RunDays"] = tbbsd.RunDays
	}
	if tbbsd.RunTimes != nil {
		objectMap["RunTimes"] = tbbsd.RunTimes
	}
	if tbbsd.ScheduleKind != "" {
		objectMap["ScheduleKind"] = tbbsd.ScheduleKind
	}
	return json.Marshal(objectMap)
}

// AsFrequencyBasedBackupScheduleDescription is the BasicBackupScheduleDescription implementation for TimeBasedBackupScheduleDescription.
func (tbbsd TimeBasedBackupScheduleDescription) AsFrequencyBasedBackupScheduleDescription() (*FrequencyBasedBackupScheduleDescription, bool) {
	return nil, false
}

// AsTimeBasedBackupScheduleDescription is the BasicBackupScheduleDescription implementation for TimeBasedBackupScheduleDescription.
func (tbbsd TimeBasedBackupScheduleDescription) AsTimeBasedBackupScheduleDescription() (*TimeBasedBackupScheduleDescription, bool) {
	return &tbbsd, true
}

// AsBackupScheduleDescription is the BasicBackupScheduleDescription implementation for TimeBasedBackupScheduleDescription.
func (tbbsd TimeBasedBackupScheduleDescription) AsBackupScheduleDescription() (*BackupScheduleDescription, bool) {
	return nil, false
}

// AsBasicBackupScheduleDescription is the BasicBackupScheduleDescription implementation for TimeBasedBackupScheduleDescription.
func (tbbsd TimeBasedBackupScheduleDescription) AsBasicBackupScheduleDescription() (BasicBackupScheduleDescription, bool) {
	return &tbbsd, true
}

// TimeOfDay defines an hour and minute of the day specified in 24 hour time.
type TimeOfDay struct {
	// Hour - Represents the hour of the day. Value must be between 0 and 23 inclusive.
	Hour *int32 `json:"Hour,omitempty"`
	// Minute - Represents the minute of the hour. Value must be between 0 to 59 inclusive.
	Minute *int32 `json:"Minute,omitempty"`
}

// TimeRange defines a time range in a 24 hour day specified by a start and end time.
type TimeRange struct {
	// StartTime - Defines an hour and minute of the day specified in 24 hour time.
	StartTime *TimeOfDay `json:"StartTime,omitempty"`
	// EndTime - Defines an hour and minute of the day specified in 24 hour time.
	EndTime *TimeOfDay `json:"EndTime,omitempty"`
}

// UniformInt64RangePartitionSchemeDescription describes a partitioning scheme where an integer range is
// allocated evenly across a number of partitions.
type UniformInt64RangePartitionSchemeDescription struct {
	// Count - The number of partitions.
	Count *int32 `json:"Count,omitempty"`
	// LowKey - String indicating the lower bound of the partition key range that
	// should be split between the partitions.
	LowKey *string `json:"LowKey,omitempty"`
	// HighKey - String indicating the upper bound of the partition key range that
	// should be split between the partitions.
	HighKey *string `json:"HighKey,omitempty"`
	// PartitionScheme - Possible values include: 'PartitionSchemePartitionSchemeDescription', 'PartitionSchemeNamed1', 'PartitionSchemeSingleton1', 'PartitionSchemeUniformInt64Range1'
	PartitionScheme PartitionSchemeBasicPartitionSchemeDescription `json:"PartitionScheme,omitempty"`
}

// MarshalJSON is the custom marshaler for UniformInt64RangePartitionSchemeDescription.
func (ui6rpsd UniformInt64RangePartitionSchemeDescription) MarshalJSON() ([]byte, error) {
	ui6rpsd.PartitionScheme = PartitionSchemeUniformInt64Range1
	objectMap := make(map[string]interface{})
	if ui6rpsd.Count != nil {
		objectMap["Count"] = ui6rpsd.Count
	}
	if ui6rpsd.LowKey != nil {
		objectMap["LowKey"] = ui6rpsd.LowKey
	}
	if ui6rpsd.HighKey != nil {
		objectMap["HighKey"] = ui6rpsd.HighKey
	}
	if ui6rpsd.PartitionScheme != "" {
		objectMap["PartitionScheme"] = ui6rpsd.PartitionScheme
	}
	return json.Marshal(objectMap)
}

// AsNamedPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for UniformInt64RangePartitionSchemeDescription.
func (ui6rpsd UniformInt64RangePartitionSchemeDescription) AsNamedPartitionSchemeDescription() (*NamedPartitionSchemeDescription, bool) {
	return nil, false
}

// AsSingletonPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for UniformInt64RangePartitionSchemeDescription.
func (ui6rpsd UniformInt64RangePartitionSchemeDescription) AsSingletonPartitionSchemeDescription() (*SingletonPartitionSchemeDescription, bool) {
	return nil, false
}

// AsUniformInt64RangePartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for UniformInt64RangePartitionSchemeDescription.
func (ui6rpsd UniformInt64RangePartitionSchemeDescription) AsUniformInt64RangePartitionSchemeDescription() (*UniformInt64RangePartitionSchemeDescription, bool) {
	return &ui6rpsd, true
}

// AsPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for UniformInt64RangePartitionSchemeDescription.
func (ui6rpsd UniformInt64RangePartitionSchemeDescription) AsPartitionSchemeDescription() (*PartitionSchemeDescription, bool) {
	return nil, false
}

// AsBasicPartitionSchemeDescription is the BasicPartitionSchemeDescription implementation for UniformInt64RangePartitionSchemeDescription.
func (ui6rpsd UniformInt64RangePartitionSchemeDescription) AsBasicPartitionSchemeDescription() (BasicPartitionSchemeDescription, bool) {
	return &ui6rpsd, true
}

// UnplacedReplicaInformation contains information for an unplaced replica.
type UnplacedReplicaInformation struct {
	autorest.Response `json:"-"`
	// ServiceName - The name of the service.
	ServiceName *string `json:"ServiceName,omitempty"`
	// PartitionID - The ID of the partition.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// UnplacedReplicaDetails - List of reasons due to which a replica cannot be placed.
	UnplacedReplicaDetails *[]string `json:"UnplacedReplicaDetails,omitempty"`
}

// UnprovisionApplicationTypeDescriptionInfo describes the operation to unregister or unprovision an
// application type and its version that was registered with the Service Fabric.
type UnprovisionApplicationTypeDescriptionInfo struct {
	// ApplicationTypeVersion - The version of the application type as defined in the application manifest.
	ApplicationTypeVersion *string `json:"ApplicationTypeVersion,omitempty"`
	// Async - The flag indicating whether or not unprovision should occur asynchronously. When set to true, the unprovision operation returns when the request is accepted by the system, and the unprovision operation continues without any timeout limit. The default value is false. However, we recommend setting it to true for large application packages that were provisioned.
	Async *bool `json:"Async,omitempty"`
}

// UnprovisionFabricDescription describes the parameters for unprovisioning a cluster.
type UnprovisionFabricDescription struct {
	// CodeVersion - The cluster code package version.
	CodeVersion *string `json:"CodeVersion,omitempty"`
	// ConfigVersion - The cluster manifest version.
	ConfigVersion *string `json:"ConfigVersion,omitempty"`
}

// UpdateClusterUpgradeDescription parameters for updating a cluster upgrade.
type UpdateClusterUpgradeDescription struct {
	// UpgradeKind - The type of upgrade out of the following possible values. Possible values include: 'UpgradeTypeInvalid', 'UpgradeTypeRolling', 'UpgradeTypeRollingForceRestart'
	UpgradeKind UpgradeType `json:"UpgradeKind,omitempty"`
	// UpdateDescription - Describes the parameters for updating a rolling upgrade of application or cluster.
	UpdateDescription *RollingUpgradeUpdateDescription `json:"UpdateDescription,omitempty"`
	// ClusterHealthPolicy - Defines a health policy used to evaluate the health of the cluster or of a cluster node.
	ClusterHealthPolicy *ClusterHealthPolicy `json:"ClusterHealthPolicy,omitempty"`
	// EnableDeltaHealthEvaluation - When true, enables delta health evaluation rather than absolute health evaluation after completion of each upgrade domain.
	EnableDeltaHealthEvaluation *bool `json:"EnableDeltaHealthEvaluation,omitempty"`
	// ClusterUpgradeHealthPolicy - Defines a health policy used to evaluate the health of the cluster during a cluster upgrade.
	ClusterUpgradeHealthPolicy *ClusterUpgradeHealthPolicyObject `json:"ClusterUpgradeHealthPolicy,omitempty"`
	// ApplicationHealthPolicyMap - Defines the application health policy map used to evaluate the health of an application or one of its children entities.
	ApplicationHealthPolicyMap *ApplicationHealthPolicies `json:"ApplicationHealthPolicyMap,omitempty"`
}

// UpgradeDomainDeltaNodesCheckHealthEvaluation represents health evaluation for delta unhealthy cluster
// nodes in an upgrade domain, containing health evaluations for each unhealthy node that impacted current
// aggregated health state.
// Can be returned during cluster upgrade when cluster aggregated health state is Warning or Error.
type UpgradeDomainDeltaNodesCheckHealthEvaluation struct {
	// UpgradeDomainName - Name of the upgrade domain where nodes health is currently evaluated.
	UpgradeDomainName *string `json:"UpgradeDomainName,omitempty"`
	// BaselineErrorCount - Number of upgrade domain nodes with aggregated heath state Error in the health store at the beginning of the cluster upgrade.
	BaselineErrorCount *int64 `json:"BaselineErrorCount,omitempty"`
	// BaselineTotalCount - Total number of upgrade domain nodes in the health store at the beginning of the cluster upgrade.
	BaselineTotalCount *int64 `json:"BaselineTotalCount,omitempty"`
	// MaxPercentDeltaUnhealthyNodes - Maximum allowed percentage of upgrade domain delta unhealthy nodes from the ClusterUpgradeHealthPolicy.
	MaxPercentDeltaUnhealthyNodes *int32 `json:"MaxPercentDeltaUnhealthyNodes,omitempty"`
	// TotalCount - Total number of upgrade domain nodes in the health store.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state. Includes all the unhealthy NodeHealthEvaluation that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) MarshalJSON() ([]byte, error) {
	uddnche.Kind = KindUpgradeDomainDeltaNodesCheck
	objectMap := make(map[string]interface{})
	if uddnche.UpgradeDomainName != nil {
		objectMap["UpgradeDomainName"] = uddnche.UpgradeDomainName
	}
	if uddnche.BaselineErrorCount != nil {
		objectMap["BaselineErrorCount"] = uddnche.BaselineErrorCount
	}
	if uddnche.BaselineTotalCount != nil {
		objectMap["BaselineTotalCount"] = uddnche.BaselineTotalCount
	}
	if uddnche.MaxPercentDeltaUnhealthyNodes != nil {
		objectMap["MaxPercentDeltaUnhealthyNodes"] = uddnche.MaxPercentDeltaUnhealthyNodes
	}
	if uddnche.TotalCount != nil {
		objectMap["TotalCount"] = uddnche.TotalCount
	}
	if uddnche.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = uddnche.UnhealthyEvaluations
	}
	if uddnche.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = uddnche.AggregatedHealthState
	}
	if uddnche.Description != nil {
		objectMap["Description"] = uddnche.Description
	}
	if uddnche.Kind != "" {
		objectMap["Kind"] = uddnche.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return &uddnche, true
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return nil, false
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainDeltaNodesCheckHealthEvaluation.
func (uddnche UpgradeDomainDeltaNodesCheckHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &uddnche, true
}

// UpgradeDomainInfo information about an upgrade domain.
type UpgradeDomainInfo struct {
	// Name - The name of the upgrade domain
	Name *string `json:"Name,omitempty"`
	// State - The state of the upgrade domain. Possible values include: 'UpgradeDomainStateInvalid', 'UpgradeDomainStatePending', 'UpgradeDomainStateInProgress', 'UpgradeDomainStateCompleted'
	State UpgradeDomainState `json:"State,omitempty"`
}

// UpgradeDomainNodesHealthEvaluation represents health evaluation for cluster nodes in an upgrade domain,
// containing health evaluations for each unhealthy node that impacted current aggregated health state. Can
// be returned when evaluating cluster health during cluster upgrade and the aggregated health state is
// either Error or Warning.
type UpgradeDomainNodesHealthEvaluation struct {
	// UpgradeDomainName - Name of the upgrade domain where nodes health is currently evaluated.
	UpgradeDomainName *string `json:"UpgradeDomainName,omitempty"`
	// MaxPercentUnhealthyNodes - Maximum allowed percentage of unhealthy nodes from the ClusterHealthPolicy.
	MaxPercentUnhealthyNodes *int32 `json:"MaxPercentUnhealthyNodes,omitempty"`
	// TotalCount - Total number of nodes in the current upgrade domain.
	TotalCount *int64 `json:"TotalCount,omitempty"`
	// UnhealthyEvaluations - List of unhealthy evaluations that led to the aggregated health state. Includes all the unhealthy NodeHealthEvaluation that impacted the aggregated health.
	UnhealthyEvaluations *[]HealthEvaluationWrapper `json:"UnhealthyEvaluations,omitempty"`
	// AggregatedHealthState - The health state of a Service Fabric entity such as Cluster, Node, Application, Service, Partition, Replica etc. Possible values include: 'HealthStateInvalid', 'HealthStateOk', 'HealthStateWarning', 'HealthStateError', 'HealthStateUnknown'
	AggregatedHealthState HealthState `json:"AggregatedHealthState,omitempty"`
	// Description - Description of the health evaluation, which represents a summary of the evaluation process.
	Description *string `json:"Description,omitempty"`
	// Kind - Possible values include: 'KindHealthEvaluation', 'KindApplication', 'KindApplications', 'KindApplicationTypeApplications', 'KindDeltaNodesCheck', 'KindDeployedApplication', 'KindDeployedApplications', 'KindDeployedServicePackage', 'KindDeployedServicePackages', 'KindEvent', 'KindNode', 'KindNodes', 'KindPartition', 'KindPartitions', 'KindReplica', 'KindReplicas', 'KindService', 'KindServices', 'KindSystemApplication', 'KindUpgradeDomainDeltaNodesCheck', 'KindUpgradeDomainNodes'
	Kind Kind `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) MarshalJSON() ([]byte, error) {
	udnhe.Kind = KindUpgradeDomainNodes
	objectMap := make(map[string]interface{})
	if udnhe.UpgradeDomainName != nil {
		objectMap["UpgradeDomainName"] = udnhe.UpgradeDomainName
	}
	if udnhe.MaxPercentUnhealthyNodes != nil {
		objectMap["MaxPercentUnhealthyNodes"] = udnhe.MaxPercentUnhealthyNodes
	}
	if udnhe.TotalCount != nil {
		objectMap["TotalCount"] = udnhe.TotalCount
	}
	if udnhe.UnhealthyEvaluations != nil {
		objectMap["UnhealthyEvaluations"] = udnhe.UnhealthyEvaluations
	}
	if udnhe.AggregatedHealthState != "" {
		objectMap["AggregatedHealthState"] = udnhe.AggregatedHealthState
	}
	if udnhe.Description != nil {
		objectMap["Description"] = udnhe.Description
	}
	if udnhe.Kind != "" {
		objectMap["Kind"] = udnhe.Kind
	}
	return json.Marshal(objectMap)
}

// AsApplicationHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsApplicationHealthEvaluation() (*ApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsApplicationsHealthEvaluation() (*ApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsApplicationTypeApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsApplicationTypeApplicationsHealthEvaluation() (*ApplicationTypeApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsDeltaNodesCheckHealthEvaluation() (*DeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsDeployedApplicationHealthEvaluation() (*DeployedApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedApplicationsHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsDeployedApplicationsHealthEvaluation() (*DeployedApplicationsHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackageHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsDeployedServicePackageHealthEvaluation() (*DeployedServicePackageHealthEvaluation, bool) {
	return nil, false
}

// AsDeployedServicePackagesHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsDeployedServicePackagesHealthEvaluation() (*DeployedServicePackagesHealthEvaluation, bool) {
	return nil, false
}

// AsEventHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsEventHealthEvaluation() (*EventHealthEvaluation, bool) {
	return nil, false
}

// AsNodeHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsNodeHealthEvaluation() (*NodeHealthEvaluation, bool) {
	return nil, false
}

// AsNodesHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsNodesHealthEvaluation() (*NodesHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsPartitionHealthEvaluation() (*PartitionHealthEvaluation, bool) {
	return nil, false
}

// AsPartitionsHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsPartitionsHealthEvaluation() (*PartitionsHealthEvaluation, bool) {
	return nil, false
}

// AsReplicaHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsReplicaHealthEvaluation() (*ReplicaHealthEvaluation, bool) {
	return nil, false
}

// AsReplicasHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsReplicasHealthEvaluation() (*ReplicasHealthEvaluation, bool) {
	return nil, false
}

// AsServiceHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsServiceHealthEvaluation() (*ServiceHealthEvaluation, bool) {
	return nil, false
}

// AsServicesHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsServicesHealthEvaluation() (*ServicesHealthEvaluation, bool) {
	return nil, false
}

// AsSystemApplicationHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsSystemApplicationHealthEvaluation() (*SystemApplicationHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainDeltaNodesCheckHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsUpgradeDomainDeltaNodesCheckHealthEvaluation() (*UpgradeDomainDeltaNodesCheckHealthEvaluation, bool) {
	return nil, false
}

// AsUpgradeDomainNodesHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsUpgradeDomainNodesHealthEvaluation() (*UpgradeDomainNodesHealthEvaluation, bool) {
	return &udnhe, true
}

// AsHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsHealthEvaluation() (*HealthEvaluation, bool) {
	return nil, false
}

// AsBasicHealthEvaluation is the BasicHealthEvaluation implementation for UpgradeDomainNodesHealthEvaluation.
func (udnhe UpgradeDomainNodesHealthEvaluation) AsBasicHealthEvaluation() (BasicHealthEvaluation, bool) {
	return &udnhe, true
}

// UpgradeOrchestrationServiceState service state of Service Fabric Upgrade Orchestration Service.
type UpgradeOrchestrationServiceState struct {
	autorest.Response `json:"-"`
	// ServiceState - The state of Service Fabric Upgrade Orchestration Service.
	ServiceState *string `json:"ServiceState,omitempty"`
}

// UpgradeOrchestrationServiceStateSummary service state summary of Service Fabric Upgrade Orchestration
// Service.
type UpgradeOrchestrationServiceStateSummary struct {
	autorest.Response `json:"-"`
	// CurrentCodeVersion - The current code version of the cluster.
	CurrentCodeVersion *string `json:"CurrentCodeVersion,omitempty"`
	// CurrentManifestVersion - The current manifest version of the cluster.
	CurrentManifestVersion *string `json:"CurrentManifestVersion,omitempty"`
	// TargetCodeVersion - The target code version of  the cluster.
	TargetCodeVersion *string `json:"TargetCodeVersion,omitempty"`
	// TargetManifestVersion - The target manifest version of the cluster.
	TargetManifestVersion *string `json:"TargetManifestVersion,omitempty"`
	// PendingUpgradeType - The type of the pending upgrade of the cluster.
	PendingUpgradeType *string `json:"PendingUpgradeType,omitempty"`
}

// UploadChunkRange information about which portion of the file to upload.
type UploadChunkRange struct {
	// StartPosition - The start position of the portion of the file. It's represented by the number of bytes.
	StartPosition *string `json:"StartPosition,omitempty"`
	// EndPosition - The end position of the portion of the file. It's represented by the number of bytes.
	EndPosition *string `json:"EndPosition,omitempty"`
}

// UploadSession information about a image store upload session
type UploadSession struct {
	autorest.Response `json:"-"`
	// UploadSessions - When querying upload session by upload session ID, the result contains only one upload session. When querying upload session by image store relative path, the result might contain multiple upload sessions.
	UploadSessions *[]UploadSessionInfo `json:"UploadSessions,omitempty"`
}

// UploadSessionInfo information about an image store upload session. A session is associated with a
// relative path in the image store.
type UploadSessionInfo struct {
	// StoreRelativePath - The remote location within image store. This path is relative to the image store root.
	StoreRelativePath *string `json:"StoreRelativePath,omitempty"`
	// SessionID - A unique ID of the upload session. A session ID can be reused only if the session was committed or removed.
	SessionID *uuid.UUID `json:"SessionId,omitempty"`
	// ModifiedDate - The date and time when the upload session was last modified.
	ModifiedDate *date.Time `json:"ModifiedDate,omitempty"`
	// FileSize - The size in bytes of the uploading file.
	FileSize *string `json:"FileSize,omitempty"`
	// ExpectedRanges - List of chunk ranges that image store has not received yet.
	ExpectedRanges *[]UploadChunkRange `json:"ExpectedRanges,omitempty"`
}

// ValidationFailedChaosEvent chaos event corresponding to a failure during validation.
type ValidationFailedChaosEvent struct {
	// Reason - Describes why the ValidationFailedChaosEvent was generated. This may happen because more than MaxPercentUnhealthyNodes are unhealthy for more than MaxClusterStabilizationTimeout. This reason will be in the Reason property of the ValidationFailedChaosEvent as a string.
	Reason *string `json:"Reason,omitempty"`
	// TimeStampUtc - The UTC timestamp when this Chaos event was generated.
	TimeStampUtc *date.Time `json:"TimeStampUtc,omitempty"`
	// Kind - Possible values include: 'KindChaosEvent', 'KindExecutingFaults', 'KindStarted', 'KindStopped', 'KindTestError', 'KindValidationFailed', 'KindWaiting'
	Kind KindBasicChaosEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for ValidationFailedChaosEvent.
func (vfce ValidationFailedChaosEvent) MarshalJSON() ([]byte, error) {
	vfce.Kind = KindValidationFailed
	objectMap := make(map[string]interface{})
	if vfce.Reason != nil {
		objectMap["Reason"] = vfce.Reason
	}
	if vfce.TimeStampUtc != nil {
		objectMap["TimeStampUtc"] = vfce.TimeStampUtc
	}
	if vfce.Kind != "" {
		objectMap["Kind"] = vfce.Kind
	}
	return json.Marshal(objectMap)
}

// AsExecutingFaultsChaosEvent is the BasicChaosEvent implementation for ValidationFailedChaosEvent.
func (vfce ValidationFailedChaosEvent) AsExecutingFaultsChaosEvent() (*ExecutingFaultsChaosEvent, bool) {
	return nil, false
}

// AsStartedChaosEvent is the BasicChaosEvent implementation for ValidationFailedChaosEvent.
func (vfce ValidationFailedChaosEvent) AsStartedChaosEvent() (*StartedChaosEvent, bool) {
	return nil, false
}

// AsStoppedChaosEvent is the BasicChaosEvent implementation for ValidationFailedChaosEvent.
func (vfce ValidationFailedChaosEvent) AsStoppedChaosEvent() (*StoppedChaosEvent, bool) {
	return nil, false
}

// AsTestErrorChaosEvent is the BasicChaosEvent implementation for ValidationFailedChaosEvent.
func (vfce ValidationFailedChaosEvent) AsTestErrorChaosEvent() (*TestErrorChaosEvent, bool) {
	return nil, false
}

// AsValidationFailedChaosEvent is the BasicChaosEvent implementation for ValidationFailedChaosEvent.
func (vfce ValidationFailedChaosEvent) AsValidationFailedChaosEvent() (*ValidationFailedChaosEvent, bool) {
	return &vfce, true
}

// AsWaitingChaosEvent is the BasicChaosEvent implementation for ValidationFailedChaosEvent.
func (vfce ValidationFailedChaosEvent) AsWaitingChaosEvent() (*WaitingChaosEvent, bool) {
	return nil, false
}

// AsChaosEvent is the BasicChaosEvent implementation for ValidationFailedChaosEvent.
func (vfce ValidationFailedChaosEvent) AsChaosEvent() (*ChaosEvent, bool) {
	return nil, false
}

// AsBasicChaosEvent is the BasicChaosEvent implementation for ValidationFailedChaosEvent.
func (vfce ValidationFailedChaosEvent) AsBasicChaosEvent() (BasicChaosEvent, bool) {
	return &vfce, true
}

// VolumeProperties describes properties of a volume resource.
type VolumeProperties struct {
	// Description - User readable description of the volume.
	Description *string `json:"description,omitempty"`
	// Status - READ-ONLY; Status of the volume. Possible values include: 'ResourceStatusUnknown', 'ResourceStatusReady', 'ResourceStatusUpgrading', 'ResourceStatusCreating', 'ResourceStatusDeleting', 'ResourceStatusFailed'
	Status ResourceStatus `json:"status,omitempty"`
	// StatusDetails - READ-ONLY; Gives additional information about the current status of the volume.
	StatusDetails *string `json:"statusDetails,omitempty"`
	// Provider - Provider of the volume.
	Provider *string `json:"provider,omitempty"`
	// AzureFileParameters - This type describes a volume provided by an Azure Files file share.
	AzureFileParameters *VolumeProviderParametersAzureFile `json:"azureFileParameters,omitempty"`
}

// MarshalJSON is the custom marshaler for VolumeProperties.
func (vp VolumeProperties) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if vp.Description != nil {
		objectMap["description"] = vp.Description
	}
	if vp.Provider != nil {
		objectMap["provider"] = vp.Provider
	}
	if vp.AzureFileParameters != nil {
		objectMap["azureFileParameters"] = vp.AzureFileParameters
	}
	return json.Marshal(objectMap)
}

// VolumeProviderParametersAzureFile this type describes a volume provided by an Azure Files file share.
type VolumeProviderParametersAzureFile struct {
	// AccountName - Name of the Azure storage account for the File Share.
	AccountName *string `json:"accountName,omitempty"`
	// AccountKey - Access key of the Azure storage account for the File Share.
	AccountKey *string `json:"accountKey,omitempty"`
	// ShareName - Name of the Azure Files file share that provides storage for the volume.
	ShareName *string `json:"shareName,omitempty"`
}

// VolumeReference describes a reference to a volume resource.
type VolumeReference struct {
	// Name - Name of the volume being referenced.
	Name *string `json:"name,omitempty"`
	// ReadOnly - The flag indicating whether the volume is read only. Default is 'false'.
	ReadOnly *bool `json:"readOnly,omitempty"`
	// DestinationPath - The path within the container at which the volume should be mounted. Only valid path characters are allowed.
	DestinationPath *string `json:"destinationPath,omitempty"`
}

// VolumeResourceDescription this type describes a volume resource.
type VolumeResourceDescription struct {
	autorest.Response `json:"-"`
	// Name - Name of the Volume resource.
	Name *string `json:"name,omitempty"`
	// VolumeProperties - Describes properties of a volume resource.
	*VolumeProperties `json:"properties,omitempty"`
}

// MarshalJSON is the custom marshaler for VolumeResourceDescription.
func (vrd VolumeResourceDescription) MarshalJSON() ([]byte, error) {
	objectMap := make(map[string]interface{})
	if vrd.Name != nil {
		objectMap["name"] = vrd.Name
	}
	if vrd.VolumeProperties != nil {
		objectMap["properties"] = vrd.VolumeProperties
	}
	return json.Marshal(objectMap)
}

// UnmarshalJSON is the custom unmarshaler for VolumeResourceDescription struct.
func (vrd *VolumeResourceDescription) UnmarshalJSON(body []byte) error {
	var m map[string]*json.RawMessage
	err := json.Unmarshal(body, &m)
	if err != nil {
		return err
	}
	for k, v := range m {
		switch k {
		case "name":
			if v != nil {
				var name string
				err = json.Unmarshal(*v, &name)
				if err != nil {
					return err
				}
				vrd.Name = &name
			}
		case "properties":
			if v != nil {
				var volumeProperties VolumeProperties
				err = json.Unmarshal(*v, &volumeProperties)
				if err != nil {
					return err
				}
				vrd.VolumeProperties = &volumeProperties
			}
		}
	}

	return nil
}

// WaitForInbuildReplicaSafetyCheck safety check that waits for the replica build operation to finish. This
// indicates that there is a replica that is going through the copy or is providing data for building
// another replica. Bring the node down will abort this copy operation which are typically expensive
// involving data movements.
type WaitForInbuildReplicaSafetyCheck struct {
	// PartitionID - Id of the partition which is undergoing the safety check.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// Kind - Possible values include: 'KindSafetyCheck', 'KindPartitionSafetyCheck', 'KindEnsureAvailability', 'KindEnsurePartitionQuorum', 'KindEnsureSeedNodeQuorum', 'KindWaitForInbuildReplica', 'KindWaitForPrimaryPlacement', 'KindWaitForPrimarySwap', 'KindWaitForReconfiguration'
	Kind KindBasicSafetyCheck `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) MarshalJSON() ([]byte, error) {
	wfirsc.Kind = KindWaitForInbuildReplica
	objectMap := make(map[string]interface{})
	if wfirsc.PartitionID != nil {
		objectMap["PartitionId"] = wfirsc.PartitionID
	}
	if wfirsc.Kind != "" {
		objectMap["Kind"] = wfirsc.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionSafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool) {
	return nil, false
}

// AsBasicPartitionSafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsBasicPartitionSafetyCheck() (BasicPartitionSafetyCheck, bool) {
	return &wfirsc, true
}

// AsEnsureAvailabilitySafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool) {
	return nil, false
}

// AsEnsurePartitionQuorumSafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool) {
	return nil, false
}

// AsSeedNodeSafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsSeedNodeSafetyCheck() (*SeedNodeSafetyCheck, bool) {
	return nil, false
}

// AsWaitForInbuildReplicaSafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool) {
	return &wfirsc, true
}

// AsWaitForPrimaryPlacementSafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimarySwapSafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool) {
	return nil, false
}

// AsWaitForReconfigurationSafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool) {
	return nil, false
}

// AsSafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsSafetyCheck() (*SafetyCheck, bool) {
	return nil, false
}

// AsBasicSafetyCheck is the BasicSafetyCheck implementation for WaitForInbuildReplicaSafetyCheck.
func (wfirsc WaitForInbuildReplicaSafetyCheck) AsBasicSafetyCheck() (BasicSafetyCheck, bool) {
	return &wfirsc, true
}

// WaitForPrimaryPlacementSafetyCheck safety check that waits for the primary replica that was moved out of
// the node due to upgrade to be placed back again on that node.
type WaitForPrimaryPlacementSafetyCheck struct {
	// PartitionID - Id of the partition which is undergoing the safety check.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// Kind - Possible values include: 'KindSafetyCheck', 'KindPartitionSafetyCheck', 'KindEnsureAvailability', 'KindEnsurePartitionQuorum', 'KindEnsureSeedNodeQuorum', 'KindWaitForInbuildReplica', 'KindWaitForPrimaryPlacement', 'KindWaitForPrimarySwap', 'KindWaitForReconfiguration'
	Kind KindBasicSafetyCheck `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) MarshalJSON() ([]byte, error) {
	wfppsc.Kind = KindWaitForPrimaryPlacement
	objectMap := make(map[string]interface{})
	if wfppsc.PartitionID != nil {
		objectMap["PartitionId"] = wfppsc.PartitionID
	}
	if wfppsc.Kind != "" {
		objectMap["Kind"] = wfppsc.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool) {
	return nil, false
}

// AsBasicPartitionSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsBasicPartitionSafetyCheck() (BasicPartitionSafetyCheck, bool) {
	return &wfppsc, true
}

// AsEnsureAvailabilitySafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool) {
	return nil, false
}

// AsEnsurePartitionQuorumSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool) {
	return nil, false
}

// AsSeedNodeSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsSeedNodeSafetyCheck() (*SeedNodeSafetyCheck, bool) {
	return nil, false
}

// AsWaitForInbuildReplicaSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimaryPlacementSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool) {
	return &wfppsc, true
}

// AsWaitForPrimarySwapSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool) {
	return nil, false
}

// AsWaitForReconfigurationSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool) {
	return nil, false
}

// AsSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsSafetyCheck() (*SafetyCheck, bool) {
	return nil, false
}

// AsBasicSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimaryPlacementSafetyCheck.
func (wfppsc WaitForPrimaryPlacementSafetyCheck) AsBasicSafetyCheck() (BasicSafetyCheck, bool) {
	return &wfppsc, true
}

// WaitForPrimarySwapSafetyCheck safety check that waits for the primary replica to be moved out of the
// node before starting an upgrade to ensure the availability of the primary replica for the partition.
type WaitForPrimarySwapSafetyCheck struct {
	// PartitionID - Id of the partition which is undergoing the safety check.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// Kind - Possible values include: 'KindSafetyCheck', 'KindPartitionSafetyCheck', 'KindEnsureAvailability', 'KindEnsurePartitionQuorum', 'KindEnsureSeedNodeQuorum', 'KindWaitForInbuildReplica', 'KindWaitForPrimaryPlacement', 'KindWaitForPrimarySwap', 'KindWaitForReconfiguration'
	Kind KindBasicSafetyCheck `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) MarshalJSON() ([]byte, error) {
	wfpssc.Kind = KindWaitForPrimarySwap
	objectMap := make(map[string]interface{})
	if wfpssc.PartitionID != nil {
		objectMap["PartitionId"] = wfpssc.PartitionID
	}
	if wfpssc.Kind != "" {
		objectMap["Kind"] = wfpssc.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool) {
	return nil, false
}

// AsBasicPartitionSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsBasicPartitionSafetyCheck() (BasicPartitionSafetyCheck, bool) {
	return &wfpssc, true
}

// AsEnsureAvailabilitySafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool) {
	return nil, false
}

// AsEnsurePartitionQuorumSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool) {
	return nil, false
}

// AsSeedNodeSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsSeedNodeSafetyCheck() (*SeedNodeSafetyCheck, bool) {
	return nil, false
}

// AsWaitForInbuildReplicaSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimaryPlacementSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimarySwapSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool) {
	return &wfpssc, true
}

// AsWaitForReconfigurationSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool) {
	return nil, false
}

// AsSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsSafetyCheck() (*SafetyCheck, bool) {
	return nil, false
}

// AsBasicSafetyCheck is the BasicSafetyCheck implementation for WaitForPrimarySwapSafetyCheck.
func (wfpssc WaitForPrimarySwapSafetyCheck) AsBasicSafetyCheck() (BasicSafetyCheck, bool) {
	return &wfpssc, true
}

// WaitForReconfigurationSafetyCheck safety check that waits for the current reconfiguration of the
// partition to be completed before starting an upgrade.
type WaitForReconfigurationSafetyCheck struct {
	// PartitionID - Id of the partition which is undergoing the safety check.
	PartitionID *uuid.UUID `json:"PartitionId,omitempty"`
	// Kind - Possible values include: 'KindSafetyCheck', 'KindPartitionSafetyCheck', 'KindEnsureAvailability', 'KindEnsurePartitionQuorum', 'KindEnsureSeedNodeQuorum', 'KindWaitForInbuildReplica', 'KindWaitForPrimaryPlacement', 'KindWaitForPrimarySwap', 'KindWaitForReconfiguration'
	Kind KindBasicSafetyCheck `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) MarshalJSON() ([]byte, error) {
	wfrsc.Kind = KindWaitForReconfiguration
	objectMap := make(map[string]interface{})
	if wfrsc.PartitionID != nil {
		objectMap["PartitionId"] = wfrsc.PartitionID
	}
	if wfrsc.Kind != "" {
		objectMap["Kind"] = wfrsc.Kind
	}
	return json.Marshal(objectMap)
}

// AsPartitionSafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsPartitionSafetyCheck() (*PartitionSafetyCheck, bool) {
	return nil, false
}

// AsBasicPartitionSafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsBasicPartitionSafetyCheck() (BasicPartitionSafetyCheck, bool) {
	return &wfrsc, true
}

// AsEnsureAvailabilitySafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsEnsureAvailabilitySafetyCheck() (*EnsureAvailabilitySafetyCheck, bool) {
	return nil, false
}

// AsEnsurePartitionQuorumSafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsEnsurePartitionQuorumSafetyCheck() (*EnsurePartitionQuorumSafetyCheck, bool) {
	return nil, false
}

// AsSeedNodeSafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsSeedNodeSafetyCheck() (*SeedNodeSafetyCheck, bool) {
	return nil, false
}

// AsWaitForInbuildReplicaSafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsWaitForInbuildReplicaSafetyCheck() (*WaitForInbuildReplicaSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimaryPlacementSafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsWaitForPrimaryPlacementSafetyCheck() (*WaitForPrimaryPlacementSafetyCheck, bool) {
	return nil, false
}

// AsWaitForPrimarySwapSafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsWaitForPrimarySwapSafetyCheck() (*WaitForPrimarySwapSafetyCheck, bool) {
	return nil, false
}

// AsWaitForReconfigurationSafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsWaitForReconfigurationSafetyCheck() (*WaitForReconfigurationSafetyCheck, bool) {
	return &wfrsc, true
}

// AsSafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsSafetyCheck() (*SafetyCheck, bool) {
	return nil, false
}

// AsBasicSafetyCheck is the BasicSafetyCheck implementation for WaitForReconfigurationSafetyCheck.
func (wfrsc WaitForReconfigurationSafetyCheck) AsBasicSafetyCheck() (BasicSafetyCheck, bool) {
	return &wfrsc, true
}

// WaitingChaosEvent describes a Chaos event that gets generated when Chaos is waiting for the cluster to
// become ready for faulting, for example, Chaos may be waiting for the on-going upgrade to finish.
type WaitingChaosEvent struct {
	// Reason - Describes why the WaitingChaosEvent was generated, for example, due to a cluster upgrade.
	Reason *string `json:"Reason,omitempty"`
	// TimeStampUtc - The UTC timestamp when this Chaos event was generated.
	TimeStampUtc *date.Time `json:"TimeStampUtc,omitempty"`
	// Kind - Possible values include: 'KindChaosEvent', 'KindExecutingFaults', 'KindStarted', 'KindStopped', 'KindTestError', 'KindValidationFailed', 'KindWaiting'
	Kind KindBasicChaosEvent `json:"Kind,omitempty"`
}

// MarshalJSON is the custom marshaler for WaitingChaosEvent.
func (wce WaitingChaosEvent) MarshalJSON() ([]byte, error) {
	wce.Kind = KindWaiting
	objectMap := make(map[string]interface{})
	if wce.Reason != nil {
		objectMap["Reason"] = wce.Reason
	}
	if wce.TimeStampUtc != nil {
		objectMap["TimeStampUtc"] = wce.TimeStampUtc
	}
	if wce.Kind != "" {
		objectMap["Kind"] = wce.Kind
	}
	return json.Marshal(objectMap)
}

// AsExecutingFaultsChaosEvent is the BasicChaosEvent implementation for WaitingChaosEvent.
func (wce WaitingChaosEvent) AsExecutingFaultsChaosEvent() (*ExecutingFaultsChaosEvent, bool) {
	return nil, false
}

// AsStartedChaosEvent is the BasicChaosEvent implementation for WaitingChaosEvent.
func (wce WaitingChaosEvent) AsStartedChaosEvent() (*StartedChaosEvent, bool) {
	return nil, false
}

// AsStoppedChaosEvent is the BasicChaosEvent implementation for WaitingChaosEvent.
func (wce WaitingChaosEvent) AsStoppedChaosEvent() (*StoppedChaosEvent, bool) {
	return nil, false
}

// AsTestErrorChaosEvent is the BasicChaosEvent implementation for WaitingChaosEvent.
func (wce WaitingChaosEvent) AsTestErrorChaosEvent() (*TestErrorChaosEvent, bool) {
	return nil, false
}

// AsValidationFailedChaosEvent is the BasicChaosEvent implementation for WaitingChaosEvent.
func (wce WaitingChaosEvent) AsValidationFailedChaosEvent() (*ValidationFailedChaosEvent, bool) {
	return nil, false
}

// AsWaitingChaosEvent is the BasicChaosEvent implementation for WaitingChaosEvent.
func (wce WaitingChaosEvent) AsWaitingChaosEvent() (*WaitingChaosEvent, bool) {
	return &wce, true
}

// AsChaosEvent is the BasicChaosEvent implementation for WaitingChaosEvent.
func (wce WaitingChaosEvent) AsChaosEvent() (*ChaosEvent, bool) {
	return nil, false
}

// AsBasicChaosEvent is the BasicChaosEvent implementation for WaitingChaosEvent.
func (wce WaitingChaosEvent) AsBasicChaosEvent() (BasicChaosEvent, bool) {
	return &wce, true
}
